@@ -17,7 +17,7 @@
 #   include	"docBuf.h"
 #   include	"docEdit.h"
 
-#   define	DEB_PARTICULES		0
+#   define	DEB_PARTICULES	0
 
 /************************************************************************/
 /*									*/
@@ -179,15 +179,15 @@ static int docReinsertFieldParticule(	BufferDocument *	bd,
 	}
     else{
 	tpTo= docInsertTextParticule( oldBi, part, stroff,
-		tpField->tpStrlen, tpField->tpKind, tpField->tpTextAttribute );
+					tpField->tpStrlen, tpField->tpKind,
+					tpField->tpTextAttributeNumber );
 	}
 
 #   if DEB_PARTICULES
-    appDebug( "NFL %3d: [%4d..%4d] %s <%s> \"%.*s\" len= %d\n", part,
+    appDebug( "NFL %3d: [%4d..%4d] %s \"%.*s\" len= %d\n", part,
 		    tpTo->tpStroff,
 		    tpTo->tpStroff+ tpTo->tpStrlen,
 		    docKindStr( tpTo->tpKind ),
-		    docAttributeStr( tpTo->tpTextAttribute ),
 		    (int)tpTo->tpStrlen,
 		    oldBi->biParaString+ tpTo->tpStroff,
 		    tpTo->tpStrlen );
@@ -261,7 +261,7 @@ static int docReinsertFieldParticule(	BufferDocument *	bd,
 static void docFindSubstitutionBegin(	int *			pPartBegin,
 					TextParticule **	pTpBegin,
 					const BufferItem *	bi,
-					TextAttribute		addedAttr,
+					int			addedAttrNr,
 					int			stroffBegin,
 					int			partBegin )
     {
@@ -293,8 +293,7 @@ static void docFindSubstitutionBegin(	int *			pPartBegin,
     if  ( partBegin > 0							&&
 	  tpBegin[-1].tpKind == DOCkindTEXT				&&
 	  tpBegin->tpStroff == stroffBegin				&&
-	  docEqualTextAttributes( &addedAttr,
-					&tpBegin[-1].tpTextAttribute )	)
+	  addedAttrNr == tpBegin[-1].tpTextAttributeNumber		)
 	{ tpBegin--; partBegin--; }
 
     *pPartBegin= partBegin; *pTpBegin= tpBegin; return;
@@ -303,7 +302,7 @@ static void docFindSubstitutionBegin(	int *			pPartBegin,
 static void docFindSubstitutionEnd(	int *			pPartEnd,
 					TextParticule **	pTpEnd,
 					const BufferItem *	bi,
-					TextAttribute		addedAttr,
+					int			addedAttrNr,
 					int			stroffEnd,
 					int			partBegin )
     {
@@ -341,8 +340,7 @@ static void docFindSubstitutionEnd(	int *			pPartEnd,
     if  ( partEnd < bi->biParaParticuleCount- 1				&&
 	  tpEnd[1].tpKind == DOCkindTEXT				&&
 	  tpEnd->tpStroff+ tpEnd->tpStrlen == stroffEnd			&&
-	  docEqualTextAttributes( &addedAttr,
-					&tpEnd[1].tpTextAttribute )	)
+	  addedAttrNr == tpEnd[1].tpTextAttributeNumber			)
 	{ tpEnd++; partEnd++;	}
 
     *pPartEnd= partEnd; *pTpEnd= tpEnd; return;
@@ -357,7 +355,7 @@ int docParaReplaceText(		EditOperation *		eo,
 				unsigned int		stroffEnd,
 				const unsigned char *	addedString,
 				unsigned int		addedStrlen,
-				TextAttribute		addedAttr )
+				int			addedAttributeNumber )
     {
     int				stroffShift;
     int				partShift;
@@ -384,9 +382,7 @@ int docParaReplaceText(		EditOperation *		eo,
     static TextParticule *	fieldParticules;
     int				fieldCount= 0;
 
-    TextAttribute		savedAttr;
-
-    savedAttr= addedAttr;
+    int				savedAttributeNumber;
 
     /*  1  */
     if  ( docEditClaimFieldParticules( &fieldParticules,
@@ -401,32 +397,39 @@ int docParaReplaceText(		EditOperation *		eo,
     if  ( addedStrlen == 0 )
 	{
 	tpBegin= bi->biParaParticules+ partBegin;
-	addedAttr= tpBegin->tpTextAttribute;
+	addedAttributeNumber= tpBegin->tpTextAttributeNumber;
 	}
 
     /*  4,5  */
     docFindSubstitutionBegin( &partBegin, &tpBegin, bi,
-				    addedAttr, stroffBegin, partBegin );
+				addedAttributeNumber, stroffBegin, partBegin );
 
     /*  6,7  */
     docFindSubstitutionEnd( &partEnd, &tpEnd, bi,
-				    addedAttr, stroffEnd, partBegin );
+				addedAttributeNumber, stroffEnd, partBegin );
     stroffEndEnd= tpEnd->tpStroff+ tpEnd->tpStrlen;
 
     /*  8  */
-    if  ( tpBegin->tpKind == DOCkindTEXT				    &&
-	  docEqualTextAttributes( &addedAttr, &(tpBegin->tpTextAttribute) ) )
+    if  ( tpBegin->tpKind == DOCkindTEXT				&&
+	  addedAttributeNumber == tpBegin->tpTextAttributeNumber	)
 	{ newBegin= tpBegin->tpStroff;			}
     else{
 	newBegin= stroffBegin;
 
 	if  ( newBegin > tpBegin->tpStroff )
 	    { excludeBegin= 1; }
+
+	if  ( newBegin == bi->biParaStrlen )
+	    {
+	    if  ( tpBegin->tpKind == DOCkindNOTE	||
+		  tpBegin->tpKind == DOCkindFIELDEND	)
+		{ excludeBegin= 1;	}
+	    }
 	}
 
     /*  9  */
-    if  ( tpEnd->tpKind == DOCkindTEXT					  &&
-	  docEqualTextAttributes( &addedAttr, &(tpEnd->tpTextAttribute) ) )
+    if  ( tpEnd->tpKind == DOCkindTEXT					&&
+	  addedAttributeNumber == tpEnd->tpTextAttributeNumber		)
 	{ newEnd= tpEnd->tpStroff+ tpEnd->tpStrlen;	}
     else{
 	newEnd= stroffEnd;
@@ -441,11 +444,10 @@ int docParaReplaceText(		EditOperation *		eo,
 	tpBegin->tpStrlen= newBegin- tpBegin->tpStroff;
 
 #	if DEB_PARTICULES
-	appDebug( "<<= %3d: [%4d..%4d] %s <%s> \"%.*s\"\n", partBegin,
+	appDebug( "<<= %3d: [%4d..%4d] %s \"%.*s\"\n", partBegin,
 				tpBegin->tpStroff,
 				tpBegin->tpStroff+ tpBegin->tpStrlen,
 				docKindStr( tpBegin->tpKind ),
-				docAttributeStr( tpBegin->tpTextAttribute ),
 				(int)tpBegin->tpStrlen,
 				bi->biParaString+ tpBegin->tpStroff );
 #	endif
@@ -459,8 +461,8 @@ int docParaReplaceText(		EditOperation *		eo,
 	      excludeBegin		)
 	    {
 	    tp= docInsertTextParticule( bi, partEnd+ 1,
-					newEnd, stroffEndEnd- newEnd,
-					DOCkindTEXT, tpEnd->tpTextAttribute );
+				newEnd, stroffEndEnd- newEnd,
+				DOCkindTEXT, tpEnd->tpTextAttributeNumber );
 	    if  ( ! tp )
 		{ XDEB(tp); return -1;	}
 
@@ -473,11 +475,10 @@ int docParaReplaceText(		EditOperation *		eo,
 	    }
 
 #	if DEB_PARTICULES
-	appDebug( "=>> %3d: [%4d..%4d] %s <%s> \"%.*s\"\n", partEnd,
+	appDebug( "=>> %3d: [%4d..%4d] %s \"%.*s\"\n", partEnd,
 				tpEnd->tpStroff,
 				tpEnd->tpStroff+ tpEnd->tpStrlen,
 				docKindStr( tpEnd->tpKind ),
-				docAttributeStr( tpEnd->tpTextAttribute ),
 				(int)tpEnd->tpStrlen,
 				bi->biParaString+ tpEnd->tpStroff );
 #	endif
@@ -493,18 +494,17 @@ int docParaReplaceText(		EditOperation *		eo,
     tpBegin= bi->biParaParticules+ partBegin;
     tpEnd= bi->biParaParticules+ partEnd;
 
-    savedAttr= tpBegin->tpTextAttribute;
+    savedAttributeNumber= tpBegin->tpTextAttributeNumber;
 
     /*  11  */
     tp= tpBegin;
     for ( part= partBegin; part <= partEnd; tp++, part++ )
 	{
 #	if DEB_PARTICULES
-	appDebug( "OLD %3d: [%4d..%4d] %s <%s> \"%.*s\"\n", part,
+	appDebug( "OLD %3d: [%4d..%4d] %s \"%.*s\"\n", part,
 		    tp->tpStroff,
 		    tp->tpStroff+ tp->tpStrlen,
 		    docKindStr( tp->tpKind ),
-		    docAttributeStr( tp->tpTextAttribute ),
 		    (int)tp->tpStrlen,
 		    bi->biParaString+ tp->tpStroff );
 #	endif
@@ -581,8 +581,9 @@ int docParaReplaceText(		EditOperation *		eo,
 	{ LLDEB(newBegin,newEnd); }
 
     /*  17  */
-    partsDone= docRedivideStringInParticules( bi, newBegin, newEnd- newBegin,
-					part, oldCount- newCount, addedAttr );
+    partsDone= docRedivideStringInParticules( bi,
+			    newBegin, newEnd- newBegin,
+			    part, oldCount- newCount, addedAttributeNumber );
     if  ( partsDone < 0 )
 	{ LDEB(partsDone); return -1;	}
     newCount += partsDone;
@@ -624,7 +625,7 @@ int docParaReplaceText(		EditOperation *		eo,
 		{ LLDEB(part,bi->biParaStrlen);	}
 
 	    tp= docInsertTextParticule( bi, part, bi->biParaStrlen,
-						0, DOCkindTEXT, savedAttr );
+					0, DOCkindTEXT, savedAttributeNumber );
 	    if  ( ! tp )
 		{ XDEB(tp); return -1;	}
 
@@ -643,11 +644,10 @@ int docParaReplaceText(		EditOperation *		eo,
     part= 0; tp= bi->biParaParticules;
     while( part < bi->biParaParticuleCount )
 	{
-	appDebug( "=== %3d: [%4d..%4d] %s <%s> \"%.*s\"\n", part,
+	appDebug( "=== %3d: [%4d..%4d] %s \"%.*s\"\n", part,
 		    tp->tpStroff,
 		    tp->tpStroff+ tp->tpStrlen,
 		    docKindStr( tp->tpKind ),
-		    docAttributeStr( tp->tpTextAttribute ),
 		    (int)tp->tpStrlen,
 		    bi->biParaString+ tp->tpStroff );
 	part++; tp++;
@@ -663,7 +663,8 @@ int docParaReplaceText(		EditOperation *		eo,
 
 /************************************************************************/
 /*									*/
-/*  Append the tail of a paragraph to another paragraph.		*/
+/*  Append the tail of a paragraph to another paragraph in the same	*/
+/*  document.								*/
 /*									*/
 /*  1)  Claim memory for the appended contents.				*/
 /*  2)  Claim memory for the appended particules.			*/
@@ -675,14 +676,15 @@ int docParaReplaceText(		EditOperation *		eo,
 /************************************************************************/
 
 static int docParaAppend(	EditOperation *		eo,
-				const BufferDocument *	bdFrom,
-				int *			fieldMap,
+				DocumentCopyJob *	dcj,
 				unsigned int *		pFieldUpd,
 				BufferItem *		biTo,
 				BufferItem *		biFrom,
 				int			part,
 				unsigned int		stroffFrom )
     {
+    int			rval= 0;
+
     TextParticule *	tpFrom= biFrom->biParaParticules+ part;
     TextParticule *	newTp;
 
@@ -702,7 +704,7 @@ static int docParaAppend(	EditOperation *		eo,
     newTp= (TextParticule *)realloc( biTo->biParaParticules,
 				    particuleCount* sizeof(TextParticule) );
     if  ( ! newTp )
-	{ LXDEB(particuleCount,newTp); return -1;	}
+	{ LXDEB(particuleCount,newTp); rval= -1; goto ready;	}
     biTo->biParaParticules= newTp;
 
     if  ( biTo->biParaParticuleCount > 0 )
@@ -711,8 +713,8 @@ static int docParaAppend(	EditOperation *		eo,
 
 	/*  3  */
 	if  ( tpFrom->tpKind == DOCkindTEXT				&&
-	      docEqualTextAttributes( &(tpFrom->tpTextAttribute),
-				      &(newTp->tpTextAttribute) )	)
+	      tpFrom->tpTextAttributeNumber ==
+					  newTp->tpTextAttributeNumber	)
 	    {
 	    int		ignored;
 
@@ -722,8 +724,8 @@ static int docParaAppend(	EditOperation *		eo,
 				biTo->biParaStrlen,
 				biFrom->biParaString+ stroffFrom,
 				tpFrom->tpStroff+ tpFrom->tpStrlen- stroffFrom,
-				tpFrom->tpTextAttribute ) )
-		{ LDEB(tpFrom->tpStrlen); return -1;	}
+				tpFrom->tpTextAttributeNumber ) )
+		{ LDEB(tpFrom->tpStrlen); rval= -1; goto ready;	}
 
 	    stroffFrom= tpFrom->tpStroff+ tpFrom->tpStrlen;
 	    part++; tpFrom++;
@@ -736,21 +738,18 @@ static int docParaAppend(	EditOperation *		eo,
     if  ( docParaStringReplace( &stroffShift, biTo, stroffTo, stroffTo,
 					biFrom->biParaString+ stroffFrom,
 					biFrom->biParaStrlen- stroffFrom ) )
-	{ LDEB(biFrom->biParaStrlen); return -1;	}
+	{ LDEB(biFrom->biParaStrlen); rval= -1; goto ready;	}
 
     while( part < biFrom->biParaParticuleCount )
 	{
-	int	len= tpFrom->tpStroff+ tpFrom->tpStrlen- stroffFrom;
+	int		strlenTo;
+	const int	partTo= -1;
 
-	newTp= docCopyParticule( biTo, -1, stroffTo, len,
-						    tpFrom->tpKind, tpFrom );
+	strlenTo= tpFrom->tpStroff+ tpFrom->tpStrlen- stroffFrom;
 
-	if  ( ! newTp )
-	    { LDEB(biTo->biParaParticuleCount); return -1;	}
-
-	if  ( docCopyParticuleData( eo->eoBd, bdFrom, fieldMap, pFieldUpd,
-						biTo, biFrom, newTp, tpFrom ) )
-	    { LDEB(part); return -1;	}
+	if  ( docCopyParticuleAndData( &newTp, dcj, pFieldUpd, biTo, partTo,
+				    stroffTo, strlenTo, biFrom, tpFrom ) )
+	    { LDEB(1); rval= -1; goto ready;	}
 
 	/*
 	appDebug( "NEW %3d: [%4d..%4d] %s \"%.*s\"\n",
@@ -762,11 +761,13 @@ static int docParaAppend(	EditOperation *		eo,
 		    biTo->biParaString+ newTp->tpStroff );
 	*/
 
-	stroffTo += len; stroffFrom += len;
+	stroffTo += strlenTo; stroffFrom += strlenTo;
 	tpFrom++; part++; partShift++;
 	}
 
-    return 0;
+  ready:
+
+    return rval;
     }
 
 /************************************************************************/
@@ -777,13 +778,15 @@ static int docParaAppend(	EditOperation *		eo,
 /*									*/
 /************************************************************************/
 
-int docRemoveSelectionTail(		EditOperation *		eo,
-					const DocumentSelection *	ds )
+int docRemoveSelectionTail(	EditOperation *			eo,
+				const DocumentSelection *	ds )
     {
     BufferItem *	nextBi;
     int			paraShift= 0;
     int			sectShift= 0;
 
+    int			firstDeleted= -1;
+
     nextBi= docNextParagraph( ds->dsBegin.dpBi );
     if  ( ! nextBi )
 	{ XDEB(nextBi); return -1;	}
@@ -793,8 +796,14 @@ int docRemoveSelectionTail(		EditOperation *		eo,
 	BufferItem *	parent= nextBi->biParent;
 	BufferItem *	pp;
 
-	docEditDeleteItems( eo, parent, nextBi->biNumberInParent, 1 );
-	paraShift++;
+	int		firstParaDeleted= -1;
+	int		paragraphsDeleted= 0;
+
+	docEditDeleteItems( eo, &firstParaDeleted, &paragraphsDeleted,
+					parent, nextBi->biNumberInParent, 1 );
+	paraShift += paragraphsDeleted;
+	if  ( firstDeleted < 0 )
+	    { firstDeleted= firstParaDeleted;	}
 
 	if  ( nextBi == ds->dsEnd.dpBi )
 	    {
@@ -822,15 +831,18 @@ int docRemoveSelectionTail(		EditOperation *		eo,
 
     if  ( paraShift > 0 )
 	{
-	int	paraNr= docNumberOfParagraph( ds->dsBegin.dpBi );
-
 	if  ( ds->dsBegin.dpBi->biInExternalItem == DOCinBODY )
 	    {
-	    docShiftNoteReferences( eo->eoBd, paraNr+ 1, 0,
+	    if  ( firstDeleted < 0 )
+		{ LDEB(firstDeleted);	}
+
+	    docShiftNoteReferences( eo->eoBd, firstDeleted+ paraShift, 0,
 						-sectShift, -paraShift, 0 );
 	    }
 
+	/*  NO! done by docEditDeleteItems()
 	docEditShiftReformatRangeParaNr( eo, paraNr+ 1, -paraShift );
+	*/
 	}
 
     return 0;
@@ -840,65 +852,73 @@ int docRemoveSelectionTail(		EditOperation *		eo,
 /*									*/
 /*  Replace a selection with a piece of text.				*/
 /*									*/
-/*  b)  Replace the tail of the beginning paragraph with the new text.	*/
-/*  c)  Merge the two paragraphs.					*/
-/*  d)  Erase all paragraphs after the beginning of the selection	*/
-/*	to, including the end.						*/
-/*  e)  Update the paragraph buffer and the particule administration.	*/
+/*  1)  If the selection spans more than one paragraph...		*/
+/*  2)  Replace the tail of the beginning paragraph with the new text.	*/
+/*  3)  Merge the two paragraphs.					*/
+/*  4)  Erase all paragraphs after the beginning of the selection	*/
+/*	upto and including the end of the selection.			*/
+/*  5)  Otherwise just replace the text inside the selected paragraph.	*/
 /*									*/
 /************************************************************************/
 
-int docReplaceSelection(	EditOperation *		eo,
-				const DocumentSelection *	ds,
-				int *			pPartShift,
-				int *			pStroffShift,
-				const unsigned char *	addedString,
-				int			addedStrlen,
-				TextAttribute		addedAttribute )
+int docReplaceSelection(EditOperation *			eo,
+			const DocumentSelection *	ds,
+			int *				pPartShift,
+			int *				pStroffShift,
+			const unsigned char *		addedString,
+			int				addedStrlen,
+			int				addedAttributeNumber )
     {
-    /*  a  */
+    int			rval= 0;
+
+    DocumentCopyJob	dcj;
+
+    docInitDocumentCopyJob( &dcj );
+
+    /*  1  */
     if  ( ds->dsBegin.dpBi != ds->dsEnd.dpBi )
 	{
-	/*  b  */
+	/*  2  */
 	if  ( docParaReplaceText( eo, ds->dsBegin.dpBi,
 		    ds->dsBegin.dpParticule, ds->dsBegin.dpStroff,
 		    pPartShift, pStroffShift,
 		    ds->dsBegin.dpBi->biParaStrlen,
-		    addedString, addedStrlen, addedAttribute ) )
-	    { LDEB(addedStrlen); return -1;	}
+		    addedString, addedStrlen, addedAttributeNumber ) )
+	    { LDEB(addedStrlen); rval= -1; goto ready;	}
 
-	/*  c  */
+	/*  3  */
 	if  ( ds->dsEnd.dpStroff < ds->dsEnd.dpBi->biParaStrlen )
 	    {
-	    int *		fieldMap;
 	    unsigned int	fieldUpd= FIELDdoNOTHING;
 
-	    fieldMap= docAllocateFieldMap( eo->eoBd );
-	    if  ( ! fieldMap )
-		{ XDEB(fieldMap); return -1;	}
+	    if  ( docSet1DocumentCopyJob( &dcj, eo->eoBd ) )
+		{ LDEB(1); rval= -1; goto ready;	}
 
-	    if  ( docParaAppend( eo, eo->eoBd, fieldMap, &fieldUpd,
+	    if  ( docParaAppend( eo, &dcj, &fieldUpd,
 				ds->dsBegin.dpBi, ds->dsEnd.dpBi,
 				ds->dsEnd.dpParticule, ds->dsEnd.dpStroff ) )
-		{ LDEB(addedStrlen); free( fieldMap ); return -1;	}
-
-	    free( fieldMap );
+		{ LDEB(addedStrlen); rval= -1; goto ready;	}
 	    }
 
+	/*  4  */
 	if  ( docRemoveSelectionTail( eo, ds ) )
-	    { LDEB(1); return -1;	}
+	    { LDEB(1); rval= -1; goto ready;	}
 	}
     else{
-	/*  e  */
+	/*  5  */
 	if  ( docParaReplaceText( eo, ds->dsBegin.dpBi, ds->dsBegin.dpParticule,
 			    ds->dsBegin.dpStroff,
 			    pPartShift, pStroffShift,
 			    ds->dsEnd.dpStroff,
-			    addedString, addedStrlen, addedAttribute ) )
-	    { LDEB(addedStrlen); return -1;	}
+			    addedString, addedStrlen, addedAttributeNumber ) )
+	    { LDEB(addedStrlen); rval= -1; goto ready;	}
 	}
 
-    return 0;
+  ready:
+
+    docCleanDocumentCopyJob( &dcj );
+
+    return rval;
     }
 
 /************************************************************************/
@@ -921,6 +941,8 @@ static int docSplitParaItemLow(	BufferDocument *	bd,
 				BufferItem *		oldBi,
 				int			stroff )
     {
+    int				rval= 0;
+
     BufferItem *		newBi;
     int				part= 0;
     TextParticule *		tp;
@@ -942,21 +964,39 @@ static int docSplitParaItemLow(	BufferDocument *	bd,
     PropertyMask		ppChgMask;
     PropertyMask		ppUpdMask;
 
+    DocumentCopyJob		dcj;
+
+    docInitDocumentCopyJob( &dcj );
+
+    if  ( docSet1DocumentCopyJob( &dcj, bd ) )
+	{ LDEB(1); rval= -1; goto ready;	}
+
     if  ( docEditClaimFieldParticules( &fieldParticules,
 						oldBi->biParaParticuleCount ) )
-	{ LDEB(oldBi->biParaParticuleCount); return -1; }
+	{ LDEB(oldBi->biParaParticuleCount); rval= -1; goto ready; }
 
     /*  a,b  */
     if  ( docFindParticule( &part, oldBi, stroff, /* lastOne */ 0 ) )
-	{ LLDEB(oldBi->biParaStrlen,stroff); return -1; }
+	{ LLDEB(oldBi->biParaStrlen,stroff); rval= -1; goto ready; }
 
     tp= oldBi->biParaParticules+ part;
 
+    if  ( part < oldBi->biParaParticuleCount-1		&&
+	  stroff == tp->tpStroff+ tp->tpStrlen		&&
+	  tp[1].tpKind == DOCkindFIELDEND		)
+	{ tp++; part++;	}
+
+    while( part < oldBi->biParaParticuleCount		&&
+	   tp->tpStrlen == 0				&&
+	   ( tp->tpKind == DOCkindFIELDEND	||
+	     tp->tpKind == DOCkindNOTE		)	)
+	{ tp++; part++;	}
+
     /*  1  */
     newBi= docInsertItem( bd, oldBi->biParent,
 				oldBi->biNumberInParent+ 1, oldBi->biLevel );
     if  ( ! newBi )
-	{ XDEB(newBi); return -1;	}
+	{ XDEB(newBi); rval= -1; goto ready;	}
 
     PROPmaskCLEAR( &ppChgMask );
 
@@ -965,8 +1005,8 @@ static int docSplitParaItemLow(	BufferDocument *	bd,
 
     /*  1a  */
     if  ( docUpdParaProperties( &ppChgMask, &(newBi->biParaProperties),
-				    &ppUpdMask, &(oldBi->biParaProperties) ) )
-	{ LDEB(oldBi->biParaTabCount); return -1;	}
+		&ppUpdMask, &(oldBi->biParaProperties), dcj.dcjColorMap ) )
+	{ LDEB(1); rval= -1; goto ready;	}
 
     /*  2  */
     if  ( docParaStringReplace( &stroffShift, newBi, 0, 0,
@@ -975,13 +1015,14 @@ static int docSplitParaItemLow(	BufferDocument *	bd,
 	{
 	LLDEB(oldBi->biParaStrlen,stroff);
 	docDeleteItems( bd, oldBi->biParent, oldBi->biNumberInParent+ 1, 1 );
-	return -1;
+	rval= -1; goto ready;
 	}
 
     splittingField= docParticuleInField( oldBi, part );
 
     /*  3  */
-    if  ( stroff > tp->tpStroff )
+    if  ( part < oldBi->biParaParticuleCount	&&
+	  stroff > tp->tpStroff			)
 	{
 	if  ( stroff == tp->tpStroff+ tp->tpStrlen )
 	    {
@@ -990,32 +1031,17 @@ static int docSplitParaItemLow(	BufferDocument *	bd,
 	    part++; tp++;
 	    }
 	else{
-	    newTp= docCopyParticule( newBi, partShift++,
-				newStrlen, tp->tpStroff+ tp->tpStrlen- stroff,
-				DOCkindTEXT, tp );
-	    if  ( ! newTp )
-		{ LDEB(partShift); return -1;	}
+	    if  ( docSplitTextParticule( &tp, &newTp, oldBi, part, stroff ) )
+		{ LDEB(part); rval= -1; goto ready;	}
 
-	    newStrlen= tp->tpStroff+ tp->tpStrlen- stroff;
-	    tp->tpStrlen= stroff- tp->tpStroff;
+	    newStrlen= 0;
 	    truncatedParticuleCount= part+ 1;
-	    part++; tp++;
+	    part++; tp= newTp;
 	    }
 	}
     else{
-	newTp= docCopyParticule( newBi, partShift++, newStrlen, tp->tpStrlen,
-							    tp->tpKind, tp );
-	if  ( ! newTp )
-	    { LDEB(partShift); return -1;	}
-
-	if  ( docCopyParticuleData( bd, bd, fieldMap, &fieldUpd,
-						newBi, oldBi, newTp, tp ) )
-	    { LDEB(part); return -1;	}
-
-	newStrlen= tp->tpStrlen;
+	newStrlen= 0;
 	truncatedParticuleCount= part;
-
-	part++; tp++;
 	}
 
     /*  4  */
@@ -1052,15 +1078,11 @@ static int docSplitParaItemLow(	BufferDocument *	bd,
 		break;
 	    }
 
-	newTp= docCopyParticule( newBi, partShift++, newStrlen, tp->tpStrlen,
-							    tp->tpKind, tp );
-	if  ( ! newTp )
-	    {
-	    LXDEB(partShift,newTp); return -1;	}
-
-	if  ( docCopyParticuleData( bd, bd, fieldMap, &fieldUpd,
-						newBi, oldBi, newTp, tp ) )
-	    { LDEB(part); return -1;	}
+	if  ( docCopyParticuleAndData( &newTp, &dcj, &fieldUpd,
+						newBi, partShift++,
+						newStrlen, tp->tpStrlen,
+						oldBi, tp ) )
+	    { LDEB(part); rval= -1; goto ready;	}
 
 	newStrlen += tp->tpStrlen; part++; tp++;
 	}
@@ -1073,7 +1095,7 @@ static int docSplitParaItemLow(	BufferDocument *	bd,
 	fieldCount= docCleanupFields( bd, oldBi, fieldParticules, fieldCount );
 
 	if  ( fieldCount < 0 )
-	    { LDEB(fieldCount); return -1;	}
+	    { LDEB(fieldCount); rval= -1; goto ready;	}
 
 	tpField= fieldParticules;
 	tp= oldBi->biParaParticules+ truncatedParticuleCount;
@@ -1085,7 +1107,7 @@ static int docSplitParaItemLow(	BufferDocument *	bd,
 					truncatedParticuleCount, tpField, tp );
 
 	    if  ( res < 0 )
-		{ LDEB(res); return -1;	}
+		{ LDEB(res); rval= -1; goto ready;	}
 
 	    if  ( res > 0 )
 		{ truncatedParticuleCount++; tp++;	}
@@ -1099,26 +1121,32 @@ static int docSplitParaItemLow(	BufferDocument *	bd,
 	{
 	tp= oldBi->biParaParticules+ oldBi->biParaParticuleCount- 1;
 
-	newTp= docCopyParticule( newBi, -1, 0, 0, DOCkindTEXT, tp );
+	newTp= docInsertTextParticule( newBi, -1, 0, 0,
+				    DOCkindTEXT, tp->tpTextAttributeNumber );
 	if  ( ! newTp )
-	    { LDEB(newBi->biParaParticuleCount); return -1; }
+	    { LDEB(newBi->biParaParticuleCount); rval= -1; goto ready; }
 	}
 
     if  ( oldBi->biParaParticuleCount == 0 )
 	{
 	tp= oldBi->biParaParticules;
 
-	newTp= docCopyParticule( oldBi, -1, 0, 0, DOCkindTEXT,
-						    oldBi->biParaParticules );
+	newTp= docInsertTextParticule( oldBi, -1, 0, 0,
+				    DOCkindTEXT, tp->tpTextAttributeNumber );
 	if  ( ! newTp )
-	    { LDEB(oldBi->biParaParticuleCount); return -1; }
+	    { LDEB(oldBi->biParaParticuleCount); rval= -1; goto ready; }
 	}
 
     newBi->biParaLineCount= 0;
     oldBi->biParaLineCount= 0;
 
     *pNewBi= newBi;
-    return 0;
+
+  ready:
+
+    docCleanDocumentCopyJob( &dcj );
+
+    return rval;
     }
 
 int docSplitParaItem(		BufferDocument *	bd,
@@ -1142,18 +1170,21 @@ int docSplitParaItem(		BufferDocument *	bd,
 
 /************************************************************************/
 /*									*/
-/*  Insert a tab.							*/
+/*  Insert a tab or another kind of special particule for something	*/
+/*  that takes up some space in the document.				*/
 /*									*/
-/*  -1) Insert a space in the text string.				*/
-/*  0)  Replace the empty particule in an empty paragraph.		*/
-/*  1)  Can be inserted before the current particule.			*/
-/*  2)  Can be inserted after the current particule.			*/
-/*  3)  First split the particule,					*/
-/*  4)  Then insert between the two halves.				*/
+/*  1)  Insert a space in the text string.				*/
+/*  2)  Replace the empty particule in an empty paragraph.		*/
+/*  3)  Can be inserted before the current particule.			*/
+/*  4)  Can be inserted after the current particule.			*/
+/*  5)  First split the particule,					*/
+/*  6)  Then insert between the two halves.				*/
+/*  7)  Shift particile offsets in the rest of the paragraph.		*/
 /*									*/
 /************************************************************************/
 
-TextParticule * docParaSpecialParticule(BufferItem *		bi,
+TextParticule * docParaSpecialParticule(BufferDocument *	bd,
+					BufferItem *		bi,
 					int			kind,
 					int			part,
 					int			stroff,
@@ -1164,7 +1195,7 @@ TextParticule * docParaSpecialParticule(BufferItem *		bi,
     TextParticule *	tpRet;
     int			stroffShift= 0;
 
-    /*  -1  */
+    /*  1  */
     if  ( docParaStringReplace( &stroffShift, bi, stroff, stroff,
 						(unsigned char *)" ", 1 ) )
 	{ LDEB(stroff); return (TextParticule *)0;	}
@@ -1174,11 +1205,12 @@ TextParticule * docParaSpecialParticule(BufferItem *		bi,
 	{
 	tp= bi->biParaParticules+ part- 1;
 
-	tpRet= docCopyParticule( bi, part, stroff, 1, kind, tp );
+	tpRet= docInsertTextParticule( bi, part, stroff, 1,
+					    kind, tp->tpTextAttributeNumber );
 	if  ( ! tpRet )
 	    { LXDEB(part,tpRet); return (TextParticule *)0;	}
 
-	tpRet->tpObjectNumber= -1; /* set later on */
+	tpRet->tpObjectNumber= -1; /* set by caller if needed */
 
 	*pPartShift= 1; *pStroffShift= stroffShift;
 	return tpRet;
@@ -1189,7 +1221,7 @@ TextParticule * docParaSpecialParticule(BufferItem *		bi,
 
     tp= bi->biParaParticules+ part;
 
-    /*  0  */
+    /*  2  */
     if  ( bi->biParaParticuleCount == 1			&&
 	  bi->biParaStrlen == 1				&&
 	  bi->biParaParticules->tpKind == DOCkindTEXT	)
@@ -1201,55 +1233,57 @@ TextParticule * docParaSpecialParticule(BufferItem *		bi,
 	return bi->biParaParticules;
 	}
 
-    /*  1  */
+    /*  3  */
     if  ( stroff == tp->tpStroff )
 	{
-	tpRet= docCopyParticule( bi, part, stroff, 1, kind, tp );
+	tpRet= docInsertTextParticule( bi, part, stroff, 1,
+					    kind, tp->tpTextAttributeNumber );
 	if  ( ! tpRet )
 	    { LXDEB(part,tpRet); return (TextParticule *)0;	}
 
-	tpRet->tpObjectNumber= -1; /* set later on */
+	tpRet->tpObjectNumber= -1; /* set by caller if needed */
 
 	*pPartShift= 1;
 	part++;
 	}
     else{
-	/*  2  */
+	/*  4  */
 	if  ( stroff == tp->tpStroff+ tp->tpStrlen )
 	    {
-	    tpRet= docCopyParticule( bi, part+ 1, stroff, 1, kind, tp );
+	    tpRet= docInsertTextParticule( bi, part+ 1, stroff, 1,
+					    kind, tp->tpTextAttributeNumber );
 	    if  ( ! tpRet )
-		{ LXDEB(part+1,tpRet); return (TextParticule *)0;	}
+		{ LXDEB(part,tpRet); return (TextParticule *)0;	}
 
-	    tpRet->tpObjectNumber= -1; /* set later on */
+	    tpRet->tpObjectNumber= -1; /* set by caller if needed */
 
 	    *pPartShift= 2;
 	    part += 2;
 	    }
 	else{
-	    /*  3  */
-	    if  ( ! docCopyParticule( bi, part+ 1, stroff,
-			tp->tpStroff+ tp->tpStrlen- stroff, DOCkindTEXT, tp ) )
-		{ LDEB(part+1); return (TextParticule *)0;	}
+	    TextParticule *	tpNew;
 
-	    tp= bi->biParaParticules+ part++;
-	    tp->tpStrlen= stroff- tp->tpStroff;
+	    /*  5  */
+	    if  ( docSplitTextParticule( &tp, &tpNew, bi, part, stroff ) )
+		{ LLDEB(part,stroff); return (TextParticule *)0;	}
 
-	    /*  4  */
-	    tpRet= docCopyParticule( bi, part, stroff, 1, kind, tp );
+	    /*  6  */
+	    tpRet= docInsertTextParticule( bi, part+ 1, stroff, 1,
+					    kind, tp->tpTextAttributeNumber );
 	    if  ( ! tpRet )
 		{ LXDEB(part+1,tpRet); return (TextParticule *)0;	}
 
-	    tpRet->tpObjectNumber= -1; /* set later on */
+	    tpRet->tpObjectNumber= -1; /* set by caller if needed */
 
 	    *pPartShift= 2;
-	    part++;
+	    part += 2;
 	    }
 	}
 
-    tp= bi->biParaParticules+ part;
-    while( part < bi->biParaParticuleCount )
-	{ tp->tpStroff++; part++; tp++;	}
+    /*  7  */
+    if  ( docShiftParticuleOffsets( bd, bi,
+					part, bi->biParaParticuleCount, 1 ) )
+	{ LDEB(1);	}
 
     *pStroffShift= stroffShift;
 
@@ -1334,6 +1368,8 @@ void docCloseItemObjects(	int *			pNoteCount,
 /************************************************************************/
 
 void docEditDeleteItems(	EditOperation *		eo,
+				int *			pFirstParaDeleted,
+				int *			pParagraphsDeleted,
 				BufferItem *		bi,
 				int			first,
 				int			count )
@@ -1341,7 +1377,7 @@ void docEditDeleteItems(	EditOperation *		eo,
     int		i;
     int		f= first;
 
-    int		firstDeleted= -1;
+    int		firstParaDeleted= -1;
     int		paragraphsDeleted= 0;
     int		notesDeleted= 0;
 
@@ -1352,7 +1388,7 @@ void docEditDeleteItems(	EditOperation *		eo,
 	/*  1  */
 	if  ( docFirstPosition( &dp, bi->biChildren[f] ) )
 	    { LDEB(1);						}
-	else{ firstDeleted= docNumberOfParagraph( dp.dpBi );	}
+	else{ firstParaDeleted= docNumberOfParagraph( dp.dpBi );	}
 
 	/*  2  */
 	docEditIncludeItemInReformatRange( eo, bi );
@@ -1365,19 +1401,20 @@ void docEditDeleteItems(	EditOperation *		eo,
 					eo->eoVoidadd, eo->eoCloseObject );
 	}
 
-    eo->eoParagraphsDeleted += paragraphsDeleted;
     eo->eoNotesDeleted += notesDeleted;
 
     docDeleteItems( eo->eoBd, bi, first, count );
 
-    if  ( firstDeleted < 0 )
-	{ LDEB(firstDeleted);	}
+    if  ( firstParaDeleted < 0 )
+	{ LDEB(firstParaDeleted);	}
     else{
 	docEditShiftReformatRangeParaNr( eo,
-					firstDeleted+ paragraphsDeleted,
+					firstParaDeleted+ paragraphsDeleted,
 					-paragraphsDeleted );
 	}
 
+    *pFirstParaDeleted= firstParaDeleted;
+    *pParagraphsDeleted= paragraphsDeleted;
     return;
     }
 
@@ -1391,21 +1428,21 @@ int docEditMakeParagraphEmpty(		BufferItem *		paraBi,
 					EditOperation *		eo )
     {
     TextParticule *	tp= paraBi->biParaParticules;
-    TextAttribute	ta= tp->tpTextAttribute;
+    int			textAttributeNumber= tp->tpTextAttributeNumber;
 
     int			notesDeleted= 0;
 
-    paraBi->biParaStrlen= 0;
-
     docCloseParaObjects( &notesDeleted,
 			eo->eoBd, paraBi, eo->eoVoidadd, eo->eoCloseObject );
 
     eo->eoNotesDeleted += notesDeleted;
 
+    paraBi->biParaStrlen= 0;
     docDeleteParticules( paraBi, 0, paraBi->biParaParticuleCount );
     docDeleteLines( paraBi, 0, paraBi->biParaLineCount );
 
-    if  ( ! docInsertTextParticule( paraBi, 0, 0, 0, DOCkindTEXT, ta ) )
+    if  ( ! docInsertTextParticule( paraBi, 0, 0, 0, DOCkindTEXT,
+							textAttributeNumber ) )
 	{ LDEB(1); return -1;	}
 
     return 0;
