@@ -4,7 +4,7 @@
 /*									*/
 /************************************************************************/
 
-#   include	"config.h"
+#   include	"tedConfig.h"
 
 #   include	<stdlib.h>
 #   include	<string.h>
@@ -14,65 +14,7 @@
 
 #   include	<appUnit.h>
 #   include	"docBuf.h"
-
-/************************************************************************/
-/*									*/
-/*  Count the number of particules inside a text level field. The	*/
-/*  begin and end particules are not counted.				*/
-/*									*/
-/*  1)  Do some sanity checks on the proper nesting of fields.. Just	*/
-/*	to have a safe feeling.						*/
-/*									*/
-/************************************************************************/
-
-static int docCountParticulesInField(	BufferItem *		bi,
-					int			part,
-					int			partUpto )
-    {
-    int				fieldLevel= 0;
-    int				partCount= 0;
-    int				fieldNumber;
-
-    const TextParticule *	tp;
-
-    tp= bi->biParaParticules+ part;
-    if  ( tp->tpKind != DOCkindFIELDSTART )
-	{ LDEB(tp->tpKind); return -1;	}
-    fieldNumber= tp->tpObjectNumber;
-
-    tp++; part++;
-    while( part < partUpto )
-	{
-	if  ( tp->tpKind == DOCkindFIELDSTART )
-	    {
-	    /*  1  */
-	    if  ( tp->tpObjectNumber == fieldNumber )
-		{ LLDEB(tp->tpObjectNumber,fieldNumber);	}
-
-	    fieldLevel++;
-	    }
-
-	if  ( tp->tpKind == DOCkindFIELDEND )
-	    {
-	    if  ( tp->tpObjectNumber == fieldNumber )
-		{
-		/*  1 
-		if  ( fieldLevel != 0 )
-		    { LDEB(fieldLevel);	}
-		*/
-
-		return partCount;
-		}
-
-	    fieldLevel--;
-	    }
-
-	partCount++, part++; tp++;
-	}
-
-    /*  1  */
-    LLDEB(fieldLevel,fieldNumber); return -1;
-    }
+#   include	"docEvalField.h"
 
 /************************************************************************/
 /*									*/
@@ -83,7 +25,7 @@ static int docCountParticulesInField(	BufferItem *		bi,
 /*									*/
 /************************************************************************/
 
-static int docRecalculateParaStringTextParticules(
+int docRecalculateParaStringTextParticules(
 				int *				pCalculated,
 				BufferDocument *		bd,
 				int *				pPartShift,
@@ -104,7 +46,7 @@ static int docRecalculateParaStringTextParticules(
 
     TextParticule *			tp= bi->biParaParticules+ part;
 
-    TextAttribute			ta;
+    int					textAttributeNumber;
 
     int					past;
     int					stroff;
@@ -115,7 +57,7 @@ static int docRecalculateParaStringTextParticules(
     const FieldKindInformation *	fki= DOC_FieldKinds+ df->dfKind;
 
     if  ( (*fki->fkiCalculateTextString)( &calculated, &newSize, scratch,
-					    sizeof(scratch)- 1, bd, bi, df ) )
+			    sizeof(scratch)- 1, bd, bi, part, partCount, df ) )
 	{ SDEB(fki->fkiLabel); return -1;	}
 
     if  ( ! calculated )
@@ -136,7 +78,7 @@ static int docRecalculateParaStringTextParticules(
 	return 0;
 	}
 
-    ta= tp[1].tpTextAttribute;
+    textAttributeNumber= tp[1].tpTextAttributeNumber;
     past= tp[1+partCount].tpStroff+ *pStroffShift;
     stroff= tp[1].tpStroff+ *pStroffShift;
 
@@ -151,7 +93,7 @@ static int docRecalculateParaStringTextParticules(
 	}
 
     partsMade= docRedivideStringInParticules( bi, stroff, newSize,
-						part+ 1, partCount, ta );
+				    part+ 1, partCount, textAttributeNumber );
     if  ( partsMade < partCount )
 	{
 	docDeleteParticules( bi, part+ 1+ partsMade, partCount- partsMade );
@@ -193,17 +135,15 @@ static int docRecalculateParaStringTextParticules(
 /************************************************************************/
 
 static int docRecalculateParaTextFields(
-				int *			pChanged,
-				BufferDocument *	bd,
+				RecalculateFields *	rf,
 				int *			pPartShift,
 				int *			pStroffShift,
 				BufferItem *		bi,
 				int			part,
-				int			partUpto,
-				unsigned int		whenMask,
-				void *			voidadd,
-				DOC_CLOSE_OBJECT	closeObject )
+				int			partUpto )
     {
+    BufferDocument *	bd= rf->rfBd;
+
     /*  1  */
     for ( part= part; part < partUpto+ *pPartShift; part++ )
 	{
@@ -216,7 +156,8 @@ static int docRecalculateParaTextFields(
 	int				partCount;
 
 	/*  2  */
-	tp->tpStroff += *pStroffShift;
+	if  ( docShiftParticuleOffsets( bd, bi, part, part+ 1, *pStroffShift ) )
+	    { LDEB(*pStroffShift);	}
 
 	/*  3  */
 	if  ( tp->tpKind != DOCkindFIELDSTART )
@@ -229,6 +170,13 @@ static int docRecalculateParaTextFields(
 	    }
 
 	/*  4  */
+	if  ( tp->tpObjectNumber < 0				||
+	      tp->tpObjectNumber >= bd->bdFieldList.dflFieldCount	)
+	    {
+	    LLDEB(tp->tpObjectNumber,bd->bdFieldList.dflFieldCount);
+	    return -1;
+	    }
+
 	df= bd->bdFieldList.dflFields+ tp->tpObjectNumber;
 	if  ( df->dfKind < 0 || df->dfKind >= DOC_FieldKindCount )
 	    { LLDEB(df->dfKind,DOC_FieldKindCount); continue;	}
@@ -246,11 +194,11 @@ static int docRecalculateParaTextFields(
 	/*  5  */
 	partCount= docCountParticulesInField( bi, part, partUpto+ *pPartShift );
 	if  ( partCount < 0 )
-	    { LDEB(partCount); continue;	}
+	    { SLDEB(docFieldKindStr(df->dfKind),partCount); continue;	}
 	endPart= part+ 1+ partCount;
 
 	/*  6  */
-	if  ( ( fki->fkiCalculateWhen & whenMask )		&&
+	if  ( ( fki->fkiCalculateWhen & rf->rfUpdateFlags )	&&
 	      fki->fkiCalculateTextParticules			)
 	    {
 	    int			partShift= 0;
@@ -258,21 +206,19 @@ static int docRecalculateParaTextFields(
 
 	    if  ( (*fki->fkiCalculateTextParticules)( &calculated, bd,
 			&partShift, pStroffShift, bi, part, partCount, df,
-			voidadd, closeObject ) )
+			rf->rfVoidadd, rf->rfCloseObject ) )
 		{ LDEB(1); return -1;	}
 
 	    if  ( calculated )
-		{ *pChanged= 1; }
+		{ rf->rfFieldsUpdated++; }
 
 	    endPart += partShift;
 	    *pPartShift += partShift;
 	    }
 	else{
-	    int		p;
-
-	    tp= bi->biParaParticules+ part+ 1;
-	    for ( p= part+ 1; p < endPart; tp++, p++ )
-		{ tp->tpStroff += *pStroffShift; }
+	    if  ( docShiftParticuleOffsets( bd, bi,
+					part+ 1, endPart, *pStroffShift ) )
+		{ LDEB(*pStroffShift);	}
 	    }
 
 	/*  7  */
@@ -281,10 +227,8 @@ static int docRecalculateParaTextFields(
 	    int			partShift= 0;
 	    int			stroffShift= 0;
 
-	    if  ( docRecalculateParaTextFields( pChanged, bd,
-					&partShift, &stroffShift,
-					bi, part+ 1, endPart, whenMask,
-					voidadd, closeObject ) )
+	    if  ( docRecalculateParaTextFields( rf,
+			    &partShift, &stroffShift, bi, part+ 1, endPart ) )
 		{ LDEB(1); return -1;	}
 
 	    endPart += partShift;
@@ -312,13 +256,45 @@ static int docRecalculateParaTextFields(
 /*									*/
 /************************************************************************/
 
-int docRecalculateTextLevelFields(	int *			pChanged,
-					BufferDocument *	bd,
-					BufferItem *		bi,
-					unsigned int		whenMask,
-					void *			voidadd,
-					DOC_CLOSE_OBJECT	closeObject )
+int docRecalculateTextLevelFieldsInExternalItem(
+					RecalculateFields *	rf,
+					ExternalItem *		ei,
+					const BufferItem *	bodySectBi,
+					int			page )
     {
+    int		rval= 0;
+    int		ret;
+    int		saveFieldsUpdated= rf->rfFieldsUpdated;
+
+    if  ( ! ei->eiItem )
+	{ return 0;	}
+
+    if  ( ei->eiItem->biSectHeaderFooterUseForSectBi )
+	{ XDEB(ei->eiItem->biSectHeaderFooterUseForSectBi); return -1;	}
+
+    ei->eiItem->biSectHeaderFooterUseForSectBi= bodySectBi;
+    ei->eiItem->biSectHeaderFooterUseForPage= page;
+
+    rf->rfFieldsUpdated= 0;
+
+    ret= docRecalculateTextLevelFields( rf, ei->eiItem );
+    if  ( ret )
+	{ LDEB(ret); rval= -1;	}
+
+    if  ( rf->rfFieldsUpdated > 0 )
+	{ ei->eiPageFormattedFor= -1; }
+
+    rf->rfFieldsUpdated += saveFieldsUpdated;
+
+    ei->eiItem->biSectHeaderFooterUseForSectBi= (const BufferItem *)0;
+
+    return rval;
+    }
+
+int docRecalculateTextLevelFields(	RecalculateFields *	rf,
+					BufferItem *		bi )
+    {
+    BufferDocument *	bd= rf->rfBd;
     int			i;
 
     int			partShift;
@@ -327,16 +303,113 @@ int docRecalculateTextLevelFields(	int *			pChanged,
     switch( bi->biLevel )
 	{
 	case DOClevDOC:
-	case DOClevSECT:
+	    for ( i= 0; i < bi->biChildCount; i++ )
+		{
+		if  ( docRecalculateTextLevelFields( rf, bi->biChildren[i] ) )
+		    { LDEB(i); return -1;	}
+		}
+
+	    {
+	    const int	page= -1;
+
+	    if  ( docRecalculateTextLevelFieldsInExternalItem( rf,
+			&(bd->bdEiFtnsep), (BufferItem *)0, page )	)
+		{ LDEB(1); return -1;	}
+
+	    if  ( docRecalculateTextLevelFieldsInExternalItem( rf,
+			&(bd->bdEiFtnsepc), (BufferItem *)0, page )	)
+		{ LDEB(1); return -1;	}
+
+	    if  ( docRecalculateTextLevelFieldsInExternalItem( rf,
+			&(bd->bdEiFtncn), (BufferItem *)0, page )	)
+		{ LDEB(1); return -1;	}
+
+	    if  ( docRecalculateTextLevelFieldsInExternalItem( rf,
+			&(bd->bdEiAftnsep), (BufferItem *)0, page )	)
+		{ LDEB(1); return -1;	}
+
+	    if  ( docRecalculateTextLevelFieldsInExternalItem( rf,
+			&(bd->bdEiAftnsepc), (BufferItem *)0, page )	)
+		{ LDEB(1); return -1;	}
+
+	    if  ( docRecalculateTextLevelFieldsInExternalItem( rf,
+			&(bd->bdEiAftncn), (BufferItem *)0, page )	)
+		{ LDEB(1); return -1;	}
+	    }
+
+	    return 0;
+
 	case DOClevROW:
 	case DOClevCELL:
 	    for ( i= 0; i < bi->biChildCount; i++ )
 		{
-		if  ( docRecalculateTextLevelFields( pChanged, bd,
-					    bi->biChildren[i], whenMask,
-					    voidadd, closeObject ) )
+		if  ( docRecalculateTextLevelFields( rf, bi->biChildren[i] ) )
 		    { LDEB(i); return -1;	}
 		}
+
+	    return 0;
+
+	case DOClevSECT:
+	    for ( i= 0; i < bi->biChildCount; i++ )
+		{
+		if  ( docRecalculateTextLevelFields( rf, bi->biChildren[i] ) )
+		    { LDEB(i); return -1;	}
+		}
+
+	    if  ( bi->biInExternalItem == DOCinBODY )
+		{
+		DocumentNote *	dn;
+		const int	page= -1;
+
+		if  ( docRecalculateTextLevelFieldsInExternalItem( rf,
+				    &(bi->biSectHeader), bi, page ) )
+		    { LDEB(1); return -1;	}
+
+		if  ( docRecalculateTextLevelFieldsInExternalItem( rf,
+				    &(bi->biSectFirstPageHeader), bi, page ) )
+		    { LDEB(1); return -1;	}
+
+		if  ( docRecalculateTextLevelFieldsInExternalItem( rf,
+				    &(bi->biSectLeftPageHeader), bi, page ) )
+		    { LDEB(1); return -1;	}
+
+		if  ( docRecalculateTextLevelFieldsInExternalItem( rf,
+				    &(bi->biSectRightPageHeader), bi, page ) )
+		    { LDEB(1); return -1;	}
+
+
+		if  ( docRecalculateTextLevelFieldsInExternalItem( rf,
+				    &(bi->biSectFooter), bi, page ) )
+		    { LDEB(1); return -1;	}
+
+		if  ( docRecalculateTextLevelFieldsInExternalItem( rf,
+				    &(bi->biSectFirstPageFooter), bi, page ) )
+		    { LDEB(1); return -1;	}
+
+		if  ( docRecalculateTextLevelFieldsInExternalItem( rf,
+				    &(bi->biSectLeftPageFooter), bi, page ) )
+		    { LDEB(1); return -1;	}
+
+		if  ( docRecalculateTextLevelFieldsInExternalItem( rf,
+				    &(bi->biSectRightPageFooter), bi, page ) )
+		    { LDEB(1); return -1;	}
+
+		dn= bd->bdNotes;
+		for ( i= 0; i < bd->bdNoteCount; dn++, i++ )
+		    {
+		    ExternalItem *	ei= &(dn->dnExternalItem);
+
+		    if  ( dn->dnSectNr != bi->biNumberInParent )
+			{ continue;	}
+		    if  ( ! ei->eiItem )
+			{ continue;	}
+
+		    if  ( docRecalculateTextLevelFieldsInExternalItem( rf,
+							    ei, bi, page ) )
+			{ LDEB(1); return -1;	}
+		    }
+		}
+
 	    return 0;
 
 	case DOClevPARA:
@@ -344,11 +417,10 @@ int docRecalculateTextLevelFields(	int *			pChanged,
 	    partShift= 0;
 	    stroffShift= 0;
 
-	    if  ( docRecalculateParaTextFields( pChanged, bd,
-				&partShift, &stroffShift,
-				bi, 0, bi->biParaParticuleCount, whenMask,
-				voidadd, closeObject ) )
+	    if  ( docRecalculateParaTextFields( rf, &partShift, &stroffShift,
+					bi, 0, bi->biParaParticuleCount ) )
 		{ LDEB(1); return -1;	}
+
 	    return 0;
 
 	default:
@@ -390,9 +462,9 @@ const FieldKindInformation DOC_FieldKinds[]=
 		DOClevTEXT,
 		FIELD_IN_RTF,
 		NO_DEST,
-		(CALCULATE_TEXT_PARTICULES)0,
+		docRecalculateHyperlinkField,
 		(CALCULATE_TEXT_STRING)0,
-		FIELDdoNEVER,
+		FIELDdoDOC_COMPLETE, /* Must Be: FIELDdoNEVER, */
 		RESULT_EDITABLE,
     },
     {
@@ -435,6 +507,26 @@ const FieldKindInformation DOC_FieldKinds[]=
 		FIELDdoNEVER,
 		RESULT_EDITABLE,
     },
+    {
+	"SECTION", (char *)0,
+		DOClevTEXT,
+		FIELD_IN_RTF,
+		NO_DEST,
+		docRecalculateParaStringTextParticules,
+		docCalculateSectionFieldString,
+		FIELDdoDOC_FORMATTED,
+		RESULT_READONLY,
+    },
+    {
+	"SECTIONPAGES", (char *)0,
+		DOClevTEXT,
+		FIELD_IN_RTF,
+		NO_DEST,
+		docRecalculateParaStringTextParticules,
+		docCalculateSectionPagesFieldString,
+		FIELDdoDOC_FORMATTED,
+		RESULT_READONLY,
+    },
     {
 	"PAGE", (char *)0,
 		DOClevTEXT,
@@ -525,6 +617,16 @@ const FieldKindInformation DOC_FieldKinds[]=
 		FIELDdoDOC_INFO,
 		RESULT_READONLY,
     },
+    {
+	"DOCCOMM", (char *)0,
+		DOClevTEXT,
+		FIELD_IN_RTF,
+		NO_DEST,
+		docRecalculateParaStringTextParticules,
+		docCalculateDocStringFieldString,
+		FIELDdoDOC_INFO,
+		RESULT_READONLY,
+    },
     {
 	"KEYWORDS", (char *)0,
 		DOClevTEXT,
@@ -566,13 +668,13 @@ const FieldKindInformation DOC_FieldKinds[]=
 		RESULT_READONLY,
     },
     {
-	"-FOOTNOTE", (char *)0,
+	"SYMBOL", (char *)0,
 		DOClevTEXT,
-		NOT_IN_RTF,
+		FIELD_IN_RTF,
 		NO_DEST,
-		(CALCULATE_TEXT_PARTICULES)0,
-		(CALCULATE_TEXT_STRING)0,
-		FIELDdoNEVER,
+		docRecalculateParaSymbolTextParticules,
+		docCalculateSymbolFieldString,
+		FIELDdoDOC_COMPLETE,
 		RESULT_READONLY,
     },
     {
@@ -580,9 +682,9 @@ const FieldKindInformation DOC_FieldKinds[]=
 		DOClevTEXT,
 		NOT_IN_RTF,
 		NO_DEST,
-		(CALCULATE_TEXT_PARTICULES)0,
-		(CALCULATE_TEXT_STRING)0,
-		FIELDdoNEVER,
+		docRecalculateParaChftnTextParticules,
+		docCalculateChftnFieldString,
+		FIELDdoDOC_COMPLETE|FIELDdoCHFTN,
 		RESULT_READONLY,
     },
     {
