@@ -4,7 +4,7 @@
 /*									*/
 /************************************************************************/
 
-#   include	"config.h"
+#   include	"tedConfig.h"
 
 #   include	<stdlib.h>
 #   include	<string.h>
@@ -35,18 +35,20 @@ DocumentField *	docClaimFieldCopy(	DocumentFieldList *	dfl,
     DocumentField *	dfTo;
     int			fieldNumber;
 
+    DocumentField	dfSaved= *dfFrom;
+
     dfTo= docClaimField( &fieldNumber, dfl );
     if  ( ! dfTo )
 	{ XDEB(dfTo); return (DocumentField *)0;	}
 
-    dfTo->dfKind= dfFrom->dfKind;
+    dfTo->dfKind= dfSaved.dfKind;
 
-    if  ( utilCopyMemoryBuffer( &dfTo->dfData, &dfFrom->dfData ) 	||
-	  utilCopyMemoryBuffer( &dfTo->dfInstructions,
-					    &dfFrom->dfInstructions )	)
+    if  ( utilCopyMemoryBuffer( &(dfTo->dfData), &(dfSaved.dfData) ) 	||
+	  utilCopyMemoryBuffer( &(dfTo->dfInstructions),
+					&(dfSaved.dfInstructions) )	)
 	{
 	LDEB(1);
-	docCleanField( (BufferDocument *)0, dfTo );
+	docCleanField( dfTo );
 	docInitField( dfTo );
 	return (DocumentField *)0;
 	}
@@ -56,19 +58,15 @@ DocumentField *	docClaimFieldCopy(	DocumentFieldList *	dfl,
 
 /************************************************************************/
 /*									*/
-/*  Management of links.						*/
+/*  Management of fields.						*/
 /*									*/
 /************************************************************************/
 
-void docCleanField(	BufferDocument *	bd,
-			DocumentField *		df )
+void docCleanField(	DocumentField *		df )
     {
     utilCleanMemoryBuffer( &df->dfData );
     utilCleanMemoryBuffer( &df->dfInstructions );
 
-    if  ( df->dfReferencedItem )
-	{ docFreeItem( bd, df->dfReferencedItem );	}
-
     return;
     }
 
@@ -80,8 +78,6 @@ void docInitField(	DocumentField *	df )
     df->dfPage= -1;
     df->dfKind= DOCfkFREE;
 
-    df->dfReferencedItem= (BufferItem *)0;
-
     return;
     }
 
@@ -129,8 +125,7 @@ void docInitFieldList(	DocumentFieldList *	dfl )
     return;
     }
 
-void docCleanFieldList(	BufferDocument *	bd,
-			DocumentFieldList *	dfl )
+void docCleanFieldList(	DocumentFieldList *	dfl )
     {
     int			i;
     DocumentField *	df;
@@ -141,7 +136,7 @@ void docCleanFieldList(	BufferDocument *	bd,
 	if  ( df->dfKind == DOCfkFREE )
 	    { continue;	}
 
-	docCleanField( bd, df );
+	docCleanField( df );
 	}
 
     if  ( dfl->dflFields )
@@ -229,3 +224,55 @@ void docAdaptBookmarkName(	int *		pLength,
     *pLength= len; return;
     }
 
+/************************************************************************/
+/*									*/
+/*  Look for a bookmark in the field list.				*/
+/*									*/
+/************************************************************************/
+
+int docGetBookmarkField(	DocumentField **		pDf,
+				const char *			markName,
+				int				markSize,
+				const DocumentFieldList *	dfl )
+    {
+    DocumentField *	df;
+    int			i;
+
+    df= dfl->dflFields;
+    for ( i= 0; i < dfl->dflFieldCount; df++, i++ )
+	{
+	const char *	name;
+	int		size;
+
+	if  ( df->dfKind != DOCfkBOOKMARK		||
+	      docFieldGetBookmark( df, &name, &size )	)
+	    { continue;		}
+
+	if  ( size == 0			||
+	      size > DOCmaxBOOKMARK	)
+	    { continue;	}
+
+	if  ( size == markSize			&&
+	      ! strncmp( markName, name, size )	)
+	    { *pDf= df; return i;	}
+	}
+
+    return -1;
+    }
+
+/************************************************************************/
+/*									*/
+/*  Initialize a field recalculate job.					*/
+/*									*/
+/************************************************************************/
+
+extern void docInitRecalculateFields(   RecalculateFields *     rf )
+    {
+    rf->rfBd= (BufferDocument *)0;
+    rf->rfVoidadd= (void *)0;
+    rf->rfCloseObject= (DOC_CLOSE_OBJECT)0;
+    rf->rfFieldsUpdated= 0;
+    rf->rfUpdateFlags= 0;
+
+    return;
+    }
