@@ -4,7 +4,7 @@
 /*									*/
 /************************************************************************/
 
-#   include	"config.h"
+#   include	"tedConfig.h"
 
 #   include	<stdlib.h>
 #   include	<string.h>
@@ -24,35 +24,6 @@
 
 #   define	FIC_COUNT	15
 
-DocumentField *	docFindBookmarkField(	const DocumentFieldList * dfl,
-					const char *		markName,
-					int			markSize )
-    {
-    DocumentField *	df;
-    int			i;
-
-    df= dfl->dflFields;
-    for ( i= 0; i < dfl->dflFieldCount; df++, i++ )
-	{
-	const char *	foundName;
-	int		foundSize;
-
-	if  ( df->dfKind != DOCfkBOOKMARK )
-	    { continue;	}
-
-	if  ( docFieldGetBookmark( df, &foundName, &foundSize ) )
-	    { continue;	}
-	if  ( foundSize != markSize )
-	    { continue;	}
-	if  ( memcmp( foundName, markName, markSize ) )
-	    { continue;	}
-
-	return df;
-	}
-
-    return (DocumentField *)0;
-    }
-
 /************************************************************************/
 /*									*/
 /*  See whether the field instructions are to a hyperlink or to a	*/
@@ -63,14 +34,14 @@ DocumentField *	docFindBookmarkField(	const DocumentFieldList * dfl,
 /*									*/
 /************************************************************************/
 
-int docGetBookmarkForPosition(		const BufferDocument *	bd,
-					const BufferPosition *	bp,
-					int *			pBeginPart,
-					int *			pEndPart,
-					const char **		pMarkName,
-					int *			pMarkSize )
+int docGetBookmarkForPosition(	const BufferDocument *		bd,
+				const DocumentPosition *	dp,
+				int *				pBeginPart,
+				int *				pEndPart,
+				const char **			pMarkName,
+				int *				pMarkSize )
     {
-    BufferItem *	bi= bp->bpBi;
+    BufferItem *	bi= dp->dpBi;
     int			beginPart;
     int			endPart;
     TextParticule *	tp;
@@ -80,7 +51,7 @@ int docGetBookmarkForPosition(		const BufferDocument *	bd,
 
     int			beginObjectNumber= -1;
 
-    beginPart= bp->bpParticule;
+    beginPart= dp->dpParticule;
     tp= bi->biParaParticules+ beginPart;
     while( beginPart >= 0 )
 	{
@@ -97,7 +68,7 @@ int docGetBookmarkForPosition(		const BufferDocument *	bd,
     if  ( beginPart < 0 )
 	{ return -1;	}
 
-    endPart= bp->bpParticule;
+    endPart= dp->dpParticule;
     tp= bi->biParaParticules+ endPart;
     while( endPart < bi->biParaParticuleCount )
 	{
@@ -370,16 +341,16 @@ int docFieldGetIncludePicture(	const DocumentField *	df,
     return 0;
     }
 
-int docGetHyperlinkForPosition(		const BufferDocument *	bd,
-					const BufferPosition *	bp,
-					int *			pBeginPart,
-					int *			pEndPart,
-					const char **		pFileName,
-					int *			pFileSize,
-					const char **		pMarkName,
-					int *			pMarkSize )
+int docGetHyperlinkForPosition(	const BufferDocument *		bd,
+				const DocumentPosition *	dp,
+				int *				pBeginPart,
+				int *				pEndPart,
+				const char **			pFileName,
+				int *				pFileSize,
+				const char **			pMarkName,
+				int *				pMarkSize )
     {
-    BufferItem *	bi= bp->bpBi;
+    BufferItem *	bi= dp->dpBi;
     int			beginPart;
     int			endPart;
     TextParticule *	tp;
@@ -393,7 +364,7 @@ int docGetHyperlinkForPosition(		const BufferDocument *	bd,
     const char *	markName;
     int			markSize;
 
-    beginPart= bp->bpParticule;
+    beginPart= dp->dpParticule;
     tp= bi->biParaParticules+ beginPart;
     while( beginPart >= 0 )
 	{
@@ -414,7 +385,7 @@ int docGetHyperlinkForPosition(		const BufferDocument *	bd,
     if  ( beginPart < 0 )
 	{ return -1;	}
 
-    endPart= bp->bpParticule;
+    endPart= dp->dpParticule;
     tp= bi->biParaParticules+ endPart;
     while( endPart < bi->biParaParticuleCount )
 	{
@@ -551,73 +522,89 @@ int docFieldSetPageref(		DocumentField *			df,
 
 /************************************************************************/
 /*									*/
-/*  Copy a hyperlink to another paragraph.				*/
+/*  Extract the values from a symbol field.				*/
 /*									*/
-/*  If the hyperlink is relative to the name of the target document, it */
-/*  it is converted to a relative link (the document name is removed)	*/
 /*									*/
 /************************************************************************/
 
-int docCopyFieldRelative(	BufferDocument *	bdTo,
-				const BufferDocument *	bdFrom,
-				int *			fieldMap,
-				unsigned int *		pFieldUpd,
-				BufferItem *		biTo,
-				const BufferItem *	biFrom,
-				TextParticule *		tpTo,
-				const TextParticule *	tpFrom,
-				const char *		refFileName,
-				int			refFileSize )
+int docFieldGetSymbol(		const DocumentField *	df,
+				const char **		pFontName,
+				int *			pFontSize,
+				const char **		pSymbName,
+				int *			pSymbSize,
+				const char **		pSizeString,
+				int *			pSizeSize )
     {
-    DocumentField *	dfFrom;
+    FieldInstructionsComponent	fic[FIC_COUNT];
+    int				n;
+    int				comp;
+    unsigned char *		bytes= df->dfInstructions.mbBytes;
 
-    const char *	fileName;
-    int			fileSize;
-    const char *	markName;
-    int			markSize;
+    const char *		fontName= (const char *)0;
+    int				fontSize= 0;
+    const char *		symbName= (const char *)0;
+    int				symbSize= 0;
+    const char *		sizeString= (const char *)0;
+    int				sizeSize= 0;
 
-    dfFrom= bdFrom->bdFieldList.dflFields+ tpFrom->tpObjectNumber;
+    if  ( df->dfKind != DOCfkSYMBOL )
+	{ return -1;	}
 
-    if  ( ! docFieldGetHyperlink( dfFrom, &fileName, &fileSize,
-						&markName, &markSize )	&&
-	  fileSize == refFileSize 					&&
-	  markSize > 0							&&
-	  ! strncmp( fileName, refFileName, refFileSize )		)
+    n= docSplitFieldInstructions( &(df->dfInstructions), fic, FIC_COUNT );
+    if  ( n < 2 )
+	{ LDEB(n); return -1;	}
+
+    comp= 0;
+    if  ( fic[comp].ficSize != 6				||
+	  memcmp( bytes+ fic[comp].ficOffset, "SYMBOL", 6 )	)
+	{ SDEB((char *)bytes); return -1;	}
+    comp++;
+
+    while( comp < n )
 	{
-	const DocumentFieldList *	dflFrom= &(bdFrom->bdFieldList);
-	DocumentField *			dfTo;
-	char *				copy= (char *)malloc( markSize+ 1 );
-	int				fieldNumber;
+	if  ( fic[comp].ficSize == 2				&&
+	      ! memcmp( bytes+ fic[comp].ficOffset, "\\f", 2 )	)
+	    {
+	    fontName= (char *)bytes+ fic[comp+ 1].ficOffset;
+	    fontSize= fic[comp+ 1].ficSize;
+	    comp += 2; continue;
+	    }
 
-	if  ( ! copy )
-	    { XDEB(copy); return -1;	}
+	if  ( fic[comp].ficSize == 2				&&
+	      ! memcmp( bytes+ fic[comp].ficOffset, "\\h", 2 )	)
+	    { comp++; continue;	}
 
-	strncpy( copy, markName, markSize )[markSize]= '\0';
+	if  ( fic[comp].ficSize == 2				&&
+	      ! memcmp( bytes+ fic[comp].ficOffset, "\\s", 2 )	)
+	    {
+	    sizeString= (char *)bytes+ fic[comp+ 1].ficOffset;
+	    sizeSize= fic[comp+ 1].ficSize;
+	    comp += 2; continue;
+	    }
 
-	dfTo= docClaimField( &fieldNumber, &(bdTo->bdFieldList) );
-	if  ( ! dfTo )
-	    { XDEB(dfTo); return -1;	}
+	if  ( bytes[fic[comp].ficOffset] != '\\' )
+	    {
+	    symbName= (char *)bytes+ fic[comp].ficOffset;
+	    symbSize= fic[comp].ficSize;
+	    comp++; continue;
+	    }
 
-	if  ( docFieldSetHyperlink( dfTo, (unsigned char *)0, 0,
-					(unsigned char *)copy, markSize ) )
-	    { SDEB(copy); return -1;	}
-
-	dfTo->dfKind= dfFrom->dfKind;
-	tpTo->tpObjectNumber= fieldNumber;
-
-	if  ( tpFrom->tpObjectNumber < 0				||
-	      tpFrom->tpObjectNumber >= dflFrom->dflFieldCount	)
-	    { LDEB(tpFrom->tpObjectNumber); }
-	else{ fieldMap[tpFrom->tpObjectNumber]= fieldNumber;		}
-
-	free( copy );
-
-	return 0;
+	SDEB((char *)bytes);
+	break;
 	}
 
-    if  ( docCopyParticuleData( bdTo, bdFrom, fieldMap, pFieldUpd,
-						biTo, biFrom, tpTo, tpFrom ) )
-	{ LDEB(1); return -1;	}
+    while( comp < n )
+	{ LSDEB(comp,(char *)bytes+ fic[comp].ficOffset); comp++; }
+
+    if  ( ! symbName )
+	{ XDEB(symbName); return -1;	}
+
+    *pFontName= fontName;
+    *pFontSize= fontSize;
+    *pSymbName= symbName;
+    *pSymbSize= symbSize;
+    *pSizeString= sizeString;
+    *pSizeSize= sizeSize;
 
     return 0;
     }
