@@ -9,11 +9,6 @@
 #   include	<stddef.h>
 #   include	<stdio.h>
 #   include	<stdlib.h>
-#   define	y0	math_y0
-#   define	y1	math_y1
-#   include	<math.h>
-#   undef	y0
-#   undef	y1
 
 #   include	<appImage.h>
 #   include	"docLayout.h"
@@ -24,24 +19,49 @@
 /*									*/
 /*  Initialisation for paragraph layout.				*/
 /*									*/
+/*  1)  Optimalisation: If the previous particule has the same text	*/
+/*	attributes, it MUST have the same physical font.		*/
+/*  2)  If no physical font has been determined for this particule,	*/
+/*	open its physical font.						*/
+/*  2a) Translate attribute number to the actual attribute values.	*/
+/*  2b) Get the physical font for those attributes.			*/
+/*									*/
 /************************************************************************/
 
-static int docLayoutOpenParaFonts(	BufferItem *			paraBi,
-					AppDrawingData *		add,
-					const DocumentProperties *	dp )
+static int docLayoutOpenParaFonts(	const BufferDocument *		bd,
+					BufferItem *			paraBi,
+					AppDrawingData *		add )
     {
+    const DocumentProperties *	dp= &(bd->bdProperties);
+    const DocumentFontList *	dfl= &(dp->dpFontList);
+
     int				part;
     TextParticule *		tp;
 
-    const DocumentFontList *	dfl= &(dp->dpFontList);
+				/*  1  */
+    int				curTextAttributeNumber= -1;
+    int				curPhysicalFont= -1;
 
     tp= paraBi->biParaParticules;
     for ( part= 0; part < paraBi->biParaParticuleCount; tp++, part++ )
 	{
+	/*  1  */
+	if  ( tp->tpPhysicalFont < 0					&&
+	      curTextAttributeNumber >= 0				&&
+	      tp->tpTextAttributeNumber == curTextAttributeNumber	)
+	    { tp->tpPhysicalFont= curPhysicalFont;	}
+
+	/*  2  */
 	if  ( tp->tpPhysicalFont < 0 )
 	    {
-	    tp->tpPhysicalFont= appOpenDocumentFont( add, dfl,
-							tp->tpTextAttribute );
+	    TextAttribute	ta;
+
+	    /*  2a  */
+	    utilGetTextAttributeByNumber( &ta, &(bd->bdTextAttributeList),
+						tp->tpTextAttributeNumber );
+
+	    /*  2b  */
+	    tp->tpPhysicalFont= appOpenDocumentFont( add, dfl, ta );
 
 	    if  ( tp->tpPhysicalFont < 0 )
 		{
@@ -51,16 +71,23 @@ static int docLayoutOpenParaFonts(	BufferItem *			paraBi,
 		return -1;
 		}
 	    }
+
+	/*  1  */
+	if  ( tp->tpPhysicalFont >= 0 )
+	    {
+	    curTextAttributeNumber= tp->tpTextAttributeNumber;
+	    curPhysicalFont= tp->tpPhysicalFont;
+	    }
 	}
 
     return 0;
     }
 
-static int docLayoutParaInit(	BufferItem *			paraBi,
-				AppDrawingData *		add,
-				const DocumentProperties *	dp )
+static int docLayoutParaInit(	const BufferDocument *		bd,
+				BufferItem *			paraBi,
+				AppDrawingData *		add )
     {
-    if  ( docLayoutOpenParaFonts( paraBi, add, dp ) )
+    if  ( docLayoutOpenParaFonts( bd, paraBi, add ) )
 	{ LDEB(1); return -1;	}
 
     if  ( docPsParagraphLineExtents( &(add->addPhysicalFontList), paraBi ) )
@@ -71,534 +98,13 @@ static int docLayoutParaInit(	BufferItem *			paraBi,
     return 0;
     }
 
-static int docPsPlaceParaInit(	BufferItem *			paraBi,
-				AppDrawingData *		add,
-				const DocumentProperties *	dp )
-    {
-    return 0;
-    }
-
-/************************************************************************/
-/*									*/
-/*  Layout successive lines of a paragraph.				*/
-/*  Claim space above a particular line.				*/
-/*									*/
-/*  1)  Cope with the output of jade: Ignore enormous space before's	*/
-/*	in footers.							*/
-/*									*/
-/************************************************************************/
-
-static void docPsAboveLine(	LayoutPosition *		lp,
-				int				fromLinePos,
-				const BufferItem *		paraBi,
-				int				part,
-				const DocumentGeometry *	dg,
+static int docPlaceParaInit(	const BufferDocument *		bd,
+				BufferItem *			paraBi,
 				AppDrawingData *		add )
     {
-    double		xfac= add->addMagnifiedPixelsPerTwip;
-
-    int			spaceAboveLineTwips= 0;
-    int			spaceAboveLinePixels= 0;
-
-    /*  1  */
-    if  ( part == 0					&&
-	  ! fromLinePos					&&
-	  paraBi->biInExternalItem == DOCinBODY		)
-	{
-	int		isFirst= 0;
-
-	if  ( paraBi->biParaInTable )
-	    {
-	    if  ( paraBi->biNumberInParent == 0 )
-		{ isFirst= 1;	}
-	    }
-	else{
-	    if  ( lp->lpPageYTwips == dg->dgTopMarginTwips )
-		{ isFirst= 1;	}
-	    }
-
-	spaceAboveLineTwips= paraBi->biParaSpaceAboveLinesTwips;
-
-	if  ( ! isFirst )
-	    { spaceAboveLineTwips += paraBi->biParaSpaceBeforeTwips;	}
-
-	spaceAboveLinePixels= TWIPStoPIXELS( xfac, spaceAboveLineTwips );
-	}
-
-    lp->lpPageYTwips += spaceAboveLineTwips;
-    /* still at top */
-
-    return;
-    }
-
-/************************************************************************/
-/*									*/
-/*  Layout successive lines of a paragraph.				*/
-/*  Claim space below a particular line.				*/
-/*									*/
-/*  Also decide whether it will fit in the current formatting frame.	*/
-/*									*/
-/************************************************************************/
-
-static int docPsBelowLine(	const LayoutPosition *		lpTop,
-				LayoutPosition *		lp,
-				const TextLine *		tl,
-				const BufferItem *		paraBi,
-				int				past,
-				const ParagraphFrame *		pf,
-				const NotesReservation *	nrLine,
-				AppDrawingData *		add )
-    {
-    int			spaceBelowLineTwips= 0;
-    int			lineBottom;
-    int			lineHeight;
-    int			frameBottom;
-
-    int			isLast= 0;
-    int			footnoteHeight;
-
-    if  ( past == paraBi->biParaParticuleCount )
-	{
-	if  ( paraBi->biParaInTable )
-	    {
-	    if  ( paraBi->biNumberInParent ==
-					paraBi->biParent->biChildCount- 1 )
-		{ isLast= 1;	}
-	    }
-
-	if  ( ! isLast )
-	    {
-	    spaceBelowLineTwips= paraBi->biParaSpaceBelowLinesTwips;
-	    spaceBelowLineTwips += paraBi->biParaSpaceAfterTwips;
-	    }
-	}
-
-    lineBottom= lp->lpPageYTwips+ tl->tlLineHeightTwips+ spaceBelowLineTwips;
-    lineHeight= lineBottom- lpTop->lpPageYTwips;
-
-    footnoteHeight= nrLine->nrFtnsepHeight+ nrLine->nrFootnoteHeight;
-    frameBottom= pf->pfPageY1Twips- footnoteHeight;
-
-    if  ( pf->pfHasBottom				&&
-	  lineBottom > frameBottom			&&
-	  lineHeight < pf->pfPageHigh			&&
-	  ( pf->pfStripHigh < 0			||
-	    lineHeight < pf->pfStripHigh	)	)
-	{
-	lp->lpPageYTwips= pf->pfPageY1Twips+ 1;
-	lp->lpAtTopOfColumn= 0;
-	return 1;
-	}
-
-    lp->lpPageYTwips += tl->tlLineSpacingTwips;
-    lp->lpAtTopOfColumn= 0;
-
-    if  ( past == paraBi->biParaParticuleCount /* && ! isLast */ )
-	{
-	spaceBelowLineTwips= paraBi->biParaSpaceBelowLinesTwips;
-
-	lp->lpPageYTwips += spaceBelowLineTwips;
-	lp->lpAtTopOfColumn= 0;
-	}
-
     return 0;
     }
 
-/************************************************************************/
-/*									*/
-/*  Layout successive lines of a paragraph.				*/
-/*  Place a line on the page without reformatting its text. Just the	*/
-/*  position changes, not the layout of the text inside the line.	*/
-/*									*/
-/************************************************************************/
-
-static int docPsPlace_Line(	TextLine *			resTl,
-				NotesReservation *		pNrLine,
-				const BufferItem *		paraBi,
-				int				part,
-				ParticuleData *			pd,
-				const ParagraphLayoutContext *	plc,
-				const ParagraphFrame *		pf,
-				const LayoutPosition *		lpTop,
-				LayoutPosition *		lpBottom,
-				const ScreenLayout *		sl )
-    {
-    BufferDocument *		bd= plc->plcBd;
-    AppDrawingData *		add= plc->plcAdd;
-
-    int				accepted;
-    int				res;
-    int				i;
-
-    TextParticule *		tp= paraBi->biParaParticules+ part;
-
-    TextLine			workTl;
-    TextLine *			tl= &(workTl);
-
-    LayoutPosition		lp;
-    const SectionProperties *	sp;
-    const DocumentGeometry *	dg;
-
-    const int			fromLinePos= 0;
-
-    NotesReservation		nrLine;
-
-    lp= *lpTop;
-    sp= &(paraBi->biParent->biParent->biParent)->biSectProperties;
-    dg= &(sp->spDocumentGeometry);
-
-    workTl= *resTl;
-    /*
-    tl->tlStroff= tp->tpStroff;
-    tl->tlFirstParticule= part;
-    */
-
-    docInitNotesReservation( &nrLine );
-
-    /*  1  */
-    docPsAboveLine( &lp, fromLinePos, paraBi, part, dg, add );
-
-    tl->tlTopPosition= lp;
-    accepted= tl->tlParticuleCount;
-
-    if  ( accepted < 1 )
-	{ LDEB(accepted); docListItem(0,paraBi); return -1; }
-
-    if  ( docLayoutCollectParaFootnoteHeight( &nrLine,
-					    tl->tlTopPosition.lpPage,
-					    tl->tlTopPosition.lpColumn,
-					    bd, paraBi, tl->tlFirstParticule,
-					    tl->tlFirstParticule+ accepted ) )
-	{ LDEB(1); return -1;	}
-
-    res= docPsBelowLine( lpTop, &lp, tl, paraBi, part+ accepted,
-							pf, &nrLine, add );
-    if  ( res < 0 )
-	{ LDEB(res); return -1;	}
-    if  ( res > 0 )
-	{ *lpBottom= lp; return 0; }
-
-    if  ( sl && sl->slLayoutLine					&&
-	  (*sl->slLayoutLine)( tl, paraBi, part, accepted, add,
-							pf, pd ) < 0	)
-	{ LDEB(accepted); return -1;	}
-
-    tl->tlX0Pixels= pf->pfX0Pixels;
-    tl->tlX1Pixels= pf->pfX1Pixels;
-
-    for ( i= 0; i < accepted; tp++, i++ )
-	{
-	if  ( tp->tpKind == DOCkindFIELDSTART )
-	    {
-	    DocumentField *	df;
-
-	    df= bd->bdFieldList.dflFields+ tp->tpObjectNumber;
-
-	    df->dfPage= lp.lpPage;
-	    }
-	}
-
-    *lpBottom= lp;
-    *pNrLine= nrLine;
-    *resTl= *tl;
-
-    return accepted;
-    }
-
-/************************************************************************/
-/*									*/
-/*  Layout successive lines of a paragraph.				*/
-/*  Place the lines on the page without reformatting their text. Just	*/
-/*  the positions of the lines change, not the layout of the text	*/
-/*  inside the line.							*/
-/*									*/
-/************************************************************************/
-
-static int docPsPlaceLines(	ParagraphLayoutPosition *	plp,
-				BlockFrame *			bf,
-				const ParagraphLayoutContext *	plc,
-				BufferItem *			paraBi )
-    {
-    ParagraphStripPosition *	psp= &(plp->plpProgress);
-    int				part= psp->pspPart;
-
-    int				done= 0;
-    int				line= psp->pspLine;
-
-    TextLine *			tl= paraBi->biParaLines+ line;
-
-    ParticuleData *		pd= (ParticuleData *)0;
-
-    LayoutPosition		lp;
-
-    lp= plp->plpPos;
-
-    while( part < paraBi->biParaParticuleCount )
-	{
-	NotesReservation	nrLine;
-	int			accepted;
-	TextLine		boxLine;
-
-	DocumentRectangle *	drChanged= plc->plcChangedRectanglePixels;
-
-	docInitNotesReservation( &nrLine );
-
-	if  ( line >= paraBi->biParaLineCount )
-	    { done= paraBi->biParaParticuleCount- part; break;	}
-
-	boxLine= *tl;
-
-	accepted= docPsPlace_Line( &boxLine, &nrLine,
-				    paraBi, part, pd, plc,
-				    &(plp->plpFormattingFrame),
-				    &lp, &lp, &(plc->plcScreenLayout) );
-
-	if  ( accepted < 0 )
-	    { LLDEB(line,accepted); return -1; }
-
-	docLayoutReserveNoteHeight( &(plp->plpFormattingFrame), bf, &nrLine );
-
-	if  ( drChanged	)
-	    {
-	    DocumentRectangle	drTl;
-	    DocumentRectangle	drBox;
-
-	    drTl.drX0= tl->tlX0Pixels;
-	    drTl.drX1= tl->tlX1Pixels;
-	    drTl.drY0= TL_TOP_PIXELS( plc->plcAdd, tl );
-	    drTl.drY1= TL_BELOW_PIXELS( plc->plcAdd, tl );
-
-	    drBox.drX0= boxLine.tlX0Pixels;
-	    drBox.drX1= boxLine.tlX1Pixels;
-	    drBox.drY0= TL_TOP_PIXELS( plc->plcAdd, &boxLine );
-	    drBox.drY1= TL_BELOW_PIXELS( plc->plcAdd, &boxLine );
-
-	    if  ( drTl.drY0 != drBox.drY0	||
-		  drTl.drX0 != drBox.drX0	)
-		{
-		docUnionRectangle( drChanged, drChanged, &drTl );
-		docUnionRectangle( drChanged, drChanged, &drBox );
-		}
-	    }
-
-	if  ( accepted == 0 )
-	    { break;	}
-
-	*tl= boxLine;
-
-	part += accepted; done += accepted; line++; tl++;
-
-	if  ( boxLine.tlHasPageBreak )
-	    {
-	    lp.lpPageYTwips= plp->plpFormattingFrame.pfPageY1Twips+ 1;
-	    lp.lpAtTopOfColumn= 0;
-	    break;
-	    }
-	}
-
-    psp->pspLine= line;
-    plp->plpPos= lp;
-
-    return done;
-    }
-
-/************************************************************************/
-/*									*/
-/*  Layout successive lines of a paragraph.				*/
-/*									*/
-/*  1)  Cope with the output of sgmls: Ignore enormous space before's	*/
-/*	in footers.							*/
-/*									*/
-/************************************************************************/
-
-static int docPsLayout_Line(	TextLine *			resTl,
-				NotesReservation *		pNrLine,
-				int				fromLinePos,
-				const BufferItem *		paraBi,
-				int				part,
-				ParticuleData *			pd,
-				const ParagraphLayoutContext *	plc,
-				const ParagraphFrame *		pf,
-				const LayoutPosition *		lpTop,
-				LayoutPosition *		lpBottom,
-				const ScreenLayout *		sl )
-    {
-    BufferDocument *		bd= plc->plcBd;
-    AppDrawingData *		add= plc->plcAdd;
-
-    const DocumentProperties *	dp= &(bd->bdProperties);
-    int				accepted;
-    int				res;
-    int				i;
-
-    TextParticule *		tp= paraBi->biParaParticules+ part;
-
-    TextLine			workTl;
-    TextLine *			tl= &(workTl);
-
-    LayoutPosition		lp;
-    const SectionProperties *	sp;
-    const DocumentGeometry *	dg;
-
-    NotesReservation		nrLine;
-
-    lp= *lpTop;
-    sp= &(paraBi->biParent->biParent->biParent)->biSectProperties;
-    dg= &(sp->spDocumentGeometry);
-
-    docInitTextLine( tl );
-    tl->tlStroff= tp->tpStroff;
-    tl->tlFirstParticule= part;
-
-    docInitNotesReservation( &nrLine );
-
-    /*  1  */
-    docPsAboveLine( &lp, fromLinePos, paraBi, part, dg, add );
-
-    tl->tlTopPosition= lp;
-    accepted= docLayoutLineBox( tl, paraBi, part,
-			    dp->dpTabIntervalTwips, &dp->dpFontList,
-			    &(add->addPhysicalFontList), tp, pd, pf );
-
-    if  ( accepted < 1 )
-	{ LDEB(accepted); return -1;	}
-
-    if  ( docLayoutCollectParaFootnoteHeight( &nrLine,
-					tl->tlTopPosition.lpPage,
-					tl->tlTopPosition.lpColumn,
-					bd, paraBi, tl->tlFirstParticule,
-					tl->tlFirstParticule+ accepted ) )
-	{ LDEB(1); return -1;	}
-
-    res= docPsBelowLine( lpTop, &lp, tl, paraBi, part+ accepted,
-							pf, &nrLine, add );
-    if  ( res < 0 )
-	{ LDEB(res); return -1;	}
-    if  ( res > 0 )
-	{ *lpBottom= lp; return 0; }
-
-    /* above
-    tl->tlStroff= tp->tpStroff;
-    tl->tlFirstParticule= part;
-    */
-    tl->tlStrlen=
-	    tp[accepted-1].tpStroff+ tp[accepted-1].tpStrlen- tp->tpStroff;
-    tl->tlParticuleCount= accepted;
-
-    if  ( sl && sl->slLayoutLine					&&
-	  (*sl->slLayoutLine)( tl, paraBi, part, accepted, add,
-							pf, pd ) < 0	)
-	{ LDEB(accepted); return -1;	}
-
-    tl->tlX0Pixels= pf->pfX0Pixels;
-    tl->tlX1Pixels= pf->pfX1Pixels;
-
-    for ( i= 0; i < accepted; tp++, i++ )
-	{
-	if  ( tp->tpKind == DOCkindFIELDSTART )
-	    {
-	    DocumentField *	df;
-
-	    df= bd->bdFieldList.dflFields+ tp->tpObjectNumber;
-
-	    df->dfPage= lp.lpPage;
-	    }
-	}
-
-    *lpBottom= lp;
-    *pNrLine= nrLine;
-    *resTl= *tl;
-
-    return accepted;
-    }
-
-static int docLayoutLines(	ParagraphLayoutPosition *	plp,
-				BlockFrame *			bf,
-				const ParagraphLayoutContext *	plc,
-				BufferItem *			paraBi )
-    {
-    ParagraphStripPosition *	psp= &(plp->plpProgress);
-    int				part= psp->pspPart;
-
-    int				done= 0;
-    int				line= psp->pspLine;
-
-
-    ParticuleData *		pd;
-
-    LayoutPosition		lp;
-
-    lp= plp->plpPos;
-
-    if  ( docPsClaimParticuleData( paraBi, &pd ) )
-	{ LDEB(paraBi->biParaParticuleCount); return -1;	}
-
-    while( part < paraBi->biParaParticuleCount )
-	{
-	int			accepted;
-	TextLine		boxLine;
-	TextLine *		tl;
-	const int		fromLinePos= 0;
-
-	NotesReservation	nrLine;
-
-	DocumentRectangle *	drChanged= plc->plcChangedRectanglePixels;
-
-	docInitNotesReservation( &nrLine );
-
-	accepted= docPsLayout_Line( &boxLine, &nrLine,
-				fromLinePos, paraBi, part, pd, plc,
-				&(plp->plpFormattingFrame), &lp, &lp,
-				&(plc->plcScreenLayout) );
-
-	if  ( accepted < 0 )
-	    { LDEB(accepted); return -1;	}
-
-	if  ( accepted == 0 )
-	    { break;	}
-
-	if  ( drChanged	)
-	    {
-	    DocumentRectangle	drBox;
-
-	    drBox.drX0= boxLine.tlX0Pixels;
-	    drBox.drX1= boxLine.tlX1Pixels;
-	    drBox.drY0= TL_TOP_PIXELS( plc->plcAdd, &boxLine );
-	    drBox.drY1= TL_BELOW_PIXELS( plc->plcAdd, &boxLine );
-
-	    docUnionRectangle( drChanged, drChanged, &drBox );
-	    }
-
-	docLayoutReserveNoteHeight( &(plp->plpFormattingFrame), bf, &nrLine );
-
-	if  ( line >= paraBi->biParaLineCount )
-	    {
-	    tl= docInsertTextLine( paraBi, -1 );
-	    if  ( ! tl )
-		{ XDEB(tl); return -1;		}
-	    }
-	else{
-	    tl= paraBi->biParaLines+ line;
-	    }
-
-	*tl= boxLine;
-
-	part += accepted; done += accepted; line++;
-
-	if  ( boxLine.tlHasPageBreak )
-	    {
-	    lp.lpPageYTwips= plp->plpFormattingFrame.pfPageY1Twips+ 1;
-	    lp.lpAtTopOfColumn= 0;
-	    break;
-	    }
-	}
-
-    psp->pspLine= line;
-    plp->plpPos= lp;
-
-    return done;
-    }
-
 static int docLayoutParaItem(	BufferItem *			paraBi,
 				BlockFrame *			bf,
 				LayoutJob *			lj,
@@ -622,60 +128,6 @@ static int docLayoutParaItem(	BufferItem *			paraBi,
     return 0;
     }
 
-/************************************************************************/
-/*									*/
-/*  Adjust the position of the top of a section, depending on the kind	*/
-/*  of break								*/
-/*									*/
-/*  1)  Note that even and odd are swapped as from the users point of	*/
-/*	view page numbers count from one.				*/
-/*  2)  As this routine is only called for the body of the document,	*/
-/*	page wraps can be performed unconditionally.			*/
-/*									*/
-/************************************************************************/
-
-static void docLayoutPlaceSectTop(	BufferItem *		sectBi,
-					BlockFrame *		bf,
-					LayoutJob *		lj )
-    {
-    if  ( sectBi->biSectBreakKind != DOCsbkNONE		||
-	  ( sectBi->biParent			&&
-	    sectBi->biNumberInParent == 0	)	)
-	{
-	if  ( sectBi->biNumberInParent == 0 )
-	    {
-	    lj->ljPosition.lpPage= 0;
-	    lj->ljPosition.lpColumn= 0;
-
-	    docLayoutSectColumnTop( sectBi, lj->ljBd, &(lj->ljPosition), bf );
-	    }
-	else{
-	    if  ( BF_HAS_FOOTNOTES( bf )				&&
-		  docLayoutFootnotesForColumn( bf, &(lj->ljPosition), lj ) )
-		{ LDEB(1); return;	}
-
-	    docLayoutToNextColumn( sectBi, lj->ljBd, &(lj->ljPosition), bf );
-	    }
-
-	/*  1  */
-	if  ( ( sectBi->biSectBreakKind == DOCsbkEVEN	&&
-		! ( lj->ljPosition.lpPage % 2 )		)	||
-	      ( sectBi->biSectBreakKind == DOCsbkODD	&&
-		( lj->ljPosition.lpPage % 2 )		)	)
-	    {
-	    docLayoutToNextColumn( sectBi, lj->ljBd, &(lj->ljPosition), bf );
-	    }
-	}
-
-    sectBi->biTopPosition= lj->ljPosition;
-
-    if  ( sectBi->biNumberInParent == 0		&&
-	  sectBi->biParent			)
-	{ sectBi->biParent->biTopPosition= lj->ljPosition; }
-
-    return;
-    }
-
 /************************************************************************/
 /*									*/
 /*  Adjust the bottom of an item to changes inside.			*/
@@ -736,6 +188,14 @@ static void docPsLayoutAdjustBottom(	int *			pChanged,
 /*									*/
 /*  Place successive items, after the predecessor.			*/
 /*									*/
+/*  NOTE: In a future incremental version of the formatter, the loop	*/
+/*	over the children can be stopped if the bottom position of a	*/
+/*	child is not changed, and there is no reason to continue	*/
+/*	anyway.								*/
+/*									*/
+/*	One of the reasons to continue anyway is that a previous table	*/
+/*	header has changed height.					*/
+/*									*/
 /************************************************************************/
 
 static int docLayoutPlaceChildren(
@@ -787,6 +247,8 @@ static int docLayoutPlaceChildren(
 		{
 		ParagraphLayoutJob	plj;
 
+		docInitParagraphLayoutJob( &plj );
+
 		docPsBeginParagraphLayoutProgress( &plj,
 						0, 0, 0, child->biChildCount,
 						&(lj->ljPosition) );
@@ -924,22 +386,33 @@ static void docLayoutAdjustDocumentRectangle( LayoutJob *		ljRef )
 /*  Fixup routine that is called when the first child item does not	*/
 /*  have the same top position as its parent.				*/
 /*									*/
+/*  This can happen when during a reformat of part of the document	*/
+/*  the child moves to a different page. E.G: When a property change	*/
+/*  of a paragraph makes it higher or lower.				*/
+/*									*/
 /************************************************************************/
 
 static int docPsFixupParentGeometry(	BufferItem *		bi,
 					BufferItem *		biParent )
     {
-    LayoutPosition *	lpBi= &(bi->biTopPosition);
-    LayoutPosition *	lpPa= &(biParent->biTopPosition);
+    LayoutPosition	lpBi= bi->biTopPosition;
+    LayoutPosition	lpPa= biParent->biTopPosition;
 
-    if  ( lpPa->lpPage != lpBi->lpPage			||
-	  lpPa->lpPageYTwips != lpBi->lpPageYTwips	)
+    lpBi.lpPageYTwips -= biParent->biTopInsetTwips;
+
+    if  ( lpPa.lpPage != lpBi.lpPage			||
+	  lpPa.lpPageYTwips != lpBi.lpPageYTwips	)
 	{
-	SSDEB(docLevelStr(biParent->biLevel),docLevelStr(bi->biLevel));
-	LLDEB(lpPa->lpPage,lpBi->lpPage);
-	LLDEB(lpPa->lpPageYTwips,lpBi->lpPageYTwips);
+#	if 0
+	if  ( bi->biLevel ==DOClevPARA)
+	    { SDEB((char *)bi->biParaString);	}
 
-	biParent->biTopPosition= *lpBi;
+	SSDEB(docLevelStr(biParent->biLevel),docLevelStr(bi->biLevel));
+	LLDEB(lpPa.lpPage,lpBi.lpPage);
+	LLDEB(lpPa.lpPageYTwips,lpBi.lpPageYTwips);
+#	endif
+
+	biParent->biTopPosition= lpBi;
 
 	return -1;
 	}
@@ -951,6 +424,14 @@ static int docPsFixupParentGeometry(	BufferItem *		bi,
 /*									*/
 /*  Adjust the geometry of a parent item to changes in a child.		*/
 /*									*/
+/*  This actually is a full layout action of everything below the	*/
+/*  actually reformatted part. There are two differences:		*/
+/*  a)  Reformatting stops when a buffer item lands in the same		*/
+/*	position where it was before the change that forces us to redo	*/
+/*	part of the layout.						*/
+/*  b)  We try not to recalculate the layout of the text inside the	*/
+/*	individual text lines.						*/
+/*									*/
 /************************************************************************/
 
 static int docAdjustParentGeometry(	BufferItem *		bi,
@@ -972,8 +453,7 @@ static int docAdjustParentGeometry(	BufferItem *		bi,
     plc.plcBd= lj->ljBd;
 
     plc.plcScreenLayout= lj->ljPlaceScreen;
-    plc.plcStartParagraph= docPsPlaceParaInit;
-    plc.plcLayoutLines= docPsPlaceLines;
+    plc.plcStartParagraph= docPlaceParaInit;
     plc.plcAdjustBottom= docPsPlaceAdjustBottom;
 
     parentBi= bi->biParent;
@@ -983,7 +463,7 @@ static int docAdjustParentGeometry(	BufferItem *		bi,
 	ParagraphLayoutJob	plj;
 	int			advanced= 0;
 
-	docPsInitParagraphLayoutJob( &plj );
+	docInitParagraphLayoutJob( &plj );
 
 	docPsBeginParagraphLayoutProgress( &plj,
 				    bi->biNumberInParent+ 1, 0, 0,
@@ -1038,10 +518,12 @@ static int docAdjustParentGeometry(	BufferItem *		bi,
 
 	switch( parentBi->biLevel )
 	    {
+	    int		keepRowOnPage;
+
 	    case DOClevDOC:
 		if  ( bi->biNumberInParent == 0			&&
 		      docPsFixupParentGeometry( bi, parentBi )	)
-		    { LDEB(from); from= 0; }
+		    { from= 0; lj->ljPosition= bi->biTopPosition; }
 
 		if  ( from <= parentBi->biChildCount- 1 )
 		    {
@@ -1061,7 +543,7 @@ static int docAdjustParentGeometry(	BufferItem *		bi,
 	    case DOClevSECT:
 		if  ( bi->biNumberInParent == 0			&&
 		      docPsFixupParentGeometry( bi, parentBi )	)
-		    { LDEB(from); from= 0; }
+		    { from= 0; lj->ljPosition= bi->biTopPosition; }
 
 		if  ( from <= parentBi->biChildCount- 1 )
 		    {
@@ -1079,11 +561,10 @@ static int docAdjustParentGeometry(	BufferItem *		bi,
 
 		break;
 
-
 	    parentRowAsGroup:
 		if  ( bi->biNumberInParent == 0			&&
 		      docPsFixupParentGeometry( bi, parentBi )	)
-		    { LDEB(from); from= 0; }
+		    { from= 0; lj->ljPosition= bi->biTopPosition; }
 
 		if  ( from <= parentBi->biChildCount- 1 )
 		    {
@@ -1096,13 +577,13 @@ static int docAdjustParentGeometry(	BufferItem *		bi,
 	    case DOClevCELL:
 		if  ( bi->biNumberInParent == 0			&&
 		      docPsFixupParentGeometry( bi, parentBi )	)
-		    { LDEB(from); from= 0; }
+		    { from= 0; lj->ljPosition= bi->biTopPosition; }
 
 		if  ( from <= parentBi->biChildCount- 1 )
 		    {
 		    ParagraphLayoutJob	plj;
 
-		    docPsInitParagraphLayoutJob( &plj );
+		    docInitParagraphLayoutJob( &plj );
 
 		    docPsBeginParagraphLayoutProgress( &plj,
 					    from, 0, 0, parentBi->biChildCount,
@@ -1121,9 +602,12 @@ static int docAdjustParentGeometry(	BufferItem *		bi,
 		if  ( ! parentBi->biRowHasTableParagraphs )
 		    { goto parentRowAsGroup;	}
 
+		keepRowOnPage= parentBi->biRowKeepOnOnePage ||
+					    parentBi->biRowIsTableHeader;
+
 		if  ( parentBi->biNumberInParent > 0	&&
 		      parentBi->biParent		&&
-		      parentBi->biRowKeepOnOnePage	)
+		      keepRowOnPage			)
 		    {
 		    BufferItem *	pp= parentBi->biParent;
 		    BufferItem *	ch;
@@ -1228,70 +712,6 @@ static int docAdjustParentGeometry(	BufferItem *		bi,
     return 0;
     }
 
-/************************************************************************/
-/*									*/
-/*  Do the layout of a section.						*/
-/*									*/
-/************************************************************************/
-
-static int docLayoutSectItem(		BufferItem *		sectBi,
-					BlockFrame *		bf,
-					LayoutJob *		lj )
-    {
-    BufferDocument *		bd= lj->ljBd;
-    const DocumentProperties *	dp= &(bd->bdProperties);
-    const NotesProperties *	npEndnotes= &(dp->dpEndnoteProperties);
-
-    DocumentNote *		dn;
-
-    const SectionProperties *	sp= &(sectBi->biSectProperties);
-    const DocumentGeometry *	dgSect= &(sp->spDocumentGeometry);
-
-    int				i;
-
-    /**/
-
-    if  ( docSectHeaderFooterPrelayout( sectBi, lj ) )
-	{ LDEB(1); return -1;	}
-
-    /**/
-
-    dn= bd->bdNotes;
-    for ( i= 0; i < bd->bdNoteCount; dn++, i++ )
-	{
-	ExternalItem *	ei= (&dn->dnExternalItem);
-
-	if  ( dn->dnSectNr != sectBi->biNumberInParent )
-	    { continue;	}
-	if  ( ! ei->eiItem )
-	    { continue;	}
-
-	if  ( docExternalItemPrelayout( ei, dgSect, lj ) )
-	    { LDEB(1); return -1;	}
-	}
-
-    /**/
-
-    docLayoutPlaceSectTop( sectBi, bf, lj );
-
-    for ( i= 0; i < sectBi->biChildCount; i++ )
-	{
-	if  ( docLayoutItemImplementation( sectBi->biChildren[i], bf, lj ) )
-	    { LDEB(i); return -1;	}
-	}
-
-    /**/
-
-    if  ( sectBi->biInExternalItem == DOCinBODY		&&
-	  npEndnotes->npPosition == DPftnPOS_SECT_END	)
-	{
-	if  ( docLayoutEndnotesForSection( sectBi->biNumberInParent, bf, lj ) )
-	    { LDEB(sectBi->biNumberInParent); return -1;	}
-	}
-
-    return 0;
-    }
-
 /************************************************************************/
 /*									*/
 /*  Do the layout of the document root item.				*/
@@ -1351,7 +771,6 @@ int docLayoutItemImplementation(	BufferItem *		bi,
 					BlockFrame *		bf,
 					LayoutJob *		lj )
     {
-    BufferDocument *		bd= lj->ljBd;
     DocumentRectangle *		drChanged= lj->ljChangedRectanglePixels;
     int				i;
     int				y1;
@@ -1364,7 +783,6 @@ int docLayoutItemImplementation(	BufferItem *		bi,
 
     plc.plcScreenLayout= lj->ljLayoutScreen;
     plc.plcStartParagraph= docLayoutParaInit;
-    plc.plcLayoutLines= docLayoutLines;
     plc.plcAdjustBottom= docPsLayoutAdjustBottom;
 
     y1= BI_BELOW_PIXELS( lj->ljAdd, bi )- 1;
@@ -1408,7 +826,7 @@ int docLayoutItemImplementation(	BufferItem *		bi,
 	    {
 	    ParagraphLayoutJob	plj;
 
-	    docPsInitParagraphLayoutJob( &plj );
+	    docInitParagraphLayoutJob( &plj );
 
 	    docPsBeginParagraphLayoutProgress( &plj,
 					    0, 0, 0, bi->biChildCount,
@@ -1418,8 +836,6 @@ int docLayoutItemImplementation(	BufferItem *		bi,
 		{ LDEB(1); return -1;	}
 
 	    lj->ljPosition= plj.pljPos.plpPos;
-
-	    docFieldRefreshFlags( bi, bd );
 	    }
 	    break;
 
@@ -1435,15 +851,11 @@ int docLayoutItemImplementation(	BufferItem *		bi,
 	    if  ( ! bi->biRowHasTableParagraphs )
 		{ goto rowAsGroup;	}
 
-	    docFieldRefreshFlags( bi, bd );
-
 	    if  ( docLayoutRowItem( bi, bf, lj, &plc ) )
 		{ LDEB(1); return -1;	}
 	    break;
 
 	case DOClevPARA:
-	    docFieldRefreshFlags( bi, bd );
-
 	    if  ( docLayoutParaItem( bi, bf, lj, &plc ) )
 		{ LDEB(1); return -1;	}
 
@@ -1521,7 +933,7 @@ static int docLayoutFinishParaAdjust(
 	{
 	int	accepted;
 
-	accepted= (*plc->plcLayoutLines)( plp, bf, plc, paraBi );
+	accepted= docLayoutLines( plp, bf, plc, paraBi );
 
 	if  ( accepted < 0 )
 	    { LDEB(accepted); return -1;	}
@@ -1608,7 +1020,7 @@ static int docPsLayoutOneLine(	TextLine *			tl,
     docInitNotesReservation( &nrLine );
 
     /*  2,7  */
-    accepted= docPsLayout_Line( &boxLine, &nrLine, fromLinePos,
+    accepted= docLayout_Line( &boxLine, &nrLine, bf, fromLinePos,
 				paraBi, psp->pspPart, pd, plc,
 				&(plp->plpFormattingFrame),
 				&(plp->plpPos), &(plp->plpPos),
@@ -1633,8 +1045,8 @@ static int docPsLayoutOneLine(	TextLine *			tl,
 
 	docInitNotesReservation( &nrLine );
 
-	accepted= docPsLayout_Line(
-			    &boxLine, &nrLine, fromLinePos,
+	accepted= docLayout_Line(
+			    &boxLine, &nrLine, bf, fromLinePos,
 			    paraBi, psp->pspPart, pd, plc,
 			    &(plp->plpFormattingFrame),
 			    &(plp->plpPos), &(plp->plpPos),
@@ -1670,14 +1082,13 @@ static int docPsLayoutOneLine(	TextLine *			tl,
 /*									*/
 /************************************************************************/
 
-int docPsAdjustParaLayout(	BufferItem *		paraBi,
+int docAdjustParaLayout(	BufferItem *		paraBi,
 				int			fromLine,
 				int			stroffShift,
 				int			stroffUpto,
 				LayoutJob *		lj )
     {
     BufferDocument *		bd= lj->ljBd;
-    const DocumentProperties *	dp= &(bd->bdProperties);
     DocumentRectangle *		drChanged= lj->ljChangedRectanglePixels;
     AppDrawingData *		add= lj->ljAdd;
 
@@ -1723,19 +1134,17 @@ int docPsAdjustParaLayout(	BufferItem *		paraBi,
 
     plcL.plcScreenLayout= lj->ljLayoutScreen;
     plcL.plcStartParagraph= docLayoutParaInit;
-    plcL.plcLayoutLines= docLayoutLines;
     plcL.plcAdjustBottom= docPsLayoutAdjustBottom;
 
     plcP.plcScreenLayout= lj->ljPlaceScreen;
-    plcP.plcStartParagraph= docPsPlaceParaInit;
-    plcP.plcLayoutLines= docPsPlaceLines;
+    plcP.plcStartParagraph= docPlaceParaInit;
     plcP.plcAdjustBottom= docPsPlaceAdjustBottom;
 
     if  ( docPsClaimParticuleData( paraBi, &pd ) )
 	{ LDEB(paraBi->biParaParticuleCount); return -1;	}
 
     /*  1  */
-    if  ( docLayoutOpenParaFonts( paraBi, add, dp ) )
+    if  ( docLayoutOpenParaFonts( bd, paraBi, add ) )
 	{ LDEB(1); return -1;	}
 
     /*  0  */
