@@ -15,6 +15,8 @@
 #   undef	y0
 #   undef	y1
 
+#   include	<charnames.h>
+
 #   include	"docLayout.h"
 
 #   include	<appDebugon.h>
@@ -26,7 +28,7 @@
 /*									*/
 /************************************************************************/
 
-#   define LINEDISTFAC	5
+#   define LINEDISTFAC	6
 
 /************************************************************************/
 /*									*/
@@ -68,6 +70,8 @@ void docLayoutInitBlockFrame(	BlockFrame *	bf )
     bf->bfY0Twips= 0;
     bf->bfY1Twips= 0;
 
+    bf->bfColumnWidthTwips= 0;
+
     docInitNotesReservation( &(bf->bfNotesReservation) );
     }
 
@@ -120,148 +124,131 @@ void docLayoutToNextColumn(	BufferItem *		bi,
 
 /************************************************************************/
 /*									*/
-/*  Calculate the height of a series of lines in a paragraph.		*/
+/*  Add the space occupied by a border to an inset.			*/
 /*									*/
 /************************************************************************/
 
-static void docPsAboveParagraph(	BufferItem *			bi )
+void docLayoutAddBorderToInset(	int *				pInset,
+				const BorderProperties *	bp )
     {
-    int				above= 0;
-    const BorderProperties *	bp= (const BorderProperties *)0;
+    if  ( bp->bpStyle == DOCbsNONE )
+	{ return;	}
 
-    above= bi->biParaSpaceBeforeTwips;
-
-    if  ( bi->biParaInTable			&&
-	  bi->biNumberInParent == 0		)
-	{
-	const BufferItem *	rowBi= bi->biParent->biParent;
-	const CellProperties *	cp;
-	int			col= bi->biParent->biNumberInParent;
-
-	if  ( col >= rowBi->biRowCellCount )
-	    { LLDEB(col,rowBi->biRowCellCount);	}
-	else{
-	    cp= &(rowBi->biRowCells[col]);
-
-	    if  ( cp->cpTopBorder.bpStyle != DOCbsNONE )
-		{
-		above += cp->cpTopBorder.bpSpacingTwips;
-		/*  Word seems not to allocate this
-		above += cp->cpTopBorder.bpWidthTwips;
-		*/
-		}
-	    }
-
-	/*  Nicer but Word does not do this.
-	above += ( bi->biParaLeadingTwips+ 1 )/ 2;
-	*/
-	}
-
-    if  ( bi->biParaTopBorder.bpStyle != DOCbsNONE )
-	{
-	/*  Word seems not to allocate this
-	above += bi->biParaTopBorder.bpWidthTwips;
-	*/
-	above += bi->biParaTopBorder.bpSpacingTwips;
-	bp= &(bi->biParaTopBorder);
-	}
-    else{
-	if  ( bi->biParaBoxBorder.bpStyle != DOCbsNONE )
-	    {
-	    BufferItem *	prevBi= (BufferItem *)0;
-
-	    if  ( bi->biNumberInParent > 0 )
-		{
-		prevBi= bi->biParent->biChildren[
-					    bi->biNumberInParent- 1];
-		}
-	    if  ( ! prevBi					||
-		  prevBi->biParaBoxBorder.bpStyle == DOCbsNONE	)
-		{
-		/*  Word seems not to allocate this
-		above += bi->biParaBoxBorder.bpWidthTwips;
-		*/
-		above += bi->biParaBoxBorder.bpSpacingTwips;
-		bp= &(bi->biParaBoxBorder);
-		}
-	    }
-	}
-
-    bi->biParaSpaceAboveLinesTwips= above;
-    bi->biParaBorderAboveParagraph= bp;
+    (*pInset) += bp->bpSpacingTwips;
+    (*pInset) += bp->bpPenWideTwips;
 
     return;
     }
 
-static void docPsBelowParagraph(	BufferItem *			bi )
+void docLayoutStretchInsetForBorder(
+				int *				pInset,
+				const BorderProperties *	bp )
     {
-    int				below= 0;
-    const BorderProperties *	bp= (const BorderProperties *)0;
+    int		inset= 0;
 
-    if  ( bi->biParaBottomBorder.bpStyle != DOCbsNONE )
+    inset += bp->bpSpacingTwips;
+    inset += bp->bpPenWideTwips;
+
+    if  ( (*pInset) < inset )
+	{ (*pInset)=  inset;	}
+
+    return;
+    }
+
+/************************************************************************/
+/*									*/
+/*  Calculate the height of a series of lines in a paragraph.		*/
+/*									*/
+/************************************************************************/
+
+static void docLayoutCalculateParaTopInset(	BufferItem *	paraBi )
+    {
+    int				topInset= 0;
+    const BorderProperties *	bp= (const BorderProperties *)0;
+    const BufferItem *		prevBi= (const BufferItem *)0;
+
+    if  ( paraBi->biNumberInParent > 0 )
+	{ prevBi= paraBi->biParent->biChildren[paraBi->biNumberInParent- 1]; }
+
+    topInset= paraBi->biParaSpaceBeforeTwips;
+
+    if  ( paraBi->biParaTopBorder.bpStyle != DOCbsNONE )
 	{
-	below += bi->biParaLeadingTwips;
-	below += bi->biParaBottomBorder.bpSpacingTwips;
-	/*  Word seems not to allocate this
-	below += bi->biParaBottomBorder.bpWidthTwips;
-	*/
-	bp= &(bi->biParaBottomBorder);
+	if  ( ! prevBi						||
+	      prevBi->biParaTopBorder.bpStyle == DOCbsNONE	)
+	    {
+	    docLayoutAddBorderToInset( &topInset, &(paraBi->biParaTopBorder) );
+	    bp= &(paraBi->biParaTopBorder);
+	    }
 	}
     else{
-	if  ( bi->biParaBoxBorder.bpStyle != DOCbsNONE )
+	if  ( paraBi->biParaBoxBorder.bpStyle != DOCbsNONE )
 	    {
-	    BufferItem *	nextBi= (BufferItem *)0;
-
-	    if  ( bi->biNumberInParent <
-				bi->biParent->biChildCount- 1 )
+	    if  ( ! prevBi					||
+		  prevBi->biParaBoxBorder.bpStyle == DOCbsNONE	)
 		{
-		nextBi= bi->biParent->biChildren[
-					    bi->biNumberInParent+ 1];
+		docLayoutAddBorderToInset( &topInset,
+						&(paraBi->biParaBoxBorder) );
+		bp= &(paraBi->biParaBoxBorder);
 		}
+	    }
+	}
 
+    paraBi->biTopInsetTwips= topInset;
+    paraBi->biParaBorderAboveParagraph= bp;
+
+    return;
+    }
+
+static void docParaSpaceForBottomBorder(int *				pBelow,
+					const BorderProperties *	bp,
+					const BufferItem *		paraBi )
+    {
+    int		below= *pBelow;
+
+    below += paraBi->biParaLeadingTwips;
+
+    docLayoutAddBorderToInset( &below, bp );
+
+    *pBelow= below;
+    }
+
+static void docLayoutCalculateParaBottomInset(	BufferItem *	paraBi )
+    {
+    int				bottomInset= 0;
+    const BorderProperties *	bp= (const BorderProperties *)0;
+    const BufferItem *		nextBi= (const BufferItem *)0;
+
+    if  ( paraBi->biNumberInParent < paraBi->biParent->biChildCount- 1 )
+	{ nextBi= paraBi->biParent->biChildren[paraBi->biNumberInParent+ 1]; }
+
+    if  ( paraBi->biParaBottomBorder.bpStyle != DOCbsNONE )
+	{
+	if  ( ! nextBi						||
+	      nextBi->biParaBottomBorder.bpStyle == DOCbsNONE	)
+	    {
+	    docParaSpaceForBottomBorder( &bottomInset,
+				    &(paraBi->biParaBottomBorder), paraBi );
+	    bp= &(paraBi->biParaBottomBorder);
+	    }
+	}
+    else{
+	if  ( paraBi->biParaBoxBorder.bpStyle != DOCbsNONE )
+	    {
 	    if  ( ! nextBi					||
 		  nextBi->biParaBoxBorder.bpStyle == DOCbsNONE	)
 		{
-		below += bi->biParaLeadingTwips;
-		below += bi->biParaBoxBorder.bpSpacingTwips;
-		/*  Word seems not to allocate this
-		below += bi->biParaBoxBorder.bpWidthTwips;
-		*/
-		bp= &(bi->biParaBoxBorder);
+		docParaSpaceForBottomBorder( &bottomInset,
+					&(paraBi->biParaBoxBorder), paraBi );
+		bp= &(paraBi->biParaBoxBorder);
 		}
 	    }
 	}
 
-    if  ( bi->biParaInTable					&&
-	  bi->biNumberInParent == bi->biParent->biChildCount- 1	)
-	{
-	const BufferItem *	rowBi= bi->biParent->biParent;
-	const CellProperties *	cp;
-	int			col= bi->biParent->biNumberInParent;
+    bottomInset += paraBi->biParaSpaceAfterTwips;
 
-	/*  Nicer but Word does not do this.
-	below += bi->biParaLeadingTwips/ 2;
-	*/
-
-	if  ( col >= rowBi->biRowCellCount )
-	    { LLDEB(col,rowBi->biRowCellCount);	}
-	else{
-	    cp= &(rowBi->biRowCells[col]);
-
-	    if  ( cp->cpBottomBorder.bpStyle != DOCbsNONE )
-		{
-		below += cp->cpBottomBorder.bpSpacingTwips;
-		/*  Word seems not to allocate this
-		below += cp->cpBottomBorder.bpWidthTwips;
-		*/
-		}
-	    }
-	}
-
-    below += bi->biParaSpaceAfterTwips;
-
-    bi->biParaSpaceBelowLinesTwips= below;
-    bi->biParaBorderBelowParagraph= bp;
+    paraBi->biBottomInsetTwips= bottomInset;
+    paraBi->biParaBorderBelowParagraph= bp;
 
     return;
     }
@@ -340,8 +327,8 @@ int docPsParagraphLineExtents(	const AppPhysicalFontList *	apfl,
 
     bi->biParaLeadingTwips= size/ LINEDISTFAC;
 
-    docPsAboveParagraph( bi );
-    docPsBelowParagraph( bi );
+    docLayoutCalculateParaTopInset( bi );
+    docLayoutCalculateParaBottomInset( bi );
 
     return 0;
     }
@@ -438,6 +425,7 @@ void docBlockFrameTwips(	BlockFrame *		bf,
 
     bf->bfX0Twips= dg->dgLeftMarginTwips;
     bf->bfX1Twips= dg->dgPageWideTwips- dg->dgRightMarginTwips;
+    bf->bfColumnWidthTwips= bf->bfX1Twips- bf->bfX0Twips;
 
     docInitNotesReservation( &(bf->bfNotesReservation) );
 
@@ -553,7 +541,7 @@ void docParagraphFrameTwips(	ParagraphFrame *	pf,
 	    if  ( col0 == 0 )
 		{ pf->pfX0GeometryTwips += rowBi->biRowLeftIndentTwips;	}
 	    else{
-		CellProperties *	cpp;
+		const CellProperties *	cpp;
 
 		cpp= rp->rpCells+ col0- 1;
 		pf->pfX0GeometryTwips += cpp->cpRightBoundaryTwips;
@@ -581,3 +569,193 @@ void docParagraphFrameTwips(	ParagraphFrame *	pf,
     return;
     }
 
+/************************************************************************/
+/*									*/
+/*  Scale an inserted object to fit where we want to place it.		*/
+/*									*/
+/*  1)  If the object fits on the page at its current scale, nothing	*/
+/*	needs to be done.						*/
+/*  2)  Otherwise scale the object to fit on the page. X and Y scale	*/
+/*	are the same, irrespective of the original scaling.		*/
+/*									*/
+/************************************************************************/
+
+void docLayoutScaleObjectToFitParagraphFrame(	InsertedObject *	io,
+						const ParagraphFrame *	pf )
+    {
+    int		textWideTwips= pf->pfX1TextLinesTwips- pf->pfX0TextLinesTwips;
+
+    int		objectWideTwips= ( io->ioScaleX* io->ioTwipsWide )/ 100.0;
+    int		objectHighTwips= ( io->ioScaleY* io->ioTwipsHigh )/ 100.0;
+
+    double	scaleX;
+    double	scaleY;
+
+    /*  1  */
+    if  ( objectWideTwips <= textWideTwips	&&
+	  objectHighTwips <= pf->pfPageHigh	)
+	{ return;	}
+
+    /*  2  */
+    scaleX= (double)textWideTwips/ (double)io->ioTwipsWide;
+    scaleY= (double)pf->pfPageHigh/ (double)io->ioTwipsHigh;
+
+    if  ( scaleY > scaleX )
+	{ scaleY=  scaleX;	}
+
+    if  ( scaleY < 0.99 )
+	{
+	io->ioScaleX= (int)( 99* scaleY );
+	io->ioScaleY= (int)( 99* scaleY );
+	}
+
+    return;
+    }
+
+/************************************************************************/
+/*									*/
+/*  Push a position to one at least below another one. An example of	*/
+/*  its use is pushing down the bottom of a row to the bottom of its	*/
+/*  tallest cell.							*/
+/*									*/
+/************************************************************************/
+
+void docLayoutPushBottomDown(		LayoutPosition *	lpParentBottom,
+					const LayoutPosition *	lpChildBottom )
+    {
+    if  ( lpParentBottom->lpPage < lpChildBottom->lpPage )
+	{
+	lpParentBottom->lpPage= lpChildBottom->lpPage;
+	lpParentBottom->lpPageYTwips= lpChildBottom->lpPageYTwips;
+	lpParentBottom->lpAtTopOfColumn= lpChildBottom->lpAtTopOfColumn;
+	}
+    else{
+	if  ( lpParentBottom->lpPage == lpChildBottom->lpPage )
+	    {
+	    if  ( lpParentBottom->lpPageYTwips < lpChildBottom->lpPageYTwips )
+		{
+		lpParentBottom->lpPageYTwips= lpChildBottom->lpPageYTwips;
+		lpParentBottom->lpAtTopOfColumn= lpChildBottom->lpAtTopOfColumn;
+		}
+	    }
+	}
+
+    return;
+    }
+
+void docLayoutPushBottomDownShifted(	LayoutPosition *	lpParentBottom,
+					const LayoutPosition *	lpChildBottom,
+					int			pShift,
+					int			yShift )
+    {
+    LayoutPosition	lpShifted;
+
+    lpShifted= *lpChildBottom;
+    lpShifted.lpPage += pShift;
+    lpShifted.lpPageYTwips += yShift;
+
+    if  ( yShift != 0 )
+	{ lpShifted.lpAtTopOfColumn= 0;	}
+
+    docLayoutPushBottomDown( lpParentBottom, &lpShifted );
+    }
+
+/************************************************************************/
+/*									*/
+/*  Translate a stretch of text to upper case for capitals or smallcaps	*/
+/*  drawing.								*/
+/*									*/
+/*  3)  Worst case alternating, starting with lower case. 1+ len	*/
+/*	segments. But when it is just one lower case letter we need 3!	*/
+/*									*/
+/************************************************************************/
+
+int docMakeCapsString(		unsigned char **	pUpperString,
+				int **			pSegments,
+				int *			pSegmentCount,
+				const BufferDocument *	bd,
+				const TextAttribute *	ta,
+				const unsigned char *	printString,
+				int			len )
+    {
+    int				rval= 0;
+
+    const DocumentProperties *	dp= &(bd->bdProperties);
+    const DocumentFontList *	dfl= &(dp->dpFontList);
+    const DocumentFont *	df= dfl->dflFonts+ ta->taFontNumber;
+
+    int				segmentCount= 0;
+    unsigned char *		upperString= (unsigned char *)0;
+    int *			segments= (int *)0;
+
+    if  ( df->dfEncodingUsed >= 0		&&
+	  df->dfEncodingUsed < ENCODINGps_COUNT	)
+	{
+	const unsigned char *	charKinds;
+	const unsigned char *	charShifts;
+	int			n;
+	const unsigned char *	from;
+	unsigned char *		to;
+
+	charKinds= PS_Encodings[df->dfEncodingUsed].fcCharKinds;
+	charShifts= PS_Encodings[df->dfEncodingUsed].fcCharShifts;
+
+	upperString= malloc( len+ 1 );
+	if  ( ! upperString )
+	    { XDEB(upperString); rval= -1; goto ready;	}
+
+	if  ( ta->taSmallCaps && ! ta->taCapitals )
+	    {
+	    /*  3  */
+	    segments= malloc( (len+ 2)* sizeof(int) );
+	    if  ( ! segments )
+		{ XDEB(segments); rval= -1; goto ready;	}
+	    }
+
+	to= upperString;
+	from= printString;
+	n= 0;
+	while( n < len )
+	    {
+	    int		n0= 0;
+	    int		n1= 0;
+
+	    while( n < len && ! ( charKinds[*from] & CHARisLOWER ) )
+		{
+		*to= *from;
+		n0++; from++; to++; n++;
+		}
+
+	    while( n < len &&   ( charKinds[*from] & CHARisLOWER ) )
+		{
+		*to= charShifts[*from];
+		n1++; from++; to++; n++;
+		}
+
+	    if  ( segments )
+		{
+		segments[2*segmentCount+ 0]= n0;
+		segments[2*segmentCount+ 1]= n1;
+		segmentCount++;
+		}
+	    }
+
+	*to= '\0';
+	}
+
+    if  ( segmentCount > len+ 2 )
+	{ LLDEB(len,segmentCount);	}
+
+    *pUpperString= upperString; upperString= (unsigned char *)0;
+    *pSegments= segments; segments= (int *)0;
+    *pSegmentCount= segmentCount;
+
+  ready:
+
+    if  ( upperString )
+	{ free( upperString );	}
+    if  ( segments )
+	{ free( segments );	}
+
+    return rval;
+    }
