@@ -4,7 +4,7 @@
 /*									*/
 /************************************************************************/
 
-#   include	"config.h"
+#   include	"tedConfig.h"
 
 #   include	<stdlib.h>
 #   include	<string.h>
@@ -30,7 +30,7 @@ typedef struct PlainReadingContext
 
 static void docPlainInitReadingContext( PlainReadingContext *	prc )
     {
-    docInitTextAttribute( &(prc->prcTextAttribute) );
+    utilInitTextAttribute( &(prc->prcTextAttribute) );
 
     prc->prcParaItem= (BufferItem *)0;
     prc->prcLongestParagraph= 0;
@@ -50,7 +50,7 @@ static int docPlainReadParaCharacter(	SimpleInputStream *	sis,
 
     if  ( c == '\t' || c == '\n' )
 	{
-	*pC= c; pTa->taFontIsBold= 0; pTa->taIsUnderlined= 0;
+	*pC= c; pTa->taFontIsBold= 0; pTa->taTextIsUnderlined= 0;
 	return 0;
 	}
 
@@ -60,7 +60,7 @@ static int docPlainReadParaCharacter(	SimpleInputStream *	sis,
 	if  ( cc != EOF )
 	    { sioInUngetLastRead( sis );	}
 
-	*pC= c; pTa->taFontIsBold= 0; pTa->taIsUnderlined= 0;
+	*pC= c; pTa->taFontIsBold= 0; pTa->taTextIsUnderlined= 0;
 	return 0;
 	}
     else{
@@ -71,35 +71,36 @@ static int docPlainReadParaCharacter(	SimpleInputStream *	sis,
 	    if  ( cc != EOF )
 		{ sioInUngetLastRead( sis );	}
 
-	    *pC= c; pTa->taFontIsBold= 0; pTa->taIsUnderlined= 0;
+	    *pC= c; pTa->taFontIsBold= 0; pTa->taTextIsUnderlined= 0;
 	    return 0;
 	    }
 
 	if  ( c == '_' )
 	    {
-	    *pC= cc; pTa->taFontIsBold= 0; pTa->taIsUnderlined= 1;
+	    *pC= cc; pTa->taFontIsBold= 0; pTa->taTextIsUnderlined= 1;
 	    return 0;
 	    }
 
 	if  ( c == cc )
 	    {
-	    *pC= cc; pTa->taFontIsBold= 1; pTa->taIsUnderlined= 0;
+	    *pC= cc; pTa->taFontIsBold= 1; pTa->taTextIsUnderlined= 0;
 	    return 0;
 	    }
 
-	*pC= cc; pTa->taFontIsBold= 0; pTa->taIsUnderlined= 0;
+	*pC= cc; pTa->taFontIsBold= 0; pTa->taTextIsUnderlined= 0;
 	return 0;
 	}
     }
 
-static int docPlainReadParagraph(	const BufferDocument *	bd,
-					BufferItem *		cellBi,
+static int docPlainReadParagraph(	BufferDocument *	bd,
 					SimpleInputStream *	sis,
 					PlainReadingContext *	prc )
     {
     int			c;
     BufferItem *	bi;
+
     TextAttribute	ta= prc->prcTextAttribute;
+    int			textAttrNr;
 
     int			res;
 
@@ -110,9 +111,23 @@ static int docPlainReadParagraph(	const BufferDocument *	bd,
     if  ( res > 0 )
 	{ return res;	}
 
-    bi= docInsertItem( bd, cellBi, -1, DOClevPARA );
-    if  ( ! bi )
-	{ XDEB(bi); return -1;	}
+    if  ( ! prc->prcParaItem )
+	{
+	DocumentPosition	dp;
+
+	if  ( docFirstPosition( &dp, &(bd->bdItem) ) )
+	    { LDEB(1);	}
+
+	bi= dp.dpBi;
+	prc->prcParaItem= bi;
+	}
+    else{
+	bi= docInsertItem( bd, prc->prcParaItem->biParent, -1, DOClevPARA );
+	if  ( ! bi )
+	    { XDEB(bi); return -1;	}
+
+	prc->prcParaItem= bi;
+	}
 
     while( c != '\n' )
 	{
@@ -136,16 +151,12 @@ static int docPlainReadParagraph(	const BufferDocument *	bd,
 
 	if  ( c == '\t' )
 	    {
-	    if  ( docInflateTextString( bi, 1 ) )
+	    const int	atBegin= 0;
+
+	    if  ( docSaveSpecialParticule( bd, bi, atBegin,
+				    &(prc->prcTextAttribute), DOCkindTAB ) )
 		{ LDEB(bi->biParaStrlen); return -1;	}
 
-	    if  ( ! docInsertTextParticule( bi, -1, bi->biParaStrlen,
-				    1, DOCkindTAB, prc->prcTextAttribute ) )
-		{ LDEB(bi->biParaParticuleCount); return -1;	}
-
-	    bi->biParaString[bi->biParaStrlen++]= ' ';
-	    bi->biParaString[bi->biParaStrlen  ]= '\0';
-
 	    res= docPlainReadParaCharacter( sis, &c, &ta );
 
 	    if  ( res < 0 )
@@ -202,8 +213,12 @@ static int docPlainReadParagraph(	const BufferDocument *	bd,
 	if  ( res < 0 )
 	    { LDEB(res); return res;	}
 
+	textAttrNr= docTextAttributeNumber( bd, &prevTa );
+	if  ( textAttrNr < 0 )
+	    { LDEB(textAttrNr); return -1;	}
+
 	if  ( ! docInsertTextParticule( bi, -1,
-			off, bi->biParaStrlen- off, DOCkindTEXT, prevTa ) )
+			off, bi->biParaStrlen- off, DOCkindTEXT, textAttrNr ) )
 	    { LDEB(bi->biParaParticuleCount); return -1;	}
 
 	if  ( res > 0 )
@@ -212,8 +227,9 @@ static int docPlainReadParagraph(	const BufferDocument *	bd,
 
     if  ( bi->biParaParticuleCount == 0 )
 	{
-	if  ( ! docInsertTextParticule( bi, 0, 0, 0,
-					DOCkindTEXT, prc->prcTextAttribute ) )
+	textAttrNr= docTextAttributeNumber( bd, &(prc->prcTextAttribute) );
+
+	if  ( ! docInsertTextParticule( bi, 0, 0, 0, DOCkindTEXT, textAttrNr ) )
 	    { LDEB( bi->biParaParticuleCount); return -1;	}
 	}
 
@@ -233,66 +249,62 @@ static int docPlainReadParagraph(	const BufferDocument *	bd,
 /*									*/
 /************************************************************************/
 
-BufferDocument * docPlainReadFile(	SimpleInputStream *		sis,
-					int *				pMxL,
-					const DocumentGeometry *	dg )
+BufferDocument * docPlainReadFile(
+			SimpleInputStream *		sis,
+			int *				pMxL,
+			int				docCharset,
+			int				ansiCodepage,
+			const DocumentGeometry *	dg )
     {
+    const int		fontSizeHalfPoints= 18;
+
     BufferDocument *	bd;
-    BufferItem *	bi;
 
     PlainReadingContext	prc;
 
-    bd= (BufferDocument *)malloc( sizeof(BufferDocument) );
+    docPlainInitReadingContext( &prc );
+
+    bd= docNewFile( &(prc.prcTextAttribute),
+				    "Courier", fontSizeHalfPoints,
+				    docCharset, ansiCodepage, dg );
     if  ( ! bd )
 	{ XDEB(bd); return bd;	}
 
-    docPlainInitReadingContext( &prc );
-    prc.prcTextAttribute.taFontNumber= 0;
-    prc.prcTextAttribute.taFontSizeHalfPoints= 18;
     bd->bdProperties.dpTabIntervalTwips= 960;
 
-    docInitDocument( bd );
-    bd->bdProperties.dpGeometry= *dg;
+#   if 0
 
-    if  ( ! docInsertFont( &(bd->bdProperties.dpFontList), 0,
-						    "fmodern", "Courier" ) )
-	{ LDEB(1); docFreeDocument( bd ); return (BufferDocument *)0;	}
+    int			encoding= 0;
+    AfmBBox		abb;
+    int			wide;
 
-    bi= docInsertItem( bd, &(bd->bdItem), -1, DOClevSECT );
-    if  ( ! bi )
-	{ XDEB(bi); docFreeDocument( bd ); return (BufferDocument *)0;	}
-    bi= docInsertItem( bd, bi, -1, DOClevROW );
-    if  ( ! bi )
-	{ XDEB(bi); docFreeDocument( bd ); return (BufferDocument *)0;	}
-    bi= docInsertItem( bd, bi, -1, DOClevCELL );
-    if  ( ! bi )
-	{ XDEB(bi); docFreeDocument( bd ); return (BufferDocument *)0;	}
+    wide= psCalculateStringExtents( &abb, "00000000", 8,
+				10* prc.prcTextAttribute.taFontSizeHalfPoints,
+				0, encoding, afi );
+    if  ( wide > 0 )
+	{ bd->bdProperties.dpTabIntervalTwips= wide;	}
+
+#   endif
 
     for (;;)
 	{
 	int	res;
 
-	res= docPlainReadParagraph( bd, bi, sis, &prc );
+	res= docPlainReadParagraph( bd, sis, &prc );
 
 	if  ( res > 0 )
 	    { break;	}
-
 	if  ( res < 0 )
-	    { LDEB(res); docFreeDocument( bd ); return (BufferDocument *)0; }
-	}
-
-    if  ( bi->biChildCount == 0 )
-	{
-	bi= docInsertItem( bd, bi, -1, DOClevPARA );
-	if  ( ! bi )
-	    { XDEB(bi); docFreeDocument( bd ); return (BufferDocument *)0; }
-
-	if  ( ! docInsertTextParticule( bi, 0, 0, 0,
-					DOCkindTEXT, prc.prcTextAttribute ) )
-	    { LDEB(1); docFreeDocument( bd ); return (BufferDocument *)0; }
+	    { LDEB(res); goto failed; }
 	}
 
     *pMxL= prc.prcLongestParagraph;
 
     return bd;
+
+  failed:
+
+    docFreeDocument( bd );
+
+    return (BufferDocument *)0;
     }
