@@ -1,9 +1,12 @@
 /************************************************************************/
 /*									*/
-/*  Read a plain text file into a BufferDocument			*/
+/*  Read/Write RTF files.						*/
 /*									*/
 /************************************************************************/
 
+#   ifndef	DOC_RTF_H
+#   define	DOC_RTF_H
+
 #   include	<stdlib.h>
 #   include	<string.h>
 #   include	<stdio.h>
@@ -34,10 +37,27 @@ typedef struct RtfBookmarkLevel
     struct RtfBookmarkLevel *	rblPrevious;
     } RtfBookmarkLevel;
 
+typedef struct RtfReadingState
+    {
+    int				rrsBytesPerUnicode;
+    int				rrsUnicodeBytesToSkip;
+
+    TextAttribute		rrsTextAttribute;
+    ParagraphProperties		rrsParagraphProperties;
+    SectionProperties		rrsSectionProperties;
+
+    MemoryBuffer		rrsSavedText;
+
+    struct RtfReadingState *	rrsPrev;
+    } RtfReadingState;
+
 typedef struct RtfReadingContext
     {
+    RtfReadingState *		rrcState;
+
     int				rrcLevel;
     int				rrcSplitLevel;
+    int				rrcInIgnoredGroup;
 				/****************************************/
 				/*  The depth in the document		*/
 				/*  == Type of BufferItem.		*/
@@ -47,6 +67,7 @@ typedef struct RtfReadingContext
 				/*  Complain about unknown control	*/
 				/****************************************/
     int				rrcCharacterAhead;
+    int				rrcAfterNoteref;
 				/****************************************/
 				/*  Character that could not be unread	*/
 				/*  using ungetc().			*/
@@ -57,7 +78,6 @@ typedef struct RtfReadingContext
 				/****************************************/
 				/*  The current position in the input.	*/
 				/****************************************/
-    TextAttribute		rrcCurrentTextAttribute;
     int				rrcInDeletedText;
     BorderProperties		rrcTextBorderProperties;
     int				rrcTextShadingPattern;
@@ -68,19 +88,18 @@ typedef struct RtfReadingContext
 				/*  Text attributes of the current pos.	*/
 				/****************************************/
     TextFrameProperties		rrcTextFrameProperties;
-    ParagraphProperties		rrcParagraphProperties;
     CellProperties		rrcCellProperties;
     RowProperties		rrcRowProperties;
-    SectionProperties		rrcSectionProperties;
     BorderProperties		rrcBorderProperties;
     ShapeProperties		rrcShapeProperties;
     DrawingShape		rrcShape;
-    DocumentStyle		rrcStyle;
+    DocumentStyle		rrcDocumentStyle;
     TabStop			rrcTabStop;
     RGB8Color			rrcColor;
     int				rrcGotComponent;
     struct tm			rrcTm;
     unsigned char *		rrcInfoText;
+    int				rrcExternalItemKind;
 
     ParagraphNumber		rrcParagraphNumber;
     int				rrcWhichPnText;
@@ -88,6 +107,10 @@ typedef struct RtfReadingContext
     int				rrcParagraphNumberCount;
 
     DocumentList		rrcDocumentList;
+    DocumentListLevel		rrcDocumentListLevel;
+
+    ListOverride		rrcListOverride;
+    ListOverrideLevel		rrcListOverrideLevel;
 
 				/****************************************/
 				/*  Document properties.		*/
@@ -108,7 +131,6 @@ typedef struct RtfReadingContext
 				/*  For reading 'fields'.		*/
 				/****************************************/
     int				rrcJustAfterPntext;
-    int				rrcPageAfterPntext;
 				/****************************************/
 				/*  For coping with the way word saves	*/
 				/*  {\pntext ... }			*/
@@ -176,7 +198,7 @@ typedef struct RtfWritingContext
 typedef struct RtfControlWord
     {
     char *		rcwWord;
-    RtfTag		rcwId;
+    RtfTag		rcwID;
     ItemLevel		rcwLevel;
     int			(*rcwApply)( SimpleInputStream *		sis,
 				    const struct RtfControlWord *	rcw,
@@ -196,6 +218,8 @@ typedef int (*RtfAddTextParticule)(	RtfReadingContext *		rrc,
 					const unsigned char *		text,
 					int				len );
 
+typedef int (*RtfCommitGroup)(		RtfReadingContext *		rrc );
+
 /************************************************************************/
 /*  Common data.							*/
 /************************************************************************/
@@ -203,10 +227,12 @@ typedef int (*RtfAddTextParticule)(	RtfReadingContext *		rrc,
 extern RtfControlWord	docRtfEmptyTable[];
 extern RtfControlWord	docRtfDocumentWords[];
 extern RtfControlWord	docRtfDocumentGroups[];
-extern RtfControlWord	docRtfPropertyWords[];
+extern RtfControlWord	docRtfListLevelGroups[];
 
 /************************************************************************/
+/*									*/
 /*  Routine declarations.						*/
+/*									*/
 /************************************************************************/
 
 extern void docRtfInitReadingContext(	RtfReadingContext *		rrc );
@@ -237,7 +263,9 @@ extern int docRtfReadGroup(	SimpleInputStream *	sis,
 
 				const RtfControlWord *	contolWords,
 				const RtfControlWord *	groupWords,
-				RtfAddTextParticule	addParticule	);
+
+				RtfAddTextParticule	addParticule,
+				RtfCommitGroup		commitGroup );
 
 extern int docRtfFindControl(	SimpleInputStream *	sis,
 				RtfReadingContext *	rrc,
@@ -251,17 +279,27 @@ extern int docRtfIgnoreWord(	SimpleInputStream *	sis,
 				int			arg,
 				RtfReadingContext *	rrc	);
 
-extern int docRtfIgnoreParticule(	RtfReadingContext *	rrc,
-					const unsigned char *	text,
-					int			len	);
-
 extern int docRtfIgnoreText(	RtfReadingContext *	rrc,
 				const unsigned char *	text,
-				int			len	);
+				int			len );
 
 extern int docRtfRefuseText(	RtfReadingContext *	rrc,
 				const unsigned char *	text,
-				int			len	);
+				int			len );
+
+extern int docRtfSaveText(	RtfReadingContext *	rrc,
+				const unsigned char *	text,
+				int			len );
+
+extern int docRtfRememberText(	char **			pTarget,
+				int *			pSize,
+				RtfReadingContext *	rrc,
+				int			removeSemicolon );
+
+extern int docRtfRememberUText(	unsigned char **	pTarget,
+				int *			pSize,
+				RtfReadingContext *	rrc,
+				int			removeSemicolon );
 
 extern int docRtfSkipGroup(	SimpleInputStream *	sis,
 				const RtfControlWord *	groupRcw,
@@ -270,12 +308,10 @@ extern int docRtfSkipGroup(	SimpleInputStream *	sis,
 
 extern int docRtfConsumeGroup(	SimpleInputStream *	sis,
 				int			textLevel,
-
-				RtfReadingContext *	internRrc,
-
+				RtfReadingContext *	rrc,
 				const RtfControlWord *	contolWords,
 				const RtfControlWord *	groupWords,
-				RtfAddTextParticule	addParticule	);
+				RtfAddTextParticule	addParticule );
 
 extern int docRtfReadPict(	SimpleInputStream *	sis,
 				const RtfControlWord *	rcw,
@@ -299,7 +335,7 @@ extern int docRtfReadObject(	SimpleInputStream *	sis,
 
 extern int docRtfAdjustLevel(	RtfReadingContext *	rrc,
 				int			toLevel,
-				int			textLevel	);
+				int			textLevel );
 
 extern int docRtfReadShape(	SimpleInputStream *	sis,
 				const RtfControlWord *	rcw,
@@ -320,7 +356,7 @@ extern int docRtfReadLookupEntry(SimpleInputStream *	sis,
 				int			arg,
 				RtfReadingContext *	rrc );
 
-extern int docRtfReadHdFt(	SimpleInputStream *	sis,
+extern int docRtfReadExtIt(	SimpleInputStream *	sis,
 				const RtfControlWord *	rcw,
 				int			arg,
 				RtfReadingContext *	rrc );
@@ -379,7 +415,7 @@ extern int docRtfStylesheet(		SimpleInputStream *	sis,
 					int			arg,
 					RtfReadingContext *	rrc );
 
-extern int docRtfListtable(		SimpleInputStream *	sis,
+extern int docRtfListTable(		SimpleInputStream *	sis,
 					const RtfControlWord *	rcw,
 					int			arg,
 					RtfReadingContext *	rrc );
@@ -409,6 +445,14 @@ extern void docRtfEscapeString(		const unsigned char *	s,
 					int			n,
 					SimpleOutputStream *	sos );
 
+extern void docRtfWriteStringDestination(
+					const char *		tag,
+					int *			pCol,
+					const unsigned char *	s,
+					int			n,
+					int			addSemicolon,
+					SimpleOutputStream *	sos );
+
 extern void docRtfWriteNextLine(	int *			pCol,
 					SimpleOutputStream *	sos );
 
@@ -419,12 +463,12 @@ extern void docRtfSaveBorder(		char *				tag,
 
 extern void docRtfSaveTextAttribute(	SimpleOutputStream *	sos,
 					int *			pCol,
-					unsigned int		updMask,
+					const PropertyMask *	updMask,
 					const TextAttribute *	ta );
 
 extern void docRtfSaveParagraphProperties( SimpleOutputStream *	sos,
 				int *				pCol,
-				unsigned int			updMask,
+				const PropertyMask *		updMask,
 				const ParagraphProperties *	pp );
 
 extern void docRtfWriteDestinationBegin( const char *		tag,
@@ -458,7 +502,7 @@ extern int docRtfReadWordGroup(	SimpleInputStream *	sis,
 extern void docRtfSaveSectionProperties( SimpleOutputStream *	sos,
 				const unsigned char *		outputMapping,
 				int *				pCol,
-				unsigned int			updMask,
+				const PropertyMask *		updMask,
 				const SectionProperties *	sp );
 
 extern void docRtfWriteFontTable( SimpleOutputStream *		sos,
@@ -504,6 +548,11 @@ extern int docRtfRememberTextProperty(	SimpleInputStream *	sis,
 					int			arg,
 					RtfReadingContext *	rrc );
 
+extern int docRtfRememberPntextProperty(SimpleInputStream *	sis,
+					const RtfControlWord *	rcw,
+					int			arg,
+					RtfReadingContext *	rrc );
+
 extern void docRtfWriteArgDestinationBegin( const char *	tag,
 					int *			pCol,
 					int			arg,
@@ -520,7 +569,7 @@ extern void docRtfReadSetAnsicpg(	RtfReadingContext *	rrc,
 extern void docRtfSaveTextFrameProperties(
 				SimpleOutputStream *		sos,
 				int *				pCol,
-				unsigned int			updMask,
+				const PropertyMask *		updMask,
 				const TextFrameProperties *	tfp );
 
 extern int docRtfRememberTextFrameProperty(
@@ -529,3 +578,138 @@ extern int docRtfRememberTextFrameProperty(
 					int			arg,
 					RtfReadingContext *	rrc );
 
+extern int docRtfSaveSectHeadersFooters(SimpleOutputStream *	sos,
+					int *			pCol,
+					const BufferItem *	bi,
+					const BufferDocument *	bd,
+					RtfWritingContext *	rwc );
+
+extern int docRtfSaveExternalItem(	SimpleOutputStream *	sos,
+					int *			pCol,
+					const BufferItem *	bi,
+					const BufferDocument *	bd,
+					const char *		tag,
+					RtfWritingContext *	rwc );
+
+extern int docRtfSaveDocNotesSeparators(SimpleOutputStream *	sos,
+					int *			pCol,
+					const BufferDocument *	bd,
+					RtfWritingContext *	rwc );
+
+extern void docRtfSaveCellProperties(	const CellProperties *	cp,
+					int *			pCol,
+					int			shiftLeft,
+					SimpleOutputStream *	sos );
+
+extern int docRtfRememberCellProperty(	SimpleInputStream *	sis,
+					const RtfControlWord *	rcw,
+					int			arg,
+					RtfReadingContext *	rrc );
+
+extern void docRtfSaveRowProperties(	const RowProperties *	rp,
+					int *			pCol,
+					int			col0,
+					int			col1,
+					SimpleOutputStream *	sos );
+
+extern int docRtfRememberRowProperty(	SimpleInputStream *	sis,
+					const RtfControlWord *	rcw,
+					int			arg,
+					RtfReadingContext *	rrc );
+
+extern void docRtfSaveAutoSpace(	int *			pCol,
+					SimpleOutputStream *	sos,
+					const char *		unitTag,
+					int			unit,
+					const char *		numberTag,
+					int			number );
+
+extern int docRtfRememberStyleProperty(	SimpleInputStream *	sis,
+					const RtfControlWord *	rcw,
+					int			arg,
+					RtfReadingContext *	rrc );
+
+extern int docRtfRememberListLevelProperty(
+					SimpleInputStream *	sis,
+					const RtfControlWord *	rcw,
+					int			arg,
+					RtfReadingContext *	rrc );
+
+extern int docRtfRememberListProperty(	SimpleInputStream *	sis,
+					const RtfControlWord *	rcw,
+					int			arg,
+					RtfReadingContext *	rrc );
+
+extern void docRtfWriteListTable(
+				SimpleOutputStream *		sos,
+				const unsigned char *		outputMapping,
+				int *				pCol,
+				const DocumentListTable *	dlt );
+
+extern void docRtfPushReadingState(	RtfReadingContext *	rrc,
+					RtfReadingState *	rrs );
+
+extern void docRtfPopReadingState(	RtfReadingContext *	rrc );
+
+extern int docRtfRememberListOverrideLevelProperty(
+					SimpleInputStream *	sis,
+					const RtfControlWord *	rcw,
+					int			arg,
+					RtfReadingContext *	rrc );
+
+extern int docRtfRememberListOverrideProperty(
+					SimpleInputStream *	sis,
+					const RtfControlWord *	rcw,
+					int			arg,
+					RtfReadingContext *	rrc );
+
+extern int docRtfListOverrideTable(	SimpleInputStream *	sis,
+					const RtfControlWord *	rcw,
+					int			arg,
+					RtfReadingContext *	rrc );
+
+extern void docRtfWriteListOverrideTable(
+				SimpleOutputStream *		sos,
+				const unsigned char *		outputMapping,
+				int *				pCol,
+				const ListOverrideTable *	lot );
+
+extern void docRtfWriteListLevel(
+				SimpleOutputStream *		sos,
+				const unsigned char *		outputMapping,
+				int *				pCol,
+				const DocumentListLevel *	dll );
+
+extern int docRtfRememberTabStopProperty(
+				SimpleInputStream *		sis,
+				const RtfControlWord *		rcw,
+				int				arg,
+				RtfReadingContext *		rrc );
+
+extern void docRtfSaveTabStopList(
+				SimpleOutputStream *		sos,
+				int *				pCol,
+				const TabStopList *		tsl );
+
+extern int docRtfFontProperty(	SimpleInputStream *		sis,
+				const RtfControlWord *		rcw,
+				int				arg,
+				RtfReadingContext *		rrc );
+
+extern unsigned long docRtfTagHash(	const unsigned char *	key,
+					int			len );
+
+extern int docRtfTagIndex(		const unsigned char *	key,
+					int			len );
+
+extern const RtfControlWord * docRtfFindPropertyWord(
+					const char *		controlWord );
+
+extern int docRtfObjectProperty(	SimpleInputStream *	sis,
+					const RtfControlWord *	rcw,
+					int			arg,
+					RtfReadingContext *	rrc );
+
+extern int docRtfCommitListLevelStyle(	RtfReadingContext *	rrc );
+
+#    endif	/*  DOC_RTF_H	*/
