@@ -1,10 +1,11 @@
 /************************************************************************/
 /*									*/
 /*  Read an RTF text file into a BufferDocument				*/
+/*  Read fields, bookmarks, footnotes, headers, footers.		*/
 /*									*/
 /************************************************************************/
 
-#   include	"config.h"
+#   include	"tedConfig.h"
 
 #   include	<stdlib.h>
 #   include	<string.h>
@@ -31,7 +32,7 @@ static int docRtfReadFldrslt(	SimpleInputStream *	sis,
     res= docRtfReadGroup( sis, DOClevPARA,
 				(RtfControlWord *)0, 0, 0, rrc,
 				docRtfDocumentWords, docRtfDocumentGroups,
-				docRtfTextParticule );
+				docRtfTextParticule, (RtfCommitGroup)0 );
 
     if  ( res )
 	{ SLDEB(rcw->rcwWord,res);	}
@@ -67,7 +68,7 @@ static int docRtfReadFldinst(	SimpleInputStream *	sis,
     res= docRtfReadGroup( sis, DOClevPARA,
 				(RtfControlWord *)0, 0, 0, rrc,
 				docRtfDocumentWords, docRtfDocumentGroups,
-				docRtfFldinst );
+				docRtfFldinst, (RtfCommitGroup)0 );
 
     if  ( res )
 	{ SLDEB(rcw->rcwWord,res);	}
@@ -97,11 +98,11 @@ static int docRtfReadStartField(	int *			pPart,
 					int *			pFieldNumber,
 					RtfReadingContext *	rrc	)
     {
+    RtfReadingState *	rrs= rrc->rrcState;
     BufferDocument *	bd= rrc->rrcBd;
     BufferItem *	bi= rrc->rrcBi;
 
     DocumentField *	df;
-    TextParticule *	tp;
 
     int			part= bi->biParaParticuleCount;
 
@@ -113,12 +114,14 @@ static int docRtfReadStartField(	int *			pPart,
 	{ XDEB(df); return -1;	}
     df->dfKind= DOCfkUNKNOWN;
 
-    tp= docInsertTextParticule( bi, -1, bi->biParaStrlen, 0,
-			DOCkindFIELDSTART, rrc->rrcCurrentTextAttribute );
-    if  ( ! tp )
-	{ LDEB(bi->biParaParticuleCount); return -1;	}
+    if  ( docInsertAdminParticule( bd, bi, bi->biParaStrlen,
+					    bi->biParaParticuleCount,
+					    rrc->rrcFieldNumber,
+					    &(rrs->rrsTextAttribute),
+					    DOCkindFIELDSTART ) )
+	{ LDEB(1); return -1;	}
 
-    tp->tpObjectNumber= rrc->rrcFieldNumber;
+    rrc->rrcAfterNoteref= 0;
 
     *pFieldNumber= rrc->rrcFieldNumber;
     *pPart= part;
@@ -130,16 +133,20 @@ static int docRtfTerminateField(	BufferItem *		bi,
 					int			fieldNumber,
 					RtfReadingContext *	rrc )
     {
-    TextParticule *	tp;
+    RtfReadingState *	rrs= rrc->rrcState;
+    BufferDocument *	bd= rrc->rrcBd;
 
     if  ( rrc->rrcBi != bi )
 	{ XXDEB(rrc->rrcBi,bi); }
 
-    tp= docInsertTextParticule( bi, -1, bi->biParaStrlen, 0,
-			DOCkindFIELDEND, rrc->rrcCurrentTextAttribute );
-    if  ( ! tp )
-	{ LDEB(bi->biParaParticuleCount); return -1;	}
-    tp->tpObjectNumber= fieldNumber;
+    if  ( docInsertAdminParticule( bd, bi, bi->biParaStrlen,
+					    bi->biParaParticuleCount,
+					    fieldNumber,
+					    &(rrs->rrsTextAttribute),
+					    DOCkindFIELDEND ) )
+	{ LDEB(1); return -1;	}
+
+    rrc->rrcAfterNoteref= 0;
 
     return 0;
     }
@@ -219,7 +226,7 @@ int docRtfReadField(	SimpleInputStream *	sis,
     res= docRtfReadGroup( sis, DOClevPARA,
 				(RtfControlWord *)0, 0, 0, rrc,
 				docRtfFieldWords, docRtfFieldGroups,
-				docRtfIgnoreText );
+				docRtfIgnoreText, (RtfCommitGroup)0 );
     if  ( res )
 	{ SLDEB(rcw->rcwWord,res);	}
 
@@ -267,7 +274,7 @@ int docRtfReadLookupEntry(	SimpleInputStream *	sis,
     RtfControlWord *	contolWords= (RtfControlWord *)0;
     RtfControlWord *	groupWords= (RtfControlWord *)0;
 
-    switch( rcw->rcwId )
+    switch( rcw->rcwID )
 	{
 	case RTFidXE:
 	    particuleKind= DOCkindXE;
@@ -297,7 +304,7 @@ int docRtfReadLookupEntry(	SimpleInputStream *	sis,
     res= docRtfReadGroup( sis, DOClevPARA,
 				firstApply, 0, 0, rrc,
 				contolWords, groupWords,
-				docRtfFldinst );
+				docRtfFldinst, (RtfCommitGroup)0 );
 
     if  ( res )
 	{ SLDEB(rcw->rcwWord,res);	}
@@ -313,7 +320,7 @@ int docRtfReadLookupEntry(	SimpleInputStream *	sis,
 
 static int docRtfBookmarkName(	RtfReadingContext *	rrc,
 				const unsigned char *	name,
-				int			len	)
+				int			len )
     {
     if  ( rrc->rrcBookmarkSize+ len > DOCmaxBOOKMARK )
 	{
@@ -341,7 +348,7 @@ static RtfControlWord	docRtfBkmkStartWords[]=
 int docRtfBkmkStart(	SimpleInputStream *	sis,
 			const RtfControlWord *	rcw,
 			int			arg,
-			RtfReadingContext *	rrc	)
+			RtfReadingContext *	rrc )
     {
     BufferDocument *	bd= rrc->rrcBd;
     RtfBookmarkLevel *	rbl;
@@ -361,7 +368,7 @@ int docRtfBkmkStart(	SimpleInputStream *	sis,
     res= docRtfReadGroup( sis, DOClevPARA,
 				(RtfControlWord *)0, 0, 0, rrc,
 				docRtfBkmkStartWords, docRtfEmptyTable,
-				docRtfBookmarkName );
+				docRtfBookmarkName, (RtfCommitGroup)0 );
 
     if  ( res )
 	{ SLDEB(rcw->rcwWord,res); return res;	}
@@ -436,7 +443,7 @@ int docRtfBkmkEnd(	SimpleInputStream *	sis,
     res= docRtfReadGroup( sis, DOClevPARA,
 				(RtfControlWord *)0, 0, 0, rrc,
 				docRtfEmptyTable, docRtfEmptyTable,
-				docRtfBookmarkName );
+				docRtfBookmarkName, (RtfCommitGroup)0 );
 
     if  ( res )
 	{ SLDEB(rcw->rcwWord,res); return res;	}
@@ -491,7 +498,7 @@ int docRtfBkmkEnd(	SimpleInputStream *	sis,
 int docRtfSpecialToField(	SimpleInputStream *	sis,
 				const RtfControlWord *	rcw,
 				int			arg,
-				RtfReadingContext *	rrc	)
+				RtfReadingContext *	rrc )
     {
     BufferItem *	bi= rrc->rrcBi;
     BufferDocument *	bd= rrc->rrcBd;
@@ -500,7 +507,7 @@ int docRtfSpecialToField(	SimpleInputStream *	sis,
     int			startParticule;
     int			fieldNumber;
 
-    switch( rcw->rcwId )
+    switch( rcw->rcwID )
 	{
 	case RTFidCHPGN:
 	    break;
@@ -530,99 +537,160 @@ int docRtfSpecialToField(	SimpleInputStream *	sis,
 
 /************************************************************************/
 /*									*/
-/*  Consume Headers and Footers.					*/
+/*  Consume Headers, Footers, Notes &c: Separate item trees that are.	*/
+/*  embedded in the document.						*/
+/*									*/
+/*  1)  Save the current position.					*/
+/*  2)  Make the header/note &c.					*/
+/*  3)  Consume its contents.						*/
+/*  4)  Make sure that no bookmarks protrude beyond the end of the	*/
+/*	external item.							*/
+/*  5)  If the external item happens to be empty, insert an empty	*/
+/*	paragraph to avoid problems later on.				*/
+/*  6)  Restore the current position.					*/
 /*									*/
 /************************************************************************/
 
 static int docRtfReadExternalItem(	BufferItem **		pBi,
+					int *			pExtItKind,
 					SimpleInputStream *	sis,
 					RtfReadingContext *	rrc,
-					int			inHdFt )
+					const SelectionScope *	ss )
     {
+    RtfReadingState *	rrs= rrc->rrcState;
     BufferItem *	bi;
 
     BufferItem *	savedBi;
     int			savedLevel;
-    int			savedInField;
+    int			savedInTable;
+    int			savedExternalItemKind;
 
     int			res;
 
+    /*  1  */
     savedBi= rrc->rrcBi;
     savedLevel= rrc->rrcLevel;
-    savedInField= rrc->rrcParagraphProperties.ppInTable;
+    savedInTable= rrs->rrsParagraphProperties.ppInTable;
+    savedExternalItemKind= rrc->rrcExternalItemKind;
     if  ( ! savedBi )
 	{ XDEB(savedBi); return -1;	}
 
+    /*  2  */
     bi= *pBi;
     if  ( ! bi )
 	{
 	bi= docMakeExternalItem( rrc->rrcBd,
-				    &(rrc->rrcSectionProperties), inHdFt );
+				    ss, &(rrs->rrsSectionProperties) );
 	if  ( ! bi )
 	    { XDEB(bi); return -1;	}
 
 	*pBi= bi;
 	}
 
+    /*  3  */
     rrc->rrcBi= bi;
     rrc->rrcLevel= DOClevSECT;
+    rrc->rrcExternalItemKind= ss->ssInExternalItem;
 
     res= docRtfReadGroup( sis, DOClevPARA,
 				(RtfControlWord *)0, 0, 0, rrc,
 				docRtfDocumentWords, docRtfDocumentGroups,
-				docRtfTextParticule );
+				docRtfTextParticule, (RtfCommitGroup)0 );
     if  ( res )
 	{ LDEB(res);	}
 
+    /*  4  */
     if  ( docRtfFlushBookmarks( -1, bi, rrc ) )
 	{ LDEB(1); return -1;	}
 
-    rrc->rrcParagraphProperties.ppInTable= savedInField;
+    /*  5  */
+    {
+    DocumentPosition	dp;
+
+    if  ( docFirstPosition( &dp, bi ) )
+	{
+	const int	textAttributeNumber= 0;
+
+	if  ( ! docInsertEmptyParagraph( rrc->rrcBd, bi,
+						textAttributeNumber ) )
+	    { LDEB(textAttributeNumber);	}
+	}
+    }
+
+    *pExtItKind= rrc->rrcExternalItemKind;
+
+    /*  6  */
+    rrc->rrcExternalItemKind= savedExternalItemKind;
+    rrs->rrsParagraphProperties.ppInTable= savedInTable;
     rrc->rrcLevel= savedLevel;
     rrc->rrcBi= savedBi;
 
     return res;
     }
 
-int docRtfReadHdFt(		SimpleInputStream *	sis,
+int docRtfReadExtIt(		SimpleInputStream *	sis,
 				const RtfControlWord *	rcw,
 				int			arg,
 				RtfReadingContext *	rrc )
     {
-    BufferItem **	pBi;
+    BufferItem *	sectBi= rrc->rrcBi;
+    BufferDocument *	bd= rrc->rrcBd;
+    ExternalItem *	ei;
 
-    switch( rcw->rcwId )
+    SelectionScope	ss;
+    int			extItKind;
+
+    docInitSelectionScope( &ss );
+
+    switch( rcw->rcwID )
 	{
 	case DOCinSECT_HEADER:
-	    pBi= &(rrc->rrcBi->biSectHeader.hfItem);
-	    break;
 	case DOCinFIRST_HEADER:
-	    pBi= &(rrc->rrcBi->biSectFirstPageHeader.hfItem);
-	    break;
 	case DOCinLEFT_HEADER:
-	    pBi= &(rrc->rrcBi->biSectLeftPageHeader.hfItem);
-	    break;
 	case DOCinRIGHT_HEADER:
-	    pBi= &(rrc->rrcBi->biSectRightPageHeader.hfItem);
+	case DOCinSECT_FOOTER:
+	case DOCinFIRST_FOOTER:
+	case DOCinLEFT_FOOTER:
+	case DOCinRIGHT_FOOTER:
+
+	    if  ( ! sectBi || sectBi->biLevel != DOClevSECT )
+		{ XDEB(sectBi); return -1;	}
+
+	    ei= docSectionHeaderFooter( sectBi, rcw->rcwID );
+	    if  ( ! ei )
+		{ LXDEB(rcw->rcwID,ei); return -1;	}
+
+	    ss.ssSectNrExternalTo= sectBi->biNumberInParent;
+
 	    break;
 
-	case DOCinSECT_FOOTER:
-	    pBi= &(rrc->rrcBi->biSectFooter.hfItem);
-	    break;
-	case DOCinFIRST_FOOTER:
-	    pBi= &(rrc->rrcBi->biSectFirstPageFooter.hfItem);
-	    break;
-	case DOCinLEFT_FOOTER:
-	    pBi= &(rrc->rrcBi->biSectLeftPageFooter.hfItem);
-	    break;
-	case DOCinRIGHT_FOOTER:
-	    pBi= &(rrc->rrcBi->biSectRightPageFooter.hfItem);
+	case DOCinFTNSEP:
+	case DOCinFTNSEPC:
+	case DOCinFTNCN:
+	case DOCinAFTNSEP:
+	case DOCinAFTNSEPC:
+	case DOCinAFTNCN:
+
+	    if  ( ! bd )
+		{ XDEB(bd); return -1;	}
+
+	    ei= docDocumentNoteSeparator( bd, rcw->rcwID );
+	    if  ( ! ei )
+		{ LXDEB(rcw->rcwID,ei); return -1;	}
+
 	    break;
+
 	default:
 	    SDEB(rcw->rcwWord); return -1;
 	}
 
-    if  ( docRtfReadExternalItem( pBi, sis, rrc, rcw->rcwId ) )
+    if  ( ei->eiItem )
+	{ SXDEB(rcw->rcwWord,ei->eiItem);	}
+
+    ss.ssInExternalItem= rcw->rcwID;
+    ss.ssNoteArrayIndex= -1;
+
+    if  ( docRtfReadExternalItem( &(ei->eiItem), &extItKind, sis, rrc, &ss ) )
 	{ SDEB(rcw->rcwWord); return -1;	}
 
     return 0;
@@ -639,24 +707,56 @@ int docRtfReadFootnote(		SimpleInputStream *	sis,
 				int			arg,
 				RtfReadingContext *	rrc )
     {
+    RtfReadingState *	rrs= rrc->rrcState;
     BufferDocument *	bd= rrc->rrcBd;
+    BufferItem *	bi= rrc->rrcBi;
 
-    int			startParticule;
-    int			fieldNumber= -1;
-    DocumentField *	df;
+    DocumentNote *	dn;
+    int			noteIndex;
 
-    if  ( docRtfReadStartField( &startParticule, &fieldNumber, rrc ) )
+    BufferItem *	sectBi= bi;
+    SelectionScope	ss;
+
+    int			externalItemKind;
+    int			autoNumber= 0;
+
+    const int		objectNumber= -1;
+
+    docInitSelectionScope( &ss );
+
+    while( sectBi && sectBi->biLevel != DOClevSECT )
+	{ sectBi= sectBi->biParent;	}
+
+    if  ( ! sectBi )
+	{ XDEB(sectBi); return -1;	}
+
+    if  ( rrc->rrcAfterNoteref )
+	{ autoNumber= 1;	}
+
+    noteIndex= docInsertNote( &dn, bd, bi, bi->biParaStrlen, autoNumber );
+    if  ( noteIndex < 0 )
+	{ LDEB(noteIndex); return -1;	}
+
+    if  ( docInsertAdminParticule( bd, bi, bi->biParaStrlen,
+					    bi->biParaParticuleCount,
+					    objectNumber,
+					    &(rrs->rrsTextAttribute),
+					    DOCkindNOTE ) )
 	{ LDEB(1); return -1;	}
 
-    df= bd->bdFieldList.dflFields+ fieldNumber;
-    df->dfKind= DOCfkFOOTNOTE;
+    rrc->rrcAfterNoteref= 0;
 
-    if  ( docRtfReadExternalItem( &(df->dfReferencedItem), sis, rrc,
-							DOCinFOOTNOTE ) )
+    ss.ssInExternalItem= DOCinFOOTNOTE;
+    ss.ssSectNrExternalTo= sectBi->biNumberInParent;
+    ss.ssNoteArrayIndex= noteIndex;
+
+    if  ( docRtfReadExternalItem( &(dn->dnExternalItem.eiItem),
+					&externalItemKind, sis, rrc, &ss ) )
 	{ SDEB(rcw->rcwWord); return -1;	}
 
-    if  ( docRtfTerminateField( rrc->rrcBi, fieldNumber, rrc ) )
-	{ LDEB(1); return -1;	}
+    dn->dnAutoNumber= autoNumber;
+    dn->dnExternalItemKind= externalItemKind;
+    docSetExternalItemKind( dn->dnExternalItem.eiItem, externalItemKind );
 
     return 0;
     }
@@ -666,30 +766,28 @@ int docRtfChftn(		SimpleInputStream *	sis,
 				int			arg,
 				RtfReadingContext *	rrc )
     {
+    RtfReadingState *	rrs= rrc->rrcState;
     BufferDocument *	bd= rrc->rrcBd;
 
     int			startParticule;
     int			fieldNumber= -1;
     DocumentField *	df;
 
-    TextAttribute	ta= rrc->rrcCurrentTextAttribute;
-
     if  ( docRtfReadStartField( &startParticule, &fieldNumber, rrc ) )
 	{ LDEB(1); return -1;	}
 
     df= bd->bdFieldList.dflFields+ fieldNumber;
     df->dfKind= DOCfkCHFTN;
 
-    /*  No: just fake \\up in the formatter temporarily?
-    ta.taSuperSub= DOCfontSUPERSCRIPT;
-    */
-
-    if  ( docSaveParticules( rrc->rrcBi, ta, rrc->rrcInputMapping,
-					    (const unsigned char *)"1", 1 ) )
+    if  ( docSaveParticules( rrc->rrcBd, rrc->rrcBi,
+				    &(rrs->rrsTextAttribute),
+				    (const unsigned char *)"1", 1 ) )
 	{ LDEB(1);	}
 
     if  ( docRtfTerminateField( rrc->rrcBi, fieldNumber, rrc ) )
 	{ LDEB(1); return -1;	}
 
+    rrc->rrcAfterNoteref= 1;
+
     return 0;
     }
