@@ -4,7 +4,7 @@
 /*									*/
 /************************************************************************/
 
-#   include	"config.h"
+#   include	"tedConfig.h"
 
 #   include	<stdlib.h>
 #   include	<string.h>
@@ -235,7 +235,7 @@ static int docRtfShpProperty(	SimpleInputStream *	sis,
 				int			arg,
 				RtfReadingContext *	rrc	)
     {
-    switch( rcw->rcwId )
+    switch( rcw->rcwID )
 	{
 	case	RTFidSHPLEFT:
 	    rrc->rrcShapeProperties.spTwipsLeftOfAnchor= arg;
@@ -335,7 +335,7 @@ static int docRtfShpText(	SimpleInputStream *	sis,
     res= docRtfReadGroup( sis, DOClevPARA,
 				(RtfControlWord *)0, 0, 0, rrc,
 				docRtfDocumentWords, docRtfDocumentGroups,
-				docRtfTextParticule );
+				docRtfTextParticule, (RtfCommitGroup)0 );
 
     if  ( res )
 	{ SLDEB(rcw->rcwWord,res);	}
@@ -477,7 +477,7 @@ static int docRtfShapeArray(	SimpleInputStream *	sis,
 static int docRtfShapePicture(	SimpleInputStream *	sis,
 				const RtfControlWord *	rcw,
 				int			arg,
-				RtfReadingContext *	rrc	)
+				RtfReadingContext *	rrc )
     {
     int				control;
     char			controlWord[TEDszRTFCONTROL+1];
@@ -489,6 +489,10 @@ static int docRtfShapePicture(	SimpleInputStream *	sis,
     if  ( control != RTFfiCTRLGROUP )
 	{ SLDEB(rcw->rcwWord,control); return -1; }
 
+    /*
+    LSDEB(rrc->rrcCurrentLine,controlWord);
+    */
+
     if  ( strcmp( controlWord, "pict" ) )
 	{ SDEB(controlWord); return -1;	}
 
@@ -552,10 +556,10 @@ static int docRtfShapeNumber(	SimpleInputStream *	sis,
 
     sioInUngetLastRead( sis );
 
-    switch( rcw->rcwId )
+    switch( rcw->rcwID )
 	{
 	case RTFSHP_shapeType:
-	    if  ( rcw->rcwId != RTFSHP_shapeType )
+	    if  ( rcw->rcwID != RTFSHP_shapeType )
 		{ SDEB(docRtfShapeTypeString(val));	}
 
 	    ds->ds_shapeType= val;
@@ -1221,13 +1225,35 @@ static int docRtfShpinst(	SimpleInputStream *	sis,
     res= docRtfReadGroup( sis, DOClevPARA,
 				(RtfControlWord *)0, 0, 0, rrc,
 				docRtfShapeWords, docRtfShpinstGroups,
-				docRtfIgnoreText );
+				docRtfIgnoreText, (RtfCommitGroup)0 );
     if  ( res )
 	{ SLDEB(rcw->rcwWord,res);	}
 
     return res;
     }
 
+static int docRtfShprslt(	SimpleInputStream *	sis,
+				const RtfControlWord *	rcw,
+				int			arg,
+				RtfReadingContext *	rrc	)
+    {
+    int		res;
+
+    rrc->rrcInIgnoredGroup++;
+
+    res= docRtfReadGroup( sis, DOClevANY,
+				(RtfControlWord *)0, 0, 0, rrc,
+				(const RtfControlWord *)0,
+				(const RtfControlWord *)0,
+				docRtfIgnoreText, (RtfCommitGroup)0 );
+    if  ( res )
+	{ SLDEB(rcw->rcwWord,res);	}
+
+    rrc->rrcInIgnoredGroup--;
+
+    return res;
+    }
+
 /************************************************************************/
 /*									*/
 /*  The actual shape as opposed to the result in simpler RTF.		*/
@@ -1237,7 +1263,7 @@ static int docRtfShpinst(	SimpleInputStream *	sis,
 static RtfControlWord	docRtfShapeGroups[]=
     {
 	{ "shptxt",	RTFidSHPTXT,	DOClevPARA, docRtfShpText, },
-	{ "shprslt",	RTFidSHPRSLT,	DOClevPARA, docRtfSkipGroup, },
+	{ "shprslt",	RTFidSHPRSLT,	DOClevPARA, docRtfShprslt, },
 	{ "shpgrp",	RTFidSHPGRP,	DOClevPARA, docRtfReadShape, },
 	{ "shp",	RTFidSHP,	DOClevPARA, docRtfReadShape, },
 	{ "shpinst",	RTFidSHPINST,	DOClevPARA, docRtfShpinst, },
@@ -1255,7 +1281,7 @@ int docRtfReadShape(	SimpleInputStream *	sis,
     res= docRtfReadGroup( sis, DOClevTEXT,
 				(RtfControlWord *)0, 0, 0, rrc,
 				docRtfShapeWords, docRtfShapeGroups,
-				docRtfIgnoreParticule );
+				docRtfIgnoreText, (RtfCommitGroup)0 );
 
     if  ( res )
 	{ SLDEB(rcw->rcwWord,res);	}
