@@ -231,10 +231,11 @@ int docRtfRememberSectionProperty(	SimpleInputStream *	sis,
 					int			arg,
 					RtfReadingContext *	rrc )
     {
-    SectionProperties *		sp= &(rrc->rrcSectionProperties);
+    RtfReadingState *		rrs= rrc->rrcState;
+    SectionProperties *		sp= &(rrs->rrsSectionProperties);
     DocumentGeometry *		dg= &(sp->spDocumentGeometry);
 
-    switch( rcw->rcwId )
+    switch( rcw->rcwID )
 	{
 	case SPprop_NONE:
 	    {
@@ -245,10 +246,10 @@ int docRtfRememberSectionProperty(	SimpleInputStream *	sis,
 		{ XDEB(rrc->rrcBd); return -1;	}
 
 	    dgDoc= &(rrc->rrcBd->bdProperties.dpGeometry);
-	    dgSect= &(rrc->rrcSectionProperties.spDocumentGeometry);
+	    dgSect= &(sp->spDocumentGeometry);
 
-	    docCleanSectionProperties( &(rrc->rrcSectionProperties) );
-	    docInitSectionProperties(  &(rrc->rrcSectionProperties) );
+	    docCleanSectionProperties( sp );
+	    docInitSectionProperties( sp );
 
 	    *dgSect= *dgDoc;
 	    }
@@ -293,8 +294,8 @@ int docRtfRememberSectionProperty(	SimpleInputStream *	sis,
 	    break;
 
 	case SPpropSTYLE:
+	    rrc->rrcDocumentStyle.dsLevel= DOClevSECT;
 	    sp->spStyle= arg;
-	    rrc->rrcStyle.dsLevel= DOClevSECT;
 	    break;
 
 	case SPpropTITLEPG:
@@ -334,7 +335,7 @@ int docRtfRememberSectionProperty(	SimpleInputStream *	sis,
 	    return 0;
 	}
 
-    PROPmaskADD( &(rrc->rrcStyle.dsSectMask), rcw->rcwId );
+    PROPmaskADD( &(rrc->rrcDocumentStyle.dsSectMask), rcw->rcwID );
 
     return 0;
     }
