@@ -25,39 +25,47 @@
 
 static int docRtfStyleName(	RtfReadingContext *	rrc,
 				const unsigned char *	name,
-				int			len	)
+				int			len )
     {
+    RtfReadingState *	rrs= rrc->rrcState;
     DocumentStyle *	ds;
 
     PropertyMask	ppChgMask;
     PropertyMask	ppUpdMask;
 
-    switch( rrc->rrcStyle.dsLevel )
+    ds= &(rrc->rrcDocumentStyle);
+    switch( ds->dsLevel )
 	{
 	case DOClevTEXT:
 	    break;
 
+	case -1:
+	    LDEB(rrc->rrcDocumentStyle.dsLevel);
+	    ds->dsStyleNumber= 0;
+	    ds->dsLevel= DOClevPARA;
+	    break;
+
 	case DOClevPARA:
-	    rrc->rrcStyle.dsStyleNumber= rrc->rrcParagraphProperties.ppStyle;
+	    ds->dsStyleNumber= rrs->rrsParagraphProperties.ppStyle;
 	    break;
 
 	case DOClevSECT:
-	    rrc->rrcStyle.dsStyleNumber= rrc->rrcSectionProperties.spStyle;
+	    ds->dsStyleNumber= rrs->rrsSectionProperties.spStyle;
 	    break;
 
 	default:
-	    LDEB(rrc->rrcStyle.dsLevel); return -1;
+	    LDEB(rrc->rrcDocumentStyle.dsLevel); return -1;
 	}
 
     if  ( len > 0 && name[len-1] == ';' )
 	{ len--;	}
 
     ds= docInsertStyle( &(rrc->rrcBd->bdStyleSheet),
-						rrc->rrcStyle.dsStyleNumber );
+					rrc->rrcDocumentStyle.dsStyleNumber );
     if  ( ! ds )
 	{ XDEB(ds); return -1;	}
 
-    if  ( docCopyStyle( ds, &(rrc->rrcStyle) ) )
+    if  ( docCopyStyle( ds, &(rrc->rrcDocumentStyle) ) )
 	{ LDEB(1); return -1;	}
 
     if  ( len > 0 );
@@ -74,7 +82,7 @@ static int docRtfStyleName(	RtfReadingContext *	rrc,
 	}
 
     docCopySectionProperties( &(ds->dsSectionProperties),
-					    &(rrc->rrcSectionProperties) );
+					    &(rrs->rrsSectionProperties) );
 
     PROPmaskCLEAR( &ppChgMask );
 
@@ -82,17 +90,18 @@ static int docRtfStyleName(	RtfReadingContext *	rrc,
     PROPmaskFILL( &ppUpdMask, PPprop_COUNT );
 
     docUpdParaProperties( &ppChgMask, &(ds->dsParagraphProperties),
-				&ppUpdMask, &(rrc->rrcParagraphProperties) );
+				&ppUpdMask, &(rrs->rrsParagraphProperties),
+				(const int *)0 );
 
-    ds->dsTextAttribute= rrc->rrcCurrentTextAttribute;
+    ds->dsTextAttribute= rrs->rrsTextAttribute;
 
-    docCleanStyle( &(rrc->rrcStyle) );
-    docInitStyle( &(rrc->rrcStyle) );
+    docCleanDocumentStyle( &(rrc->rrcDocumentStyle) );
+    docInitDocumentStyle( &(rrc->rrcDocumentStyle) );
 
-    docCleanParagraphProperties( &(rrc->rrcParagraphProperties) );
-    docInitParagraphProperties( &(rrc->rrcParagraphProperties) );
-    docInitTextAttribute( &(rrc->rrcCurrentTextAttribute) );
-    rrc->rrcCurrentTextAttribute.taFontSizeHalfPoints= 24;
+    docCleanParagraphProperties( &(rrs->rrsParagraphProperties) );
+    docInitParagraphProperties( &(rrs->rrsParagraphProperties) );
+    utilInitTextAttribute( &(rrs->rrsTextAttribute) );
+    rrs->rrsTextAttribute.taFontSizeHalfPoints= 24;
     rrc->rrcInDeletedText= 0;
 
     return 0;
@@ -101,33 +110,35 @@ static int docRtfStyleName(	RtfReadingContext *	rrc,
 static int docRtfStyleGroup(		SimpleInputStream *	sis,
 					const RtfControlWord *	rcw,
 					int			arg,
-					RtfReadingContext *	rrc	)
+					RtfReadingContext *	rrc )
     {
-    switch( rcw->rcwId )
+    RtfReadingState *	rrs= rrc->rrcState;
+
+    switch( rcw->rcwID )
 	{
 	case RTFidCS:
-	    rrc->rrcStyle.dsStyleNumber= arg;
-	    rrc->rrcStyle.dsLevel= DOClevTEXT;
+	    rrc->rrcDocumentStyle.dsStyleNumber= arg;
+	    rrc->rrcDocumentStyle.dsLevel= DOClevTEXT;
 	    break;
 
 	case RTFidDS:
-	    rrc->rrcSectionProperties.spStyle= arg;
-	    rrc->rrcStyle.dsLevel= DOClevSECT;
+	    rrs->rrsSectionProperties.spStyle= arg;
+	    rrc->rrcDocumentStyle.dsLevel= DOClevSECT;
 	    break;
 
 	case RTFidS:
-	    rrc->rrcParagraphProperties.ppStyle= arg;
-	    rrc->rrcStyle.dsLevel= DOClevPARA;
+	    rrs->rrsParagraphProperties.ppStyle= arg;
+	    rrc->rrcDocumentStyle.dsLevel= DOClevPARA;
 	    break;
 
 	default:
-	    LDEB(rcw->rcwId);
+	    LDEB(rcw->rcwID);
 	}
 
     if  ( docRtfReadGroup( sis, rcw->rcwLevel,
 				    (RtfControlWord *)0, 0, 0, rrc,
 				    docRtfEmptyTable, docRtfEmptyTable,
-				    docRtfStyleName ) )
+				    docRtfStyleName, (RtfCommitGroup)0 ) )
 	{ SLDEB(rcw->rcwWord,arg); return -1;	}
 
     return 0;
@@ -157,6 +168,7 @@ int docRtfStylesheet(		SimpleInputStream *	sis,
 				int			arg,
 				RtfReadingContext *	rrc )
     {
+    RtfReadingState *		rrs= rrc->rrcState;
     int				res;
 
     char			controlWord[TEDszRTFCONTROL+1];
@@ -167,13 +179,13 @@ int docRtfStylesheet(		SimpleInputStream *	sis,
     
     const int			listOnly= 1;
 
-    docCleanStyle( &(rrc->rrcStyle) );
-    docInitStyle( &(rrc->rrcStyle) );
+    docCleanDocumentStyle( &(rrc->rrcDocumentStyle) );
+    docInitDocumentStyle( &(rrc->rrcDocumentStyle) );
 
 #   if 0
     docCleanParagraphProperties( &(rrc->rrcParagraphProperties) );
     docInitParagraphProperties( &(rrc->rrcParagraphProperties) );
-    docInitTextAttribute( &(rrc->rrcCurrentTextAttribute) );
+    utilInitTextAttribute( &(rrc->rrcCurrentTextAttribute) );
     rrc->rrcCurrentTextAttribute.taFontSizeHalfPoints= 24;
     rrc->rrcInDeletedText= 0;
 #   endif
@@ -182,11 +194,13 @@ int docRtfStylesheet(		SimpleInputStream *	sis,
     if  ( res < 0 )
 	{ LDEB(res); return -1;	}
 
-    rrc->rrcStyle.dsLevel= DOClevPARA;
-    rrc->rrcParagraphProperties.ppStyle= 0;
+    rrc->rrcDocumentStyle.dsLevel= DOClevPARA;
+    rrs->rrsParagraphProperties.ppStyle= 0;
 
     for (;;)
 	{
+	rrs= rrc->rrcState;
+
 	switch( res )
 	    {
 	    case RTFfiCLOSE:
@@ -228,8 +242,12 @@ int docRtfStylesheet(		SimpleInputStream *	sis,
 		if  ( rcw )
 		    { goto groupFound; }
 
+		rrc->rrcInIgnoredGroup++;
+
 		if  ( docRtfReadUnknownGroup( sis, rrc ) )
-		    { LDEB(1); return -1;	}
+		    { LDEB(1); rrc->rrcInIgnoredGroup--; return -1;	}
+
+		rrc->rrcInIgnoredGroup--;
 
 		res= docRtfFindControl( sis, rrc, &c,
 					    controlWord, &gotArg, &arg );
@@ -321,6 +339,8 @@ void docRtfWriteStyleSheet(	SimpleOutputStream *		sos,
 	    { docRtfWriteTag( "\\sautoupd", pCol, sos ); }
 	if  ( ds->dsHidden )
 	    { docRtfWriteTag( "\\shidden", pCol, sos ); }
+	if  ( ds->dsPersonal )
+	    { docRtfWriteTag( "\\spersonal", pCol, sos ); }
 
 	sioOutPutCharacter( ' ', sos ); *pCol += 1;
 	docRtfEscapeString( ds->dsName, outputMapping,
@@ -337,3 +357,50 @@ void docRtfWriteStyleSheet(	SimpleOutputStream *		sos,
     return;
     }
 
+/************************************************************************/
+/*									*/
+/*  Handle a style property when reading RTF.				*/
+/*									*/
+/************************************************************************/
+
+int docRtfRememberStyleProperty(	SimpleInputStream *	sis,
+					const RtfControlWord *	rcw,
+					int			arg,
+					RtfReadingContext *	rrc )
+    {
+    DocumentStyle *	ds= &(rrc->rrcDocumentStyle);
+
+    switch( rcw->rcwID )
+	{
+	/* \s \cs \ds tags but \s and \ds handeled with the para/sect style */
+	case DSpropSTYLE_NUMBER:
+	    ds->dsStyleNumber= arg;
+	    ds->dsLevel= rcw->rcwEnumValue;
+	    break;
+
+	case DSpropADDITIVE:
+	    ds->dsAdditive= arg != 0;
+	    break;
+	case DSpropBASED_ON:
+	    ds->dsBasedOn= arg;
+	    break;
+	case DSpropNEXT:
+	    ds->dsNext= arg;
+	    break;
+	case DSpropAUTO_UPD:
+	    ds->dsAutoupd= arg != 0;
+	    break;
+	case DSpropHIDDEN:
+	    ds->dsHidden= arg != 0;
+	    break;
+	case DSpropPERSONAL:
+	    ds->dsPersonal= arg != 0;
+	    break;
+
+	default:
+	    SDEB(rcw->rcwWord);
+	    return -1;
+	}
+
+    return 0;
+    }
