@@ -4,7 +4,7 @@
 /*									*/
 /************************************************************************/
 
-#   include	"config.h"
+#   include	"tedConfig.h"
 
 #   include	<stdlib.h>
 #   include	<string.h>
@@ -19,11 +19,14 @@
 #   include	"docRtf.h"
 
 /************************************************************************/
+/*									*/
 /*  An empty tag table.							*/
+/*									*/
 /************************************************************************/
+
 RtfControlWord	docRtfEmptyTable[]=
     {
-	{ 0, }
+	{ 0, 0, 0, }
     };
 
 /************************************************************************/
@@ -38,6 +41,9 @@ void docRtfInitReadingContext(	RtfReadingContext *	rrc	)
 
     rrc->rrcLevel= DOClevOUT;
     rrc->rrcSplitLevel= DOClevOUT;
+    rrc->rrcState= (RtfReadingState *)0;
+    rrc->rrcInIgnoredGroup= 0;
+
 				/****************************************/
 				/*  The depth in the document		*/
 				/*  == Type of BufferItem.		*/
@@ -47,6 +53,7 @@ void docRtfInitReadingContext(	RtfReadingContext *	rrc	)
 				/*  Complain about unknown control.	*/
 				/****************************************/
     rrc->rrcCharacterAhead= EOF;
+    rrc->rrcAfterNoteref= 0;
 
     rrc->rrcCurrentLine= 1;
     rrc->rrcBd= (BufferDocument *)0;
@@ -54,11 +61,8 @@ void docRtfInitReadingContext(	RtfReadingContext *	rrc	)
 				/****************************************/
 				/*  The current position in the input.	*/
 				/****************************************/
-    docInitTextAttribute( &(rrc->rrcCurrentTextAttribute) );
-    docInitBorderProperties( &(rrc->rrcTextBorderProperties) );
     rrc->rrcInDeletedText= 0;
 
-    rrc->rrcCurrentTextAttribute.taFontSizeHalfPoints= 24;
     rrc->rrcTextShadingPattern= DOCspSOLID;
     rrc->rrcTextShadingLevel= 0;
     rrc->rrcTextForeground= -1;
@@ -67,12 +71,15 @@ void docRtfInitReadingContext(	RtfReadingContext *	rrc	)
 				/* Attributes of the current position.	*/
 				/****************************************/
     docInitTextFrameProperties( &(rrc->rrcTextFrameProperties) );
-    docInitParagraphProperties( &(rrc->rrcParagraphProperties) );
     docInitCellProperties( &(rrc->rrcCellProperties) );
     docInitRowProperties( &(rrc->rrcRowProperties) );
-    docInitSectionProperties( &(rrc->rrcSectionProperties) );
+
+    rrc->rrcExternalItemKind= DOCinBODY;
 
     docInitDocumentList( &(rrc->rrcDocumentList) );
+    docInitListOverride( &(rrc->rrcListOverride) );
+    docInitDocumentListLevel( &(rrc->rrcDocumentListLevel) );
+    docInitListOverrideLevel( &(rrc->rrcListOverrideLevel) );
 
     docInitBorderProperties( &(rrc->rrcBorderProperties) );
     /*docInitShapeProperties( &(rrc->rrcShapeProperties) );*/
@@ -92,8 +99,8 @@ void docRtfInitReadingContext(	RtfReadingContext *	rrc	)
 				/****************************************/
 				/*  Document properties.		*/
 				/****************************************/
-    docInitFont( &(rrc->rrcCurrentFont) );
-    docInitStyle( &(rrc->rrcStyle) );
+    docInitDocumentFont( &(rrc->rrcCurrentFont) );
+    docInitDocumentStyle( &(rrc->rrcDocumentStyle) );
 				/****************************************/
 				/*  For reading the font table.		*/
 				/****************************************/
@@ -107,7 +114,6 @@ void docRtfInitReadingContext(	RtfReadingContext *	rrc	)
 				/*  For reading 'fields'.		*/
 				/****************************************/
     rrc->rrcJustAfterPntext= 0;
-    rrc->rrcPageAfterPntext= 0;
 				/****************************************/
 				/*  For coping with the way word saves	*/
 				/*  {\pntext ... }			*/
@@ -117,19 +123,26 @@ void docRtfInitReadingContext(	RtfReadingContext *	rrc	)
 	{ rrc->rrcInputMapping[i]= i;	}
     }
 
-void docRtfCleanReadingContext(	RtfReadingContext *	rrc	)
+void docRtfCleanReadingContext(	RtfReadingContext *	rrc )
     {
-    docCleanParagraphProperties( &(rrc->rrcParagraphProperties) );
+    while( rrc->rrcState )
+	{
+	XDEB(rrc->rrcState);
+	docRtfPopReadingState( rrc );
+	}
+
     /*docCleanCellProperties( &(rrc->rrcRowProperties) );*/
     docCleanRowProperties( &(rrc->rrcRowProperties) );
-    docCleanSectionProperties( &(rrc->rrcSectionProperties) );
     /*docCleanShapeProperties( &(rrc->rrcShapeProperties) );*/
     docCleanShape( &(rrc->rrcShape) );
-    docCleanFont( &(rrc->rrcCurrentFont) );
+    docCleanDocumentFont( &(rrc->rrcCurrentFont) );
 
     docCleanDocumentList( &(rrc->rrcDocumentList) );
+    docCleanListOverride( &(rrc->rrcListOverride) );
+    docCleanDocumentListLevel( &(rrc->rrcDocumentListLevel) );
+    docCleanListOverrideLevel( &(rrc->rrcListOverrideLevel) );
 
-    docCleanStyle( &(rrc->rrcStyle) );
+    docCleanDocumentStyle( &(rrc->rrcDocumentStyle) );
 
     if  ( rrc->rrcParagraphNumbers )
 	{ free( rrc->rrcParagraphNumbers );	}
@@ -150,41 +163,54 @@ void docRtfCleanReadingContext(	RtfReadingContext *	rrc	)
 int docRtfCopyReadingContext(	RtfReadingContext *		to,
 				const RtfReadingContext *	from )
     {
-    ParagraphProperties		ppCopy;
     RowProperties		rpCopy;
-    SectionProperties		spCopy;
     DocumentStyle		dsCopy;
 
-    docInitParagraphProperties( &ppCopy );
-    if  ( docCopyParagraphProperties( &ppCopy,
-					    &(from->rrcParagraphProperties) ) )
-	{ LDEB(1); return -1;	}
+    PropertyMask		ppChgMask;
+    PropertyMask		ppUpdMask;
+
+    PROPmaskCLEAR( &ppChgMask );
+
+    PROPmaskCLEAR( &ppUpdMask );
+    PROPmaskFILL( &ppUpdMask, PPprop_COUNT );
 
     docInitRowProperties( &rpCopy );
-    if  ( docCopyRowProperties( &rpCopy, &(from->rrcRowProperties) ) )
+    if  ( docCopyRowProperties( &rpCopy,
+			    &(from->rrcRowProperties), (const int *)0 ) )
 	{ LDEB(1); return -1;	}
 
-    docInitSectionProperties( &spCopy );
-    if  ( docCopySectionProperties( &spCopy, &(from->rrcSectionProperties) ) )
+    docInitDocumentStyle( &dsCopy );
+    if  ( docCopyStyle( &dsCopy, &(from->rrcDocumentStyle) ) )
 	{ LDEB(1); return -1;	}
 
-    docInitStyle( &dsCopy );
-    if  ( docCopyStyle( &dsCopy, &(from->rrcStyle) ) )
-	{ LDEB(1); return -1;	}
+    docCleanDocumentList( &(to->rrcDocumentList) );
+    docCleanListOverride( &(to->rrcListOverride) );
+    docCleanDocumentListLevel( &(to->rrcDocumentListLevel) );
+    docCleanListOverrideLevel( &(to->rrcListOverrideLevel) );
 
     *to= *from;
 
     to->rrcParagraphNumbers= (ParagraphNumber *)0;
     to->rrcParagraphNumberCount= 0;
 
-    to->rrcParagraphProperties= ppCopy;
     to->rrcRowProperties= rpCopy;
-    to->rrcSectionProperties= spCopy;
-    to->rrcStyle= dsCopy;
+    to->rrcDocumentStyle= dsCopy;
 
     to->rrcTopBookmark= from->rrcTopBookmark;
 
     docInitDocumentList( &(to->rrcDocumentList) );
+    docInitListOverride( &(to->rrcListOverride) );
+    docInitDocumentListLevel( &(to->rrcDocumentListLevel) );
+    docInitListOverrideLevel( &(to->rrcListOverrideLevel) );
+
+    docCopyDocumentList( &(to->rrcDocumentList),
+					    &(from->rrcDocumentList) );
+    docCopyListOverride( &(to->rrcListOverride),
+					    &(from->rrcListOverride) );
+    docCopyDocumentListLevel( &(to->rrcDocumentListLevel),
+					    &(from->rrcDocumentListLevel) );
+    docCopyListOverrideLevel( &(to->rrcListOverrideLevel),
+					    &(from->rrcListOverrideLevel) );
 
     return 0;
     }
@@ -199,7 +225,6 @@ void docRtfCopyReadingContextBack(	RtfReadingContext *	to,
     to->rrcCurrentLine= from->rrcCurrentLine;
     to->rrcFieldNumber= from->rrcFieldNumber;
     to->rrcJustAfterPntext= from->rrcJustAfterPntext;
-    to->rrcPageAfterPntext= from->rrcPageAfterPntext;
 
     memcpy( to->rrcBookmarkName, from->rrcBookmarkName, DOCmaxBOOKMARK+1 );
     to->rrcBookmarkSize= from->rrcBookmarkSize;
@@ -207,13 +232,82 @@ void docRtfCopyReadingContextBack(	RtfReadingContext *	to,
     from->rrcTopBookmark= (RtfBookmarkLevel *)0;
 
     to->rrcCurrentFont= from->rrcCurrentFont;
-    docInitFont( &(from->rrcCurrentFont) );
+    docInitDocumentFont( &(from->rrcCurrentFont) );
 
     to->rrcParagraphNumber= from->rrcParagraphNumber;
 
+    to->rrcExternalItemKind= from->rrcExternalItemKind;
+
     to->rrcTm= from->rrcTm;
 
-    docInitDocumentList( &(to->rrcDocumentList) );
+    docCopyDocumentList( &(to->rrcDocumentList),
+					    &(from->rrcDocumentList) );
+    docCopyListOverride( &(to->rrcListOverride),
+					    &(from->rrcListOverride) );
+    docCopyDocumentListLevel( &(to->rrcDocumentListLevel),
+					    &(from->rrcDocumentListLevel) );
+    docCopyListOverrideLevel( &(to->rrcListOverrideLevel),
+					    &(from->rrcListOverrideLevel) );
+    }
+
+void docRtfPushReadingState(	RtfReadingContext *	rrc,
+				RtfReadingState *	rrs )
+    {
+    utilInitTextAttribute( &(rrs->rrsTextAttribute) );
+    rrs->rrsTextAttribute.taFontSizeHalfPoints= 24;
+
+    docInitParagraphProperties( &(rrs->rrsParagraphProperties) );
+    docInitSectionProperties( &(rrs->rrsSectionProperties) );
+
+    utilInitMemoryBuffer( &(rrs->rrsSavedText) );
+
+    rrs->rrsBytesPerUnicode= 1;
+    rrs->rrsUnicodeBytesToSkip= 0;
+
+    if  ( rrc->rrcState )
+	{
+	const RtfReadingState *	above= rrc->rrcState;
+
+	rrs->rrsBytesPerUnicode= above->rrsBytesPerUnicode;
+	rrs->rrsUnicodeBytesToSkip= 0;
+
+	/*
+	utilCopyTextAttribute( &(rrs->rrsTextAttribute),
+					&(above->rrsTextAttribute) );
+	*/
+	rrs->rrsTextAttribute= above->rrsTextAttribute;
+
+	docCopyParagraphProperties( &(rrs->rrsParagraphProperties),
+				    &(above->rrsParagraphProperties) );
+	docCopySectionProperties( &(rrs->rrsSectionProperties),
+				    &(above->rrsSectionProperties) );
+	}
+
+    rrs->rrsPrev= rrc->rrcState;
+    rrc->rrcState= rrs;
+
+    return;
+    }
+
+void docRtfPopReadingState(	RtfReadingContext *	rrc )
+    {
+    RtfReadingState *	rrs= rrc->rrcState;
+
+    if  ( ! rrs )
+	{ XDEB(rrs); return;	}
+
+    /*
+    utilCleanTextAttribute( &(rrs->rrsTextAttribute) );
+    */
+    docCleanParagraphProperties( &(rrs->rrsParagraphProperties) );
+    docCleanSectionProperties( &(rrs->rrsSectionProperties) );
+
+    utilCleanMemoryBuffer( &(rrs->rrsSavedText) );
+
+    /**/
+    rrc->rrcState= rrs->rrsPrev;
+
+    return;
     }
 
 /************************************************************************/
@@ -229,10 +323,11 @@ static int docRtfReadControlWord(	SimpleInputStream *	sis,
 					int *			pGotArg,
 					int *			pArg )
     {
-    int		c;
-    int		len= 0;
-    int		arg;
-    int		sign= 1;
+    RtfReadingState *	rrs= rrc->rrcState;
+
+    int			c;
+    int			len= 0;
+    int			sign= 1;
 
     c= sioInGetCharacter( sis );
     while( c == '\n' || c == '\r' )
@@ -331,13 +426,33 @@ static int docRtfReadControlWord(	SimpleInputStream *	sis,
     *pGotArg= isdigit( c );
     if  ( *pGotArg )
 	{
-	arg= c- '0'; c= sioInGetCharacter( sis );
+	int arg= c- '0';
+
+	c= sioInGetCharacter( sis );
+
 	while( isdigit( c ) )
 	    { arg= 10* arg+ c- '0'; c= sioInGetCharacter( sis ); }
 
 	*pArg= sign* arg;
 	}
 
+    /* HACK */
+    if  ( *pGotArg && ! strcmp( controlWord, "u" ) )
+	{
+	if  ( *pArg >= 0 && *pArg <= 255 )
+	    {
+	    *pC= *pArg;
+
+	    rrs->rrsUnicodeBytesToSkip= rrs->rrsBytesPerUnicode;
+	    }
+	else{ *pC= ISO1_currency;	}
+
+	if  ( c != ' ' )
+	    { sioInUngetLastRead( sis );	}
+
+	return 2;
+	}
+
     if  ( c != ' ' )
 	{ sioInUngetLastRead( sis );	}
 
@@ -354,21 +469,8 @@ const RtfControlWord * docRtfFindWord(	const char *		controlWord,
 					const RtfControlWord *	contolWords,
 					int			listOnly )
     {
-    if  ( ! contolWords )
-	{ return (RtfControlWord *)0;	}
-
-    while( contolWords->rcwWord )
+    if  ( contolWords )
 	{
-	if  ( ! strcmp( contolWords->rcwWord, controlWord ) )
-	    { return contolWords;	}
-
-	contolWords++;
-	}
-
-    if  ( ! listOnly )
-	{
-	contolWords= docRtfPropertyWords;
-
 	while( contolWords->rcwWord )
 	    {
 	    if  ( ! strcmp( contolWords->rcwWord, controlWord ) )
@@ -378,6 +480,13 @@ const RtfControlWord * docRtfFindWord(	const char *		controlWord,
 	    }
 	}
 
+    if  ( ! listOnly )
+	{
+	contolWords= docRtfFindPropertyWord( controlWord );
+
+	return contolWords;
+	}
+
     return (RtfControlWord *)0;
     }
 
@@ -396,24 +505,6 @@ int docRtfApplyControlWord(	SimpleInputStream *	sis,
     return 0;
     }
 
-/************************************************************************/
-/*									*/
-/*  Just ignore a control word, a text particule.			*/
-/*									*/
-/************************************************************************/
-
-int docRtfIgnoreWord(	SimpleInputStream *	sis,
-			const RtfControlWord *	rcw,
-			int			arg,
-			RtfReadingContext *	rrc	)
-    { return 0;	}
-
-int docRtfIgnoreParticule(	RtfReadingContext *	rrc,
-				const unsigned char *	text,
-				int			len	)
-    { /* SDEB(text); */ return 0;	}
-
-
 /************************************************************************/
 /*									*/
 /*  Find out what to do.						*/
@@ -433,8 +524,10 @@ int docRtfFindControl(		SimpleInputStream *	sis,
 				int *			pGotArg,
 				int *			pArg		)
     {
-    int				c;
-    int				res;
+    RtfReadingState *	rrs= rrc->rrcState;
+
+    int			c;
+    int			res;
 
     if  ( rrc->rrcCharacterAhead != EOF )
 	{
@@ -463,27 +556,34 @@ int docRtfFindControl(		SimpleInputStream *	sis,
 		    switch( c )
 			{
 			case '\\': case '{': case '}':
-			    goto defaultCase;
+			    *pC= c; return RTFfiCHAR;
+
 			case '~':
 			    c= ISO1_nobreakspace;
-			    goto defaultCase;
+			    *pC= c; return RTFfiCHAR;
+
 			case '\'':
 			    if  ( res == 1 )
 				{
-				char	b[3];
+				char		b[3];
+				unsigned int	uc;
 
 				b[0]= sioInGetCharacter( sis );
 				b[1]= sioInGetCharacter( sis );
 				b[2]= '\0';
 
-				sscanf( b, "%x", &c );
+				sscanf( b, "%x", &uc ); c= uc;
 				}
 			    goto defaultCase;
+
 			default:
-			    goto defaultCase;
+			    if  ( res == 2 )
+				{ *pC= c; return RTFfiCHAR;	}
+			    else{ goto defaultCase;		}
 			}
 		    }
 		return RTFfiWORD;
+
 	    case '{':
 		c= sioInGetCharacter( sis );
 		while( c == '\n' || c == '\r' )
@@ -492,6 +592,7 @@ int docRtfFindControl(		SimpleInputStream *	sis,
 			{ rrc->rrcCurrentLine++;	}
 		    c= sioInGetCharacter( sis );
 		    }
+
 		if  ( c == '\\' )
 		    {
 		    sioInUngetLastRead( sis );
@@ -513,16 +614,23 @@ int docRtfFindControl(		SimpleInputStream *	sis,
 
 			if  ( c=='\'' )
 			    {
-			    char	b[3];
+			    char		b[3];
+			    unsigned int	uc;
 
 			    b[0]= sioInGetCharacter( sis );
 			    b[1]= sioInGetCharacter( sis );
 			    b[2]= '\0';
 
-			    sscanf( b, "%x", &c );
+			    sscanf( b, "%x", &uc ); c= uc;
 			    }
 
-			rrc->rrcCharacterAhead= c;
+			if  ( rrs->rrsUnicodeBytesToSkip > 0 )
+			    { rrs->rrsUnicodeBytesToSkip= 0;	}
+
+			if  ( res != 2 )
+			    { rrc->rrcCharacterAhead= rrc->rrcInputMapping[c]; }
+			else{ rrc->rrcCharacterAhead= c; }
+
 			return RTFfiTEXTGROUP;
 			}
 
@@ -531,17 +639,25 @@ int docRtfFindControl(		SimpleInputStream *	sis,
 
 		sioInUngetLastRead( sis );
 		return RTFfiTEXTGROUP;
+
 	    case '\n':
 		rrc->rrcCurrentLine++;
 		/*FALLTHROUGH*/
 	    case '\r':
 		continue;
+
 	    case '}':
 		*pC= c; return RTFfiCLOSE;
+
 	    case '\t':
 		*pC= c; return RTFfiTAB;
+
 	    default: defaultCase:
-		*pC= c; return RTFfiCHAR;
+
+		if  ( rrs->rrsUnicodeBytesToSkip > 0 )
+		    { rrs->rrsUnicodeBytesToSkip--; continue;	}
+
+		*pC= rrc->rrcInputMapping[c]; return RTFfiCHAR;
 	    }
 	}
     }
@@ -561,8 +677,9 @@ int docRtfFindControl(		SimpleInputStream *	sis,
 
 int docRtfAdjustLevel(	RtfReadingContext *	rrc,
 			int			toLevel,
-			int			textLevel	)
+			int			textLevel )
     {
+    RtfReadingState *	rrs= rrc->rrcState;
     BufferItem *	bi= rrc->rrcBi;
 
     if  ( ! bi )
@@ -573,13 +690,20 @@ int docRtfAdjustLevel(	RtfReadingContext *	rrc,
 	  bi->biLevel == DOClevPARA	&&
 	  bi->biParaParticuleCount == 0	)
 	{
+	const int	part= 0;
+	const int	stroff= 0;
+	const int	objectNumber= -1;
+
 	if  ( docInflateTextString( bi, 0 ) )
 	    { LDEB(bi->biParaStrlen); return -1;	}
 	bi->biParaString[bi->biParaStrlen]= '\0';
 
-	if  ( ! docInsertTextParticule( bi, -1, 0, 0,
-				DOCkindTEXT, rrc->rrcCurrentTextAttribute ) )
-	    { LDEB(bi->biParaParticuleCount); return -1;	}
+	if  ( docInsertAdminParticule( rrc->rrcBd, bi,
+			    stroff, part, objectNumber,
+			    &(rrs->rrsTextAttribute), DOCkindTEXT ) )
+	    { LDEB(1); return -1;	}
+
+	rrc->rrcAfterNoteref= 0;
 	}
 
     while( toLevel < rrc->rrcLevel )
@@ -591,15 +715,31 @@ int docRtfAdjustLevel(	RtfReadingContext *	rrc,
 
 	if  ( bi->biLevel == DOClevPARA )
 	    {
+	    PropertyMask	ppChgMask;
+	    PropertyMask	ppUpdMask;
+
 	    if  ( docRtfFlushBookmarks( -1, bi, rrc ) )
 		{ LDEB(1); return -1;	}
 
+	    PROPmaskCLEAR( &ppChgMask );
+
+	    PROPmaskCLEAR( &ppUpdMask );
+	    PROPmaskFILL( &ppUpdMask, PPprop_COUNT );
+
 	    /*  3  */
-	    if  ( docCopyParagraphProperties( &(bi->biParaProperties),
-					     &(rrc->rrcParagraphProperties) ) )
+	    if  ( docUpdParaProperties( &ppChgMask, &(bi->biParaProperties),
+				&ppUpdMask, &(rrs->rrsParagraphProperties),
+				(const int *)0 ) )
 		{ LDEB(1); return -1;	}
 	    }
 
+	if  ( bi->biLevel == DOClevSECT )
+	    {
+	    int		changed= 0;
+
+	    docRenumberNotes( &changed, rrc->rrcBd );
+	    }
+
 	bi= bi->biParent; rrc->rrcLevel--;
 	}
 
@@ -620,17 +760,28 @@ int docRtfAdjustLevel(	RtfReadingContext *	rrc,
 	    return -1;
 	    }
 
+	rrc->rrcAfterNoteref= 0;
+
 	bi= fresh;
 	
 	rrc->rrcLevel++;
 
 	if  ( rrc->rrcLevel == DOClevPARA )
 	    {
+	    PropertyMask	ppChgMask;
+	    PropertyMask	ppUpdMask;
+
 	    BufferItem *	cellBi= bi->biParent;
 	    BufferItem *	rowBi= cellBi->biParent;
 
-	    if  ( docCopyParagraphProperties( &(bi->biParaProperties),
-					 &(rrc->rrcParagraphProperties) ) )
+	    PROPmaskCLEAR( &ppChgMask );
+
+	    PROPmaskCLEAR( &ppUpdMask );
+	    PROPmaskFILL( &ppUpdMask, PPprop_COUNT );
+
+	    if  ( docUpdParaProperties( &ppChgMask, &(bi->biParaProperties),
+				&ppUpdMask, &(rrs->rrsParagraphProperties),
+				(const int *)0 ) )
 		{ LDEB(1); return -1;	}
 
 	    /*  5  */
@@ -647,7 +798,7 @@ int docRtfAdjustLevel(	RtfReadingContext *	rrc,
 		    if  ( ! rowBi->biRowHasTableParagraphs )
 			{
 			if  ( docCopyRowProperties( &(rowBi->biRowProperties),
-						 &(rrc->rrcRowProperties) ) )
+				 &(rrc->rrcRowProperties), (const int *)0 ) )
 			    { LDEB(1); return -1;	}
 
 			rowBi->biRowHasTableParagraphs= 1;
@@ -657,22 +808,26 @@ int docRtfAdjustLevel(	RtfReadingContext *	rrc,
 		    if  ( ! rowBi->biRowHasTableParagraphs )
 			{
 			BufferItem *	insBi;
+			BufferItem *	aftBi;
+
 			if  ( cellBi->biChildCount > 1 )
 			    {
-			    if  ( docSplitGroupItem( rrc->rrcBd, &insBi, cellBi,
-						    bi->biNumberInParent ) )
+			    if  ( docSplitGroupItem(
+					rrc->rrcBd, &insBi, &aftBi, cellBi,
+					bi->biNumberInParent, DOClevCELL ) )
 				{ LDEB(1); return -1;	}
 			    }
 
 			if  ( rowBi->biChildCount > 1 )
 			    {
-			    if  ( docSplitGroupItem( rrc->rrcBd, &insBi, rowBi,
-						cellBi->biNumberInParent ) )
+			    if  ( docSplitGroupItem(
+					rrc->rrcBd, &insBi, &aftBi, rowBi,
+					cellBi->biNumberInParent, DOClevROW ) )
 				{ LDEB(1); return -1;	}
 			    }
 
 			if  ( docCopyRowProperties( &(rowBi->biRowProperties),
-						 &(rrc->rrcRowProperties) ) )
+				 &(rrc->rrcRowProperties), (const int *)0 ) )
 			    { LDEB(1); return -1;	}
 
 			rowBi->biRowHasTableParagraphs= 1;
@@ -688,14 +843,14 @@ int docRtfAdjustLevel(	RtfReadingContext *	rrc,
 	if  ( rrc->rrcLevel == DOClevROW )
 	    {
 	    if  ( docCopyRowProperties( &(bi->biRowProperties),
-					 &(rrc->rrcRowProperties) ) )
+				 &(rrc->rrcRowProperties), (const int *)0 ) )
 		{ LDEB(1); return -1;	}
 	    }
 
 	if  ( rrc->rrcLevel == DOClevSECT )
 	    {
 	    if  ( docCopySectionProperties( &(bi->biSectProperties),
-					     &(rrc->rrcSectionProperties) ) )
+					     &(rrs->rrsSectionProperties) ) )
 		{ LDEB(1); return -1;	}
 
 	    if  ( rrc->rrcParagraphNumberCount > 0 )
@@ -769,12 +924,15 @@ static int docRtfReadText(	SimpleInputStream *	sis,
 
     for (;;)
 	{
+	int		map= 1;
+
 	c= sioInGetCharacter( sis );
 
 	switch( c )
 	    {
 	    case EOF:
 		LDEB(c); return -1;
+
 	    case '}':
 		if  ( len > 0 )
 		    {
@@ -782,6 +940,7 @@ static int docRtfReadText(	SimpleInputStream *	sis,
 			{ LDEB(len); return -1;	}
 		    }
 		return RTFfiCLOSE;
+
 	    case '{':
 		sioInUngetLastRead( sis );
 		res= docRtfFindControl( sis, rrc, &c,
@@ -791,11 +950,14 @@ static int docRtfReadText(	SimpleInputStream *	sis,
 		if  ( (*addParticule)( rrc, collectedText, len ) )
 		    { LDEB(len); return -1;	}
 		*pC= c; return res;
+
 	    case '\n':
 		rrc->rrcCurrentLine++;
 		/*FALLTHROUGH*/
+
 	    case '\r' :
 		continue;
+
 	    case '\\':
 		sioInUngetLastRead( sis );
 		res= docRtfFindControl( sis, rrc, &c,
@@ -810,7 +972,10 @@ static int docRtfReadText(	SimpleInputStream *	sis,
 
 		    return res;
 		    }
+
+		map= 0;
 		/*FALLTHROUGH*/
+
 	    default:
 		if  ( len >= collectedSize )
 		    {
@@ -826,9 +991,12 @@ static int docRtfReadText(	SimpleInputStream *	sis,
 		    collectedText= fresh;
 		    collectedSize= size;
 		    }
-		collectedText[len++]= c;
+		if  ( map )
+		    { collectedText[len++]= rrc->rrcInputMapping[c];	}
+		else{ collectedText[len++]= c;				}
 		collectedText[len  ]= '\0';
 		break;
+
 	    case '\t':
 		if  ( (*addParticule)( rrc, collectedText, len ) )
 		    { LDEB(len); return -1;	}
@@ -856,7 +1024,8 @@ int docRtfSkipGroup(	SimpleInputStream *	sis,
 
     if  ( docRtfReadGroup( sis, rcw->rcwLevel,
 		(RtfControlWord *)0, 0, 0, rrc,
-		docRtfEmptyTable, docRtfEmptyTable, docRtfIgnoreParticule ) )
+		docRtfEmptyTable, docRtfEmptyTable,
+		docRtfIgnoreText, (RtfCommitGroup)0 ) )
 	{ SDEB(rcw->rcwWord); return -1;	}
 
     rrc->rrcComplainUnknown= complainUnknown;
@@ -872,41 +1041,39 @@ int docRtfSkipGroup(	SimpleInputStream *	sis,
 
 int docRtfReadGroup(	SimpleInputStream *	sis,
 			int			textLevel,
-
 			const RtfControlWord *	applyFirst,
 			int			gotArg,
 			int			arg,
-
-			RtfReadingContext *	externRrc,
-
+			RtfReadingContext *	rrc,
 			const RtfControlWord *	contolWords,
 			const RtfControlWord *	groupWords,
-			RtfAddTextParticule	addParticule	)
+			RtfAddTextParticule	addParticule,
+			RtfCommitGroup		commitGroup )
     {
     int				rval;
-    RtfReadingContext		internRrc;
+    RtfReadingState		internRrs;
 
-    docRtfInitReadingContext( &internRrc );
-    docRtfCopyReadingContext( &internRrc, externRrc );
+    docRtfPushReadingState( rrc, &internRrs );
 
-    externRrc->rrcCharacterAhead= EOF;
+    rrc->rrcCharacterAhead= EOF;
 
     if  ( applyFirst )
 	{
-	rval= docRtfApplyControlWord( sis, applyFirst,
-						    gotArg, arg, &internRrc );
+	rval= docRtfApplyControlWord( sis, applyFirst, gotArg, arg, rrc );
 	if  ( rval )
 	    { SDEB(applyFirst->rcwWord);	}
 	}
 
-    rval= docRtfConsumeGroup( sis, textLevel, &internRrc,
+    rval= docRtfConsumeGroup( sis, textLevel, rrc,
 				    contolWords, groupWords, addParticule );
 
     if  ( rval )
 	{ LDEB(rval);	}
 
-    docRtfCopyReadingContextBack( externRrc, &internRrc );
-    docRtfCleanReadingContext( &internRrc );
+    if  ( commitGroup && (*commitGroup)( rrc ) )
+	{ LDEB(1); rval= -1;	}
+
+    docRtfPopReadingState( rrc );
 
     return rval;
     }
@@ -927,7 +1094,7 @@ int docRtfReadUnknownGroup(	SimpleInputStream *	sis,
     if  ( docRtfReadGroup( sis, rrc->rrcLevel,
 					(RtfControlWord *)0, 0, 0, rrc,
 					docRtfEmptyTable, docRtfEmptyTable,
-					docRtfIgnoreParticule ) )
+					docRtfIgnoreText, (RtfCommitGroup)0 ) )
 	{ LDEB(1); return -1;	}
 
     rrc->rrcComplainUnknown= complainUnknown;
@@ -959,7 +1126,7 @@ int docRtfReadWordGroup(	SimpleInputStream *	sis,
 	if  ( docRtfReadGroup( sis, textLevel,
 				rcw, gotArg, arg, rrc,
 				contolWords, groupWords,
-				addParticule ) )
+				addParticule, (RtfCommitGroup)0 ) )
 	    { SDEB(rcw->rcwWord); return -1;	}
 	}
     else{
@@ -1013,9 +1180,11 @@ int docRtfConsumeGroup(	SimpleInputStream *	sis,
 		if  ( res < 0 )
 		    { SLDEB(controlWord,res); return -1;	}
 		break;
+
 	    case RTFfiTAB:
 		strcpy( controlWord, "tab" );
 		/*FALLTHROUGH*/
+
 	    case RTFfiWORD:
 		rcw= docRtfFindWord( controlWord, contolWords, 0 );
 		if  ( ! rcw )
@@ -1113,8 +1282,12 @@ int docRtfConsumeGroup(	SimpleInputStream *	sis,
 		if  ( rcw )
 		    { goto groupFound; }
 
+		rrc->rrcInIgnoredGroup++;
+
 		if  ( docRtfReadUnknownGroup( sis, rrc ) )
-		    { LDEB(1); return -1;	}
+		    { LDEB(1); rrc->rrcInIgnoredGroup--; return -1;	}
+
+		rrc->rrcInIgnoredGroup--;
 
 		res= docRtfFindControl( sis, rrc, &c,
 					    controlWord, &gotArg, &arg );
@@ -1126,7 +1299,7 @@ int docRtfConsumeGroup(	SimpleInputStream *	sis,
 		if  ( docRtfReadGroup( sis, textLevel,
 					(RtfControlWord *)0, 0, 0, rrc,
 					contolWords, groupWords,
-					addParticule ) )
+					addParticule, (RtfCommitGroup)0 ) )
 		    { LDEB(1); return -1;	}
 
 		res= docRtfFindControl( sis, rrc, &c,
@@ -1141,12 +1314,97 @@ int docRtfConsumeGroup(	SimpleInputStream *	sis,
 	}
     }
 
+/************************************************************************/
+/*									*/
+/*  Just ignore a control word.						*/
+/*									*/
+/************************************************************************/
+
+int docRtfIgnoreWord(	SimpleInputStream *	sis,
+			const RtfControlWord *	rcw,
+			int			arg,
+			RtfReadingContext *	rrc )
+    { return 0;	}
+
+/************************************************************************/
+/*									*/
+/*  Handle text..							*/
+/*									*/
+/*  1)  Ignore it.							*/
+/*  2)  Refuse it.							*/
+/*  3)  Save it for later use.						*/
+/*									*/
+/************************************************************************/
+
+/*  1  */
 int docRtfIgnoreText(	RtfReadingContext *	rrc,
 			const unsigned char *	text,
-			int			len	)
+			int			len )
     { return 0; }
 
+/*  2  */
 int docRtfRefuseText(	RtfReadingContext *	rrc,
 			const unsigned char *	text,
-			int			len	)
+			int			len )
     { LDEB(1); return -1; }
+
+/*  3  */
+int docRtfSaveText(	RtfReadingContext *	rrc,
+			const unsigned char *	text,
+			int			len )
+    {
+    RtfReadingState *	rrs= rrc->rrcState;
+
+    if  ( utilAddToMemoryBuffer( &(rrs->rrsSavedText), text, len ) )
+	{ LDEB(len); return -1;	}
+
+    return 0;
+    }
+
+int docRtfRememberText(	char **			pTarget,
+			int *			pSize,
+			RtfReadingContext *	rrc,
+			int			removeSemicolon )
+    {
+    return docRtfRememberUText( (unsigned char **)pTarget, pSize,
+						rrc, removeSemicolon );
+    }
+
+int docRtfRememberUText(	unsigned char **	pTarget,
+				int *			pSize,
+				RtfReadingContext *	rrc,
+				int			removeSemicolon )
+    {
+    RtfReadingState *	rrs= rrc->rrcState;
+
+    unsigned char *	fresh;
+    int			size;
+
+    if  ( rrs->rrsSavedText.mbSize == 0 )
+	{
+	*pSize= 0;
+	return 0;
+	}
+
+    size= rrs->rrsSavedText.mbSize;
+    fresh= realloc( *pTarget, size+ 1 );
+    if  ( ! fresh )
+	{ LXDEB(size,fresh); return -1;	}
+
+    memcpy( fresh, rrs->rrsSavedText.mbBytes, size );
+    fresh[size]= '\0';
+
+    if  ( removeSemicolon		&&
+	  size > 0			&&
+	  fresh[size- 1] == ';'		)
+	{
+	size--;
+	fresh[size]= '\0';
+	}
+
+    utilEmptyMemoryBuffer( &(rrs->rrsSavedText) );
+
+    *pTarget= fresh;
+    *pSize= size;
+    return 0;
+    }
