@@ -4,7 +4,7 @@
 /*									*/
 /************************************************************************/
 
-#   include	"config.h"
+#   include	"tedConfig.h"
 
 #   include	<stdlib.h>
 #   include	<string.h>
@@ -25,12 +25,12 @@
 /*									*/
 /************************************************************************/
 
-static int docRtfSaveItem(	SimpleOutputStream *	sos,
-				int *			pCol,
-				const BufferItem *	bi,
-				const BufferDocument *	bd,
-				const BufferSelection *	bs,
-				RtfWritingContext *	rwc );
+static int docRtfSaveItem(	SimpleOutputStream *		sos,
+				int *				pCol,
+				const BufferItem *		bi,
+				const BufferDocument *		bd,
+				const DocumentSelection *	ds,
+				RtfWritingContext *		rwc );
 
 /************************************************************************/
 /*									*/
@@ -44,22 +44,25 @@ static void docRtfParaSaveProperties( SimpleOutputStream *	sos,
 				int				fromDefault,
 				int				saveIntbl,
 				int *				pPropertyChange,
+				int *				pTabsSaved,
 				int *				pCol,
 				const ParagraphProperties *	newPP,
 				const ParagraphProperties *	prevPP )
     {
     ParagraphProperties		ppp;
-    unsigned int		updMask;
+    PropertyMask		updMask;
 
     PROPmaskCLEAR( &updMask );
 
     if  ( ! fromDefault )
 	{
+	PROPmaskCLEAR( &updMask );
 	PROPmaskFILL( &updMask, PPprop_COUNT );
+	PROPmaskUNSET( &updMask, PPpropIN_TABLE );
 
-	updMask= docParaPropertyDifference( newPP, prevPP, updMask );
+	docParaPropertyDifference( &updMask, newPP, prevPP, &updMask );
 
-	if  ( ! updMask )
+	if  ( PROPmaskISEMPTY( &updMask ) )
 	    { return;	}
 	}
 
@@ -74,12 +77,17 @@ static void docRtfParaSaveProperties( SimpleOutputStream *	sos,
 
     PROPmaskCLEAR( &updMask );
     PROPmaskFILL( &updMask, PPprop_COUNT );
+    PROPmaskUNSET( &updMask, PPpropIN_TABLE );
 
-    updMask= docParaPropertyDifference( newPP, &ppp, updMask );
+    docParaPropertyDifference( &updMask, newPP, &ppp, &updMask );
 
     docCleanParagraphProperties( &ppp );
 
-    docRtfSaveParagraphProperties( sos, pCol, updMask, newPP );
+    docRtfSaveParagraphProperties( sos, pCol, &updMask, newPP );
+
+    if  ( PROPmaskISSET( &updMask, PPpropTAB_STOPS )	&&
+	  newPP->ppTabStopList.tslTabStopCount > 0	)
+	{ *pTabsSaved= 1;	}
 
     return;
     }
@@ -89,6 +97,7 @@ int docRtfSaveRuler(	SimpleOutputStream *		sos,
     {
     int				col= 0;
     int				propChange= 1;
+    int				tabsSaved= 0;
     int				saveIntbl= 0;
     int				noRestart= 0;
 
@@ -100,7 +109,7 @@ int docRtfSaveRuler(	SimpleOutputStream *		sos,
     docRtfWriteTag( "\\pard", &col, sos );
 
     docRtfParaSaveProperties( sos, noRestart, saveIntbl,
-					&propChange, &col, pp, &refPP );
+				&propChange, &tabsSaved, &col, pp, &refPP );
 
     docRtfWriteDestinationEnd( &col, sos );
 
@@ -122,7 +131,18 @@ static int docRtfSaveObject(	SimpleOutputStream *		sos,
 	{
 	case DOCokPICTWMETAFILE:
 	    docRtfWriteDestinationBegin( "\\pict", pCol, sos );
-	    docRtfWriteTag( "\\wmetafile8", pCol, sos );
+	    docRtfWriteArgTag( "\\wmetafile", pCol, io->ioMapMode, sos );
+
+	    if  ( io->ioMetafileIsBitmap )
+		{
+		docRtfWriteTag( "\\picbmp", pCol, sos );
+
+		if  ( io->ioMetafileBitmapBpp > 0 )
+		    {
+		    docRtfWriteArgTag( "\\picbpp",
+					pCol, io->ioMetafileBitmapBpp, sos );
+		    }
+		}
 
 	    docRtfSavePictureTags( io, pCol, sos );
 
@@ -197,7 +217,10 @@ static int docRtfSaveObject(	SimpleOutputStream *		sos,
 	    if  ( io->ioResultKind == DOCokPICTWMETAFILE )
 		{
 		docRtfWriteDestinationBegin(
-			    "\\result {\\pict\\wmetafile8", pCol, sos );
+			    "\\result {\\pict", pCol, sos );
+
+		docRtfWriteArgTag( "\\wmetafile",
+					    pCol, io->ioResultMapMode, sos );
 
 		docRtfSavePictureTags( io, pCol, sos );
 
@@ -360,164 +383,21 @@ static void docRtfFinishBookmark(	SimpleOutputStream *	sos,
     return;
     }
 
-/************************************************************************/
-/*									*/
-/*  Save Cell properties.						*/
-/*									*/
-/************************************************************************/
-
-static void docRtfSaveCellProperties(	const CellProperties *	cp,
-					int *			pCol,
-					int			shiftLeft,
-					SimpleOutputStream *	sos )
-    {
-    int		didAnything= 0;
-
-    if  ( cp->cpLeftInMergedRow )
-	{ docRtfWriteTag( "\\clmgf", pCol, sos ); didAnything= 1;	}
-    if  ( cp->cpMergedWithLeft )
-	{ docRtfWriteTag( "\\clmrg", pCol, sos ); didAnything= 1;	}
-    if  ( cp->cpTopInMergedColumn )
-	{ docRtfWriteTag( "\\clvmgf", pCol, sos ); didAnything= 1;	}
-    if  ( cp->cpMergedWithAbove )
-	{ docRtfWriteTag( "\\clvmrg", pCol, sos ); didAnything= 1;	}
-
-    docRtfWriteNextLine( pCol, sos );
-
-    if  ( cp->cpTopBorder.bpStyle != DOCbsNONE )
-	{ docRtfSaveBorder( "\\clbrdrt", pCol, &(cp->cpTopBorder), sos ); }
-    if  ( cp->cpLeftBorder.bpStyle != DOCbsNONE )
-	{ docRtfSaveBorder( "\\clbrdrl", pCol, &(cp->cpLeftBorder), sos ); }
-    if  ( cp->cpRightBorder.bpStyle != DOCbsNONE )
-	{ docRtfSaveBorder( "\\clbrdrr", pCol, &(cp->cpRightBorder), sos ); }
-    if  ( cp->cpBottomBorder.bpStyle != DOCbsNONE )
-	{ docRtfSaveBorder( "\\clbrdrb", pCol, &(cp->cpBottomBorder), sos ); }
-
-    switch( cp->cpShadingPattern )
-	{
-	case DOCspSOLID:
-	    break;
-
-	case DOCspBGBDIAG:
-	    docRtfWriteTag( "\\clbgbdiag", pCol, sos );
-	    break;
-	case DOCspBGCROSS:
-	    docRtfWriteTag( "\\clbgcross", pCol, sos );
-	    break;
-	case DOCspBGDCROSS:
-	    docRtfWriteTag( "\\clbgdcross", pCol, sos );
-	    break;
-	case DOCspBGDKBDIAG:
-	    docRtfWriteTag( "\\clbgdkbdiag", pCol, sos );
-	    break;
-	case DOCspBGDKCROSS:
-	    docRtfWriteTag( "\\clbgdkcross", pCol, sos );
-	    break;
-	case DOCspBGDKDCROSS:
-	    docRtfWriteTag( "\\clbgdkdcross", pCol, sos );
-	    break;
-	case DOCspBGDKFDIAG:
-	    docRtfWriteTag( "\\clbgdkfdiag", pCol, sos );
-	    break;
-	case DOCspBGDKHORIZ:
-	    docRtfWriteTag( "\\clbgdkhor", pCol, sos );
-	    break;
-	case DOCspBGDKVERT:
-	    docRtfWriteTag( "\\clbgdkvert", pCol, sos );
-	    break;
-	case DOCspBGFDIAG:
-	    docRtfWriteTag( "\\clbgfdiag", pCol, sos );
-	    break;
-	case DOCspBGHORIZ:
-	    docRtfWriteTag( "\\clbghoriz", pCol, sos );
-	    break;
-	case DOCspBGVERT:
-	    docRtfWriteTag( "\\clbgvert", pCol, sos );
-	    break;
-
-	default:
-	    LDEB(cp->cpShadingPattern);
-	    break;
-	}
-
-    docRtfWriteArgTag( "\\cellx", pCol,
-				cp->cpRightBoundaryTwips- shiftLeft, sos );
-
-    docRtfWriteNextLine( pCol, sos );
-    }
-
-static void docRtfSaveRowProperties(	const RowProperties *	rp,
-					int *			pCol,
-					int			col0,
-					int			col1,
-					SimpleOutputStream *	sos )
-    {
-    const CellProperties *	cp= rp->rpCells;
-
-    int				col;
-    int				shiftLeft= 0;
-
-    docRtfWriteTag( "\\trowd", pCol, sos );
-
-    if  ( rp->rpHalfGapWidthTwips != 0 )
-	{ docRtfWriteArgTag( "\\trgaph", pCol, rp->rpHalfGapWidthTwips, sos ); }
-
-    if  ( rp->rpLeftIndentTwips != 0 )
-	{ docRtfWriteArgTag( "\\trleft", pCol, rp->rpLeftIndentTwips, sos ); }
-
-    if  ( rp->rpHeightTwips != 0 )
-	{ docRtfWriteArgTag( "\\trrh", pCol, rp->rpHeightTwips, sos ); }
-
-    if  ( rp->rpIsTableHeader )
-	{ docRtfWriteTag( "\\trhdr", pCol, sos ); }
-    if  ( rp->rpKeepOnPage )
-	{ docRtfWriteTag( "\\trkeep", pCol, sos ); }
-
-    docRtfWriteNextLine( pCol, sos );
-
-    if  ( rp->rpTopBorder.bpStyle != DOCbsNONE )
-	{ docRtfSaveBorder( "\\trbrdrt", pCol, &(rp->rpTopBorder), sos ); }
-    if  ( rp->rpLeftBorder.bpStyle != DOCbsNONE )
-	{ docRtfSaveBorder( "\\trbrdrl", pCol, &(rp->rpLeftBorder), sos ); }
-    if  ( rp->rpRightBorder.bpStyle != DOCbsNONE )
-	{ docRtfSaveBorder( "\\trbrdrr", pCol, &(rp->rpRightBorder), sos ); }
-    if  ( rp->rpBottomBorder.bpStyle != DOCbsNONE )
-	{ docRtfSaveBorder( "\\trbrdrb", pCol, &(rp->rpBottomBorder), sos ); }
-
-    if  ( rp->rpHasVerticalBorders )
-	{
-	docRtfSaveBorder( "\\trbrdrv", pCol, &(rp->rpVerticalBorder), sos );
-	}
-    if  ( rp->rpHasHorizontalBorders )
-	{
-	docRtfSaveBorder( "\\trbrdrh", pCol, &(rp->rpHorizontalBorder), sos );
-	}
-
-    if  ( col0 > 0 )
-	{ shiftLeft= cp[col0-1].cpRightBoundaryTwips; }
-
-    for ( col= 0; col < rp->rpCellCount; cp++, col++ )
-	{
-	if  ( ( col0 < 0 || col >= col0 )	&&
-	      ( col1 < 0 || col <= col1 )	)
-	    { docRtfSaveCellProperties( cp, pCol, shiftLeft, sos );	}
-	}
-
-    return;
-    }
-
 /************************************************************************/
 /*									*/
 /*  Enter/Leave a table.						*/
 /*									*/
 /************************************************************************/
 
-static int docRtfPushTable(	RtfWritingContext *	rwc,
-				int *			pCol,
-				SimpleOutputStream *	sos,
-				const BufferItem *	rowBi,
-				const BufferSelection *	bs )
+static int docRtfPushTable(	RtfWritingContext *		rwc,
+				int *				pCol,
+				SimpleOutputStream *		sos,
+				const BufferItem *		rowBi,
+				const DocumentSelection *	ds )
     {
+    PropertyMask	ppChgMask;
+    PropertyMask	ppUpdMask;
+
     int			col0= -1;
     int			col1= -1;
 
@@ -528,18 +408,24 @@ static int docRtfPushTable(	RtfWritingContext *	rwc,
 
     rwc->rwcOutsideTableTextAttribute= rwc->rwcTextAttribute;
 
-    if  ( docCopyParagraphProperties(
+    PROPmaskCLEAR( &ppChgMask );
+
+    PROPmaskCLEAR( &ppUpdMask );
+    PROPmaskFILL( &ppUpdMask, PPprop_COUNT );
+
+    if  ( docUpdParaProperties( &ppChgMask,
 			    &(rwc->rwcOutsideTableParagraphProperties),
-			    &(rwc->rwcParagraphProperties) ) )
+			    &ppUpdMask, &(rwc->rwcParagraphProperties),
+			    (const int *)0 ) )
 	{ LDEB(1); return -1;	}
 
-    if  ( bs )
-	{ col0= bs->bsCol0; col1= bs->bsCol1; 	}
+    if  ( ds )
+	{ col0= ds->dsCol0; col1= ds->dsCol1; 	}
 
     docRtfSaveRowProperties( &(rowBi->biRowProperties), pCol, col0, col1, sos );
 
     if  ( docCopyRowProperties( &(rwc->rwcRowProperties),
-					&(rowBi->biRowProperties) ) )
+				&(rowBi->biRowProperties), (const int *)0 ) )
 	{ LDEB(1); return -1;	}
 
     return 0;
@@ -549,6 +435,9 @@ static int docRtfPopTable(	RtfWritingContext *	rwc,
 				int *			pCol,
 				SimpleOutputStream *	sos )
     {
+    PropertyMask	ppChgMask;
+    PropertyMask	ppUpdMask;
+
     docRtfWriteDestinationEnd( pCol, sos );
     docRtfWriteNextLine( pCol, sos );
 
@@ -559,9 +448,15 @@ static int docRtfPopTable(	RtfWritingContext *	rwc,
 
     rwc->rwcTextAttribute= rwc->rwcOutsideTableTextAttribute;
 
-    if  ( docCopyParagraphProperties(
-			    &(rwc->rwcParagraphProperties),
-			    &(rwc->rwcOutsideTableParagraphProperties) ) )
+    PROPmaskCLEAR( &ppChgMask );
+
+    PROPmaskCLEAR( &ppUpdMask );
+    PROPmaskFILL( &ppUpdMask, PPprop_COUNT );
+
+    if  ( docUpdParaProperties( &ppChgMask,
+			    &(rwc->rwcParagraphProperties), &ppUpdMask,
+			    &(rwc->rwcOutsideTableParagraphProperties),
+			    (const int *)0 ) )
 	{ LDEB(1); return -1;	}
 
     return 0;
@@ -573,7 +468,7 @@ static int docRtfPopTable(	RtfWritingContext *	rwc,
 /*									*/
 /************************************************************************/
 
-static int docRtfSaveExternalItem(	SimpleOutputStream *	sos,
+int docRtfSaveExternalItem(		SimpleOutputStream *	sos,
 					int *			pCol,
 					const BufferItem *	bi,
 					const BufferDocument *	bd,
@@ -583,6 +478,7 @@ static int docRtfSaveExternalItem(	SimpleOutputStream *	sos,
     int			i;
 
     int			savedInTable= rwc->rwcInTable;
+    TextAttribute	taOutside;
 
     if  ( ! bi )
 	{ return 0;	}
@@ -590,11 +486,13 @@ static int docRtfSaveExternalItem(	SimpleOutputStream *	sos,
     if  ( bi->biLevel != DOClevSECT )
 	{ SDEB(docLevelStr(bi->biLevel)); return -1;	}
 
+    taOutside= rwc->rwcTextAttribute;
+
     docRtfWriteDestinationBegin( tag, pCol, sos );
 
     docCleanParagraphProperties( &(rwc->rwcParagraphProperties) );
     docInitParagraphProperties( &(rwc->rwcParagraphProperties) );
-    docInitTextAttribute( &(rwc->rwcTextAttribute) );
+    utilInitTextAttribute( &(rwc->rwcTextAttribute) );
 
     rwc->rwcInTable= 0;
 
@@ -605,13 +503,13 @@ static int docRtfSaveExternalItem(	SimpleOutputStream *	sos,
     for ( i= 0; i < bi->biChildCount; i++ )
 	{
 	if  ( docRtfSaveItem( sos, pCol, bi->biChildren[i], bd,
-						(BufferSelection *)0, rwc ) )
+						(DocumentSelection *)0, rwc ) )
 	    { LDEB(i); return -1;	}
 	}
 
     docCleanParagraphProperties( &(rwc->rwcParagraphProperties) );
     docInitParagraphProperties( &(rwc->rwcParagraphProperties) );
-    docInitTextAttribute( &(rwc->rwcTextAttribute) );
+    utilInitTextAttribute( &(rwc->rwcTextAttribute) );
 
     while( rwc->rwcInFldrslt > 0 )
 	{ docRtfFinishFldrslt( sos, pCol, rwc );	}
@@ -622,6 +520,7 @@ static int docRtfSaveExternalItem(	SimpleOutputStream *	sos,
     docRtfWriteDestinationEnd( pCol, sos );
     docRtfWriteNextLine( pCol, sos );
 
+    rwc->rwcTextAttribute= taOutside;
     rwc->rwcInTable= savedInTable;
 
     return 0;
@@ -640,24 +539,30 @@ static int docRtfSaveExternalItem(	SimpleOutputStream *	sos,
 
 static void docRtfSwitchTextAttributes(	SimpleOutputStream *	sos,
 					int *			pCol,
+					const BufferDocument *	bd,
 					const TextParticule *	tp,
 					RtfWritingContext *	rwc )
     {
-    if  ( tp->tpTextAttribute.taFontNumber >= 0 )
+    TextAttribute		ta;
+
+    utilGetTextAttributeByNumber( &ta, &(bd->bdTextAttributeList),
+						tp->tpTextAttributeNumber );
+
+    if  ( ta.taFontNumber >= 0 )
 	{
-	unsigned int	chgMask= TAupdALL;
-	unsigned int	updMask= 0;
+	PropertyMask	chgMask;
+	PropertyMask	updMask;
 
-	PROPmaskUNSET( &chgMask, TApropSHOWASLINK );
+	PROPmaskCLEAR( &chgMask );
+	PROPmaskFILL( &chgMask, TAprop_COUNT );
 
-	docUpdateTextAttribute( &updMask, chgMask,
-					    &(rwc->rwcTextAttribute),
-					    tp->tpTextAttribute );
+	PROPmaskCLEAR( &updMask );
 
-	if  ( updMask )
+	utilUpdateTextAttribute( &updMask, &(rwc->rwcTextAttribute),
+							    &ta, &chgMask );
+	if  ( ! PROPmaskISEMPTY( &updMask ) )
 	    {
-	    docRtfSaveTextAttribute( sos, pCol,
-				    updMask, &(tp->tpTextAttribute) );
+	    docRtfSaveTextAttribute( sos, pCol, &updMask, &ta );
 	    rwc->rwcHasPrecedingTags= 1;
 	    }
 	}
@@ -665,12 +570,12 @@ static void docRtfSwitchTextAttributes(	SimpleOutputStream *	sos,
     return;
     }
 
-static int docRtfSaveParaItem(	SimpleOutputStream *	sos,
-				int *			pCol,
-				const BufferDocument *	bd,
-				const BufferItem *	bi,
-				const BufferSelection *	bs,
-				RtfWritingContext *	rwc )
+static int docRtfSaveParaItem(	SimpleOutputStream *		sos,
+				int *				pCol,
+				const BufferDocument *		bd,
+				const BufferItem *		bi,
+				const DocumentSelection *	ds,
+				RtfWritingContext *		rwc )
     {
     TextParticule *			tp;
     unsigned char *			s;
@@ -682,11 +587,15 @@ static int docRtfSaveParaItem(	SimpleOutputStream *	sos,
     int					saveIntbl;
     int					restartFromDefault= 0;
     int					propChange= 0;
+    int					tabsSaved= 0;
 
     const DocumentField *		df;
     const FieldKindInformation *	fki;
 
-    saveIntbl= ! bs || ! docSelectionInsideCell( bs );
+    PropertyMask			ppChgMask;
+    PropertyMask			ppUpdMask;
+
+    saveIntbl= ! ds || ! docSelectionInsideCell( ds );
 
     if  ( saveIntbl )
 	{
@@ -699,7 +608,7 @@ static int docRtfSaveParaItem(	SimpleOutputStream *	sos,
 	}
 
     docRtfParaSaveProperties( sos, restartFromDefault, saveIntbl,
-					&propChange, pCol,
+					&propChange, &tabsSaved, pCol,
 					&(bi->biParaProperties),
 					&(rwc->rwcParagraphProperties) );
 
@@ -709,7 +618,7 @@ static int docRtfSaveParaItem(	SimpleOutputStream *	sos,
     if  ( restartFromDefault || propChange )
 	{
 	/*  1  */
-	docInitTextAttribute( &(rwc->rwcTextAttribute) );
+	utilInitTextAttribute( &(rwc->rwcTextAttribute) );
 	rwc->rwcTextAttribute.taFontSizeHalfPoints= 24;
 
 	/*  2  */
@@ -720,22 +629,22 @@ static int docRtfSaveParaItem(	SimpleOutputStream *	sos,
     if  ( bi->biParaParticuleCount == 0 )
 	{ LDEB(bi->biParaParticuleCount);	}
 
-    if  ( bs && bs->bsBegin.bpBi == bi )
+    if  ( ds && ds->dsBegin.dpBi == bi )
 	{
-	part= bs->bsBegin.bpParticule;
-	stroff= bs->bsBegin.bpStroff;
+	part= ds->dsBegin.dpParticule;
+	stroff= ds->dsBegin.dpStroff;
 	}
 
     tp= bi->biParaParticules+ part;
     s= bi->biParaString+ stroff;
 
-    if  ( bs && bs->bsEnd.bpBi == bi )
-	{ stroffUpto= bs->bsEnd.bpStroff;	}
+    if  ( ds && ds->dsEnd.dpBi == bi )
+	{ stroffUpto= ds->dsEnd.dpStroff;	}
     else{ stroffUpto= -1;			}
 
     for ( ; part < bi->biParaParticuleCount; part++, tp++ )
 	{
-	docRtfSwitchTextAttributes( sos, pCol, tp, rwc );
+	docRtfSwitchTextAttributes( sos, pCol, bd, tp, rwc );
 
 	if  ( rwc->rwcSaveAsLink )
 	    {
@@ -762,6 +671,14 @@ static int docRtfSaveParaItem(	SimpleOutputStream *	sos,
 	    {
 	    int		n;
 
+	    case DOCkindCHFTNSEP:
+		if  ( stroffUpto >= 0 && stroff >= stroffUpto )
+		    { break;	}
+
+		docRtfWriteTag( "\\chftnsep", pCol, sos ); s++; stroff++;
+		rwc->rwcHasPrecedingTags= 1;
+		continue;
+
 	    case DOCkindTAB:
 		if  ( stroffUpto >= 0 && stroff >= stroffUpto )
 		    { break;	}
@@ -825,6 +742,40 @@ static int docRtfSaveParaItem(	SimpleOutputStream *	sos,
 		rwc->rwcHasPrecedingTags= 0;
 		continue;
 
+	    case DOCkindNOTE:
+		{
+		DocumentNote *	dn;
+
+		if  ( docGetNote( &dn, bd, bi, tp->tpStroff ) >= 0	&&
+		      dn->dnExternalItem.eiItem				)
+		    {
+		    switch( dn->dnExternalItemKind )
+			{
+			case DOCinFOOTNOTE:
+			    if  ( docRtfSaveExternalItem( sos, pCol,
+					dn->dnExternalItem.eiItem,
+					bd, "\\footnote", rwc )	)
+				{ LDEB(tp->tpKind); return -1;	}
+			    break;
+
+			case DOCinENDNOTE:
+			    if  ( docRtfSaveExternalItem( sos, pCol,
+					dn->dnExternalItem.eiItem,
+					bd, "\\footnote\\ftnalt", rwc )	)
+				{ LDEB(tp->tpKind); return -1;	}
+			    break;
+
+			default:
+			    LDEB(dn->dnExternalItemKind);
+			    break;
+			}
+		    }
+		else{ LDEB(tp->tpObjectNumber);	}
+
+		s += tp->tpStrlen; stroff += tp->tpStrlen; /* += 0 */
+		}
+		continue;
+
 	    case DOCkindFIELDSTART:
 		if  ( stroffUpto >= 0 && stroff >= stroffUpto )
 		    { break;	}
@@ -846,28 +797,20 @@ static int docRtfSaveParaItem(	SimpleOutputStream *	sos,
 		    rwc->rwcHasPrecedingTags= 0;
 		    }
 
-		if  ( df->dfKind == DOCfkFOOTNOTE )
-		    {
-		    if  ( docRtfSaveExternalItem( sos, pCol,
-					df->dfReferencedItem, bd,
-					"\\footnote", rwc )	)
-			{ LDEB(1); return -1;	}
-		    }
-
 		if  ( df->dfKind == DOCfkCHFTN )
 		    {
+		    int		count;
+
+		    count= docCountParticulesInField( bi, part,
+						bi->biParaParticuleCount );
+
 		    docRtfWriteTag( "\\chftn", pCol, sos );
 		    rwc->rwcHasPrecedingTags= 1;
 
-		    /*HACK*/
-		    s += tp->tpStrlen; stroff += tp->tpStrlen; /* += 0 */
-
-		    while( part < bi->biParaParticuleCount- 1	&&
-			   tp[1].tpKind != DOCkindFIELDEND	)
-			{
-			s += tp[1].tpStrlen; stroff += tp[1].tpStrlen;
-			part++; tp++;
-			}
+		    part += count+ 1;
+		    tp += count+ 1;
+		    stroff= tp->tpStroff;
+		    s= bi->biParaString+ stroff;
 
 		    continue;
 		    }
@@ -893,8 +836,7 @@ static int docRtfSaveParaItem(	SimpleOutputStream *	sos,
 			{ docRtfFinishFldrslt( sos, pCol, rwc );	}
 		    }
 
-		if  ( df->dfKind == DOCfkFOOTNOTE	||
-		      df->dfKind == DOCfkCHFTN		)
+		if  ( df->dfKind == DOCfkCHFTN )
 		    { /* nothing */	}
 
 		s += tp->tpStrlen; stroff += tp->tpStrlen; /* += 0 */
@@ -970,7 +912,9 @@ static int docRtfSaveParaItem(	SimpleOutputStream *	sos,
 	while( rwc->rwcInFldrslt > 0 )
 	    { docRtfFinishFldrslt( sos, pCol, rwc ); }
 
-	if  ( ! bi->biParaInTable					||
+	if  ( ( ! bi->biParaInTable				&&
+	        bi->biInExternalItem == DOCinBODY		)	||
+	      tabsSaved							||
 	      bi->biNumberInParent < bi->biParent->biChildCount- 1	)
 	    {
 	    docRtfWriteTag( "\\par", pCol, sos );
@@ -979,8 +923,13 @@ static int docRtfSaveParaItem(	SimpleOutputStream *	sos,
 	    }
 	}
 
-    if  ( docCopyParagraphProperties( &(rwc->rwcParagraphProperties),
-						&(bi->biParaProperties) ) )
+    PROPmaskCLEAR( &ppChgMask );
+
+    PROPmaskCLEAR( &ppUpdMask );
+    PROPmaskFILL( &ppUpdMask, PPprop_COUNT );
+
+    if  ( docUpdParaProperties( &ppChgMask, &(rwc->rwcParagraphProperties),
+			&ppUpdMask, &(bi->biParaProperties), (const int *)0 ) )
 	{ LDEB(1); return -1;	}
 
     return 0;
@@ -992,17 +941,17 @@ static int docRtfSaveParaItem(	SimpleOutputStream *	sos,
 /*									*/
 /************************************************************************/
 
-static int docRtfSaveRowItem(	SimpleOutputStream *	sos,
-				int *			pCol,
-				const BufferDocument *	bd,
-				const BufferItem *	rowBi,
-				const BufferSelection *	bs,
-				RtfWritingContext *	rwc )
+static int docRtfSaveRowItem(	SimpleOutputStream *		sos,
+				int *				pCol,
+				const BufferDocument *		bd,
+				const BufferItem *		rowBi,
+				const DocumentSelection *	ds,
+				RtfWritingContext *		rwc )
     {
     int				col;
 
-    if  ( ! bs								||
-	  bs->bsBegin.bpBi->biParent != bs->bsEnd.bpBi->biParent	)
+    if  ( ! ds								||
+	  ds->dsBegin.dpBi->biParent != ds->dsEnd.dpBi->biParent	)
 	{
 	if  ( ! docEqualRows( &(rowBi->biRowProperties),
 						&(rwc->rwcRowProperties) ) )
@@ -1012,7 +961,7 @@ static int docRtfSaveRowItem(	SimpleOutputStream *	sos,
 	    }
 
 	if  ( ! rwc->rwcInTable					&&
-	      docRtfPushTable( rwc, pCol, sos, rowBi, bs )	)
+	      docRtfPushTable( rwc, pCol, sos, rowBi, ds )	)
 	    { LDEB(1);	}
 	}
 
@@ -1021,29 +970,29 @@ static int docRtfSaveRowItem(	SimpleOutputStream *	sos,
 	int		par;
 	BufferItem *	cellBi= rowBi->biChildren[col];
 
-	if  ( bs && docCompareItemPositions( cellBi, bs->bsBegin.bpBi ) < 0 )
+	if  ( ds && docCompareItemPositions( cellBi, ds->dsBegin.dpBi ) < 0 )
 	    { continue;	}
 
-	if  ( bs && docCompareItemPositions( cellBi, bs->bsEnd.bpBi ) > 0 )
+	if  ( ds && docCompareItemPositions( cellBi, ds->dsEnd.dpBi ) > 0 )
 	    { continue;	}
 
-	if  ( ! bs						||
-	      ( ( bs->bsCol0 < 0 || col >= bs->bsCol0 )	&&
-		( bs->bsCol1 < 0 || col <= bs->bsCol1 )	)	)
+	if  ( ! ds						||
+	      ( ( ds->dsCol0 < 0 || col >= ds->dsCol0 )	&&
+		( ds->dsCol1 < 0 || col <= ds->dsCol1 )	)	)
 	    {
 	    for ( par= 0; par < cellBi->biChildCount; par++ )
 		{
 		BufferItem *	paraBi= cellBi->biChildren[par];
 
-		if  ( bs && docCompareItemPositions( paraBi,
-						    bs->bsBegin.bpBi ) < 0 )
+		if  ( ds && docCompareItemPositions( paraBi,
+						    ds->dsBegin.dpBi ) < 0 )
 		    { continue;	}
 
-		if  ( bs && docCompareItemPositions( paraBi,
-						    bs->bsEnd.bpBi ) > 0 )
+		if  ( ds && docCompareItemPositions( paraBi,
+						    ds->dsEnd.dpBi ) > 0 )
 		    { continue;	}
 
-		if  ( docRtfSaveParaItem( sos, pCol, bd, paraBi, bs, rwc ) )
+		if  ( docRtfSaveParaItem( sos, pCol, bd, paraBi, ds, rwc ) )
 		    { LLDEB(col,par); return -1;	}
 		}
 
@@ -1059,85 +1008,40 @@ static int docRtfSaveRowItem(	SimpleOutputStream *	sos,
     return 0;
     }
 
-/************************************************************************/
-/*									*/
-/*  Save the headers and footers of a Section.				*/
-/*									*/
-/************************************************************************/
-
-static int docRtfSaveHeadersFooters(	SimpleOutputStream *	sos,
-					int *			pCol,
-					const BufferItem *	bi,
-					const BufferDocument *	bd,
-					RtfWritingContext *	rwc )
-    {
-    if  ( docRtfSaveExternalItem( sos, pCol, bi->biSectHeader.hfItem,
-						bd, "\\header", rwc )	)
-	{ LDEB(1); return -1;	}
-
-    if  ( docRtfSaveExternalItem( sos, pCol, bi->biSectFirstPageHeader.hfItem,
-						bd, "\\headerf", rwc )	)
-	{ LDEB(1); return -1;	}
-
-    if  ( docRtfSaveExternalItem( sos, pCol, bi->biSectLeftPageHeader.hfItem,
-						bd, "\\headerl", rwc )	)
-	{ LDEB(1); return -1;	}
-
-    if  ( docRtfSaveExternalItem( sos, pCol, bi->biSectRightPageHeader.hfItem,
-						bd, "\\headerr", rwc )	)
-	{ LDEB(1); return -1;	}
-
-    if  ( docRtfSaveExternalItem( sos, pCol, bi->biSectFooter.hfItem,
-						bd, "\\footer", rwc )	)
-	{ LDEB(1); return -1;	}
-
-    if  ( docRtfSaveExternalItem( sos, pCol, bi->biSectFirstPageFooter.hfItem,
-						bd, "\\footerf", rwc )	)
-	{ LDEB(1); return -1;	}
-
-    if  ( docRtfSaveExternalItem( sos, pCol, bi->biSectLeftPageFooter.hfItem,
-						bd, "\\footerl", rwc )	)
-	{ LDEB(1); return -1;	}
-
-    if  ( docRtfSaveExternalItem( sos, pCol, bi->biSectRightPageFooter.hfItem,
-						bd, "\\footerr", rwc )	)
-	{ LDEB(1); return -1;	}
-
-    return 0;
-    }
-
 static int docSaveSectionProperties(	SimpleOutputStream *		sos,
 					int *				pCol,
 					const DocumentProperties *	dp,
-					const BufferSelection *		bs,
+					const DocumentSelection *	ds,
 					const SectionProperties *	sp,
 					RtfWritingContext *		rwc )
     {
     SectionProperties		spDef;
-    unsigned int		updMask;
+    PropertyMask		updMask;
 
-    updMask= DGupdALL;
     docInitSectionProperties( &spDef );
+
+    PROPmaskCLEAR( &updMask );
+    PROPmaskFILL( &updMask, DGprop_COUNT );
     PROPmaskUNSET( &updMask, DGpropHEADER_POSITION );
     PROPmaskUNSET( &updMask, DGpropFOOTER_POSITION );
 
     appSetDocumentGeometry( &(spDef.spDocumentGeometry),
-				    &(dp->dpGeometry), &updMask, updMask );
+				    &(dp->dpGeometry), &updMask, &updMask );
 
     PROPmaskCLEAR( &updMask );
     PROPmaskFILL( &updMask, SPprop_COUNT );
 
-    updMask= docSectPropertyDifference( &spDef, sp, updMask );
+    docSectPropertyDifference( &updMask, &spDef, sp, &updMask );
 
     docCleanSectionProperties( &spDef );
-    if  ( bs )
+    if  ( ds )
 	{ PROPmaskUNSET( &updMask, SPpropTITLEPG );	}
 
     docRtfWriteNextLine( pCol, sos );
     docRtfWriteTag( "\\sectd", pCol, sos );
 
     docRtfSaveSectionProperties( sos,
-				rwc->rwcOutputMapping, pCol, updMask, sp );
+				rwc->rwcOutputMapping, pCol, &updMask, sp );
 
     return 0;
     }
@@ -1149,23 +1053,39 @@ static int docSaveSectionProperties(	SimpleOutputStream *		sos,
 /*									*/
 /************************************************************************/
 
-static int docRtfSaveItem(	SimpleOutputStream *	sos,
-				int *			pCol,
-				const BufferItem *	bi,
-				const BufferDocument *	bd,
-				const BufferSelection *	bs,
-				RtfWritingContext *	rwc )
+static int docRtfSaveGroupItem(	SimpleOutputStream *		sos,
+				int *				pCol,
+				const BufferItem *		bi,
+				const BufferDocument *		bd,
+				const DocumentSelection *	ds,
+				RtfWritingContext *		rwc )
+    {
+    int				i;
+
+    for ( i= 0; i < bi->biChildCount; i++ )
+	{
+	if  ( docRtfSaveItem( sos, pCol, bi->biChildren[i], bd, ds, rwc ) )
+	    { LDEB(i); return -1;	}
+	}
+
+    return 0;
+    }
+
+static int docRtfSaveItem(	SimpleOutputStream *		sos,
+				int *				pCol,
+				const BufferItem *		bi,
+				const BufferDocument *		bd,
+				const DocumentSelection *	ds,
+				RtfWritingContext *		rwc )
     {
     const DocumentProperties *	dp= &(bd->bdProperties);
 
-    int				i;
-
-    if  ( bs )
+    if  ( ds )
 	{
-	if  ( docCompareItemPositions( bi, bs->bsBegin.bpBi ) < 0	)
+	if  ( docCompareItemPositions( bi, ds->dsBegin.dpBi ) < 0	)
 	    { return 0;	}
 
-	if  ( docCompareItemPositions( bi, bs->bsEnd.bpBi ) > 0		)
+	if  ( docCompareItemPositions( bi, ds->dsEnd.dpBi ) > 0		)
 	    { return 0;	}
 	}
 
@@ -1173,38 +1093,33 @@ static int docRtfSaveItem(	SimpleOutputStream *	sos,
 	{
 	case DOClevDOC:
 	case DOClevCELL:
-	rowAsGroup:
-	    for ( i= 0; i < bi->biChildCount; i++ )
-		{
-		if  ( docRtfSaveItem( sos, pCol,
-					    bi->biChildren[i], bd, bs, rwc ) )
-		    { LDEB(i); return -1;	}
-		}
+	    if  ( docRtfSaveGroupItem( sos, pCol, bi, bd, ds, rwc ) )
+		{ LDEB(1); return -1;	}
 	    break;
+
 	case DOClevSECT:
 
-	    if  ( docSaveSectionProperties( sos, pCol, dp, bs,
+	    if  ( docSaveSectionProperties( sos, pCol, dp, ds,
 					    &(bi->biSectProperties), rwc ) )
 		{ LDEB(1); return -1;	}
 
-	    if  ( ! bs							&&
-		  docRtfSaveHeadersFooters( sos, pCol, bi, bd, rwc )	)
+	    if  ( ! ds							&&
+		  docRtfSaveSectHeadersFooters( sos, pCol, bi, bd, rwc ) )
 		{ LDEB(1); return -1;	}
 
-	    for ( i= 0; i < bi->biChildCount; i++ )
-		{
-		if  ( docRtfSaveItem( sos, pCol,
-					bi->biChildren[i], bd, bs, rwc ) )
-		    { LDEB(i); return -1;	}
-		}
+	    if  ( docRtfSaveGroupItem( sos, pCol, bi, bd, ds, rwc ) )
+		{ LDEB(1); return -1;	}
 
 	    if  ( rwc->rwcInTable && docRtfPopTable( rwc, pCol, sos ) )
-		{ LDEB(1);	}
+		{ LDEB(rwc->rwcInTable);	}
 
 	    while( rwc->rwcInFldrslt > 0 )
 		{ docRtfFinishFldrslt( sos, pCol, rwc );	}
 
-	    docRtfWriteTag( "\\sect", pCol, sos );
+	    if  ( bi->biParent						&&
+		  bi->biNumberInParent < bi->biParent->biChildCount- 1	)
+		{ docRtfWriteTag( "\\sect", pCol, sos );	}
+
 	    docRtfWriteNextLine( pCol, sos );
 	    break;
 
@@ -1213,16 +1128,18 @@ static int docRtfSaveItem(	SimpleOutputStream *	sos,
 		{
 		if  ( rwc->rwcInTable && docRtfPopTable( rwc, pCol, sos ) )
 		    { LDEB(1);	}
-		goto rowAsGroup;
+
+		if  ( docRtfSaveGroupItem( sos, pCol, bi, bd, ds, rwc ) )
+		    { LDEB(1); return -1;	}
+		}
+	    else{
+		if  ( docRtfSaveRowItem( sos, pCol, bd, bi, ds, rwc ) )
+		    { LDEB(1); return -1;	}
 		}
-
-	    if  ( docRtfSaveRowItem( sos, pCol, bd, bi, bs, rwc ) )
-		{ LDEB(1); return -1;	}
-
 	    break;
 
 	case DOClevPARA:
-	    if  ( docRtfSaveParaItem( sos, pCol, bd, bi, bs, rwc ) )
+	    if  ( docRtfSaveParaItem( sos, pCol, bd, bi, ds, rwc ) )
 		{ LDEB(1); return -1;	}
 	    break;
 
@@ -1237,11 +1154,11 @@ static void docRtfInitWritingContext(	RtfWritingContext *	rwc )
     {
     int		i;
 
-    docInitTextAttribute( &(rwc->rwcTextAttribute) );
+    utilInitTextAttribute( &(rwc->rwcTextAttribute) );
     docInitParagraphProperties( &(rwc->rwcParagraphProperties) );
     docInitRowProperties( &(rwc->rwcRowProperties) );
 
-    docInitTextAttribute( &(rwc->rwcOutsideTableTextAttribute) );
+    utilInitTextAttribute( &(rwc->rwcOutsideTableTextAttribute) );
     docInitParagraphProperties( &(rwc->rwcOutsideTableParagraphProperties) );
 
     rwc->rwcOutsideFldrsltAttribute= (PushedAttribute *)0;
@@ -1269,27 +1186,27 @@ static void docRtfCleanWritingContext(	RtfWritingContext *	rwc )
 
     docCleanParagraphProperties( &rwc->rwcOutsideTableParagraphProperties );
 
-    docCleanField( (BufferDocument *)0, &(rwc->rwcSaveAsHyperlinkField) );
-    docCleanField( (BufferDocument *)0, &(rwc->rwcSaveAsRefField) );
-    docCleanField( (BufferDocument *)0, &(rwc->rwcSaveAsPagerefField) );
+    docCleanField( &(rwc->rwcSaveAsHyperlinkField) );
+    docCleanField( &(rwc->rwcSaveAsRefField) );
+    docCleanField( &(rwc->rwcSaveAsPagerefField) );
     }
 
-int docRtfSaveDocument(		SimpleOutputStream *	sos,
-				BufferDocument *	bd,
-				const SelectionScope *	ss,
-				const BufferSelection *	bs,
-				int			saveBookmarks )
+int docRtfSaveDocument(	SimpleOutputStream *		sos,
+			BufferDocument *		bd,
+			const DocumentSelection *	ds,
+			int				saveBookmarks )
     {
-    const BufferItem *		bi= &bd->bdItem;
+    const BufferItem *		bi= &(bd->bdItem);
     int				col= 0;
 
     RtfWritingContext		rwc;
 
-    if  ( bs )
+    if  ( ds )
 	{
-	const int	nearest= 1;
+	ExternalItem *	ei;
+	BufferItem *	bodySectBi;
 
-	bi= docGetSelectionRoot( bd, nearest, ss, bs );
+	bi= docGetSelectionRoot( &ei, &bodySectBi, bd, ds );
 	if  ( ! bi )
 	    { XDEB(bi); return -1;	}
 	}
@@ -1303,12 +1220,18 @@ int docRtfSaveDocument(		SimpleOutputStream *	sos,
     if  ( docRtfSaveDocumentProperties( sos, &col, &rwc, bd ) )
 	{ LDEB(1); return -1;	}
 
-    if  ( docRtfSaveItem( sos, &col, bi, bd, bs, &rwc ) )
+    if  ( docRtfSaveDocNotesSeparators( sos, &col, bd, &rwc ) )
+	{ LDEB(1); docRtfCleanWritingContext( &rwc ); return -1;	}
+
+    if  ( docRtfSaveItem( sos, &col, bi, bd, ds, &rwc ) )
 	{ LDEB(bi->biLevel); return -1; }
 
     while( rwc.rwcInFldrslt > 0 )
 	{ docRtfFinishFldrslt( sos, &col, &rwc ); }
 
+    if  ( rwc.rwcInTable && docRtfPopTable( &rwc, &col, sos ) )
+	{ LDEB(rwc.rwcInTable);	}
+
     docRtfWriteDestinationEnd( &col, sos );
     docRtfWriteNextLine( &col, sos );
 
@@ -1317,18 +1240,17 @@ int docRtfSaveDocument(		SimpleOutputStream *	sos,
     return 0;
     }
 
-int docRtfSaveSelectionAsLink(	SimpleOutputStream *	sos,
-				const BufferDocument *	bd,
-				const SelectionScope *	ss,
-				const BufferSelection *	bs,
-				int			asRef,
-				int			asPageref,
-				const unsigned char *	fileName,
-				int			fileSize,
-				const unsigned char *	markName,
-				int			markSize )
+int docRtfSaveSelectionAsLink(	SimpleOutputStream *		sos,
+				const BufferDocument *		bd,
+				const DocumentSelection *	ds,
+				int				asRef,
+				int				asPageref,
+				const unsigned char *		fileName,
+				int				fileSize,
+				const unsigned char *		markName,
+				int				markSize )
     {
-    const BufferItem *		bi= &bd->bdItem;
+    const BufferItem *		bi= &(bd->bdItem);
     int				col= 0;
 
     RtfWritingContext		rwc;
@@ -1361,12 +1283,15 @@ int docRtfSaveSelectionAsLink(	SimpleOutputStream *	sos,
 
     docRtfWriteNextLine( &col, sos );
 
-    if  ( docRtfSaveItem( sos, &col, bi, bd, bs, &rwc ) )
+    if  ( docRtfSaveItem( sos, &col, bi, bd, ds, &rwc ) )
 	{ LDEB(bi->biLevel); docRtfCleanWritingContext( &rwc ); return -1; }
 
     while( rwc.rwcInFldrslt > 0 )
 	{ docRtfFinishFldrslt( sos, &col, &rwc ); }
 
+    if  ( rwc.rwcInTable && docRtfPopTable( &rwc, &col, sos ) )
+	{ LDEB(rwc.rwcInTable);	}
+
     docRtfWriteDestinationEnd( &col, sos );
     docRtfWriteNextLine( &col, sos );
 
