@@ -547,7 +547,18 @@ int docPrevWord(	DocumentPosition *	dp,
     }
 
 
-int docPrevLine(	TextParticule **	pTp,
+/************************************************************************/
+/*									*/
+/*  Move one line up.							*/
+/*									*/
+/*  1)  Go to previous line of the paragraph or to the last line of the	*/
+/*	last paragraph before this one that has any lines.		*/
+/*  2)  In a clumsy attempt to implement vertical movement in tables,	*/
+/*	try to stay in the same column of a table.			*/
+/*									*/
+/************************************************************************/
+
+int docLineUp(		TextParticule **	pTp,
 			TextLine **		pTl,
 			DocumentPosition *	dp )
     {
@@ -563,6 +574,7 @@ int docPrevLine(	TextParticule **	pTp,
 
     line--;
 
+    /*  1  */
     while( bi )
 	{
 	if  ( bi->biParaLineCount == 0 )
@@ -591,6 +603,39 @@ int docPrevLine(	TextParticule **	pTp,
 	    return 0;
 	    }
 
+	/*  2  */
+	if  ( bi->biLevel == DOClevPARA		&&
+	      bi->biParaInTable			&&
+	      bi->biNumberInParent == 0		)
+	    {
+	    BufferItem *	cellBi= bi->biParent;
+	    BufferItem *	rowBi= cellBi->biParent;
+	    BufferItem *	sectBi= rowBi->biParent;
+
+	    if  ( rowBi->biNumberInParent > 0 )
+		{
+		rowBi= sectBi->biChildren[rowBi->biNumberInParent- 1];
+
+		if  ( rowBi->biRowHasTableParagraphs		&&
+		      rowBi->biChildCount > 0			)
+		    {
+		    if  ( cellBi->biNumberInParent < rowBi->biChildCount )
+			{ cellBi= rowBi->biChildren[cellBi->biNumberInParent]; }
+		    else{ cellBi= rowBi->biChildren[rowBi->biChildCount- 1]; }
+
+		    while( cellBi->biChildCount > 0 )
+			{ cellBi= cellBi->biChildren[cellBi->biChildCount-1]; }
+		    if  ( cellBi->biLevel == DOClevPARA )
+			{
+			bi= cellBi;
+			part= bi->biParaParticuleCount- 1;
+			line= bi->biParaLineCount- 1;
+			continue;
+			}
+		    }
+		}
+	    }
+
 	bi= docPrevParagraph( bi );
 	if  ( bi )
 	    {
@@ -602,7 +647,18 @@ int docPrevLine(	TextParticule **	pTp,
     return -1;
     }
 
-int docNextLine(	TextParticule **	pTp,
+/************************************************************************/
+/*									*/
+/*  Move one line down.							*/
+/*									*/
+/*  1)  Go to the next line of the paragraph or to the first line of	*/
+/*	the first paragraph after this one that has any lines.		*/
+/*  2)  In a clumsy attempt to implement vertical movement in tables,	*/
+/*	try to stay in the same column of a table.			*/
+/*									*/
+/************************************************************************/
+
+int docLineDown(	TextParticule **	pTp,
 			TextLine **		pTl,
 			DocumentPosition *	dp )
     {
@@ -618,6 +674,7 @@ int docNextLine(	TextParticule **	pTp,
 
     line++;
 
+    /*  1  */
     while( bi )
 	{
 	if  ( bi->biLevel == DOClevPARA		&&
@@ -640,6 +697,34 @@ int docNextLine(	TextParticule **	pTp,
 	    return 0;
 	    }
 
+	/*  2  */
+	if  ( bi->biLevel == DOClevPARA					&&
+	      bi->biParaInTable						&&
+	      bi->biNumberInParent == bi->biParent->biChildCount- 1	)
+	    {
+	    BufferItem *	cellBi= bi->biParent;
+	    BufferItem *	rowBi= cellBi->biParent;
+	    BufferItem *	sectBi= rowBi->biParent;
+
+	    if  ( rowBi->biNumberInParent < sectBi->biChildCount- 1 )
+		{
+		rowBi= sectBi->biChildren[rowBi->biNumberInParent+ 1];
+
+		if  ( rowBi->biRowHasTableParagraphs		&&
+		      rowBi->biChildCount > 0			)
+		    {
+		    if  ( cellBi->biNumberInParent < rowBi->biChildCount )
+			{ cellBi= rowBi->biChildren[cellBi->biNumberInParent]; }
+		    else{ cellBi= rowBi->biChildren[rowBi->biChildCount- 1]; }
+
+		    while( cellBi->biChildCount > 0 )
+			{ cellBi= cellBi->biChildren[0]; }
+		    if  ( cellBi->biLevel == DOClevPARA )
+			{ bi= cellBi; part= 0; line= 0; continue; }
+		    }
+		}
+	    }
+
 	bi= docNextParagraph( bi );
 	part= 0; line= 0;
 	}
@@ -963,55 +1048,6 @@ void docSetSelectionScope(	DocumentSelection *	ds,
     return;
     }
 
-/************************************************************************/
-/*									*/
-/*  Find the range of rows for a 'table'				*/
-/*									*/
-/************************************************************************/
-
-int docDelimitTable(	BufferItem *		paraBi,
-			BufferItem **		pSectBi,
-			int *			pCol,
-			int *			pRow0,
-			int *			pRow,
-			int *			pRow1 )
-    {
-    BufferItem *	rowBi;
-    BufferItem *	sectBi;
-    int			col;
-
-    int			row;
-    int			row0;
-    int			row1;
-
-    if  ( paraBi->biLevel != DOClevPARA		||
-	  ! paraBi->biParaInTable		)
-	{ /* LLDEB(paraBi->biLevel,paraBi->biParaInTable); */ return -1; }
-
-    rowBi= paraBi->biParent->biParent;
-    sectBi= rowBi->biParent;
-    col= paraBi->biParent->biNumberInParent;
-
-    row= rowBi->biNumberInParent;
-    row0= rowBi->biNumberInParent;
-    row1= rowBi->biNumberInParent;
-
-    while( row0 > 0							&&
-	   docAlignedColumns( &(rowBi->biRowProperties),
-		&(sectBi->biChildren[row0-1]->biRowProperties) )	)
-	{ row0--;	}
-
-    while( row1 < sectBi->biChildCount- 1				&&
-	   docAlignedColumns( &(rowBi->biRowProperties),
-		&(sectBi->biChildren[row1+1]->biRowProperties) )	)
-	{ row1++;	}
-
-    *pSectBi= sectBi; *pCol= col;
-    *pRow0= row0; *pRow= row; *pRow1= row1;
-
-    return 0;
-    }
-
 /************************************************************************/
 /*									*/
 /*  Are we in a something.						*/
