@@ -4,7 +4,7 @@
 /*									*/
 /************************************************************************/
 
-#   include	"config.h"
+#   include	"tedConfig.h"
 
 #   include	<stdlib.h>
 #   include	<string.h>
@@ -21,28 +21,59 @@
 
 /************************************************************************/
 /*									*/
-/*  Find the first misspelling from a certain position in a document.	*/
+/*  Find the first misspelling from a certain position in a paragraph.	*/
+/*									*/
+/*  1)	Skip initial particules: Find the first particule that starts	*/
+/*	after the starting point for spell checking.			*/
+/*  2)	Skip particules that can be included in a spell check range.	*/
+/*  3)	If spell checking starts in the middle of a word, skip to the	*/
+/*	end of the word.						*/
+/*  4)	Do not extend the string beyond the beginning/end of a field.	*/
+/*									*/
+/*  TODO: Do not check the contents of fields that are not editable.	*/
 /*									*/
 /************************************************************************/
 
-int docSpellParaFindNext(	const BufferItem *	paraBi,
-				int			stroff,
-				int *			pStart,
-				int *			pLength,
-				void *			through )
+int docSpellParaFindNext(	DocumentSelection *		ds,
+				BufferItem *			paraBi,
+				const BufferDocument *		bd,
+				const DocumentPosition *	dpFrom,
+				void *				through )
     {
     SpellCheckContext *		scc= (SpellCheckContext *)through;
     SpellScanJob		ssj;
+    int				stroff= dpFrom->dpStroff;
     int				acceptedPos= stroff;
     const unsigned char *	str= paraBi->biParaString+ stroff;
 
+    int				part= dpFrom->dpParticule;
+    const TextParticule *	tp= paraBi->biParaParticules+ part;
+
+    int				stroffUpto;
+
     indInitSpellScanJob( &ssj );
 
+    /*  1  */
+    while( part < paraBi->biParaParticuleCount	&&
+	   tp->tpStroff <= stroff		)
+	{ part++; tp++;	}
+
+    /*  2  */
+    while( part < paraBi->biParaParticuleCount	&&
+	   ! DOCkindBETWEEN_TEXT( tp->tpKind )	)
+	{ part++; tp++;	}
+
+    /*  3  */
     if  ( stroff > 0						&&
 	  ( scc->sccCharKinds[ str[ 0] ] & CHARisALNUM )	&&
 	  ( scc->sccCharKinds[ str[-1] ] & CHARisALNUM )	)
 	{
-	while( stroff < paraBi->biParaStrlen			&&
+	/*  4  */
+	if  ( part < paraBi->biParaParticuleCount )
+	    { stroffUpto= tp->tpStroff;		}
+	else{ stroffUpto= paraBi->biParaStrlen; }
+
+	while( stroff < stroffUpto				&&
 	       ( scc->sccCharKinds[ *str ] & CHARisALNUM )	)
 	    { stroff++; str++; }
 	}
@@ -51,7 +82,22 @@ int docSpellParaFindNext(	const BufferItem *	paraBi,
 	{
 	int		count;
 
-	while( stroff < paraBi->biParaStrlen			&&
+	/*  1  */
+	while( part < paraBi->biParaParticuleCount	&&
+	       tp->tpStroff <= stroff		)
+	    { part++; tp++;	}
+
+	/*  2  */
+	while( part < paraBi->biParaParticuleCount	&&
+	       ! DOCkindBETWEEN_TEXT( tp->tpKind )	)
+	    { part++; tp++;	}
+
+	/*  4  */
+	if  ( part < paraBi->biParaParticuleCount )
+	    { stroffUpto= tp->tpStroff;		}
+	else{ stroffUpto= paraBi->biParaStrlen; }
+
+	while( stroff < stroffUpto				&&
 	       ! ( scc->sccCharKinds[ *str ] & CHARisALNUM )	)
 	    {
 	    indAddCharacterToPossibilities( &ssj, *str );
@@ -67,7 +113,7 @@ int docSpellParaFindNext(	const BufferItem *	paraBi,
 	    { CDEB(*str); return -1;	}
 
 	stroff++; str++;
-	while( stroff < paraBi->biParaStrlen		&&
+	while( stroff < stroffUpto				&&
 	       ( scc->sccCharKinds[*str] & CHARisALNUM )	)
 	    {
 	    indAddCharacterToPossibilities( &ssj, *str );
@@ -80,16 +126,16 @@ int docSpellParaFindNext(	const BufferItem *	paraBi,
 	if  ( count == 0 )
 	    {
 	    PossibleWord *	maxpw;
+	    const int		direction= 1;
 
 	    maxpw= indMaximalPossibility( &ssj );
 
 	    if  ( ! maxpw )
-		{
-		XDEB(maxpw); return -1;	}
-
-	    *pStart= maxpw->pwStartPosition;
-	    *pLength= maxpw->pwInsertionPoint;
+		{ XDEB(maxpw); return -1;	}
 
+	    docSetParaSelection( ds, paraBi, direction,
+						maxpw->pwStartPosition,
+						maxpw->pwInsertionPoint );
 	    indCleanSpellScanJob( &ssj );
 
 	    return 0;
