@@ -1,10 +1,13 @@
 #   include	<appFrame.h>
 #   include	<appImage.h>
 
-#   include	<appGeo.h>
+#   include	<utilPropMask.h>
+#   include	<utilMemoryBuffer.h>
+#   include	<docExpandedTextAttribute.h>
+
+#   include	<geo2DInteger.h>
 #   include	<appDraw.h>
 #   include	"docBuf.h"
-#   include	<sioMemory.h>
 
 #   ifndef	TED_APP_H
 #   define	TED_APP_H
@@ -21,11 +24,11 @@ typedef struct TedDocument
     {
     BufferDocument *	tdDocument;
     TextAttribute	tdCurrentTextAttribute;
+    int			tdCurrentTextAttributeNumber;
     int			tdCurrentPhysicalFont;
 
-    BufferSelection	tdSelection;
-    SelectionScope	tdSelectionScope;
-    DocumentRectangle	tdSelectedRectangle;
+    DocumentSelection	tdDocumentSelection;
+    SelectionGeometry	tdSelectionGeometry;
 
     APP_WIDGET		tdCopyWidget;
     APP_WIDGET		tdCutWidget;
@@ -38,6 +41,9 @@ typedef struct TedDocument
     APP_WIDGET		tdInsSymbolOption;
     APP_WIDGET		tdInsHyperlinkOption;
     APP_WIDGET		tdInsBookmarkOption;
+    APP_WIDGET		tdInsInsertFootnoteOption;
+    APP_WIDGET		tdInsInsertEndnoteOption;
+    APP_WIDGET		tdInsInsertChftnsepOption;
     APP_WIDGET		tdInsInsertTableOption;
     APP_WIDGET		tdInsInsertPageNumberOption;
     APP_WIDGET		tdInsInsertLineBreakOption;
@@ -71,10 +77,11 @@ typedef struct TedDocument
     APP_WIDGET		tdToolsMenuButton;
     APP_WIDGET		tdToolsFormatToolOption;
 
-    APP_COLOR_RGB	tdSelColor;
-    APP_COLOR_RGB	tdXSelColor;
     APP_COLOR_RGB	tdFieldColor;
-    APP_COLOR_RGB	tdTableColor;
+
+    RGB8Color		tdTableColor;
+    RGB8Color		tdSelColor;
+    RGB8Color		tdCopiedSelColor;
 
     int			tdVisibleSelectionCopied;
     int			tdCanReplaceSelection;
@@ -142,6 +149,9 @@ typedef struct TedAppResources
     char *			tarDefaultAnsicpgString;
     int				tarDefaultAnsicpgInt;
 
+    char *			tarShowTableGridString;
+    int				tarShowTableGridInt;
+
 				/****************************************/
 				/*  Global data.			*/
 				/****************************************/
@@ -206,106 +216,77 @@ extern AppSelectionTargetType TedRulerTargets[];
 /*									*/
 /************************************************************************/
 
-extern int tedLayoutItem(	BufferItem *		bi,
-				const BufferDocument *	bd,
-				int			inHeaderFooter,
-				AppDrawingData *	add,
-				DocumentRectangle *	drChanged );
-
-extern int tedLayoutExternal(	HeaderFooter *			hf,
-				int				page,
-				const SectionProperties *	sp,
-				const BufferDocument *		bd,
-				AppDrawingData *		add,
-				DocumentRectangle *		drChanged );
-
-extern int tedRedoHeaderFooterLayout( int *			pDone,
-				HeaderFooter *			hf,
-				int				page,
-				const BufferItem *		sectBi,
+extern int tedLayoutItem(	BufferItem *			bi,
 				BufferDocument *		bd,
 				AppDrawingData *		add,
 				DocumentRectangle *		drChanged );
 
-extern int tedFindPosition(	BufferItem *		rootBi,
-				const AppDrawingData *	add,
-				int			x,
-				int			y,
-				BufferPosition *	bp );
+extern int tedLayoutExternalItem(
+				int *				pY1Twips,
+				ExternalItem *			ei,
+				int				page,
+				int				y0Twips,
+				BufferDocument *		bd,
+				AppDrawingData *		add,
+				DocumentRectangle *		drChanged );
 
-extern int tedCalculateX(	const BufferItem *	bi,
-				const TextParticule *	tp,
-				const AppDrawingData *	add,
-				int			stroff	);
+extern int tedFindPosition(	DocumentPosition *		dp,
+				PositionGeometry *		pg,
+				const BufferDocument *		bd,
+				BufferItem *			rootBi,
+				const AppDrawingData *		add,
+				int				x,
+				int				y );
 
-extern int tedFindParticule(	TextLine *	tl,
-				TextParticule *	tp,
-				int		x,
-				int		y	);
+extern int tedFindParticule(	TextLine *			tl,
+				TextParticule *			tp,
+				int				x,
+				int				y );
 
-extern int tedFindStringOffset(	const TextParticule *	tp,
-				const unsigned char *	paraString,
-				const AppDrawingData *	add,
-				int *			pBarX,
-				int			x,
-				int			y	);
+extern int tedFindStringOffset(	const BufferDocument *		bd,
+				const BufferItem *		paraBi,
+				int				part,
+				const AppDrawingData *		add,
+				int *				pBarX,
+				int				x,
+				int				y );
 
-extern int tedHasSelection(	const TedDocument *   td );
-extern int tedHasIBarSelection(	const TedDocument *   td );
+extern int tedHasSelection(	const TedDocument *		td );
+extern int tedHasIBarSelection(	const TedDocument *		td );
 
-extern int tedReplaceSelection(	DocumentRectangle *	drChanged,
-				BufferSelection *	bsRep,
-				EditDocument *		ed,
-				const unsigned char *	addedText,
-				int			addedLength );
+extern int tedArrowDown(	DocumentPosition *		bp,
+				const BufferDocument *		bd,
+				const AppDrawingData *		add );
 
-extern int tedFirstPosition(	const AppDrawingData *	add,
-				BufferDocument *	bd,
-				BufferPosition *	bp	);
+extern int tedLineUp(		DocumentPosition *		bp,
+				const BufferDocument *		bd,
+				const AppDrawingData *		add );
 
-extern int tedLastPosition(	const AppDrawingData *	add,
-				BufferDocument *	bd,
-				BufferPosition *	bp	);
+extern int tedPageUp(	DocumentPosition *			dp,
+			BufferDocument *			bd,
+			const AppDrawingData *			add,
+			int					pageHeight );
 
-extern int tedPrevPosition(	const AppDrawingData *	add,
-				BufferPosition *	bp,
-				int			lastOne );
-extern int tedNextPosition(	const AppDrawingData *	add,
-				BufferPosition *	bp	);
-extern int tedBeginOfLine(	const AppDrawingData *	add,
-				BufferPosition *	bp	);
-extern int tedEndOfLine(	const AppDrawingData *	add,
-				BufferPosition *	bp	);
-extern int tedNextLine(		const AppDrawingData *	add,
-				BufferPosition *	bp	);
-extern int tedPrevLine(		const AppDrawingData *	add,
-				BufferPosition *	bp	);
+extern int tedPageDown(	DocumentPosition *			dp,
+			BufferDocument *			bd,
+			const AppDrawingData *			add,
+			int					docHeight,
+			int					pageHeight );
 
-extern int tedPageUp(	BufferPosition *	bp,
-			BufferDocument *	bd,
-			const AppDrawingData *	add,
-			int			pageHeight );
-
-extern int tedPageDown(	BufferPosition *	bp,
-			BufferDocument *	bd,
-			const AppDrawingData *	add,
-			int			docHeight,
-			int			pageHeight );
-
-extern APP_EVENT_HANDLER( tedObserveFocus, w, voided, event );
+extern APP_EVENT_HANDLER_H( tedObserveFocus, w, voided, event );
 
 extern void tedSplitParagraph(		EditDocument *		ed,
 					int			onNewPage );
 
-extern void tedSetTab(			EditDocument *		ed );
+extern void tedEditReplaceSelectionWithTab(	EditDocument *		ed );
 
 extern int tedChangeSelectionProperties(
 				EditDocument *			ed,
-				unsigned int			taUpdMask,
-				TextAttribute			taNew,
-				unsigned int			ppUpdMask,
+				const PropertyMask *		taSetMask,
+				const TextAttribute *		taSet,
+				const PropertyMask *		ppUpdMask,
 				const ParagraphProperties *	ppNew,
-				unsigned int			spUpdMask,
+				const PropertyMask *		spUpdMask,
 				const SectionProperties *	spNew );
 
 extern void tedRedrawRectangle(	APP_WIDGET		w,
@@ -338,69 +319,59 @@ extern int tedIncludeRtfDocument(	APP_WIDGET		w,
 extern int tedGetDocumentAttributeString(	char *		scratch,
 						TedDocument *	td );
 
-extern int tedGetDocumentAttributes(	TedDocument *	td,
-					char **		pFamilyName,
-					int *		pPsFamilyNumber,
-					unsigned int *	pUpdMask,
-					TextAttribute *	pTaNew );
+extern int tedGetDocumentAttributes(
+				TedDocument *			td,
+				PropertyMask *			pUpdMask,
+				TextAttribute *			pTaNew );
 
-extern void tedGetSelectionAttributes(	TedDocument *	td,
-					char **		pFamilyName,
-					int *		pPsFamilyNumber,
-					unsigned int *	pUpdMask,
-					TextAttribute *	pTaNew );
-
-extern void tedGetPositionAttributes(	TedDocument *		td,
-					const BufferPosition *	bp,
-					char **			pFamilyName,
-					int *			pPsFamilyNumber,
-					unsigned int *		pUpdMask,
-					TextAttribute *		pTaNew );
+extern void tedGetSelectionAttributes(
+				BufferDocument *		bd,
+				const DocumentSelection *	ds,
+				PropertyMask *			pUpdMask,
+				TextAttribute *			pTaNew );
 
 extern void tedDocCopy(		EditDocument *	ed );
 extern void tedDocCut(		EditDocument *	ed );
 
 extern void tedDocSelAll(	EditDocument *	ed );
 
-extern void tedSetSelection(	EditDocument *		ed,
-				const SelectionScope *	ssSet,
-				const BufferSelection *	bsSet,
-				int *			pScrolledX,
-				int *			pScrolledY );
+extern void tedSetSelection(	EditDocument *			ed,
+				const DocumentSelection *	dsSet,
+				int *				pScrolledX,
+				int *				pScrolledY );
 
-extern void tedAppReplaceSelection(	EditDocument *		ed,
+extern void tedDocEditPaste(	APP_WIDGET			option,
+				void *				voided,
+				void *				voidpbcs );
+
+extern void tedDocReplaceSelection(	EditDocument *		ed,
 					const unsigned char *	word,
-					int			len	);
+					int			len );
 
-extern void tedSetSelectedPosition(	EditDocument *		ed,
-					const SelectionScope *	ss,
-					const BufferPosition *	bp,
-					int *			pScrolledX,
-					int *			pScrolledY );
+extern int tedDeleteTableSliceSelection(	EditDocument *	ed );
 
-extern void tedShowFontsTool(		APP_WIDGET		fontsOption,
-					EditApplication *	ea,
-					void *			voidcbs	 );
+extern void tedSetSelectedPosition(
+				EditDocument *			ed,
+				const DocumentPosition *	dp,
+				int *				pScrolledX,
+				int *				pScrolledY );
 
 extern void tedAdaptFontIndicatorsToSelection(	EditApplication *	ea,
 						EditDocument *		ed );
 
-extern void tedAdaptFontsToolToDocument(	EditApplication *	ea,
-						EditDocument *		ed );
-
 extern void tedAdaptToolsToSelection(	EditDocument *		ed );
 
-extern int tedExtendSelectionToXY(	EditDocument *		ed,
-					BufferItem *		rootBi,
-					const BufferPosition *	bpAnchor,
-					const SelectionScope *	ss,
-					int			mouseX,
-					int			mouseY );
+extern int tedExtendSelectionToXY(
+				EditDocument *			ed,
+				BufferItem *			rootBi,
+				const DocumentPosition *	bpAnchor,
+				int				mouseX,
+				int				mouseY );
 
-extern int tedExtendSelectionToPosition(EditDocument *		ed,
-					const SelectionScope *	ss,
-					const BufferPosition *	bpAnchor,
-					const BufferPosition *	bpFound );
+extern int tedExtendSelectionToPosition(
+				EditDocument *			ed,
+				const DocumentPosition *	dpAnchor,
+				const DocumentPosition *	dpFound );
 
 extern void tedScrollToSelection(	EditDocument *		ed,
 					int *			pScrolledX,
@@ -439,27 +410,19 @@ extern void tedGetNamedPictures(	EditApplication *	ea );
 extern int tedAppChangeSelectionAttributeString( EditDocument *	ed,
 					char *		attributeString );
 
-extern void tedAdaptFontIndicatorsToValues( EditDocument *	ed,
-					char *			familyName,
-					int			psFamilyNumber,
-					unsigned int		updMask,
-					TextAttribute		taNew );
-
 extern int tedDocSetTopRuler(	EditDocument *	ed );
 extern int tedDocSetLeftRuler(	EditDocument *	ed );
 
 extern void tedDocAdaptHorizontalRuler(	EditDocument *		ed,
 					BufferItem *		bi );
 
-extern void tedAdaptToolsToPosition(	EditDocument *		ed );
-
-extern void tedRedrawIBar(	TedDocument *		td,
-				int			ox,
-				int			oy,
-				AppDrawingData *	add );
-
 extern void tedUndrawIBar(	const EditDocument *	ed );
 
+extern int tedDrawIBar(		const PositionGeometry *	pg,
+				int				ox,
+				int				oy,
+				AppDrawingData *		add );
+
 extern void tedStartCursorBlink(	EditDocument *	ed );
 extern void tedStopCursorBlink(		EditDocument *	ed );
 extern void tedCleanCursorBlink(	TedDocument *	td );
@@ -482,47 +445,59 @@ extern void tedCloseObject(		BufferDocument *	bd,
 
 extern void tedDocInsertPicture(	APP_WIDGET	option,
 					void *		voided,
-					void *		voidpbcs	 );
+					void *		voidpbcs );
 
 extern void tedDocInsertFile(		APP_WIDGET	option,
 					void *		voided,
-					void *		voidpbcs	 );
+					void *		voidpbcs );
 
 extern void tedDocInsertLink(	APP_WIDGET	option,
 				void *		voided,
-				void *		voidpbcs	 );
+				void *		voidpbcs );
 
 extern void tedDocInsertBookmark(	APP_WIDGET	option,
 					void *		voided,
-					void *		voidpbcs	 );
+					void *		voidpbcs );
+
+extern void tedDocInsertFootnote(	APP_WIDGET	option,
+					void *		voided,
+					void *		voidpbcs );
+
+extern void tedDocInsertEndnote(	APP_WIDGET	option,
+					void *		voided,
+					void *		voidpbcs );
+
+extern void tedDocInsertChftnsep(	APP_WIDGET	option,
+					void *		voided,
+					void *		voidpbcs );
 
 extern void tedDocInsertPageNumber(	APP_WIDGET	option,
 					void *		voided,
-					void *		voidpbcs	 );
+					void *		voidpbcs );
 
 extern void tedDocInsertLineBreak(	APP_WIDGET	option,
 					void *		voided,
-					void *		voidpbcs	 );
+					void *		voidpbcs );
 
 extern void tedDocInsertPageBreak(	APP_WIDGET	option,
 					void *		voided,
-					void *		voidpbcs	 );
+					void *		voidpbcs );
 
 extern void tedDocInsertSectBreak(	APP_WIDGET	option,
 					void *		voided,
-					void *		voidpbcs	 );
+					void *		voidpbcs );
 
 extern void TEDmiDocInsertInsSectBreak(	APP_WIDGET	option,
 					void *		voided,
-					void *		voidpbcs	 );
+					void *		voidpbcs );
 
 extern void tedDocFormatOnePara(	APP_WIDGET	option,
 					void *		voided,
 					void *		voidpbcs );
 
-extern int tedInsertNewObject(		APP_WIDGET		w,
-					InsertedObject *	io,
-					EditDocument *		ed );
+extern int tedReplaceSelectionWithObject(
+					EditDocument *		ed,
+					InsertedObject *	io );
 
 extern APP_PASTE_REPLY( tedPastePrimaryRtf, w, event, voided );
 extern APP_PASTE_REPLY( tedPastePrimaryString, w, event, voided );
@@ -538,63 +513,62 @@ extern APP_GIVE_COPY( tedCopyPrimaryPixmap, w, event, voided );
 extern APP_GIVE_COPY( tedCopyFontTed, w, event, voided );
 extern APP_GIVE_COPY( tedCopyRulerTed, w, event, voided );
 
-extern int tedCopyAsLink(	EditDocument *		ed,
-				const BufferItem *	bi,
-				const SelectionScope *	ss,
-				const BufferSelection *	bs,
-				int			asRef,
-				int			asPageref,
-				const char *		file,
-				const char *		markName,
-				int			markSize );
+extern int tedCopyAsLink(	EditDocument *			ed,
+				const DocumentSelection *	ds,
+				int				asRef,
+				int				asPageref,
+				const char *			file,
+				const char *			markName,
+				int				markSize );
 
 extern void tedPrimaryLost(	APP_WIDGET			w,
 				void *				voided,
 				APP_EVENT *			clrEvent );
 
-extern int tedObjectInsertBitmap(	EditDocument *		ed,
+extern InsertedObject * tedObjectMakeBitmapObject(
+				    	EditDocument *		ed,
+					BufferItem *		bi,
 					AppBitmapImage *	abi );
 
 extern int tedGetObjectSelection(	TedDocument *   	td,
-					BufferPosition *	bp,
-					InsertedObject **	pIo	);
+					DocumentPosition *	dpObject,
+					InsertedObject **	pIo );
 
-extern void tedSetObjectWindows(	EditDocument *		ed,
-					const BufferPosition *	bp,
-					int			ox,
-					int			oy );
+extern void tedSetObjectWindows(EditDocument *			ed,
+				const PositionGeometry *	pg,
+				const InsertedObject *		io,
+				int				ox,
+				int				oy );
 
-extern void tedMoveObjectWindows(	APP_WIDGET		w,
-					EditDocument *		ed );
+extern void tedMoveObjectWindows(	EditDocument *		ed );
 
-extern int tedResizeObject(		EditDocument *		ed,
-					const BufferPosition *	bp );
+extern int tedResizeObject(		EditDocument *			ed,
+					const DocumentPosition *	dpObj,
+					const PositionGeometry *	pgObj );
 
 extern int tedReopenObject(	BufferDocument *	bd,
 				BufferItem *		bi,
 				TextParticule *		tp,
 				AppColors *		ac,
-				DocumentFontList *	dfl,
 				AppDrawingData *	add );
 
 extern int tedSaveObjectPicture(	AppBitmapImage *	abiTo,
 					InsertedObject *	io );
 
-extern void tedExposeSelection(	const EditDocument *	ed,
-				const BufferSelection *	bs,
-				int			scrolledX,
-				int			scrolledY );
+extern void tedExposeSelection(	const EditDocument *		ed,
+				const DocumentSelection *	ds,
+				int				scrolledX,
+				int				scrolledY );
 
-extern void tedPositionCoordinates(	BufferPosition *	bp,
-					const AppDrawingData *	add );
+extern void tedPositionGeometry(	PositionGeometry *		pg,
+					const DocumentPosition *	dp,
+					const BufferDocument *		bd,
+					const AppDrawingData *		add );
 
-extern void tedSelectionCoordinates(	BufferSelection *	bs,
-					const AppDrawingData *	add );
-
-extern void tedSelectionRectangle(	DocumentRectangle *		drClip,
-					const AppDrawingData *		add,
-					const DocumentProperties *	dp,
-					const BufferSelection *		bs );
+extern void tedSelectionGeometry(	SelectionGeometry *		sg,
+					const DocumentSelection *	ds,
+					const BufferDocument *		bd,
+					const AppDrawingData *		add );
 
 extern void tedDrawRectangle(	APP_WIDGET		w,
 				EditDocument *		ed,
@@ -629,25 +603,9 @@ extern int tedLayoutDocument(	void *				privateData,
 extern int tedMakeDocumentWidget(	EditApplication *	ea,
 					EditDocument *		ed );
 
-extern int tedChangeParticuleAttribute( AppDrawingData *	add,
-					TextParticule *		tp,
-					DocumentFontList *	dfl,
-					unsigned int		updMask,
-					TextAttribute		taNew );
-
-extern int tedInsertTable(	EditDocument *		ed,
-				int			rows,
-				int			columns );
-
-extern void tedParaScreenGeometry(	const BufferItem *		bi,
-					const DocumentProperties *	dp,
-					const AppDrawingData *		add );
-
-extern int tedInsertRowsInTable(	BufferItem *		sectBi,
-					const RowProperties *	rp,
-					EditDocument *		ed,
-					int			row,
-					int			rows );
+extern int tedInsertTable(		EditDocument *		ed,
+					int			rows,
+					int			columns );
 
 extern int tedInsertColumnInTable(	EditDocument *		ed );
 extern int tedAppendColumnToTable(	EditDocument *		ed );
@@ -670,54 +628,40 @@ extern void tedDocToolInsertSymbol(	APP_WIDGET	symbolOption,
 					void *		voided,
 					void *		voidcbs );
 
-extern int tedSplitParaContents(	BufferItem **		pNewBi,
-					BufferItem *		bi,
-					DocumentRectangle *	drChanged,
-					EditDocument *		ed,
-					int			splitLevel,
-					int			onNewPage );
-
-extern int tedEditFinishIBarSelection(	EditDocument *		ed,
-				BufferItem *			bi,
-				const DocumentRectangle *	drChanged,
-				int				oldBackY1,
-				int				stroff );
-
-extern void tedDeleteItems(		BufferDocument *	bd,
-					BufferItem *		bi,
-					int			first,
-					int			count,
-					AppDrawingData *	add );
+extern void tedDocSetTableProperties(	EditDocument *		ed,
+					const TableRectangle *	tr,
+					const PropertyMask *	rpSetMask,
+					const PropertyMask *	cpSetMask,
+					const RowProperties *	rp );
 
 extern void tedAppSetTableProperties(	EditApplication *	ea,
 					const TableRectangle *	tr,
+					const PropertyMask *	rpSetMask,
+					const PropertyMask *	cpSetMask,
 					const RowProperties *	rp );
 
-extern int tedGetTableRectangle(	TableRectangle *	tr,
-					const SelectionScope *	ss,
-					const BufferSelection *	bs );
-
 extern void tedAdaptFormatToolToDocument(	EditDocument *	ed );
 
 extern void tedAppSetTableSelection(	EditDocument *		ed,
-					const SelectionScope *	ss,
 					const TableRectangle *	tr );
 
 extern void tedDocTableSelectTableRectangle(	EditDocument *		ed,
-						const SelectionScope *	ss,
 						const TableRectangle *	tr );
 
 extern void tedChangeTableLayout(	EditDocument *		ed,
-					BufferItem *		bi,
 					BufferItem *		sectBi,
-					const RowProperties *	rp,
-					int			rowHeightAlso,
-					int			row0,
-					int			row1 );
+					int			row0Change,
+					int			row1Change,
+					int			col0Change,
+					int			col1Change,
+					int			row1Reformat,
+					const PropertyMask *	rpSetMask,
+					const PropertyMask *	cpSetMask,
+					const RowProperties *	rp );
 
 extern void tedDocTableDrawGrid(	APP_WIDGET	option,
 					void *		voided,
-					void *		voidcbs	 );
+					void *		voidcbs );
 
 extern void tedMergeParagraphsInSelection(	EditDocument *	ed );
 
@@ -731,20 +675,11 @@ extern int tedDeleteColumnsFromRows(	EditDocument *	ed,
 					int		delCol0,
 					int		delCol1 );
 
-extern void tedExposeRectangle(	const EditDocument *			ed,
-				const DocumentRectangle *	drChanged,
-				int				scrolledX,
-				int				scrolledY );
-
 extern int tedRunPropertyDialog( EditDocument *			ed,
 				const DocumentStatistics *	ds,
 				APP_WIDGET			option,
 				const char *			pixmapName );
 
-extern int tedBorderThick(		int *				pWide,
-					const BorderProperties *	bp,
-					const AppDrawingData *		add );
-
 extern int tedSetHyperlink(	EditDocument *		ed,
 				const char *		file,
 				const char *		mark,
@@ -818,73 +753,68 @@ extern int tedInsertSection(	EditApplication *	ea,
 extern int tedGetParaLineHeight(	int *			pLineHeight,
 					EditDocument *		ed );
 
-
 extern void tedShowFormatTool(	APP_WIDGET		toolOption,
 				EditApplication *	ea,
 				const char *		widgetName,
 				const char *		pixmapName );
 
 extern void tedFormatToolAdaptToSelection( AppInspector *	ai,
-				const BufferDocument *		bd,
-				const SelectionScope *		ss,
-				const BufferSelection *		bs,
+				BufferDocument *		bd,
+				const DocumentSelection *	ds,
+				const SelectionGeometry *	sg,
 				int				fileReadonly );
 
 extern int tedNewDocument(	EditApplication *	ea,
 				EditDocument *		ed,
 				const char *		filename );
 
-extern int tedPrintDocument(	FILE *				f,
-				const void *			privateData,
-				int				format,
-				AppDrawingData *		add,
-				const char *			title,
-				EditApplication *		ea,
-				const DocumentGeometry *	dgPaper,
-				int				nup,
-				int				horizontal,
+extern int tedPrintDocument(	SimpleOutputStream *		sos,
+				const PrintJob *		pj,
+				const PrintGeometry *		pg,
 				int				firstPage,
 				int				lastPage );
 
-extern int tedAdjustParagraphLayout( BufferItem *		bi,
-				int				line,
-				int				stroffShift,
-				int				upto,
-				AppDrawingData *		add,
-				const BufferDocument *		bd,
-				DocumentRectangle *		drChanged );
-
 extern void tedFreeDocument(		void *			voidtd,
 					int			format,
 					AppDrawingData *	add );
 
 extern void tedRedoDocumentLayout(	EditDocument *		ed );
 
-extern void tedParagraphFramePixels( 	FormattingFrame *	ffPixels,
-					const AppDrawingData *	add,
-					const BufferItem *	bi );
-
 extern void tedFormatShowPagePage( EditApplication *		ea );
+extern void tedFormatShowFontPage( EditApplication *		ea );
 
-extern void tedSetPageLayout(	EditDocument *			ed,
-				unsigned int			spUpdMask,
-				const DocumentGeometry *	dg );
+extern void tedSetPageLayout(	EditApplication *		ea,
+				const DocumentGeometry *	dg,
+				const PropertyMask *		setMask,
+				int				whileDocument );
 
-extern void tedSetDocumentProperties( EditApplication *		ea,
+extern int tedSetDocumentProperties(
+				EditApplication *		ea,
 				const DocumentProperties *	dpNew,
-				unsigned int			updMask );
+				const PropertyMask *		updMask );
 
-extern APP_EVENT_HANDLER( tedKeyPressed, w, voided, keyEvent );
-extern APP_EVENT_HANDLER( tedMouseButtonPressed, w, voided, downEvent );
+extern int tedChangeCurrentNote(
+				EditApplication *		ea,
+				int				extItKind );
+
+extern APP_EVENT_HANDLER_H( tedKeyPressed, w, voided, keyEvent );
+extern APP_EVENT_HANDLER_H( tedMouseButtonPressed, w, voided, downEvent );
 
 extern void tedHideObjectWindows(	EditDocument *	ed );
 
-extern int tedAppChangeSectionProperties( EditApplication *	ea,
-				unsigned int			spUpdMask,
+extern int tedAppChangeSectionProperties(
+				EditApplication *		ea,
+				const PropertyMask *		spUpdMask,
 				const SectionProperties *	spNew );
 
-extern int tedAppChangeParagraphProperties( EditApplication *		ea,
-				unsigned int			ppUpdMask,
+extern int tedAppChangeParagraphProperties(
+				EditApplication *		ea,
+				const PropertyMask *		ppUpdMask,
+				const ParagraphProperties *	ppNew );
+
+extern int tedDocChangeParagraphProperties(
+				EditDocument *			ed,
+				const PropertyMask *		ppUpdMask,
 				const ParagraphProperties *	ppNew );
 
 extern void tedAppDeleteHeaderFooter(	EditApplication *	ea,
@@ -895,18 +825,74 @@ extern void tedAppEditHeaderFooter(	EditApplication *	ea,
 					APP_WIDGET		option,
 					int			which );
 
-extern int tedGetSelection(	SelectionScope *	ss,
-				BufferSelection *	bs,
+extern int tedGetSelection(	DocumentSelection *	ds,
+				SelectionGeometry *	sg,
 				TedDocument *		td );
 
-extern int tedGetHeaderFooterBox( DocumentRectangle *		dr,
-				BufferItem *			sectBi,
-				const DocumentProperties *	dp,
-				AppDrawingData *		add,
-				int				page,
-				int				inHdFt );
-
 extern void tedFillBookmarkList(	APP_WIDGET		list,
 					DocumentFieldList *	dfl );
 
+extern int tedTextWidth(		const AppPhysicalFont *		apf,
+					const TextAttribute *		ta,
+					const BufferDocument *		bd,
+					const AppDrawingData *		add,
+					const unsigned char *		s,
+					int				len );
+
+extern int tedFindRootForPosition(	ExternalItem **		pEi,
+					BufferItem **		pRootBi,
+					BufferItem **		pBodySectBi,
+					int *			pSectNr,
+					int *			pPage,
+					EditDocument *		ed,
+					int			docX,
+					int			docY );
+
+extern void tedDetermineDefaultSettings(	TedAppResources *	tar );
+
+extern int tedReplaceSelectionWithBitmapImage(	EditDocument *		ed,
+						AppBitmapImage *	abi );
+
+extern void tedAppEditNote(		EditApplication *	ea );
+extern void tedAppGotoNoteRef(		EditApplication *	ea );
+
+extern int tedSetIBarSelection(		EditDocument *		ed,
+					BufferItem *		bi,
+					int			stroff,
+					int *			pScrolledX,
+					int *			pScrolledY );
+
+extern void tedDrawShadedRectangle(
+			AppDrawingData *	add,
+			APP_BITMAP_IMAGE	shadingPixmaps[DOCsp_COUNT],
+			int			pattern,
+			int			x0,
+			int			x1,
+			int			y0,
+			int			y1 );
+
+extern void tedDrawHorizontalBorderLine(AppDrawingData *	add,
+					int			style,
+					int			minThick,
+					int			x0,
+					int			x1,
+					int			y );
+
+extern int tedAppAllocateDocumentColor(	EditApplication *	ea,
+					const RGB8Color *	rgb8 );
+
+extern int tedDocAllocateDocumentColor(	EditDocument *		ed,
+					const RGB8Color *	rgb8 );
+
+extern void tedFontToolSet(	void *				voidea,
+				const ExpandedTextAttribute *	etaSet,
+				const PropertyMask *		taSetMask );
+
+extern int tedOpenDocumentFile(	EditApplication *	ea,
+				int *			pFormat,
+				BufferDocument **	pBd,
+				const char *		filename,
+				APP_WIDGET		relative,
+				APP_WIDGET		option );
+
 #   endif	/*  TED_APP_H	*/
