@@ -1,24 +1,24 @@
 /************************************************************************/
 /*									*/
-/*  A Format Tool.							*/
+/*  The Column page of the format tool.					*/
 /*									*/
 /************************************************************************/
 
-#   include	"config.h"
+#   include	"tedConfig.h"
 
 #   include	<stdlib.h>
 #   include	<stdio.h>
 #   include	<stddef.h>
 #   include	<limits.h>
 
-#   include	<appDebugon.h>
-
 #   include	<appGeoString.h>
 #   include	<appUnit.h>
 
 #   include	"tedApp.h"
 #   include	"tedFormatTool.h"
 
+#   include	<appDebugon.h>
+
 /************************************************************************/
 /*									*/
 /*  Refresh the 'Column' page of the format tool.			*/
@@ -34,6 +34,20 @@ static void tedFormatToolRefreshColumnPage(	ColumnTool *	ct )
     int				width;
     char			scratch[50];
 
+    EditApplication *		ea= ct->ctApplication;
+    EditDocument *		ed= ea->eaCurrentDocument;
+
+    TedDocument *		td;
+    BufferDocument *		bd;
+    DocumentProperties *	dp;
+
+    if  ( ! ed )
+	{ XDEB(ed); return;	}
+
+    td= (TedDocument *)ed->edPrivateData;
+    bd= td->tdDocument;
+    dp= &(bd->bdProperties);
+
     appIntegerToTextWidget( ct->ctNumberText, tr->trCol0+ 1 );
 
     if  ( tr->trCol0 == 0 )
@@ -51,16 +65,17 @@ static void tedFormatToolRefreshColumnPage(	ColumnTool *	ct )
 
     cp= rp->rpCells;
 
-    tedFormatReflectBorder( &(ct->ctLeftBorderTool),
+    tedBorderToolSetProperties( &(ct->ctLeftBorderTool), dp,
 					&(cp[tr->trCol0].cpLeftBorder) );
-    tedFormatReflectBorder( &(ct->ctRightBorderTool),
+    tedBorderToolSetProperties( &(ct->ctRightBorderTool), dp,
 					&(cp[tr->trCol0].cpRightBorder) );
     }
 
-void tedFormatToolRefreshColumnTool(	ColumnTool *		ct,
-					InspectorSubject *	is,
-					const TableRectangle *	tr,
-					const BufferSelection *	bs )
+void tedFormatToolRefreshColumnTool(
+				ColumnTool *			ct,
+				int *				pEnabled,
+				InspectorSubject *		is,
+				const DocumentSelection *	ds )
     {
     const RowProperties *	rp;
     const DocumentGeometry *	dg;
@@ -68,7 +83,16 @@ void tedFormatToolRefreshColumnTool(	ColumnTool *		ct,
     const BufferItem *		rowBi;
     const BufferItem *		sectBi;
 
-    rowBi= bs->bsBegin.bpBi;
+    TableRectangle *		tr= &(ct->ctTableRectangle);
+
+    if  ( docGetTableRectangle( tr, ds )	||
+	  tr->trCol1 != tr->trCol0		)
+	{
+	docInitTableRectangle( tr );
+	*pEnabled= 0; return;
+	}
+
+    rowBi= ds->dsBegin.dpBi;
     rowBi= rowBi->biParent;
     rowBi= rowBi->biParent;
     sectBi= rowBi->biParent;
@@ -81,18 +105,24 @@ void tedFormatToolRefreshColumnTool(	ColumnTool *		ct,
 
     rp= &(rowBi->biRowProperties);
 
-    if  ( docCopyRowProperties( &(ct->ctPropertiesChosen), rp ) )
+    if  ( docCopyRowProperties( &(ct->ctPropertiesChosen),
+						    rp, (const int *)0 ) )
 	{ LDEB(1); return;	}
-    if  ( docCopyRowProperties( &(ct->ctPropertiesSet), rp ) )
+    if  ( docCopyRowProperties( &(ct->ctPropertiesSet),
+						    rp, (const int *)0 ) )
 	{ LDEB(1); return;	}
 
-    ct->ctTableRectangle= *tr;
-
-    appGuiEnableWidget( is->isPrevButton, tr->trCol0 > 0 );
-    appGuiEnableWidget( is->isNextButton, tr->trCol1 < tr->trCol11 );
+    appGuiEnableWidget( is->isPrevButton,
+				ct->ctTableRectangle.trCol0 > 0 );
+    appGuiEnableWidget( is->isNextButton,
+				ct->ctTableRectangle.trCol1 <
+				ct->ctTableRectangle.trCol11 );
 
     tedFormatToolRefreshColumnPage( ct );
 
+    appGuiEnableWidget( is->isDeleteButton, rp->rpCellCount > 1 );
+
+    *pEnabled= 1;
     return;
     }
 
@@ -166,14 +196,11 @@ static int tedFormatToolGetColumnWidth(	const RowProperties *	rp,
 /*									*/
 /************************************************************************/
 
-static void tedTableChangeColumnPushed(	APP_WIDGET	w,
-					void *		voidct,
-					void *		voidpbcs	 )
+static APP_BUTTON_CALLBACK_H( tedTableChangeColumnPushed, w, voidct )
     {
     ColumnTool *		ct= (ColumnTool *)voidct;
     RowProperties *		rp= &(ct->ctPropertiesChosen);
     const TableRectangle *	tr= &(ct->ctTableRectangle);
-    CellProperties *		cp= &(rp->rpCells[tr->trCol0]);
 
     int				col;
 
@@ -183,78 +210,106 @@ static void tedTableChangeColumnPushed(	APP_WIDGET	w,
 
     TableRectangle		trChange;
 
+    BorderProperties		bp;
+
+    PropertyMask		rpSetMask;
+    PropertyMask		cpSetMask;
+    PropertyMask		bpSetMask;
+
+    /**/
+    EditApplication *		ea= ct->ctApplication;
+    EditDocument *		ed= ea->eaCurrentDocument;
+
+    TedDocument *		td;
+    BufferDocument *		bd;
+    DocumentProperties *	dp;
+
+    if  ( ! ed )
+	{ XDEB(ed); return;	}
+
+    td= (TedDocument *)ed->edPrivateData;
+    bd= td->tdDocument;
+    dp= &(bd->bdProperties);
+
+    /**/
+
+    PROPmaskCLEAR( &rpSetMask );
+    PROPmaskCLEAR( &cpSetMask );
+
+    docInitBorderProperties( &bp );
+
+    /****/
     res= tedFormatToolGetColumnWidth( rp, tr->trCol0,
 			    ct->ctPageRight, &width, &d, ct->ctWidthText );
     if  ( res != 0 )
-	{ return;	}
+	{ goto ready;	}
+    PROPmaskADD( &cpSetMask, CLpropRIGHT_BOUNDARY );
 
-    if  ( tedBorderToolGetWidth( &(ct->ctLeftBorderTool),
-						&(cp->cpLeftBorder) ) )
-	{ return;	}
+    /****/
+    PROPmaskCLEAR( &bpSetMask );
+    if  ( tedBorderToolGetProperties( &bp, &bpSetMask,
+					    &(ct->ctLeftBorderTool), dp ) )
+	{ goto ready;	}
 
-    if  ( tedBorderToolGetWidth( &(ct->ctRightBorderTool),
-						&(cp->cpRightBorder) ) )
-	{ return;	}
+    if  ( ! PROPmaskISEMPTY( &bpSetMask ) )
+	{
+	docRowSetLeftBorderInCols( rp, tr->trCol0, tr->trCol1,
+							    &bpSetMask, &bp );
+	PROPmaskADD( &cpSetMask, CLpropLEFT_BORDER );
+	}
 
+    /****/
+    PROPmaskCLEAR( &bpSetMask );
+    if  ( tedBorderToolGetProperties( &bp, &bpSetMask,
+					    &(ct->ctRightBorderTool), dp ) )
+	{ goto ready;	}
+
+    if  ( ! PROPmaskISEMPTY( &bpSetMask ) )
+	{
+	docRowSetRightBorderInCols( rp, tr->trCol0, tr->trCol1,
+							    &bpSetMask, &bp );
+	PROPmaskADD( &cpSetMask, CLpropRIGHT_BORDER );
+	}
+
+    {
+    CellProperties *		cp;
+
+    cp= &(rp->rpCells[tr->trCol0]);
     for ( col= tr->trCol0; col < rp->rpCellCount; cp++, col++ )
 	{ cp->cpRightBoundaryTwips += d; }
+    }
 
     trChange= *tr;
     docExpandTableRectangleToWholeColumns( &trChange );
 
-    tedAppSetTableProperties( ct->ctApplication, &trChange, rp );
+    tedAppSetTableProperties( ct->ctApplication, &trChange,
+						&rpSetMask, &cpSetMask, rp );
+
+  ready:
+    docCleanBorderProperties( &bp );
 
     return;
     }
 
-static void tedFormatRevertColumnPushed(	APP_WIDGET	w,
-						void *		voidct,
-						void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( tedFormatRevertColumnPushed, w, voidct )
     {
     ColumnTool *	ct= (ColumnTool *)voidct;
 
-    docCopyRowProperties( &(ct->ctPropertiesChosen), &(ct->ctPropertiesSet) );
+    docCopyRowProperties( &(ct->ctPropertiesChosen),
+				    &(ct->ctPropertiesSet), (const int *)0 );
 
     tedFormatToolRefreshColumnPage( ct );
 
     return;
     }
 
-/************************************************************************/
-/*									*/
-/*  A border style has been chosen.					*/
-/*									*/
-/************************************************************************/
-
-static void tedColumnBorderStyleChosen(	APP_WIDGET	w,
-					void *		voidct,
-					void *		voidtbcs )
-    {
-    ColumnTool *		ct= (ColumnTool *)voidct;
-    RowProperties *		rp= &(ct->ctPropertiesChosen);
-    const TableRectangle *	tr= &(ct->ctTableRectangle);
-    CellProperties *		cp= &(rp->rpCells[tr->trCol0]);
-
-    if  ( ! tedBorderToolGetStyle( &(ct->ctLeftBorderTool),
-						&(cp->cpLeftBorder), w ) )
-	{ return;	}
-
-    if  ( ! tedBorderToolGetStyle( &(ct->ctRightBorderTool),
-						&(cp->cpRightBorder), w ) )
-	{ return;	}
-
-    XDEB(w); return;
-    }
-
 /************************************************************************/
 /*									*/
 /*  Check a number as typed by the user.				*/
 /*									*/
 /************************************************************************/
 
-static void tedColumnWidthChanged(	APP_WIDGET	w,
-					void *		voidct,
-					void *		voidpbcs )
+static APP_TXACTIVATE_CALLBACK_H( tedColumnWidthChanged, w, voidct )
     {
     ColumnTool *		ct= (ColumnTool *)voidct;
     RowProperties *		rp= &(ct->ctPropertiesChosen);
@@ -280,9 +335,7 @@ static void tedColumnWidthChanged(	APP_WIDGET	w,
 /*									*/
 /************************************************************************/
 
-static void tedColumnPreviousColumn(	APP_WIDGET	w,
-					void *		voidct,
-					void *		voidpbcs	 )
+static APP_BUTTON_CALLBACK_H( tedColumnPreviousColumn, w, voidct )
     {
     ColumnTool *	ct= (ColumnTool *)voidct;
     EditApplication *	ea= ct->ctApplication;
@@ -302,14 +355,12 @@ static void tedColumnPreviousColumn(	APP_WIDGET	w,
     if  ( docShiftTableRectangleByColumns( &tr, -1 ) )
 	{ return;	}
 
-    tedAppSetTableSelection( ed, &(td->tdSelectionScope), &tr );
+    tedAppSetTableSelection( ed, &tr );
 
     return;
     }
 
-static void tedTableSelectColumn(	APP_WIDGET	w,
-					void *		voidct,
-					void *		voidpbcs	 )
+static APP_BUTTON_CALLBACK_H( tedTableSelectColumn, w, voidct )
     {
     ColumnTool *	ct= (ColumnTool *)voidct;
     EditApplication *	ea= ct->ctApplication;
@@ -326,14 +377,12 @@ static void tedTableSelectColumn(	APP_WIDGET	w,
 
     docExpandTableRectangleToWholeColumns( &tr );
 
-    tedAppSetTableSelection( ed, &(td->tdSelectionScope), &tr );
+    tedAppSetTableSelection( ed, &tr );
 
     return;
     }
 
-static void tedColumnNextColumn(		APP_WIDGET	w,
-					void *		voidct,
-					void *		voidpbcs	 )
+static APP_BUTTON_CALLBACK_H( tedColumnNextColumn, w, voidct )
     {
     ColumnTool *	ct= (ColumnTool *)voidct;
     EditApplication *	ea= ct->ctApplication;
@@ -353,7 +402,7 @@ static void tedColumnNextColumn(		APP_WIDGET	w,
     if  ( docShiftTableRectangleByColumns( &tr, +1 ) )
 	{ return;	}
 
-    tedAppSetTableSelection( ed, &(td->tdSelectionScope), &tr );
+    tedAppSetTableSelection( ed, &tr );
 
     return;
     }
@@ -364,9 +413,7 @@ static void tedColumnNextColumn(		APP_WIDGET	w,
 /*									*/
 /************************************************************************/
 
-static void tedTableDeleteColumn(	APP_WIDGET	w,
-					void *		voidct,
-					void *		voidpbcs	 )
+static APP_BUTTON_CALLBACK_H( tedTableDeleteColumn, w, voidct )
     {
     ColumnTool *		ct= (ColumnTool *)voidct;
     const TableRectangle *	tr= &(ct->ctTableRectangle);
@@ -391,9 +438,7 @@ static void tedTableDeleteColumn(	APP_WIDGET	w,
 /*									*/
 /************************************************************************/
 
-static void tedTableInsertColumn(	APP_WIDGET	w,
-					void *		voidct,
-					void *		voidpbcs	 )
+static APP_BUTTON_CALLBACK_H( tedTableInsertColumn, w, voidct )
     {
     ColumnTool *	ct= (ColumnTool *)voidct;
     EditApplication *	ea= ct->ctApplication;
@@ -408,9 +453,7 @@ static void tedTableInsertColumn(	APP_WIDGET	w,
     appDocumentChanged( ed, 1 );
     }
 
-static void tedTableAppendColumn(	APP_WIDGET	w,
-					void *		voidct,
-					void *		voidpbcs	 )
+static APP_BUTTON_CALLBACK_H( tedTableAppendColumn, w, voidct )
     {
     ColumnTool *	ct= (ColumnTool *)voidct;
     EditApplication *	ea= ct->ctApplication;
@@ -427,40 +470,29 @@ static void tedTableAppendColumn(	APP_WIDGET	w,
 
 /************************************************************************/
 /*									*/
-/*  One of the 'Column Border' toggles has been activated.		*/
+/*  Callback from the general color chooser: a color was chosen.	*/
 /*									*/
 /************************************************************************/
 
-static void tedColumnToggleBorder(	APP_WIDGET	w,
-					void *		voidct,
-					void *		voidtbcs )
+static void tedColumnToolGotColor(	void *			voidct,
+					int 			which,
+					const RGB8Color *	rgb8 )
     {
-    ColumnTool *		ct= (ColumnTool *)voidct;
-    RowProperties *		rp= &(ct->ctPropertiesChosen);
-    const TableRectangle *	tr= &(ct->ctTableRectangle);
-    CellProperties *		cp= &(rp->rpCells[tr->trCol0]);
+    ColumnTool *			ct= (ColumnTool *)voidct;
 
-    int				set;
-
-    const int			influenceStyleMenu= 1;
-
-    set= appGuiGetToggleStateFromCallback( w, voidtbcs );
-
-    if  ( w == ct->ctLeftBorderTool.btOnOffToggle )
+    switch( which )
 	{
-	tedFormatToggleBorder( &(cp->cpLeftBorder), &(ct->ctLeftBorderTool),
-						    influenceStyleMenu, set );
-	return;
-	}
+	case CLpropLEFT_BORDER:
+	    tedBorderSetExplicitColorChoice( &(ct->ctLeftBorderTool), rgb8 );
+	    break;
 
-    if  ( w == ct->ctRightBorderTool.btOnOffToggle )
-	{
-	tedFormatToggleBorder( &(cp->cpRightBorder), &(ct->ctRightBorderTool),
-						    influenceStyleMenu, set );
-	return;
-	}
+	case CLpropRIGHT_BORDER:
+	    tedBorderSetExplicitColorChoice( &(ct->ctRightBorderTool), rgb8 );
+	    break;
 
-    XDEB(w); return;
+	default:
+	    LDEB(which); return;
+	}
     }
 
 /************************************************************************/
@@ -471,6 +503,8 @@ static void tedColumnToggleBorder(	APP_WIDGET	w,
 
 void tedFormatFillColumnPage(	ColumnTool *			ct,
 				const ColumnPageResources *	cpr,
+				AppInspector *			ai,
+				int				subjectPage,
 				InspectorSubject *		is,
 				APP_WIDGET			pageWidget,
 				const InspectorSubjectResources * isr )
@@ -482,13 +516,13 @@ void tedFormatFillColumnPage(	ColumnTool *			ct,
 
     const int	textColumns= 10;
 
-    /**************/
+    /**/
+    ct->ctPageResources= cpr;
 
-    docInitTableRectangle( &(ct->ctTableRectangle) );
-
-    docInitRowProperties( &(ct->ctPropertiesSet) );
-    docInitRowProperties( &(ct->ctPropertiesChosen) );
+    is->isPrivate= ct;
+    is->isGotColor= tedColumnToolGotColor;
 
+    /**/
     appMakeLabelAndTextRow( &row, &columnLabel, &(ct->ctNumberText),
 			    pageWidget, isr->isrSubjectName, textColumns, 0 );
 
@@ -498,11 +532,13 @@ void tedFormatFillColumnPage(	ColumnTool *			ct,
     appGuiSetGotValueCallbackForText( ct->ctWidthText,
 					tedColumnWidthChanged, (void *)ct );
 
-    tedFormatMakeBorderTool( &(ct->ctLeftBorderTool), &row, pageWidget,
-		    cpr->cprLeftBorder, tedColumnToggleBorder, (void *)ct );
+    tedMakeBorderTool( &(ct->ctLeftBorderTool), ai, pageWidget,
+		cpr->cprLeftBorder, &(cpr->cprBorderToolResources),
+		subjectPage, CLpropLEFT_BORDER );
 
-    tedFormatMakeBorderTool( &(ct->ctRightBorderTool), &row, pageWidget,
-		    cpr->cprRightBorder, tedColumnToggleBorder, (void *)ct );
+    tedMakeBorderTool( &(ct->ctRightBorderTool), ai, pageWidget,
+		cpr->cprRightBorder, &(cpr->cprBorderToolResources),
+		subjectPage, CLpropRIGHT_BORDER );
 
     appInspectorMakeButtonRow( &row, pageWidget,
 		    &(is->isPrevButton), &(is->isNextButton),
@@ -531,10 +567,6 @@ void tedFormatFillColumnPage(	ColumnTool *			ct,
 void tedFormatFillColumnChoosers(	ColumnTool *			ct,
 					const ColumnPageResources *	cpr )
     {
-    tedFormatFillBorderTool( &(ct->ctLeftBorderTool),
-				    tedColumnBorderStyleChosen, (void *)ct );
-    tedFormatFillBorderTool( &(ct->ctRightBorderTool),
-				    tedColumnBorderStyleChosen, (void *)ct );
     return;
     }
 
@@ -542,8 +574,48 @@ void tedFormatFinishColumnPage(		ColumnTool *			ct,
 					TedFormatTool *			tft,
 					const ColumnPageResources *	cpr )
     {
-    tedFormatFinishBorderTool( &(ct->ctLeftBorderTool), tft );
-    tedFormatFinishBorderTool( &(ct->ctRightBorderTool), tft );
+    tedFinishBorderTool( &(ct->ctLeftBorderTool) );
+    tedFinishBorderTool( &(ct->ctRightBorderTool) );
+    }
+
+/************************************************************************/
+/*									*/
+/*  Clean a column tool							*/
+/*									*/
+/************************************************************************/
+
+void tedInitColumnTool(	ColumnTool *	ct )
+    {
+    ct->ctApplication= (EditApplication *)0;
+    ct->ctInspector= (AppInspector *)0;
+    ct->ctPageResources= (const ColumnPageResources *)0;
+
+    docInitTableRectangle( &(ct->ctTableRectangle) );
+
+    ct->ctPageRight= 0;
+    ct->ctPageLeftMargin= 0;
+
+    docInitRowProperties( &(ct->ctPropertiesSet) );
+    docInitRowProperties( &(ct->ctPropertiesChosen) );
+
+    ct->ctNumberText= (APP_WIDGET)0;
+    ct->ctWidthText= (APP_WIDGET)0;
+
+    tedInitBorderTool( &(ct->ctLeftBorderTool) );
+    tedInitBorderTool( &(ct->ctRightBorderTool) );
+
+    return;
+    }
+
+void tedCleanColumnTool(	ColumnTool *	ct )
+    {
+    docCleanRowProperties( &(ct->ctPropertiesSet) );
+    docCleanRowProperties( &(ct->ctPropertiesChosen) );
+
+    tedCleanBorderTool( &(ct->ctLeftBorderTool) );
+    tedCleanBorderTool( &(ct->ctRightBorderTool) );
+
+    return;
     }
 
 /************************************************************************/
@@ -588,12 +660,33 @@ static AppConfigurableResource TED_TedColumnToolResourceTable[]=
     APP_RESOURCE( "tableToolWidth",
 		offsetof(ColumnPageResources,cprWidth),
 		"Width" ),
+
+    /**/
     APP_RESOURCE( "tableToolColumnLeftBorder",
 		offsetof(ColumnPageResources,cprLeftBorder),
 		"Left Border" ),
     APP_RESOURCE( "tableToolColumnRightBorder",
 		offsetof(ColumnPageResources,cprRightBorder),
 		"Right Border" ),
+
+    /**/
+    APP_RESOURCE( "formatToolColumnBorderWidth",
+	    offsetof(ColumnPageResources, cprBorderToolResources.btrWidth),
+	    "Width" ),
+    APP_RESOURCE( "formatToolColumnBorderStyle",
+	    offsetof(ColumnPageResources, cprBorderToolResources.btrStyle),
+	    "Style" ),
+    APP_RESOURCE( "formatToolColumnBorderColor",
+	    offsetof(ColumnPageResources, cprBorderToolResources.btrColor),
+	    "Color" ),
+    APP_RESOURCE( "formatToolColumnBorderColorChooserAutomatic",
+	    offsetof(ColumnPageResources,cprBorderToolResources.
+			    btrColorChooserResources.ccrAutomaticColor),
+	    "Automatic" ),
+    APP_RESOURCE( "formatToolColumnBorderColorChooserMoreColors",
+	    offsetof(ColumnPageResources,cprBorderToolResources.
+			    btrColorChooserResources.ccrMoreColors),
+	    "More Colors..." ),
     };
 
 void tedFormatToolGetColumnResourceTable(EditApplication *		ea,
