@@ -1,16 +1,20 @@
 /************************************************************************/
-/*  Ted, main module.							*/
+/*									*/
+/*  Ted: Copy/Paste related functionality.				*/
+/*									*/
 /************************************************************************/
 
-#   include	"config.h"
+#   include	"tedConfig.h"
 
 #   include	<stddef.h>
 #   include	<stdlib.h>
 #   include	<stdio.h>
+#   include	<string.h>
 #   include	<ctype.h>
 
 #   include	"tedApp.h"
 
+#   include	<sioMemory.h>
 #   include	<sioXprop.h>
 
 #   include	<appDebugon.h>
@@ -26,16 +30,22 @@
 /*									*/
 /************************************************************************/
 
-static void tedFinishTextCopy(	TedDocument *		td,
-				const BufferSelection *	bs )
+static void tedFinishTextCopy(	TedDocument *			td,
+				AppDrawingData *		add,
+				const DocumentSelection *	ds )
     {
+    BufferDocument *		bd= td->tdDocument;
+    SelectionGeometry		sg;
+
+    tedSelectionGeometry( &sg, ds, bd, add );
+
     td->tdCopiedSelectionClosed= 1;
 
-    if  ( (unsigned)bs->bsEnd.bpStroff < bs->bsEnd.bpBi->biParaStrlen )
+    if  ( (unsigned)ds->dsEnd.dpStroff < ds->dsEnd.dpBi->biParaStrlen )
 	{ td->tdCopiedSelectionClosed= 0;	}
 
-    if  ( bs->bsBegin.bpBi == bs->bsEnd.bpBi		&&
-	  bs->bsBegin.bpLine == bs->bsEnd.bpLine	)
+    if  ( docIsParaSelection( ds )		&&
+	  sg.sgBegin.pgLine == sg.sgEnd.pgLine	)
 	{ td->tdCopiedSelectionClosed= 0;	}
 
     appCleanBitmapImage( &(td->tdCopiedImage) );
@@ -48,22 +58,25 @@ static int tedDocCopySelection(	EditDocument *	ed )
     {
     EditApplication *		ea= ed->edApplication;
     TedDocument *		td= (TedDocument *)ed->edPrivateData;
-    SelectionScope *		ss= &(td->tdSelectionScope);
     BufferDocument *		bd= td->tdDocument;
+    AppDrawingData *		add= &(ed->edDrawingData);
 
     BufferItem *		bi;
     TextParticule *		tp;
-    BufferPosition		bpObject;
+    DocumentPosition		dpObject;
     InsertedObject *		io;
 
     SimpleOutputStream *	sos;
 
-    BufferSelection		bsNew= td->tdSelection;
+    DocumentSelection		dsNew;
     const int			saveBookmarks= 0;
 
     int				startPart;
     int				endPart;
 
+    DocumentSelection		ds;
+    SelectionGeometry		sg;
+
     const char *		fileName= (const char *)0;
     int				fileSize= 0;
     const char *		markName= (const char *)0;
@@ -72,25 +85,30 @@ static int tedDocCopySelection(	EditDocument *	ed )
     if  ( ! tedHasSelection( td ) || tedHasIBarSelection( td ) )
 	{ return -1; }
 
+    if  ( tedGetSelection( &ds, &sg, td ) )
+	{ return -1;	}
+
+    dsNew= ds;
+
     /*  2  */
-    if  ( ! docGetHyperlinkForPosition( bd, &(bsNew.bsBegin),
+    if  ( ! docGetHyperlinkForPosition( bd, &(dsNew.dsBegin),
 				&startPart, &endPart,
 				&fileName, &fileSize, &markName, &markSize ) )
 	{
-	bi= bsNew.bsBegin.bpBi;
-	tp= bi->biParaParticules+ bsNew.bsBegin.bpParticule;
+	bi= dsNew.dsBegin.dpBi;
+	tp= bi->biParaParticules+ dsNew.dsBegin.dpParticule;
 
-	if  ( startPart < bsNew.bsBegin.bpParticule			&&
+	if  ( startPart < dsNew.dsBegin.dpParticule			&&
 	      bi->biParaParticules[startPart].tpStroff == tp->tpStroff	)
-	    { bsNew.bsBegin.bpParticule= startPart;	}
+	    { dsNew.dsBegin.dpParticule= startPart;	}
 
-	if  ( bsNew.bsEnd.bpBi == bi )
+	if  ( dsNew.dsEnd.dpBi == bi )
 	    {
-	    tp= bi->biParaParticules+ bsNew.bsEnd.bpParticule;
+	    tp= bi->biParaParticules+ dsNew.dsEnd.dpParticule;
 
-	    if  ( endPart > bsNew.bsEnd.bpParticule			&&
+	    if  ( endPart > dsNew.dsEnd.dpParticule			&&
 		  bi->biParaParticules[endPart].tpStroff == tp->tpStroff )
-		{ bsNew.bsEnd.bpParticule= endPart;	}
+		{ dsNew.dsEnd.dpParticule= endPart;	}
 	    }
 	}
 
@@ -98,18 +116,18 @@ static int tedDocCopySelection(	EditDocument *	ed )
     if  ( ! sos )
 	{ XDEB(sos); return -1;    }
 
-    if  ( docRtfSaveDocument( sos, td->tdDocument,
-					    ss, &bsNew, saveBookmarks ) )
+    if  ( docRtfSaveDocument( sos, td->tdDocument, &dsNew, saveBookmarks ) )
 	{ LDEB(1); sioOutClose( sos ); return -1;	}
 
     if  ( sioOutClose( sos ) )
 	{ LDEB(1); return -1;	}
 
-    tedFinishTextCopy( td, &bsNew );
+    tedFinishTextCopy( td, add, &dsNew );
 
-    docInitPosition( &bpObject );
+    docInitDocumentPosition( &dpObject );
 
-    if  ( ! tedGetObjectSelection( td, &bpObject, &io )		&&
+    if  ( ea->eaSupportXvCopyPaste				&&
+	  ! tedGetObjectSelection( td, &dpObject, &io )		&&
 	  ! tedSaveObjectPicture( &(td->tdCopiedImage), io )	)
 	{
 	/*  8  */
@@ -132,9 +150,7 @@ static int tedDocCopySelection(	EditDocument *	ed )
 /************************************************************************/
 
 int tedCopyAsLink(	EditDocument *			ed,
-			const BufferItem *		bi,
-			const SelectionScope *		ss,
-			const BufferSelection *		bs,
+			const DocumentSelection *	ds,
 			int				asRef,
 			int				asPageref,
 			const char *			fileName,
@@ -143,6 +159,7 @@ int tedCopyAsLink(	EditDocument *			ed,
     {
     SimpleOutputStream *	sos;
     TedDocument *		td= (TedDocument *)ed->edPrivateData;
+    AppDrawingData *		add= &(ed->edDrawingData);
 
     int				fileSize= 0;
 
@@ -153,7 +170,7 @@ int tedCopyAsLink(	EditDocument *			ed,
     if  ( ! sos )
 	{ XDEB(sos); return -1;    }
 
-    if  ( docRtfSaveSelectionAsLink( sos, td->tdDocument, ss, bs,
+    if  ( docRtfSaveSelectionAsLink( sos, td->tdDocument, ds,
 				asRef, asPageref,
 				(const unsigned char *)fileName, fileSize,
 				(const unsigned char *)markName, markSize ) )
@@ -162,7 +179,7 @@ int tedCopyAsLink(	EditDocument *			ed,
     if  ( sioOutClose( sos ) )
 	{ LDEB(1); return -1;	}
 
-    tedFinishTextCopy( td, bs );
+    tedFinishTextCopy( td, add, ds );
 
     if  ( appDocOwnSelection( ed, "PRIMARY",
 		    TedPrimaryTextTargets, TedPrimaryTextTargetCount ) )
@@ -175,13 +192,29 @@ void tedDocCut(			EditDocument *	ed )
     {
     TedDocument *		td= (TedDocument *)ed->edPrivateData;
 
-    if  ( ! td->tdCanReplaceSelection )
+    int				isTableSlice= 0;
+    int				isRowSlice= 0;
+    int				isColSlice= 0;
+    TableRectangle		tr;
+
+    DocumentSelection		ds;
+    SelectionGeometry		sg;
+
+    if  ( tedGetSelection( &ds, &sg, td ) )
+	{ return;	}
+
+    if  ( ! docGetTableSliceSelection( &isRowSlice, &isColSlice, &tr, &ds ) )
+	{ isTableSlice= 1;	}
+
+    if  ( ! isTableSlice && ! td->tdCanReplaceSelection )
 	{ LDEB(td->tdCanReplaceSelection); return;	}
 
     if  ( tedDocCopySelection( ed ) )
 	{ LDEB(1); return;	}
 
-    tedAppReplaceSelection( ed, (const unsigned char *)0, 0 );
+    if  ( isTableSlice )
+	{ tedDeleteTableSliceSelection( ed );				}
+    else{ tedDocReplaceSelection( ed, (const unsigned char *)0, 0 );	}
 
     return;
     }
@@ -190,12 +223,21 @@ void tedDocCopy(		EditDocument *	ed )
     {
     TedDocument *		td= (TedDocument *)ed->edPrivateData;
 
+    DocumentSelection		ds;
+    SelectionGeometry		sg;
+
+    const int			scrolledX= 0;
+    const int			scrolledY= 0;
+
+    if  ( tedGetSelection( &ds, &sg, td ) )
+	{ return;	}
+
     if  ( tedDocCopySelection( ed ) )
 	{ return;	}
 
     td->tdVisibleSelectionCopied= 1;
 
-    tedExposeSelection( ed, &(td->tdSelection), /*scrolledX,Y*/ 0,0 );
+    appDocExposeRectangle( ed, &(sg.sgRectangle), scrolledX, scrolledY );
 
     return;
     }
@@ -210,14 +252,23 @@ void tedPrimaryLost(	APP_WIDGET			w,
 			void *				voided,
 			APP_EVENT *			event )
     {
-    EditDocument *	ed= (EditDocument *)voided;
-    TedDocument *	td= (TedDocument *)ed->edPrivateData;
+    EditDocument *		ed= (EditDocument *)voided;
+    TedDocument *		td= (TedDocument *)ed->edPrivateData;
+
+    DocumentSelection		ds;
+    SelectionGeometry		sg;
+
+    if  ( tedGetSelection( &ds, &sg, td ) )
+	{ return;	}
 
     td->tdVisibleSelectionCopied= 0;
 
     if  ( tedHasSelection( td ) && ! tedHasIBarSelection( td ) )
 	{
-	tedExposeSelection( ed, &(td->tdSelection), /*scrolledX,Y*/ 0,0 );
+	const int	scrolledX= 0;
+	const int	scrolledY= 0;
+
+	appDocExposeRectangle( ed, &(sg.sgRectangle), scrolledX, scrolledY );
 	}
     }
 
@@ -243,16 +294,21 @@ APP_GIVE_COPY( tedCopyPrimaryRtf, w, event, voided )
 
     if  ( sioOutWriteBytes( sos, td->tdCopiedSelection.mbBytes,
 				    td->tdCopiedSelection.mbSize ) <= 0 )
-	{ LDEB(1); sioOutClose( sos ); return;	}
+	{ LDEB(td->tdCopiedSelection.mbSize); sioOutClose( sos ); return; }
 
     sioOutClose( sos );
 
 #   if 0
     sos= sioOutStdioOpen( "/tmp/copied.rtf" );
-    if  ( sioOutWriteBytes( sos, td->tdCopiedSelection.mbBytes,
+    XDEB(sos);
+    if  ( sos )
+	{
+	if  ( sioOutWriteBytes( sos, td->tdCopiedSelection.mbBytes,
 				    td->tdCopiedSelection.mbSize ) <= 0 )
-	{ LDEB(1); sioOutClose( sos ); return;	}
-    sioOutClose( sos );
+	    { LDEB(td->tdCopiedSelection.mbSize); }
+
+	sioOutClose( sos );
+	}
 #   endif
 
     return;
@@ -280,7 +336,7 @@ APP_GIVE_COPY( tedCopyPrimaryString, w, event, voided )
     if  ( ! sis )
 	{ XDEB(sis); sioOutClose( sos ); return;	}
 
-    bd= docRtfReadFile( sis, td->tdDocument->bdProperties.dpAnsiCodePage );
+    bd= docRtfReadFile( sis, td->tdDocument->bdProperties.dpAnsiCodepage );
 
     sioInClose( sis );
 
@@ -288,7 +344,7 @@ APP_GIVE_COPY( tedCopyPrimaryString, w, event, voided )
 	{ XDEB(bd); sioOutClose( sos ); return; }
 
     res= docPlainSaveDocument( sos, bd,
-				    (SelectionScope *)0, (BufferSelection *)0,
+				    (DocumentSelection *)0,
 				    0, td->tdCopiedSelectionClosed );
 
     docFreeDocument( bd );
@@ -400,26 +456,31 @@ APP_GIVE_COPY( tedCopyRulerTed, w, event, voided )
 /*  Finish a paste action.						*/
 /*									*/
 /************************************************************************/
-typedef BufferDocument * (*TED_READ) (	SimpleInputStream *		sis,
-					int				defCpg,
-					const DocumentGeometry *	dg );
+
+typedef BufferDocument * (*TED_READ) (
+				SimpleInputStream *		sis,
+				int				ansiCodepage,
+				const DocumentGeometry *	dg );
 
 typedef int	(*TED_INCLUDE) (	APP_WIDGET		w,
 					EditDocument *		ed,
 					BufferDocument *	bd );
 
-static BufferDocument * tedPasteRtfReadFile( SimpleInputStream *	sis,
-					int				defCpg,
-					const DocumentGeometry *	dg )
-    { return docRtfReadFile( sis, defCpg );	}
+static BufferDocument * tedPasteRtfReadFile(
+				SimpleInputStream *		sis,
+				int				ansiCodepage,
+				const DocumentGeometry *	dg )
+    { return docRtfReadFile( sis, ansiCodepage );	}
 
-static BufferDocument * tedPastePlainReadFile( SimpleInputStream *	sis,
-					int				defCpg,
-					const DocumentGeometry *	dg )
+static BufferDocument * tedPastePlainReadFile(
+				SimpleInputStream *		sis,
+				int				ansiCodepage,
+				const DocumentGeometry *	dg )
     {
     int		longestPara;
 
-    return docPlainReadFile( sis, &longestPara, dg );
+    return docPlainReadFile( sis, &longestPara,
+					DOCcharsetANSI, ansiCodepage, dg );
     }
 
 static void tedPastePrimaryGeneric(	APP_WIDGET		w,
@@ -436,23 +497,7 @@ static void tedPastePrimaryGeneric(	APP_WIDGET		w,
 
     TedAppResources *		tar= (TedAppResources *)ea->eaResourceData;
 
-    if  ( tar->tarDefaultAnsicpgInt < 0		&&
-	  tar->tarDefaultAnsicpgString		)
-	{
-	char *		past;
-	int		val;
-
-	val= strtol( tar->tarDefaultAnsicpgString, &past, 10 );
-	if  ( past != tar->tarDefaultAnsicpgString )
-	    {
-	    while( *past == ' ' )
-		{ past++;	}
-	    }
-	if  ( past != tar->tarDefaultAnsicpgString	&&
-	      ! *past					)
-	    { tar->tarDefaultAnsicpgInt= val;		}
-	else{ SDEB(tar->tarDefaultAnsicpgString);	}
-	}
+    tedDetermineDefaultSettings( tar );
 
     sis= sioInOpenPaste( w, event );
     if  ( ! sis )
@@ -513,16 +558,14 @@ APP_PASTE_REPLY( tedPastePrimaryPng, w, event, voided )
     if  ( ! sis )
 	{ XDEB(sis); return;	}
 
-    if  ( bmPngReadPng( &abi->abiBitmap, &abi->abiBuffer, sis ) )
+    if  ( bmPngReadPng( &(abi->abiBitmap), &(abi->abiBuffer), sis ) )
 	{ LDEB(1); return;	}
 
-    tedAppReplaceSelection( ed, (const unsigned char *)0, 0 );
-
-    if  ( tedObjectInsertBitmap( ed, abi ) )
-	{ LDEB(1); sioInClose( sis ); return; }
-
     sioInClose( sis );
 
+    if  ( tedReplaceSelectionWithBitmapImage( ed, abi ) )
+	{ LDEB(1); return; }
+
     return;
     }
 
@@ -544,10 +587,8 @@ APP_PASTE_REPLY( tedPastePrimaryPixmap, w, event, voided )
     if  ( appImgPastePixmap( abi, w, event ) )
 	{ LDEB(1); return;	}
 
-    tedAppReplaceSelection( ed, (const unsigned char *)0, 0 );
-
-    if  ( tedObjectInsertBitmap( ed, abi ) )
-	{ appCleanBitmapImage( abi ); free( abi ); return; }
+    if  ( tedReplaceSelectionWithBitmapImage( ed, abi ) )
+	{ LDEB(1); return; }
 
     return;
     }
@@ -594,15 +635,17 @@ APP_PASTE_REPLY( tedPasteRulerTed, w, event, voided )
     SimpleInputStream *		sis;
 
     ParagraphProperties		ppNew;
-    TextAttribute		taNew;
+    TextAttribute		taSet;
 
-    unsigned int		ppUpdMask;
-    unsigned int		spUpdMask;
+    PropertyMask		taSetMask;
+    PropertyMask		ppUpdMask;
+    PropertyMask		spUpdMask;
 
+    PROPmaskCLEAR( &taSetMask );
     PROPmaskCLEAR( &spUpdMask );
 
     docInitParagraphProperties( &ppNew );
-    docInitTextAttribute( &taNew );
+    utilInitTextAttribute( &taSet );
 
     sis= sioInOpenPaste( w, event );
     if  ( ! sis )
@@ -621,12 +664,71 @@ APP_PASTE_REPLY( tedPasteRulerTed, w, event, voided )
     PROPmaskADD( &ppUpdMask, PPpropTAB_STOPS );
 
     if  ( tedChangeSelectionProperties( ed,
-				    TAupd_NONE, taNew,
-				    ppUpdMask, &ppNew,
-				    spUpdMask, (SectionProperties *)0 ) )
+				    &taSetMask, &taSet,
+				    &ppUpdMask, &ppNew,
+				    &spUpdMask, (SectionProperties *)0 ) )
 	{ LDEB(1); docCleanParagraphProperties( &ppNew ); return; }
 
     docCleanParagraphProperties( &ppNew );
 
     return;
     }
+
+/************************************************************************/
+/*									*/
+/*  Custom 'Paste' functionality.					*/
+/*									*/
+/*  1)  The regular X11 manner.						*/
+/*  2)  Try to get a possible 'paste' from xv. This is just to make it	*/
+/*	possible to cooperate with John Bradley's XV. As XV is a little	*/
+/*	outdated, only support this if the appropriate resource is set.	*/
+/*									*/
+/************************************************************************/
+
+void tedDocEditPaste(		APP_WIDGET	option,
+				void *		voided,
+				void *		voidpbcs )
+    {
+    EditDocument *			ed= (EditDocument *)voided;
+    EditApplication *			ea= ed->edApplication;
+    TedDocument *			td= (TedDocument *)ed->edPrivateData;
+
+    int					res;
+    BitmapDescription			bd;
+    unsigned char *			buffer;
+
+    if  ( ! td->tdCanReplaceSelection )
+	{ LDEB(td->tdCanReplaceSelection); return;	}
+
+    /*  1  */
+    res= appDocAskForPaste( ed, "PRIMARY" );
+    if  ( ! res )
+	{ return;	}
+
+    /*  2  */
+    if  ( ea->eaSupportXvCopyPaste					&&
+	  ! appImgGetXvClipboard( &bd, &buffer,
+					ea->eaToplevel.atTopWidget )	)
+	{
+	AppBitmapImage *	abi;
+
+	abi= (AppBitmapImage *)malloc( sizeof(AppBitmapImage) );
+	if  ( ! abi )
+	    { XDEB(abi); return;	}
+	appInitBitmapImage( abi );
+
+	if  ( bmCopyDescription( &(abi->abiBitmap), &bd ) )
+	    { bmCleanDescription( &bd ); return; }
+
+	bmCleanDescription( &bd );
+	abi->abiBuffer= buffer;
+
+	if  ( tedReplaceSelectionWithBitmapImage( ed, abi ) )
+	    { LDEB(1); return; }
+
+	return;
+	}
+
+    return;
+    }
+
