@@ -4,14 +4,16 @@
 /*									*/
 /************************************************************************/
 
-#   include	"config.h"
+#   include	"tedConfig.h"
 
 #   include	<stddef.h>
 #   include	<stdlib.h>
 #   include	<stdio.h>
 #   include	<ctype.h>
 
-#   include	"docPs.h"
+#   include	<sioMemory.h>
+
+#   include	"tedLayout.h"
 
 #   include	"tedApp.h"
 #   include	"tedRuler.h"
@@ -163,12 +165,26 @@ static void tedUpdateTableColumns(	EditDocument *		ed,
 
     if  ( changed )
 	{
-	int		rowPropertiesAlso= 0;
+	PropertyMask	rpSetMask;
+	PropertyMask	cpSetMask;
 
-	tedChangeTableLayout( ed, bi, sectBi,
-					rp, rowPropertiesAlso, row0, row1 );
+	const int	col0= 0;
+	const int	col1= csCount- 1;
+
+	PROPmaskCLEAR( &rpSetMask );
+	PROPmaskCLEAR( &cpSetMask );
+
+	PROPmaskADD( &rpSetMask, RPpropLEFT_INDENT );
+	PROPmaskADD( &cpSetMask, CLpropRIGHT_BOUNDARY );
+
+	tedChangeTableLayout( ed, sectBi, row0, row1, col0, col1, row1,
+						&rpSetMask, &cpSetMask, rp );
+
+	tedAdaptToolsToSelection( ed );
 
 	tedAdaptFormatToolToDocument( ed );
+
+	appDocumentChanged( ed, 1 );
 	}
 
     return;
@@ -182,40 +198,46 @@ static void tedUpdateTableColumns(	EditDocument *		ed,
 
 static int tedGetRulerTwips(
 			const AppDrawingData *		add,
-			const BufferItem *		bi,
+			BufferItem *			bi,
 			ParagraphProperties *		pp,
-			const DocumentProperties *	dp,
+			BufferDocument *		bd,
 			int				firstIndentPixels,
 			int				leftIndentPixels,
 			int				rightIndentPixels,
 			int				tabCount,
 			const TabStop *			tabStops )
     {
-    FormattingFrame		ff;
+    ParagraphFrame		pf;
     double			xfac= add->addMagnifiedPixelsPerTwip;
 
-    int				otab;
-    int				ntab;
-    int				tab0;
-    int				otab1;
-    int				ntab1;
-
-    int				pixels;
-    int				twips;
-
     const int			bottom= -1;
     const int			stripHigh= -1;
 
     int				x0GeometryPixels;
     int				x1GeometryPixels;
 
-    docParagraphFrameTwips( &ff, bottom, stripHigh, bi, dp,
-					    bi->biTopPosition.lpPage,
-					    bi->biTopPosition.lpColumn );
-    tedParagraphFramePixels( &ff, add, bi );
+    BlockFrame			bf;
 
-    x0GeometryPixels= TWIPStoPIXELS( xfac, ff.ffX0GeometryTwips );
-    x1GeometryPixels= TWIPStoPIXELS( xfac, ff.ffX1GeometryTwips );
+    docBlockFrameTwips( &bf, bi, bd, bi->biTopPosition.lpPage,
+					    bi->biTopPosition.lpColumn );
+
+    docParagraphFrameTwips( &pf, &bf, bottom, stripHigh, bi );
+    tedParagraphFramePixels( &pf, add, bi );
+
+    {
+    TabStopList *	tsl= &(bi->biParaTabStopList);
+    int			tab;
+    TabStop *		ts= tsl->tslTabStops;
+
+    for ( tab= 0; tab < tsl->tslTabStopCount; ts++, tab++ )
+	{
+	ts->tsPixels= TWIPStoPIXELS( xfac,
+				    pf.pfX0GeometryTwips+ ts->tsTwips );
+	}
+    }
+
+    x0GeometryPixels= TWIPStoPIXELS( xfac, pf.pfX0GeometryTwips );
+    x1GeometryPixels= TWIPStoPIXELS( xfac, pf.pfX1GeometryTwips );
 
     leftIndentPixels -= x0GeometryPixels;
     pp->ppLeftIndentTwips= PIXELStoTWIPS( leftIndentPixels, xfac );
@@ -227,102 +249,26 @@ static int tedGetRulerTwips(
     rightIndentPixels= x1GeometryPixels- rightIndentPixels;
     pp->ppRightIndentTwips= PIXELStoTWIPS( rightIndentPixels, xfac );
 
-    if  ( tabCount > pp->ppTabCount )
-	{
-	TabStop *	fresh;
-
-	fresh= (TabStop *)realloc( pp->ppTabStops, tabCount* sizeof(TabStop) );
-	if  ( ! fresh )
-	    { LXDEB(tabCount,fresh); return -1;	}
-
-	pp->ppTabStops= fresh;
-	}
-
-    tab0= 0;
-    while( tab0 < pp->ppTabCount && tab0 < tabCount )
-	{
-	twips= pp->ppTabStops[tab0].tsTwips;
-	pixels= x0GeometryPixels+ TWIPStoPIXELS( xfac, twips );
-
-	if  ( pixels != tabStops[tab0].tsPixels )
-	    { break;	}
-
-	pp->ppTabStops[tab0]= tabStops[tab0];
-	pp->ppTabStops[tab0].tsTwips= twips;
-
-	tab0++;
-	}
-
-    otab1= pp->ppTabCount- 1;
-    ntab1= tabCount- 1;
-    while( otab1 >= tab0 && ntab1 >= tab0 )
-	{
-	twips= pp->ppTabStops[otab1].tsTwips;
-	pixels= x0GeometryPixels+ TWIPStoPIXELS( xfac, twips );
-
-	if  ( pixels != tabStops[ntab1].tsPixels )
-	    { break;	}
-
-	/*  No.. Later on
-	pp->ppTabStops[otab1]= tabStops[ntab1];
-	pp->ppTabStops[otab1].tsTwips= twips;
-	*/
-
-	otab1--; ntab1--;
-	}
-
-    if  ( pp->ppTabCount < tabCount )
-	{
-	ntab= tabCount- 1;
-	for ( otab= pp->ppTabCount- 1; otab > otab1; ntab--, otab-- )
-	    {
-	    twips= pp->ppTabStops[otab].tsTwips;
-
-	    pp->ppTabStops[ntab]= tabStops[ntab];
-	    pp->ppTabStops[ntab].tsTwips= twips;
-	    }
-	}
-    else{
-	ntab= ntab1+ 1;
-	for ( otab= otab1+ 1; otab < pp->ppTabCount; ntab++, otab++ )
-	    {
-	    twips= pp->ppTabStops[otab].tsTwips;
-
-	    pp->ppTabStops[ntab]= tabStops[ntab];
-	    pp->ppTabStops[ntab].tsTwips= twips;
-	    }
-	}
-
-    otab= tab0;
-    for ( ntab= tab0; ntab <= ntab1; otab++, ntab++ )
-	{
-	pixels= tabStops[ntab].tsPixels;
-	pixels -= x0GeometryPixels;
-	twips= pixels/ xfac;
-
-	pp->ppTabStops[otab]= tabStops[ntab];
-	pp->ppTabStops[otab].tsTwips= twips;
-	}
-
-    pp->ppTabCount= tabCount;
+    if  ( docRulerMergeTabs( &(pp->ppTabStopList), xfac, x0GeometryPixels,
+						    tabCount, tabStops ) )
+	{ LDEB(tabCount); return -1;	}
 
     return 0;
     }
 
-static APP_EVENT_HANDLER( tedHorizontalRulerButtonDown, w, voided, downEvent )
+static APP_EVENT_HANDLER_H( tedHorizontalRulerButtonDown, w, voided, downEvent )
     {
     EditDocument *		ed= (EditDocument *)voided;
     EditApplication *		ea= ed->edApplication;
     TedDocument *		td= (TedDocument *)ed->edPrivateData;
     BufferDocument *		bd= td->tdDocument;
-    const DocumentProperties *	dp= &(bd->bdProperties);
-    BufferItem *		bi= td->tdSelection.bsBegin.bpBi;
+    BufferItem *		bi;
     AppDrawingData *		add= &(ed->edDrawingData);
 
     int				leftIndentNew;
     int				firstIndentNew;
     int				rightIndentNew;
-    int				tabCountNew= bi->biParaProperties.ppTabCount;
+    int				tabCountNew= 0;
     TabStop *			tabsNew= (TabStop *)0;
 
     int				csCountNew= 0;
@@ -331,38 +277,54 @@ static APP_EVENT_HANDLER( tedHorizontalRulerButtonDown, w, voided, downEvent )
     double			xfac= add->addMagnifiedPixelsPerTwip;
     double			magnification= ea->eaMagnification;
 
-    unsigned int		change;
-    unsigned int		ppUpdMask;
-    unsigned int		spUpdMask;
+    int				prop= PPprop_NONE;
+
+    PropertyMask		taSetMask;
+    PropertyMask		ppChgMask;
+    PropertyMask		ppUpdMask;
+    PropertyMask		spUpdMask;
 
     ParagraphProperties		ppNew;
-    TextAttribute		taNew;
+    TextAttribute		taSet;
 
-    FormattingFrame		ff;
+    ParagraphFrame		pf;
 
     const int			bottom= -1;
     const int			stripHigh= -1;
 
+    BlockFrame			bf;
+
+    DocumentSelection		ds;
+    SelectionGeometry		sg;
+
+    if  ( tedGetSelection( &ds, &sg, td ) )
+	{ LDEB(1); return;	}
+    bi= ds.dsBegin.dpBi;
+
+    tabCountNew= bi->biParaTabStopList.tslTabStopCount;
+
+    PROPmaskCLEAR( &taSetMask );
     PROPmaskCLEAR( &ppUpdMask );
     PROPmaskCLEAR( &spUpdMask );
 
-    docParagraphFrameTwips( &ff, bottom, stripHigh, bi, dp,
-					    bi->biTopPosition.lpPage,
+    docBlockFrameTwips( &bf, bi, bd, bi->biTopPosition.lpPage,
 					    bi->biTopPosition.lpColumn );
-    tedParagraphFramePixels( &ff, add, bi );
 
-    leftIndentNew= ff.ffX0TextLinesPixels;
-    firstIndentNew= ff.ffX0FirstLinePixels;
-    rightIndentNew= ff.ffX1TextLinesPixels;
+    docParagraphFrameTwips( &pf, &bf, bottom, stripHigh, bi );
+    tedParagraphFramePixels( &pf, add, bi );
+
+    leftIndentNew= pf.pfX0TextLinesPixels;
+    firstIndentNew= pf.pfX0FirstLinePixels;
+    rightIndentNew= pf.pfX1TextLinesPixels;
 
     tedHorizontalRulerTrackMouse(
 			    &firstIndentNew, &leftIndentNew, &rightIndentNew,
 			    &tabCountNew, &tabsNew,
-			    &csCountNew, &csNew, &change,
+			    &csCountNew, &csNew, &prop,
 			    w, ea, downEvent, ed->edTopRuler,
 			    (void *)ed, tedDragVerticalHair );
 
-    switch( change )
+    switch( prop )
 	{
 	case PPprop_NONE:
 	    return;
@@ -375,7 +337,7 @@ static APP_EVENT_HANDLER( tedHorizontalRulerButtonDown, w, voided, downEvent )
 
 	case PPpropRIGHT_INDENT:
 	case PPpropTAB_STOPS:
-	    PROPmaskADD( &ppUpdMask, change );
+	    PROPmaskADD( &ppUpdMask, prop );
 	    break;
 
 	case PPpropCOLUMNS:
@@ -383,24 +345,27 @@ static APP_EVENT_HANDLER( tedHorizontalRulerButtonDown, w, voided, downEvent )
 				    xfac, magnification, csCountNew, csNew );
 	    return;
 	default:
-	    LDEB(change); return;
+	    LDEB(prop); return;
 	}
 
     docInitParagraphProperties( &ppNew );
-    docInitTextAttribute( &taNew );
+    utilInitTextAttribute( &taSet );
 
-    if  ( docCopyParagraphProperties( &ppNew, &(bi->biParaProperties ) ) )
+    PROPmaskCLEAR( &ppChgMask );
+
+    if  ( docUpdParaProperties( &ppChgMask, &ppNew, &ppUpdMask,
+				&(bi->biParaProperties), (const int *)0 ) )
 	{ LDEB(1); return;	}
 
-    if  ( tedGetRulerTwips( add, bi, &ppNew, dp,
+    if  ( tedGetRulerTwips( add, bi, &ppNew, bd,
 			    firstIndentNew, leftIndentNew, rightIndentNew,
 			    tabCountNew, tabsNew ) )
 	{ LDEB(1); return;	}
 
     if  ( tedChangeSelectionProperties( ed,
-				    TAupd_NONE, taNew,
-				    ppUpdMask, &ppNew,
-				    spUpdMask, (SectionProperties *)0 ) )
+				    &taSetMask, &taSet,
+				    &ppUpdMask, &ppNew,
+				    &spUpdMask, (SectionProperties *)0 ) )
 	{ LDEB(1);	}
 
     docCleanParagraphProperties( &ppNew );
@@ -419,43 +384,60 @@ void tedDocAdaptHorizontalRuler(	EditDocument *		ed,
     {
     if  ( ed->edTopRuler )
 	{
-	TedDocument *			td= (TedDocument *)ed->edPrivateData;
-	BufferDocument *		bd= td->tdDocument;
-	const DocumentProperties *	dp= &(bd->bdProperties);
+	TedDocument *		td= (TedDocument *)ed->edPrivateData;
+	BufferDocument *	bd= td->tdDocument;
 
-	AppDrawingData *		add= &(ed->edDrawingData);
-	double				xfac= add->addMagnifiedPixelsPerTwip;
-	ParagraphProperties *		pp= &(bi->biParaProperties);
+	AppDrawingData *	add= &(ed->edDrawingData);
+	double			xfac= add->addMagnifiedPixelsPerTwip;
 
-	int				leftIndent;
-	int				firstIndent;
-	int				rightIndent;
+	int			leftIndent;
+	int			firstIndent;
+	int			rightIndent;
 
-	FormattingFrame			ff;
+	ParagraphFrame		pf;
 
-	int				x0GeometryPixels;
-	int				x1GeometryPixels;
+	int			x0GeometryPixels;
+	int			x1GeometryPixels;
 
-	const int			bottom= -1;
-	const int			stripHigh= -1;
+	const int		bottom= -1;
+	const int		stripHigh= -1;
 
-	docParagraphFrameTwips( &ff, bottom, stripHigh, bi, dp,
-					    bi->biTopPosition.lpPage,
+	BlockFrame		bf;
+
+	docBlockFrameTwips( &bf, bi, bd, bi->biTopPosition.lpPage,
 					    bi->biTopPosition.lpColumn );
-	tedParagraphFramePixels( &ff, add, bi );
 
-	x0GeometryPixels= TWIPStoPIXELS( xfac, ff.ffX0GeometryTwips );
-	x1GeometryPixels= TWIPStoPIXELS( xfac, ff.ffX1GeometryTwips );
+	docParagraphFrameTwips( &pf, &bf, bottom, stripHigh, bi );
+	tedParagraphFramePixels( &pf, add, bi );
 
-	leftIndent= ff.ffX0TextLinesPixels;
-	firstIndent= ff.ffX0FirstLinePixels;
-	rightIndent= ff.ffX1TextLinesPixels;
+	{
+	TabStopList *	tsl= &(bi->biParaTabStopList);
+	int		tab;
+	TabStop *	ts= tsl->tslTabStops;
+
+	for ( tab= 0; tab < tsl->tslTabStopCount; ts++, tab++ )
+	    {
+	    ts->tsPixels= TWIPStoPIXELS( xfac,
+					pf.pfX0GeometryTwips+ ts->tsTwips );
+	    }
+	}
+
+	x0GeometryPixels= TWIPStoPIXELS( xfac, pf.pfX0GeometryTwips );
+	x1GeometryPixels= TWIPStoPIXELS( xfac, pf.pfX1GeometryTwips );
+
+	leftIndent= pf.pfX0TextLinesPixels;
+	firstIndent= pf.pfX0FirstLinePixels;
+	rightIndent= pf.pfX1TextLinesPixels;
+
+	{
+	TabStopList *	tsl= &(bi->biParaTabStopList);
 
 	tedAdaptHorizontalRuler( ed->edTopRuler, ed->edTopRulerWidget,
 		    add->addDocRect.drX0, add->addDocRect.drX1,
 		    firstIndent, leftIndent, rightIndent,
 		    add->addPaperRect.drX1,
-		    pp->ppTabCount, pp->ppTabStops );
+		    tsl->tslTabStopCount, tsl->tslTabStops );
+	}
 
 	if  ( bi->biParaInTable )
 	    {
@@ -504,12 +486,11 @@ int tedDocSetTopRuler(	EditDocument *	ed )
     AppDrawingData *		add= &(ed->edDrawingData);
 
     int				topRulerHeight= ed->edTopRulerHeightPixels;
-    double			horPixPerMM= ed->edPixelsPerMM;
     int				unitInt= ea->eaUnitInt;
 
     ed->edTopRuler= tedMakeTopRuler(
 			topRulerHeight,		/*  sizeAcross		*/
-			horPixPerMM,		/*  pixPerMM		*/
+			add->addMagnifiedPixelsPerTwip,
 			add->addMagnification,	/*  magnification	*/
 
 			ed->edLeftRulerWidthPixels,	/*  minUnused	*/
@@ -542,12 +523,11 @@ int tedDocSetLeftRuler(	EditDocument *	ed )
     AppDrawingData *		add= &(ed->edDrawingData);
 
     int				leftRulerWidth= ed->edLeftRulerWidthPixels;
-    double			horPixPerMM= ed->edPixelsPerMM;
     int				unitInt= ea->eaUnitInt;
 
     ed->edLeftRuler= tedMakeLeftRuler(
 			leftRulerWidth,		/*  sizeAcross		*/
-			horPixPerMM,		/*  pixPerMM		*/
+			add->addMagnifiedPixelsPerTwip,
 			add->addMagnification,	/*  magnification	*/
 
 			0, 0,			/*  {min,max}Unused	*/
@@ -583,19 +563,27 @@ void tedDocFormatCopyRul(	APP_WIDGET	fontsOption,
 				void *		voided,
 				void *		voidpbcbs )
     {
-    EditDocument *			ed= (EditDocument *)voided;
-    TedDocument *			td= (TedDocument *)ed->edPrivateData;
-    SimpleOutputStream *		sos;
+    EditDocument *		ed= (EditDocument *)voided;
+    TedDocument *		td= (TedDocument *)ed->edPrivateData;
+    SimpleOutputStream *	sos;
 
-    if  ( ! td->tdSelection.bsBegin.bpBi )
-	{ XDEB(td->tdSelection.bsBegin.bpBi); return;	}
+    BufferItem *		bi;
+
+    DocumentSelection		ds;
+    SelectionGeometry		sg;
+
+    if  ( tedGetSelection( &ds, &sg, td ) )
+	{ LDEB(1); return;	}
+    bi= ds.dsBegin.dpBi;
+
+    if  ( ! bi )
+	{ XDEB(bi); return;	}
 
     sos= sioOutMemoryOpen( &(td->tdCopiedRuler) );
     if  ( ! sos )
 	{ XDEB(sos); return;	}
 
-    if  ( docRtfSaveRuler( sos,
-			&(td->tdSelection.bsBegin.bpBi->biParaProperties) ) )
+    if  ( docRtfSaveRuler( sos, &(bi->biParaProperties) ) )
 	{ LDEB(1); sioOutClose( sos ); return;	}
 
     if  ( sioOutClose( sos ) )
