@@ -33,7 +33,7 @@ typedef struct PropertyDialog
     char *			dpdAuthorText;
     char *			dpdSubjectText;
     char *			dpdKeywordsText;
-    char *			dpdCommentText;
+    char *			dpdDoccommText;
     char *			dpdHlinkbaseText;
 
     char *			dpdCreatimText;
@@ -50,7 +50,7 @@ typedef struct PropertyDialog
     APP_WIDGET			dpdAuthorWidget;
     APP_WIDGET			dpdSubjectWidget;
     APP_WIDGET			dpdKeywordsWidget;
-    APP_WIDGET			dpdCommentWidget;
+    APP_WIDGET			dpdDoccommWidget;
     APP_WIDGET			dpdHlinkbaseWidget;
 
     APP_WIDGET			dpdCreatimWidget;
@@ -80,8 +80,8 @@ static AppConfigurableResource TED_PropertyDialogResourceTable[]=
 	APP_RESOURCE( "propDialogKeywords",
 		    offsetof(PropertyDialog,dpdKeywordsText),
 		    "Keywords" ),
-	APP_RESOURCE( "propDialogComment",
-		    offsetof(PropertyDialog,dpdCommentText),
+	APP_RESOURCE( "propDialogDoccom",
+		    offsetof(PropertyDialog,dpdDoccommText),
 		    "Comment" ),
 	APP_RESOURCE( "propDialogHlinkbase",
 		    offsetof(PropertyDialog,dpdHlinkbaseText),
@@ -127,9 +127,7 @@ static AppConfigurableResource TED_PropertyDialogResourceTable[]=
 /*									*/
 /************************************************************************/
 
-static void tedPropertyChanged(	APP_WIDGET	w,
-				void *		voiddpd,
-				void *		call_data )
+static APP_TXTYPING_CALLBACK_H( tedPropertyChanged, w, voiddpd )
     {
     PropertyDialog *	dpd= (PropertyDialog *)voiddpd;
 
@@ -146,9 +144,7 @@ static void tedPropertyChanged(	APP_WIDGET	w,
 /*									*/
 /************************************************************************/
 
-static void tedPropertiesCancelPushed(	APP_WIDGET	w,
-					void *		voiddpd,
-					void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( tedPropertiesCancelPushed, w, voiddpd )
     {
     PropertyDialog *	dpd= (PropertyDialog *)voiddpd;
 
@@ -157,9 +153,7 @@ static void tedPropertiesCancelPushed(	APP_WIDGET	w,
     return;
     }
 
-static void tedPropertiesOkPushed(	APP_WIDGET	w,
-					void *		voiddpd,
-					void *		voidpbcs	 )
+static APP_BUTTON_CALLBACK_H( tedPropertiesOkPushed, w, voiddpd )
     {
     PropertyDialog *	dpd= (PropertyDialog *)voiddpd;
 
@@ -174,7 +168,7 @@ static void tedPropertiesOkPushed(	APP_WIDGET	w,
 /*									*/
 /************************************************************************/
 
-static APP_CLOSE_CALLBACK( tedClosePropertyDialog, w, voiddpd )
+static APP_CLOSE_CALLBACK_H( tedClosePropertyDialog, w, voiddpd )
     {
     PropertyDialog *	dpd= (PropertyDialog *)voiddpd;
 
@@ -241,8 +235,8 @@ static void tedPropsDialogMakeTextForm( APP_WIDGET		parent,
     tedPropsDialogMakePropertyRow( &row, &label, &(dpd->dpdKeywordsWidget),
 			    parent, dpd, dpd->dpdKeywordsText, changeable );
 
-    tedPropsDialogMakePropertyRow( &row, &label, &(dpd->dpdCommentWidget),
-			    parent, dpd, dpd->dpdCommentText, changeable );
+    tedPropsDialogMakePropertyRow( &row, &label, &(dpd->dpdDoccommWidget),
+			    parent, dpd, dpd->dpdDoccommText, changeable );
 
     tedPropsDialogMakePropertyRow( &row, &label, &(dpd->dpdHlinkbaseWidget),
 			    parent, dpd, dpd->dpdHlinkbaseText, changeable );
@@ -337,7 +331,7 @@ static PropertyDialog * tedMakePropertyDialog( EditDocument *	ed,
 
     appMakeVerticalDialog( &(dpd->dpdDialog), &paned, ea,
 						tedClosePropertyDialog,
-						(APP_CALLBACK_FUNC)0,
+						(APP_DESTROY_CALLBACK_T)0,
 						(void *)dpd,
 						"tedPropertyDialog" );
 
@@ -468,7 +462,7 @@ int tedRunPropertyDialog(	EditDocument *			ed,
     tedPropToWidget( dpd->dpdAuthorWidget, dp->dpAuthor );
     tedPropToWidget( dpd->dpdSubjectWidget, dp->dpSubject );
     tedPropToWidget( dpd->dpdKeywordsWidget, dp->dpKeywords );
-    tedPropToWidget( dpd->dpdCommentWidget, dp->dpComment );
+    tedPropToWidget( dpd->dpdDoccommWidget, dp->dpDoccomm );
     tedPropToWidget( dpd->dpdHlinkbaseWidget, dp->dpHlinkbase );
 
     tedDateToWidget( dpd->dpdCreatimWidget, &(dp->dpCreatim) );
@@ -515,8 +509,8 @@ int tedRunPropertyDialog(	EditDocument *			ed,
 						dpd->dpdKeywordsWidget ) )
 	{ LDEB(1); return -1;	}
 
-    if  ( tedPropChangeProperty( &changed, &(dp->dpComment),
-						dpd->dpdCommentWidget ) )
+    if  ( tedPropChangeProperty( &changed, &(dp->dpDoccomm),
+						dpd->dpdDoccommWidget ) )
 	{ LDEB(1); return -1;	}
 
     if  ( tedPropChangeProperty( &changed, &(dp->dpHlinkbase),
