@@ -1,4 +1,4 @@
-#   include	"config.h"
+#   include	"tedConfig.h"
 
 #   include	<stddef.h>
 #   include	<stdlib.h>
@@ -7,12 +7,15 @@
 #   include	<ctype.h>
 
 #   include	<sioStdio.h>
+#   include	<appSystem.h>
 
 #   include	<appSpellTool.h>
 
 #   include	"tedApp.h"
 #   include	"tedRuler.h"
-#   include	"docPs.h"
+
+#   include	"docLayout.h"
+#   include	"docEdit.h"
 
 #   include	<appDebugon.h>
 
@@ -46,24 +49,28 @@ void tedDrawRectangle(	APP_WIDGET		w,
 /*									*/
 /************************************************************************/
 
-void tedMoveObjectWindows(	APP_WIDGET		w,
-				EditDocument *		ed )
+void tedMoveObjectWindows(	EditDocument *		ed )
     {
+    AppDrawingData *		add= &(ed->edDrawingData);
     TedDocument *		td= (TedDocument *)ed->edPrivateData;
+    BufferDocument *		bd= td->tdDocument;
 
     InsertedObject *		io;
 
     int				ox= ed->edVisibleRect.drX0;
     int				oy= ed->edVisibleRect.drY0;
 
-    BufferPosition		bp;
+    DocumentPosition		dp;
+    PositionGeometry		pg;
 
-    docInitPosition( &bp );
+    docInitDocumentPosition( &dp );
 
-    if  ( tedGetObjectSelection( td, &bp, &io ) )
+    if  ( tedGetObjectSelection( td, &dp, &io ) )
 	{ LDEB(1); return;	}
 
-    tedSetObjectWindows( ed, &bp, ox, oy );
+    tedPositionGeometry( &pg, &dp, bd, add );
+
+    tedSetObjectWindows( ed, &pg, io, ox, oy );
     }
 
 void tedDocHorizontalScrollbarCallback(	APP_WIDGET	w,
@@ -76,7 +83,7 @@ void tedDocHorizontalScrollbarCallback(	APP_WIDGET	w,
     appDocHorizontalScrollbarCallback( w, voided, voidscbs );
 
     if  ( td->tdObjectSelected )
-	{ tedMoveObjectWindows( w, ed );	}
+	{ tedMoveObjectWindows( ed );	}
     }
 
 void tedDocVerticalScrollbarCallback(	APP_WIDGET	w,
@@ -89,13 +96,16 @@ void tedDocVerticalScrollbarCallback(	APP_WIDGET	w,
     appDocVerticalScrollbarCallback( w, voided, voidscbs );
 
     if  ( td->tdObjectSelected )
-	{ tedMoveObjectWindows( w, ed );	}
+	{ tedMoveObjectWindows( ed );	}
     }
 
 /************************************************************************/
 /*									*/
 /*  Last phase in building a document window.				*/
 /*									*/
+/*  1)  Set the position by hand, because the regular routine sends an	*/
+/*	expose to a window that does not exist yet.			*/
+/*									*/
 /************************************************************************/
 
 int tedFinishDocumentSetup(	EditApplication *	ea,
@@ -114,6 +124,38 @@ int tedFinishDocumentSetup(	EditApplication *	ea,
     char *			tableColorName= "gray80";
     */
 
+    {
+    DocumentPosition	dp;
+
+    /*  1  */
+    if  ( ! docFirstDocumentPosition( &dp, bd ) )
+	{
+	const TextParticule *		tp;
+
+	docSetIBarSelection( &(td->tdDocumentSelection), &dp );
+
+	tedSelectionGeometry( &(td->tdSelectionGeometry),
+				     &(td->tdDocumentSelection), bd, add );
+
+	tp= dp.dpBi->biParaParticules+ dp.dpParticule;
+
+	utilGetTextAttributeByNumber( &(td->tdCurrentTextAttribute),
+						&(bd->bdTextAttributeList),
+						tp->tpTextAttributeNumber );
+	td->tdCurrentTextAttributeNumber= tp->tpTextAttributeNumber;
+
+	td->tdCurrentPhysicalFont= tp->tpPhysicalFont;
+	}
+    else{ docListItem( 0, &(bd->bdItem) );	}
+
+    /*
+    LDEB(1);
+    if  ( docCheckItem( &(bd->bdItem) ) )
+	{ LDEB(2); docListItem( 0, &(bd->bdItem) ); abort();	}
+    */
+
+    }
+
     if  ( appFinishDocumentSetup( ea, ed ) )
 	{ LDEB(1); return -1;	}
 
@@ -122,30 +164,36 @@ int tedFinishDocumentSetup(	EditApplication *	ea,
 
     if  ( ed->edColors.acAllocator.caDepth < 4 )
 	{
-	if  ( appColorRgb( &(td->tdSelColor), &(ed->edColors), 0, 0, 0 ) )
-	    { LDEB(1); return -1;	}
-
-	if  ( appColorRgb( &(td->tdXSelColor), &(ed->edColors), 0, 0, 0 ) )
-	    { LDEB(1); return -1;	}
-
 	if  ( appColorRgb( &(td->tdFieldColor), &(ed->edColors), 0, 0, 0 ) )
 	    { LDEB(1); return -1;	}
 
-	if  ( appColorRgb( &(td->tdTableColor), &(ed->edColors), 0, 0, 0 ) )
-	    { LDEB(1); return -1;	}
+	td->tdSelColor.rgb8Red= 0;
+	td->tdSelColor.rgb8Green= 0;
+	td->tdSelColor.rgb8Blue= 0;
+
+	td->tdCopiedSelColor.rgb8Red= 0;
+	td->tdCopiedSelColor.rgb8Green= 0;
+	td->tdCopiedSelColor.rgb8Blue= 0;
+
+	td->tdTableColor.rgb8Red= 0;
+	td->tdTableColor.rgb8Green= 0;
+	td->tdTableColor.rgb8Blue= 0;
 	}
     else{
-	if  ( appColorNamed( &td->tdSelColor, &ed->edColors, selColorName ) )
+	APP_COLOR_RGB	acSel;
+	APP_COLOR_RGB	acCopiedSel;
+
+	if  ( appColorNamed( &acSel, &ed->edColors, selColorName ) )
 	    {
-	    if  ( appColorFindRgb( &(td->tdSelColor),
+	    if  ( appColorFindRgb( &acSel,
 					   &(ed->edColors), 176, 196, 222 ) )
 		{ SDEB(selColorName); return -1;	}
 	    }
 
-	if  ( appColorNamed( &(td->tdXSelColor),
+	if  ( appColorNamed( &acCopiedSel,
 					    &(ed->edColors), xselColorName ) )
 	    {
-	    if  ( appColorFindRgb( &(td->tdXSelColor),
+	    if  ( appColorFindRgb( &acCopiedSel,
 					    &(ed->edColors), 176, 176, 176 ) )
 		{ SDEB(xselColorName); return -1;	}
 	    }
@@ -157,18 +205,49 @@ int tedFinishDocumentSetup(	EditApplication *	ea,
 		{ LDEB(1); return -1;	}
 	    }
 
-	td->tdTableColor= add->addBackColor;
+#	ifdef USE_MOTIF
+	td->tdSelColor.rgb8Red= acSel.red/ 256;
+	td->tdSelColor.rgb8Green= acSel.green/ 256;
+	td->tdSelColor.rgb8Blue= acSel.blue/ 256;
+
+	td->tdCopiedSelColor.rgb8Red= acCopiedSel.red/ 256;
+	td->tdCopiedSelColor.rgb8Green= acCopiedSel.green/ 256;
+	td->tdCopiedSelColor.rgb8Blue= acCopiedSel.blue/ 256;
+
+	td->tdTableColor.rgb8Red= add->addBackColor.red/ 256;
+	td->tdTableColor.rgb8Green= add->addBackColor.green/ 256;
+	td->tdTableColor.rgb8Blue= add->addBackColor.blue/ 256;
+#	endif
+
+#	ifdef USE_GTK
+	td->tdSelColor.rgb8Red= acSel.red/ 256;
+	td->tdSelColor.rgb8Green= acSel.green/ 256;
+	td->tdSelColor.rgb8Blue= acSel.blue/ 256;
+
+	td->tdCopiedSelColor.rgb8Red= acCopiedSel.red/ 256;
+	td->tdCopiedSelColor.rgb8Green= acCopiedSel.green/ 256;
+	td->tdCopiedSelColor.rgb8Blue= acCopiedSel.blue/ 256;
+
+	td->tdTableColor.rgb8Red= add->addBackColor.red/ 256;
+	td->tdTableColor.rgb8Green= add->addBackColor.green/ 256;
+	td->tdTableColor.rgb8Blue= add->addBackColor.blue/ 256;
+#	endif
 	}
 
     if  ( tedOpenItemObjects( &(bd->bdItem), &(ed->edColors),
 						    &(ed->edDrawingData) ) )
 	{ LDEB(1);	}
 
+    td->tdDrawTableGrid= tar->tarShowTableGridInt;
+
+    appGuiSetToggleItemState( td->tdDrawTableGridOption,
+						td->tdDrawTableGrid >= 0 );
+
     if  ( ed->edIsReadonly )
 	{ tedMakeDocumentReadonly( ed );	}
 
     if  ( tedHasSelection( td ) )
-	{ tedAdaptToolsToPosition( ed );	}
+	{ tedAdaptToolsToSelection( ed );	}
 
     return 0;
     }
@@ -198,9 +277,12 @@ void tedFreeDocument(		void *			voidtd,
     if  ( td->tdDocument )
 	{
 	BufferDocument *	bd= td->tdDocument;
+	int			noteCount= 0;
+	int			paragraphCount= 0;
 
-	docCloseItemObjects( bd, &(bd->bdItem),
-					(void *)add, tedCloseObject );
+	docCloseItemObjects( &noteCount, &paragraphCount,
+					    bd, &(bd->bdItem),
+					    (void *)add, tedCloseObject );
 
 	docFreeDocument( bd );
 	}
@@ -248,39 +330,35 @@ int tedMakeDocumentWidget(	EditApplication *	ea,
 static int tedDetermineCodepage(	BufferDocument *	bd,
 					const AppDrawingData *	add )
     {
-    DocumentProperties *	dp= &(bd->bdProperties);
-
-    PostScriptFont *		fontList= (PostScriptFont *)0;
-    int				fontCount= 0;
+    PostScriptFaceList		psfl;
+    PostScriptFace *		psf;
     int				i;
 
-    int				encoding= -1;
-    int				encodingCount= 0;
+    int				charset= -1;
+    int				charsetCount= 0;
 
-    if  ( docPsPrintGetItemFonts( &(bd->bdItem),
-				&(dp->dpFontList), &(add->addPhysicalFontList),
-				&fontList, &fontCount ) )
+    utilInitPostScriptFaceList( &psfl );
+
+    if  ( docPsPrintGetDocumentFonts( bd, &psfl, &(add->addPhysicalFontList) ) )
 	{ LDEB(1); return -1;	}
 
-    for ( i= 0; i < fontCount; i++ )
+    psf= psfl.psflFaces;
+    for ( i= 0; i < psfl.psflFaceCount; psf++, i++ )
 	{
-	AfmFontInfo *	afi= fontList[i].psfAfi;
-
-	if  ( encoding < 0 )
-	    { encoding= afi->afiEncoding;	}
+	if  ( charset < 0 )
+	    { charset= psf->psfEncodingUsed; charsetCount= 1;	}
 	else{
-	    if  ( encoding != afi->afiEncoding )
-		{ encodingCount++; }
+	    if  ( charset != psf->psfEncodingUsed )
+		{ charsetCount++; }
 	    }
 	}
 
-    if  ( fontList )
-	{ free( fontList );	}
+    utilCleanPostScriptFaceList( &psfl );
 
-    if  ( encodingCount != 1 )
+    if  ( charsetCount != 1 )
 	{ return -1;	}
 
-    switch( encoding )
+    switch( charset )
 	{
 	case ENCODINGpsISO_8859_1:
 	    return 1252;
@@ -289,9 +367,8 @@ static int tedDetermineCodepage(	BufferDocument *	bd,
 	    return 1250;
 
 	default:
-	    LDEB(encoding); return -1;
+	    LDEB(charset); return -1;
 	}
-
     }
 
 /************************************************************************/
@@ -306,80 +383,66 @@ int tedNewDocument(	EditApplication *	ea,
     {
     TedDocument *		td= (TedDocument *)ed->edPrivateData;
     BufferDocument *		bd;
-
-    unsigned int		updMask;
-    TextAttribute		taNew;
-
-    char *			familyName;
-
-    char			scratch[120];
-    long			now;
     DocumentProperties *	dp;
 
+    PropertyMask		newMask;
+    ExpandedTextAttribute	etaNew;
+    TextAttribute		ta;
+
+    time_t			now;
+
     TedAppResources *		tar= (TedAppResources *)ea->eaResourceData;
+    int				rval= 0;
 
-    if  ( tar->tarDefaultAnsicpgInt < 0		&&
-	  tar->tarDefaultAnsicpgString		)
+    tedDetermineDefaultSettings( tar );
+
+    PROPmaskCLEAR( &newMask );
+    docInitExpandedTextAttribute( &etaNew );
+
+    if  ( docExpandedAttributeFromString( ea->eaDefaultFont,
+							&newMask, &etaNew ) )
+	{ SDEB(ea->eaDefaultFont); rval= -1; goto ready;	}
+
+    if  ( ! PROPmaskISSET( &newMask, TApropFONTSIZE )	||
+	  etaNew.etaFontSizeHalfPoints < 6		)
 	{
-	char *		past;
-	int		val;
-
-	val= strtol( tar->tarDefaultAnsicpgString, &past, 10 );
-	if  ( past != tar->tarDefaultAnsicpgString )
-	    {
-	    while( *past == ' ' )
-		{ past++;	}
-	    }
-	if  ( past != tar->tarDefaultAnsicpgString	&&
-	      ! *past					)
-	    { tar->tarDefaultAnsicpgInt= val;		}
-	else{ SDEB(tar->tarDefaultAnsicpgString);	}
+	LDEB(etaNew.etaFontSizeHalfPoints);
+	etaNew.etaFontSizeHalfPoints= 24;
+	PROPmaskADD( &newMask, TApropFONTSIZE );
 	}
 
-    strcpy( scratch, ea->eaDefaultFont );
-    if  ( appFontGetAttributes( scratch, &familyName, &updMask, &taNew ) )
-	{ SDEB(ea->eaDefaultFont); return -1;	}
-
-    if  ( ! PROPmaskISSET( updMask, TApropFONTSIZE )	||
-	  taNew.taFontSizeHalfPoints < 6		)
-	{
-	LDEB(taNew.taFontSizeHalfPoints);
-	taNew.taFontSizeHalfPoints= 24;
-	PROPmaskADD( &updMask, TApropFONTSIZE );
-	}
-
-    bd= docNewFile( familyName, 10* taNew.taFontSizeHalfPoints,
-					    &(ea->eaDefaultDocumentGeometry) );
+    bd= docNewFile( &ta, etaNew.etaFontFamilyName,
+				    etaNew.etaFontSizeHalfPoints,
+				    DOCcharsetANSI, tar->tarDefaultAnsicpgInt,
+				    &(ea->eaDefaultDocumentGeometry) );
     if  ( ! bd )
-	{ XDEB(bd); return -1;	}
+	{ XDEB(bd); rval= -1; goto ready;	}
 
     ed->edFormat= TEDdockindRTF; /* rtf */
     td->tdDocument= bd;
     dp= &(bd->bdProperties);
-    dp->dpAnsiCodePage= tar->tarDefaultAnsicpgInt;
 
     if  ( filename )
 	{
-	const char *	slash= strrchr( filename, '/' );
-	const char *	dot;
+	const char *	ext= appFileExtensionOfName( filename );
 
-	if  ( slash )
-	    { dot= strrchr( slash+ 1, '.' );	}
-	else{ dot= strrchr( filename, '.' );	}
-
-	if  ( dot && ! strcmp( dot, ".rtf" ) )
+	if  ( ext && ! strcmp( ext, "rtf" ) )
 	    { ed->edFormat= TEDdockindRTF;	}
-	if  ( dot && ! strcmp( dot, ".txt" ) )
+	if  ( ext && ! strcmp( ext, "txt" ) )
 	    { ed->edFormat= TEDdockindTEXT_SAVE_FOLDED;	}
 	}
 
     if  ( ea->eaAuthor )
 	{ dp->dpAuthor= (unsigned char *)strdup( ea->eaAuthor ); }
 
-    now= time( (long *)0 );
+    now= time( (time_t *)0 );
     dp->dpCreatim= *localtime( &now );
 
-    return 0;
+  ready:
+
+    docCleanExpandedTextAttribute( &etaNew );
+
+    return rval;
     }
 
 /************************************************************************/
@@ -387,7 +450,7 @@ int tedNewDocument(	EditApplication *	ea,
 /*  Perform the setup of the editing environment for a document.	*/
 /*									*/
 /*  8)  Determine the code page of the document.			*/
-/*  9)  Do not use the 1252 cidepage as an explicit setting: It is	*/
+/*  9)  Do not use the 1252 codepage as an explicit setting: It is	*/
 /*	almost certainly returned by a default 'Ted' installation, and	*/
 /*	is almost impossible to get rid of.				*/
 /*									*/
@@ -403,58 +466,48 @@ int tedLayoutDocument(		void *				privateData,
     const DocumentProperties *	dp= &(bd->bdProperties);
     const DocumentGeometry *	dgDoc= &(dp->dpGeometry);
 
-    int				changed= 0;
+    int				noteNumbersChanged= 0;
+
+    RecalculateFields		rf;
+
+    docInitRecalculateFields( &rf );
+
+    rf.rfBd= bd;
+    rf.rfVoidadd= (void *)add;
+    rf.rfCloseObject= tedCloseObject;
+    rf.rfUpdateFlags= FIELDdoDOC_INFO|FIELDdoDOC_COMPLETE|FIELDdoCHFTN;
+    rf.rfFieldsUpdated= 0;
 
     tedScreenRectangles( add, dgDoc );
 
-    if  ( docRecalculateTextLevelFields( &changed, bd, &(bd->bdItem),
-				    FIELDdoDOC_INFO|FIELDdoDOC_COMPLETE,
-				    (void *)add, tedCloseObject ) )
+    if  ( docRecalculateTextLevelFields( &rf, &(bd->bdItem) ) )
 	{ LDEB(1); return -1;	}
 
     if  ( tedLayoutDocumentTree( td, add ) )
 	{ LDEB(1); return -1;	}
 
-    changed= 0;
-    if  ( docRecalculateTextLevelFields( &changed, bd, &(bd->bdItem),
-				    FIELDdoDOC_FORMATTED,
-				    (void *)add, tedCloseObject ) )
+    docRenumberNotes( &noteNumbersChanged, bd );
+
+    rf.rfUpdateFlags= FIELDdoDOC_FORMATTED;
+    rf.rfFieldsUpdated= 0;
+
+    if  ( noteNumbersChanged )
+	{ rf.rfUpdateFlags |= FIELDdoCHFTN;	}
+
+    if  ( docRecalculateTextLevelFields( &rf, &(bd->bdItem) ) )
 	{ LDEB(1); return -1;	}
 
-    if  ( changed && tedLayoutDocumentTree( td, add ) )
+    if  ( rf.rfFieldsUpdated > 0 && tedLayoutDocumentTree( td, add ) )
 	{ LDEB(1); return -1;	}
 
-    {
-    BufferPosition	bp;
-
-    if  ( ! tedFirstPosition( add, bd, &bp ) )
-	{
-	td->tdSelection.bsBegin=
-		    td->tdSelection.bsEnd=
-		    td->tdSelection.bsAnchor= bp;
-
-	td->tdCurrentTextAttribute=
-		bp.bpBi->biParaParticules[bp.bpParticule].tpTextAttribute;
-	td->tdCurrentPhysicalFont=
-		bp.bpBi->biParaParticules[bp.bpParticule].tpPhysicalFont;
-
-
-	td->tdSelection.bsDirection= 0;
-
-	tedSelectionRectangle( &(td->tdSelectedRectangle),
-						 add, dp, &(td->tdSelection) );
-	}
-    else{ docListItem( 0, &(bd->bdItem) );	}
-    }
-
     /*  8  */
-    if  ( bd->bdProperties.dpAnsiCodePage < 0 )
+    if  ( bd->bdProperties.dpAnsiCodepage < 0 )
 	{
-	bd->bdProperties.dpAnsiCodePage= tedDetermineCodepage( bd, add );
+	bd->bdProperties.dpAnsiCodepage= tedDetermineCodepage( bd, add );
 
 	/*  9  */
-	if  ( bd->bdProperties.dpAnsiCodePage == 1252 )
-	    { bd->bdProperties.dpAnsiCodePage= -1;	}
+	if  ( bd->bdProperties.dpAnsiCodepage == 1252 )
+	    { bd->bdProperties.dpAnsiCodepage= -1;	}
 	}
 
     return 0;
@@ -481,7 +534,7 @@ int tedSaveDocument(	const void *		privateData,
 
     const int			asMimeAggr= 0;
 
-    long			now;
+    time_t			now;
 
     sos= sioOutStdioOpen( filename );
     if  ( ! sos )
@@ -490,13 +543,11 @@ int tedSaveDocument(	const void *		privateData,
     switch( format )
 	{
 	case TEDdockindRTF:
-	    now= time( (long *)0 );
+	    now= time( (time_t *)0 );
 	    dp->dpRevtim= *localtime( &now );
 
-	    if  ( docRtfSaveDocument( sos, bd,
-					    (const SelectionScope *)0,
-					    (const BufferSelection *)0,
-					    saveBookmarks ) )
+	    if  ( docRtfSaveDocument( sos, bd, (const DocumentSelection *)0,
+							    saveBookmarks ) )
 		{ SDEB(filename); sioOutClose( sos ); return -1;	}
 	    break;
 
@@ -506,16 +557,14 @@ int tedSaveDocument(	const void *		privateData,
 
 	case TEDdockindTEXT_SAVE_FOLDED:
 	    if  ( docPlainSaveDocument( sos, bd,
-					    (const SelectionScope *)0,
-					    (const BufferSelection *)0,
+					    (const DocumentSelection *)0,
 					    1, 1 ) )
 		{ SDEB(filename); sioOutClose( sos ); return -1;	}
 	    break;
 
 	case TEDdockindTEXT_SAVE_WIDE:
 	    if  ( docPlainSaveDocument( sos, bd,
-					    (const SelectionScope *)0,
-					    (const BufferSelection *)0,
+					    (const DocumentSelection *)0,
 					    0, 1 ) )
 		{ SDEB(filename); sioOutClose( sos ); return -1;	}
 	    break;
@@ -551,7 +600,8 @@ void * tedMakePrivateData()
 	{ XDEB(td); return (void *)0;	}
 
     td->tdDocument= (BufferDocument *)0;
-    docInitTextAttribute( &(td->tdCurrentTextAttribute) );
+    utilInitTextAttribute( &(td->tdCurrentTextAttribute) );
+    td->tdCurrentTextAttributeNumber= -1;
     td->tdCurrentPhysicalFont= -1;
 
     td->tdFormatMenu= (APP_WIDGET)0;
@@ -569,6 +619,9 @@ void * tedMakePrivateData()
     td->tdInsSymbolOption= (APP_WIDGET)0;
     td->tdInsHyperlinkOption= (APP_WIDGET)0;
     td->tdInsBookmarkOption= (APP_WIDGET)0;
+    td->tdInsInsertFootnoteOption= (APP_WIDGET)0;
+    td->tdInsInsertEndnoteOption= (APP_WIDGET)0;
+    td->tdInsInsertChftnsepOption= (APP_WIDGET)0;
     td->tdInsInsertTableOption= (APP_WIDGET)0;
     td->tdInsInsertPageNumberOption= (APP_WIDGET)0;
     td->tdInsInsertLineBreakOption= (APP_WIDGET)0;
@@ -597,9 +650,8 @@ void * tedMakePrivateData()
     td->tdToolsMenu= (APP_WIDGET)0;
     td->tdToolsMenuButton= (APP_WIDGET)0;
 
-    docInitSelection( &(td->tdSelection) );
-    docInitSelectionScope( &(td->tdSelectionScope) );
-    docInitRectangle( &(td->tdSelectedRectangle) );
+    docInitDocumentSelection( &(td->tdDocumentSelection) );
+    docInitSelectionGeometry( &(td->tdSelectionGeometry) );
 
     td->tdVisibleSelectionCopied= 0;
     td->tdCanReplaceSelection= 0;
@@ -620,10 +672,10 @@ void * tedMakePrivateData()
     td->tdShowIBarId= 0;
 #   endif
 
-    td->tdObjectWindow= (APP_ATOM)0;
-    td->tdObjectBottomWindow= (APP_ATOM)0;
-    td->tdObjectRightWindow= (APP_ATOM)0;
-    td->tdObjectCornerWindow= (APP_ATOM)0;
+    td->tdObjectWindow= (APP_WINDOW)0;
+    td->tdObjectBottomWindow= (APP_WINDOW)0;
+    td->tdObjectRightWindow= (APP_WINDOW)0;
+    td->tdObjectCornerWindow= (APP_WINDOW)0;
     td->tdObjectSelected= 0;
 
     td->tdDrawTableGrid= 1;
@@ -637,7 +689,7 @@ void * tedMakePrivateData()
 /*									*/
 /************************************************************************/
 
-APP_EVENT_HANDLER( tedObserveFocus, w, voided, event )
+APP_EVENT_HANDLER_H( tedObserveFocus, w, voided, event )
     {
     EditDocument *	ed= (EditDocument *)voided;
     TedDocument *	td= (TedDocument *)ed->edPrivateData;
@@ -654,9 +706,6 @@ APP_EVENT_HANDLER( tedObserveFocus, w, voided, event )
 	    {
 	    appSetCurrentDocument( ea, ed );
 
-	    if  ( ea->eaFontsTool )
-		{ tedAdaptFontsToolToDocument( ea, ed ); }
-
 	    if  ( ea->eaFindTool && ed->edIsReadonly )
 		{ appFindToolDisableReplace( ea->eaFindTool ); }
 
@@ -676,207 +725,200 @@ APP_EVENT_HANDLER( tedObserveFocus, w, voided, event )
 	{ tedStopCursorBlink( ed ); }
     }
 
-/************************************************************************/
-/*									*/
-/*  Follow a link.							*/
-/*									*/
-/*  1)  As a HACK, invoke netscape for all links with a protocol	*/
-/*	different from 'file:'.						*/
-/*  2)  Theoretically, mailcap/mimecap should have been consulted for	*/
-/*	the application corresponding to the extension of the file.	*/
-/*									*/
-/************************************************************************/
-
-/*  1  */
-static int tedCallNetscape(	const char *		fileName,
-				int			fileSize,
-				const char *		markName,
-				int			markSize )
-    {
-    char *	scratch;
-    int		rval= 0;
-    int		size;
-    char *	to;
-
-    size= 0;
-    size += 60+ fileSize+ 1+ markSize;
-    size += 4;
-    size += 11+ fileSize+ 1+ markSize;
-    size += 2+ 1;
-
-    to= scratch= malloc( size );
-    if  ( ! scratch )
-	{ XDEB(scratch); return -1;	}
-
-    strcpy( to, "netscape -noraise -remote 'openUrl(" );
-    to += strlen( to );
-    memcpy( to, fileName, fileSize ); to += fileSize;
-    if  ( markSize > 0 )
-	{
-	*(to++)= '#';
-	memcpy( to, markName, markSize ); to += markSize;
-	}
-    strcpy( to, ",new-window)' 2>/dev/null" ); to += strlen( to );
-
-    strcpy( to, " || " ); to += strlen( to );
-
-    strcpy( to, "netscape '" ); to += strlen( to );
-
-    memcpy( to, fileName, fileSize ); to += fileSize;
-    if  ( markSize > 0 )
-	{
-	*(to++)= '#';
-	memcpy( to, markName, markSize ); to += markSize;
-	}
-
-    strcpy( to, "' &" ); to += strlen( to );
-
-    if  ( system( scratch ) )
-	{ SDEB(scratch); rval= -1;	}
-
-    free( scratch );
-    return rval;
-    }
-
-int tedFollowLink(	APP_WIDGET		relative,
-			APP_WIDGET		option,
-			EditDocument *		edFrom,
-			const char *		fileName,
-			int			fileSize,
-			const char *		markName,
-			int			markSize )
-    {
-    int				rval= 0;
-    EditApplication *		ea= edFrom->edApplication;
-
-    char *			slash= (char *)0;
-    EditDocument *		edTo;
-
-    char *			scratch= (char *)0;
-
-#   if 0
-    TedDocument *		tdFrom= (TedDocument *)edFrom->edPrivateData;
-    BufferDocument *		bdFrom= tdFrom->tdDocument;
-    const DocumentProperties *	dpFrom= &(bdFrom->bdProperties);
-
-    const char *		base= (const char *)dpFrom->dpHlinkbase;
-    int				baseLen= 0;
-
-    if  ( base )
-	{ baseLen= strlen( base );	}
-#   endif
-
-    if  ( fileSize > 0 )
-	{
-	const int	readonly= 1;
-	int		absolute= fileName[0] == '/';
-
-	int		protocolSize;
-
-	protocolSize= 0;
-	while( protocolSize < fileSize && isalpha( fileName[protocolSize] ) )
-	    { protocolSize++;	}
-
-	/*  1  */
-	if  ( protocolSize < fileSize && fileName[protocolSize] == ':' )
-	    {
-	    if  ( protocolSize == 4			&&
-		  ! strncmp( fileName, "file", 4 )	)
-		{
-		fileName += 5;
-		fileSize -= 5;
-		absolute= fileName[0] == '/';
-		}
-	    else{
-		return tedCallNetscape( fileName, fileSize,
-							markName, markSize );
-		}
-	    }
-
-	if  ( edFrom->edFilename )
-	    { slash= strrchr( edFrom->edFilename, '/' ); }
-
-	if  ( absolute || ! slash )
-	    {
-	    scratch= (char *)malloc( fileSize+ 1 );
-	    if  ( ! scratch )
-		{ XDEB(scratch); return -1;	}
-
-	    strncpy( scratch, fileName, fileSize )[fileSize]= '\0';
-
-	    edTo= appOpenDocument( ea, relative, option, readonly, scratch );
-	    if  ( ! edTo )
-		{ SXDEB(scratch,edTo);	}
-	    }
-	else{
-	    int	dir= ( slash- edFrom->edFilename )+ 1;
-	    int	len= dir+ fileSize+ 1;
-
-	    scratch= (char *)malloc( len );
-	    if  ( ! scratch )
-		{ LXDEB(len,scratch); return -1;	}
-
-	    strncpy( scratch, edFrom->edFilename, dir );
-	    strncpy( scratch+ dir, fileName, fileSize )[fileSize]= '\0';
-
-	    edTo= appOpenDocument( ea, relative, option, readonly, scratch );
-	    if  ( ! edTo )
-		{ /* SXDEB(scratch,edTo); */ rval= -1;	}
-	    }
-	}
-    else{ edTo= edFrom;	}
-
-    if  ( edTo && markSize > 0 )
-	{ tedGoToBookmark( edTo, markName, markSize );	}
-
-    if  ( scratch )
-	{ free( scratch );	}
-
-    return rval;
-    }
-
 /************************************************************************/
 /*									*/
 /*  Print a document to PostScript.					*/
 /*									*/
 /************************************************************************/
 
-int tedPrintDocument(	FILE *				f,
-			const void *			privateData,
-			int				format,
-			AppDrawingData *		add,
-			const char *			title,
-			EditApplication *		ea,
-			const DocumentGeometry *	dgPaper,
-			int				nup,
-			int				horizontal,
+int tedPrintDocument(	SimpleOutputStream *		sos,
+			const PrintJob *		pj,
+			const PrintGeometry *		pg,
 			int				firstPage,
 			int				lastPage )
     {
-    TedDocument *		td= (TedDocument *)privateData;
+    EditApplication *		ea= pj->pjApplication;
+    TedDocument *		td= (TedDocument *)pj->pjPrivateData;
     BufferDocument *		bd= td->tdDocument;
     DocumentProperties *	dp= &(bd->bdProperties);
 
-    long			now;
+    time_t			now;
 
-    int				changed= 0;
+    RecalculateFields		rf;
 
-    now= time( (long *)0 );
+    docInitRecalculateFields( &rf );
+
+    rf.rfBd= bd;
+    rf.rfVoidadd= (void *)pj->pjDrawingData;
+    rf.rfCloseObject= tedCloseObject;
+    rf.rfUpdateFlags= FIELDdoDOC_FORMATTED|FIELDdoDOC_COMPLETE|FIELDdoDOC_INFO;
+    rf.rfFieldsUpdated= 0;
+
+    now= time( (time_t *)0 );
     dp->dpPrintim= *localtime( &now );
 
-    if  ( docRecalculateTextLevelFields( &changed, bd, &(bd->bdItem),
-		    FIELDdoDOC_FORMATTED|FIELDdoDOC_COMPLETE|FIELDdoDOC_INFO,
-		    (void *)add, tedCloseObject ) )
+    if  ( docRecalculateTextLevelFields( &rf, &(bd->bdItem) ) )
 	{ LDEB(1); return -1;	}
 
-    if  ( changed && tedLayoutDocumentTree( td, add ) )
+    if  ( rf.rfFieldsUpdated > 0				&&
+	  tedLayoutDocumentTree( td, pj->pjDrawingData )	)
 	{ LDEB(1); return -1;	}
 
-    if  ( docPsPrintDocument( f, title, ea->eaApplicationName,
-				ea->eaReference, add, td->tdDocument,
-				nup, horizontal, firstPage, lastPage,
-				dgPaper, tedLayoutExternal, tedCloseObject ) )
+    if  ( docPsPrintDocument( sos, pj->pjTitle, ea->eaApplicationName,
+				ea->eaReference, pj->pjDrawingData,
+				td->tdDocument, pg,
+				pj->pjUsePostScriptFilters,
+				pj->pjUsePostScriptIndexedImages,
+				firstPage, lastPage,
+				tedLayoutExternalItem, tedCloseObject ) )
 	{ LDEB(1); return -1;	}
 
     return 0;
     }
+
+/************************************************************************/
+/*									*/
+/*  Open a document.							*/
+/*									*/
+/*  1)  Open an input stream.						*/
+/*  2)  Try to read as RTF.						*/
+/*  3)  If this fails, try as plain text.				*/
+/*									*/
+/************************************************************************/
+
+int tedOpenDocumentFile(	EditApplication *	ea,
+				int *			pFormat,
+				BufferDocument **	pBd,
+				const char *		filename,
+				APP_WIDGET		relative,
+				APP_WIDGET		option )
+    {
+    const char *		ext;
+    SimpleInputStream *		sis;
+
+    BufferDocument *		bd;
+    int				triedRtf= 0;
+
+    int				resp;
+    AppFileMessageResources *	afmr= &(ea->eaFileMessageResources);
+    TedAppResources *		tar= (TedAppResources *)ea->eaResourceData;
+
+    int				longestPara;
+
+    tedDetermineDefaultSettings( tar );
+
+    ext= appFileExtensionOfName( filename );
+
+    /*  1  */
+    sis= sioInStdioOpen( filename );
+    if  ( ! sis )
+	{
+	appQuestionRunSubjectErrorDialog( ea, relative, option,
+					filename, afmr->afmrFileNoAccess );
+
+	return -1;
+	}
+
+    if  ( ext && ! strcmp( ext, "rtf" ) )
+	{
+	/*  2  */
+	bd= docRtfReadFile( sis, tar->tarDefaultAnsicpgInt );
+
+	sioInClose( sis );
+	
+	if  ( bd )
+	    {
+	    *pFormat= TEDdockindRTF; *pBd= bd;
+
+	    if  ( docPropertiesSetFilename( &(bd->bdProperties), filename ) )
+		{ LDEB(1);	}
+
+	    return 0;
+	    }
+	else{
+	    resp= appQuestionRunSubjectOkCancelDialog( ea,
+				    relative, option,
+				    filename, tar->tarFileNotRtf,
+				    (char *)0, (char *)0 );
+
+	    if  ( resp != AQDrespOK )
+		{ return -1;	}
+
+	    triedRtf= 1;
+	    }
+	}
+
+    if  ( ext && ! strcmp( ext, "txt" ) )
+	{
+	bd= docPlainReadFile( sis, &longestPara,
+				    DOCcharsetANSI, tar->tarDefaultAnsicpgInt,
+				    &(ea->eaDefaultDocumentGeometry) );
+
+	sioInClose( sis );
+
+	if  ( bd )
+	    {
+	    *pBd= bd;
+
+	    if  ( longestPara > 76 )
+		{ *pFormat= TEDdockindTEXT_SAVE_WIDE;		}
+	    else{ *pFormat= TEDdockindTEXT_SAVE_FOLDED;		}
+
+	    if  ( docPropertiesSetFilename( &(bd->bdProperties), filename ) )
+		{ LDEB(1);	}
+
+	    return 0;
+	    }
+	else{ SXDEB(filename,bd); return -1;	}
+	}
+
+    /*  2  */
+    if  ( ! triedRtf )
+	{
+	bd= docRtfReadFile( sis, tar->tarDefaultAnsicpgInt );
+	if  ( bd )
+	    {
+	    *pFormat= TEDdockindRTF; *pBd= bd;
+	    
+	    if  ( docPropertiesSetFilename( &(bd->bdProperties), filename ) )
+		{ LDEB(1);	}
+
+	    return 0;
+	    }
+
+	resp= appQuestionRunSubjectOkCancelDialog( ea,
+					ea->eaToplevel.atTopWidget, option,
+					filename, tar->tarFileNotRtf,
+					(char *)0, (char *)0 );
+
+	if  ( resp != AQDrespOK )
+	    { return -1;	}
+	}
+
+    sis= sioInStdioOpen( filename );
+    if  ( ! sis )
+	{ SXDEB(filename,sis); return -1;	}
+
+    bd= docPlainReadFile( sis, &longestPara,
+				    DOCcharsetANSI, tar->tarDefaultAnsicpgInt,
+				    &(ea->eaDefaultDocumentGeometry) );
+
+    sioInClose( sis );
+
+    if  ( bd )
+	{
+	*pBd= bd;
+
+	if  ( longestPara > 76 )
+	    { *pFormat= TEDdockindTEXT_SAVE_WIDE;		}
+	else{ *pFormat= TEDdockindTEXT_SAVE_FOLDED;		}
+
+	if  ( docPropertiesSetFilename( &(bd->bdProperties), filename ) )
+	    { LDEB(1);	}
+
+	return 0;
+	}
+
+    SXDEB(filename,bd); return -1;
+    }
+
