@@ -14,14 +14,14 @@
 #   include	<stddef.h>
 #   include	<limits.h>
 
-#   include	<appDebugon.h>
-
 #   include	<appGeoString.h>
 #   include	<appUnit.h>
 
 #   include	"tedApp.h"
 #   include	"tedFormatTool.h"
 
+#   include	<appDebugon.h>
+
 /************************************************************************/
 /*									*/
 /*  Resources for the format tool.					*/
@@ -32,150 +32,21 @@ typedef struct TedFormatToolResources
     {
     InspectorSubjectResources	tttrSubjectResources[TEDtsi_COUNT];
 
-    TablePageResources		tttrTablePageResources;
-    RowPageResources		tttrRowPageResources;
-    ColumnPageResources		tttrColumnPageResources;
-    CellPageResources		tttrCellPageResources;
-    ParagraphPageResources	tttrParaPageResources;
-    SectionPageResources	tttrSectPageResources;
-    PageLayoutPageResources	tttrPageLayoutPageResources;
-    HeaderFooterPageResources	tttrHeadFootPageResources;
-    NotesPageResources		tttrNotesPageResources;
-    TabsPageResources		tttrTabsPageResources;
+    TablePageResources			tttrTablePageResources;
+    RowPageResources			tttrRowPageResources;
+    ColumnPageResources			tttrColumnPageResources;
+    CellPageResources			tttrCellPageResources;
+    ParagraphLayoutPageResources	tttrParaLayoutPageResources;
+    TabsPageResources			tttrTabsPageResources;
+    ParagraphOrnamentsPageResources	tttrParaOrnamentsPageResources;
+    SectionPageResources		tttrSectPageResources;
+    PageLayoutPageResources		tttrPageLayoutPageResources;
+    HeaderFooterPageResources		tttrHeadFootPageResources;
+    NotesPageResources			tttrNotesPageResources;
+    AppFontToolResources		tttrFontToolResources;
+    RgbChooserPageResources		tttrRgbChooserPageResources;
     } TedFormatToolResources;
 
-/************************************************************************/
-/*									*/
-/*  Validate a dimension						*/
-/*									*/
-/************************************************************************/
-
-int tedFormatValidateDimension(		int *		pNewValue,
-					int *		pChanged,
-					APP_WIDGET	w,
-					int		oldValue )
-    {
-    int				changed;
-
-    const int			minValue= 1;
-    const int			adaptToMin= 0;
-    const int			maxValue= INT_MAX;
-    const int			adaptToMax= 0;
-
-    if  ( ! appGetLengthFromTextWidget( w, &oldValue, &changed, UNITtyPOINTS,
-				minValue, adaptToMin, maxValue, adaptToMax ) )
-	{
-	char	scratch[50];
-
-	appGeoLengthToString( scratch, oldValue, UNITtyPOINTS );
-	appStringToTextWidget( w, scratch );
-
-	if  ( changed )
-	    { *pNewValue= oldValue;	}
-
-	*pChanged= changed; return 0;
-	}
-
-    return -1;
-    }
-
-int tedFormatToolGetGapWidth(		const RowProperties *	rp,
-					int *			pValue,
-					APP_WIDGET		w )
-    {
-    int				value;
-    int				changed;
-
-    int				minWidth;
-    const CellProperties *	cp;
-    int				col;
-
-    const int			minValue= 1;
-    const int			adaptToMin= 0;
-    const int			maxValue= INT_MAX;
-    const int			adaptToMax= 0;
-
-    value= rp->rpHalfGapWidthTwips;
-    if  ( appGetLengthFromTextWidget( w, &value, &changed, UNITtyPOINTS,
-				minValue, adaptToMin, maxValue, adaptToMax ) )
-	{ return -1;	}
-
-    cp= rp->rpCells;
-    minWidth= cp->cpRightBoundaryTwips- rp->rpLeftIndentTwips;
-
-    for ( col= 1; col < rp->rpCellCount; cp++, col++ )
-	{
-	int	cellWidth;
-
-	cellWidth= cp[1].cpRightBoundaryTwips- cp[0].cpRightBoundaryTwips;
-
-	if  ( minWidth > cellWidth )
-	    { minWidth=  cellWidth;	}
-	}
-
-    if  ( 2* value >= minWidth )
-	{
-	char		scratch[50];
-
-	value= minWidth/3;
-
-	appGeoLengthToString( scratch, value, UNITtyPOINTS );
-	appStringToTextWidget( w, scratch );
-	appRefuseTextValue( w );
-	return 1;
-	}
-
-    *pValue= value;
-    return 0;
-    }
-
-int tedFormatToolGetRowLeftIndent(	const RowProperties *	rp,
-					const int		pageLeftMargin,
-					int *			pValue,
-					APP_WIDGET		w )
-    {
-    int				value;
-    int				changed;
-
-    char			scratch[50];
-
-    const int			minValue= INT_MIN;
-    const int			adaptToMin= 0;
-    const int			maxValue= INT_MAX;
-    const int			adaptToMax= 0;
-
-    value= rp->rpLeftIndentTwips;
-    if  ( appGetLengthFromTextWidget( w, &value, &changed, UNITtyPOINTS,
-				minValue, adaptToMin, maxValue, adaptToMax ) )
-	{ return -1;	}
-
-    if  ( value < -pageLeftMargin )
-	{
-	value= -pageLeftMargin+ 10;
-
-	appGeoLengthToString( scratch, value, UNITtyPOINTS );
-	appStringToTextWidget( w, scratch );
-	appRefuseTextValue( w );
-	return 1;
-	}
-
-    if  ( value+ 2* rp->rpHalfGapWidthTwips >=
-					rp->rpCells[0].cpRightBoundaryTwips )
-	{
-	value= rp->rpCells[0].cpRightBoundaryTwips- 3* rp->rpHalfGapWidthTwips;
-
-	if  ( value < -pageLeftMargin )
-	    { appRefuseTextValue( w ); return -1;	}
-
-	appGeoLengthToString( scratch, value, UNITtyPOINTS );
-	appStringToTextWidget( w, scratch );
-	appRefuseTextValue( w );
-	return 1;
-	}
-
-    *pValue= value; return 0;
-    }
-
 static void tedFormatFillPages(	const TedFormatToolResources *	tftr,
 				TedFormatTool *			tft,
 				AppInspector *			ai )
@@ -188,27 +59,30 @@ static void tedFormatFillPages(	const TedFormatToolResources *	tftr,
 
     tedFormatFillRowPage( &(tft->tftRowTool),
 			    &(tftr->tttrRowPageResources),
+			    ai, TEDtsiROW,
 			    &(ai->aiSubjects[TEDtsiROW]),
 			    ai->aiSubjects[TEDtsiROW].isPage,
 			    &(tftr->tttrSubjectResources[TEDtsiROW]) );
 
     tedFormatFillColumnPage( &(tft->tftColumnTool),
 			    &(tftr->tttrColumnPageResources),
+			    ai, TEDtsiCOLUMN,
 			    &(ai->aiSubjects[TEDtsiCOLUMN]),
 			    ai->aiSubjects[TEDtsiCOLUMN].isPage,
 			    &(tftr->tttrSubjectResources[TEDtsiCOLUMN]) );
 
     tedFormatFillCellPage( &(tft->tftCellTool),
 			    &(tftr->tttrCellPageResources),
+			    ai, TEDtsiCELL,
 			    &(ai->aiSubjects[TEDtsiCELL]),
 			    ai->aiSubjects[TEDtsiCELL].isPage,
 			    &(tftr->tttrSubjectResources[TEDtsiCELL]) );
 
-    tedFormatFillParagraphPage( &(tft->tftParagraphTool),
-			    &(tftr->tttrParaPageResources),
-			    &(ai->aiSubjects[TEDtsiPARA]),
-			    ai->aiSubjects[TEDtsiPARA].isPage,
-			    &(tftr->tttrSubjectResources[TEDtsiPARA]) );
+    tedFormatFillParagraphLayoutPage( &(tft->tftParagraphLayoutTool),
+			    &(tftr->tttrParaLayoutPageResources),
+			    &(ai->aiSubjects[TEDtsiPARA_LAY]),
+			    ai->aiSubjects[TEDtsiPARA_LAY].isPage,
+			    &(tftr->tttrSubjectResources[TEDtsiPARA_LAY]) );
 
     tedFormatFillTabsPage( &(tft->tftTabsTool),
 			    &(tftr->tttrTabsPageResources),
@@ -216,6 +90,13 @@ static void tedFormatFillPages(	const TedFormatToolResources *	tftr,
 			    ai->aiSubjects[TEDtsiTABS].isPage,
 			    &(tftr->tttrSubjectResources[TEDtsiTABS]) );
 
+    tedFormatFillParagraphOrnamentsPage( &(tft->tftParagraphOrnamentsTool),
+			    &(tftr->tttrParaOrnamentsPageResources),
+			    ai, TEDtsiPARA_ORN,
+			    &(ai->aiSubjects[TEDtsiPARA_ORN]),
+			    ai->aiSubjects[TEDtsiPARA_ORN].isPage,
+			    &(tftr->tttrSubjectResources[TEDtsiPARA_ORN]) );
+
     tedFormatFillSectionPage( &(tft->tftSectionTool),
 			    &(tftr->tttrSectPageResources),
 			    &(ai->aiSubjects[TEDtsiSECT]),
@@ -240,6 +121,22 @@ static void tedFormatFillPages(	const TedFormatToolResources *	tftr,
 			    ai->aiSubjects[TEDtsiNOTES].isPage,
 			    &(tftr->tttrSubjectResources[TEDtsiNOTES]) );
 
+    appFontToolFillPage( &(tft->tftFontTool),
+			    &(tftr->tttrFontToolResources),
+			    TEDtsiFONT,
+			    &(ai->aiSubjects[TEDtsiFONT]),
+			    ai->aiSubjects[TEDtsiFONT].isPage,
+			    &(tftr->tttrSubjectResources[TEDtsiFONT]) );
+
+    appRgbChooserPageFillPage( &(tft->tftRgbPage),
+			    &(tftr->tttrRgbChooserPageResources),
+			    &(ai->aiSubjects[TEDtsiRGB]),
+			    ai->aiSubjects[TEDtsiRGB].isPage,
+			    &(tftr->tttrSubjectResources[TEDtsiRGB]) );
+
+    /**/
+    appInspectorSetRgbPage( ai, (void *)&(tft->tftRgbPage), TEDtsiRGB );
+
     return;
     }
 
@@ -251,92 +148,24 @@ static void tedFormatFillPages(	const TedFormatToolResources *	tftr,
 
 static void tedDestroyFormatTool(	void *	voidtft )
     {
-    int			i;
-
     TedFormatTool *	tft= (TedFormatTool *)voidtft;
 
-    ParagraphTool *	pt= &(tft->tftParagraphTool);
-    RowTool *		rt= &(tft->tftRowTool);
-    ColumnTool *	cot= &(tft->tftColumnTool);
-    CellTool *		clt= &(tft->tftCellTool);
-    TableTool *		tblt= &(tft->tftTableTool);
-    SectionTool *	st= &(tft->tftSectionTool);
-    PageLayoutTool *	plt= &(tft->tftPageLayoutTool);
-    HeaderFooterTool *	hft= &(tft->tftHeaderFooterTool);
-    NotesTool *		nt= &(tft->tftNotesTool);
-    TabsTool *		tbst= &(tft->tftTabsTool);
-
     EditApplication *	ea= tft->tftApplication;
     TedAppResources *	tar= (TedAppResources *)ea->eaResourceData;
 
-    docCleanParagraphProperties( &(pt->ptPropertiesSet) );
-    docCleanParagraphProperties( &(pt->ptPropertiesChosen) );
-#   if 0
-    tedFormatCleanShadingTool( &(pt->ptShadingTool) );
-#   endif
-    tedFormatCleanBorderTool( &(pt->ptTopBorderTool) );
-    tedFormatCleanBorderTool( &(pt->ptBottomBorderTool) );
-
-    docCleanDocumentProperties( &(tbst->ttDocPropertiesSet) );
-    docCleanDocumentProperties( &(tbst->ttDocPropertiesChosen) );
-    docCleanParagraphProperties( &(tbst->ttParaPropertiesSet) );
-    docCleanParagraphProperties( &(tbst->ttParaPropertiesChosen) );
-
-    docCleanRowProperties( &(rt->rtPropertiesSet) );
-    docCleanRowProperties( &(rt->rtPropertiesChosen) );
-    tedFormatCleanBorderTool( &(rt->rtTopBorderTool) );
-    tedFormatCleanBorderTool( &(rt->rtBottomBorderTool) );
-
-    docCleanRowProperties( &(cot->ctPropertiesSet) );
-    docCleanRowProperties( &(cot->ctPropertiesChosen) );
-    tedFormatCleanBorderTool( &(cot->ctLeftBorderTool) );
-    tedFormatCleanBorderTool( &(cot->ctRightBorderTool) );
-
-    docCleanCellProperties( &(clt->ctPropertiesSet) );
-    docCleanCellProperties( &(clt->ctPropertiesChosen) );
-#   if 0
-    tedFormatCleanShadingTool( &(clt->ctShadingTool) );
-#   endif
-    tedFormatCleanBorderTool( &(clt->ctTopBorderTool) );
-    tedFormatCleanBorderTool( &(clt->ctBottomBorderTool) );
-    tedFormatCleanBorderTool( &(clt->ctLeftBorderTool) );
-    tedFormatCleanBorderTool( &(clt->ctRightBorderTool) );
-
-    docCleanRowProperties( &(tblt->ttPropertiesSet) );
-    docCleanRowProperties( &(tblt->ttPropertiesChosen) );
-
-    docCleanSectionProperties( &(st->stPropertiesSet) );
-    docCleanSectionProperties( &(st->stPropertiesChosen) );
-
-    appCleanPaperChooser( &(plt->pltPaperChooser) );
-    appCleanDrawingData( &(plt->pltDrawingData) );
-
-    docCleanDocumentProperties( &(nt->ntPropertiesSet) );
-    docCleanDocumentProperties( &(nt->ntPropertiesChosen) );
-
-    docCleanDocumentProperties( &(hft->hftDocPropertiesChosen) );
-    docCleanDocumentProperties( &(hft->hftDocPropertiesSet) );
-    docCleanSectionProperties( &(hft->hftSectPropertiesChosen) );
-    docCleanSectionProperties( &(hft->hftSectPropertiesSet) );
-
-    for ( i= 0; i < DOCsp_COUNT; i++ )
-	{
-	if  ( tft->tftShadingPixmaps[i] )
-	    {
-	    appDrawFreePixmap( &(tft->tftPixmapAdd),
-					    tft->tftShadingPixmaps[i] );
-	    }
-	}
-    for ( i= 0; i < DOCbs_COUNT; i++ )
-	{
-	if  ( tft->tftBorderPixmaps[i] )
-	    {
-	    appDrawFreePixmap( &(tft->tftPixmapAdd),
-					    tft->tftBorderPixmaps[i] );
-	    }
-	}
-
-    appCleanDrawingData( &(tft->tftPixmapAdd) );
+    tedFormatCleanTableTool( &(tft->tftTableTool) );
+    tedCleanRowTool( &(tft->tftRowTool) );
+    tedCleanColumnTool( &(tft->tftColumnTool) );
+    tedCleanCellTool( &(tft->tftCellTool) );
+    tedFormatCleanParaLayoutTool( &(tft->tftParagraphLayoutTool) );
+    tedFormatCleanParaTabsTool( &(tft->tftTabsTool) );
+    tedCleanParaOrnamentsTool( &(tft->tftParagraphOrnamentsTool) );
+    tedFormatCleanSectionTool( &(tft->tftSectionTool) );
+    tedFormatCleanPageLayoutTool( &(tft->tftPageLayoutTool) );
+    tedFormatCleanHeaderFooterTool( &(tft->tftHeaderFooterTool) );
+    tedFormatCleanNotesTool( &(tft->tftNotesTool) );
+    appFontChooserCleanPage( &(tft->tftFontTool) );
+    appRgbChooserPageCleanPage( &(tft->tftRgbPage) );
 
     free( tft );
 
@@ -345,6 +174,69 @@ static void tedDestroyFormatTool(	void *	voidtft )
     return;
     }
 
+/************************************************************************/
+/*									*/
+/*  Intermediary routine to refresh the font tool.			*/
+/*									*/
+/************************************************************************/
+
+static void tedRefreshFontTool(	AppFontChooser *		afc,
+				int *				pEnabled,
+				InspectorSubject *		is,
+				const DocumentSelection *	ds,
+				BufferDocument *		bd )
+    {
+    const DocumentProperties *	dp= &(bd->bdProperties);
+    const DocumentFontList *	dfl= &(dp->dpFontList);
+
+    PropertyMask		updMask;
+    PropertyMask		doneMask;
+
+    TextAttribute		ta;
+    ExpandedTextAttribute	eta;
+
+    PROPmaskCLEAR( &updMask );
+    PROPmaskCLEAR( &doneMask );
+
+    utilInitTextAttribute( &ta );
+    docInitExpandedTextAttribute( &eta );
+
+    tedGetSelectionAttributes( bd, ds, &updMask, &ta );
+
+    docExpandTextAttribute( &doneMask, &eta, &ta, &updMask, dfl,
+					    dp->dpColors, dp->dpColorCount );
+
+    if  ( appFontSetCurrentFont( afc, &updMask, &eta,
+					    dp->dpColors, dp->dpColorCount ) )
+	{ LDEB(1);	}
+
+    docCleanExpandedTextAttribute( &eta );
+
+    *pEnabled= 1;
+
+    return;
+    }
+
+/************************************************************************/
+/*									*/
+/*  Called when the user switches to another page of the format tool.	*/
+/*									*/
+/************************************************************************/
+
+static void tedFormatToolNotifySubject(	AppInspector *		ai,
+					int			subjectFrom,
+					int			subjectTo )
+    {
+    if  ( subjectFrom == TEDtsiRGB && subjectTo != TEDtsiRGB )
+	{
+	const int	enabled= 0;
+
+	appEnableInspectorSubject( ai, TEDtsiRGB, enabled );
+	}
+
+    return;
+    }
+
 /************************************************************************/
 /*									*/
 /*  make/show the format tool.						*/
@@ -358,8 +250,7 @@ void tedShowFormatTool(	APP_WIDGET		option,
     {
     AppInspector *			ai;
     TedFormatTool *			tft;
-    int					i;
-   	 
+
     static TedFormatToolResources	tftr;
     static int				gotResources;
 
@@ -394,14 +285,18 @@ void tedShowFormatTool(	APP_WIDGET		option,
 				&(tftr.tttrCellPageResources),
 				&(tftr.tttrSubjectResources[TEDtsiCELL]) );
 
-	tedFormatToolGetParaResourceTable( ea,
-				&(tftr.tttrParaPageResources),
-				&(tftr.tttrSubjectResources[TEDtsiPARA]) );
+	tedFormatToolGetParaLayoutResourceTable( ea,
+				&(tftr.tttrParaLayoutPageResources),
+				&(tftr.tttrSubjectResources[TEDtsiPARA_LAY]) );
 
 	tedFormatToolGetTabsResourceTable( ea,
 				&(tftr.tttrTabsPageResources),
 				&(tftr.tttrSubjectResources[TEDtsiTABS]) );
 
+	tedFormatToolGetParaOrnamentsResourceTable( ea,
+				&(tftr.tttrParaOrnamentsPageResources),
+				&(tftr.tttrSubjectResources[TEDtsiPARA_ORN]) );
+
 	tedFormatToolGetSectResourceTable( ea,
 				&(tftr.tttrSectPageResources),
 				&(tftr.tttrSubjectResources[TEDtsiSECT]) );
@@ -418,6 +313,14 @@ void tedShowFormatTool(	APP_WIDGET		option,
 				&(tftr.tttrNotesPageResources),
 				&(tftr.tttrSubjectResources[TEDtsiNOTES]) );
 
+	appFontToolGetResourceTable( ea,
+				&(tftr.tttrFontToolResources),
+				&(tftr.tttrSubjectResources[TEDtsiFONT]) );
+
+	appRgbChooserPageGetResourceTable( ea,
+				&(tftr.tttrRgbChooserPageResources),
+				&(tftr.tttrSubjectResources[TEDtsiRGB]) );
+
 	gotResources= 1;
 	}
 
@@ -427,79 +330,107 @@ void tedShowFormatTool(	APP_WIDGET		option,
 
     tft->tftApplication= ea;
 
+    /******/
+    tedInitColumnTool( &(tft->tftColumnTool) );
+    tedInitCellTool( &(tft->tftCellTool) );
+    tedInitParaOrnamentsTool( &(tft->tftParagraphOrnamentsTool) );
+    tedInitRowTool( &(tft->tftRowTool) );
+
+    /******/
+    tft->tftTableTool.ttApplication= ea;
+    tft->tftRowTool.rtApplication= ea;
+    tft->tftColumnTool.ctApplication= ea;
+    tft->tftCellTool.ctApplication= ea;
+    tft->tftParagraphLayoutTool.ptApplication= ea;
+    tft->tftTabsTool.ttApplication= ea;
+    tft->tftParagraphOrnamentsTool.potApplication= ea;
     tft->tftSectionTool.stApplication= ea;
     tft->tftPageLayoutTool.pltApplication= ea;
     tft->tftHeaderFooterTool.hftApplication= ea;
-    tft->tftParagraphTool.ptApplication= ea;
-    tft->tftColumnTool.ctApplication= ea;
-    tft->tftRowTool.rtApplication= ea;
-    tft->tftTableTool.ttApplication= ea;
     tft->tftNotesTool.ntApplication= ea;
-    tft->tftTabsTool.ttApplication= ea;
-
-    appInitDrawingData( &(tft->tftPixmapAdd) );
-    tft->tftPixmapAddAllocated= 0;
-
-    for ( i= 0; i < DOCsp_COUNT; i++ )
-	{ tft->tftShadingPixmaps[i]= (APP_BITMAP_IMAGE)0; }
-    for ( i= 0; i < DOCbs_COUNT; i++ )
-	{ tft->tftBorderPixmaps[i]= (APP_BITMAP_IMAGE)0; }
+    tft->tftFontTool.afcApplication= ea;
+    tft->tftRgbPage.rcpApplication= ea;
 
     ai= appMakeInspector( ea, option, pixmapName, widgetName,
 				    tftr.tttrSubjectResources, TEDtsi_COUNT,
 				    tedDestroyFormatTool, (void *)tft );
 
     tft->tftInspector= ai;
+    ai->aiNotifySubject= tedFormatToolNotifySubject;
 
+    /******/
+    tft->tftTableTool.ttInspector= ai;
+    tft->tftRowTool.rtInspector= ai;
+    tft->tftColumnTool.ctInspector= ai;
+    tft->tftCellTool.ctInspector= ai;
+    tft->tftParagraphLayoutTool.ptInspector= ai;
+    tft->tftTabsTool.ttInspector= ai;
+    tft->tftParagraphOrnamentsTool.potInspector= ai;
     tft->tftSectionTool.stInspector= ai;
     tft->tftPageLayoutTool.pltInspector= ai;
     tft->tftHeaderFooterTool.hftInspector= ai;
-    tft->tftParagraphTool.ptInspector= ai;
-    tft->tftColumnTool.ctInspector= ai;
-    tft->tftRowTool.rtInspector= ai;
-    tft->tftTableTool.ttInspector= ai;
     tft->tftNotesTool.ntInspector= ai;
-    tft->tftTabsTool.ttInspector= ai;
+    tft->tftFontTool.afcInspector= ai;
+    tft->tftRgbPage.rcpInspector= ai;
+
+    /******/
 
     tedFormatFillPages( &tftr, tft, ai );
 
+    /******/
+
+    tft->tftFontTool.afcSetFont= tedFontToolSet;
+
+    /******/
+    /*tedFormatFillTableChoosers()*/
     tedFormatFillRowChoosers( &(tft->tftRowTool),
 					    &(tftr.tttrRowPageResources) );
-    tedFormatFillParagraphChoosers( &(tft->tftParagraphTool),
-					    &(tftr.tttrParaPageResources) );
+    tedFormatFillColumnChoosers( &(tft->tftColumnTool),
+					&(tftr.tttrColumnPageResources) );
+    tedFormatFillCellChoosers( &(tft->tftCellTool),
+					&(tftr.tttrCellPageResources) );
+    tedFormatFillParagraphLayoutChoosers( &(tft->tftParagraphLayoutTool),
+				    &(tftr.tttrParaLayoutPageResources) );
     tedFormatFillTabsChoosers( &(tft->tftTabsTool) );
+    tedFormatFillParagraphOrnamentsChoosers( &(tft->tftParagraphOrnamentsTool),
+				    &(tftr.tttrParaOrnamentsPageResources) );
     tedFormatFillSectionChoosers( &(tft->tftSectionTool),
 					    &(tftr.tttrSectPageResources) );
     appPageLayoutPageFillChoosers( &(tft->tftPageLayoutTool),
 					&(tftr.tttrPageLayoutPageResources) );
     tedFormatFillHeaderFooterChoosers( &(tft->tftHeaderFooterTool),
 					&(tftr.tttrHeadFootPageResources) );
+    appFontToolFillChoosers( &(tft->tftFontTool),
+					&(tftr.tttrFontToolResources) );
     tedFormatFillNotesChoosers( &(tft->tftNotesTool),
 					&(tftr.tttrNotesPageResources) );
-    tedFormatFillColumnChoosers( &(tft->tftColumnTool),
-					&(tftr.tttrColumnPageResources) );
-    tedFormatFillCellChoosers( &(tft->tftCellTool),
-					&(tftr.tttrCellPageResources) );
+    /******/
 
     appFinishInspector( ai );
 
     tedFormatFinishRowPage( &(tft->tftRowTool), tft,
 					    &(tftr.tttrRowPageResources) );
-    tedFormatFinishParagraphPage( &(tft->tftParagraphTool), tft,
-					    &(tftr.tttrParaPageResources) );
+    tedFormatFinishParaLayoutPage( &(tft->tftParagraphLayoutTool), tft,
+					&(tftr.tttrParaLayoutPageResources) );
     tedFormatFinishTabsPage( &(tft->tftTabsTool) );
+    tedFormatFinishParaOrnamentsPage( &(tft->tftParagraphOrnamentsTool), tft,
+				    &(tftr.tttrParaOrnamentsPageResources) );
     tedFormatFinishSectionPage( &(tft->tftSectionTool), tft,
 					    &(tftr.tttrSectPageResources) );
     tedFormatFinishPageLayoutPage( &(tft->tftPageLayoutTool), tft,
 					&(tftr.tttrPageLayoutPageResources) );
     tedFormatFinishHeaderFooterPage( &(tft->tftHeaderFooterTool), tft,
 					&(tftr.tttrHeadFootPageResources) );
-    tedFormatFinishNotesPage( &(tft->tftNotesTool), tft,
-					&(tftr.tttrNotesPageResources) );
     tedFormatFinishColumnPage( &(tft->tftColumnTool), tft,
 					&(tftr.tttrColumnPageResources) );
     tedFormatFinishCellPage( &(tft->tftCellTool), tft,
 					&(tftr.tttrCellPageResources) );
+    tedFormatFinishNotesPage( &(tft->tftNotesTool), tft,
+					&(tftr.tttrNotesPageResources) );
+    appFontToolFinishPage( &(tft->tftFontTool),
+					&(tftr.tttrFontToolResources) );
+    appRgbChooserPageFinishPage( &(tft->tftRgbPage),
+					&(tftr.tttrRgbChooserPageResources) );
 
     tar->tarInspector= ai;
 
@@ -526,6 +457,20 @@ void tedFormatShowPagePage(	EditApplication *	ea )
     return;
     }
 
+void tedFormatShowFontPage(	EditApplication *	ea )
+    {
+    TedAppResources *		tar= (TedAppResources *)ea->eaResourceData;
+
+    if  ( ! tar->tarInspector )
+	{ XDEB(tar->tarInspector); return;	}
+
+    appEnableInspectorSubject( tar->tarInspector, TEDtsiFONT, 1 );
+
+    appInspectorSelectSubject( tar->tarInspector, TEDtsiFONT );
+
+    return;
+    }
+
 static void tedFormatRefreshToolPages(	int *				enabled,
 					TedFormatTool *			tft,
 					AppInspector *			ai,
@@ -549,15 +494,19 @@ static void tedFormatRefreshToolPages(	int *				enabled,
 				    enabled+ TEDtsiCELL,
 				    ai->aiSubjects+ TEDtsiCELL, ds );
 
-    tedFormatToolRefreshParagraphTool(  &(tft->tftParagraphTool),
-				    enabled+ TEDtsiPARA,
-				    ai->aiSubjects+ TEDtsiPARA, ds );
+    tedFormatToolRefreshParaLayoutTool(  &(tft->tftParagraphLayoutTool),
+				    enabled+ TEDtsiPARA_LAY,
+				    ai->aiSubjects+ TEDtsiPARA_LAY, ds );
 
     tedFormatToolRefreshTabsTool( &(tft->tftTabsTool),
 				    enabled+ TEDtsiTABS,
 				    ai->aiSubjects+ TEDtsiTABS,
 				    ds, &(bd->bdProperties) );
 
+    tedFormatToolRefreshParaOrnamentsTool(  &(tft->tftParagraphOrnamentsTool),
+				    enabled+ TEDtsiPARA_ORN,
+				    ai->aiSubjects+ TEDtsiPARA_ORN, ds );
+
     tedFormatToolRefreshSectionTool( &(tft->tftSectionTool),
 				    enabled+ TEDtsiSECT,
 				    ai->aiSubjects+ TEDtsiSECT, ds );
@@ -577,6 +526,14 @@ static void tedFormatRefreshToolPages(	int *				enabled,
 				    ai->aiSubjects+ TEDtsiNOTES,
 				    ds, bd );
 
+    tedRefreshFontTool( &(tft->tftFontTool),
+				    enabled+ TEDtsiFONT,
+				    ai->aiSubjects+ TEDtsiFONT,
+				    ds, bd );
+
+    /*  no refresh  */
+    enabled[TEDtsiRGB]= 0;
+
     return;
     }
 
