@@ -4,18 +4,25 @@
 /*									*/
 /************************************************************************/
 
-#   include	"config.h"
+#   include	"tedConfig.h"
 
 #   include	<stdlib.h>
 #   include	<stdio.h>
 #   include	<stddef.h>
 #   include	<limits.h>
 
-#   include	<appDebugon.h>
-
 #   include	"tedApp.h"
 #   include	"tedFormatTool.h"
 
+#   include	<appDebugon.h>
+
+/************************************************************************/
+/*									*/
+/*  Set all the controls on the Header/Footer tool, to reflect the	*/
+/*  section properties currently chosen.				*/
+/*									*/
+/************************************************************************/
+
 static void tedFormatToolRefreshHeadFootPage(	HeaderFooterTool *	hft )
     {
     const SectionProperties *	sp= &(hft->hftSectPropertiesChosen);
@@ -51,14 +58,22 @@ static void tedFormatToolRefreshHeadFootPage(	HeaderFooterTool *	hft )
     return;
     }
 
+/************************************************************************/
+/*									*/
+/*  Adapt the Header/Footer tool to the current selection in the text.	*/
+/*									*/
+/************************************************************************/
+
 void tedFormatToolRefreshHeaderFooterTool(
-					HeaderFooterTool *		hft,
-					InspectorSubject *		is,
-					const BufferSelection *		bs,
-					const DocumentProperties *	dpSet )
+				HeaderFooterTool *		hft,
+				int *				pEnabled,
+				InspectorSubject *		is,
+				const DocumentSelection *	ds,
+				const SelectionGeometry *	sg,
+				const DocumentProperties *	dpSet )
     {
-    unsigned int		changed;
-    unsigned int		update;
+    PropertyMask		changed;
+    PropertyMask		update;
 
     BufferItem *		sectBi;
     const SectionProperties *	spSet;
@@ -67,20 +82,20 @@ void tedFormatToolRefreshHeaderFooterTool(
 
     PROPmaskCLEAR( &changed );
 
-    sectBi= bs->bsBegin.bpBi;
+    sectBi= ds->dsBegin.dpBi;
     while( sectBi && sectBi->biLevel > DOClevSECT )
 	{ sectBi= sectBi->biParent;	}
 
     if  ( ! sectBi )
 	{ XDEB(sectBi); return;	}
 
-    inHeaderFooter= sectBi->biInHeaderFooter;
+    inHeaderFooter= sectBi->biInExternalItem;
     if  ( inHeaderFooter == DOCinBODY )
 	{
-	HeaderFooter *	hf;
-	int		page= bs->bsBegin.bpTopPosition.lpPage;
+	ExternalItem *	ei;
+	int		page= sg->sgBegin.pgTopPosition.lpPage;
 
-	inHeaderFooter= docWhatPageHeader( &hf, sectBi, page, dpSet );
+	inHeaderFooter= docWhatPageHeader( &ei, sectBi, page, dpSet );
 	}
 
     hft->hftSectionNumber= sectBi->biNumberInParent;
@@ -92,20 +107,20 @@ void tedFormatToolRefreshHeaderFooterTool(
     PROPmaskADD( &update, DPpropFACING_PAGES );
 
     if  ( docUpdDocumentProperties( &changed, &(hft->hftDocPropertiesChosen),
-							update, dpSet ) )
+							&update, dpSet ) )
 	{ LDEB(1); return;	}
     if  ( docUpdDocumentProperties( &changed, &(hft->hftDocPropertiesSet),
-							update, dpSet ) )
+							&update, dpSet ) )
 	{ LDEB(1); return;	}
 
     PROPmaskCLEAR( &update );
     PROPmaskADD( &update, SPpropTITLEPG );
 
     if  ( docUpdSectProperties( &changed, &(hft->hftSectPropertiesChosen),
-							update, spSet ) )
+							&update, spSet ) )
 	{ LDEB(1); return;	}
     if  ( docUpdSectProperties( &changed, &(hft->hftSectPropertiesSet),
-							update, spSet ) )
+							&update, spSet ) )
 	{ LDEB(1); return;	}
 
     /**********/
@@ -153,15 +168,30 @@ void tedFormatToolRefreshHeaderFooterTool(
 	    appSetOptionmenu( &(hft->hftPagesOptionmenu), hft->hftPagesChosen );
 	    break;
 
+	case DOCinFOOTNOTE:
+	case DOCinENDNOTE:
+
 	case DOCinBODY:
+
+	case DOCinFTNSEP:
+	case DOCinFTNSEPC:
+	case DOCinFTNCN:
+	case DOCinAFTNSEP:
+	case DOCinAFTNSEPC:
+	case DOCinAFTNCN:
+
+	    *pEnabled= 0; return;
+
 	default:
-	    LDEB(inHeaderFooter); break;
+	    LDEB(inHeaderFooter);
+	    *pEnabled= 0; return;
 	}
 
     /**********/
 
     tedFormatToolRefreshHeadFootPage( hft );
 
+    *pEnabled= 1;
     return;
     }
 
@@ -171,50 +201,44 @@ void tedFormatToolRefreshHeaderFooterTool(
 /*									*/
 /************************************************************************/
 
-static void tedHeadFootChangeDocPushed(		APP_WIDGET	w,
-						void *		voidhft,
-						void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( tedHeadFootChangeDocPushed, w, voidhft )
     {
     HeaderFooterTool *		hft= (HeaderFooterTool *)voidhft;
     DocumentProperties *	dp= &(hft->hftDocPropertiesChosen);
 
-    unsigned int		update;
+    PropertyMask		update;
 
     PROPmaskCLEAR( &update );
     PROPmaskADD( &update, DPpropFACING_PAGES );
 
-    tedSetDocumentProperties( hft->hftApplication, dp, update );
+    tedSetDocumentProperties( hft->hftApplication, dp, &update );
 
     return;
     }
 
-static void tedHeadFootChangeSectPushed(	APP_WIDGET	w,
-						void *		voidhft,
-						void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( tedHeadFootChangeSectPushed, w, voidhft )
     {
     HeaderFooterTool *		hft= (HeaderFooterTool *)voidhft;
     SectionProperties *		sp= &(hft->hftSectPropertiesChosen);
 
-    unsigned int		update;
+    PropertyMask		update;
 
     PROPmaskCLEAR( &update );
     PROPmaskADD( &update, SPpropTITLEPG );
 
-    if  ( tedAppChangeSectionProperties( hft->hftApplication, update, sp ) )
+    if  ( tedAppChangeSectionProperties( hft->hftApplication, &update, sp ) )
 	{ LDEB(1);	}
 
     return;
     }
 
-static void tedHeadFootRevertDocPushed(		APP_WIDGET	w,
-						void *		voidhft,
-						void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( tedHeadFootRevertDocPushed, w, voidhft )
     {
     HeaderFooterTool *		hft= (HeaderFooterTool *)voidhft;
     DocumentProperties *	dp= &(hft->hftDocPropertiesChosen);
 
-    unsigned int		update;
-    unsigned int		changed;
+    PropertyMask		update;
+    PropertyMask		changed;
 
     PROPmaskCLEAR( &changed );
 
@@ -222,7 +246,7 @@ static void tedHeadFootRevertDocPushed(		APP_WIDGET	w,
     PROPmaskADD( &update, DPpropFACING_PAGES );
 
     if  ( docUpdDocumentProperties( &changed, dp,
-				    update, &(hft->hftDocPropertiesSet) ) )
+				    &update, &(hft->hftDocPropertiesSet) ) )
 	{ LDEB(1); return;	}
 
     tedFormatToolRefreshHeadFootPage( hft );
@@ -230,15 +254,13 @@ static void tedHeadFootRevertDocPushed(		APP_WIDGET	w,
     return;
     }
 
-static void tedHeadFootRevertSectPushed(	APP_WIDGET	w,
-						void *		voidhft,
-						void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( tedHeadFootRevertSectPushed, w, voidhft )
     {
     HeaderFooterTool *		hft= (HeaderFooterTool *)voidhft;
     SectionProperties *		sp= &(hft->hftSectPropertiesChosen);
 
-    unsigned int		update;
-    unsigned int		changed;
+    PropertyMask		update;
+    PropertyMask		changed;
 
     PROPmaskCLEAR( &changed );
 
@@ -246,7 +268,7 @@ static void tedHeadFootRevertSectPushed(	APP_WIDGET	w,
     PROPmaskADD( &update, SPpropTITLEPG );
 
     if  ( docUpdSectProperties( &changed, sp,
-				    update, &(hft->hftSectPropertiesSet) ) )
+				    &update, &(hft->hftSectPropertiesSet) ) )
 	{ LDEB(1); return;	}
 
     tedFormatToolRefreshHeadFootPage( hft );
@@ -261,9 +283,7 @@ static void tedHeadFootRevertSectPushed(	APP_WIDGET	w,
 /*									*/
 /************************************************************************/
 
-static void tedHeaderFooterToggleTitlepg(	APP_WIDGET	w,
-						void *		voidhft,
-						void *		voidtbcs )
+static APP_TOGGLE_CALLBACK_H( tedHeaderFooterToggleTitlepg, w, voidhft, voidtbcs )
     {
     HeaderFooterTool *		hft= (HeaderFooterTool *)voidhft;
     SectionProperties *		sp= &(hft->hftSectPropertiesChosen);
@@ -276,9 +296,7 @@ static void tedHeaderFooterToggleTitlepg(	APP_WIDGET	w,
     return;
     }
 
-static void tedHeaderFooterToggleFacingp(	APP_WIDGET	w,
-						void *		voidhft,
-						void *		voidtbcs )
+static APP_TOGGLE_CALLBACK_H( tedHeaderFooterToggleFacingp, w, voidhft, voidtbcs )
     {
     HeaderFooterTool *		hft= (HeaderFooterTool *)voidhft;
     DocumentProperties *	dp= &(hft->hftDocPropertiesChosen);
@@ -297,23 +315,17 @@ static void tedHeaderFooterToggleFacingp(	APP_WIDGET	w,
 /*									*/
 /************************************************************************/
 
-static void tedHeadFootPagesChosen(	APP_WIDGET	w,
-					void *		voidhft,
-					void *		voidtbcs )
+static APP_OITEM_CALLBACK_H( tedHeadFootPagesChosen, w, voidhft )
     {
     HeaderFooterTool *		hft= (HeaderFooterTool *)voidhft;
 
     int				pages= -1;
 
-    for ( pages= 0; pages < PAGES__COUNT; pages++ )
-	{
-	if  ( hft->hftPagesItems[pages] == w )
-	    { break;	}
-	}
+    pages= appGuiGetOptionmenuItemIndex( &(hft->hftPagesOptionmenu), w );
+    if  ( pages < 0 || pages >= PAGES__COUNT )
+	{ LLDEB(pages,PAGES__COUNT); return;		}
 
-    if  ( pages >= 0 && pages < PAGES__COUNT )
-	{ hft->hftPagesChosen= pages;		}
-    else{ LLDEB(pages,PAGES__COUNT);		}
+    hft->hftPagesChosen= pages;
 
     return;
     }
@@ -325,27 +337,7 @@ static void tedHeadFootPagesChosen(	APP_WIDGET	w,
 /*									*/
 /************************************************************************/
 
-static const int TED_HeaderScopes[PAGES__COUNT]=
-    {
-    DOCinFIRST_HEADER,
-    DOCinSECT_HEADER,
-    DOCinSECT_HEADER,
-    DOCinRIGHT_HEADER,
-    DOCinLEFT_HEADER,
-    };
-
-static const int TED_FooterScopes[PAGES__COUNT]=
-    {
-    DOCinFIRST_FOOTER,
-    DOCinSECT_FOOTER,
-    DOCinSECT_FOOTER,
-    DOCinRIGHT_FOOTER,
-    DOCinLEFT_FOOTER,
-    };
-
-static void tedHeadFootDeleteHeaderPushed(	APP_WIDGET	w,
-						void *		voidhft,
-						void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( tedHeadFootDeleteHeaderPushed, w, voidhft )
     {
     HeaderFooterTool *		hft= (HeaderFooterTool *)voidhft;
 
@@ -354,13 +346,11 @@ static void tedHeadFootDeleteHeaderPushed(	APP_WIDGET	w,
 	{ LLDEB(hft->hftPagesChosen,PAGES__COUNT); return;	}
 
     tedAppDeleteHeaderFooter( hft->hftApplication,
-				    TED_HeaderScopes[hft->hftPagesChosen] );
+				    DOC_HeaderScopes[hft->hftPagesChosen] );
     return;
     }
 
-static void tedHeadFootEditHeaderPushed(	APP_WIDGET	button,
-						void *		voidhft,
-						void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( tedHeadFootEditHeaderPushed, w, voidhft )
     {
     HeaderFooterTool *		hft= (HeaderFooterTool *)voidhft;
     APP_WIDGET			relative;
@@ -371,14 +361,13 @@ static void tedHeadFootEditHeaderPushed(	APP_WIDGET	button,
 
     relative= hft->hftInspector->aiTopWidget;
 
-    tedAppEditHeaderFooter( hft->hftApplication, relative, button,
-				    TED_HeaderScopes[hft->hftPagesChosen] );
+    tedAppEditHeaderFooter( hft->hftApplication, relative,
+				    hft->hftEditHeaderButton,
+				    DOC_HeaderScopes[hft->hftPagesChosen] );
     return;
     }
 
-static void tedHeadFootDeleteFooterPushed(	APP_WIDGET	w,
-						void *		voidhft,
-						void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( tedHeadFootDeleteFooterPushed, w, voidhft )
     {
     HeaderFooterTool *		hft= (HeaderFooterTool *)voidhft;
 
@@ -387,13 +376,11 @@ static void tedHeadFootDeleteFooterPushed(	APP_WIDGET	w,
 	{ LLDEB(hft->hftPagesChosen,PAGES__COUNT); return;	}
 
     tedAppDeleteHeaderFooter( hft->hftApplication,
-				    TED_FooterScopes[hft->hftPagesChosen] );
+				    DOC_FooterScopes[hft->hftPagesChosen] );
     return;
     }
 
-static void tedHeadFootEditFooterPushed(	APP_WIDGET	button,
-						void *		voidhft,
-						void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( tedHeadFootEditFooterPushed, w, voidhft )
     {
     HeaderFooterTool *		hft= (HeaderFooterTool *)voidhft;
     APP_WIDGET			relative;
@@ -404,8 +391,9 @@ static void tedHeadFootEditFooterPushed(	APP_WIDGET	button,
 
     relative= hft->hftInspector->aiTopWidget;
 
-    tedAppEditHeaderFooter( hft->hftApplication, relative, button,
-				    TED_FooterScopes[hft->hftPagesChosen] );
+    tedAppEditHeaderFooter( hft->hftApplication, relative,
+				    hft->hftEditFooterButton,
+				    DOC_FooterScopes[hft->hftPagesChosen] );
     return;
     }
 
@@ -425,12 +413,16 @@ void tedFormatFillHeaderFooterPage( HeaderFooterTool *		hft,
 
     const int	heightResizable= 0;
 
+    /**/
+    hft->hftPageResources= hfpr;
+
+    /**/
     docInitDocumentProperties( &(hft->hftDocPropertiesSet) );
     docInitDocumentProperties( &(hft->hftDocPropertiesChosen) );
     docInitSectionProperties( &(hft->hftSectPropertiesSet) );
     docInitSectionProperties( &(hft->hftSectPropertiesChosen) );
 
-    /**************/
+    /**/
 
     appMakeColumnFrameInColumn( &(hft->hftSectionFrame),
 			    &(hft->hftSectionPaned),
@@ -516,6 +508,22 @@ void tedFormatFinishHeaderFooterPage( HeaderFooterTool *		hft,
     return;
     }
 
+/************************************************************************/
+/*									*/
+/*  Clean the header footer tool.					*/
+/*									*/
+/************************************************************************/
+
+void tedFormatCleanHeaderFooterTool(	HeaderFooterTool *		hft )
+    {
+    docCleanDocumentProperties( &(hft->hftDocPropertiesChosen) );
+    docCleanDocumentProperties( &(hft->hftDocPropertiesSet) );
+    docCleanSectionProperties( &(hft->hftSectPropertiesChosen) );
+    docCleanSectionProperties( &(hft->hftSectPropertiesSet) );
+
+    return;
+    }
+
 /************************************************************************/
 /*									*/
 /*  Get Header/Footer tool resources.					*/
