@@ -1,24 +1,24 @@
 /************************************************************************/
 /*									*/
-/*  A Format Tool.							*/
+/*  Widget to select a height on the Format Tool.			*/
 /*									*/
 /************************************************************************/
 
-#   include	"config.h"
+#   include	"tedConfig.h"
 
 #   include	<stdlib.h>
 #   include	<stdio.h>
 #   include	<stddef.h>
 #   include	<limits.h>
 
-#   include	<appDebugon.h>
-
 #   include	<appGeoString.h>
 #   include	<appUnit.h>
 
 #   include	"tedApp.h"
 #   include	"tedFormatTool.h"
 
+#   include	<appDebugon.h>
+
 /************************************************************************/
 /*									*/
 /*  Refresh a height chooser.						*/
@@ -108,22 +108,12 @@ int tedFormatToolGetHeight(	int *		pHeight,
 /*									*/
 /************************************************************************/
 
-void tedFormatHeightChosen(	APP_WIDGET		w,
+void tedFormatHeightChosen(	int			how,
 				HeightChooser *		hc,
 				int			defaultValue )
     {
     char		scratch[50];
     int			val;
-    int			how;
-
-    for ( how= 0; how < HC__COUNT; how++ )
-	{
-	if  ( hc->hcMenuItems[how] == w )
-	    { break;	}
-	}
-
-    if  ( how < 0 || how >= HC__COUNT )
-	{ LDEB(how); return;	}
 
     switch( how )
 	{
@@ -158,11 +148,11 @@ void tedFormatHeightChosen(	APP_WIDGET		w,
     LDEB(1);
     }
 
-void tedFormatMakeHeightRow(	APP_WIDGET *		pRow,
-				void *			through,
-				APP_WIDGET		parent,
-				HeightChooser *		hc,
-				APP_CALLBACK_FUNC	callback )
+void tedFormatMakeHeightRow(	APP_WIDGET *			pRow,
+				void *				through,
+				APP_WIDGET			parent,
+				HeightChooser *			hc,
+				APP_TXACTIVATE_CALLBACK_T	callback )
     {
     APP_WIDGET	row;
     APP_WIDGET	text;
@@ -195,7 +185,7 @@ void tedFormatMakeHeightRow(	APP_WIDGET *		pRow,
 /************************************************************************/
 
 void tedFormatFillHeightChooser(	HeightChooser *		hc,
-					APP_CALLBACK_FUNC	callback,
+					APP_OITEM_CALLBACK_T	callback,
 					void *			voidtft,
 					const char *		freeText,
 					const char *		atLeastText,
@@ -218,3 +208,22 @@ void tedFormatFillHeightChooser(	HeightChooser *		hc,
     appOptionmenuRefreshWidth( &(hc->hcOptionmenu) );
     }
 
+/************************************************************************/
+/*									*/
+/*  Initialize.								*/
+/*									*/
+/************************************************************************/
+
+void tedInitHeightChooser(	HeightChooser *		hc )
+    {
+    int		i;
+
+    appInitOptionmenu( &(hc->hcOptionmenu) );
+    hc->hcText= (APP_WIDGET)0;
+
+    for  ( i= 0; i < HC__COUNT; i++ )
+	{ hc->hcMenuItems[i]= (APP_WIDGET)0;	}
+
+    hc->hcHow= HC_FREE;
+    }
+
