@@ -4,14 +4,14 @@
 /*									*/
 /************************************************************************/
 
-#   include	"config.h"
+#   include	"tedConfig.h"
 
 #   include	<stddef.h>
 #   include	<stdlib.h>
 #   include	<stdio.h>
 #   include	<ctype.h>
 
-#   include	"docPs.h"
+#   include	"docLayout.h"
 #   include	"tedApp.h"
 #   include	"tedRuler.h"
 
@@ -28,9 +28,6 @@
 
 #   include	<X11/keysym.h>
 
-#   define KEY_CONTROL_MASK	ControlMask
-#   define KEY_SHIFT_MASK	ShiftMask
-
 #  ifdef XK_ISO_Left_Tab
 #   define KEY_ISO_Left_Tab	XK_ISO_Left_Tab
 #  endif
@@ -41,8 +38,20 @@
 #   define KEY_KP_Enter		XK_KP_Enter
 #   define KEY_Return		XK_Return
 #   define KEY_KP_Delete	XK_KP_Delete
-#   define KEY_Delete		XK_Delete
-#   define KEY_BackSpace	XK_BackSpace
+
+#   ifdef __VMS
+	/* VMS normally has only a "Delete" configured which is on the same
+	 * location on the keyboard and serves the same purposes as the
+	 * BackSpace on Unix. To simulate a BackSpace on VMS normally
+	 * <CTRL>H is used.
+	 */
+#	define KEY_Delete	XK_BackSpace
+#	define KEY_BackSpace	XK_Delete
+#   else
+#	define KEY_Delete	XK_Delete
+#	define KEY_BackSpace	XK_BackSpace
+#   endif
+
 #   define KEY_KP_Home		XK_KP_Home
 #   define KEY_Home		XK_Home
 #   define KEY_KP_End		XK_KP_End
@@ -79,9 +88,6 @@
 
 #   include	<gdk/gdkkeysyms.h>
 
-#   define KEY_CONTROL_MASK	GDK_CONTROL_MASK
-#   define KEY_SHIFT_MASK	GDK_SHIFT_MASK
-
 #   define KEY_ISO_Left_Tab	GDK_ISO_Left_Tab
 #   define KEY_i		GDK_i
 #   define KEY_Tab		GDK_Tab
@@ -160,48 +166,51 @@ static void tedLogEvent(	Widget		w,
 /*  Handle mouse clicks.						*/
 /*									*/
 /************************************************************************/
+
 #   define	TED_DRAG_INTERVAL	(150L)
 
 typedef struct DraggingContext
     {
     int				dcMouseX;
     int				dcMouseY;
-    BufferPosition		dcAnchorPosition;
+    DocumentPosition		dcAnchorPosition;
     BufferItem *		dcRootItem;
-    SelectionScope		dcSelectionScope;
     EditDocument *		dcEd;
     } DraggingContext;
 
-static APP_EVENT_HANDLER( tedInputDragMouseMove, w, vdc, event )
+static APP_EVENT_HANDLER_H( tedInputDragMouseMove, w, vdc, event )
     {
     DraggingContext *	dc= (DraggingContext *)vdc;
+    EditDocument *	ed= dc->dcEd;
 
-    if  ( appGetCoordinatesFromMouseMoveEvent(
-			    &(dc->dcMouseX), &(dc->dcMouseY), w, event ) )
+    int			ox= ed->edVisibleRect.drX0;
+    int			oy= ed->edVisibleRect.drY0;
+
+    int			mouseX;
+    int			mouseY;
+
+    if  ( appGetCoordinatesFromMouseMoveEvent( &mouseX, &mouseY, w, event ) )
 	{ return;	}
 
+    if  ( mouseX < -ox )
+	{ mouseX=  -ox;	}
+    if  ( mouseY < -oy )
+	{ mouseY=  -oy;	}
+
+    dc->dcMouseX= mouseX;
+    dc->dcMouseY= mouseY;
+
     if  ( tedExtendSelectionToXY( dc->dcEd, dc->dcRootItem,
-				    &(dc->dcAnchorPosition),
-				    &(dc->dcSelectionScope),
-				    dc->dcMouseX, dc->dcMouseY ) )
+			&(dc->dcAnchorPosition), dc->dcMouseX, dc->dcMouseY ) )
 	{ LDEB(1); return;	}
 
     return;
     }
 
-static APP_EVENT_HANDLER( tedInputDragMouseUp, w, vdc, event )
+static APP_EVENT_HANDLER_H( tedInputDragMouseUp, w, vdc, event )
     {
     DraggingContext *		dc= (DraggingContext *)vdc;
     EditDocument *		ed= dc->dcEd;
-    TedDocument *		td= (TedDocument *)ed->edPrivateData;
-
-    BufferSelection		bsNew= td->tdSelection;
-
-    int				scrolledX= 0;
-    int				scrolledY= 0;
-
-    tedSetSelection( ed, &(dc->dcSelectionScope), &bsNew,
-						    &scrolledX, &scrolledY );
 
     tedAdaptToolsToSelection( ed );
 
@@ -217,8 +226,8 @@ static APP_TIMER_HANDLER( tedTick, voiddc )
     int				ox= ed->edVisibleRect.drX0;
     int				oy= ed->edVisibleRect.drY0;
 
-    int				mouse_x;
-    int				mouse_y;
+    int				mouseX;
+    int				mouseY;
 
     int				scrolledX= 0;
     int				scrolledY= 0;
@@ -227,43 +236,49 @@ static APP_TIMER_HANDLER( tedTick, voiddc )
     {
     Window		root;
     Window		child;
-    int			root_x;
-    int			root_y;
-    int			win_x;
-    int			win_y;
+    int			rootX;
+    int			rootY;
+    int			winX;
+    int			winY;
     unsigned int	mask;
 
     XQueryPointer( add->addDisplay, add->addDrawable,
 						    &root, &child,
-						    &root_x, &root_y,
-						    &win_x, &win_y, &mask );
-    mouse_x= win_x; mouse_y= win_y;
+						    &rootX, &rootY,
+						    &winX, &winY, &mask );
+    mouseX= winX; mouseY= winY;
     }
 #   endif
 
 #   ifdef USE_GTK
     {
-    gint		win_x;
-    gint		win_y;
+    gint		winX;
+    gint		winY;
     GdkModifierType	mask;
 
-    gdk_window_get_pointer( add->addDrawable, &win_x, &win_y, &mask );
+    gdk_window_get_pointer( add->addDrawable, &winX, &winY, &mask );
 
-    mouse_x= win_x; mouse_y= win_y;
+    mouseX= winX; mouseY= winY;
     }
 #   endif
 
-    appScrollToRectangle( ed,
-				mouse_x+ ox, mouse_y+ oy,
-				mouse_x+ ox, mouse_y+ oy,
-				&scrolledX, &scrolledY );
+    if  ( mouseX < -ox )
+	{ mouseX=  -ox;	}
+    if  ( mouseY < -oy )
+	{ mouseY=  -oy;	}
+    if  ( mouseX > add->addBackRect.drX1- ox )
+	{ mouseX=  add->addBackRect.drX1- ox;	}
+    if  ( mouseY > add->addBackRect.drY1- oy )
+	{ mouseY=  add->addBackRect.drY1- oy;	}
+
+    appScrollToRectangle( ed, mouseX+ ox, mouseY+ oy,
+					mouseX+ ox, mouseY+ oy,
+					&scrolledX, &scrolledY );
 
     if  ( scrolledX || scrolledY )
 	{
 	if  ( tedExtendSelectionToXY( ed, dc->dcRootItem,
-					&(dc->dcAnchorPosition),
-					&(dc->dcSelectionScope),
-					dc->dcMouseX, dc->dcMouseY ) )
+			&(dc->dcAnchorPosition), dc->dcMouseX, dc->dcMouseY ) )
 	    { LDEB(1);	}
 	}
 
@@ -275,15 +290,15 @@ static APP_TIMER_HANDLER( tedTick, voiddc )
 #   endif
     }
 
+#   define	CLICK2_TO_CHANGE_ROOT	0
+
 static void tedButton1Pressed(	APP_WIDGET			w,
 				EditDocument *			ed,
 				APP_EVENT *			downEvent )
     {
-    TedDocument *		td= (TedDocument *)ed->edPrivateData;
     AppDrawingData *		add= &(ed->edDrawingData);
-    double			xfac= add->addMagnifiedPixelsPerTwip;
+    TedDocument *		td= (TedDocument *)ed->edPrivateData;
     BufferDocument *		bd= td->tdDocument;
-    const DocumentProperties *	dp= &(bd->bdProperties);
 
     DraggingContext		dc;
 
@@ -295,192 +310,204 @@ static void tedButton1Pressed(	APP_WIDGET			w,
     int				docX;
     int				docY;
 
-    int				pageY;
     int				page;
-    BufferItem *		sectBi= (BufferItem *)0;
-    HeaderFooter *		hf= (HeaderFooter *)0;
-    HeaderFooter *		hfx= (HeaderFooter *)0;
-    const DocumentGeometry *	dgSect;
-    int				otherHeaderFooter= 0;
+    int				sectNr;
+
+    int				otherRoot= 0;
 
     int				scrolledX= 0;
     int				scrolledY= 0;
 
-    int				sectNr;
-
     int				button;
     int				upDown;
     int				seq;
-    int				inHeaderFooter;
+    unsigned int		keyState= 0;
+
+    DocumentSelection		dsOld;
+    SelectionGeometry		sg;
+
+    PositionGeometry		pgClick;
+
+    ExternalItem *		eiSet= (ExternalItem *)0;
+    BufferItem *		bodySectBiSet;
+    BufferItem *		rootBiSet;
+
+    if  ( tedGetSelection( &dsOld, &sg, td ) )
+	{ LDEB(1); return;	}
 
     if  ( appGetCoordinatesFromMouseButtonEvent( &mouseX, &mouseY,
-						    &button, &upDown, &seq,
-						    w, downEvent ) )
+					    &button, &upDown, &seq, &keyState,
+					    w, downEvent ) )
 	{ LDEB(1); return;	}
 
+    if  ( mouseX < -ox )
+	{ mouseX=  -ox;	}
+    if  ( mouseY < -oy )
+	{ mouseY=  -oy;	}
+
+    dc.dcMouseX= mouseX;
+    dc.dcMouseY= mouseY;
+
     docX= mouseX+ ox;
     docY= mouseY+ oy;
 
     dc.dcRootItem= &(bd->bdItem);
-    dc.dcSelectionScope.ssInHeaderFooter= DOCinBODY;
-    dc.dcSelectionScope.ssHeaderFooterSection= -1;
 
-    page= docY/ add->addPageStepPixels;
-    pageY= docY- ( page* add->addPageStepPixels );
-    for ( sectNr= 0; sectNr < bd->bdItem.biChildCount; sectNr++ )
+    if  ( tedFindRootForPosition( &eiSet, &rootBiSet, &bodySectBiSet,
+					    &sectNr, &page, ed, docX, docY ) )
+	{ LLDEB(docX,docY); return;	}
+
+    dc.dcRootItem= rootBiSet;
+
+    otherRoot= docSelectionDifferentRoot( &dsOld, rootBiSet );
+
+    if  ( otherRoot )
 	{
-	sectBi= bd->bdItem.biChildren[sectNr];
+	DocumentPosition		dp;
+	int				inHeadFoot;
+	DocumentRectangle		drChanged;
 
-	if  ( sectBi->biTopPosition.lpPage <= page	&&
-	      sectBi->biBelowPosition.lpPage >= page	)
-	    { break;	}
-	}
+#	if  CLICK2_TO_CHANGE_ROOT
+	if  ( seq < 2 )
+	    { return;	}
+#	endif
 
-    if  ( sectNr >= bd->bdItem.biChildCount )
-	{ LLDEB(page,sectNr); return;	}
+	docInitDocumentPosition( &dp );
 
-    dgSect= &(sectBi->biSectDocumentGeometry);
+	inHeadFoot= dc.dcRootItem->biInExternalItem;
 
-    hfx= (HeaderFooter *)0;
-    inHeaderFooter= docWhatPageHeader( &hfx, sectBi, page, dp );
-    if  ( hfx							&&
-	  hfx->hfItem						&&
-	  pageY < TWIPStoPIXELS( xfac, hfx->hfY1ReservedTwips ) )
-	{
-	hf= hfx;
-	dc.dcSelectionScope.ssInHeaderFooter= inHeaderFooter;
-	}
-
-    hfx= (HeaderFooter *)0;
-    inHeaderFooter= docWhatPageFooter( &hfx, sectBi, page, dp );
-    if  ( hfx							&&
-	  hfx->hfItem						&&
-	  pageY > TWIPStoPIXELS( xfac, hfx->hfY0ReservedTwips ) )
-	{
-	hf= hfx;
-	dc.dcSelectionScope.ssInHeaderFooter= inHeaderFooter;
-	}
-
-    if  ( dc.dcSelectionScope.ssInHeaderFooter		!=
-	  td->tdSelectionScope.ssInHeaderFooter		)
-	{ otherHeaderFooter= 1;	}
-
-    if  ( dc.dcSelectionScope.ssInHeaderFooter != DOCinBODY )
-	{
-	if  ( sectNr != td->tdSelectionScope.ssHeaderFooterSection )
-	    { otherHeaderFooter= 1;	}
-	if  ( page != td->tdSelectionScope.ssHeaderFooterPage )
-	    { otherHeaderFooter= 1;	}
-	}
-
-    if  ( dc.dcSelectionScope.ssInHeaderFooter != DOCinBODY )
-	{
-	dc.dcRootItem= hf->hfItem;
-	dc.dcSelectionScope.ssHeaderFooterSection= sectNr;
-	dc.dcSelectionScope.ssHeaderFooterPage= page;
-	}
-    else{
-	dc.dcSelectionScope.ssHeaderFooterSection= -1;
-	dc.dcSelectionScope.ssHeaderFooterPage= -1;
-	}
-
-    if  ( otherHeaderFooter )
-	{
-	if  ( seq > 1 )
+	switch( inHeadFoot )
 	    {
-	    BufferPosition		bp;
-	    BufferItem *		bi;
-	    int				inHeadFoot;
+	    case DOCinBODY:
+	    case DOCinFOOTNOTE:
+	    case DOCinENDNOTE:
+		break;
 
-	    int				done;
-	    DocumentRectangle		drChanged;
+	    case DOCinSECT_HEADER:
+	    case DOCinFIRST_HEADER:
+	    case DOCinLEFT_HEADER:
+	    case DOCinRIGHT_HEADER:
+	    case DOCinSECT_FOOTER:
+	    case DOCinFIRST_FOOTER:
+	    case DOCinLEFT_FOOTER:
+	    case DOCinRIGHT_FOOTER:
 
-	    docInitPosition( &bp );
+		if  ( ! eiSet )
+		    { XDEB(eiSet); return;	}
+		if  ( eiSet->eiPageFormattedFor != page )
+		    {
+		    if  ( docLayoutExternalItem( eiSet, &drChanged,
+				page, eiSet->eiY0UsedTwips,
+				bd, bodySectBiSet, add,
+				tedLayoutExternalItem, tedCloseObject ) )
+			{ LDEB(page); return; }
 
-	    inHeadFoot= dc.dcSelectionScope.ssInHeaderFooter;
+		    eiSet->eiPageSelectedUpon= page;
+		    }
+		break;
 
-	    if  ( dc.dcSelectionScope.ssInHeaderFooter == DOCinBODY )
-		{ bi= sectBi;	}
-	    else{
-		if  ( tedRedoHeaderFooterLayout( &done, hf, page,
-					    sectBi, bd, add, &drChanged ) )
-		    { LDEB(page); return;	}
+	    case DOCinFTNSEP:
+	    case DOCinFTNSEPC:
+	    case DOCinFTNCN:
+	    case DOCinAFTNSEP:
+	    case DOCinAFTNSEPC:
+	    case DOCinAFTNCN:
 
-		bi= hf->hfItem;
-		}
+		if  ( ! eiSet )
+		    { XDEB(eiSet); return;	}
+		if  ( eiSet->eiPageFormattedFor != page )
+		    {
+		    LLDEB(eiSet->eiPageFormattedFor,page);
 
-	    if  ( tedFindPosition( bi, add, docX, docY, &bp ) )
-		{ LLDEB(docX,docY); return;	}
+		    if  ( docLayoutExternalItem( eiSet, &drChanged,
+				page, eiSet->eiY0UsedTwips,
+				bd, bodySectBiSet, add,
+				tedLayoutExternalItem, tedCloseObject ) )
+			{ LDEB(page); return; }
+		    }
+		break;
 
-	    tedSetSelectedPosition( ed, &(dc.dcSelectionScope), &bp,
-						    &scrolledX, &scrolledY );
-
-	    tedAdaptToolsToPosition( ed );
-
-	    appExposeRectangle( add, 0, 0, 0, 0 );
+	    default:
+		LDEB(dc.dcRootItem->biInExternalItem);
 	    }
 
+	if  ( eiSet )
+	    { eiSet->eiPageSelectedUpon= page;	}
+
+#	if CLICK2_TO_CHANGE_ROOT
+	{
+	PositionGeometry	pg;
+
+	if  ( tedFindPosition( &dp, &pg, dc.dcRootItem, add, docX, docY ) )
+	    { LLDEB(docX,docY); return;	}
+
+	tedSetSelectedPosition( ed, &dp, &scrolledX, &scrolledY );
+
+	tedAdaptToolsToSelection( ed );
+
+	appExposeRectangle( add, 0, 0, 0, 0 );
+
 	return;
 	}
+#	endif
+
+	appExposeRectangle( add, 0, 0, 0, 0 );
+	}
 
     if  ( seq > 1 )
 	{
 	int			wasObject= 0;
 
-	BufferSelection		bs;
+	DocumentSelection	dsNew;
 
-	docInitSelection( &bs );
+	docInitDocumentSelection( &dsNew );
 
-	if  ( tedFindPosition( dc.dcRootItem, add, docX, docY,
-						    &(dc.dcAnchorPosition) ) )
+	if  ( tedFindPosition( &(dc.dcAnchorPosition), &pgClick,
+					bd, dc.dcRootItem, add, docX, docY ) )
 	    { LLDEB(docX,docY); return; }
 
-	bs.bsBegin= dc.dcAnchorPosition;
-	bs.bsEnd= dc.dcAnchorPosition;
+	docSetIBarSelection( &dsNew, &(dc.dcAnchorPosition) );
 
 	if  ( seq > 2 )
 	    {
-	    docLinePositions( &(bs.bsBegin), &(bs.bsEnd),
-			dc.dcAnchorPosition.bpBi, dc.dcAnchorPosition.bpLine );
+	    docLineSelection( &dsNew,
+				dc.dcAnchorPosition.dpBi, pgClick.pgLine );
 	    }
 	else{
-	    TextParticule *	tp;
-
-	    tp= dc.dcAnchorPosition.bpBi->biParaParticules+
-					    dc.dcAnchorPosition.bpParticule;
-
-	    bs.bsBegin.bpStroff= tp->tpStroff;
-	    bs.bsEnd.bpStroff= tp->tpStroff+ tp->tpStrlen;
-
-	    if  ( tp->tpKind == DOCkindOBJECT )
-		{ wasObject= 1;	}
+	    docWordSelection( &dsNew, &wasObject, &(dc.dcAnchorPosition) );
 	    }
 
-	bs.bsDirection= 1;
-	bs.bsAnchor= bs.bsBegin;
-
-	tedSetSelection( ed, &(dc.dcSelectionScope), &bs,
-						    &scrolledX, &scrolledY );
+	tedSetSelection( ed, &dsNew, &scrolledX, &scrolledY );
 
 	tedAdaptToolsToSelection( ed );
 
 	if  ( wasObject )
-	    {
-	    tedPositionCoordinates( &(bs.bsBegin), add );
-	    tedSetObjectWindows( ed, &(bs.bsBegin), ox, oy );
-	    }
+	    { tedMoveObjectWindows( ed );	}
 
 	return;
 	}
 
-    if  ( tedFindPosition( dc.dcRootItem, add, docX, docY,
-						    &(dc.dcAnchorPosition) ) )
+    if  ( tedFindPosition( &(dc.dcAnchorPosition), &pgClick,
+					bd, dc.dcRootItem, add, docX, docY ) )
 	{ LLDEB(docX,docY); return;	}
 
-    tedSetSelectedPosition( ed, &(dc.dcSelectionScope), &(dc.dcAnchorPosition),
+    if  ( ! otherRoot && ( keyState & KEY_SHIFT_MASK ) )
+	{
+	dc.dcAnchorPosition= dsOld.dsAnchor;
+
+	if  ( tedExtendSelectionToXY( ed, dc.dcRootItem,
+			&(dc.dcAnchorPosition), dc.dcMouseX, dc.dcMouseY ) )
+	    { LDEB(1); return;	}
+
+	tedAdaptToolsToSelection( ed );
+
+	return;
+	}
+
+    tedSetSelectedPosition( ed, &(dc.dcAnchorPosition),
 						    &scrolledX, &scrolledY );
 
-    tedAdaptToolsToPosition( ed );
+    tedAdaptToolsToSelection( ed );
 
     if  ( ed->edFileReadOnly )
 	{
@@ -524,61 +551,54 @@ static void tedButton1Pressed(	APP_WIDGET			w,
 /*									*/
 /************************************************************************/
 
-static void tedInputSetSelectedPosition( EditDocument *		ed,
-					const SelectionScope *	ss,
-					const BufferPosition *	bp )
+static void tedInputSetSelectedPosition( EditDocument *			ed,
+					const DocumentPosition *	dp )
     {
     int			scrolledX= 0;
     int			scrolledY= 0;
 
-    tedSetSelectedPosition( ed, ss, bp, &scrolledX, &scrolledY );
-
-    tedAdaptToolsToPosition( ed );
-
-    return;
-    }
-
-static void tedInputExtendSelection(	EditDocument *		ed,
-					const SelectionScope *	ss,
-					const BufferPosition *	bpSet )
-    {
-    TedDocument *		td= (TedDocument *)ed->edPrivateData;
-    BufferDocument *		bd= td->tdDocument;
-    const DocumentProperties *	dp= &(bd->bdProperties);
-    const AppDrawingData *	add= &(ed->edDrawingData);
-    BufferPosition		bpNew;
-
-    bpNew= *bpSet;
-
-    tedPositionCoordinates( &bpNew, add );
-
-    tedExtendSelectionToPosition( ed, ss, &(td->tdSelection.bsAnchor), &bpNew );
-
-    tedSelectionRectangle( &(td->tdSelectedRectangle),
-						 add, dp, &(td->tdSelection) );
+    tedSetSelectedPosition( ed, dp, &scrolledX, &scrolledY );
 
     tedAdaptToolsToSelection( ed );
 
     return;
     }
 
-static void tedInputChangeSelection(	EditDocument *		ed,
-					unsigned int		keyState,
-					const SelectionScope *	ss,
-					const BufferPosition *	bp )
+static void tedInputExtendSelection(	EditDocument *			ed,
+					const DocumentPosition *	dpSet )
     {
-    if  ( keyState & KEY_SHIFT_MASK )
-	{
-	tedInputExtendSelection( ed, ss, bp );
-	return;
-	}
+    TedDocument *		td= (TedDocument *)ed->edPrivateData;
 
-    tedInputSetSelectedPosition( ed, ss, bp );
+    DocumentSelection		ds;
+    SelectionGeometry		sg;
+
+    if  ( tedGetSelection( &ds, &sg, td ) )
+	{ LDEB(1); return;	}
+
+    tedExtendSelectionToPosition( ed, &(ds.dsAnchor), dpSet );
+
+    tedAdaptToolsToSelection( ed );
 
     return;
     }
 
-static int tedMoveRightOnKey(	BufferPosition *	bpNew,
+static void tedInputChangeSelection(
+				EditDocument *			ed,
+				unsigned int			keyState,
+				const DocumentPosition *	dp )
+    {
+    if  ( keyState & KEY_SHIFT_MASK )
+	{
+	tedInputExtendSelection( ed, dp );
+	return;
+	}
+
+    tedInputSetSelectedPosition( ed, dp );
+
+    return;
+    }
+
+static int tedMoveRightOnKey(	DocumentPosition *	dpNew,
 				int			keyState,
 				const AppDrawingData *	add )
     {
@@ -586,20 +606,18 @@ static int tedMoveRightOnKey(	BufferPosition *	bpNew,
 	{
 	const int	lastOne= 1;
 
-	if  ( docNextWord( bpNew, lastOne ) )
+	if  ( docNextWord( dpNew, lastOne ) )
 	    { return -1;	}
-
-	tedPositionCoordinates( bpNew, add );
 	}
     else{
-	if  ( tedNextPosition( add, bpNew ) )
+	if  ( docNextPosition( dpNew ) )
 	    { return -1;	}
 	}
 
     return 0;
     }
 
-static int tedMoveLeftOnKey(	BufferPosition *	bpNew,
+static int tedMoveLeftOnKey(	DocumentPosition *	dpNew,
 				int			keyState,
 				const AppDrawingData *	add )
     {
@@ -607,13 +625,11 @@ static int tedMoveLeftOnKey(	BufferPosition *	bpNew,
 
     if  ( keyState & KEY_CONTROL_MASK )
 	{
-	if  ( docPrevWord( bpNew, lastOne ) )
+	if  ( docPrevWord( dpNew, lastOne ) )
 	    { return -1;	}
-
-	tedPositionCoordinates( bpNew, add );
 	}
     else{
-	if  ( tedPrevPosition( add, bpNew, lastOne ) )
+	if  ( docPrevPosition( dpNew, lastOne ) )
 	    { return -1;	}
 	}
 
@@ -629,9 +645,16 @@ static void tedProcessKeyEvent(	EditDocument *		ed,
 				unsigned int		state )
     {
     TedDocument *		td= (TedDocument *)ed->edPrivateData;
-    SelectionScope *		ss= &(td->tdSelectionScope);
+    BufferDocument *		bd= td->tdDocument;
     const AppDrawingData *	add= &(ed->edDrawingData);
-    BufferSelection *		bs= &(td->tdSelection);
+
+    int				res;
+
+    DocumentSelection		ds;
+    SelectionGeometry		sg;
+
+    if  ( tedGetSelection( &ds, &sg, td ) )
+	{ LDEB(1); return;	}
 
     if  ( gotString > 0 )
 	{
@@ -653,7 +676,7 @@ static void tedProcessKeyEvent(	EditDocument *		ed,
 
 	if  ( appCharExistsInFont( apf->apfFontStruct, scratch[0] ) )
 	    {
-	    tedAppReplaceSelection( ed, scratch, gotString );
+	    tedDocReplaceSelection( ed, scratch, gotString );
 	    return;
 	    }
 	}
@@ -663,12 +686,12 @@ static void tedProcessKeyEvent(	EditDocument *		ed,
 
     switch( keySym )
 	{
-	BufferPosition	bpNew;
+	DocumentPosition	dpNew;
 
 #	ifdef KEY_ISO_Left_Tab
 	case  KEY_ISO_Left_Tab:
-	    if  ( bs->bsBegin.bpBi			&&
-		  bs->bsBegin.bpBi->biParaInTable	&&
+	    if  ( ds.dsBegin.dpBi			&&
+		  ds.dsBegin.dpBi->biParaInTable	&&
 		  ! ( state & KEY_CONTROL_MASK )	)
 		{ goto shiftTab;	}
 	    else{ return;		}
@@ -679,26 +702,26 @@ static void tedProcessKeyEvent(	EditDocument *		ed,
 		{ goto unknown; }
 	    /*FALLTHROUGH*/
 	case KEY_Tab:
-	    if  ( bs->bsBegin.bpBi			&&
-		  bs->bsBegin.bpBi->biParaInTable	&&
+	    if  ( ds.dsBegin.dpBi			&&
+		  ds.dsBegin.dpBi->biParaInTable	&&
 		  ! ( state & KEY_CONTROL_MASK )	)
 		{
 		if  ( state & KEY_SHIFT_MASK )
 		    {
 		  shiftTab:
-		    if  ( docFirstPosition( bs->bsBegin.bpBi->biParent,
-							    &bpNew )	||
-			  tedPrevPosition( add, &bpNew, /*last=*/ 0 )	)
+		    if  ( docFirstPosition( &dpNew,
+					ds.dsBegin.dpBi->biParent )	||
+			  docPrevPosition( &dpNew, /*last=*/ 0 )	)
 			{ return;	}
 		    }
 		else{
-		    if  ( docLastPosition( bs->bsBegin.bpBi->biParent,
-							    &bpNew )	||
-			  tedNextPosition( add, &bpNew )		)
+		    if  ( docLastPosition( &dpNew,
+					ds.dsBegin.dpBi->biParent )	||
+			  docNextPosition( &dpNew )			)
 			{ return;	}
 		    }
 
-		tedInputSetSelectedPosition( ed, ss, &bpNew );
+		tedInputSetSelectedPosition( ed, &dpNew );
 
 		return;
 		}
@@ -709,11 +732,7 @@ static void tedProcessKeyEvent(	EditDocument *		ed,
 	    if  ( ! td->tdCanReplaceSelection )
 		{ return;	}
 
-	    tedAppReplaceSelection( ed, scratch, 0 );
-
-	    tedSetTab( ed );
-
-	    appDocumentChanged( ed, 1 );
+	    tedEditReplaceSelectionWithTab( ed );
 
 	    return;
 
@@ -726,8 +745,6 @@ static void tedProcessKeyEvent(	EditDocument *		ed,
 	    if  ( ! td->tdCanReplaceSelection )
 		{ return;	}
 
-	    tedAppReplaceSelection( ed, scratch, 0 );
-
 	    tedSplitParagraph( ed,
 			    state == KEY_CONTROL_MASK	&&
 			    keySym != KEY_j		&&
@@ -739,195 +756,223 @@ static void tedProcessKeyEvent(	EditDocument *		ed,
 
 	case KEY_KP_Delete:
 	case KEY_Delete:
+
+	    res= tedDeleteTableSliceSelection( ed );
+	    if  ( res < 0 )
+		{ LDEB(res); return;	}
+	    if  ( res == 0 )
+		{ return;		}
+
 	    if  ( ! td->tdCanReplaceSelection )
 		{ return;	}
 
 	    if  ( tedHasIBarSelection( td ) )
 		{
-		bpNew= bs->bsBegin;
+		dpNew= ds.dsBegin;
 
-		if  ( tedMoveRightOnKey( &bpNew, state, add ) )
+		if  ( tedMoveRightOnKey( &dpNew, state, add ) )
 		    { return;	}
 
-		if  ( ! docPositionsInsideCell( &(bs->bsBegin), &bpNew ) )
+		if  ( ! docPositionsInsideCell( &(ds.dsBegin), &dpNew ) )
 		    { return;	}
 
-		if  ( tedMoveRightOnKey( &(bs->bsEnd), state, add ) )
+		if  ( tedMoveRightOnKey( &(ds.dsEnd), state, add ) )
 		    { return;	}
+
+		    {
+		    int		scrolledX= 0;
+		    int		scrolledY= 0;
+
+		    tedSetSelection( ed, &ds, &scrolledX, &scrolledY );
+		    }
 		}
 
-	    tedAppReplaceSelection( ed, scratch, 0 );
+	    tedDocReplaceSelection( ed, (const unsigned char *)0, 0 );
 
 	    return;
 
 	case KEY_BackSpace:
+
+	    res= tedDeleteTableSliceSelection( ed );
+	    if  ( res < 0 )
+		{ LDEB(res); return;	}
+	    if  ( res == 0 )
+		{ return;		}
+
 	    if  ( ! td->tdCanReplaceSelection )
 		{ return;	}
 
 	    if  ( tedHasIBarSelection( td ) )
 		{
-		bpNew= bs->bsBegin;
+		dpNew= ds.dsBegin;
 
-		if  ( tedMoveLeftOnKey( &bpNew, state, add ) )
+		if  ( tedMoveLeftOnKey( &dpNew, state, add ) )
 		    { return;	}
 
-		if  ( ! docPositionsInsideCell( &bpNew, &(bs->bsBegin) ) )
+		if  ( ! docPositionsInsideCell( &dpNew, &(ds.dsBegin) ) )
 		    {
 		    const int		keyState= 0;
 
-		    tedInputChangeSelection( ed, keyState, ss, &bpNew );
+		    tedInputChangeSelection( ed, keyState, &dpNew );
 		    return;
 		    }
 
-		if  ( tedMoveLeftOnKey( &(bs->bsBegin), state, add ) )
+		if  ( tedMoveLeftOnKey( &(ds.dsBegin), state, add ) )
 		    { return;	}
+
+		    {
+		    int		scrolledX= 0;
+		    int		scrolledY= 0;
+
+		    tedSetSelection( ed, &ds, &scrolledX, &scrolledY );
+		    }
 		}
 
-	    tedAppReplaceSelection( ed, scratch, 0 );
+	    tedDocReplaceSelection( ed, (const unsigned char *)0, 0 );
 	    return;
 
 	case KEY_KP_Home:
 	case KEY_Home:
-	    if  ( ( state & KEY_SHIFT_MASK ) && bs->bsDirection >= 0 )
-		{ bpNew= bs->bsEnd;	}
-	    else{ bpNew= bs->bsBegin;	}
+	    if  ( ( state & KEY_SHIFT_MASK ) && ds.dsDirection >= 0 )
+		{ dpNew= ds.dsEnd;	}
+	    else{ dpNew= ds.dsBegin;	}
 
-	    if  ( tedBeginOfLine( add, &bpNew ) )
+	    if  ( docBeginOfLine( &dpNew ) )
 		{ return;	}
 
-	    tedInputChangeSelection( ed, state, ss, &bpNew );
+	    tedInputChangeSelection( ed, state, &dpNew );
 	    return;
 
 	case KEY_KP_End:
 	case KEY_End:
-	    if  ( ! ( state & KEY_SHIFT_MASK ) || bs->bsDirection >= 0 )
-		{ bpNew= bs->bsEnd;	}
-	    else{ bpNew= bs->bsBegin;	}
+	    if  ( ! ( state & KEY_SHIFT_MASK ) || ds.dsDirection >= 0 )
+		{ dpNew= ds.dsEnd;	}
+	    else{ dpNew= ds.dsBegin;	}
 
-	    if  ( tedEndOfLine( add, &bpNew ) )
+	    if  ( docEndOfLine( &dpNew ) )
 		{ return;	}
 
-	    tedInputChangeSelection( ed, state, ss, &bpNew );
+	    tedInputChangeSelection( ed, state, &dpNew );
 	    return;
 
 	case KEY_KP_Left:
 	case KEY_Left:
-	    if  ( ( state & KEY_SHIFT_MASK ) && bs->bsDirection >= 0 )
-		{ bpNew= bs->bsEnd;	}
-	    else{ bpNew= bs->bsBegin;	}
+	    if  ( ( state & KEY_SHIFT_MASK ) && ds.dsDirection >= 0 )
+		{ dpNew= ds.dsEnd;	}
+	    else{ dpNew= ds.dsBegin;	}
 
-	    if  ( tedMoveLeftOnKey( &bpNew, state, add ) )
+	    if  ( tedMoveLeftOnKey( &dpNew, state, add ) )
 		{ return;	}
 
-	    tedInputChangeSelection( ed, state, ss, &bpNew );
+	    tedInputChangeSelection( ed, state, &dpNew );
 	    return;
 
 	case KEY_KP_Right:
 	case KEY_Right:
-	    if  ( ! ( state & KEY_SHIFT_MASK ) || bs->bsDirection >= 0 )
-		{ bpNew= bs->bsEnd;	}
-	    else{ bpNew= bs->bsBegin;	}
+	    if  ( ! ( state & KEY_SHIFT_MASK ) || ds.dsDirection >= 0 )
+		{ dpNew= ds.dsEnd;	}
+	    else{ dpNew= ds.dsBegin;	}
 
-	    if  ( tedMoveRightOnKey( &bpNew, state, add ) )
+	    if  ( tedMoveRightOnKey( &dpNew, state, add ) )
 		{ return;	}
 
-	    tedInputChangeSelection( ed, state, ss, &bpNew );
+	    tedInputChangeSelection( ed, state, &dpNew );
 	    return;
 
 	case KEY_KP_Up:
 	case KEY_Up:
-	    if  ( ( state & KEY_SHIFT_MASK ) && bs->bsDirection >= 0 )
-		{ bpNew= bs->bsEnd;	}
-	    else{ bpNew= bs->bsBegin;	}
+	    if  ( ( state & KEY_SHIFT_MASK ) && ds.dsDirection >= 0 )
+		{ dpNew= ds.dsEnd;	}
+	    else{ dpNew= ds.dsBegin;	}
 
 	    if  ( state & KEY_CONTROL_MASK )
 		{
 		const int	lastOne= 1;
 
-		if  ( bpNew.bpStroff == 0 )
+		if  ( dpNew.dpStroff == 0 )
 		    {
-		    bpNew.bpBi= docPrevParagraph( bpNew.bpBi );
-		    if  ( ! bpNew.bpBi )
+		    dpNew.dpBi= docPrevParagraph( dpNew.dpBi );
+		    if  ( ! dpNew.dpBi )
 			{ return;	}
 
-		    if  ( docParaBegin( &bpNew, lastOne ) )
+		    if  ( docParaBegin( &dpNew, lastOne ) )
 			{ return;	}
 		    }
 		else{
-		    if  ( docParaBegin( &bpNew, lastOne ) )
+		    if  ( docParaBegin( &dpNew, lastOne ) )
 			{ return;	}
 		    }
 		}
 	    else{
-		if  ( tedPrevLine( add, &bpNew ) )
+		if  ( tedLineUp( &dpNew, bd, add ) )
 		    { return;	}
 		}
 
-	    tedInputChangeSelection( ed, state, ss, &bpNew );
+	    tedInputChangeSelection( ed, state, &dpNew );
 	    return;
 
 	case KEY_KP_Down:
 	case KEY_Down:
-	    if  ( ! ( state & KEY_SHIFT_MASK ) || bs->bsDirection >= 0 )
-		{ bpNew= bs->bsEnd;	}
-	    else{ bpNew= bs->bsBegin;	}
+	    if  ( ! ( state & KEY_SHIFT_MASK ) || ds.dsDirection >= 0 )
+		{ dpNew= ds.dsEnd;	}
+	    else{ dpNew= ds.dsBegin;	}
 
 	    if  ( state & KEY_CONTROL_MASK )
 		{
 		const int	lastOne= 1;
 
-		if  ( bpNew.bpStroff < bpNew.bpBi->biParaStrlen		&&
-		      ( state & KEY_SHIFT_MASK )		)
+		if  ( dpNew.dpStroff < dpNew.dpBi->biParaStrlen		&&
+		      ( state & KEY_SHIFT_MASK )			)
 		    {
-		    if  ( docParaEnd( &bpNew, lastOne ) )
+		    if  ( docParaEnd( &dpNew, lastOne ) )
 			{ return;	}
 		    }
 		else{
-		    bpNew.bpBi= docNextParagraph( bpNew.bpBi );
-		    if  ( ! bpNew.bpBi )
+		    dpNew.dpBi= docNextParagraph( dpNew.dpBi );
+		    if  ( ! dpNew.dpBi )
 			{ return;	}
 
-		    if  ( docParaBegin( &bpNew, lastOne ) )
+		    if  ( docParaBegin( &dpNew, lastOne ) )
 			{ return;	}
 		    }
 		}
 	    else{
-		if  ( tedNextLine( add, &bpNew ) )
+		if  ( tedArrowDown( &dpNew, bd, add ) )
 		    { return;	}
 		}
 
-	    tedInputChangeSelection( ed, state, ss, &bpNew );
+	    tedInputChangeSelection( ed, state, &dpNew );
 	    return;
 
 	case KEY_KP_Prior:
 	case KEY_Prior:
-	    if  ( ( state & KEY_SHIFT_MASK ) && bs->bsDirection >= 0 )
-		{ bpNew= bs->bsEnd;	}
-	    else{ bpNew= bs->bsBegin;	}
+	    if  ( ( state & KEY_SHIFT_MASK ) && ds.dsDirection >= 0 )
+		{ dpNew= ds.dsEnd;	}
+	    else{ dpNew= ds.dsBegin;	}
 
-	    if  ( tedPageUp( &bpNew, td->tdDocument, add,
+	    if  ( tedPageUp( &dpNew, bd, add,
 					ed->edVisibleRect.drY1-
 					ed->edVisibleRect.drY0 )	&&
-		  tedFirstPosition( add, td->tdDocument, &bpNew )	)
+		  docFirstDocumentPosition( &dpNew, bd )	)
 		{ return;	}
 
-	    tedInputChangeSelection( ed, state, ss, &bpNew );
+	    tedInputChangeSelection( ed, state, &dpNew );
 	    return;
 
 	case KEY_KP_Next:
 	case KEY_Next:
-	    if  ( ! ( state & KEY_SHIFT_MASK ) || bs->bsDirection >= 0 )
-		{ bpNew= bs->bsEnd;	}
-	    else{ bpNew= bs->bsBegin;	}
+	    if  ( ! ( state & KEY_SHIFT_MASK ) || ds.dsDirection >= 0 )
+		{ dpNew= ds.dsEnd;	}
+	    else{ dpNew= ds.dsBegin;	}
 
-	    if  ( tedPageDown( &bpNew, td->tdDocument, add,
+	    if  ( tedPageDown( &dpNew, bd, add,
 					add->addDocRect.drY1,
 					ed->edVisibleRect.drY1-
 					ed->edVisibleRect.drY0 )	&&
-		  tedLastPosition( add, td->tdDocument, &bpNew )	)
+		  docLastDocumentPosition( &dpNew, bd )	)
 		{ return;	}
 
-	    tedInputChangeSelection( ed, state, ss, &bpNew );
+	    tedInputChangeSelection( ed, state, &dpNew );
 	    return;
 
 #	if 0
@@ -960,9 +1005,9 @@ static void tedProcessKeyEvent(	EditDocument *		ed,
 	    return;
 #	endif
 
-#	if 1
+#	if 0
 	case KEY_y:
-	    docListItem(0,bs->bsBegin.bpBi);
+	    docListItem(0,ds.dsBegin.dpBi);
 	    break;
 #	endif
 
@@ -977,18 +1022,31 @@ static void tedProcessKeyEvent(	EditDocument *		ed,
 	case KEY_KP_Insert:
 	case KEY_Num_Lock:
 	    return;
+
 	default: unknown:
 #	    ifdef USE_GTK
-	    gtk_accel_group_activate( ed->edToplevel.atAccelGroup,
+#		if GTK_MAJOR_VERSION < 2
+		gtk_accel_group_activate( ed->edToplevel.atAccelGroup,
 							    keySym, state );
+#		else
+		gtk_accel_groups_activate(
+			    G_OBJECT( ed->edToplevel.atAccelGroup ),
+							    keySym, state );
+#		endif
 #	    endif
 
 #	    ifdef APP_DEBUG
 #	    ifdef USE_MOTIF
-	    appDebug( "INPUT \"%s\": %s (%s%s<Key>%s) \"%.*s\"\n",
+	    appDebug( "INPUT \"%s\": %s (%s%s%s%s%s%s%s%s<Key>%s) \"%.*s\"\n",
 			ed->edTitle, APP_X11EventNames[event->type],
 			(state&KEY_SHIFT_MASK)?"Shift":"",
 			(state&KEY_CONTROL_MASK)?"Ctrl":"",
+			(state&LockMask)?"Lock":"",
+			(state&Mod1Mask)?"Mod1":"",
+			(state&Mod2Mask)?"Mod2":"",
+			(state&Mod3Mask)?"Mod3":"",
+			(state&Mod4Mask)?"Mod4":"",
+			(state&Mod5Mask)?"Mod5":"",
 			XKeysymToString( keySym ), gotString, scratch );
 #	    endif
 #	    endif
@@ -997,11 +1055,10 @@ static void tedProcessKeyEvent(	EditDocument *		ed,
     return;
     }
 
-APP_EVENT_HANDLER( tedKeyPressed, w, voided, keyEvent )
+APP_EVENT_HANDLER_H( tedKeyPressed, w, voided, keyEvent )
     {
     EditDocument *		ed= (EditDocument *)voided;
     TedDocument *		td= (TedDocument *)ed->edPrivateData;
-    BufferSelection *		bs= &(td->tdSelection);
 
     APP_KEY_VALUE		keySym;
     unsigned char		scratch[40];
@@ -1009,7 +1066,7 @@ APP_EVENT_HANDLER( tedKeyPressed, w, voided, keyEvent )
     int				gotKey;
     unsigned int		state;
 
-    if  ( ! bs->bsBegin.bpBi )
+    if  ( ! tedHasSelection( td ) )
 	{
 #	ifdef USE_MOTIF
 	const AppDrawingData *	add= &(ed->edDrawingData);
@@ -1053,7 +1110,8 @@ typedef struct TedObjectDrag
     EditDocument *	todEd;
 
     InsertedObject *	todIo;
-    BufferPosition	todBp;
+    DocumentPosition	todDp;
+    PositionGeometry	todPg;
 
     int			todMouseX;
     int			todMouseY;
@@ -1080,9 +1138,9 @@ static void tedObjectHandleMove(	TedObjectDrag *		tod,
     int				wide= tod->todIo->ioDragWide;
     int				high= tod->todIo->ioDragHigh;
 
-    bi= tod->todBp.bpBi;
-    tp= bi->biParaParticules+ tod->todBp.bpParticule;
-    tl= bi->biParaLines+ tod->todBp.bpLine;
+    bi= tod->todDp.dpBi;
+    tp= bi->biParaParticules+ tod->todDp.dpParticule;
+    tl= bi->biParaLines+ tod->todPg.pgLine;
 
     if  ( tod->todRight && tod->todMouseX != mouseX )
 	{ tod->todIo->ioDragWide += mouseX- tod->todMouseX; moved= 1; }
@@ -1110,7 +1168,7 @@ static void tedObjectHandleMove(	TedObjectDrag *		tod,
     tod->todMouseY= mouseY;
     }
 
-static APP_EVENT_HANDLER( tedObjectDragMouseMove, w, vtod, event )
+static APP_EVENT_HANDLER_H( tedObjectDragMouseMove, w, vtod, event )
     {
     TedObjectDrag *		tod= (TedObjectDrag *)vtod;
 
@@ -1125,7 +1183,7 @@ static APP_EVENT_HANDLER( tedObjectDragMouseMove, w, vtod, event )
     return;
     }
 
-static APP_EVENT_HANDLER( tedObjectDragMouseUp, w, vtod, event )
+static APP_EVENT_HANDLER_H( tedObjectDragMouseUp, w, vtod, event )
     {
     TedObjectDrag *		tod= (TedObjectDrag *)vtod;
 
@@ -1135,9 +1193,10 @@ static APP_EVENT_HANDLER( tedObjectDragMouseUp, w, vtod, event )
     int				button;
     int				upDown;
     int				seq;
+    unsigned int		keyState;
 
-    if  ( appGetCoordinatesFromMouseButtonEvent(
-				&x, &y, &button, &upDown, &seq, w, event ) )
+    if  ( appGetCoordinatesFromMouseButtonEvent( &x, &y,
+			    &button, &upDown, &seq, &keyState, w, event ) )
 	{ return;	}
 
     tedObjectHandleMove( tod, x, y );
@@ -1151,6 +1210,8 @@ static int tedObjectDrag(	APP_WIDGET	w,
     {
     EditApplication *	ea= ed->edApplication;
     TedDocument *	td= (TedDocument *)ed->edPrivateData;
+    BufferDocument *	bd= td->tdDocument;
+    AppDrawingData *	add= &(ed->edDrawingData);
 
     int			ox= ed->edVisibleRect.drX0;
     int			oy= ed->edVisibleRect.drY0;
@@ -1161,6 +1222,7 @@ static int tedObjectDrag(	APP_WIDGET	w,
     int			button;
     int			upDown;
     int			seq;
+    unsigned int	keyState;
 
     int			x;
     int			y;
@@ -1173,17 +1235,20 @@ static int tedObjectDrag(	APP_WIDGET	w,
 
     if  ( appGetCoordinatesFromMouseButtonEvent(
 				    &(tod.todMouseX), &(tod.todMouseY),
-				    &button, &upDown, &seq, w, downEvent ) )
+				    &button, &upDown, &seq, &keyState,
+				    w, downEvent ) )
 	{ LDEB(1); return -1;	}
 
-    docInitPosition( &(tod.todBp) );
-    if  ( tedGetObjectSelection( td, &(tod.todBp), &(tod.todIo) ) )
+    docInitDocumentPosition( &(tod.todDp) );
+    if  ( tedGetObjectSelection( td, &(tod.todDp), &(tod.todIo) ) )
 	{ return 2;	}
 
+    tedPositionGeometry( &(tod.todPg), &(tod.todDp), bd, add );
+
     wide= tod.todIo->ioPixelsWide;
     high= tod.todIo->ioPixelsHigh;
-    x= tod.todBp.bpXPixels;
-    y= tod.todBp.bpBaselinePixels;
+    x= tod.todPg.pgXPixels;
+    y= tod.todPg.pgBaselinePixels;
 
     if  ( seq > 1					||
 	  upDown < 1					||
@@ -1222,7 +1287,6 @@ static int tedObjectDrag(	APP_WIDGET	w,
     if  ( tod.todIo->ioDragWide != tod.todIo->ioPixelsWide	||
 	  tod.todIo->ioDragHigh != tod.todIo->ioPixelsHigh	)
 	{
-	AppDrawingData *	add= &(ed->edDrawingData);
 	double			xfac= add->addMagnifiedPixelsPerTwip;
 
 	if  ( tod.todIo->ioDragWide != tod.todIo->ioPixelsWide )
@@ -1235,7 +1299,7 @@ static int tedObjectDrag(	APP_WIDGET	w,
 	    tod.todIo->ioPixelsWide= TWIPStoPIXELS( xfac,
 		    ( tod.todIo->ioScaleX* tod.todIo->ioTwipsWide )/ 100 );
 
-	    tp= tod.todBp.bpBi->biParaParticules+ tod.todBp.bpParticule;
+	    tp= tod.todDp.dpBi->biParaParticules+ tod.todDp.dpParticule;
 
 	    tp->tpPixelsWide= tod.todIo->ioPixelsWide;
 	    }
@@ -1249,9 +1313,11 @@ static int tedObjectDrag(	APP_WIDGET	w,
 		    ( tod.todIo->ioScaleY* tod.todIo->ioTwipsHigh )/ 100 );
 	    }
 
-	if  ( tedResizeObject( ed, &(td->tdSelection.bsBegin) ) )
+	if  ( tedResizeObject( ed, &(tod.todDp), &(tod.todPg) ) )
 	    { LDEB(1);	}
 
+	tedPositionGeometry( &(tod.todPg), &(tod.todDp), bd, add );
+
 	appDocumentChanged( ed, 1 );
 	}
 
@@ -1261,7 +1327,7 @@ static int tedObjectDrag(	APP_WIDGET	w,
     ox= ed->edVisibleRect.drX0;
     oy= ed->edVisibleRect.drY0;
 
-    tedSetObjectWindows( ed, &(td->tdSelection.bsBegin), ox, oy );
+    tedSetObjectWindows( ed, &(tod.todPg), tod.todIo, ox, oy );
 
     return 0;
     }
@@ -1272,7 +1338,7 @@ static int tedObjectDrag(	APP_WIDGET	w,
 /*									*/
 /************************************************************************/
 
-APP_EVENT_HANDLER( tedMouseButtonPressed, w, voided, downEvent )
+APP_EVENT_HANDLER_H( tedMouseButtonPressed, w, voided, downEvent )
     {
     EditDocument *	ed= (EditDocument *)voided;
     TedDocument *	td= (TedDocument *)ed->edPrivateData;
@@ -1297,6 +1363,15 @@ APP_EVENT_HANDLER( tedMouseButtonPressed, w, voided, downEvent )
 	    break;
 	case Button3:
 	    break;
+
+	case Button4:
+	    appMouseWheelUp( ed );
+	    break;
+
+	case Button5:
+	    appMouseWheelDown( ed );
+	    break;
+
 	default:
 	    LDEB(downEvent->xbutton.button);
 	    break;
@@ -1323,6 +1398,15 @@ APP_EVENT_HANDLER( tedMouseButtonPressed, w, voided, downEvent )
 	    break;
 	case 3:
 	    break;
+
+	case 4:
+	    appMouseWheelUp( ed );
+	    break;
+
+	case 5:
+	    appMouseWheelDown( ed );
+	    break;
+
 	default:
 	    LDEB(downEvent->button.button);
 	    break;
