@@ -4,7 +4,7 @@
 /*									*/
 /************************************************************************/
 
-#   include	"config.h"
+#   include	"tedConfig.h"
 
 #   include	<stddef.h>
 #   include	<stdio.h>
@@ -17,81 +17,113 @@
 #   undef	y1
 
 #   include	"tedApp.h"
-#   include	"docPs.h"
+#   include	"tedEdit.h"
+#   include	"tedLayout.h"
 
 #   include	<appDebugon.h>
 
-/************************************************************************/
-/*									*/
-/*  Calculate border width.						*/
-/*									*/
-/************************************************************************/
-
-int tedBorderThick(		int *				pWide,
-				const BorderProperties *	bp,
-				const AppDrawingData *		add )
-    {
-    int			wide= bp->bpPenWideTwips+ bp->bpSpacingTwips;
-    int			thick= bp->bpPenWideTwips;
-
-    if  ( bp->bpStyle == DOCbsNONE )
-	{ thick= 0;	}
-
-    if  ( thick > 0 )
-	{
-	thick *= add->addMagnifiedPixelsPerTwip;
-	wide  *= add->addMagnifiedPixelsPerTwip;
-
-	if  ( thick == 0 )
-	    { thick= 1;	}
-
-	*pWide= wide;
-	}
-    else{ *pWide= 0;	}
-
-    return thick;
-    }
-
 /************************************************************************/
 /*									*/
 /*  Calculate the width of a string.					*/
 /*									*/
 /************************************************************************/
 
-static int tedTextWidth(	const AppPhysicalFont *		apf,
-				const AppDrawingData *		add,
+static int tedSegmentedWidth(	const AppDrawingData *		add,
+				const AppPhysicalFont *		apf,
 				const unsigned char *		s,
+				const int *			segments,
+				int				segmentCount )
+    {
+    int				seg;
+
+    int				x= 0;
+
+    int				wide;
+    int				fontAscent;
+    int				fontDescent;
+
+    for ( seg= 0; seg < segmentCount; seg++ )
+	{
+	if  ( segments[2* seg+ 0] > 0 )
+	    {
+	    appDrawTextExtents( &wide, &fontAscent, &fontDescent,
+				    add, apf->apfFontStruct,
+				    (const char *)s, segments[2* seg+ 0] );
+
+	    s += segments[2* seg+ 0];
+	    x += wide;
+	    }
+
+	if  ( segments[2* seg+ 1] > 0 )
+	    {
+	    appDrawTextExtents( &wide, &fontAscent, &fontDescent,
+				    add, apf->apfScapsFontStruct,
+				    (const char *)s, segments[2* seg+ 1] );
+
+	    s += segments[2* seg+ 1];
+	    x += wide;
+	    }
+	}
+
+    return x;
+    }
+
+int tedTextWidth(		const AppPhysicalFont *		apf,
+				const TextAttribute *		ta,
+				const BufferDocument *		bd,
+				const AppDrawingData *		add,
+				const unsigned char *		printString,
 				int				len )
     {
-    int		wide;
-    int		fontAscent;
-    int		fontDescent;
+    unsigned char *		upperString= (unsigned char *)0;
+    int *			segments= (int *)0;
+    int				segmentCount= 0;
 
-    appDrawTextExtents( &wide, &fontAscent, &fontDescent,
-			    add, apf->apfFontStruct, (const char *)s, len );
+    int				wide;
+
+    if  ( ta->taSmallCaps || ta->taCapitals )
+	{
+	if  ( docMakeCapsString( &upperString, &segments, &segmentCount,
+						bd, ta, printString, len ) )
+	    { LDEB(len);		}
+	else{ printString= upperString;	}
+	}
+
+    if  ( ta->taSmallCaps && ! ta->taCapitals )
+	{
+	wide= tedSegmentedWidth( add, apf,
+				    printString, segments, segmentCount );
+	}
+    else{
+	int		fontAscent;
+	int		fontDescent;
+
+	appDrawTextExtents( &wide, &fontAscent, &fontDescent,
+		    add, apf->apfFontStruct, (const char *)printString, len );
+	}
+
+    if  ( upperString )
+	{ free( upperString );	}
+    if  ( segments )
+	{ free( segments );	}
 
     return wide;
     }
 
 static int tedVisibleTextWidth(	const AppPhysicalFont *		apf,
+				const TextAttribute *		ta,
+				const BufferDocument *		bd,
 				const AppDrawingData *		add,
-				const unsigned char *		s,
+				const unsigned char *		printString,
 				int				len )
     {
-    int		wide;
-    int		fontAscent;
-    int		fontDescent;
-
-    while( len > 0 && s[len- 1] == ' ' )
+    while( len > 0 && printString[len- 1] == ' ' )
 	{ len--;	}
 
     if  ( len == 0 )
 	{ return 0;	}
 
-    appDrawTextExtents( &wide, &fontAscent, &fontDescent,
-			    add, apf->apfFontStruct, (const char *)s, len );
-
-    return wide;
+    return tedTextWidth( apf, ta, bd, add, printString, len );
     }
 
 /************************************************************************/
@@ -104,24 +136,27 @@ static int tedVisibleTextWidth(	const AppPhysicalFont *		apf,
 /************************************************************************/
 
 static int tedLayoutParticules(	const BufferItem *		bi,
+				const BufferDocument *		bd,
 				AppDrawingData *		add,
-				const FormattingFrame *		ffPixels,
+				const ParagraphFrame *		ffPixels,
 				TextParticule *			tp,
 				ParticuleData *			pd,
 				int				particuleCount,
 				int				x0 )
     {
     int				accepted;
-    int				width;
+    int				wide;
 
     int				len= 0;
     int				xFrom= x0;
     const unsigned char *	from= bi->biParaString+ tp->tpStroff;
 
     int				physicalFont= tp->tpPhysicalFont;
+    int				textAttr= tp->tpTextAttributeNumber;
     AppPhysicalFontList *	apfl= &(add->addPhysicalFontList);
     AppPhysicalFont *		apf= apfl->apflFonts+ physicalFont;
 
+
     accepted= 0;
     while( accepted < particuleCount )
 	{
@@ -131,27 +166,33 @@ static int tedLayoutParticules(	const BufferItem *		bi,
 
 	    case DOCkindTEXT:
 		while( accepted < particuleCount		&&
-		       tp->tpPhysicalFont == physicalFont	&&
+		       tp->tpTextAttributeNumber == textAttr	&&
 		       tp->tpKind == DOCkindTEXT		)
 		    {
+		    TextAttribute	ta;
+
+		    utilGetTextAttributeByNumber( &ta,
+						&(bd->bdTextAttributeList),
+						tp->tpTextAttributeNumber );
+
 		    len += tp->tpStrlen;
-		    width= tedTextWidth( apf, add, from, len );
+		    wide= tedTextWidth( apf, &ta, bd, add, from, len );
 
 		    pd->pdVisiblePixels= tedVisibleTextWidth(
-						apf, add,
+						apf, &ta, bd, add,
 						bi->biParaString+ tp->tpStroff,
 						tp->tpStrlen );
-		    pd->pdWhiteUnits= xFrom+ width- x0- pd->pdVisiblePixels;
+		    pd->pdWhiteUnits= xFrom+ wide- x0- pd->pdVisiblePixels;
 
 		    tp->tpX0= x0;
-		    tp->tpPixelsWide= xFrom+ width- x0;
-		    x0= xFrom+ width;
+		    tp->tpPixelsWide= xFrom+ wide- x0;
+		    x0= xFrom+ wide;
 
-		    accepted++; tp++;
-		    pd++; break;
+		    accepted++; tp++; pd++; continue;
 		    }
 
 		break;
+
 	    case DOCkindTAB:
 		x1= TWIPStoPIXELS( add->addMagnifiedPixelsPerTwip,
 							pd->pdTabPosition );
@@ -164,39 +205,46 @@ static int tedLayoutParticules(	const BufferItem *		bi,
 		tp->tpPixelsWide= x1- x0;
 		x0= x1;
 
-		accepted++; tp++;
-		pd++;
-		break;
+		accepted++; tp++; pd++; break;
 
 	    case DOCkindOBJECT:
-		width= bi->biParaObjects[tp->tpObjectNumber].ioPixelsWide;
+		wide= bi->biParaObjects[tp->tpObjectNumber].ioPixelsWide;
 
-		pd->pdVisiblePixels= width;
+		pd->pdVisiblePixels= wide;
 		pd->pdWhiteUnits= 0;
 
 		tp->tpX0= x0;
-		tp->tpPixelsWide= width;
-		x0 += width;
+		tp->tpPixelsWide= wide;
+		x0 += wide;
 
-		accepted++; tp++;
-		pd++;
-		break;
+		accepted++; tp++; pd++; break;
 
+	    case DOCkindCHFTNSEP:
+		wide= TWIPStoPIXELS( add->addMagnifiedPixelsPerTwip, 2880 );
+
+		pd->pdVisiblePixels= wide;
+		pd->pdWhiteUnits= 0;
+
+		tp->tpX0= x0;
+		tp->tpPixelsWide= wide;
+		x0 += wide;
+
+		accepted++; tp++; pd++; break;
+
+	    case DOCkindNOTE:
 	    case DOCkindFIELDSTART:
 	    case DOCkindFIELDEND:
 	    case DOCkindXE:
 	    case DOCkindTC:
 	    case DOCkindLINEBREAK:
 	    case DOCkindPAGEBREAK:
-		if  ( pd )
-		    { pd->pdVisiblePixels= 0; pd->pdWhiteUnits= 0;	}
+
+		pd->pdVisiblePixels= 0; pd->pdWhiteUnits= 0;
 
 		tp->tpX0= x0;
 		tp->tpPixelsWide= 0;
-		accepted++; tp++;
-		if  ( pd )
-		    { pd++;	}
-		break;
+
+		accepted++; tp++; pd++; break;
 
 	    default:
 		LDEB(tp->tpKind); return -1;
@@ -206,6 +254,7 @@ static int tedLayoutParticules(	const BufferItem *		bi,
 	    { return accepted;	}
 
 	physicalFont= tp->tpPhysicalFont;
+	textAttr= tp->tpTextAttributeNumber;
 	apf= apfl->apflFonts+ physicalFont;
 	from= bi->biParaString+ tp->tpStroff;
 	xFrom= x0;
@@ -246,6 +295,7 @@ static void tedLineHeight(	const BufferItem *	bi,
 	    {
 	    case DOCkindTEXT:
 	    case DOCkindTAB:
+	    case DOCkindCHFTNSEP:
 		physicalFont= tp->tpPhysicalFont;
 		apf= apfl->apflFonts+ physicalFont;
 
@@ -270,6 +320,7 @@ static void tedLineHeight(	const BufferItem *	bi,
 		    { objectHeight=  io->ioPixelsHigh;	}
 		break;
 
+	    case DOCkindNOTE:
 	    case DOCkindFIELDSTART:
 	    case DOCkindFIELDEND:
 	    case DOCkindXE:
@@ -330,7 +381,7 @@ static void tedJustifyLine(	const BufferItem *	bi,
 
     if  ( left > 0 )
 	{
-	if  ( pd[left-1].pdTabKind != DOCtkLEFT )
+	if  ( pd[left-1].pdTabKind != DOCtaLEFT )
 	    { LDEB(pd[left-1].pdTabKind); return;	}
 
 	tp += left; pd += left; accepted -= left;
@@ -508,8 +559,9 @@ static void tedSqueezeParticules(	TextParticule *		tp,
 /************************************************************************/
 
 static int tedLayoutLine(	const BufferItem *	bi,
+				const BufferDocument *	bd,
 				AppDrawingData *	add,
-				const FormattingFrame *	ff,
+				const ParagraphFrame *	pf,
 				TextParticule *		tp,
 				ParticuleData *		pd,
 				int			particuleCount,
@@ -537,7 +589,8 @@ static int tedLayoutLine(	const BufferItem *	bi,
 	if  ( past < particuleCount )
 	    { includeTab= 1; past++;	}
 
-	accepted= tedLayoutParticules( bi, add, ff, tp, pd, past- part, x0 );
+	accepted= tedLayoutParticules( bi, bd,
+					add, pf, tp, pd, past- part, x0 );
 
 	if  ( accepted != past- part )
 	    { LLDEB(accepted,past- part); return -1;	}
@@ -576,7 +629,7 @@ static int tedLayoutLine(	const BufferItem *	bi,
 		}
 	    }
 	else{
-	    x1= ff->ffX1TextLinesPixels;
+	    x1= pf->pfX1TextLinesPixels;
 
 	    if  ( tpp->tpX0+ pdd->pdVisiblePixels > x1	)
 		{
@@ -602,25 +655,27 @@ static int tedLayoutLine(	const BufferItem *	bi,
 
 static int tedLayoutScreenLine(	TextLine *			tl,
 				const BufferItem *		bi,
+				const BufferDocument *		bd,
 				int				part,
 				int				accepted,
 				AppDrawingData *		add,
-				const FormattingFrame *		ff,
-				TextParticule *			tp,
+				const ParagraphFrame *		pf,
 				ParticuleData *			pd )
     {
-    int		xShift;
-    int		x1;
+    int			xShift;
+    int			x1;
 
-    int		x0Pixels;
-    int		ascentPixels;
-    int		descentPixels;
+    int			x0Pixels;
+    int			ascentPixels;
+    int			descentPixels;
+
+    TextParticule *	tp= bi->biParaParticules+ part;
 
     if  ( part == 0 )
-	{ x0Pixels= ff->ffX0FirstLinePixels;	}
-    else{ x0Pixels= ff->ffX0TextLinesPixels;	}
+	{ x0Pixels= pf->pfX0FirstLinePixels;	}
+    else{ x0Pixels= pf->pfX0TextLinesPixels;	}
 
-    if  ( tedLayoutLine( bi, add, ff, tp, pd, accepted, x0Pixels ) )
+    if  ( tedLayoutLine( bi, bd, add, pf, tp, pd, accepted, x0Pixels ) )
 	{ LDEB(1); return -1;	}
 
     switch( bi->biParaAlignment )
@@ -629,18 +684,18 @@ static int tedLayoutScreenLine(	TextLine *			tl,
 	    xShift= 0; break;
 	case DOCiaRIGHT:
 	    x1= tp[accepted-1].tpX0+ pd[accepted-1].pdVisiblePixels;
-	    xShift= ff->ffX1TextLinesPixels- x1;
+	    xShift= pf->pfX1TextLinesPixels- x1;
 	    break;
 	case DOCiaCENTERED:
 	    x1= tp[accepted-1].tpX0+ pd[accepted-1].pdVisiblePixels;
-	    xShift= ( ff->ffX1TextLinesPixels- x1 )/ 2;
+	    xShift= ( pf->pfX1TextLinesPixels- x1 )/ 2;
 	    break;
 	case DOCiaJUSTIFIED:
 	    xShift= 0;
 	    if  ( part+ accepted < bi->biParaParticuleCount )
 		{
 		tedJustifyLine( bi, tp, pd, accepted,
-						ff->ffX1TextLinesPixels );
+						pf->pfX1TextLinesPixels );
 		}
 	    break;
 	default:
@@ -671,57 +726,28 @@ static int tedLayoutScreenLine(	TextLine *			tl,
 /*									*/
 /************************************************************************/
 
-void tedParagraphFramePixels( 	FormattingFrame *	ff,
+void tedParagraphFramePixels( 	ParagraphFrame *	pf,
 				const AppDrawingData *	add,
 				const BufferItem *	bi )
     {
     double		xfac= add->addMagnifiedPixelsPerTwip;
 
-    ff->ffX0TextLinesPixels= TWIPStoPIXELS( xfac, ff->ffX0TextLinesTwips );
-    ff->ffX0FirstLinePixels= TWIPStoPIXELS( xfac, ff->ffX0FirstLineTwips );
-    ff->ffX1TextLinesPixels= TWIPStoPIXELS( xfac, ff->ffX1TextLinesTwips );
+    pf->pfX0TextLinesPixels= TWIPStoPIXELS( xfac, pf->pfX0TextLinesTwips );
+    pf->pfX0FirstLinePixels= TWIPStoPIXELS( xfac, pf->pfX0FirstLineTwips );
+    pf->pfX1TextLinesPixels= TWIPStoPIXELS( xfac, pf->pfX1TextLinesTwips );
+
+    if  ( pf->pfX0TextLinesPixels < pf->pfX0FirstLinePixels )
+	{ pf->pfX0Pixels= pf->pfX0TextLinesPixels;	}
+    else{ pf->pfX0Pixels= pf->pfX0FirstLinePixels;	}
+
+    pf->pfX1Pixels= pf->pfX1TextLinesPixels;
 
     return;
     }
 
-/************************************************************************/
-/*									*/
-/*  Prepare for the layout of a paragraph for displaying.		*/
-/*									*/
-/************************************************************************/
-
-void tedParaScreenGeometry(	const BufferItem *		bi,
-				const DocumentProperties *	dp,
-				const AppDrawingData *		add )
-    {
-    double		xfac= add->addMagnifiedPixelsPerTwip;
-
-    int			tab;
-    TabStop *		ts;
-
-    FormattingFrame	ff;
-
-    const int		bottom= -1;
-    const int		stripHigh= -1;
-    const int		page= 0;
-    const int		column= 0;
-
-    docParagraphFrameTwips( &ff, bottom, stripHigh, bi, dp, page, column );
-
-    tedParagraphFramePixels( &ff, add, bi );
-
-    ts= bi->biParaTabStops;
-    for ( tab= 0; tab < bi->biParaTabCount; ts++, tab++ )
-	{
-	ts->tsPixels= TWIPStoPIXELS( xfac, ff.ffX0GeometryTwips+ ts->tsTwips );
-	}
-
-    return;
-    }
-
-static int tedLayoutStartScreenPara(	BufferItem *			bi,
-					AppDrawingData *		add,
-					const DocumentProperties *	dp )
+static int tedLayoutStartScreenPara(	BufferItem *		bi,
+					AppDrawingData *	add,
+					BufferDocument *	bd )
     {
     int			part;
     TextParticule *	tp;
@@ -748,9 +774,9 @@ static int tedLayoutStartScreenPara(	BufferItem *			bi,
 	    if  ( io->ioKind == DOCokMACPICT )
 		{
 		if  ( io->ioTwipsWide == 0 )
-		    { io->ioTwipsWide= 20* io->ioXExtent;	}
+		    { io->ioTwipsWide= 20* io->io_xWinExt;	}
 		if  ( io->ioTwipsHigh == 0 )
-		    { io->ioTwipsHigh= 20* io->ioYExtent;	}
+		    { io->ioTwipsHigh= 20* io->io_yWinExt;	}
 		}
 
 	    io->ioPixelsWide= TWIPStoPIXELS( xfac,
@@ -760,19 +786,17 @@ static int tedLayoutStartScreenPara(	BufferItem *			bi,
 	    }
 	}
 
-    tedParaScreenGeometry( bi, dp, add );
-
     return 0;
     }
 
-static int tedPlaceStartScreenPara( BufferItem *			bi,
+static int tedPlaceStartScreenPara( BufferItem *		bi,
 				    AppDrawingData *		add,
-				    const DocumentProperties *	dp )
+				    BufferDocument *		bd )
     { return 0;	}
 
-static int tedLayoutStartScreenRow(	BufferItem *			rowBi,
-					AppDrawingData *		add,
-					const DocumentProperties *	dp )
+static int tedLayoutStartScreenRow(	BufferItem *		rowBi,
+					AppDrawingData *	add,
+					BufferDocument *	bd )
     {
     int			i;
     CellProperties *	cp;
@@ -800,7 +824,7 @@ static int tedLayoutStartScreenRow(	BufferItem *			rowBi,
 /*									*/
 /************************************************************************/
 
-static void tedFillLayoutJob(	LayoutJob *	lj )
+static void tedSetScreenLayoutFunctions(	LayoutJob *	lj )
     {
     lj->ljLayoutScreen.slStartRow= tedLayoutStartScreenRow;
     lj->ljLayoutScreen.slStartParagraph= tedLayoutStartScreenPara;
@@ -816,21 +840,26 @@ static void tedFillLayoutJob(	LayoutJob *	lj )
     }
 
 int tedLayoutItem(	BufferItem *		bi,
-			const BufferDocument *	bd,
-			int			inHeaderFooter,
+			BufferDocument *	bd,
 			AppDrawingData *	add,
 			DocumentRectangle *	drChanged )
     {
-    const int		andParents= 1;
     LayoutJob		lj;
 
-    tedFillLayoutJob( &lj );
+    docPsInitLayoutJob( &lj );
+
+    if  ( add->addForScreenDrawing )
+	{ tedSetScreenLayoutFunctions( &lj );	}
 
     if  ( bi->biNumberInParent == 0 )
 	{
 	if  ( bi->biParent )
-	    { lj.ljPosition= bi->biParent->biTopPosition;		}
-	else{ docInitLayoutPosition( &(lj.ljPosition) );		}
+	    { lj.ljPosition= bi->biParent->biTopPosition;	}
+	else{
+	    if  ( bi->biInExternalItem == DOCinBODY )
+		{ docInitLayoutPosition( &(lj.ljPosition) );	}
+	    else{ lj.ljPosition= bi->biTopPosition;		}
+	    }
 	}
     else{
 	BufferItem *	prevBi;
@@ -841,107 +870,147 @@ int tedLayoutItem(	BufferItem *		bi,
 	}
 
     lj.ljChangedRectanglePixels= drChanged;
+    lj.ljAdd= add;
+    lj.ljBd= bd;
+    lj.ljChangedItem= bi;
 
-    if  ( docPsLayoutItemAndParents( bi, add, bd, &lj, andParents ) )
+    if  ( docLayoutItemAndParents( bi, &lj ) )
 	{ LDEB(1); return -1;	}
 
+    docPsCleanLayoutJob( &lj );
+
     return 0;
     }
 
-/************************************************************************/
-/*									*/
-/*  Determine the box around a header or a footer.			*/
-/*									*/
-/************************************************************************/
-
-int tedGetHeaderFooterBox(	DocumentRectangle *		dr,
-				BufferItem *			sectBi,
-				const DocumentProperties *	dp,
-				AppDrawingData *		add,
-				int				page,
-				int				inHdFt )
-    {
-    HeaderFooter *		hf;
-    const SectionProperties *	sp= &(sectBi->biSectProperties);
-    const DocumentGeometry *	dgSect= &(sp->spDocumentGeometry);
-    double			xfac= add->addMagnifiedPixelsPerTwip;
-
-    DocumentRectangle		drBox;
-
-    int				pageTopPixels= page* add->addPageStepPixels;
-
-    drBox.drX0= TWIPStoPIXELS( xfac, dgSect->dgLeftMarginTwips );
-    drBox.drX1= TWIPStoPIXELS( xfac, dgSect->dgPageWideTwips-
-						dgSect->dgRightMarginTwips );
-
-    if  ( docWhatPageHeader( &hf, sectBi, page, dp ) == inHdFt )
-	{
-	drBox.drY0= pageTopPixels+
-				TWIPStoPIXELS( xfac, hf->hfY0ReservedTwips );
-	drBox.drY1= pageTopPixels+
-				TWIPStoPIXELS( xfac, hf->hfY1ReservedTwips );
-	}
-    else{
-	if  ( docWhatPageFooter( &hf, sectBi, page, dp ) == inHdFt )
-	    {
-	    drBox.drY0= pageTopPixels+
-				TWIPStoPIXELS( xfac, hf->hfY0ReservedTwips );
-	    drBox.drY1= pageTopPixels+
-				TWIPStoPIXELS( xfac, hf->hfY1ReservedTwips );
-	    }
-	else{ LDEB(inHdFt); return -1;	}
-	}
-
-    *dr= drBox; return 0;
-    }
-
 /************************************************************************/
 /*									*/
 /*  Calculate the layout of a page header.				*/
 /*									*/
 /************************************************************************/
 
-int tedLayoutExternal(		HeaderFooter *			hf,
+int tedLayoutExternalItem(	int *				pY1Twips,
+				ExternalItem *			ei,
 				int				page,
-				const SectionProperties *	sp,
-				const BufferDocument *		bd,
+				int				y0Twips,
+				BufferDocument *		bd,
 				AppDrawingData *		add,
 				DocumentRectangle *		drChanged )
     {
-    const int			andParents= 0;
     LayoutJob			lj;
 
-    tedFillLayoutJob( &lj );
+    docPsInitLayoutJob( &lj );
+
+    if  ( add->addForScreenDrawing )
+	{ tedSetScreenLayoutFunctions( &lj );	}
 
     lj.ljPosition.lpPage= page;
-    lj.ljPosition.lpPageYTwips= hf->hfY0UsedTwips;
+    lj.ljPosition.lpPageYTwips= y0Twips;
     lj.ljPosition.lpAtTopOfColumn= 1; /* not really */
 
     lj.ljChangedRectanglePixels= drChanged;
+    lj.ljAdd= add;
+    lj.ljBd= bd;
+    lj.ljChangedItem= &(bd->bdItem);
 
-    if  ( docPsLayoutItemAndParents( hf->hfItem, add, bd, &lj, andParents ) )
+    if  ( docLayoutItemAndParents( ei->eiItem, &lj ) )
+	{ LDEB(1); return -1;	}
+
+    *pY1Twips= lj.ljPosition.lpPageYTwips;
+
+    docPsCleanLayoutJob( &lj );
+
+    return 0;
+    }
+
+/************************************************************************/
+/*									*/
+/*  Redo the layout of a paragraph because its contents have changed	*/
+/*  during editing.							*/
+/*									*/
+/************************************************************************/
+
+int tedAdjustParagraphLayout(	EditOperation *			eo,
+				AppDrawingData *		add,
+				BufferItem *			bi,
+				int				line,
+				int				stroffShift,
+				int				upto )
+    {
+    LayoutJob		lj;
+
+    if  ( add->addDrawable )
+	{ tedSetScreenLayoutFunctions( &lj );	}
+
+    /*  bogus  */
+    lj.ljPosition.lpPage= 0;
+    lj.ljPosition.lpPageYTwips= 0;
+
+    lj.ljChangedRectanglePixels= &(eo->eoChangedRectangle);
+    lj.ljAdd= add;
+    lj.ljBd= eo->eoBd;
+    lj.ljChangedItem= bi;
+
+    if  ( docAdjustParaLayout( bi, line, stroffShift, upto, &lj ) )
 	{ LDEB(1); return -1;	}
 
     return 0;
     }
 
-int tedRedoHeaderFooterLayout(	int *				pDone,
-				HeaderFooter *			hf,
-				int				page,
-				const BufferItem *		sectBi,
-				BufferDocument *		bd,
-				AppDrawingData *		add,
-				DocumentRectangle *		drChanged )
+/************************************************************************/
+/*									*/
+/*  Verify that the root of a selection is formatted for the current	*/
+/*  page, if not format it.						*/
+/*									*/
+/************************************************************************/
+
+static int tedCheckPageOfSelectedNoteSeparator(
+					int *			pChanged,
+					int *			pY0Twips,
+					BufferDocument *	bd,
+					AppDrawingData *	add,
+					const ExternalItem *	selRootEi,
+					int			noteItKind,
+					int			sepItKind )
     {
-    int		needed= hf->hfPageUsedFor != page;
+    int			notesIndex;
+    DocumentNote *	dnFirstNote;
+    ExternalItem *	eiNoteSep;
+    int			y0Twips;
 
-    if  ( ! hf->hfItem )
-	{ XDEB(hf->hfItem); return -1;	}
-    if  ( ! needed )
-	{ *pDone= needed; return 0;	}
+    DocumentRectangle	drExtern;
 
-    switch( hf->hfItem->biInHeaderFooter )
+    notesIndex= docGetFirstNoteOnPage( &dnFirstNote, bd, 
+			selRootEi->eiPageSelectedUpon, noteItKind );
+    if  ( notesIndex < 0 )
+	{ LLDEB(selRootEi->eiPageSelectedUpon,notesIndex); return -1; }
+
+    if  ( docNoteSeparatorRectangle( &drExtern, &eiNoteSep,
+			    &y0Twips, bd, dnFirstNote, sepItKind, add ) )
+	{ LDEB(1); return -1;	}
+
+    *pY0Twips= y0Twips;
+    return 0;
+    }
+
+int tedCheckPageOfSelectedExtItem(
+				int *				pChanged,
+				DocumentRectangle *		drChanged,
+				BufferDocument *		bd,
+				ExternalItem *			selRootEi,
+				AppDrawingData *		add )
+    {
+    int				y0Twips;
+    BufferItem *		selRootBodySectBi= (BufferItem *)0;
+
+    if  ( ! selRootEi->eiItem )
+	{ XDEB(selRootEi->eiItem); return -1;	}
+
+    switch( selRootEi->eiItem->biInExternalItem )
 	{
+	case DOCinBODY:
+	    LDEB(selRootEi->eiItem->biInExternalItem);
+	    return -1;
+
 	case DOCinSECT_HEADER:
 	case DOCinFIRST_HEADER:
 	case DOCinLEFT_HEADER:
@@ -952,617 +1021,83 @@ int tedRedoHeaderFooterLayout(	int *				pDone,
 	case DOCinLEFT_FOOTER:
 	case DOCinRIGHT_FOOTER:
 
-	    if  ( docLayoutHeaderFooter( hf, page, bd, sectBi, add,
-			    tedLayoutExternal, drChanged, tedCloseObject ) )
-		{ LDEB(page); return -1; }
-
-	    *pDone= needed;
-	    return 0;
-
-	default:
-	    LDEB(hf->hfItem->biInHeaderFooter); return -1;
-	}
-    }
-
-/************************************************************************/
-/*									*/
-/*  Redo the layout of a paragraph because its contents have changed	*/
-/*  during editing.							*/
-/*									*/
-/************************************************************************/
-
-int tedAdjustParagraphLayout(	BufferItem *			bi,
-				int				line,
-				int				stroffShift,
-				int				upto,
-				AppDrawingData *		add,
-				const BufferDocument *		bd,
-				DocumentRectangle *		drChanged )
-    {
-    LayoutJob		lj;
-
-    docFieldRefreshFlags( bi, bd );
-
-    tedFillLayoutJob( &lj );
-
-    /*  bogus  */
-    lj.ljPosition.lpPage= 0;
-    lj.ljPosition.lpPageYTwips= 0;
-
-    lj.ljChangedRectanglePixels= drChanged;
-
-    if  ( docPsAdjustParaLayout( bi, line, stroffShift, upto, add, bd, &lj ) )
-	{ LDEB(1); return -1;	}
-
-    return 0;
-    }
-
-/************************************************************************/
-/*									*/
-/*  Find the object in the document for an x,y position.		*/
-/*									*/
-/************************************************************************/
-
-static BufferItem * tedFindItem(	BufferItem *		bi,
-					const AppDrawingData *	add,
-					int			x,
-					int			y )
-    {
-    int				i;
-    BufferItem *		child;
-    BufferItem *		found;
-    const CellProperties *	cp;
-    int				page= y/ add->addPageStepPixels;
-
-    switch( bi->biLevel )
-	{
-	case DOClevDOC:
-	case DOClevSECT:
-	case DOClevCELL:
-	rowAsGroup:
-	    for ( i= 0; i < bi->biChildCount; i++ )
+	    if  ( selRootEi->eiItem )
 		{
-		BufferItem *	next= (BufferItem *)0;
+		BufferItem *		docBi= &(bd->bdItem);
 
-		child= bi->biChildren[i];
+		const SelectionScope *	ss;
 
-		if  ( i+ 1 < bi->biChildCount )
-		    { next= bi->biChildren[i+ 1];	}
-
-		if  ( y < BI_BELOW_PIXELS( add, child )			||
-		      ( next					&&
-		        next->biTopPosition.lpPage > page	)	)
-		    {
-		    found= tedFindItem( child, add, x, y );
-		    return found;
-		    }
+		ss= &(selRootEi->eiItem->biSectSelectionScope);
+		selRootBodySectBi= docBi->biChildren[ss->ssSectNrExternalTo];
 		}
-	    if  ( bi->biChildCount > 0 )
-		{
-		child= bi->biChildren[bi->biChildCount- 1];
 
-		found= tedFindItem( child, add, x, y );
-		return found;
-		}
-	    LDEB(bi->biChildCount);
+	    y0Twips= selRootEi->eiY0UsedTwips;
 	    break;
 
-	case DOClevROW:
-	    if  ( ! bi->biRowHasTableParagraphs )
-		{ goto rowAsGroup;	}
+	case DOCinFOOTNOTE:
+	case DOCinENDNOTE:
+	    *pChanged= 0;
+	    return 0;
 
-	    cp= bi->biRowProperties.rpCells;
-	    for ( i= 0; i < bi->biChildCount; cp++, i++ )
-		{
-		BufferItem *	paraBi;
-		int		j;
-		int		right= cp->cpRightBoundaryPixels;
+	case DOCinFTNSEP:
 
-		if  ( cp->cpMergedWithLeft )
-		    { continue; }
+	    if  ( selRootEi->eiPageSelectedUpon < 0 )
+		{ LDEB(selRootEi->eiPageSelectedUpon); return -1;	}
 
-		if  ( cp->cpLeftInMergedRow )
-		    {
-		    const CellProperties *	cpp= cp;
+	    if  ( selRootEi->eiPageFormattedFor ==
+					    selRootEi->eiPageSelectedUpon )
+		{ *pChanged= 0; return 0;	}
 
-		    for ( j= i; j < bi->biChildCount- 1; cpp++, j++ )
-			{
-			if  ( ! cpp[1].cpMergedWithLeft )
-			    { break;	}
+	    if  ( tedCheckPageOfSelectedNoteSeparator( pChanged, &y0Twips,
+						bd, add, selRootEi,
+						DOCinFOOTNOTE, DOCinFTNSEP ) )
+		{ LDEB(1); return -1;	}
 
-			right= cpp[1].cpRightBoundaryPixels;
-			}
-		    }
+	    break;
 
-		if  ( x <= add->addDocRect.drX0+ right		||
-		      i == bi->biChildCount- 1			)
-		    {
-		    child= bi->biChildren[i];
-		    paraBi= tedFindItem( child, add, x, y );
+	case DOCinFTNSEPC:
+	case DOCinFTNCN:
+	    LDEB(selRootEi->eiItem->biInExternalItem);
+	    return -1;
 
-		    if  ( paraBi )
-			{ return paraBi;	}
+	case DOCinAFTNSEP:
 
-		    if  ( child->biChildCount > 0 )
-			{
-			found= child->biChildren[child->biChildCount-1];
-			return found;
-			}
-		    }
-		}
+	    if  ( selRootEi->eiPageSelectedUpon < 0 )
+		{ LDEB(selRootEi->eiPageSelectedUpon); return -1;	}
 
-	    LLDEB(x,y);
-	    return (BufferItem *)0;
+	    if  ( selRootEi->eiPageFormattedFor ==
+					    selRootEi->eiPageSelectedUpon )
+		{ *pChanged= 0; return 0;	}
 
-	case DOClevPARA:
-	    return bi;
+	    if  ( tedCheckPageOfSelectedNoteSeparator( pChanged, &y0Twips,
+						bd, add, selRootEi,
+						DOCinENDNOTE, DOCinAFTNSEP ) )
+		{ LDEB(1); return -1;	}
+
+	    break;
+
+	case DOCinAFTNSEPC:
+	case DOCinAFTNCN:
+	    LDEB(selRootEi->eiItem->biInExternalItem);
+	    return -1;
 
 	default:
-	    LDEB(bi->biLevel); return (BufferItem *)0;
+	    LDEB(selRootEi->eiItem->biInExternalItem);
+	    return -1;
 	}
 
-    LSDEB(y,docLevelStr(bi->biLevel)); return (BufferItem *)0;
+    if  ( selRootEi->eiPageSelectedUpon < 0 )
+	{ LDEB(selRootEi->eiPageSelectedUpon); return -1;	}
+
+    if  ( selRootEi->eiPageFormattedFor == selRootEi->eiPageSelectedUpon )
+	{ *pChanged= 0; return 0;	}
+
+    if  ( docLayoutExternalItem( selRootEi, drChanged,
+		selRootEi->eiPageSelectedUpon, y0Twips,
+		bd, selRootBodySectBi, add,
+		tedLayoutExternalItem, tedCloseObject ) )
+	{ LDEB(selRootEi->eiPageSelectedUpon); return -1; }
+
+    *pChanged= 1; return 0;
     }
 
-static int tedFindLine(	const BufferItem *	bi,
-			const AppDrawingData *	add,
-			int			x,
-			int			y	)
-    {
-    int			page= y/ add->addPageStepPixels;
-
-    int			i;
-    TextLine *		tl;
-    int			y1;
-
-    if  ( y < BI_TOP_PIXELS( add, bi ) )
-	{ return 0;	}
-
-    if  ( bi->biLevel != DOClevPARA )
-	{ LLDEB(bi->biLevel,DOClevPARA); return -1;	}
-
-    tl= bi->biParaLines;
-    for ( i= 0; i < bi->biParaLineCount; tl++, i++ )
-	{
-	y1= TL_BELOW_PIXELS( add, tl );
-
-	if  ( y < y1						||
-	      ( i+ 1 < bi->biParaLineCount		&&
-	        tl[1].tlTopPosition.lpPage > page	)	)
-	    { return i;	}
-	}
-
-    if  ( bi->biParaLineCount > 0 )
-	{ return bi->biParaLineCount- 1;	}
-
-    LLDEB(y,bi->biParaLineCount); return -1;
-    }
-
-int tedFindParticule(	TextLine *	tl,
-			TextParticule *	tp,
-			int		x,
-			int		y	)
-    {
-    int				i;
-
-    tp += tl->tlFirstParticule;
-
-    if  ( x < tp->tpX0 )
-	{ return tl->tlFirstParticule;	}
-
-    for ( i= 0; i < tl->tlParticuleCount; tp++, i++ )
-	{
-	if  ( tp->tpX0 <= x &&  x < tp->tpX0+ tp->tpPixelsWide )
-	    { return tl->tlFirstParticule+ i;	}
-	}
-
-    return tl->tlFirstParticule+ tl->tlParticuleCount- 1;
-    }
-
-/************************************************************************/
-/*									*/
-/*  Return the X position for an I bar.					*/
-/*									*/
-/************************************************************************/
-
-int tedCalculateX(	const BufferItem *	bi,
-			const TextParticule *	tp,
-			const AppDrawingData *	add,
-			int			stroff	)
-    {
-    const AppPhysicalFontList *	apfl= &(add->addPhysicalFontList);
-    AppPhysicalFont *		apf;
-    const unsigned char *	s;
-    int				len;
-    int				x;
-
-    switch( tp->tpKind )
-	{
-	case DOCkindOBJECT:
-	case DOCkindTAB:
-	    len= stroff- tp->tpStroff;
-
-	    if  ( len == 0 )
-		{ x= tp->tpX0;	}
-	    else{
-		if  ( len == 1 )
-		    { x= tp->tpX0+ tp->tpPixelsWide;	}
-		else{
-		    LLDEB(stroff,tp->tpStroff);
-		    docListParticule( 0, "???", tp-bi->biParaParticules,
-								    bi, tp );
-		    docListItem( 0, bi );
-		    x= tp->tpX0;
-		    }
-		}
-	    return x;
-	case DOCkindTEXT:
-	    if  ( tp->tpPhysicalFont < 0 )
-		{ LDEB(tp->tpPhysicalFont); return tp->tpX0;	}
-
-	    s= bi->biParaString+ tp->tpStroff;
-	    apf= apfl->apflFonts+ tp->tpPhysicalFont;
-	    len= stroff- tp->tpStroff;
-
-	    if  ( len == tp->tpStrlen )
-		{ x= tp->tpX0+ tp->tpPixelsWide;			}
-	    else{ x= tp->tpX0+ tedTextWidth( apf, add, s, len );	}
-	    return x;
-	case DOCkindFIELDSTART:
-	case DOCkindFIELDEND:
-	case DOCkindXE:
-	case DOCkindTC:
-	case DOCkindLINEBREAK:
-	case DOCkindPAGEBREAK:
-	    x= tp->tpX0;
-	    return x;
-	default:
-	    LDEB(tp->tpKind);
-	    x= tp->tpX0;
-	    return x;
-	}
-    }
-
-/************************************************************************/
-/*									*/
-/*  Return the offset in the paragraph string of a mouse event		*/
-/*									*/
-/*  1)  For Empty particules report the position of the beginning of	*/
-/*	the particule.							*/
-/*	For clicks in the left margin, report the position of the	*/
-/*	particule.							*/
-/*  2)  Note the <=, also consider the position after the last		*/
-/*	character.							*/
-/*									*/
-/************************************************************************/
-
-int tedFindStringOffset(	const TextParticule *	tp,
-				const unsigned char *	paraString,
-				const AppDrawingData *	add,
-				int *			pBarX,
-				int			x,
-				int			y	)
-    {
-    const AppPhysicalFontList *	apfl= &(add->addPhysicalFontList);
-    int				len;
-    int				stroff= tp->tpStroff+ 1;
-    const unsigned char *	s= paraString+ tp->tpStroff;
-    AppPhysicalFont *		apf;
-    int				before= tp->tpX0;
-    int				past= tp->tpX0;
-
-    /*  1  */
-    if  ( x < before || tp->tpStrlen == 0 )
-	{ *pBarX= tp->tpX0; return tp->tpStroff; }
-
-    if  ( tp->tpPhysicalFont < 0 )
-	{ LDEB(tp->tpPhysicalFont); return tp->tpStroff;	}
-
-    apf= apfl->apflFonts+ tp->tpPhysicalFont;
-
-    /*  2  */
-    for ( len= 1; len <= tp->tpStrlen; stroff++, len++ )
-	{
-	int	between;
-
-	if  ( len == tp->tpStrlen )
-	    { past= tp->tpX0+ tp->tpPixelsWide;			}
-	else{ past= tp->tpX0+ tedTextWidth( apf, add, s, len );	}
-
-	between= ( before+ past )/ 2;
-
-	if  ( before <= x && x < between )
-	    { *pBarX= before; return stroff- 1;	}
-	if  ( between <= x && x < past )
-	    { *pBarX= past; return stroff;		}
-
-	before= past;
-	}
-
-    *pBarX= tp->tpX0+ tp->tpPixelsWide; return tp->tpStroff+ tp->tpStrlen;
-    }
-
-/************************************************************************/
-/*									*/
-/*  Translate a Window coordinate to a position in a text buffer.	*/
-/*									*/
-/************************************************************************/
-
-int tedFindPosition(	BufferItem *		rootBi,
-			const AppDrawingData *	add,
-			int			x,
-			int			y,
-			BufferPosition *	bp )
-    {
-    const BufferItem *	bi;
-
-    int			part;
-    int			stroff;
-    int			barX;
-    int			line;
-
-    TextLine *		tl;
-
-    bi= tedFindItem( rootBi, add, x, y );
-    if  ( ! bi )
-	{
-	if  ( y < BI_TOP_PIXELS( add, rootBi ) )
-	    {
-	    if  ( docFirstPosition( rootBi, bp ) )
-		{ LDEB(y); return -1; }
-
-	    bi= bp->bpBi;
-	    return 0;
-	    }
-
-	if  ( y >= BI_BELOW_PIXELS( add, rootBi ) )
-	    {
-	    if  ( docLastPosition( rootBi, bp ) )
-		{ LDEB(y); return -1; }
-
-	    bi= bp->bpBi;
-	    /*
-	    *pTl= bi->biParaLines+ bp->bpLine;
-	    *pTp= bi->biParaParticules+ bp->bpParticule;
-	    */
-
-	    return 0;
-	    }
-
-	LDEB(y);
-	return -1;
-	}
-
-    if  ( bi->biLevel != DOClevPARA )
-	{ LLDEB(bi->biLevel,DOClevPARA); return -1;	}
-
-    if  ( bi->biParaLineCount > 0				&&
-	  y < TL_TOP_PIXELS( add, &(bi->biParaLines[0]) )	)
-	{
-	line= 0;
-	part= 0;
-	stroff= 0;
-	barX= bi->biParaParticules[0].tpX0;
-	}
-    else{
-	line= tedFindLine( bi, add, x, y );
-	if  ( line < 0 )
-	    {
-	    if  ( bi->biParaLineCount < 1 )
-		{ LDEB(bi->biParaLineCount); return -1;	}
-
-	    if  ( y > BI_BELOW_PIXELS( add, bi )	&&
-		  bi->biParaLineCount > 0		&&
-		  bi->biParaParticuleCount > 0		)
-		{
-		line= bi->biParaLineCount- 1;
-		part= bi->biParaParticuleCount- 1;
-		stroff= bi->biParaStrlen;
-		barX= bi->biParaParticules[part].tpX0+
-				    bi->biParaParticules[part].tpPixelsWide;
-		}
-	    else{
-		line= 0; part= 0; stroff= 0;
-		if  ( bi->biParaParticuleCount == 0 )
-		    { LLDEB(line,bi->biParaParticuleCount); return -1;	}
-		barX= bi->biParaParticules[part].tpX0;
-		}
-	    }
-	else{
-	    part= tedFindParticule( bi->biParaLines+ line,
-						bi->biParaParticules, x, y );
-	    if  ( part < 0 )
-		{ LLDEB(x,part); return -1;			}
-
-	    stroff= tedFindStringOffset( bi->biParaParticules+ part,
-					bi->biParaString, add, &barX, x, y );
-	    if  ( stroff < 0 )
-		{ LDEB(stroff); return -1;	}
-	    }
-	}
-
-#   if 0
-    {
-    TextParticule *	tp= bi->biParaParticules+ part;
-
-    appDebug( "(%3d,%3d): %.*s|%.*s\n", x, y,
-			stroff- tp->tpStroff, bi->biParaString+ tp->tpStroff,
-			tp->tpStroff+ tp->tpStrlen- stroff,
-			bi->biParaString+ stroff );
-    }
-#   endif
-
-    tl= bi->biParaLines+ line;
-
-    bp->bpBi= (BufferItem *)bi;	/*  Discards const */
-    bp->bpLine= line;
-    bp->bpParticule= part;
-    bp->bpStroff= stroff;
-
-    bp->bpXPixels= barX;
-    bp->bpTopPosition= tl->tlTopPosition;
-    bp->bpY1Pixels= TL_BELOW_PIXELS( add, tl );
-    bp->bpBaselinePixels= TL_BASE_PIXELS( add, tl );
-
-    return 0;
-    }
-
-/************************************************************************/
-/*									*/
-/*  Calculate the smallest rectangle that contains the selection.	*/
-/*									*/
-/*  1)  Same paragraph?							*/
-/*  1b) For a single position, widen to contain the whole I-Bar.	*/
-/*  2)  Same table row?							*/
-/*  3)  Expose the whole cell.						*/
-/*									*/
-/************************************************************************/
-
-#   define	IW		5
-
-#   define	minX0(ff)	\
-		((ff).ffX0TextLinesPixels < (ff).ffX0FirstLinePixels ? \
-		 (ff).ffX0TextLinesPixels : (ff).ffX0FirstLinePixels )
-
-void tedSelectionRectangle(	DocumentRectangle *		drClip,
-				const AppDrawingData *		add,
-				const DocumentProperties *	dp,
-				const BufferSelection *		bs )
-    {
-    DocumentRectangle		dr;
-
-    FormattingFrame		ff;
-    const int			bottom= -1;
-    const int			stripHigh= -1;
-
-    drClip->drY0= BP_TOP_PIXELS( add, &(bs->bsBegin) );
-    drClip->drY1= bs->bsEnd.bpY1Pixels;
-
-    /*  1  */
-    if  ( bs->bsEnd.bpBi == bs->bsBegin.bpBi )
-	{
-	if  ( bs->bsEnd.bpLine == bs->bsBegin.bpLine )
-	    {
-	    /*  1b  */
-	    if  ( bs->bsEnd.bpStroff == bs->bsBegin.bpStroff )
-		{
-		drClip->drX0= bs->bsBegin.bpXPixels- IW;
-		drClip->drX1= bs->bsEnd.bpXPixels+ IW;
-		}
-	    else{
-		drClip->drX0= bs->bsBegin.bpXPixels;
-		drClip->drX1= bs->bsEnd.bpXPixels;
-		}
-
-	    return;
-	    }
-
-	docParagraphFrameTwips( &ff, bottom, stripHigh,
-				    bs->bsBegin.bpBi, dp,
-				    bs->bsBegin.bpBi->biTopPosition.lpPage,
-				    bs->bsBegin.bpBi->biTopPosition.lpColumn );
-	tedParagraphFramePixels( &ff, add, bs->bsBegin.bpBi );
-
-	drClip->drX0= minX0( ff );
-	drClip->drX1= ff.ffX1TextLinesPixels;
-
-	return;
-	}
-
-    /*  2  */
-    if  ( ! bs->bsBegin.bpBi->biParaInTable		||
-	  ! bs->bsEnd.bpBi->biParaInTable		||
-	  ! docSelectionInsideRow( bs )			)
-	{
-	drClip->drX0= add->addBackRect.drX0;
-	drClip->drX1= add->addBackRect.drX1;
-	}
-
-    /*  3  */
-    if  ( bs->bsBegin.bpBi->biParaInTable )
-	{
-	docParagraphFrameTwips( &ff, bottom, stripHigh,
-				    bs->bsBegin.bpBi, dp,
-				    bs->bsBegin.bpBi->biTopPosition.lpPage,
-				    bs->bsBegin.bpBi->biTopPosition.lpColumn );
-	tedParagraphFramePixels( &ff, add, bs->bsBegin.bpBi );
-
-	dr.drX0= minX0( ff );
-	dr.drX1= ff.ffX1TextLinesPixels;
-
-	if  ( docSelectionInsideCell( bs ) )
-	    {
-	    dr.drY0= BI_TOP_PIXELS( add, bs->bsBegin.bpBi );
-	    dr.drY1= BI_BELOW_PIXELS( add, bs->bsBegin.bpBi );
-	    }
-	else{
-	    BufferItem *	rowBi= bs->bsBegin.bpBi->biParent->biParent;
-
-	    dr.drY0= BI_TOP_PIXELS( add, rowBi );
-	    dr.drY1= BI_BELOW_PIXELS( add, rowBi );
-	    }
-
-	docUnionRectangle( drClip, drClip, &dr );
-	}
-
-    /*  3  */
-    if  ( bs->bsEnd.bpBi->biParaInTable )
-	{
-	docParagraphFrameTwips( &ff, bottom, stripHigh,
-				    bs->bsEnd.bpBi, dp,
-				    bs->bsEnd.bpBi->biTopPosition.lpPage,
-				    bs->bsEnd.bpBi->biTopPosition.lpColumn );
-	tedParagraphFramePixels( &ff, add, bs->bsEnd.bpBi );
-
-	dr.drX0= minX0( ff );
-	dr.drX1= ff.ffX1TextLinesPixels;
-
-	if  ( docSelectionInsideCell( bs ) )
-	    {
-	    dr.drY0= BI_TOP_PIXELS( add, bs->bsEnd.bpBi );
-	    dr.drY1= BI_BELOW_PIXELS( add, bs->bsEnd.bpBi );
-	    }
-	else{
-	    BufferItem *	rowBi= bs->bsEnd.bpBi->biParent->biParent;
-
-	    dr.drY0= BI_TOP_PIXELS( add, rowBi );
-	    dr.drY1= BI_BELOW_PIXELS( add, rowBi );
-	    }
-
-	docUnionRectangle( drClip, drClip, &dr );
-	}
-
-    return;
-    }
-
-void tedPositionCoordinates(	BufferPosition *	bp,
-				const AppDrawingData *	add )
-    {
-    BufferItem *	bi;
-    TextLine *		tl;
-    TextParticule *	tp;
-
-    bi= bp->bpBi;
-    tl= bi->biParaLines+ bp->bpLine;
-    tp= bi->biParaParticules+ bp->bpParticule;
-
-    bp->bpXPixels= tedCalculateX( bi, tp, add, bp->bpStroff );
-    bp->bpTopPosition= tl->tlTopPosition;
-    bp->bpY1Pixels= TL_BELOW_PIXELS( add, tl );
-    bp->bpBaselinePixels= TL_BASE_PIXELS( add, tl );
-
-#   if 0
-    Gives silly high I Bar.
-    if  ( bp->bpLine == bi->biParaLineCount- 1 )
-	{ bp->bpY1 += bi->biParaSpaceAfterPixels; }
-#   endif
-
-    return;
-    }
-
-void tedSelectionCoordinates(	BufferSelection *	bs,
-				const AppDrawingData *	add )
-    {
-    tedPositionCoordinates( &(bs->bsBegin), add );
-    tedPositionCoordinates( &(bs->bsEnd), add );
-    }
