@@ -4,7 +4,7 @@
 /*									*/
 /************************************************************************/
 
-#   include	"config.h"
+#   include	"tedConfig.h"
 
 #   include	<stddef.h>
 #   include	<stdlib.h>
@@ -12,11 +12,10 @@
 #   include	<string.h>
 #   include	<ctype.h>
 
-#   include	<signal.h>
+#   include	<appSystem.h>
 
 #   include	"tedApp.h"
 #   include	"tedRuler.h"
-#   include	"docPs.h"
 
 #   include	<appSymbolPicker.h>
 #   include	<appSpellTool.h>
@@ -30,7 +29,6 @@
 
 #   include	<appDebugon.h>
 
-
 static AppConfigurableResource TEDApplicationResourceTable[]=
     {
 	/****************************************************************/
@@ -96,9 +94,15 @@ static AppConfigurableResource TEDApplicationResourceTable[]=
 		    "Help" ),
 
 	/**/
+#	ifdef __VMS
 	APP_RESOURCE( "documentFileName",
 		    offsetof(TedAppResources,tarAppHelpFileName),
-		    INFODIR "/TedDocument.rtf" ),
+		    "TED$ROOT:[Ted]TedDocument-en_US.rtf" ),
+#	else
+	APP_RESOURCE( "documentFileName",
+		    offsetof(TedAppResources,tarAppHelpFileName),
+		    DOCUMENT_DIR "TedDocument-en_US.rtf" ),
+#	endif
 
 	/**/
 	APP_RESOURCE( "selectionColor",
@@ -111,7 +115,12 @@ static AppConfigurableResource TEDApplicationResourceTable[]=
 	/**/
 	APP_RESOURCE( "defaultAnsicpg",
 		    offsetof(TedAppResources,tarDefaultAnsicpgString),
-		    (char *)0 ),
+		    "1252" ),
+
+	/**/
+	APP_RESOURCE( "showTableGrid",
+		    offsetof(TedAppResources,tarShowTableGridString),
+		    "1" ),
     };
 
 
@@ -179,169 +188,6 @@ static AppConfigurableResource TEDFileMessageResourceTable[]=
 
 static TedAppResources			TEDResources;
 
-/************************************************************************/
-/*									*/
-/*  Open a document.							*/
-/*									*/
-/*  1)  Open an input stream.						*/
-/*  2)  Try to read as RTF.						*/
-/*  3)  If this fails, try as plain text.				*/
-/*									*/
-/************************************************************************/
-
-static int tedOpenDocumentFile(	EditApplication *	ea,
-				int *			pFormat,
-				BufferDocument **	pBd,
-				const char *		filename,
-				APP_WIDGET		relative,
-				APP_WIDGET		option )
-    {
-    const char *		slash= strrchr( filename, '/' );
-    const char *		dot;
-    SimpleInputStream *		sis;
-
-    BufferDocument *		bd;
-    int				triedRtf= 0;
-
-    int				resp;
-    AppFileMessageResources *	afmr= &(ea->eaFileMessageResources);
-    TedAppResources *		tar= (TedAppResources *)ea->eaResourceData;
-
-    int				longestPara;
-
-    if  ( tar->tarDefaultAnsicpgInt < 0		&&
-	  tar->tarDefaultAnsicpgString		)
-	{
-	char *		past;
-	int		val;
-
-	val= strtol( tar->tarDefaultAnsicpgString, &past, 10 );
-	if  ( past != tar->tarDefaultAnsicpgString )
-	    {
-	    while( *past == ' ' )
-		{ past++;	}
-	    }
-	if  ( past != tar->tarDefaultAnsicpgString	&&
-	      ! *past					)
-	    { tar->tarDefaultAnsicpgInt= val;		}
-	else{ SDEB(tar->tarDefaultAnsicpgString);	}
-	}
-
-    if  ( slash )
-	{ dot= strrchr( slash+ 1, '.' );	}
-    else{ dot= strrchr( filename, '.' );	}
-
-    /*  1  */
-    sis= sioInStdioOpen( filename );
-    if  ( ! sis )
-	{
-	appQuestionRunSubjectErrorDialog( ea, relative, option,
-					filename, afmr->afmrFileNoAccess );
-
-	return -1;
-	}
-
-    if  ( dot && ! strcmp( dot, ".rtf" ) )
-	{
-	/*  2  */
-	bd= docRtfReadFile( sis, tar->tarDefaultAnsicpgInt );
-
-	sioInClose( sis );
-	
-	if  ( bd )
-	    {
-	    *pFormat= TEDdockindRTF; *pBd= bd;
-
-	    if  ( docPropertiesSetFilename( &(bd->bdProperties), filename ) )
-		{ LDEB(1);	}
-
-	    return 0;
-	    }
-	else{
-	    resp= appQuestionRunSubjectOkCancelDialog( ea,
-				    relative, option,
-				    filename, TEDResources.tarFileNotRtf,
-				    (char *)0, (char *)0 );
-
-	    if  ( resp != AQDrespOK )
-		{ return -1;	}
-
-	    triedRtf= 1;
-	    }
-	}
-
-    if  ( dot && ! strcmp( dot, ".txt" ) )
-	{
-	bd= docPlainReadFile( sis, &longestPara,
-					&(ea->eaDefaultDocumentGeometry) );
-
-	sioInClose( sis );
-
-	if  ( bd )
-	    {
-	    *pBd= bd;
-
-	    if  ( longestPara > 76 )
-		{ *pFormat= TEDdockindTEXT_SAVE_WIDE;		}
-	    else{ *pFormat= TEDdockindTEXT_SAVE_FOLDED;		}
-
-	    if  ( docPropertiesSetFilename( &(bd->bdProperties), filename ) )
-		{ LDEB(1);	}
-
-	    return 0;
-	    }
-	else{ SXDEB(filename,bd); return -1;	}
-	}
-
-    /*  2  */
-    if  ( ! triedRtf )
-	{
-	bd= docRtfReadFile( sis, tar->tarDefaultAnsicpgInt );
-	if  ( bd )
-	    {
-	    *pFormat= TEDdockindRTF; *pBd= bd;
-	    
-	    if  ( docPropertiesSetFilename( &(bd->bdProperties), filename ) )
-		{ LDEB(1);	}
-
-	    return 0;
-	    }
-
-	resp= appQuestionRunSubjectOkCancelDialog( ea,
-					ea->eaToplevel.atTopWidget, option,
-					filename, TEDResources.tarFileNotRtf,
-					(char *)0, (char *)0 );
-
-	if  ( resp != AQDrespOK )
-	    { return -1;	}
-	}
-
-    sis= sioInStdioOpen( filename );
-    if  ( ! sis )
-	{ SXDEB(filename,sis); return -1;	}
-
-    bd= docPlainReadFile( sis, &longestPara,
-				    &(ea->eaDefaultDocumentGeometry) );
-
-    sioInClose( sis );
-
-    if  ( bd )
-	{
-	*pBd= bd;
-
-	if  ( longestPara > 76 )
-	    { *pFormat= TEDdockindTEXT_SAVE_WIDE;		}
-	else{ *pFormat= TEDdockindTEXT_SAVE_FOLDED;		}
-
-	if  ( docPropertiesSetFilename( &(bd->bdProperties), filename ) )
-	    { LDEB(1);	}
-
-	return 0;
-	}
-
-    SXDEB(filename,bd); return -1;
-    }
-
 /************************************************************************/
 /*									*/
 /*  Open a document.							*/
@@ -357,7 +203,7 @@ static int tedOpenDocument(	EditApplication *	ea,
     {
     TedDocument *	td= (TedDocument *)voidtd;
 
-    if  ( tedOpenDocumentFile( ea, pFormat, &td->tdDocument,
+    if  ( tedOpenDocumentFile( ea, pFormat, &(td->tdDocument),
 					    filename, relative, option ) )
 	{ /*SDEB(filename);*/ return -1;	}
 
@@ -376,15 +222,15 @@ static void tedDocFilePrint(	APP_WIDGET	printOption,
     int			firstPage= -1;
     int			lastPage= -1;
 
-    SelectionScope	ss;
-    BufferSelection	bs;
+    DocumentSelection	ds;
+    SelectionGeometry	sg;
 
     const DocumentGeometry *	dgDoc= &(bd->bdProperties.dpGeometry);
 
-    if  ( ! tedGetSelection( &ss, &bs, td )	)
+    if  ( ! tedGetSelection( &ds, &sg, td )	)
 	{
-	firstPage= bs.bsBegin.bpTopPosition.lpPage;
-	lastPage= bs.bsEnd.bpTopPosition.lpPage;
+	firstPage= sg.sgBegin.pgTopPosition.lpPage;
+	lastPage= sg.sgEnd.pgTopPosition.lpPage;
 	}
 
     appRunPrintDialog( ed, dgDoc, pageCount, firstPage, lastPage,
@@ -399,7 +245,7 @@ static void tedDocFilePrint(	APP_WIDGET	printOption,
 /*									*/
 /************************************************************************/
 
-static APP_DESTROY_CALLBACK( tedManualDestroyed, w, voidtar )
+static APP_DESTROY_CALLBACK_H( tedManualDestroyed, w, voidtar )
     {
     TedAppResources *	tar= (TedAppResources *)voidtar;
 
@@ -413,14 +259,14 @@ static void tedManual(		APP_WIDGET		option,
     {
     TedAppResources *	tar= (TedAppResources *)ea->eaResourceData;
 
-    const int		readonly= 1;
+    const int		read_only= 1;
     const char *	file= TEDResources.tarAppHelpFileName;
 
     if  ( tar->tarManualDocument )
 	{ appMakeDocVisible( ea, tar->tarManualDocument );	}
     else{
 	tar->tarManualDocument= appOpenDocument( ea, relative,
-						    option, readonly, file );
+						    option, read_only, file );
 
 	if  ( ! tar->tarManualDocument )
 	    { SXDEB(file,tar->tarManualDocument); return;	}
@@ -451,116 +297,6 @@ static void tedAppManual(	APP_WIDGET	option,
     tedManual( option, ea, voidpbcs, ea->eaToplevel.atTopWidget );
     }
 
-/************************************************************************/
-/*									*/
-/*  Handle a replace from one of the tools.				*/
-/*									*/
-/************************************************************************/
-
-void tedAppReplace(	void *			voidea,
-			const unsigned char *	word	)
-    {
-    EditApplication *	ea= (EditApplication *)voidea;
-    EditDocument *	ed= ea->eaCurrentDocument;
-    TedDocument *	td;
-
-    if  ( ! ed )
-	{ XDEB(ed); return;	}
-
-    td= (TedDocument *)ed->edPrivateData;
-
-    if  ( tedHasSelection( td )		&&
-	  ! tedHasIBarSelection( td )	&&
-	  td->tdCanReplaceSelection	)
-	{ tedAppReplaceSelection( ed, word, strlen( (char *)word ) ); }
-
-    return;
-    }
-
-/************************************************************************/
-/*									*/
-/*  Callback for the 'Insert File' menu option.				*/
-/*									*/
-/************************************************************************/
-
-static int tedInsertFile(	void *		voided,
-				APP_WIDGET	relative,
-				APP_WIDGET	option,
-				const char *	filename )
-    {
-    EditDocument *		ed= (EditDocument *)voided;
-    EditApplication *		ea= ed->edApplication;
-
-    int				format;
-    BufferDocument *		bd;
-
-    if  ( tedOpenDocumentFile( ea, &format, &bd, filename, relative, option ) )
-	{ SDEB(filename); return -1;	}
-
-    switch( format )
-	{
-	case 0:
-	    if  ( tedIncludeRtfDocument( ed->edDocumentWidget, ed, bd ) )
-		{ LDEB(1); docFreeDocument( bd ); return -1;	}
-	    break;
-	case 1:
-	    if  ( tedIncludePlainDocument( ed->edDocumentWidget, ed, bd ) )
-		{ LDEB(1); docFreeDocument( bd ); return -1;	}
-	    break;
-	default:
-	    LDEB(format);
-	    docFreeDocument( bd ); return -1;
-	}
-	    
-    docFreeDocument( bd );
-
-    appDocumentChanged( ed, 1 );
-
-    return 0;
-    }
-
-void tedDocInsertFile(		APP_WIDGET	option,
-				void *		voided,
-				void *		voidpbcs )
-    {
-    EditDocument *		ed= (EditDocument *)voided;
-    EditApplication *		ea= ed->edApplication;
-    TedDocument *		td= (TedDocument *)ed->edPrivateData;
-
-    if  ( ! td->tdCanReplaceSelection )
-	{ LDEB(td->tdCanReplaceSelection); return;	}
-
-    appRunOpenChooser( option, ed->edToplevel.atTopWidget,
-			    ea->eaFileExtensionCount, ea->eaFileExtensions,
-			    ea->eaDefaultFileFilter,
-			    voided, tedInsertFile, ea );
-
-    return;
-    }
-
-/************************************************************************/
-/*  Font Menu.								*/
-/************************************************************************/
-
-static void tedDocFontPaste(	APP_WIDGET	fontsOption,
-				void *		voided,
-				void *		voidpbcbs	 )
-    {
-    EditDocument *			ed= (EditDocument *)voided;
-
-    appDocAskForPaste( ed, "FONT" );
-    }
-
-static void tedDocToolFont(	APP_WIDGET	fontsOption,
-				void *		voided,
-				void *		voidcbs	 )
-    {
-    EditDocument *	ed= (EditDocument *)voided;
-    EditApplication *	ea= ed->edApplication;
-
-    tedShowFontsTool( fontsOption, ea, voidcbs );
-    }
-
 /************************************************************************/
 /*									*/
 /*  Table related menu option callbacks.				*/
@@ -588,33 +324,9 @@ static void tedDocTableAddRow(		APP_WIDGET	option,
 					void *		voided,
 					void *		voidpbcbs	 )
     {
-    EditDocument *		ed= (EditDocument *)voided;
-    TedDocument *		td= (TedDocument *)ed->edPrivateData;
+    EditDocument *	ed= (EditDocument *)voided;
 
-    int				rows= 1;
-
-    BufferItem *		sectBi;
-    const RowProperties *	rp;
-
-    int				col;
-    int				row;
-    int				row0;
-    int				row1;
-
-    SelectionScope		ss;
-    BufferSelection		bs;
-
-    if  ( tedGetSelection( &ss, &bs, td ) )
-	{ return;	}
-
-    if  ( docDelimitTable( bs.bsBegin.bpBi,
-				    &sectBi, &col, &row0, &row, &row1 ) )
-	{ LDEB(1); return;	}
-
-    rp= &(sectBi->biChildren[row]->biRowProperties);
-
-    if  ( tedInsertRowsInTable( sectBi, rp, ed, row+ 1, rows ) )
-	{ LDEB(rows);	}
+    tedAppendRowToTable( ed );
 
     appDocumentChanged( ed, 1 );
 
@@ -651,33 +363,67 @@ static void tedDocShowPageTool(		APP_WIDGET	pageOption,
     {
     EditDocument *	ed= (EditDocument *)voided;
     EditApplication *	ea= ed->edApplication;
-    TedDocument *	td= (TedDocument *)ed->edPrivateData;
 
 #   if 0
+
     Use the page layout page of the format tool instead
 
     if  ( ! ea->eaPageTool )
 	{
 	char *			pixmapName= "page";
 
-	ea->eaPageTool= appMakePageTool( ea, pageOption,
-					    pixmapName, tedSetPageLayout );
+	ea->eaPageTool= appMakePageTool( ea, pageOption, pixmapName );
 	}
     else{ appShowPageTool( ea ); }
 
     tedAdaptPageToolToDocument( ea, ed );
 #   else
 
+    {
+    TedDocument *	td= (TedDocument *)ed->edPrivateData;
+
     /*  To get the correct title  */
     tedDocFormatTool( td->tdToolsFormatToolOption, voided, voidcbs );
 
     tedFormatShowPagePage( ea );
+    }
 
 #   endif
 
     return;
     }
 
+/************************************************************************/
+/*									*/
+/*  Font Menu.								*/
+/*									*/
+/************************************************************************/
+
+static void tedDocFontPaste(	APP_WIDGET	fontsOption,
+				void *		voided,
+				void *		voidpbcbs	 )
+    {
+    EditDocument *			ed= (EditDocument *)voided;
+
+    appDocAskForPaste( ed, "FONT" );
+    }
+
+static void tedDocToolFont(	APP_WIDGET	fontsOption,
+				void *		voided,
+				void *		voidcbs	 )
+    {
+    EditDocument *		ed= (EditDocument *)voided;
+    EditApplication *		ea= ed->edApplication;
+    TedDocument *		td= (TedDocument *)ed->edPrivateData;
+
+    /*  To get the correct title  */
+    tedDocFormatTool( td->tdToolsFormatToolOption, voided, voidcbs );
+
+    tedFormatShowFontPage( ea );
+
+    return;
+    }
+
 /************************************************************************/
 /*									*/
 /*  Run document properties dialog.					*/
@@ -699,59 +445,6 @@ static void tedDocFileProps(	APP_WIDGET	option,
     tedRunPropertyDialog( ed, &ds, option, "props" );
     }
 
-/************************************************************************/
-/*									*/
-/*  Custom 'Paste' functionality.					*/
-/*									*/
-/*  1)  The regular X11 manner.						*/
-/*									*/
-/************************************************************************/
-
-static void tedDocEditPaste(	APP_WIDGET	option,
-				void *		voided,
-				void *		voidpbcs	 )
-    {
-    EditDocument *			ed= (EditDocument *)voided;
-    EditApplication *			ea= ed->edApplication;
-    TedDocument *			td= (TedDocument *)ed->edPrivateData;
-
-    int					res;
-    BitmapDescription			bd;
-    unsigned char *			buffer;
-
-    if  ( ! td->tdCanReplaceSelection )
-	{ LDEB(td->tdCanReplaceSelection); return;	}
-
-    res= appDocAskForPaste( ed, "PRIMARY" );
-    if  ( ! res )
-	{ return;	}
-
-    if  ( ! appImgGetXvClipboard( &bd, &buffer, ea->eaToplevel.atTopWidget ) )
-	{
-	AppBitmapImage *	abi;
-
-	abi= (AppBitmapImage *)malloc( sizeof(AppBitmapImage) );
-	if  ( ! abi )
-	    { XDEB(abi); return;	}
-	appInitBitmapImage( abi );
-
-	if  ( bmCopyDescription( &(abi->abiBitmap), &bd ) )
-	    { bmCleanDescription( &bd ); return; }
-
-	bmCleanDescription( &bd );
-	abi->abiBuffer= buffer;
-
-	tedAppReplaceSelection( ed, (unsigned char *)"", 0 );
-
-	if  ( tedObjectInsertBitmap( ed, abi ) )
-	    { LDEB(1); return; }
-
-	return;
-	}
-
-    return;
-    }
-
 /************************************************************************/
 /*									*/
 /*  Mail a document.							*/
@@ -839,76 +532,71 @@ static AppMenuItem TED_AppWinMenuItems[]=
 
 static AppMenuItem TED_DocFileMenuItems[]=
 {
-#   define	TEDmiDocFileNew		0
+#   define	TEDmiDocFileNew			0
     {
     "docFileMenuNewText",	"New",
     "docFileMenuNewKey",	"Ctrl <Key>n",
     "docFileMenuNewKeyText",	"^N",		ITEMtyOPTION, appDocFileNew,
     },
-#   define	TEDmiDocFileOpen	1
+#   define	TEDmiDocFileOpen		1
     {
     "docFileMenuOpenText",	"Open",
     "docFileMenuOpenKey",	"Ctrl <Key>o",
     "docFileMenuOpenKeyText",	"^O",		ITEMtyOPTION, appDocFileOpen,
     },
-#   define	TEDmiDocFileSave	2
+#   define	TEDmiDocFileSave		2
     {
     "docFileMenuSaveText",	"Save",
     "docFileMenuSaveKey",	"Ctrl <Key>s",
     "docFileMenuSaveKeyText",	"^S",		ITEMtyOPTION, appDocFileSave,
     },
-#   define	TEDmiDocFileSaveAs	3
+#   define	TEDmiDocFileSaveAs		3
     {
     "docFileMenuSaveAsText",	"Save As ...",
     "docFileMenuSaveAsKey",	(char *)0,
     "docFileMenuSaveAsKeyText",	(char *)0,	ITEMtyOPTION, appDocFileSaveAs,
     },
-#   define	TEDmiDocFileSaveTo	4
-    {
-    "docFileMenuSaveToText",	"Save To ...",
-    "docFileMenuSaveToKey",	(char *)0,
-    "docFileMenuSaveToKeyText",	(char *)0,	ITEMtyOPTION, appDocFileSaveTo,
-    },
-#   define	TEDmiDocFilePrint	5
+#   define	TEDmiDocFilePrint		4
     {
     "docFileMenuPrintText",	"Print ...",
     "docFileMenuPrintKey",	"Ctrl <Key>p",
     "docFileMenuPrintKeyText",	"^P",		ITEMtyOPTION, tedDocFilePrint,
     },
-#   define	TEDmiDocFileMail	6
+#   define	TEDmiDocFileMail		5
     {
     "docFileMenuMailText",	"Mail ...",
     "docFileMenuMailKey",	(char *)0,
     "docFileMenuMailKeyText",	(char *)0,	ITEMtyOPTION, tedDocFileMail,
     },
-#   define	TEDmiDocFileProperties	7
+#   define	TEDmiDocFileProperties		6
     {
     "docFileMenuPropsText",	"Properties ...",
     "docFileMenuPropsKey",	(char *)0,
     "docFileMenuPropsKeyText",	(char *)0,	ITEMtyOPTION, tedDocFileProps,
     },
-#   define	TEDmiDocFileMinimize	8
+#   define	TEDmiDocFileMinimize		7
     {
     "docFileMenuMiniText",	"Minimize",
     "docFileMenuMiniKey",	"Ctrl <Key>m",
     "docFileMenuMiniKeyText",	"^M",		ITEMtyOPTION, appDocFileMini,
     },
-#   define	TEDmiDocFileHide	9
+#   define	TEDmiDocFileHide		8
     {
     "docFileMenuHideText",	"Hide",
     "docFileMenuHideKey",	"Ctrl <Key>h",
     "docFileMenuHideKeyText",	"^H",		ITEMtyOPTION, appDocFileHide,
     },
-#   define	TEDmiDocFileClose	10
+#   define	TEDmiDocFileClose		9
     {
     "docFileMenuCloseText",	"Close",
     "docFileMenuCloseKey",	(char *)0,
     "docFileMenuCloseKeyText",	(char *)0,	ITEMtyOPTION, appDocFileClose,
     },
 
+#   define	TEDmiDocFileQuitSeparator	10
     { "",	"", "", (char *)0, "", (char *)0, ITEMtySEPARATOR, },
 
-#   define	TEDmiDocFileQuit	12
+#   define	TEDmiDocFileQuit		11
     {
     "docFileMenuQuitText",	"Quit",
     "docFileMenuQuitKey",	(char *)0,
@@ -970,14 +658,14 @@ static AppMenuItem TED_DocInsertMenuItems[]=
 {
 #   define	TEDmiDocInsertInsPict		0
     {
-    "docInsertMenuInsPictText",		"Insert Picture ...",
+    "docInsertMenuInsPictText",		"Picture ...",
     "docInsertMenuInsPictKey",		(char *)0,
     "docInsertMenuInsPictKeyText",	(char *)0,
 					ITEMtyOPTION, tedDocInsertPicture,
     },
 #   define	TEDmiDocInsertInsSymbol		1
     {
-    "docInsertMenuInsertSymbolText",	"Insert Symbol",
+    "docInsertMenuInsertSymbolText",	"Symbol",
     "docInsertMenuInsertSymbolKey",	(char *)0,
     "docInsertMenuInsertSymbolKeyText",	(char *)0,
 					ITEMtyOPTION, tedDocToolInsertSymbol,
@@ -996,60 +684,82 @@ static AppMenuItem TED_DocInsertMenuItems[]=
     "docInsertMenuBookmarkKeyText",	(char *)0,
 					ITEMtyOPTION, tedDocInsertBookmark,
     },
-#   define	TEDmiDocInsertInsFile		4
+#   define	TEDmiDocInsertInsFootnote	4
     {
-    "docInsertMenuInsertFileText",	"Insert File",
+    "docInsertMenuFootnoteText",	"Footnote",
+    "docInsertMenuFootnoteKey",		(char *)0,
+    "docInsertMenuFootnoteKeyText",	(char *)0,
+					ITEMtyOPTION, tedDocInsertFootnote,
+    },
+#   define	TEDmiDocInsertInsEndnote	5
+    {
+    "docInsertMenuEndnoteText",		"Endnote",
+    "docInsertMenuEndnoteKey",		(char *)0,
+    "docInsertMenuEndnoteKeyText",	(char *)0,
+					ITEMtyOPTION, tedDocInsertEndnote,
+    },
+#   define	TEDmiDocInsertInsChftnsep	6
+    {
+    "docInsertMenuChftnsepText",	"Footnote Separator",
+    "docInsertMenuChftnsepKey",		(char *)0,
+    "docInsertMenuChftnsepKeyText",	(char *)0,
+					ITEMtyOPTION, tedDocInsertChftnsep,
+    },
+#   define	TEDmiDocInsertInsFile		7
+    {
+    "docInsertMenuInsertFileText",	"File ...",
     "docInsertMenuInsertFileKey",	(char *)0,
     "docInsertMenuInsertFileKeyText",	(char *)0,
 					ITEMtyOPTION, tedDocInsertFile,
     },
-#   define	TEDmiDocInsertInsTable		5
+#   define	TEDmiDocInsertInsTable		8
     {
-    "docInsertMenuInsertTableText",	"Insert Table",
+    "docInsertMenuInsertTableText",	"Table",
     "docInsertMenuInsertTableKey",	(char *)0,
     "docInsertMenuInsertTableKeyText",	(char *)0,
 					ITEMtyOPTION, tedDocTableInsertTable,
     },
-#   define	TEDmiDocInsertInsPageNumber	6
+#   define	TEDmiDocInsertInsPageNumber	9
     {
-    "docInsertMenuInsertPageNumberText",	"Insert Page Number",
+    "docInsertMenuInsertPageNumberText",	"Page Number",
     "docInsertMenuInsertPageNumberKey",		(char *)0,
     "docInsertMenuInsertPageNumberKeyText",	(char *)0,
 				    ITEMtyOPTION, tedDocInsertPageNumber,
     },
 
-#   if 1
-    /*  7  */
+    /*  10  */
     { "",	"", "", (char *)0, "", (char *)0, ITEMtySEPARATOR, },
 
-#   define	TEDmiDocInsertInsLineBreak	8
+#   define	TEDmiDocInsertInsLineBreak	11
     {
-    "docInsertMenuInsertLineBreakText",	"Insert Line Break",
+    "docInsertMenuInsertLineBreakText",	"Line Break",
     "docInsertMenuInsertLineBreakKey",		(char *)0,
     "docInsertMenuInsertLineBreakKeyText",	(char *)0,
 				    ITEMtyOPTION, tedDocInsertLineBreak,
     },
-#   define	TEDmiDocInsertInsPageBreak	9
+#   define	TEDmiDocInsertInsPageBreak	12
     {
-    "docInsertMenuInsertPageBreakText",	"Insert Page Break",
+    "docInsertMenuInsertPageBreakText",	"Page Break",
     "docInsertMenuInsertPageBreakKey",		(char *)0,
     "docInsertMenuInsertPageBreakKeyText",	(char *)0,
 				    ITEMtyOPTION, tedDocInsertPageBreak,
     },
-#   define	TEDmiDocInsertInsSectBreak	10
+#   define	TEDmiDocInsertInsSectBreak	13
     {
-    "docInsertMenuInsertSectBreakText",	"Insert Section Break",
+    "docInsertMenuInsertSectBreakText",	"Section Break",
     "docInsertMenuInsertSectBreakKey",		(char *)0,
     "docInsertMenuInsertSectBreakKeyText",	(char *)0,
 				    ITEMtyOPTION, tedDocInsertSectBreak,
     },
-#   endif
 
 };
 
 /************************************************************************/
+/*									*/
 /*  Document windows menu.						*/
+/*									*/
 /************************************************************************/
+
 static AppMenuItem TED_DocWinMenuItems[]=
 {
     {
@@ -1335,7 +1045,7 @@ static AppMenuItem TED_AppHelpMenuItems[]=
 
 /************************************************************************/
 /*									*/
-/*  Make a document readonly.						*/
+/*  Make a document read_only.						*/
 /*									*/
 /************************************************************************/
 
@@ -1356,9 +1066,9 @@ void tedMakeDocumentReadonly(	EditDocument *	ed )
     XtUnmanageChild( children[TEDmiDocFileOpen] );
     XtUnmanageChild( children[TEDmiDocFileSave] );
     XtUnmanageChild( children[TEDmiDocFileSaveAs] );
-    XtUnmanageChild( children[TEDmiDocFileSaveTo] );
     XtUnmanageChild( children[TEDmiDocFileMail] );
     XtUnmanageChild( children[TEDmiDocFileProperties] );
+    XtUnmanageChild( children[TEDmiDocFileQuitSeparator] );
     XtUnmanageChild( children[TEDmiDocFileQuit] );
 
     XtVaGetValues( ed->edEditMenu,
@@ -1372,6 +1082,9 @@ void tedMakeDocumentReadonly(	EditDocument *	ed )
     XtUnmanageChild( td->tdInsPictOption );
     XtUnmanageChild( td->tdInsSymbolOption );
     XtUnmanageChild( td->tdInsFileOption );
+    XtUnmanageChild( td->tdInsInsertFootnoteOption );
+    XtUnmanageChild( td->tdInsInsertEndnoteOption );
+    XtUnmanageChild( td->tdInsInsertChftnsepOption );
     XtUnmanageChild( td->tdInsInsertTableOption );
     XtUnmanageChild( td->tdInsInsertPageNumberOption );
 
@@ -1439,6 +1152,12 @@ static void tedMakePrivateDocumentMenus( EditApplication *	ea,
 	    TED_DocInsertMenuItems[TEDmiDocInsertInsHyperlink].amiOptionWidget;
     td->tdInsBookmarkOption=
 	    TED_DocInsertMenuItems[TEDmiDocInsertInsBookmark].amiOptionWidget;
+    td->tdInsInsertFootnoteOption=
+	    TED_DocInsertMenuItems[TEDmiDocInsertInsFootnote].amiOptionWidget;
+    td->tdInsInsertEndnoteOption=
+	    TED_DocInsertMenuItems[TEDmiDocInsertInsEndnote].amiOptionWidget;
+    td->tdInsInsertChftnsepOption=
+	    TED_DocInsertMenuItems[TEDmiDocInsertInsChftnsep].amiOptionWidget;
     td->tdInsInsertTableOption=
 	    TED_DocInsertMenuItems[TEDmiDocInsertInsTable].amiOptionWidget;
     td->tdInsInsertPageNumberOption=
@@ -1535,8 +1254,6 @@ static void tedVisibleDocumentCountChanged(	EditApplication *	ea,
 	    { appEnableSymbolPicker( ea->eaSymbolPicker, 1 ); }
 	if  ( ea->eaSpellTool )
 	    { appEnableSpellTool( ea->eaSpellTool, 1 ); }
-	if  ( ea->eaFontsTool )
-	    { appEnableFontsTool( ea->eaFontsTool, 1 ); }
 	if  ( ea->eaFindTool )
 	    { appEnableFindTool( ea->eaFindTool, 1 ); }
 	if  ( ea->eaPageTool )
@@ -1554,8 +1271,6 @@ static void tedVisibleDocumentCountChanged(	EditApplication *	ea,
 	    { appEnableSymbolPicker( ea->eaSymbolPicker, 0 ); }
 	if  ( ea->eaSpellTool )
 	    { appEnableSpellTool( ea->eaSpellTool, 0 ); }
-	if  ( ea->eaFontsTool )
-	    { appEnableFontsTool( ea->eaFontsTool, 0 ); }
 	if  ( ea->eaFindTool )
 	    { appEnableFindTool( ea->eaFindTool, 0 ); }
 	if  ( ea->eaPageTool )
@@ -1671,10 +1386,8 @@ static int tedWriteRtfMail(	SimpleOutputStream *		sos,
 	    { XDEB(sosBody); return -1;	}
 	}
 
-    rval= docRtfSaveDocument( sosBody, bd,
-				(const SelectionScope *)0,
-				(const BufferSelection *)0,
-				saveBookmarks );
+    rval= docRtfSaveDocument( sosBody, bd, (const DocumentSelection *)0,
+							saveBookmarks );
 
     if  ( mimeBoundary )
 	{
@@ -1705,8 +1418,7 @@ static int tedWritePlainMail(	SimpleOutputStream *		sos,
     {
     return docPlainSaveDocument( sos,
 				(BufferDocument *)voidbd,
-				(const SelectionScope *)0,
-				(const BufferSelection *)0,
+				(const DocumentSelection *)0,
 				1, 1 );
     }
 
@@ -1733,11 +1445,12 @@ static MailContent	TedMailContents[]=
 
 static AppSelectionTargetType TedPrimaryTargets[]=
     {
-	{ "RTF",	(APP_ATOM)0,	tedPastePrimaryRtf,
+	{ "application/rtf",
+			(APP_ATOM)0,	tedPastePrimaryRtf,
 					tedCopyPrimaryRtf		},
 	{ "STRING",	(APP_ATOM)0,	tedPastePrimaryString,
 					tedCopyPrimaryString		},
-	{ "PNG",	(APP_ATOM)0,	tedPastePrimaryPng,
+	{ "image/png",	(APP_ATOM)0,	tedPastePrimaryPng,
 					tedCopyPrimaryPng		},
 	{ "PIXMAP",	(APP_ATOM)0,	tedPastePrimaryPixmap,
 					tedCopyPrimaryPixmap		},
@@ -1747,7 +1460,8 @@ static AppSelectionTargetType TedPrimaryTargets[]=
 
 AppSelectionTargetType TedPrimaryTextTargets[]=
     {
-	{ "RTF",	(APP_ATOM)0,	tedPastePrimaryRtf,
+	{ "application/rtf",
+			(APP_ATOM)0,	tedPastePrimaryRtf,
 					tedCopyPrimaryRtf		},
 	{ "STRING",	(APP_ATOM)0,	tedPastePrimaryString,
 					tedCopyPrimaryString		},
@@ -1758,9 +1472,10 @@ const int TedPrimaryTextTargetCount=
 
 AppSelectionTargetType TedPrimaryPictureTargets[]=
     {
-	{ "RTF",	(APP_ATOM)0,	tedPastePrimaryRtf,
+	{ "application/rtf",
+			(APP_ATOM)0,	tedPastePrimaryRtf,
 					tedCopyPrimaryRtf		},
-	{ "PNG",	(APP_ATOM)0,	tedPastePrimaryPng,
+	{ "image/png",	(APP_ATOM)0,	tedPastePrimaryPng,
 					tedCopyPrimaryPng		},
 	{ "PIXMAP",	(APP_ATOM)0,	tedPastePrimaryPixmap,
 					tedCopyPrimaryPixmap		},
@@ -1851,7 +1566,7 @@ static EditApplication	TedApplication=
 		    /*  Name of the picture for the application window.	*/
 		    /****************************************************/
     "Ted",
-    "Ted, Version 2.10, April 15, 2001",
+    "Ted, Version 2.14, April 6, 2003",
     "http://www.nllgg.nl/Ted",
 
 
@@ -1976,6 +1691,12 @@ static EditApplication	TedApplication=
     tedDocCopy,
     tedDocCut,
     tedDocSelAll,
+		    /****************************************************/
+		    /*							*/
+		    /*  Set page layout.				*/
+		    /*							*/
+		    /****************************************************/
+    tedSetPageLayout,
 		    /****************************************************/
 		    /*  User input on the document widget:		*/
 		    /****************************************************/
@@ -2016,33 +1737,27 @@ static EditApplication	TedApplication=
 /*									*/
 /*  Main() of the 'Ted' Application.					*/
 /*									*/
-/*  1)  It is not necessary to initialize WorldApplication to convert	*/
-/*	files. Intercept conversion calls to prevent the program from	*/
-/*	becoming interactive.						*/
-/*  3)  Do not die too easily.						*/
+/*  1)  Sanity checks							*/
 /*									*/
 /************************************************************************/
 
 int main(	int		argc,
 		char *		argv[]	)
     {
-    TEDResources.tarDefaultAnsicpgInt= -1;
+    EditApplication *	ea= &TedApplication;
 
     /*  1  */
-    if  ( argc == 4				&&
-	  ! strcmp( argv[1], "++saveTo" )	)
-	{
-	if  ( appFileConvert( &TedApplication, argv[2], argv[3] ) )
-	    { SSSDEB(argv[1],argv[2],argv[3]); return 1; }
+    if  ( RPprop_COUNT > PROPmaskMAXPROPS )
+	{ LLDEB(RPprop_COUNT,PROPmaskMAXPROPS); return 1;	}
+    if  ( PPprop_COUNT > PROPmaskMAXPROPS )
+	{ LLDEB(PPprop_COUNT,PROPmaskMAXPROPS); return 1;	}
+    if  ( CLprop_COUNT > PROPmaskMAXPROPS )
+	{ LLDEB(CLprop_COUNT,PROPmaskMAXPROPS); return 1;	}
 
-	return 0;
-	}
+    TEDResources.tarDefaultAnsicpgInt= -1;
+    TEDResources.tarShowTableGridInt= 0;
 
-    /*  3  */
-    (void) signal( SIGHUP, SIG_IGN );
-    (void) signal( SIGINT, SIG_IGN );
+    tedGetNamedPictures( ea );
 
-    tedGetNamedPictures( &TedApplication );
-
-    return appMain( &TedApplication, argc, argv );
+    return appMain( ea, argc, argv );
     }
