@@ -15,6 +15,14 @@
 
 typedef struct NotesPageResources
     {
+    char *		nprCurrentNoteText;
+    char *		nprFootnoteText;
+    char *		nprEndnoteText;
+    char *		nprToNoteRefText;
+    char *		nprToNoteText;
+    char *		nprRevertNoteText;
+    char *		nprChangeNoteText;
+
     char *		nprFootnotesText;
     char *		nprEndnotesText;
 
@@ -28,39 +36,52 @@ typedef struct NotesPageResources
     char *		nprNumberStyleMenuTexts[DPftn_NCOUNT];
     } NotesPageResources;
 
+typedef struct NotePropertiesTool
+    {
+    APP_WIDGET			nptFrame;
+    APP_WIDGET			nptPaned;
+    APP_WIDGET			nptStartNumberText;
+
+				/*  Not all positions used!	*/
+    AppOptionmenu		nptPositionOptionmenu;
+    APP_WIDGET			nptPositionOptions[DPftnPOS__COUNT];
+
+				/*  Not all positions used!	*/
+    AppOptionmenu		nptRestartOptionmenu;
+    APP_WIDGET			nptRestartOptions[DPftnRST__COUNT];
+
+    AppOptionmenu		nptStyleOptionmenu;
+    APP_WIDGET			nptStyleOptions[DPftn_NCOUNT];
+    } NotePropertiesTool;
+
 typedef struct NotesTool
     {
     EditApplication *		ntApplication;
     AppInspector *		ntInspector;
+    const NotesPageResources *	ntPageResources;
+
+    int				ntNoteKindChosen;
+    int				ntNoteKindSet;
+    int				ntInsideNote;
 
     DocumentProperties		ntPropertiesChosen;
     DocumentProperties		ntPropertiesSet;
 
-    APP_WIDGET			ntFootnotesFrame;
-    APP_WIDGET			ntFootnotesPaned;
-    APP_WIDGET			ntStartFootnoteNumberText;
+    /****/
+    APP_WIDGET			ntCurrentNoteFrame;
+    APP_WIDGET			ntCurrentNotePaned;
+    APP_WIDGET			ntFootnoteToggle;
+    APP_WIDGET			ntEndnoteToggle;
 
-    AppOptionmenu		ntFootnotePositionOptionmenu;
-    APP_WIDGET			ntFootnotePositionOptions[DPftnPOS__COUNT];
+    APP_WIDGET			ntToNoteRefButton;
+    APP_WIDGET			ntToNoteButton;
 
-    AppOptionmenu		ntFootnoteRestartOptionmenu;
-    APP_WIDGET			ntFootnoteRestartOptions[DPftnRST__COUNT];
+    APP_WIDGET			ntRevertNoteButton;
+    APP_WIDGET			ntChangeNoteButton;
 
-    AppOptionmenu		ntFootnoteStyleOptionmenu;
-    APP_WIDGET			ntFootnoteStyleOptions[DPftn_NCOUNT];
-
-    APP_WIDGET			ntEndnotesFrame;
-    APP_WIDGET			ntEndnotesPaned;
-    APP_WIDGET			ntStartEndnoteNumberText;
-
-    AppOptionmenu		ntEndnotePositionOptionmenu;
-    APP_WIDGET			ntEndnotePositionOptions[DPftnPOS__COUNT];
-
-    AppOptionmenu		ntEndnoteRestartOptionmenu;
-    APP_WIDGET			ntEndnoteRestartOptions[DPftnRST__COUNT];
-
-    AppOptionmenu		ntEndnoteStyleOptionmenu;
-    APP_WIDGET			ntEndnoteStyleOptions[DPftn_NCOUNT];
+    /****/
+    NotePropertiesTool		ntFootnotePropertiesTool;
+    NotePropertiesTool		ntEndnotePropertiesTool;
     } NotesTool;
 
 /************************************************************************/
@@ -73,4 +94,13 @@ extern void tedFormatToolGetNotesResourceTable( EditApplication *	ea,
 					NotesPageResources *		spr,
 					InspectorSubjectResources *	isr );
 
+extern void tedFormatToolRefreshNotesTool(
+				NotesTool *			nt,
+				int *				pEnabled,
+				InspectorSubject *		is,
+				const DocumentSelection *	ds,
+				BufferDocument *		bd );
+
+extern void tedFormatCleanNotesTool(	NotesTool *		nt );
+
 #   endif	/*  TED_NOTES_TOOL_H */
