@@ -1,10 +1,10 @@
 /************************************************************************/
 /*									*/
-/*  Convert a windows metafile to a Pixmap.				*/
+/*  Open/Close objects inserted in the document.			*/
 /*									*/
 /************************************************************************/
 
-#   include	"config.h"
+#   include	"tedConfig.h"
 
 #   include	<stddef.h>
 #   include	<stdio.h>
@@ -15,13 +15,14 @@
 #   include	<sioEndian.h>
 #   include	<sioStdio.h>
 #   include	<sioHex.h>
+#   include	<appSystem.h>
 
 #   include	<appDraw.h>
 #   include	<appImage.h>
 #   include	<appWinMeta.h>
 #   include	<appMacPict.h>
 
-#   include	"docPs.h"
+#   include	"docLayout.h"
 #   include	"tedApp.h"
 
 #   include	<appDebugon.h>
@@ -86,6 +87,7 @@ typedef int (*PLAY_META_X11)(	SimpleInputStream *	sis,
 				AppColors *		ac,
 				AppDrawingData *	add,
 				APP_WINDOW		pixmap,
+				int			mapMode,
 				int			xExt,
 				int			yExt,
 				int			pixelsWide,
@@ -97,11 +99,9 @@ static int tedOpenMetafileObject(	InsertedObject *	io,
 					PLAY_META_X11		playMetaX11,
 					AppColors *		ac,
 					AppDrawingData *	add,
+					int			mapMode,
 					const MemoryBuffer *	mb )
     {
-    int			xExt= io->ioXExtent;
-    int			yExt= io->ioYExtent;
-
     SimpleInputStream *	sisMem;
     SimpleInputStream *	sisMeta;
 
@@ -117,8 +117,9 @@ static int tedOpenMetafileObject(	InsertedObject *	io,
     if  ( ! io->ioPixmap )
 	{ XDEB(io->ioPixmap); return -1;	}
 
-    if  ( (*playMetaX11)( sisMeta, &io->ioPrivate, ac, add,
-					io->ioPixmap, xExt, yExt,
+    if  ( (*playMetaX11)( sisMeta, &(io->ioPrivate), ac, add,
+					io->ioPixmap, mapMode,
+					io->io_xWinExt, io->io_yWinExt,
 					io->ioPixelsWide, io->ioPixelsHigh,
 					io->ioTwipsWide, io->ioTwipsHigh ) )
 	{ LDEB(1);	}
@@ -215,21 +216,24 @@ static int tedOpenObject(	InsertedObject *	io,
 	case DOCokPICTWMETAFILE:
 	    if  ( ! io->ioPrivate					&&
 		  tedOpenMetafileObject( io, appMetaPlayWmfX11,
-					ac, add, &io->ioObjectData )	)
+						ac, add, io->ioMapMode,
+						&(io->ioObjectData) )	)
 		{ LDEB(1); return 0;	}
 	    return 0;
 
 	case DOCokPICTEMFBLIP:
 	    if  ( ! io->ioPrivate					&&
 		  tedOpenMetafileObject( io, appMetaPlayEmfX11,
-					ac, add, &io->ioObjectData )	)
+						ac, add, io->ioMapMode,
+						&(io->ioObjectData) )	)
 		{ LDEB(1); return 0;	}
 	    return 0;
 
 	case DOCokMACPICT:
 	    if  ( ! io->ioPrivate					&&
 		  tedOpenMetafileObject( io, appMacPictPlayFileX11,
-					ac, add, &io->ioObjectData )	)
+						ac, add, io->ioMapMode,
+						&(io->ioObjectData) )	)
 		{ LDEB(1); return 0;	}
 	    return 0;
 
@@ -237,7 +241,7 @@ static int tedOpenObject(	InsertedObject *	io,
 	    if  ( ! io->ioPrivate					&&
 		  tedOpenBitmapObject( bmPngReadPng, io,
 				    add->addMagnifiedPixelsPerTwip,
-					ac, add, &io->ioObjectData )	)
+					ac, add, &(io->ioObjectData) )	)
 		{ LDEB(1); return 0;	}
 	    return 0;
 
@@ -245,7 +249,7 @@ static int tedOpenObject(	InsertedObject *	io,
 	    if  ( ! io->ioPrivate					&&
 		  tedOpenBitmapObject( bmJpegReadJfif, io,
 				    add->addMagnifiedPixelsPerTwip,
-					ac, add, &io->ioObjectData )	)
+					ac, add, &(io->ioObjectData) )	)
 		{ LDEB(1); return 0;	}
 	    return 0;
 
@@ -254,7 +258,8 @@ static int tedOpenObject(	InsertedObject *	io,
 		{
 		if  ( ! io->ioPrivate					&&
 		      tedOpenMetafileObject( io, appMetaPlayWmfX11,
-					ac, add, &io->ioResultData )	)
+					ac, add, io->ioResultMapMode,
+					&(io->ioResultData) )	)
 		    { LDEB(1); return 0;	}
 
 		return 0;
@@ -263,7 +268,7 @@ static int tedOpenObject(	InsertedObject *	io,
 
 	case DOCokINCLUDEPICTURE:
 
-	    if  ( io->ioResultKind == DOCokEPSF )
+	    if  ( io->ioResultKind == DOCokEPS_FILE )
 		{
 		if  ( ! io->ioPixmap					&&
 		      tedOpenEpsfObject( io,
@@ -272,7 +277,22 @@ static int tedOpenObject(	InsertedObject *	io,
 
 		return 0;
 		}
-	    else{ LLDEB(io->ioKind,io->ioResultKind); return 0;	}
+
+	    if  ( io->ioResultKind == DOCokBITMAP_FILE	&&
+		  io->ioPrivate				)
+		{
+		APP_BITMAP_IMAGE	pixmap;
+		AppBitmapImage *	abi= (AppBitmapImage *)io->ioPrivate;
+
+		if  ( appImgMakePixmap( add, &pixmap,
+			    io->ioPixelsWide, io->ioPixelsHigh, ac, abi ) )
+		    { LLDEB(io->ioKind,io->ioResultKind); return 0;	}
+
+		io->ioPixmap= pixmap;
+		return 0;
+		}
+
+	    LLDEB(io->ioKind,io->ioResultKind); return 0;
 
 	default:
 	    LDEB(io->ioKind); return 0;
@@ -301,8 +321,7 @@ static int tedOpenParaObjects(	BufferItem *		bi,
 
 	io= bi->biParaObjects+ tp->tpObjectNumber;
 
-	if  ( ! io->ioPrivate			&&
-	      tedOpenObject( io, ac, add )	)
+	if  ( tedOpenObject( io, ac, add )	)
 	    { LDEB(part); continue;	}
 	}
 
@@ -324,37 +343,37 @@ int tedOpenItemObjects(	BufferItem *		bi,
 		    { LDEB(i); return -1;	}
 		}
 
-	    if  ( bi->biSectHeader.hfItem				&&
-		  tedOpenItemObjects( bi->biSectHeader.hfItem,
+	    if  ( bi->biSectHeader.eiItem				&&
+		  tedOpenItemObjects( bi->biSectHeader.eiItem,
 							ac, add )	)
 		{ LDEB(1); return -1;	}
-	    if  ( bi->biSectFirstPageHeader.hfItem			&&
-		  tedOpenItemObjects( bi->biSectFirstPageHeader.hfItem,
+	    if  ( bi->biSectFirstPageHeader.eiItem			&&
+		  tedOpenItemObjects( bi->biSectFirstPageHeader.eiItem,
 							ac, add )	)
 		{ LDEB(1); return -1;	}
-	    if  ( bi->biSectLeftPageHeader.hfItem			&&
-		  tedOpenItemObjects( bi->biSectLeftPageHeader.hfItem,
+	    if  ( bi->biSectLeftPageHeader.eiItem			&&
+		  tedOpenItemObjects( bi->biSectLeftPageHeader.eiItem,
 							ac, add )	)
 		{ LDEB(1); return -1;	}
-	    if  ( bi->biSectRightPageHeader.hfItem			&&
-		  tedOpenItemObjects( bi->biSectRightPageHeader.hfItem,
+	    if  ( bi->biSectRightPageHeader.eiItem			&&
+		  tedOpenItemObjects( bi->biSectRightPageHeader.eiItem,
 							ac, add )	)
 		{ LDEB(1); return -1;	}
 
-	    if  ( bi->biSectFooter.hfItem				&&
-		  tedOpenItemObjects( bi->biSectFooter.hfItem,
+	    if  ( bi->biSectFooter.eiItem				&&
+		  tedOpenItemObjects( bi->biSectFooter.eiItem,
 							ac, add )	)
 		{ LDEB(1); return -1;	}
-	    if  ( bi->biSectFirstPageFooter.hfItem			&&
-		  tedOpenItemObjects( bi->biSectFirstPageFooter.hfItem,
+	    if  ( bi->biSectFirstPageFooter.eiItem			&&
+		  tedOpenItemObjects( bi->biSectFirstPageFooter.eiItem,
 							ac, add )	)
 		{ LDEB(1); return -1;	}
-	    if  ( bi->biSectLeftPageFooter.hfItem			&&
-		  tedOpenItemObjects( bi->biSectLeftPageFooter.hfItem,
+	    if  ( bi->biSectLeftPageFooter.eiItem			&&
+		  tedOpenItemObjects( bi->biSectLeftPageFooter.eiItem,
 							ac, add )	)
 		{ LDEB(1); return -1;	}
-	    if  ( bi->biSectRightPageFooter.hfItem			&&
-		  tedOpenItemObjects( bi->biSectRightPageFooter.hfItem,
+	    if  ( bi->biSectRightPageFooter.eiItem			&&
+		  tedOpenItemObjects( bi->biSectRightPageFooter.eiItem,
 							ac, add )	)
 		{ LDEB(1); return -1;	}
 
@@ -391,7 +410,7 @@ void tedCloseObject(		BufferDocument *	bd,
     InsertedObject *	io;
 
     if  ( tp->tpKind != DOCkindOBJECT	||
-	  tp->tpPhysicalFont < 0		)
+	  tp->tpPhysicalFont < 0	)
 	{ return;	}
 
     io= bi->biParaObjects+ tp->tpObjectNumber;
@@ -415,7 +434,8 @@ void tedCloseObject(		BufferDocument *	bd,
 		    io->ioPixmap= (APP_WINDOW)0;
 		    }
 
-		appCleanBitmapImage( abi ); free( abi );
+		if  ( abi )
+		    { appCleanBitmapImage( abi ); free( abi );	}
 
 		io->ioPrivate= (void *)0;
 		}
@@ -429,6 +449,17 @@ void tedCloseObject(		BufferDocument *	bd,
 		appDrawFreePixmap( add, io->ioPixmap );
 		io->ioPixmap= (APP_WINDOW)0;
 		}
+
+	    if  ( io->ioResultKind == DOCokBITMAP_FILE )
+		{
+		AppBitmapImage *	abi= (AppBitmapImage *)io->ioPrivate;
+
+		if  ( abi )
+		    { appCleanBitmapImage( abi ); free( abi );	}
+
+		io->ioPrivate= (void *)0;
+		}
+
 	    break;
 
 	default:
@@ -448,7 +479,6 @@ int tedReopenObject(	BufferDocument *	bd,
 			BufferItem *		bi,
 			TextParticule *		tp,
 			AppColors *		ac,
-			DocumentFontList *	dfl,
 			AppDrawingData *	add )
     {
     InsertedObject *		io= bi->biParaObjects+ tp->tpObjectNumber;
@@ -466,37 +496,26 @@ static void tedScaleObjectToParagraph(	EditDocument *		ed,
     {
     TedDocument *		td= (TedDocument *)ed->edPrivateData;
     BufferDocument *		bd= td->tdDocument;
-    const DocumentProperties *	dp= &(bd->bdProperties);
-    FormattingFrame		ff;
-    double			scaleX;
-    double			scaleY;
+    ParagraphFrame		pf;
 
-    SelectionScope		ss;
-    BufferSelection		bs;
+    DocumentSelection		ds;
+    SelectionGeometry		sg;
 
     const int			bottom= -1;
     const int			stripHigh= -1;
 
-    if  ( tedGetSelection( &ss, &bs, td ) )
+    BlockFrame			bf;
+
+    if  ( tedGetSelection( &ds, &sg, td ) )
 	{ LDEB(1); return;	}
 
-    docParagraphFrameTwips( &ff, bottom, stripHigh, bs.bsBegin.bpBi, dp,
-				    bs.bsBegin.bpBi->biTopPosition.lpPage,
-				    bs.bsBegin.bpBi->biTopPosition.lpColumn );
+    docBlockFrameTwips( &bf, ds.dsBegin.dpBi, bd,
+				    ds.dsBegin.dpBi->biTopPosition.lpPage,
+				    ds.dsBegin.dpBi->biTopPosition.lpColumn );
 
-    scaleX= (double)( ff.ffX1TextLinesTwips- ff.ffX0TextLinesTwips )/
-						    (double)io->ioTwipsWide;
+    docParagraphFrameTwips( &pf, &bf, bottom, stripHigh, ds.dsBegin.dpBi );
 
-    scaleY= (double)ff.ffPageHigh/ (double)io->ioTwipsHigh;
-
-    if  ( scaleY > scaleX )
-	{ scaleY=  scaleX;	}
-
-    if  ( scaleY < 0.99 )
-	{
-	io->ioScaleX= (int)( 99* scaleY );
-	io->ioScaleY= (int)( 99* scaleY );
-	}
+    docLayoutScaleObjectToFitParagraphFrame( io, &pf );
 
     return;
     }
@@ -526,7 +545,7 @@ int tedSaveObjectPicture(	AppBitmapImage *	abiTo,
 	{ abiFrom= (AppBitmapImage *)io->ioPrivate; }
 
     if  ( ! abiFrom )
-	{ LXDEB(io->ioKind,abiFrom); return -1;	}
+	{ LXXDEB(io->ioKind,io->ioPrivate,abiFrom); return -1;	}
 
     buffer= (unsigned char *)malloc( abiFrom->abiBitmap.bdBufferLength );
     if  ( ! buffer )
@@ -549,10 +568,11 @@ int tedSaveObjectPicture(	AppBitmapImage *	abiTo,
 /*									*/
 /************************************************************************/
 
-int tedObjectInsertBitmap(	EditDocument *		ed,
-				AppBitmapImage *	abi )
+InsertedObject * tedObjectMakeBitmapObject(
+					EditDocument *		ed,
+					BufferItem *		bi,
+					AppBitmapImage *	abi )
     {
-    TedDocument *		td= (TedDocument *)ed->edPrivateData;
     AppDrawingData *		add= &(ed->edDrawingData);
 
     APP_WINDOW			pixmap;
@@ -567,41 +587,39 @@ int tedObjectInsertBitmap(	EditDocument *		ed,
 
     double			xfac= add->addMagnifiedPixelsPerTwip;
 
-    SelectionScope		ss;
-    BufferSelection		bs;
-
-    if  ( tedGetSelection( &ss, &bs, td ) )
-	{ LDEB(1); return -1;	}
-
-    io= docClaimObject( &objectNumber, bs.bsBegin.bpBi );
+    io= docClaimObject( &objectNumber, bi );
     if  ( ! io )
-	{ XDEB(io); return -1;	}
+	{ XDEB(io); return (InsertedObject *)0;	}
 
     utilInitMemoryBuffer( &mb );
 
     sosMem= sioOutMemoryOpen( &mb );
     if  ( ! sosMem )
-	{ XDEB(sosMem); return -1;	}
+	{ XDEB(sosMem); return (InsertedObject *)0;	}
 
     sosHex= sioOutHexOpen( sosMem );
     if  ( ! sosHex )
-	{ XDEB(sosHex); return -1;	}
+	{ XDEB(sosHex); return (InsertedObject *)0;	}
 
     if  ( abi->abiFormat >= 0						&&
 	  ! strcmp( bmFileFormats[abi->abiFormat].bffFileType->bftTypeId,
 								"pngFile" ) )
 	{
 	if  ( bmPngWritePng( &abi->abiBitmap, abi->abiBuffer, sosHex ) )
-	    { LDEB(1); return -1;	}
+	    { LDEB(1); return (InsertedObject *)0;	}
 
 	io->ioKind= DOCokPICTPNGBLIP;
 	}
     else{
-	if  ( appMetaSaveBitmapMetafile( &abi->abiBitmap, abi->abiBuffer,
+	if  ( appMetaSaveBitmapMetafile( &(abi->abiBitmap), abi->abiBuffer,
 								    sosHex ) )
-	    { LDEB(1); return -1;	}
+	    { LDEB(1); return (InsertedObject *)0;	}
+
+	io->ioMetafileIsBitmap= 1;
+	io->ioMetafileBitmapBpp= abi->abiBitmap.bdBitsPerPixel;
 
 	io->ioKind= DOCokPICTWMETAFILE;
+	io->ioMapMode= 8;
 	}
 
     sioOutClose( sosHex );
@@ -609,34 +627,8 @@ int tedObjectInsertBitmap(	EditDocument *		ed,
 
     io->ioObjectData= mb;
 
-    switch( abi->abiBitmap.bdUnit )
-	{
-	case BMunCM:
-	    io->ioTwipsWide= (int)
-			( ( TWIPS_PER_CM* abi->abiBitmap.bdPixelsWide )/
-					    abi->abiBitmap.bdXResolution );
-	    io->ioTwipsHigh= (int)
-			( ( TWIPS_PER_CM* abi->abiBitmap.bdPixelsHigh )/
-					    abi->abiBitmap.bdYResolution );
-	    break;
-	case BMunINCH:
-	    io->ioTwipsWide= (int)( ( 72* 20* abi->abiBitmap.bdPixelsWide )/
-					    abi->abiBitmap.bdXResolution );
-	    io->ioTwipsHigh= (int)( ( 72* 20* abi->abiBitmap.bdPixelsHigh )/
-					    abi->abiBitmap.bdYResolution );
-	    break;
-	case BMunPOINT:
-	    io->ioTwipsWide= (int)( ( 20* abi->abiBitmap.bdPixelsWide )/
-					    abi->abiBitmap.bdXResolution );
-	    io->ioTwipsHigh= (int)( ( 20* abi->abiBitmap.bdPixelsHigh )/
-					    abi->abiBitmap.bdYResolution );
-	    break;
-	default:
-	    LDEB(abi->abiBitmap.bdUnit);
-	    io->ioTwipsWide= 20* abi->abiBitmap.bdPixelsWide;
-	    io->ioTwipsHigh= 20* abi->abiBitmap.bdPixelsHigh;
-	    break;
-	}
+    bmImageSizeTwips( &(io->ioTwipsWide), &(io->ioTwipsHigh),
+							&(abi->abiBitmap) );
 
     tedScaleObjectToParagraph( ed, io );
 
@@ -646,27 +638,22 @@ int tedObjectInsertBitmap(	EditDocument *		ed,
 				( io->ioScaleY* io->ioTwipsHigh )/ 100 );
     io->ioPrivate= abi;
 
-    io->ioXExtent= (int) ( ( 1000.0* io->ioTwipsWide )/ ( 20* POINTS_PER_CM ) );
-    io->ioYExtent= (int) ( ( 1000.0* io->ioTwipsHigh )/ ( 20* POINTS_PER_CM ) );
+    io->io_xWinExt= (int) ( ( 100000.0* io->ioTwipsWide )/
+						    ( 20* POINTS_PER_M ) );
+    io->io_yWinExt= (int) ( ( 100000.0* io->ioTwipsHigh )/
+						    ( 20* POINTS_PER_M ) );
 
     if  ( appImgMakePixmap( add, &pixmap,
 				    io->ioPixelsWide, io->ioPixelsHigh,
 				    &ed->edColors, abi ) )
-	{ LDEB(1); return -1; }
+	{ LDEB(1); return (InsertedObject *)0; }
     io->ioPixmap= pixmap;
 
-    tedAppReplaceSelection( ed, (unsigned char *)"", 0 );
-
-    if  ( tedInsertNewObject( ed->edDocumentWidget, io, ed ) )
-	{ LDEB(1); return -1;	}
-
     /*  9
     io->ioBliptag= appGetTimestamp();
     */
 
-    appDocumentChanged( ed, 1 );
-
-    return 0;
+    return io;
     }
 
 /************************************************************************/
@@ -675,10 +662,11 @@ int tedObjectInsertBitmap(	EditDocument *		ed,
 /*									*/
 /************************************************************************/
 
-static int tedObjectInsertMetafile(	EditDocument *		ed,
+static InsertedObject * tedMakeMetafileObject(
+					EditDocument *		ed,
+					BufferItem *		bi,
 					const char *		filename )
     {
-    TedDocument *		td= (TedDocument *)ed->edPrivateData;
     AppDrawingData *		add= &(ed->edDrawingData);
 
     InsertedObject *		io;
@@ -692,29 +680,23 @@ static int tedObjectInsertMetafile(	EditDocument *		ed,
 
     double			xfac= add->addMagnifiedPixelsPerTwip;
 
-    SelectionScope		ss;
-    BufferSelection		bs;
-
-    if  ( tedGetSelection( &ss, &bs, td ) )
-	{ LDEB(1); return -1;	}
-
     sisIn= sioInStdioOpen( filename );
     if  ( ! sisIn )
-	{ SXDEB(filename,sisIn); return -1;	}
+	{ SXDEB(filename,sisIn); return (InsertedObject *)0;	}
 
-    io= docClaimObject( &objectNumber, bs.bsBegin.bpBi );
+    io= docClaimObject( &objectNumber, bi );
     if  ( ! io )
-	{ XDEB(io); return -1;	}
+	{ XDEB(io); return (InsertedObject *)0;	}
 
     utilInitMemoryBuffer( &mb );
 
     sosMem= sioOutMemoryOpen( &mb );
     if  ( ! sosMem )
-	{ XDEB(sosMem); return -1;	}
+	{ XDEB(sosMem); return (InsertedObject *)0;	}
 
     sosMeta= sioOutHexOpen( sosMem );
     if  ( ! sosMeta )
-	{ XDEB(sosMeta); return -1;	}
+	{ XDEB(sosMeta); return (InsertedObject *)0;	}
 
     {
     unsigned long		key;
@@ -750,16 +732,16 @@ static int tedObjectInsertMetafile(	EditDocument *		ed,
 	    { io->ioTwipsHigh= ( 20* 72* ( bottom- top ) )/ inch;	}
 	else{ io->ioTwipsHigh= ( 20* 72* ( top- bottom ) )/ inch;	}
 
-	io->ioXExtent= right- left;
-	io->ioYExtent= top- bottom;
+	io->io_xWinExt= right- left;
+	io->io_yWinExt= top- bottom;
 	}
     else{
 	if  ( 0 && key == 0x90001 )
 	    {
 	    io->ioTwipsWide= 20* 72* 4;		/*  10 cm	*/
 	    io->ioTwipsHigh= 20* 72* 4;		/*  10 cm	*/
-	    io->ioXExtent= 72* 4;
-	    io->ioYExtent= 72* 4;
+	    io->io_xWinExt= 72* 4;
+	    io->io_yWinExt= 72* 4;
 	    sioEndianPutLeUint32( key, sosMeta );
 	    }
 	else{
@@ -767,7 +749,7 @@ static int tedObjectInsertMetafile(	EditDocument *		ed,
 	    sioOutClose( sosMeta );
 	    sioOutClose( sosMem );
 	    sioInClose( sisIn );
-	    return -1;
+	    return (InsertedObject *)0;
 	    }
 	}
     }
@@ -782,7 +764,7 @@ static int tedObjectInsertMetafile(	EditDocument *		ed,
 	    { break;	}
 
 	if  ( sioOutWriteBytes( sosMeta, buf, done ) != done )
-	    { LDEB(done); return -1;	}
+	    { LDEB(done); return (InsertedObject *)0;	}
 	}
 
     sioOutClose( sosMeta );
@@ -791,6 +773,7 @@ static int tedObjectInsertMetafile(	EditDocument *		ed,
 
     io->ioObjectData= mb;
     io->ioKind= DOCokPICTWMETAFILE;
+    io->ioMapMode= 8;
 
     tedScaleObjectToParagraph( ed, io );
 
@@ -799,24 +782,19 @@ static int tedObjectInsertMetafile(	EditDocument *		ed,
     io->ioPixelsHigh= TWIPStoPIXELS( xfac,
 				( io->ioScaleY* io->ioTwipsHigh )/ 100 );
 
-    io->ioXExtent= (int) ( ( 1000.0* io->ioTwipsWide )/ ( 20* POINTS_PER_CM ) );
-    io->ioYExtent= (int) ( ( 1000.0* io->ioTwipsHigh )/ ( 20* POINTS_PER_CM ) );
-
-    tedAppReplaceSelection( ed, (unsigned char *)"", 0 );
-
-    if  ( tedInsertNewObject( ed->edDocumentWidget, io, ed ) )
-	{ LDEB(1); return -1;	}
+    io->io_xWinExt= (int) ( ( 100000.0* io->ioTwipsWide )/
+						    ( 20* POINTS_PER_M ) );
+    io->io_yWinExt= (int) ( ( 100000.0* io->ioTwipsHigh )/
+						    ( 20* POINTS_PER_M ) );
 
     /*  9
     io->ioBliptag= appGetTimestamp();
     */
 
-    appDocumentChanged( ed, 1 );
-
     if  ( tedOpenObject( io, &ed->edColors, &(ed->edDrawingData) ) )
 	{ LDEB(1);	}
 
-    return 0;
+    return io;
     }
 
 static int tedObjectOpenPicture(	void *		voided,
@@ -825,14 +803,18 @@ static int tedObjectOpenPicture(	void *		voided,
 					const char *	filename )
     {
     EditDocument *		ed= (EditDocument *)voided;
-    const char *		slash= strrchr( filename, '/' );
-    const char *		dot;
+    TedDocument *		td= (TedDocument *)ed->edPrivateData;
+    const char *		ext;
 
-    if  ( slash )
-	{ dot= strrchr( slash+ 1, '.' );	}
-    else{ dot= strrchr( filename, '.' );	}
+    DocumentSelection		ds;
+    SelectionGeometry		sg;
 
-    if  ( ! dot || strcmp( dot, ".wmf" ) )
+    if  ( tedGetSelection( &ds, &sg, td ) )
+	{ LDEB(1); return -1;	}
+
+    ext= appFileExtensionOfName( filename );
+
+    if  ( ! ext || strcmp( ext, "wmf" ) )
 	{
 	double			factor;
 	AppBitmapImage *	abi;
@@ -846,14 +828,20 @@ static int tedObjectOpenPicture(	void *		voided,
 						&abi->abiFormat, &factor ) )
 	    { SDEB(filename); return -1;	}
 
-	if  ( tedObjectInsertBitmap( ed, abi ) )
+	if  ( tedReplaceSelectionWithBitmapImage( ed, abi ) )
 	    { SDEB(filename); return -1; }
 
 	return 0;
 	}
     else{
-	if  ( tedObjectInsertMetafile( ed, filename ) )
-	    { SDEB(filename); return -1; }
+	InsertedObject *	io;
+
+	io= tedMakeMetafileObject( ed, ds.dsBegin.dpBi, filename );
+	if  ( ! io )
+	    { SXDEB(filename,io); return -1; }
+
+	if  ( tedReplaceSelectionWithObject( ed, io ) )
+	    { SDEB(filename); return -1;	}
 
 	return 0;
 	}
