@@ -2,11 +2,11 @@
 /*  Ted: interaction with the page layout tool.				*/
 /************************************************************************/
 
-#   include	"config.h"
+#   include	"tedConfig.h"
 
 #   include	"tedApp.h"
 #   include	"tedRuler.h"
-#   include	"docPs.h"
+#   include	"docLayout.h"
 
 #   include	<appDebugon.h>
 
@@ -39,12 +39,18 @@ void tedRedoDocumentLayout(	EditDocument *		ed )
 
     if  ( hasSelection )
 	{
-	tedDocAdaptHorizontalRuler( ed, td->tdSelection.bsBegin.bpBi );
+	DocumentSelection	ds;
+	SelectionGeometry	sg;
+
+	if  ( tedGetSelection( &ds, &sg, td ) )
+	    { LDEB(1); return;	}
+
+	tedDocAdaptHorizontalRuler( ed, ds.dsBegin.dpBi );
 
 	tedScrollToSelection( ed, &scrolledX, &scrolledY );
 
 	if  ( td->tdObjectSelected )
-	    { tedMoveObjectWindows( ed->edDocumentWidget, ed );	}
+	    { tedMoveObjectWindows( ed );	}
 	}
 
     appExposeRectangle( add, 0, 0, 0, 0 );
@@ -63,55 +69,88 @@ void tedRedoDocumentLayout(	EditDocument *		ed )
 /*									*/
 /************************************************************************/
 
-void tedSetPageLayout(	EditDocument *			ed,
-			unsigned int			spUpdMask,
-			const DocumentGeometry *	dg )
+void tedSetPageLayout(	EditApplication *		ea,
+			const DocumentGeometry *	dg,
+			const PropertyMask *		setMask,
+			int				wholeDocument )
     {
     AppDrawingData *		add;
     TedDocument *		td;
     BufferDocument *		bd;
     DocumentProperties *	dp;
 
-    unsigned int		changed= 0;
+    PropertyMask		changed;
+    PropertyMask		dgUpdMask;
 
     int				i;
 
+    EditDocument *		ed= ea->eaCurrentDocument;
+
+    if  ( ! ed )
+	{ XDEB(ed); return;	}
+
+    PROPmaskCLEAR( &changed );
+    PROPmaskCLEAR( &dgUpdMask );
+
     add= &(ed->edDrawingData);
     td= (TedDocument *)ed->edPrivateData;
     bd= td->tdDocument;
     dp= &(bd->bdProperties);
 
-    appSetDocumentGeometry( &(dp->dpGeometry), dg, &changed,
-						    spUpdMask & DGupdALL );
+    PROPmaskCLEAR( &dgUpdMask );
+    PROPmaskFILL( &dgUpdMask, DGprop_COUNT );
+    utilPropMaskAnd( &dgUpdMask, &dgUpdMask, setMask );
 
-    for ( i= 0; i < bd->bdItem.biChildCount; i++ )
+    if  ( wholeDocument )
 	{
-	unsigned int		sectChanged= 0;
-	BufferItem *		sectBi=  bd->bdItem.biChildren[i];
+	appSetDocumentGeometry( &(dp->dpGeometry), dg, &changed, &dgUpdMask );
 
-	appSetDocumentGeometry( &(sectBi->biSectDocumentGeometry),
-						dg, &sectChanged, spUpdMask );
+	for ( i= 0; i < bd->bdItem.biChildCount; i++ )
+	    {
+	    PropertyMask		sectChanged;
+	    BufferItem *		sectBi= bd->bdItem.biChildren[i];
 
-	changed |= sectChanged;
+	    PROPmaskCLEAR( &sectChanged );
+
+	    appSetDocumentGeometry( &(sectBi->biSectDocumentGeometry),
+						    dg, &sectChanged, setMask );
+
+	    utilPropMaskOr( &changed, &changed, &sectChanged );
+	    }
+
+	if  ( ! PROPmaskISEMPTY( &changed ) )
+	    {
+	    appDocumentChanged( ed, 1 );
+
+	    tedScreenRectangles( add, &(dp->dpGeometry) );
+
+	    tedRedoDocumentLayout( ed );
+
+	    tedAdaptFormatToolToDocument( ed );
+	    }
 	}
+    else{
+	SectionProperties		spNew;
+	PropertyMask			doneMask;
 
-    if  ( changed )
-	{
-	appDocumentChanged( ed, 1 );
+	docInitSectionProperties( &spNew );
+	PROPmaskCLEAR( &doneMask );
 
-	tedScreenRectangles( add, &(dp->dpGeometry) );
+	appSetDocumentGeometry( &(spNew.spDocumentGeometry), dg,
+							&doneMask, setMask );
 
-	tedRedoDocumentLayout( ed );
+	if  ( tedAppChangeSectionProperties( ea, setMask, &spNew ) )
+	    { LDEB(1);	}
 
-	tedAdaptFormatToolToDocument( ed );
+	docCleanSectionProperties( &spNew );
 	}
 
     return;
     }
 
-void tedSetDocumentProperties(	EditApplication *		ea,
+int tedSetDocumentProperties(	EditApplication *		ea,
 				const DocumentProperties *	dpNew,
-				unsigned int			updMask )
+				const PropertyMask *		updMask )
     {
     EditDocument *		ed= ea->eaCurrentDocument;
     AppDrawingData *		add;
@@ -119,23 +158,62 @@ void tedSetDocumentProperties(	EditApplication *		ea,
     BufferDocument *		bd;
     DocumentProperties *	dp;
 
-    unsigned int		changed;
+    PropertyMask		changed;
+
+    RecalculateFields		rf;
 
     if  ( ! ed )
-	{ XDEB(ed); return;	}
+	{ XDEB(ed); return -1;	}
 
     PROPmaskCLEAR( &changed );
 
+    docInitRecalculateFields( &rf );
+
     add= &(ed->edDrawingData);
     td= (TedDocument *)ed->edPrivateData;
     bd= td->tdDocument;
     dp= &(bd->bdProperties);
 
-    if  ( docUpdDocumentProperties( &changed, dp, updMask, dpNew ) )
-	{ LDEB(1); return;	}
+    rf.rfBd= bd;
+    rf.rfVoidadd= (void *)add;
+    rf.rfCloseObject= tedCloseObject;
+    rf.rfUpdateFlags= 0;
+    rf.rfFieldsUpdated= 0;
 
-    if  ( changed )
+    if  ( docUpdDocumentProperties( &changed, dp, updMask, dpNew ) )
+	{ LDEB(1); return -1;	}
+
+    if  ( ! PROPmaskISEMPTY( &changed ) )
 	{
+	int		noteNumbersChanged= 0;
+
+	if  ( PROPmaskISSET( &changed, DPpropFOOTNOTE_RESTART )	||
+	      PROPmaskISSET( &changed, DPpropENDNOTE_RESTART )	)
+	    { docRenumberNotes( &noteNumbersChanged, bd );	}
+
+	if  ( noteNumbersChanged				||
+	      PROPmaskISSET( &changed, DPpropFOOTNOTE_STYLE )	||
+	      PROPmaskISSET( &changed, DPpropFOOTNOTE_STARTNR )	||
+	      PROPmaskISSET( &changed, DPpropENDNOTE_STYLE )	||
+	      PROPmaskISSET( &changed, DPpropENDNOTE_STARTNR )	)
+	    { rf.rfUpdateFlags |= FIELDdoCHFTN;	}
+
+	if  ( PROPmaskISSET( &changed, DPpropTITLE )	||
+	      PROPmaskISSET( &changed, DPpropSUBJECT )	||
+	      PROPmaskISSET( &changed, DPpropKEYWORDS )	||
+	      PROPmaskISSET( &changed, DPpropDOCCOMM )	||
+	      PROPmaskISSET( &changed, DPpropAUTHOR )	||
+	      PROPmaskISSET( &changed, DPpropCREATIM )	||
+	      PROPmaskISSET( &changed, DPpropREVTIM )	||
+	      PROPmaskISSET( &changed, DPpropPRINTIM )	)
+	    { rf.rfUpdateFlags |= FIELDdoDOC_INFO;	}
+
+	if  ( rf.rfUpdateFlags != 0 )
+	    {
+	    if  ( docRecalculateTextLevelFields( &rf, &(bd->bdItem) ) )
+		{ LDEB(1); return -1;	}
+	    }
+
 	appDocumentChanged( ed, 1 );
 
 	tedScreenRectangles( add, &(dp->dpGeometry) );
@@ -145,7 +223,7 @@ void tedSetDocumentProperties(	EditApplication *		ea,
 	tedAdaptFormatToolToDocument( ed );
 	}
 
-    return;
+    return 0;
     }
 
 /************************************************************************/
@@ -153,6 +231,7 @@ void tedSetDocumentProperties(	EditApplication *		ea,
 /*  Adapt the page tool to a document.					*/
 /*									*/
 /************************************************************************/
+
 void tedAdaptPageToolToDocument(	EditApplication *	ea,
 					EditDocument *		ed )
     {
@@ -246,15 +325,30 @@ int tedLayoutDocumentTree(	TedDocument *		td,
     /*  1  */
     if  ( dp->dpFontList.dflCount == 0 )
 	{
+	DocumentFont	df;
+	const int	fontNumber= 0;
+	const char *	fontAltName= (const char *)0;
+
 	LDEB(dp->dpFontList.dflCount);
 
-	if  ( ! docInsertFont( &(dp->dpFontList), 0, "fswiss", "Helvetica" ) )
+	docInitDocumentFont( &df );
+	df.dfCharset= FONTcharsetANSI;
+
+	if  ( docFontSetFamilyStyle( &df, DFstyleFSWISS )	||
+	      docFontSetFamilyName( &df, "Helvetica" )		||
+	      docFontSetAltName( &df, fontAltName )		)
+	    { LDEB(1); goto ready;	}
+
+	if  ( ! docInsertFont( &(dp->dpFontList), fontNumber, &df ) )
 	    { LDEB(dp->dpFontList.dflCount);	}
+
+      ready:
+	docCleanDocumentFont( &df );
 	}
 
     drChanged= add->addBackRect;
 
-    if  ( tedLayoutItem( &(bd->bdItem), bd, DOCinBODY, add, &drChanged ) )
+    if  ( tedLayoutItem( &(bd->bdItem), bd, add, &drChanged ) )
 	{ LDEB(1); return -1;	}
 
     return 0;
