@@ -4,28 +4,27 @@
 /*									*/
 /************************************************************************/
 
-#   include	"config.h"
+#   include	"tedConfig.h"
 
 #   include	<stdlib.h>
 #   include	<stdio.h>
 #   include	<stddef.h>
 #   include	<limits.h>
 
-#   include	<appDebugon.h>
-
 #   include	<appGeoString.h>
 #   include	<appUnit.h>
 
 #   include	"tedApp.h"
 #   include	"tedFormatTool.h"
 
+#   include	<appDebugon.h>
+
 /************************************************************************/
 /*									*/
 /*  Refresh the row page.						*/
 /*									*/
 /************************************************************************/
 
-
 static void tedFormatToolRefreshRowPage( RowTool *	rt )
     {
     const RowProperties *	rp= &(rt->rtPropertiesChosen);
@@ -33,6 +32,20 @@ static void tedFormatToolRefreshRowPage( RowTool *	rt )
     const TableRectangle *	tr= &(rt->rtTableRectangle);
     char			scratch[50];
 
+    EditApplication *		ea= rt->rtApplication;
+    EditDocument *		ed= ea->eaCurrentDocument;
+
+    TedDocument *		td;
+    BufferDocument *		bd;
+    DocumentProperties *	dp;
+
+    if  ( ! ed )
+	{ XDEB(ed); return;	}
+
+    td= (TedDocument *)ed->edPrivateData;
+    bd= td->tdDocument;
+    dp= &(bd->bdProperties);
+
     appIntegerToTextWidget( rt->rtNumberText, tr->trRow0- tr->trRow00+ 1 );
 
     tedFormatRefreshHeightChooser( &(rt->rtHeightChooser),
@@ -45,29 +58,39 @@ static void tedFormatToolRefreshRowPage( RowTool *	rt )
     appStringToTextWidget( rt->rtLeftIndentText, scratch );
 
     cp= rp->rpCells;
-    tedFormatReflectBorder( &(rt->rtTopBorderTool),
+    tedBorderToolSetProperties( &(rt->rtTopBorderTool), dp,
 					&(cp[tr->trCol0].cpTopBorder) );
-    tedFormatReflectBorder( &(rt->rtBottomBorderTool),
+    tedBorderToolSetProperties( &(rt->rtBottomBorderTool), dp,
 					&(cp[tr->trCol0].cpBottomBorder) );
 
     appGuiSetToggleState( rt->rtKeepOnPageToggle,
-						rp->rpKeepOnPage );
+						rp->rpKeepOnOnePage );
     appGuiSetToggleState( rt->rtIsTableHeaderToggle,
 						rp->rpIsTableHeader );
+    /*
+    appGuiSetToggleState( rt->rtAutofitToggle, rp->rpAutofit );
+    */
     }
 
-void tedFormatToolRefreshRowTool(	RowTool *		rt,
-					InspectorSubject *	is,
-					const TableRectangle *	tr,
-					const BufferSelection *	bs )
+void tedFormatToolRefreshRowTool(
+				RowTool *			rt,
+				int *				pEnabled,
+				InspectorSubject *		is,
+				const DocumentSelection *	ds )
     {
+    TableRectangle *		tr= &(rt->rtTableRectangle);
+
     const RowProperties *	rp;
     const DocumentGeometry *	dg;
 
     const BufferItem *		rowBi;
     const BufferItem *		sectBi;
 
-    rowBi= bs->bsBegin.bpBi;
+    if  ( docGetTableRectangle( tr, ds )	||
+	  tr->trRow1 != tr->trRow0		)
+	{ docInitTableRectangle( tr ); *pEnabled= 0; return; }
+
+    rowBi= ds->dsBegin.dpBi;
     rowBi= rowBi->biParent;
     rowBi= rowBi->biParent;
     sectBi= rowBi->biParent;
@@ -80,17 +103,26 @@ void tedFormatToolRefreshRowTool(	RowTool *		rt,
 
     rp= &(rowBi->biRowProperties);
 
-    if  ( docCopyRowProperties( &(rt->rtPropertiesChosen), rp ) )
+    if  ( docCopyRowProperties( &(rt->rtPropertiesChosen),
+						    rp, (const int *)0 ) )
 	{ LDEB(1); return;	}
-    if  ( docCopyRowProperties( &(rt->rtPropertiesSet), rp ) )
+    if  ( docCopyRowProperties( &(rt->rtPropertiesSet),
+						    rp, (const int *)0 ) )
 	{ LDEB(1); return;	}
 
-    rt->rtTableRectangle= *tr;
-    appGuiEnableWidget( is->isPrevButton, tr->trRow0 > tr->trRow00 );
-    appGuiEnableWidget( is->isNextButton, tr->trRow1 < tr->trRow11 );
+    appGuiEnableWidget( is->isPrevButton,
+				    tr->trRow0 > tr->trRow00 );
+    appGuiEnableWidget( is->isNextButton,
+				    tr->trRow1 < tr->trRow11 );
+
+    appGuiEnableWidget( rt->rtIsTableHeaderToggle,
+				rowBi->biInExternalItem == DOCinBODY	&&
+				tr->trRow0 == tr->trRow00		&&
+				tr->trRow1 == tr->trRow00		);
 
     tedFormatToolRefreshRowPage( rt );
 
+    *pEnabled= 1;
     return;
     }
 
@@ -100,66 +132,111 @@ void tedFormatToolRefreshRowTool(	RowTool *		rt,
 /*									*/
 /************************************************************************/
 
-static void tedFormatChangeRowPushed(	APP_WIDGET	w,
-					void *		voidrt,
-					void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( tedFormatChangeRowPushed, w, voidrt )
     {
     RowTool *			rt= (RowTool *)voidrt;
     RowProperties *		rp= &(rt->rtPropertiesChosen);
-    CellProperties *		cp= rp->rpCells;
 
     TableRectangle		tr;
 
-    int				col;
-
     int				width;
     int				changed;
 
     int				res;
 
-    const int			minValue= 1;
+    const int			minValue= 0;
     const int			adaptToMin= 0;
     const int			maxValue= INT_MAX;
     const int			adaptToMax= 0;
 
+    BorderProperties		bp;
+
+    PropertyMask		rpSetMask;
+    PropertyMask		cpSetMask;
+    PropertyMask		bpSetMask;
+
+    /**/
+    EditApplication *		ea= rt->rtApplication;
+    EditDocument *		ed= ea->eaCurrentDocument;
+
+    TedDocument *		td;
+    BufferDocument *		bd;
+    DocumentProperties *	dp;
+
+    if  ( ! ed )
+	{ XDEB(ed); return;	}
+
+    td= (TedDocument *)ed->edPrivateData;
+    bd= td->tdDocument;
+    dp= &(bd->bdProperties);
+
+    /**/
+    PROPmaskCLEAR( &rpSetMask );
+    PROPmaskCLEAR( &cpSetMask );
+
+    PROPmaskADD( &rpSetMask, RPpropKEEP_ON_ONE_PAGE );
+    PROPmaskADD( &rpSetMask, RPpropIS_TABLE_HEADER );
+    /* No! not yet
+    PROPmaskADD( &rpSetMask, RPpropKEEP_WITH_NEXT );
+    */
+
+    docInitBorderProperties( &bp );
+
     if  ( tedFormatToolGetHeight( &(rp->rpHeightTwips),
 						&(rt->rtHeightChooser) ) )
-	{ return;	}
+	{ goto ready;	}
+    PROPmaskADD( &rpSetMask, RPpropHEIGHT );
 
     width= rp->rpHalfGapWidthTwips;
     if  ( appGetLengthFromTextWidget( rt->rtCellMarginText,
 				&width, &changed, UNITtyPOINTS,
 				minValue, adaptToMin, maxValue, adaptToMax ) )
-	{ return;	}
+	{ goto ready;	}
     rp->rpHalfGapWidthTwips= width;
+    PROPmaskADD( &rpSetMask, RPpropGAP_WIDTH );
 
     res= tedFormatToolGetRowLeftIndent( rp, rt->rtPageLeftMargin, &width,
 							rt->rtLeftIndentText );
     if  ( res != 0 )
-	{ return;	}
+	{ goto ready;	}
     rp->rpLeftIndentTwips= width;
+    PROPmaskADD( &rpSetMask, RPpropLEFT_INDENT );
 
     tr= rt->rtTableRectangle;
     docExpandTableRectangleToWholeRows( &tr );
 
-    if  ( tedBorderToolGetWidth( &(rt->rtTopBorderTool),
-					&(rp->rpCells[0].cpTopBorder) ) )
-	{ return;	}
+    /***/
+    PROPmaskCLEAR( &bpSetMask );
+    if  ( tedBorderToolGetProperties( &bp, &bpSetMask,
+						&(rt->rtTopBorderTool), dp ) )
+	{ goto ready;	}
 
-    if  ( tedBorderToolGetWidth( &(rt->rtBottomBorderTool),
-					&(rp->rpCells[0].cpBottomBorder) ) )
-	{ return;	}
-
-    cp= rp->rpCells+ 1;
-    for ( col= 1; col < rp->rpCellCount; cp++, col++ )
+    if  ( ! PROPmaskISEMPTY( &bpSetMask ) )
 	{
-	cp->cpTopBorder.bpPenWideTwips=
-			    rp->rpCells[0].cpTopBorder.bpPenWideTwips;
-	cp->cpBottomBorder.bpPenWideTwips=
-			    rp->rpCells[0].cpBottomBorder.bpPenWideTwips;
+	docRowSetTopBorderInCols( rp, tr.trCol0, tr.trCol1, &bpSetMask, &bp );
+	PROPmaskADD( &cpSetMask, CLpropTOP_BORDER );
 	}
 
-    tedAppSetTableProperties( rt->rtApplication, &tr, rp );
+    /***/
+    PROPmaskCLEAR( &bpSetMask );
+    if  ( tedBorderToolGetProperties( &bp, &bpSetMask,
+					    &(rt->rtBottomBorderTool), dp ) )
+	{ goto ready;	}
+
+    if  ( ! PROPmaskISEMPTY( &bpSetMask ) )
+	{
+	docRowSetBottomBorderInCols( rp, tr.trCol0, tr.trCol1,
+							&bpSetMask, &bp );
+	PROPmaskADD( &cpSetMask, CLpropBOTTOM_BORDER );
+	}
+
+    /***/
+    tedAppSetTableProperties( rt->rtApplication, &tr,
+					    &rpSetMask, &cpSetMask, rp );
+
+  ready:
+
+    docCleanBorderProperties( &bp );
 
     return;
     }
@@ -170,9 +247,7 @@ static void tedFormatChangeRowPushed(	APP_WIDGET	w,
 /*									*/
 /************************************************************************/
 
-static void tedRowHeightChanged(	APP_WIDGET	w,
-					void *		voidrt,
-					void *		voidpbcs )
+static APP_TXACTIVATE_CALLBACK_H( tedRowHeightChanged, w, voidrt )
     {
     RowTool *		rt= (RowTool *)voidrt;
     int			value;
@@ -183,9 +258,7 @@ static void tedRowHeightChanged(	APP_WIDGET	w,
 	}
     }
 
-static void tedRowCellMarginChanged(	APP_WIDGET	w,
-					void *		voidrt,
-					void *		voidpbcs )
+static APP_TXACTIVATE_CALLBACK_H( tedRowCellMarginChanged, w, voidrt )
     {
     RowTool *		rt= (RowTool *)voidrt;
     RowProperties *	rp= &(rt->rtPropertiesChosen);
@@ -201,9 +274,7 @@ static void tedRowCellMarginChanged(	APP_WIDGET	w,
 	}
     }
 
-static void tedRowLeftIndentChanged(	APP_WIDGET	w,
-					void *		voidrt,
-					void *		voidpbcs )
+static APP_TXACTIVATE_CALLBACK_H( tedRowLeftIndentChanged, w, voidrt )
     {
     RowTool *		rt= (RowTool *)voidrt;
     RowProperties *	rp= &(rt->rtPropertiesChosen);
@@ -226,22 +297,19 @@ static void tedRowLeftIndentChanged(	APP_WIDGET	w,
 /*									*/
 /************************************************************************/
 
-static void tedFormatRevertRowPushed(	APP_WIDGET	w,
-					void *		voidrt,
-					void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( tedFormatRevertRowPushed, w, voidrt )
     {
     RowTool *	rt= (RowTool *)voidrt;
 
-    docCopyRowProperties( &(rt->rtPropertiesChosen), &(rt->rtPropertiesSet) );
+    docCopyRowProperties( &(rt->rtPropertiesChosen),
+				&(rt->rtPropertiesSet), (const int *)0 );
 
     tedFormatToolRefreshRowPage( rt );
 
     return;
     }
 
-static void tedFormatPreviousRow(	APP_WIDGET	w,
-					void *		voidrt,
-					void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( tedFormatPreviousRow, w, voidrt )
     {
     RowTool *		rt= (RowTool *)voidrt;
     EditApplication *	ea= rt->rtApplication;
@@ -261,14 +329,12 @@ static void tedFormatPreviousRow(	APP_WIDGET	w,
     if  ( docShiftTableRectangleByRows( &tr, -1 ) )
 	{ return;	}
 
-    tedAppSetTableSelection( ed, &(td->tdSelectionScope), &tr );
+    tedAppSetTableSelection( ed, &tr );
 
     return;
     }
 
-static void tedFormatSelectRow(		APP_WIDGET	w,
-					void *		voidrt,
-					void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( tedFormatSelectRow, w, voidrt )
     {
     RowTool *		rt= (RowTool *)voidrt;
     EditApplication *	ea= rt->rtApplication;
@@ -285,14 +351,12 @@ static void tedFormatSelectRow(		APP_WIDGET	w,
 
     docExpandTableRectangleToWholeRows( &tr );
 
-    tedAppSetTableSelection( ed, &(td->tdSelectionScope), &tr );
+    tedAppSetTableSelection( ed, &tr );
 
     return;
     }
 
-static void tedFormatNextRow(		APP_WIDGET	w,
-					void *		voidrt,
-					void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( tedFormatNextRow, w, voidrt )
     {
     RowTool *		rt= (RowTool *)voidrt;
     EditApplication *	ea= rt->rtApplication;
@@ -312,7 +376,7 @@ static void tedFormatNextRow(		APP_WIDGET	w,
     if  ( docShiftTableRectangleByRows( &tr, +1 ) )
 	{ return;	}
 
-    tedAppSetTableSelection( ed, &(td->tdSelectionScope), &tr );
+    tedAppSetTableSelection( ed, &tr );
 
     return;
     }
@@ -323,9 +387,7 @@ static void tedFormatNextRow(		APP_WIDGET	w,
 /*									*/
 /************************************************************************/
 
-static void tedFormatDeleteRow(		APP_WIDGET	w,
-					void *		voidrt,
-					void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( tedFormatDeleteRow, w, voidrt )
     {
     RowTool *		rt= (RowTool *)voidrt;
     EditApplication *	ea= rt->rtApplication;
@@ -338,8 +400,6 @@ static void tedFormatDeleteRow(		APP_WIDGET	w,
     if  ( tedDeleteRowsFromTable( ed, tr->trRow0, tr->trRow1 ) )
 	{ LLDEB(tr->trRow0,tr->trRow1); return;	}
 
-    appDocumentChanged( ed, 1 );
-
     return;
     }
 
@@ -349,9 +409,7 @@ static void tedFormatDeleteRow(		APP_WIDGET	w,
 /*									*/
 /************************************************************************/
 
-static void tedFormatInsertRow(	APP_WIDGET	w,
-				void *		voidrt,
-				void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( tedFormatInsertRow, w, voidrt )
     {
     RowTool *		rt= (RowTool *)voidrt;
     EditApplication *	ea= rt->rtApplication;
@@ -366,9 +424,7 @@ static void tedFormatInsertRow(	APP_WIDGET	w,
     appDocumentChanged( ed, 1 );
     }
 
-static void tedFormatAppendRow(	APP_WIDGET	w,
-				void *		voidrt,
-				void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( tedFormatAppendRow, w, voidrt )
     {
     RowTool *		rt= (RowTool *)voidrt;
     EditApplication *	ea= rt->rtApplication;
@@ -382,11 +438,11 @@ static void tedFormatAppendRow(	APP_WIDGET	w,
     appDocumentChanged( ed, 1 );
     }
 
-static void tedRowHeightChosen(	APP_WIDGET	w,
-				void *		voidrt,
-				void *		voidtbcs )
+static APP_OITEM_CALLBACK_H( tedRowHeightChosen, w, voidrt )
     {
     RowTool *		rt= (RowTool *)voidrt;
+    HeightChooser *	hc= &(rt->rtHeightChooser);
+    int			how;
 
     int			defaultValue;
 
@@ -396,53 +452,22 @@ static void tedRowHeightChosen(	APP_WIDGET	w,
     if  ( defaultValue == 0 )
 	{ defaultValue= 240;	}
 
-    tedFormatHeightChosen( w, &(rt->rtHeightChooser), defaultValue );
+    how= appGuiGetOptionmenuItemIndex( &(hc->hcOptionmenu), w );
+    if  ( how < 0 || how >= HC__COUNT )
+	{ LDEB(how); return;	}
+
+    tedFormatHeightChosen( how, hc, defaultValue );
 
     return;
     }
 
-static void tedRowBorderStyleChosen(	APP_WIDGET	w,
-					void *		voidrt,
-					void *		voidtbcs )
-    {
-    RowTool *		rt= (RowTool *)voidrt;
-    RowProperties *	rp= &(rt->rtPropertiesChosen);
-    CellProperties *	cp= rp->rpCells;
-    CellProperties *	cp0= rp->rpCells;
-    int			col;
-
-    if  ( ! tedBorderToolGetStyle( &(rt->rtTopBorderTool),
-						&(cp->cpTopBorder), w ) )
-	{
-	cp++;
-	for ( col= 1; col < rp->rpCellCount; cp++, col++ )
-	    { cp->cpTopBorder.bpStyle= cp0->cpTopBorder.bpStyle; }
-
-	return;
-	}
-
-    if  ( ! tedBorderToolGetStyle( &(rt->rtBottomBorderTool),
-						&(cp->cpBottomBorder), w ) )
-	{
-	cp++;
-	for ( col= 1; col < rp->rpCellCount; cp++, col++ )
-	    { cp->cpBottomBorder.bpStyle= cp0->cpBottomBorder.bpStyle; }
-
-	return;
-	}
-
-    XDEB(w); return;
-    }
-
 /************************************************************************/
 /*									*/
 /*  A toggle was activated.						*/
 /*									*/
 /************************************************************************/
 
-static void tedRowToggleKeepOnPage(	APP_WIDGET	w,
-					void *		voidrt,
-					void *		voidtbcs )
+static APP_TOGGLE_CALLBACK_H( tedRowToggleKeepOnPage, w, voidrt, voidtbcs )
     {
     RowTool *				rt= (RowTool *)voidrt;
     RowProperties *			rp= &(rt->rtPropertiesChosen);
@@ -450,14 +475,12 @@ static void tedRowToggleKeepOnPage(	APP_WIDGET	w,
 
     set= appGuiGetToggleStateFromCallback( w, voidtbcs );
 
-    rp->rpKeepOnPage= set;
+    rp->rpKeepOnOnePage= set;
 
     return;
     }
 
-static void tedRowToggleIsTableHeader(	APP_WIDGET	w,
-					void *		voidrt,
-					void *		voidtbcs )
+static APP_TOGGLE_CALLBACK_H( tedRowToggleIsTableHeader, w, voidrt, voidtbcs )
     {
     RowTool *				rt= (RowTool *)voidrt;
     RowProperties *			rp= &(rt->rtPropertiesChosen);
@@ -472,47 +495,29 @@ static void tedRowToggleIsTableHeader(	APP_WIDGET	w,
 
 /************************************************************************/
 /*									*/
-/*  One of the 'Row Border' toggles has been activated.			*/
+/*  Callback from the general color chooser: a color was chosen.	*/
 /*									*/
 /************************************************************************/
 
-static void tedRowToggleBorder(	APP_WIDGET	w,
-				void *		voidrt,
-				void *		voidtbcs )
+static void tedRowToolGotColor(		void *			voidrt,
+					int 			which,
+					const RGB8Color *	rgb8 )
     {
-    RowTool *				rt= (RowTool *)voidrt;
-    RowProperties *			rp= &(rt->rtPropertiesChosen);
-    CellProperties *			cp;
-    int					col;
-    int					set;
+    RowTool *			rt= (RowTool *)voidrt;
 
-    set= appGuiGetToggleStateFromCallback( w, voidtbcs );
-
-    if  ( w == rt->rtTopBorderTool.btOnOffToggle )
+    switch( which )
 	{
-	cp= rp->rpCells;
-	for ( col= 0; col < rp->rpCellCount; cp++, col++ )
-	    {
-	    tedFormatToggleBorder( &(cp->cpTopBorder),
-				    &(rt->rtTopBorderTool), col == 0, set );
-	    }
+	case CLpropTOP_BORDER:
+	    tedBorderSetExplicitColorChoice( &(rt->rtTopBorderTool), rgb8 );
+	    break;
 
-	return;
+	case CLpropBOTTOM_BORDER:
+	    tedBorderSetExplicitColorChoice( &(rt->rtBottomBorderTool), rgb8 );
+	    break;
+
+	default:
+	    LDEB(which); return;
 	}
-
-    if  ( w == rt->rtBottomBorderTool.btOnOffToggle )
-	{
-	cp= rp->rpCells;
-	for ( col= 0; col < rp->rpCellCount; cp++, col++ )
-	    {
-	    tedFormatToggleBorder( &(cp->cpBottomBorder),
-				    &(rt->rtBottomBorderTool), col == 0, set );
-	    }
-
-	return;
-	}
-
-    XDEB(w); return;
     }
 
 /************************************************************************/
@@ -521,11 +526,13 @@ static void tedRowToggleBorder(	APP_WIDGET	w,
 /*									*/
 /************************************************************************/
 
-void tedFormatFillRowPage(	RowTool *			rt,
-				const RowPageResources *	rpr,
-				InspectorSubject *			is,
-				APP_WIDGET			pageWidget,
-				const InspectorSubjectResources *	isr )
+void tedFormatFillRowPage(  RowTool *				rt,
+			    const RowPageResources *		rpr,
+			    AppInspector *			ai,
+			    int					subjectPage,
+			    InspectorSubject *			is,
+			    APP_WIDGET				pageWidget,
+			    const InspectorSubjectResources *	isr )
     {
     APP_WIDGET	rowLabel;
     APP_WIDGET	widthLabel;
@@ -535,12 +542,11 @@ void tedFormatFillRowPage(	RowTool *			rt,
 
     const int	textColumns= 10;
 
-    /**************/
+    /**/
+    is->isPrivate= rt;
+    is->isGotColor= tedRowToolGotColor;
 
-    docInitTableRectangle( &(rt->rtTableRectangle) );
-
-    docInitRowProperties( &(rt->rtPropertiesSet) );
-    docInitRowProperties( &(rt->rtPropertiesChosen) );
+    rt->rtPageResources= rpr;
 
     /**************/
     appMakeLabelAndTextRow( &row, &rowLabel, &(rt->rtNumberText),
@@ -565,11 +571,13 @@ void tedFormatFillRowPage(	RowTool *			rt,
 				tedRowCellMarginChanged, (void *)rt );
 
     /**************/
-    tedFormatMakeBorderTool( &(rt->rtTopBorderTool), &row, pageWidget,
-			rpr->rprTopBorder, tedRowToggleBorder, (void *)rt );
+    tedMakeBorderTool( &(rt->rtTopBorderTool), ai, pageWidget,
+	    rpr->rprTopBorder, &(rpr->rprBorderToolResources),
+	    subjectPage, CLpropTOP_BORDER );
 
-    tedFormatMakeBorderTool( &(rt->rtBottomBorderTool), &row, pageWidget,
-			rpr->rprBottomBorder, tedRowToggleBorder, (void *)rt );
+    tedMakeBorderTool( &(rt->rtBottomBorderTool), ai, pageWidget,
+	    rpr->rprBottomBorder, &(rpr->rprBorderToolResources),
+	    subjectPage, CLpropBOTTOM_BORDER );
 
     /**************/
     appInspectorMakeToggleRow( &row, pageWidget,
@@ -597,8 +605,6 @@ void tedFormatFillRowPage(	RowTool *			rt,
 	    isr->isrRevert, isr->isrApplyToSubject,
 	    tedFormatRevertRowPushed, tedFormatChangeRowPushed, (void *)rt );
 
-    appGuiEnableWidget( rt->rtIsTableHeaderToggle, 0 );
-
     return;
     }
 
@@ -610,11 +616,6 @@ void tedFormatFillRowChoosers(		RowTool *			rt,
 					    rpr->rprHeightFree,
 					    rpr->rprHeightAtLeast,
 					    rpr->rprHeightExactly );
-
-    tedFormatFillBorderTool( &(rt->rtTopBorderTool),
-					tedRowBorderStyleChosen, (void *)rt );
-    tedFormatFillBorderTool( &(rt->rtBottomBorderTool),
-					tedRowBorderStyleChosen, (void *)rt );
     }
 
 void tedFormatFinishRowPage( 	RowTool *			rt,
@@ -623,8 +624,53 @@ void tedFormatFinishRowPage( 	RowTool *			rt,
     {
     appOptionmenuRefreshWidth( &(rt->rtHeightChooser.hcOptionmenu) );
 
-    tedFormatFinishBorderTool( &(rt->rtTopBorderTool), tft );
-    tedFormatFinishBorderTool( &(rt->rtBottomBorderTool), tft );
+    tedFinishBorderTool( &(rt->rtTopBorderTool) );
+    tedFinishBorderTool( &(rt->rtBottomBorderTool) );
+    }
+
+/************************************************************************/
+/*									*/
+/*  Clean a row tool.							*/
+/*									*/
+/************************************************************************/
+
+void tedInitRowTool(	RowTool *	rt )
+    {
+    rt->rtApplication= (EditApplication *)0;
+    rt->rtInspector= (AppInspector *)0;
+    rt->rtPageResources= (const RowPageResources *)0;
+
+    docInitRowProperties( &(rt->rtPropertiesSet) );
+    docInitRowProperties( &(rt->rtPropertiesChosen) );
+
+    docInitTableRectangle( &(rt->rtTableRectangle ) );
+
+    rt->rtPageRight= 0;
+    rt->rtPageLeftMargin= 0;
+
+    rt->rtNumberText= (APP_WIDGET)0;
+    tedInitHeightChooser( &(rt->rtHeightChooser) );
+    rt->rtCellMarginText= (APP_WIDGET)0;
+    rt->rtLeftIndentText= (APP_WIDGET)0;
+
+    tedInitBorderTool( &(rt->rtTopBorderTool) );
+    tedInitBorderTool( &(rt->rtBottomBorderTool) );
+
+    rt->rtKeepOnPageToggle= (APP_WIDGET)0;
+    rt->rtIsTableHeaderToggle= (APP_WIDGET)0;
+
+    return;
+    }
+
+void tedCleanRowTool(	RowTool *	rt )
+    {
+    docCleanRowProperties( &(rt->rtPropertiesSet) );
+    docCleanRowProperties( &(rt->rtPropertiesChosen) );
+
+    tedCleanBorderTool( &(rt->rtTopBorderTool) );
+    tedCleanBorderTool( &(rt->rtBottomBorderTool) );
+
+    return;
     }
 
 /************************************************************************/
@@ -689,6 +735,16 @@ static AppConfigurableResource TED_TedRowToolResourceTable[]=
     APP_RESOURCE( "tableToolRowBottomBorder",
 	    offsetof(RowPageResources,rprBottomBorder),
 	    "Bottom Border" ),
+
+    APP_RESOURCE( "formatToolRowBorderColorChooserAutomatic",
+	    offsetof(RowPageResources,rprBorderToolResources.
+			    btrColorChooserResources.ccrAutomaticColor),
+	    "Automatic" ),
+    APP_RESOURCE( "formatToolRowBorderColorChooserMoreColors",
+	    offsetof(RowPageResources,rprBorderToolResources.
+			    btrColorChooserResources.ccrMoreColors),
+	    "More Colors..." ),
+
     /**/
     APP_RESOURCE( "formatToolRowKeepOnPage",
 	    offsetof(RowPageResources,rprKeepOnPage),
@@ -696,6 +752,17 @@ static AppConfigurableResource TED_TedRowToolResourceTable[]=
     APP_RESOURCE( "formatToolRowIsTableHeader",
 	    offsetof(RowPageResources,rprIsTableHeader),
 	    "Is Table Header" ),
+
+    /**/
+    APP_RESOURCE( "formatToolRowBorderWidth",
+	    offsetof(RowPageResources, rprBorderToolResources.btrWidth),
+	    "Width" ),
+    APP_RESOURCE( "formatToolRowBorderStyle",
+	    offsetof(RowPageResources, rprBorderToolResources.btrStyle),
+	    "Style" ),
+    APP_RESOURCE( "formatToolRowBorderColor",
+	    offsetof(RowPageResources, rprBorderToolResources.btrColor),
+	    "Color" ),
     };
 
 void tedFormatToolGetRowResourceTable(	EditApplication *		ea,
