@@ -27,6 +27,8 @@ typedef struct RowPageResources
 
     char *		rprKeepOnPage;
     char *		rprIsTableHeader;
+
+    BorderToolResources	rprBorderToolResources;
     } RowPageResources;
 
 /************************************************************************/
@@ -39,6 +41,7 @@ typedef struct RowTool
     {
     EditApplication *		rtApplication;
     AppInspector *		rtInspector;
+    const RowPageResources *	rtPageResources;
 
     RowProperties		rtPropertiesSet;
     RowProperties		rtPropertiesChosen;
@@ -53,10 +56,6 @@ typedef struct RowTool
     APP_WIDGET			rtCellMarginText;
     APP_WIDGET			rtLeftIndentText;
 
-    APP_WIDGET			rtHeightFreeItem;
-    APP_WIDGET			rtHeightAtLeastItem;
-    APP_WIDGET			rtHeightExactlyItem;
-
     BorderTool			rtTopBorderTool;
     BorderTool			rtBottomBorderTool;
 
@@ -70,8 +69,18 @@ typedef struct RowTool
 /*									*/
 /************************************************************************/
 
-extern void tedFormatToolGetRowResourceTable(	EditApplication *	ea,
-					RowPageResources *		spr,
-					InspectorSubjectResources *	isr );
+extern void tedFormatToolGetRowResourceTable(
+				EditApplication *		ea,
+				RowPageResources *		spr,
+				InspectorSubjectResources *	isr );
+
+extern void tedFormatToolRefreshRowTool(
+				RowTool *			rt,
+				int *				pEnabled,
+				InspectorSubject *		is,
+				const DocumentSelection *	bs );
+
+extern void tedInitRowTool(	RowTool *	rt );
+extern void tedCleanRowTool(	RowTool *	rt );
 
 #   endif	/*  TED_ROW_TOOL_H */
