@@ -1,7 +1,8 @@
-#   include	"config.h"
+#   include	"tedConfig.h"
 
 #   include	<stdlib.h>
 #   include	<stdio.h>
+#   include	<string.h>
 
 #   include	"tedRuler.h"
 #   include	<appFrame.h>
@@ -72,7 +73,7 @@ typedef struct TedTopRuler
     int			ttrTabCount;
     TabStop *		ttrTabStops;
     int			ttrTabKind;		/************************/
-						/*  tsKind for fresh	*/
+						/*  alignment for fresh	*/
 						/*  tabs.		*/
 						/************************/
     int			ttrColumnCount;
@@ -96,7 +97,7 @@ typedef struct TopRulerDrag
     } TopRulerDrag;
 
 void * tedMakeTopRuler(	int			height,
-			double			pixPerMM,
+			double			magnifiedPixelsPerTwip,
 			double			magnification,
 
 			int			leftRulerWidth,
@@ -124,7 +125,7 @@ void * tedMakeTopRuler(	int			height,
     rd->rdUnit= whatUnit;
     rd->rdFontName= fontName;
 
-    appRulerCalculateIncrements( rd, pixPerMM, magnification );
+    appRulerCalculateIncrements( rd, magnifiedPixelsPerTwip, magnification );
 
     rd->rdMinUsed= leftRulerWidth;
     rd->rdMaxUsed= 0;
@@ -146,7 +147,7 @@ void * tedMakeTopRuler(	int			height,
 
     ttr->ttrTabCount= 0;
     ttr->ttrTabStops= (TabStop *)0;
-    ttr->ttrTabKind= DOCtkLEFT;
+    ttr->ttrTabKind= DOCtaLEFT;
 
     ttr->ttrColumnCount= 0;
     ttr->ttrColumns= (ColumnSeparator *)0;
@@ -509,9 +510,9 @@ static void tedDrawTabSymbol(	AppDrawingData *	add,
     xx= x- thick/2;
     appDrawFillRectangle( add, xx, y- tabHeight, thick, tabHeight );
 
-    if  ( kind == DOCtkLEFT		||
-	  kind == DOCtkCENTRE		||
-	  kind == DOCtkDECIMAL		)
+    if  ( kind == DOCtaLEFT		||
+	  kind == DOCtaCENTRE		||
+	  kind == DOCtaDECIMAL		)
 	{
 	xx= x- thick/2;
 
@@ -519,9 +520,9 @@ static void tedDrawTabSymbol(	AppDrawingData *	add,
 	appDrawFillRectangle( add, xx, y- thick, tabWidth, thick );
 	}
 
-    if  ( kind == DOCtkRIGHT		||
-	  kind == DOCtkCENTRE		||
-	  kind == DOCtkDECIMAL		)
+    if  ( kind == DOCtaRIGHT		||
+	  kind == DOCtaCENTRE		||
+	  kind == DOCtaDECIMAL		)
 	{
 	xx= x+ (thick+1)/2- tabWidth;
 
@@ -529,7 +530,7 @@ static void tedDrawTabSymbol(	AppDrawingData *	add,
 	appDrawFillRectangle( add, xx, y- thick, tabWidth, thick );
 	}
 
-    if  ( kind == DOCtkDECIMAL	)
+    if  ( kind == DOCtaDECIMAL	)
 	{
 	int	dec= ( thick+ 1 )/ 2;
 
@@ -609,16 +610,15 @@ static void tedDrawTabButton(	RulerData *	rd,
 /*									*/
 /************************************************************************/
 
-static void tedDrawHorizontalRuler(	APP_WIDGET		w,
-					TedTopRuler *		ttr,
-					DocumentRectangle *	drClip,
-					int			ox )
+static void tedDrawTopRuler(	APP_WIDGET		w,
+				TedTopRuler *		ttr,
+				DocumentRectangle *	drClip,
+				int			ox )
     {
     RulerData *		rd= &(ttr->ttrRulerData);
     AppDrawingData *	add= &(rd->rdDrawingData);
     int			sizeAcross= rd->rdSizeAcross;
 
-    double		item;
     int			tick;
     double		units;
 
@@ -680,18 +680,22 @@ static void tedDrawHorizontalRuler(	APP_WIDGET		w,
 
     appDrawSetForegroundColor( add, &(add->addForeColor) );
 
-    item= 0; units= 0; tick= 0;
-    while( item <= rd->rdVisibleC1 )
+    units= 0; tick= 0;
+    for (;;)
 	{
-	if  ( item < rd->rdVisibleC0 )
+	int	twips= units* rd->rdTwipsPerUnit;
+	int	iitem= rd->rdMagnifiedPixelsPerTwip* twips;
+
+	if  ( iitem > rd->rdVisibleC1 )
+	    { break;	}
+
+	if  ( iitem < rd->rdVisibleC0 )
 	    {
-	    tick++;
-	    units += rd->rdUnitsPerTick;
-	    item += rd->rdTickDistancePixels;
+	    units += rd->rdUnitsPerTick; tick++;
 	    continue;
 	    }
 
-	if  ( item != 0 && tick % rd->rdTicksPerText == 0 )
+	if  ( iitem != 0 && tick % rd->rdTicksPerText == 0 )
 	    {
 	    char	scratch[30];
 	    int		len;
@@ -699,29 +703,23 @@ static void tedDrawHorizontalRuler(	APP_WIDGET		w,
 
 	    appRulerTagText( scratch, &wide, &len, rd, units );
 
-	    if  ( item- ox- wide/2 >= rd->rdMinUsed	&&
-		  item- ox+ wide/2 <  rd->rdMaxUsed	)
+	    if  ( iitem- ox- wide/2 >= rd->rdMinUsed	&&
+		  iitem- ox+ wide/2 <  rd->rdMaxUsed	)
 		{
 		appDrawDrawString( add,
-			    (int)item- ox- wide/2, fontBottom, scratch, len );
+			    iitem- ox- wide/2, fontBottom, scratch, len );
 		}
 	    else{
-		appDrawFillRectangle( add, (int)item- ox, tagY, 1, tagH );
+		appDrawFillRectangle( add, iitem- ox, tagY, 1, tagH );
 		}
 	    }
 	else{
 	    if  ( tick % rd->rdTicksPerTag == 0 )
-		{
-		appDrawFillRectangle( add, (int)item- ox, tagY, 1, tagH );
-		}
-	    else{
-		appDrawFillRectangle( add, (int)item- ox, tickY, 1, tickH );
-		}
+		{ appDrawFillRectangle( add, iitem- ox, tagY,  1, tagH );  }
+	    else{ appDrawFillRectangle( add, iitem- ox, tickY, 1, tickH ); }
 	    }
 
-	tick++;
-	units += rd->rdUnitsPerTick;
-	item += rd->rdTickDistancePixels;
+	units += rd->rdUnitsPerTick; tick++;
 	continue;
 	}
 
@@ -797,7 +795,7 @@ static void tedDrawHorizontalRuler(	APP_WIDGET		w,
 	    { continue;	}
 
 	tedDrawTabSymbol( add, ts->tsPixels, ox, whiteY+ whiteH,
-						    sizeAcross, ts->tsKind );
+						sizeAcross, ts->tsAlignment );
 	}
 
     appDrawNoClipping( add );
@@ -805,7 +803,7 @@ static void tedDrawHorizontalRuler(	APP_WIDGET		w,
     return;
     }
 
-APP_EVENT_HANDLER( tedRedrawHorizontalRuler, w, voidttr, event )
+APP_EVENT_HANDLER_H( tedRedrawHorizontalRuler, w, voidttr, event )
     {
     TedTopRuler *	ttr= (TedTopRuler *)voidttr;
     RulerData *		rd= &(ttr->ttrRulerData);
@@ -826,10 +824,10 @@ APP_EVENT_HANDLER( tedRedrawHorizontalRuler, w, voidttr, event )
 
     appCollectExposures( &drClip, add, ox, oy, event );
 
-    tedDrawHorizontalRuler( w, ttr, &drClip, ox );
+    tedDrawTopRuler( w, ttr, &drClip, ox );
     }
 
-APP_EVENT_HANDLER( tedHorizontalRulerConfigure, w, voidttr, event )
+APP_EVENT_HANDLER_H( tedHorizontalRulerConfigure, w, voidttr, event )
     {
     TedTopRuler *	ttr= (TedTopRuler *)voidttr;
     RulerData *		rd= &(ttr->ttrRulerData);
@@ -951,7 +949,7 @@ void tedScrollHorizontalRuler(	void *			voidttr,
     changed= 0;
     tedRulerChangeRect( &changed, &drRedraw, &drClip, d, ttr );
 
-    tedDrawHorizontalRuler( w, ttr, &drRedraw, ox );
+    tedDrawTopRuler( w, ttr, &drRedraw, ox );
 
     if  ( d > 0 )
 	{ drClip.drX0= rd->rdVisibleC0- 1; drClip.drX1= rd->rdVisibleC0+ 1; }
@@ -961,7 +959,7 @@ void tedScrollHorizontalRuler(	void *			voidttr,
     tedRulerChangeRect( &changed, &drRedraw, &drClip, d, ttr );
 
     if  ( changed )
-	{ tedDrawHorizontalRuler( w, ttr, &drRedraw, ox ); }
+	{ tedDrawTopRuler( w, ttr, &drRedraw, ox ); }
     }
 
 void tedSetHorizontalRulerRange(	void *			voidttr,
@@ -1055,7 +1053,7 @@ static void tedHorizontalRulerSetValue(	TopRulerDrag *		trd,
 	drClip.drY1= rd->rdSizeAcross;
 	}
 
-    tedDrawHorizontalRuler( trd->trdWidget, ttr, &drClip, trd->trdOc );
+    tedDrawTopRuler( trd->trdWidget, ttr, &drClip, trd->trdOc );
 
     /*  5  */
     (*trd->trdDragHair)( trd->trdThrough, v0, nv0, asUpper );
@@ -1067,7 +1065,7 @@ static void tedHorizontalRulerSetValue(	TopRulerDrag *		trd,
     return;
     }
 
-static APP_EVENT_HANDLER( tedHorizontalRulerMouseMove, w, vtrd, event )
+static APP_EVENT_HANDLER_H( tedHorizontalRulerMouseMove, w, vtrd, event )
     {
     TopRulerDrag *		trd= (TopRulerDrag *)vtrd;
 
@@ -1082,7 +1080,7 @@ static APP_EVENT_HANDLER( tedHorizontalRulerMouseMove, w, vtrd, event )
     return;
     }
 
-static APP_EVENT_HANDLER( tedHorizontalRulerMouseUp, w, vtrd, event )
+static APP_EVENT_HANDLER_H( tedHorizontalRulerMouseUp, w, vtrd, event )
     {
     TopRulerDrag *		trd= (TopRulerDrag *)vtrd;
     const int			asUpper= 0;
@@ -1093,9 +1091,10 @@ static APP_EVENT_HANDLER( tedHorizontalRulerMouseUp, w, vtrd, event )
     int				button;
     int				upDown;
     int				seq;
+    unsigned int		keyState;
 
     if  ( appGetCoordinatesFromMouseButtonEvent(
-				&x, &y, &button, &upDown, &seq, w, event ) )
+			    &x, &y, &button, &upDown, &seq, &keyState, w, event ) )
 	{ return;	}
 
     tedHorizontalRulerSetValue( trd, x+ trd->trdOc );
@@ -1208,22 +1207,35 @@ static void tedHorizontalRulerDragTab(	int *			pChanged,
 
     TabStop *		ts= ttr->ttrTabStops+ item;
 
+    int			x0= ts->tsPixels;
+    int			x1;
+    int			dxAbs;
+
     tedHorizontalRulerDragItem( &(ts->tsPixels), (int *)0, &y, pChanged,
 			w, ea, downEvent,
 			ttr, ttr->ttrColX0, ttr->ttrColX1, x, ox,
 			voided, dragHair );
 
+    x1= ts->tsPixels;
+
+    dxAbs= x1- x0;
+    if  ( dxAbs < 0 )
+	{ dxAbs= -dxAbs;	}
+
     /*  3  */
-    if  ( y <  WHITE_TOP(sizeAcross)				||
-	  y >  WHITE_TOP(sizeAcross)+ WHITE_HEIGHT(sizeAcross)	)
+    if  ( dxAbs < 2* WHITE_HEIGHT(sizeAcross) )
 	{
-	int		i;
+	if  ( y <  WHITE_TOP(sizeAcross)				||
+	      y >  WHITE_TOP(sizeAcross)+ WHITE_HEIGHT(sizeAcross)	)
+	    {
+	    int		i;
 
-	ttr->ttrTabCount--;
-	for ( i= item; i < ttr->ttrTabCount; i++ )
-	    { ttr->ttrTabStops[i]= ttr->ttrTabStops[i+ 1]; }
+	    ttr->ttrTabCount--;
+	    for ( i= item; i < ttr->ttrTabCount; i++ )
+		{ ttr->ttrTabStops[i]= ttr->ttrTabStops[i+ 1]; }
 
-	appExposeRectangle( add, 0, 0, 0, 0 );
+	    appExposeRectangle( add, 0, 0, 0, 0 );
+	    }
 	}
 
     qsort( ttr->ttrTabStops, ttr->ttrTabCount, sizeof(TabStop),
@@ -1239,7 +1251,7 @@ void tedHorizontalRulerTrackMouse(	int *			pFirstIndent,
 					TabStop **		pTabStops,
 					int *			pCsCount,
 					ColumnSeparator **	pCs,
-					unsigned int *		pChange,
+					int *			pProperty,
 					APP_WIDGET		w,
 					EditApplication *	ea,
 					APP_EVENT *		downEvent,
@@ -1270,9 +1282,10 @@ void tedHorizontalRulerTrackMouse(	int *			pFirstIndent,
     int			button;
     int			upDown;
     int			seq;
+    unsigned int	keyState;
 
     if  ( appGetCoordinatesFromMouseButtonEvent(
-			    &x, &y, &button, &upDown, &seq, w, downEvent ) )
+			&x, &y, &button, &upDown, &seq, &keyState, w, downEvent ) )
 	{ return;	}
 
     if  ( upDown < 1 || button != 1 || seq > 1 )
@@ -1297,9 +1310,9 @@ void tedHorizontalRulerTrackMouse(	int *			pFirstIndent,
 		    {
 		    *pTabCount= ttr->ttrTabCount;
 		    *pTabStops= ttr->ttrTabStops;
-		    *pChange= PPpropTAB_STOPS;
+		    *pProperty= PPpropTAB_STOPS;
 		    }
-		else{ *pChange= PPprop_NONE; }
+		else{ *pProperty= PPprop_NONE; }
 
 		return;
 		}
@@ -1323,9 +1336,9 @@ void tedHorizontalRulerTrackMouse(	int *			pFirstIndent,
 	    *pFirstIndent= ttr->ttrFirstIndent;
 	    *pTabCount= ttr->ttrTabCount;
 	    *pTabStops= ttr->ttrTabStops;
-	    *pChange= PPpropFIRST_INDENT;
+	    *pProperty= PPpropFIRST_INDENT;
 	    }
-	else{ *pChange= PPprop_NONE; }
+	else{ *pProperty= PPprop_NONE; }
 	return;
 	}
 
@@ -1347,9 +1360,9 @@ void tedHorizontalRulerTrackMouse(	int *			pFirstIndent,
 	    *pLeftIndent= ttr->ttrLeftIndent;
 	    *pTabCount= ttr->ttrTabCount;
 	    *pTabStops= ttr->ttrTabStops;
-	    *pChange= PPpropLEFT_INDENT;
+	    *pProperty= PPpropLEFT_INDENT;
 	    }
-	else{ *pChange= PPprop_NONE; }
+	else{ *pProperty= PPprop_NONE; }
 	return;
 	}
 
@@ -1371,9 +1384,9 @@ void tedHorizontalRulerTrackMouse(	int *			pFirstIndent,
 	    *pRightIndent= ttr->ttrRightIndent;
 	    *pTabCount= ttr->ttrTabCount;
 	    *pTabStops= ttr->ttrTabStops;
-	    *pChange= PPpropRIGHT_INDENT;
+	    *pProperty= PPpropRIGHT_INDENT;
 	    }
-	else{ *pChange= PPprop_NONE; }
+	else{ *pProperty= PPprop_NONE; }
 	return;
 	}
 
@@ -1389,7 +1402,7 @@ void tedHorizontalRulerTrackMouse(	int *			pFirstIndent,
 	fresh= (TabStop *)realloc( ttr->ttrTabStops,
 				    ( ttr->ttrTabCount+ 1)* sizeof(TabStop) );
 	if  ( ! fresh )
-	    { LXDEB(ttr->ttrTabCount,fresh); *pChange= PPprop_NONE; return; }
+	    { LXDEB(ttr->ttrTabCount,fresh); *pProperty= PPprop_NONE; return; }
 
 	ttr->ttrTabStops= fresh;
 
@@ -1405,7 +1418,7 @@ void tedHorizontalRulerTrackMouse(	int *			pFirstIndent,
 	docInitTabStop( ttr->ttrTabStops+ item );
 
 	ttr->ttrTabStops[item].tsPixels= x+ ox;
-	ttr->ttrTabStops[item].tsKind= ttr->ttrTabKind;
+	ttr->ttrTabStops[item].tsAlignment= ttr->ttrTabKind;
 	ttr->ttrTabCount++;
 
 	appExposeRectangle( add, 0, 0, 0, 0 );
@@ -1415,7 +1428,7 @@ void tedHorizontalRulerTrackMouse(	int *			pFirstIndent,
 
 	*pTabCount= ttr->ttrTabCount;
 	*pTabStops= ttr->ttrTabStops;
-	*pChange= PPpropTAB_STOPS; return;
+	*pProperty= PPpropTAB_STOPS; return;
 	}
 
     /*  8  */
@@ -1451,9 +1464,9 @@ void tedHorizontalRulerTrackMouse(	int *			pFirstIndent,
 		if  ( changed )
 		    {
 		    *pCsCount= ttr->ttrColumnCount; *pCs= ttr->ttrColumns;
-		    *pChange= PPpropCOLUMNS;
+		    *pProperty= PPpropCOLUMNS;
 		    }
-		else{ *pChange= PPprop_NONE;	}
+		else{ *pProperty= PPprop_NONE;	}
 		return;
 		}
 	    }
@@ -1467,9 +1480,10 @@ void tedHorizontalRulerTrackMouse(	int *			pFirstIndent,
 	{
 	switch( ttr->ttrTabKind )
 	    {
-	    case DOCtkLEFT:	ttr->ttrTabKind= DOCtkRIGHT;	break;
-	    case DOCtkRIGHT:	ttr->ttrTabKind= DOCtkCENTRE;	break;
-	    case DOCtkCENTRE:	ttr->ttrTabKind= DOCtkLEFT;	break;
+	    case DOCtaLEFT:	ttr->ttrTabKind= DOCtaRIGHT;	break;
+	    case DOCtaRIGHT:	ttr->ttrTabKind= DOCtaCENTRE;	break;
+	    case DOCtaCENTRE:	ttr->ttrTabKind= DOCtaDECIMAL;	break;
+	    case DOCtaDECIMAL:	ttr->ttrTabKind= DOCtaLEFT;	break;
 	    default:
 		LDEB(ttr->ttrTabKind);
 	    }
@@ -1479,7 +1493,7 @@ void tedHorizontalRulerTrackMouse(	int *			pFirstIndent,
 			dr.drX1- dr.drX0+ 1, dr.drY1- dr.drY0+ 1 );
 	}
 
-    *pChange= PPprop_NONE; return;
+    *pProperty= PPprop_NONE; return;
     }
 
 /************************************************************************/
@@ -1548,7 +1562,7 @@ void tedAdaptHorizontalRuler(	void *		voidttr,
 
 	if  ( ts->tsPixels != tabStops->tsPixels )
 	    { different= 1; changed= 1;	}
-	if  ( ts->tsKind != tabStops->tsKind )
+	if  ( ts->tsAlignment != tabStops->tsAlignment )
 	    { different= 1; changed= 1;	}
 	if  ( ts->tsLeader != tabStops->tsLeader )
 	    { different= 1;	}
@@ -1657,7 +1671,7 @@ typedef struct TedLeftRuler
     } TedLeftRuler;
 
 void * tedMakeLeftRuler(	int			width,
-				double			pixPerMM,
+				double			magnifiedPixelsPerTwip,
 				double			magnification,
 
 				int			minUnused,
@@ -1711,7 +1725,7 @@ void * tedMakeLeftRuler(	int			width,
 
     rd->rdFontName= fontName;
     rd->rdUnit= whatUnit;
-    appRulerCalculateIncrements( rd, pixPerMM, magnification );
+    appRulerCalculateIncrements( rd, magnifiedPixelsPerTwip, magnification );
 
     rd->rdMinUsed= minUnused;
     rd->rdMaxUsed= 0;
@@ -1729,7 +1743,7 @@ void * tedMakeLeftRuler(	int			width,
     return (void *)tlr;
     }
 
-APP_EVENT_HANDLER( tedVerticalRulerConfigure, w, voidtlr, event )
+APP_EVENT_HANDLER_H( tedVerticalRulerConfigure, w, voidtlr, event )
     {
     TedLeftRuler *	tlr= (TedLeftRuler *)voidtlr;
     RulerData *		rd= &(tlr->tlrRulerData);
@@ -1851,7 +1865,7 @@ void tedFreeVerticalRuler(	void *		voidtlr )
 /*									*/
 /************************************************************************/
 
-static void tedFormatPageNumber(	TedLeftRuler *		tlr,
+static void tedRulerFormatPageNumber(	TedLeftRuler *		tlr,
 					int *			pLen,
 					int *			pWide,
 					int			page )
@@ -1908,21 +1922,21 @@ static void tedDrawVerticalRuler(	APP_WIDGET		w,
 
     x= rd->rdSizeAcross/8+ rd->rdVerticalFont.vxfFontStruct->ascent;
 
-    tedFormatPageNumber( tlr, &len, &wide, tlr->tlrTopPage );
+    tedRulerFormatPageNumber( tlr, &len, &wide, tlr->tlrTopPage );
 
     appRulerDrawVerticalString( rd, x, 1+ wide, tlr->tlrPageScratch, len );
 
     if  ( tlr->tlrBottomPage != tlr->tlrTopPage			&&
 	  rd->rdSizeAlong > 4* wide				)
 	{
-	tedFormatPageNumber( tlr, &len, &wide, tlr->tlrBottomPage );
+	tedRulerFormatPageNumber( tlr, &len, &wide, tlr->tlrBottomPage );
 
 	appRulerDrawVerticalString( rd,
 			    x, rd->rdSizeAlong- 1, tlr->tlrPageScratch, len );
 	}
     }
 
-APP_EVENT_HANDLER( tedRedrawVerticalRuler, w, voidtlr, event )
+APP_EVENT_HANDLER_H( tedRedrawVerticalRuler, w, voidtlr, event )
     {
     TedLeftRuler *	tlr= (TedLeftRuler *)voidtlr;
     RulerData *		rd= &(tlr->tlrRulerData);
