@@ -4,13 +4,12 @@
 /*									*/
 /************************************************************************/
 
-#   include	"config.h"
+#   include	"tedConfig.h"
 
 #   include	<stdlib.h>
 #   include	<stdio.h>
 #   include	<stddef.h>
 #   include	<limits.h>
-#   include	<appDebugon.h>
 
 #   include	<appGeoString.h>
 #   include	<appUnit.h>
@@ -18,7 +17,7 @@
 #   include	"tedApp.h"
 #   include	"tedFormatTool.h"
 
-#   define	DRH_CM		4.5
+#   include	<appDebugon.h>
 
 /************************************************************************/
 /*									*/
@@ -71,13 +70,15 @@ static void tedFormatToolRefreshSectionPage(	SectionTool *	st )
     return;
     }
 
-void tedFormatToolRefreshSectionTool(	SectionTool *		st,
-					InspectorSubject *	is,
-					const BufferSelection *	bs )
+void tedFormatToolRefreshSectionTool(
+				SectionTool *			st,
+				int *				pEnabled,
+				InspectorSubject *		is,
+				const DocumentSelection *	ds )
     {
     BufferItem *		sectBi;
 
-    sectBi= bs->bsBegin.bpBi;
+    sectBi= ds->dsBegin.dpBi;
     while( sectBi && sectBi->biLevel > DOClevSECT )
 	{ sectBi= sectBi->biParent;	}
 
@@ -100,6 +101,7 @@ void tedFormatToolRefreshSectionTool(	SectionTool *		st,
 
     tedFormatToolRefreshSectionPage( st );
 
+    *pEnabled= ds->dsSelectionScope.ssInExternalItem == DOCinBODY;
     return;
     }
 
@@ -109,16 +111,14 @@ void tedFormatToolRefreshSectionTool(	SectionTool *		st,
 /*									*/
 /************************************************************************/
 
-static void tedFormatChangeSectPushed(	APP_WIDGET	w,
-					void *		voidst,
-					void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( tedFormatChangeSectPushed, w, voidst )
     {
     SectionTool *		st= (SectionTool *)voidst;
     SectionProperties *		spNew= &(st->stPropertiesChosen);
 
     int				changed;
 
-    unsigned int		update;
+    PropertyMask		updMask;
 
     const int			minValue= 1;
     const int			adaptToMin= 0;
@@ -154,62 +154,48 @@ static void tedFormatChangeSectPushed(	APP_WIDGET	w,
 		    minValue, adaptToMin, maxValue, adaptToMax ) )
 	{ return;	}
 
-    PROPmaskCLEAR( &update );
-    PROPmaskFILL( &update, SPprop_COUNT );
+    PROPmaskCLEAR( &updMask );
+    PROPmaskFILL( &updMask, SPprop_COUNT );
 
-    if  ( tedAppChangeSectionProperties( st->stApplication, update, spNew ) )
+    if  ( tedAppChangeSectionProperties( st->stApplication, &updMask, spNew ) )
 	{ LDEB(1);	}
 
     return;
     }
 
-static void tedSectNumberStyleChosen(	APP_WIDGET	w,
-					void *		voidst,
-					void *		voidtbcs )
+static APP_OITEM_CALLBACK_H( tedSectNumberStyleChosen, w, voidst )
     {
     SectionTool *			st= (SectionTool *)voidst;
     SectionProperties *			sp= &(st->stPropertiesChosen);
 
     int					style;
 
-    for ( style= 0; style < DOCpgn_COUNT; style++ )
-	{
-	if  ( st->stNumberStyleItems[style] == w )
-	    { break;	}
-	}
+    style= appGuiGetOptionmenuItemIndex( &(st->stNumberStyleOptionmenu), w );
+    if  ( style < 0 || style >= DOCpgn_COUNT )
+	{ LLDEB(style,DOCpgn_COUNT); return;	}
 
-    if  ( style >= 0 && style < DOCpgn_COUNT )
-	{ sp->spPageNumberStyle= style;	}
-    else{ LLDEB(style,DOCpgn_COUNT);	}
+    sp->spPageNumberStyle= style;
 
     return;
     }
 
-static void tedSectBreakKindChosen(	APP_WIDGET	w,
-					void *		voidst,
-					void *		voidtbcs )
+static APP_OITEM_CALLBACK_H( tedSectBreakKindChosen, w, voidst )
     {
     SectionTool *			st= (SectionTool *)voidst;
     SectionProperties *			sp= &(st->stPropertiesChosen);
 
     int					breakKind;
 
-    for ( breakKind= 0; breakKind < DOCsbk_COUNT; breakKind++ )
-	{
-	if  ( st->stBreakKindItems[breakKind] == w )
-	    { break;	}
-	}
+    breakKind= appGuiGetOptionmenuItemIndex( &(st->stBreakKindOptionmenu), w );
+    if  ( breakKind < 0 || breakKind >= DOCsbk_COUNT )
+	{ LLDEB(breakKind,DOCsbk_COUNT); return;	}
 
-    if  ( breakKind >= 0 && breakKind < DOCsbk_COUNT )
-	{ sp->spBreakKind= breakKind;		}
-    else{ LLDEB(breakKind,DOCsbk_COUNT);	}
+    sp->spBreakKind= breakKind;
 
     return;
     }
 
-static void tedFormatPageRestartToggled(	APP_WIDGET	w,
-						void *		voidst,
-						void *		voidtbcs )
+static APP_TOGGLE_CALLBACK_H( tedFormatPageRestartToggled, w, voidst, voidtbcs )
     {
     SectionTool *		st= (SectionTool *)voidst;
     SectionProperties *		sp= &(st->stPropertiesChosen);
@@ -230,9 +216,7 @@ static void tedFormatPageRestartToggled(	APP_WIDGET	w,
 /*									*/
 /************************************************************************/
 
-static void tedSectColSpacingChanged(	APP_WIDGET	w,
-					void *		voidst,
-					void *		voidpbcs )
+static APP_TXACTIVATE_CALLBACK_H( tedSectColSpacingChanged, w, voidst )
     {
     SectionTool *		st= (SectionTool *)voidst;
     SectionProperties *		sp= &(st->stPropertiesChosen);
@@ -242,14 +226,13 @@ static void tedSectColSpacingChanged(	APP_WIDGET	w,
 
     value= sp->spColumnSpacingTwips;
 
-    tedFormatValidateDimension( &value, &changed, w, value );
+    tedFormatValidateDimension( &value, &changed,
+					    st->stColumnSpacingText, value );
 
     return;
     }
 
-static void tedSectColWidthChanged(	APP_WIDGET	w,
-					void *		voidst,
-					void *		voidpbcs )
+static APP_TXACTIVATE_CALLBACK_H( tedSectColWidthChanged, w, voidst )
     {
     SectionTool *		st= (SectionTool *)voidst;
     SectionProperties *		sp= &(st->stPropertiesChosen);
@@ -259,7 +242,8 @@ static void tedSectColWidthChanged(	APP_WIDGET	w,
 
     value= sp->spColumnWidthTwips;
 
-    tedFormatValidateDimension( &value, &changed, w, value );
+    tedFormatValidateDimension( &value, &changed,
+					    st->stColumnWidthText, value );
 
     return;
     }
@@ -270,9 +254,7 @@ static void tedSectColWidthChanged(	APP_WIDGET	w,
 /*									*/
 /************************************************************************/
 
-static void tedFormatRevertSectPushed(	APP_WIDGET	w,
-					void *		voidst,
-					void *		voidpbcs	 )
+static APP_BUTTON_CALLBACK_H( tedFormatRevertSectPushed, w, voidst )
     {
     SectionTool *	st= (SectionTool *)voidst;
 
@@ -290,9 +272,7 @@ static void tedFormatRevertSectPushed(	APP_WIDGET	w,
 /*									*/
 /************************************************************************/
 
-static void tedFormatPrevSect(		APP_WIDGET	w,
-					void *		voidst,
-					void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( tedFormatPrevSect, w, voidst )
     {
     SectionTool *		st= (SectionTool *)voidst;
     EditApplication *		ea= st->stApplication;
@@ -302,9 +282,7 @@ static void tedFormatPrevSect(		APP_WIDGET	w,
     return;
     }
 
-static void tedFormatNextSect(		APP_WIDGET	w,
-					void *		voidst,
-					void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( tedFormatNextSect, w, voidst )
     {
     SectionTool *		st= (SectionTool *)voidst;
     EditApplication *		ea= st->stApplication;
@@ -314,9 +292,7 @@ static void tedFormatNextSect(		APP_WIDGET	w,
     return;
     }
 
-static void tedFormatSelectSect(	APP_WIDGET	w,
-					void *		voidst,
-					void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( tedFormatSelectSect, w, voidst )
     {
     SectionTool *		st= (SectionTool *)voidst;
     EditApplication *		ea= st->stApplication;
@@ -326,9 +302,7 @@ static void tedFormatSelectSect(	APP_WIDGET	w,
     return;
     }
 
-static void tedFormatDeleteSect(	APP_WIDGET	w,
-					void *		voidst,
-					void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( tedFormatDeleteSect, w, voidst )
     {
     SectionTool *		st= (SectionTool *)voidst;
     EditApplication *		ea= st->stApplication;
@@ -336,9 +310,7 @@ static void tedFormatDeleteSect(	APP_WIDGET	w,
     tedDeleteCurrentSection( ea );
     }
 
-static void tedFormatInsertSect(	APP_WIDGET	w,
-					void *		voidst,
-					void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( tedFormatInsertSect, w, voidst )
     {
     SectionTool *		st= (SectionTool *)voidst;
     EditApplication *		ea= st->stApplication;
@@ -348,9 +320,7 @@ static void tedFormatInsertSect(	APP_WIDGET	w,
     return;
     }
 
-static void tedFormatAppendSect(	APP_WIDGET	w,
-					void *		voidst,
-					void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( tedFormatAppendSect, w, voidst )
     {
     SectionTool *		st= (SectionTool *)voidst;
     EditApplication *		ea= st->stApplication;
@@ -378,13 +348,16 @@ void tedFormatFillSectionPage(	SectionTool *			st,
 
     const int	textColumns= 10;
 
-    /**************/
+    /**/
 
     st->stUnitType= appUnitTypeInt( spr->sprUnitType );
     if  ( st->stUnitType < 0 )
 	{ SDEB(spr->sprUnitType); st->stUnitType= UNITtyINCH;	}
 
-    /**************/
+    /**/
+    st->stPageResources= spr;
+
+    /**/
     st->stSectionNumber= -1;
 
     docInitSectionProperties( &(st->stPropertiesSet) );
@@ -482,6 +455,20 @@ void tedFormatFinishSectionPage(	SectionTool *			st,
     return;
     }
 
+/************************************************************************/
+/*									*/
+/*  Clean a section tool						*/
+/*									*/
+/************************************************************************/
+
+void tedFormatCleanSectionTool(		SectionTool *		st )
+    {
+    docCleanSectionProperties( &(st->stPropertiesSet) );
+    docCleanSectionProperties( &(st->stPropertiesChosen) );
+
+    return;
+    }
+
 /************************************************************************/
 /*									*/
 /*  Get section tool resources.						*/
