@@ -1,6 +1,6 @@
 /************************************************************************/
 /*									*/
-/*  A Format Tool.							*/
+/*  Table page of the format tool.					*/
 /*									*/
 /************************************************************************/
 
@@ -9,7 +9,6 @@
 #   include	<stdlib.h>
 #   include	<stdio.h>
 #   include	<stddef.h>
-#   include	<appDebugon.h>
 
 #   include	<appGeoString.h>
 #   include	<appUnit.h>
@@ -17,6 +16,8 @@
 #   include	"tedApp.h"
 #   include	"tedFormatTool.h"
 
+#   include	<appDebugon.h>
+
 /************************************************************************/
 /*									*/
 /*  Refresh the 'Table' page of the format tool.			*/
@@ -48,7 +49,7 @@ void tedFormatToolRefreshTableTool(
     const BufferItem *		rowBi;
     const BufferItem *		sectBi;
 
-    if  ( tedGetTableRectangle( &(tt->ttTableRectangle), ds ) )
+    if  ( docGetTableRectangle( &(tt->ttTableRectangle), ds ) )
 	{
 	docInitTableRectangle( &(tt->ttTableRectangle) );
 	*pEnabled= 0; return;
@@ -67,9 +68,11 @@ void tedFormatToolRefreshTableTool(
 
     rp= &(rowBi->biRowProperties);
 
-    if  ( docCopyRowProperties( &(tt->ttPropertiesChosen), rp ) )
+    if  ( docCopyRowProperties( &(tt->ttPropertiesChosen),
+						    rp, (const int *)0 ) )
 	{ LDEB(1); return;	}
-    if  ( docCopyRowProperties( &(tt->ttPropertiesSet), rp ) )
+    if  ( docCopyRowProperties( &(tt->ttPropertiesSet),
+						    rp, (const int *)0 ) )
 	{ LDEB(1); return;	}
 
     tedFormatToolRefreshTablePage( tt );
@@ -84,9 +87,7 @@ void tedFormatToolRefreshTableTool(
 /*									*/
 /************************************************************************/
 
-static void tedTableChangeTablePushed(	APP_WIDGET	w,
-					void *		voidtt,
-					void *		voidpbcs	 )
+static APP_BUTTON_CALLBACK_H( tedTableChangeTablePushed, w, voidtt )
     {
     TableTool *			tt= (TableTool *)voidtt;
     RowProperties *		rp= &(tt->ttPropertiesChosen);
@@ -95,29 +96,37 @@ static void tedTableChangeTablePushed(	APP_WIDGET	w,
     int				width;
     int				res;
 
+    PropertyMask		rpSetMask;
+    PropertyMask		cpSetMask;
+
+    PROPmaskCLEAR( &rpSetMask );
+    PROPmaskCLEAR( &cpSetMask );
+
     res= tedFormatToolGetRowLeftIndent( rp, tt->ttPageLeftMargin,
 					    &width, tt->ttLeftIndentText );
     if  ( res != 0 )
 	{ return;	}
     rp->rpLeftIndentTwips= width;
+    PROPmaskADD( &rpSetMask, RPpropLEFT_INDENT );
 
     res= tedFormatToolGetGapWidth( rp, &width, tt->ttCellMarginText );
     if  ( res != 0 )
 	{ return;	}
     rp->rpHalfGapWidthTwips= width;
+    PROPmaskADD( &rpSetMask, RPpropGAP_WIDTH );
 
-    tedAppSetTableProperties( tt->ttApplication, tr, rp );
+    tedAppSetTableProperties( tt->ttApplication, tr,
+					    &rpSetMask, &cpSetMask, rp );
 
     return;
     }
 
-static void tedFormatRevertTablePushed(	APP_WIDGET	w,
-					void *		voidtt,
-					void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( tedFormatRevertTablePushed, w, voidtt )
     {
     TableTool *	tt= (TableTool *)voidtt;
 
-    docCopyRowProperties( &(tt->ttPropertiesChosen), &(tt->ttPropertiesSet) );
+    docCopyRowProperties( &(tt->ttPropertiesChosen),
+				    &(tt->ttPropertiesSet), (const int *)0 );
 
     tedFormatToolRefreshTablePage( tt );
 
@@ -131,9 +140,7 @@ static void tedFormatRevertTablePushed(	APP_WIDGET	w,
 /*									*/
 /************************************************************************/
 
-static void tedTableCellMarginChanged(	APP_WIDGET	w,
-					void *		voidtt,
-					void *		voidpbcs )
+static APP_TXACTIVATE_CALLBACK_H( tedTableCellMarginChanged, w, voidtt )
     {
     TableTool *		tt= (TableTool *)voidtt;
     RowProperties *	rp= &(tt->ttPropertiesChosen);
@@ -156,9 +163,7 @@ static void tedTableCellMarginChanged(	APP_WIDGET	w,
 /*									*/
 /************************************************************************/
 
-static void tedTableLeftIndentChanged(	APP_WIDGET	w,
-					void *		voidtt,
-					void *		voidpbcs )
+static APP_TXACTIVATE_CALLBACK_H( tedTableLeftIndentChanged, w, voidtt )
     {
     TableTool *		tt= (TableTool *)voidtt;
     RowProperties *	rp= &(tt->ttPropertiesChosen);
@@ -181,9 +186,7 @@ static void tedTableLeftIndentChanged(	APP_WIDGET	w,
 /*									*/
 /************************************************************************/
 
-static void tedTableDeleteTable(	APP_WIDGET	w,
-					void *		voidtt,
-					void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( tedTableDeleteTable, w, voidtt )
     {
     TableTool *			tt= (TableTool *)voidtt;
     EditApplication *		ea= tt->ttApplication;
@@ -194,14 +197,10 @@ static void tedTableDeleteTable(	APP_WIDGET	w,
     if  ( tedDeleteRowsFromTable( ed, tr->trRow00, tr->trRow11 ) )
 	{ LLDEB(tr->trRow00,tr->trRow11); return;	}
 
-    appDocumentChanged( ed, 1 );
-
     return;
     }
 
-static void tedTableSelectTable(	APP_WIDGET	w,
-					void *		voidtt,
-					void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( tedTableSelectTable, w, voidtt )
     {
     TableTool *			tt= (TableTool *)voidtt;
     EditApplication *		ea= tt->ttApplication;
@@ -277,6 +276,20 @@ void tedFormatFillTablePage(	TableTool *			tt,
     return;
     }
 
+/************************************************************************/
+/*									*/
+/*  Clean the table tool.						*/
+/*									*/
+/************************************************************************/
+
+void tedFormatCleanTableTool(	TableTool *	tt )
+    {
+    docCleanRowProperties( &(tt->ttPropertiesSet) );
+    docCleanRowProperties( &(tt->ttPropertiesChosen) );
+
+    return;
+    }
+
 /************************************************************************/
 /*									*/
 /*  Get Table tool resources.						*/
