@@ -29,6 +29,7 @@ typedef struct TableTool
     {
     EditApplication *		ttApplication;
     AppInspector *		ttInspector;
+    const TablePageResources *	ttPageResources;
 
     RowProperties		ttPropertiesSet;
     RowProperties		ttPropertiesChosen;
@@ -48,4 +49,23 @@ typedef struct TableTool
 /*									*/
 /************************************************************************/
 
+extern void tedFormatFillTablePage( TableTool *			tt,
+				const TablePageResources *	tpr,
+				InspectorSubject *		is,
+				APP_WIDGET			pageWidget,
+				const InspectorSubjectResources * isr );
+
+extern void tedFormatToolGetTableResourceTable(
+					EditApplication *		ea,
+					TablePageResources *		tpr,
+					InspectorSubjectResources *	isr );
+
+extern void tedFormatToolRefreshTableTool(
+				TableTool *			tt,
+				int *				pEnabled,
+				InspectorSubject *		is,
+				const DocumentSelection *	bs );
+
+extern void tedFormatCleanTableTool(	TableTool *		tt );
+
 #   endif	/*  TED_TABLE_TOOL_H */
