@@ -3,7 +3,9 @@
 
 #   include	<appGuiBase.h>
 
-#   include	<appGeo.h>
+#   include	<utilPropMask.h>
+
+#   include	<geo2DInteger.h>
 #   include	<docFont.h>
 #   include	<psFont.h>
 #   include	<bmcolor.h>
@@ -45,27 +47,34 @@ typedef struct AppPhysicalFont
     {
     TextAttribute	apfAttribute;
     int			apfDocFamilyNumber;
+    int			apfFontEncoding;
 
     int			apfPsFamilyNumber;
     int			apfPsFaceNumber;
 
     APP_FONT *		apfFontStruct;
+    APP_FONT *		apfScapsFontStruct;
     unsigned long	apfUnderLinePositionPixels;
     unsigned long	apfUnderlineThicknessPixels;
     int			apfAscentPixels;
     int			apfDescentPixels;
     int			apfFullSizePixels;
+    unsigned long	apfXHeightPixels;
     } AppPhysicalFont;
 
 typedef struct AppPhysicalFontList
     {
     const char *	apflAfmDirectory;
+    const char *	apflGhostscriptFontmap;
+    const char *	apflGhostscriptFontToXmapping;
     int			apflCount;
     AppPhysicalFont *	apflFonts;
     } AppPhysicalFontList;
 
 typedef struct AppDrawingData
     {
+    int				addForScreenDrawing;
+
     DocumentRectangle		addBackRect;
     DocumentRectangle		addPaperRect;
     DocumentRectangle		addDocRect;
@@ -139,12 +148,25 @@ typedef struct AppColors
 #   endif
     } AppColors;
 
+#   ifdef USE_MOTIF
+
+extern const char * const APP_VisualClasses[];
+
+#   endif
+
 /************************************************************************/
 /*									*/
 /*  Procedure declarations.						*/
 /*									*/
 /************************************************************************/
 
+#   ifdef USE_GTK
+
+extern void appDrawGtkSetXFillRule(	GdkGC *			gc,
+					int			rule );
+
+#   endif
+
 extern void appCollectExposures(	DocumentRectangle *	drClip,
 					AppDrawingData *	add,
 					int			ox,
@@ -159,11 +181,6 @@ extern void appInitFontList(	AppPhysicalFontList *	apfl );
 extern void appCleanFontList(	AppDrawingData *	add,
 				AppPhysicalFontList *	apfl );
 
-extern void appEnableFontsTool(	void *	voidafc,
-				int	enabled );
-
-extern void appShowFontsTool(		void *	voidefc	);
-
 extern int appFontCatalog(	AppDrawingData *	add,
 				AppFontFamily **	pFamilies,
 				int *			pCount		);
@@ -177,40 +194,17 @@ extern int appOpenDocumentFont(	AppDrawingData *		add,
 				const DocumentFontList *	dfl,
 				TextAttribute			ta );
 
-extern int appFontSetCurrentFont(	void *		voidefc,
-					double		xfac,
-					int		fontFamilyNumber,
-					unsigned int	updMask,
-					TextAttribute	taNew );
-
-extern void appFontAttributeString(	char *			target,
-					const char *		familyName,
-					unsigned int		updMask,
-					TextAttribute		taNew );
-
-extern int appGetPsFont(	int *			pFamily,
-				int *			pFace,
-				AppFontFamily **	pAff,
-				AppFontTypeface **	pAft,
-				const char *		afmDirectory,
-				const DocumentFont *	df,
-				TextAttribute		ta );
-
 extern int appFontXFont(	char *			target,
 				AppDrawingData *	add,
 				const AppFontFamily *	aff,
-				const AppFontTypeface *	aft,
+				int			encoding,
+				AppFontTypeface *	aft,
 				int			twipsSize,
 				int			variant );
 
 extern int appCharExistsInFont(	const APP_FONT *	xfs,
 				int			ch );
 
-extern int appFontGetAttributes(	char *		attributeString,
-					char **		pFamilyName,
-					unsigned int *	pUpdMask,
-					TextAttribute *	pTaNew );
-
 extern void appInitVerticalXFont(	VerticalXFont *	vxf );
 
 extern void appCleanVerticalXFont(	VerticalXFont *		vxf,
@@ -229,12 +223,15 @@ extern int appDrawVerticalString(	VerticalXFont *		vxf,
 					char *			string,
 					int			length );
 
-extern void appSetDrawingEnvironment(	AppDrawingData *	add,
-					double			magnification,
-					double			xfac,
-					double			screenPixPerMM,
-					const char *		afmDirectory,
-					APP_WIDGET		appWidget );
+extern void appSetDrawingEnvironment(
+			AppDrawingData *	add,
+			double			magnification,
+			double			xfac,
+			double			screenPixPerMM,
+			const char *		afmDirectory,
+			const char *		apflGhostscriptFontmap,
+			const char *		ghostscriptFontToXmapping,
+			APP_WIDGET		appWidget );
 
 extern void appExposeRectangle(		const AppDrawingData *	add,
 					int			x,
@@ -350,13 +347,15 @@ extern void appDrawDrawPixmap(		AppDrawingData *	add,
 					int			xDest,
 					int			yDest );
 
-extern int appGetCoordinatesFromMouseButtonEvent( int *		pX,
-						int *		pY,
-						int *		pButton,
-						int *		pUpDown,
-						int *		pSeq,
-						APP_WIDGET	w,
-						APP_EVENT *	event );
+extern int appGetCoordinatesFromMouseButtonEvent(
+					int *			pX,
+					int *			pY,
+					int *			pButton,
+					int *			pUpDown,
+					int *			pSeq,
+					unsigned int *		pKeyState,
+					const APP_WIDGET	w,
+					const APP_EVENT *	event );
 
 extern int appGetCoordinatesFromMouseMoveEvent( int *		pX,
 						int *		pY,
@@ -410,19 +409,19 @@ extern void appDrawFillArc(	AppDrawingData *	add,
 				int			angle2 );
 
 extern void appDrawSetRedrawHandler(	APP_WIDGET		w,
-					APP_EVENT_HANDLER	handler,
+					APP_EVENT_HANDLER_T	handler,
 					void *			through );
 
 extern void appDrawSetConfigureHandler(	APP_WIDGET		w,
-					APP_EVENT_HANDLER	handler,
+					APP_EVENT_HANDLER_T	handler,
 					void *			through );
 
 extern void appDrawSetButtonPressHandler( APP_WIDGET		w,
-					APP_EVENT_HANDLER	handler,
+					APP_EVENT_HANDLER_T	handler,
 					void *			through );
 
 extern void appDrawSetKeyboardHandler( APP_WIDGET		w,
-					APP_EVENT_HANDLER	handler,
+					APP_EVENT_HANDLER_T	handler,
 					void *			through );
 
 extern int appDrawGetInoutFromFocusEvent(	int *		pInOut,
