@@ -162,14 +162,11 @@ static void appChooserSetFileExtension(	char *	filename,
 					char *	extension,
 					Widget	text )
     {
-    char *	slash= strrchr( filename, '/' );
-    char *	dot;
+    const char *	ext;
 
-    if  ( slash )
-	{ dot= strrchr( slash+ 1, '.' );	}
-    else{ dot= strrchr( filename, '.' );	}
+    ext= appFileExtensionOfName( filename );
 
-    if  ( dot )
+    if  ( ext )
 	{
 	int	l;
 	char *	s;
@@ -177,7 +174,7 @@ static void appChooserSetFileExtension(	char *	filename,
 	l= strlen( filename )+ strlen( extension )+ 2;
 	s= (char *)malloc( l );
 	strcpy( s, filename );
-	strcpy( s+ ( dot- filename )+ 1, extension );
+	strcpy( s+ ( ext- filename ), extension );
 
 	appChooserSetText( text, s );
 
@@ -221,6 +218,78 @@ static void appOpenChooserOk(	Widget		fileChooser,
     return;
     }
 
+/************************************************************************/
+/*									*/
+/*  The file selection box was toold to filter via the normal motif	*/
+/*  mechanisms: Adapt the OptionMenu with extensions to the filter.	*/
+/*									*/
+/************************************************************************/
+
+static void appFileFilterActivated(	Widget		w,
+					XtPointer	voidaci,
+					XtPointer	voidpbcs )
+    {
+    AppChooserInformation *	aci= (AppChooserInformation *)voidaci;
+
+    Widget text= XmFileSelectionBoxGetChild( aci->aciDialog.adDialog,
+							XmDIALOG_FILTER_TEXT );
+
+    char *		filter= XmTextGetString( text );
+    const char *	extS;
+
+
+    if  ( ! filter )
+	{ return;	}
+
+    extS= appFileExtensionOfName( filter );
+    if  ( extS )
+	{
+	int				i;
+	const AppFileExtension *	afe= aci->aciExtensions;
+
+	Widget				current= (Widget)0;
+	WidgetList			children;
+	Cardinal			childCount= 0;
+
+	XtVaGetValues( aci->aciFilterOptionmenu.aomPulldown,
+			XmNmenuHistory,		&current,
+			XmNchildren,		&children,
+			XmNnumChildren,		&childCount,
+			NULL );
+
+	for ( i= 0; i < childCount; i++ )
+	    {
+	    if  ( children[i] == current )
+		{ break;	}
+	    }
+
+	if  ( i >= childCount				||
+	      i >= aci->aciExtensionCount		||
+	      strcmp( extS, afe[i].afeExtension )	)
+	    {
+	    for ( i= 0; i < aci->aciExtensionCount; afe++, i++ )
+		{
+		if  ( i >= childCount )
+		    { continue;	}
+		if  ( ! XtIsManaged( children[i] ) )
+		    { continue;	}
+		if  ( ! XtIsSensitive( children[i] ) )
+		    { continue;	}
+
+		if  ( ! strcmp( extS, afe->afeExtension )	)
+		    { break;	}
+		}
+
+	    if  ( i < aci->aciExtensionCount )
+		{ appSetOptionmenu( &(aci->aciFilterOptionmenu), i );	}
+	    }
+	}
+
+    XtFree( filter );
+
+    return;
+    }
+
 /************************************************************************/
 /*									*/
 /*  Add a pulldown with filters to a file chooser.			*/
@@ -229,7 +298,7 @@ static void appOpenChooserOk(	Widget		fileChooser,
 
 static void appFileFilterChosen(	Widget		w,
 					XtPointer	voidaci,
-					XtPointer	voidpbcs	 )
+					XtPointer	voidpbcs )
     {
     AppChooserInformation *	aci= (AppChooserInformation *)voidaci;
     const AppFileExtension *	afe= aci->aciExtensions;
@@ -262,18 +331,15 @@ static void appFileFilterChosen(	Widget		w,
 
     if  ( fileSelected )
 	{
-	char *	slash= strrchr( fileSelected, '/' );
-	char *	dot;
+	const char *	extS;
 
-	if  ( slash )
-	    { dot= strrchr( slash+ 1, '.' );		}
-	else{ dot= strrchr( fileSelected, '.' );	}
+	extS= appFileExtensionOfName( fileSelected );
 
-	if  ( dot							&&
+	if  ( extS							&&
 	      previousExtension >= 0					&&
 	      afe[previousExtension].afeExtension			&&
 	      afe[extensionChosen].afeExtension				&&
-	      ! strcmp( dot+ 1, afe[previousExtension].afeExtension )	)
+	      ! strcmp( extS, afe[previousExtension].afeExtension )	)
 	    {
 	    char *	ext= afe[extensionChosen].afeExtension;
 
@@ -496,6 +562,7 @@ static int appMakeFileChooser( AppChooserInformation **	pAci,
     XmString			filterString= (XmString)0;
 
     Widget			helpButton;
+    Widget			filterButton;
     AppChooserInformation *	aci;
 
     MwmHints			hints;
@@ -526,7 +593,7 @@ static int appMakeFileChooser( AppChooserInformation **	pAci,
     XtSetArg( al[ac], XmNwmTimeout,		0 ); ac++; /* LessTif BUG */
 
     XtSetArg( al[ac], XmNchildPlacement,XmPLACE_ABOVE_SELECTION ); ac++;
-#   if	XmVersion >= 2000
+#   if	XmVersion >= 2000 && ! defined( LESSTIF_VERSION )
     XtSetArg( al[ac], XmNfileFilterStyle,
 					XmFILTER_HIDDEN_FILES ); ac++;
 #   endif
@@ -556,6 +623,17 @@ static int appMakeFileChooser( AppChooserInformation **	pAci,
 			XmNforeground,	BlackPixel( display, screen ),
 			NULL );
 
+    /*  Not necessary
+    XtAddCallback( text, XmNactivateCallback,
+				    appFileFilterActivated, (void *)aci );
+    */
+
+    filterButton= XmFileSelectionBoxGetChild( aci->aciDialog.adDialog,
+						    XmDIALOG_APPLY_BUTTON );
+
+    XtAddCallback( filterButton, XmNactivateCallback,
+				    appFileFilterActivated, (void *)aci );
+
     text= XmFileSelectionBoxGetChild( aci->aciDialog.adDialog, XmDIALOG_TEXT );
     XtVaSetValues( text,
 			XmNbackground,	WhitePixel( display, screen ),
@@ -624,7 +702,7 @@ void appRunOpenChooser( Widget			option,
 			char *			defaultFilter,
 			void *			through,
 			APP_OPEN_DOCUMENT	openDocument,
-			EditApplication *	ea	)
+			EditApplication *	ea )
     {
     AppChooserInformation *	aci= (AppChooserInformation *)0;
 
@@ -782,11 +860,8 @@ static int appSaveChooserFilterExtensions( EditApplication *	ea,
     afe= ea->eaFileExtensions;
     for ( i= 0; i < (int)childCount; afe++, i++ )
 	{
-	if  ( ( afe->afeUseFlags & useFlags ) != useFlags )
-	    { XtUnmanageChild( children[i] ); continue;	}
-
-	if  ( ea->eaCanSaveDocument				&&
-	      (*ea->eaCanSaveDocument)( ed->edPrivateData, i )	)
+	if  ( appDocumentTestCanSave( ea, afe, ed->edPrivateData,
+							useFlags, i ) )
 	    { XtUnmanageChild( children[i] ); continue;	}
 
 	XtManageChild( children[i] );
@@ -836,8 +911,8 @@ static int appSaveChooserFilterExtensions( EditApplication *	ea,
     return 0;
     }
 
-static int appRunSaveChooser(	Widget			option,
-				Widget			relative,
+int appRunSaveChooser(		APP_WIDGET		option,
+				APP_WIDGET		relative,
 				EditApplication *	ea,
 				const EditDocument *	ed,
 				unsigned int		useFlags,
@@ -889,95 +964,6 @@ static int appRunSaveChooser(	Widget			option,
     return response;
     }
 
-/************************************************************************/
-/*									*/
-/*  Callback for the file menu.						*/
-/*									*/
-/*  1)  'call_data' can be NULL.					*/
-/*									*/
-/************************************************************************/
-
-void appDocFileSaveAs(	APP_WIDGET	option,
-			void *		voided,
-			void *		call_data )
-    {
-    EditDocument *	ed= (EditDocument *)voided;
-    EditApplication *	ea= ed->edApplication;
-    const int		interactive= 1;
-
-    int			response;
-    int			format;
-    char *		filename;
-
-    if  ( ! ea->eaSaveDocument )
-	{ XDEB(ea->eaSaveDocument); return;	}
-
-    response= appRunSaveChooser( option, ed->edToplevel.atTopWidget, ea, ed,
-					( APPFILE_CAN_SAVE|APPFILE_CAN_OPEN ),
-		&format, &filename );
-
-    switch( response )
-	{
-	case ACIrespCANCEL:
-	    break;
-	case ACIrespSAVE:
-	    if  ( appDocSaveDocumentByName( ed, option, interactive,
-							format, filename ) )
-		{ free( filename ); return;	}
-
-	    appDocumentChanged( ed, 0 );
-
-	    if  ( appSetDocumentFilename( ed, filename ) )
-		{ SDEB(filename);	}
-	    if  ( appSetDocumentTitle( ed, filename ) )
-		{ SDEB(filename);	}
-
-	    ed->edFileReadOnly= 0;
-	    ed->edFormat= format;
-
-	    free( filename );
-	    break;
-	default:
-	    LDEB(response); break;
-	}
-    }
-
-void appDocFileSaveTo(	APP_WIDGET	option,
-			void *		voided,
-			void *		call_data )
-    {
-    EditDocument *	ed= (EditDocument *)voided;
-    EditApplication *	ea= ed->edApplication;
-
-    const int		interactive= 1;
-    int			response;
-    int			format;
-    char *		filename;
-
-    if  ( ! ea->eaSaveDocument )
-	{ XDEB(ea->eaSaveDocument); return;	}
-
-    response= appRunSaveChooser( option, ed->edToplevel.atTopWidget, ea, ed,
-				    APPFILE_CAN_SAVE, &format, &filename );
-
-    switch( response )
-	{
-	case ACIrespCANCEL:
-	    break;
-	case ACIrespSAVE:
-	    if  ( appDocSaveDocumentByName( ed, option, interactive,
-							format, filename ) )
-		{ free( filename ); return; }
-
-	    appDocumentChanged( ed, 0 );
-
-	    free( filename );
-	    break;
-	default:
-	    LDEB(response); break;
-	}
-    }
-
 /************************************************************************/
 /*									*/
 /*  Run Filechooser for print to file.					*/
