@@ -4,12 +4,11 @@
 /*									*/
 /************************************************************************/
 
-#   include	"config.h"
+#   include	"appFrameConfig.h"
 
 #   include	<stdlib.h>
 #   include	<stdio.h>
 #   include	<stddef.h>
-#   include	<appDebugon.h>
 
 #   include	<reg.h>
 
@@ -19,6 +18,8 @@
 #   include	<X11/keysym.h>
 #   endif
 
+#   include	<appDebugon.h>
+
 /************************************************************************/
 /*									*/
 /*  Represents a find tool.						*/
@@ -136,9 +137,7 @@ static void appFindToolSetFindResult(	AppFindTool *	aft,
     return;
     }
 
-static void appFindToolPatternChanged(	APP_WIDGET	w,
-					void *		voidaft,
-					void *		call_data )
+static APP_TXTYPING_CALLBACK_H( appFindToolPatternChanged, w, voidaft )
     {
     AppFindTool *	aft= (AppFindTool *)voidaft;
 
@@ -147,9 +146,7 @@ static void appFindToolPatternChanged(	APP_WIDGET	w,
     return;
     }
 
-static void appFindToolReplacementChanged(	APP_WIDGET	w,
-						void *		voidaft,
-						void *		call_data )
+static APP_TXTYPING_CALLBACK_H( appFindToolReplacementChanged, w, voidaft )
     {
     AppFindTool *	aft= (AppFindTool *)voidaft;
 
@@ -195,11 +192,8 @@ static int appFindToolGetProgram(	regProg **	pProg,
     *pProg= prog; return 0;
     }
 
-static void appFindToolFindNext(	APP_WIDGET	w,
-					void *		voidaft,
-					void *		voidpbcs )
+static void appFindToolFindNext(	AppFindTool *	aft )
     {
-    AppFindTool *	aft= (AppFindTool *)voidaft;
     regProg *		prog;
     int			result;
 
@@ -220,9 +214,16 @@ static void appFindToolFindNext(	APP_WIDGET	w,
     return;
     }
 
-static void appFindToolFindPrev(	APP_WIDGET	w,
-					void *		voidaft,
-					void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( appFindToolFindNextPushed, w, voidaft )
+    {
+    AppFindTool *	aft= (AppFindTool *)voidaft;
+
+    appFindToolFindNext( aft );
+
+    return;
+    }
+
+static APP_BUTTON_CALLBACK_H( appFindToolFindPrev, w, voidaft )
     {
     AppFindTool *	aft= (AppFindTool *)voidaft;
     regProg *		prog;
@@ -251,11 +252,8 @@ static void appFindToolFindPrev(	APP_WIDGET	w,
 /*									*/
 /************************************************************************/
 
-static void appFindToolReplaceSelection(	APP_WIDGET	w,
-						void *		voidaft,
-						void *		voidpbcs )
+static void appFindToolReplaceSelection( AppFindTool *	aft )
     {
-    AppFindTool *	aft= (AppFindTool *)voidaft;
     char *		replacement;
 
     if  ( ! aft->aftReplace )
@@ -272,20 +270,25 @@ static void appFindToolReplaceSelection(	APP_WIDGET	w,
     return;
     }
 
-static void appFindToolReplaceNext(	APP_WIDGET	w,
-					void *		voidaft,
-					void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( appFindToolReplaceSelectionPushed, w, voidaft )
     {
-    appFindToolReplaceSelection( w, voidaft, voidpbcs );
+    AppFindTool *	aft= (AppFindTool *)voidaft;
 
-    appFindToolFindNext( w, voidaft, voidpbcs );
+    appFindToolReplaceSelection( aft );
+    }
+
+static APP_BUTTON_CALLBACK_H( appFindToolReplaceNext, w, voidaft )
+    {
+    AppFindTool *	aft= (AppFindTool *)voidaft;
+
+    appFindToolReplaceSelection( aft );
+
+    appFindToolFindNext( aft );
 
     return;
     }
 
-static void appFindRegexToggled(	APP_WIDGET	w,
-					void *		voidaft,
-					void *		voidtbcs )
+static APP_TOGGLE_CALLBACK_H( appFindRegexToggled, w, voidaft, voidtbcs )
     {
     AppFindTool *	aft= (AppFindTool *)voidaft;
     int			set;
@@ -307,8 +310,8 @@ static void appFindToolMakeButtonRow(	APP_WIDGET *		pLeftButton,
 					APP_WIDGET *		pRightButton,
 					char *			leftLabel,
 					char *			rightLabel,
-					APP_BUTTON_CALLBACK	leftCallback,
-					APP_BUTTON_CALLBACK	rightCallback,
+					APP_BUTTON_CALLBACK_T	leftCallback,
+					APP_BUTTON_CALLBACK_T	rightCallback,
 					AppFindTool *		aft,
 					APP_WIDGET		parent )
 
@@ -380,7 +383,7 @@ static APP_WIDGET appFindMakeFindFrame(	APP_WIDGET		parent,
     appFindToolMakeButtonRow(
 		&(aft->aftFindNextButton), &(aft->aftFindPrevButton),
 		aftr->aftrFindNext, aftr->aftrFindPrevious,
-		appFindToolFindNext, appFindToolFindPrev,
+		appFindToolFindNextPushed, appFindToolFindPrev,
 		aft, paned );
 
     /***************/
@@ -423,7 +426,7 @@ static APP_WIDGET appFindMakeReplaceFrame( APP_WIDGET		parent,
     appFindToolMakeButtonRow(
 		&(aft->aftReplaceButton), &(aft->aftReplaceNextButton),
 		aftr->aftrReplaceFound, aftr->aftrReplaceNext,
-		appFindToolReplaceSelection, appFindToolReplaceNext,
+		appFindToolReplaceSelectionPushed, appFindToolReplaceNext,
 		aft, paned );
 
     /***************/
@@ -445,7 +448,7 @@ static void appDestroyFindTool(	AppFindTool *	aft )
     free( aft );
     }
 
-static APP_CLOSE_CALLBACK( appCloseFindTool, w, voidaft )
+static APP_CLOSE_CALLBACK_H( appCloseFindTool, w, voidaft )
     {
     AppFindTool *	aft= (AppFindTool *)voidaft;
 
@@ -474,8 +477,11 @@ static void appFindToolCancel(		APP_WIDGET		w,
 #   endif
 
 /************************************************************************/
+/*									*/
 /*  make a find tool.							*/
+/*									*/
 /************************************************************************/
+
 #   ifdef USE_MOTIF
 static void appFindToolConfigure(	APP_WIDGET		w,
 					void *			voidaft,
