@@ -12,19 +12,14 @@
 #   include	<appGuiBase.h>
 #   include	<appGuiResource.h>
 
-#   include	"appGeo.h"
+#   include	"geo2DInteger.h"
+#   include	"utilPs.h"
 #   include	"utilPrinter.h"
 #   include	"utilMailContent.h"
 #   include	<appIcons.h>
 #   include	<appTool.h>
 #   include	<appDraw.h>
-
-#   if 1
-#	define	WIDGET_NAME	""
-#   else
-#	define	WIDGET_NAME	appWidgetName( __FILE__, __LINE__ )
-#   endif
-
+#   include	<appInspector.h>
 
 /************************************************************************/
 /*									*/
@@ -91,7 +86,9 @@ typedef struct AppMenuItem
     } AppMenuItem;
 
 /************************************************************************/
+/*									*/
 /*  Kinds of files that an application can open.			*/
+/*									*/
 /************************************************************************/
 
 #   define	APPFILE_CAN_OPEN	0x01
@@ -108,6 +105,23 @@ typedef struct AppFileExtension
     unsigned int	afeUseFlags;
     } AppFileExtension;
 
+/************************************************************************/
+/*									*/
+/*  Printing..								*/
+/*									*/
+/************************************************************************/
+
+typedef struct PrintJob
+    {
+    void *			pjPrivateData;
+    int				pjFormat;
+    int				pjUsePostScriptFilters;
+    int				pjUsePostScriptIndexedImages;
+    const char *		pjTitle;
+    AppDrawingData *		pjDrawingData;
+    struct EditApplication *	pjApplication;
+    } PrintJob;
+
 /************************************************************************/
 /*									*/
 /*  Describes a document generically.					*/
@@ -175,7 +189,6 @@ typedef struct EditDocument
 
     AppDrawingData		edDrawingData;
     DocumentRectangle		edVisibleRect;
-    double			edPixelsPerMM;
 
     void *			edPrivateData;
 
@@ -310,15 +323,9 @@ typedef struct EditApplication
 				int				format,
 				AppDrawingData *		add );
     int			(*eaPrintDocument)(
-				FILE *				f,
-				const void *			privateData,
-				int				format,
-				AppDrawingData *		add,
-				const char *			title,
-				struct EditApplication *	ea,
-				const DocumentGeometry *	dgPaper,
-				int				nup,
-				int				horizontal,
+				SimpleOutputStream *		sos,
+				const PrintJob *		pj,
+				const PrintGeometry *		pg,
 				int				firstPage,
 				int				lastPage );
     void		(*eaDrawRectangle)(
@@ -345,12 +352,19 @@ typedef struct EditApplication
     void		(*eaDocCopy)( EditDocument *);
     void		(*eaDocCut)( EditDocument *);
     void		(*eaDocSelAll)( EditDocument * );
+
+    void		(*eaSetPageLayout)(
+				    struct EditApplication *	ea,
+				    const DocumentGeometry *	dgNew,
+				    const PropertyMask *	setMask,
+				    int				wholeDocument );
+
 			/************************************************/
 			/*  User input on the document widget:		*/
 			/************************************************/
-    APP_EVENT_HANDLER	eaDocumentMouseHandler;
-    APP_EVENT_HANDLER	eaDocumentKeyboardHandler;
-    APP_EVENT_HANDLER	eaObserveFocus;
+    APP_EVENT_HANDLER_T	eaDocumentMouseHandler;
+    APP_EVENT_HANDLER_T	eaDocumentKeyboardHandler;
+    APP_EVENT_HANDLER_T	eaObserveFocus;
 
     APP_SCROLLBAR_CALLBACK	eaDocHorizontalScrollbarCallback;
     APP_SCROLLBAR_CALLBACK	eaDocVerticalScrollbarCallback;
@@ -419,11 +433,11 @@ typedef struct EditApplication
     APP_WIDGET		eaHelpMenu;
     APP_WIDGET		eaHelpMenuButton;
 
-    void *		eaFontsTool;
     void *		eaSpellTool;
     void *		eaFindTool;
     void *		eaPageTool;
     void *		eaSymbolPicker;
+    void *		eaPrintDialog;
 
     int			eaArgc;
     char **		eaArgv;
@@ -435,6 +449,9 @@ typedef struct EditApplication
     int			eaMainVisibleAsAbout;
 
     char *		eaAfmDirectory;
+    char *		eaGhostscriptFontmap;
+    char *		eaGhostscriptFontToXmapping;
+    int			eaGhostscriptMappingsRead;
     char *		eaDefaultFont;
     char *		eaRulerFont;
     char *		eaPrintDialogFont;
@@ -445,9 +462,18 @@ typedef struct EditApplication
     char *		eaPageNumberFormat;
     char *		eaFocusColor;
 
+    int				eaSupportXvCopyPaste;
+    int				eaHideSaveToOption;
+    int				eaUsePostScriptFilters;
+    int				eaUsePostScriptIndexedImages;
+
     int				eaGotPaste;
     AppFileMessageResources	eaFileMessageResources;
     char *			eaMagnificationString;
+    char *			eaSupportXvCopyPasteString;
+    char *			eaHideSaveToOptionString;
+    char *			eaUsePostScriptFiltersString;
+    char *			eaUsePostScriptIndexedImagesString;
     } EditApplication;
 
 typedef int (*APP_OPEN_DOCUMENT)(	void *		through,
@@ -507,71 +533,6 @@ typedef struct AppSelectionType
 #   define	AQDrespCANCEL		3
 #   define	AQDrespOK		4
 
-typedef void (*PageToolSetProperties) (	void *				voidapt,
-					const DocumentGeometry *	dg );
-
-typedef void (*FontChooserSetFont)( void * target,
-	char * familyText, int familyNumber, int bold, int slant, int size );
-
-/************************************************************************/
-/*									*/
-/*  Facilities to make property inspectors.				*/
-/*									*/
-/************************************************************************/
-
-typedef struct InspectorSubject
-    {
-    APP_WIDGET	isPage;
-    APP_WIDGET	isMenuitem;
-    void *	isPrivate;
-    int		isEnabled;
-
-    APP_WIDGET	isPrevButton;
-    APP_WIDGET	isNextButton;
-
-    APP_WIDGET	isSelectButton;
-    APP_WIDGET	isDeleteButton;
-
-    APP_WIDGET	isInsertButton;
-    APP_WIDGET	isAppendButton;
-
-    APP_WIDGET	isRevertButton;
-    APP_WIDGET	isApplyButton;
-    } InspectorSubject;
-
-typedef struct AppInspector
-    {
-    APP_WIDGET			aiTopWidget;
-    APP_WIDGET			aiPaned;
-    APP_WIDGET			aiSeparator;
-    APP_WIDGET			aiPageParent;
-
-    AppOptionmenu		aiSubjectOptionmenu;
-
-    AppToolDestroy		aiDestroy;
-    void *			aiTarget;
-
-    int				aiSubjectCount;
-    int				aiCurrentSubject;
-    InspectorSubject		aiSubjects[1];		/*  LAST !!	*/
-    } AppInspector;
-
-typedef struct InspectorSubjectResources
-    {
-    char *		isrSubjectName;
-    char *		isrApplyToSubject;
-    char *		isrRevert;
-
-    char *		isrNextButtonText;
-    char *		isrPrevButtonText;
-
-    char *		isrSelectButtonText;
-    char *		isrDeleteButtonText;
-
-    char *		isrInsertButtonText;
-    char *		isrAppendButtonText;
-    } InspectorSubjectResources;
-
 /************************************************************************/
 /*									*/
 /*  Routine declarations.						*/
@@ -591,7 +552,7 @@ extern APP_WIDGET appMakeMenu(		APP_WIDGET *		pButton,
 extern EditDocument * appOpenDocument(	EditApplication *	ea,
 					APP_WIDGET		relative,
 					APP_WIDGET		option,
-					int			readonly,
+					int			read_only,
 					const char *		filename );
 
 extern int appNewDocument(	EditApplication *	ea,
@@ -648,11 +609,10 @@ extern void appAppFileNew(	APP_WIDGET	option,
 
 extern void appDocSetScrollbarValues(	EditDocument *	ed );
 
-extern void appDocFileSaveAs(	APP_WIDGET	option,
-				void *		voided,
-				void *		call_data );
+extern void appMouseWheelUp(		EditDocument *	ed );
+extern void appMouseWheelDown(		EditDocument *	ed );
 
-extern void appDocFileSaveTo(	APP_WIDGET	option,
+extern void appDocFileSaveAs(	APP_WIDGET	option,
 				void *		voided,
 				void *		call_data );
 
@@ -710,8 +670,8 @@ extern void appDocEditSelAll(	APP_WIDGET	option,
 				void *		voided,
 				void *		voidpbcs );
 
-extern APP_CLOSE_CALLBACK( appDocFileCloseCallback, w, voided );
-extern APP_CLOSE_CALLBACK( appAppWmClose, w, voidea );
+extern APP_CLOSE_CALLBACK_H( appDocFileCloseCallback, w, voided );
+extern APP_CLOSE_CALLBACK_H( appAppWmClose, w, voidea );
 
 extern void appRunReallyCloseDialog(	APP_WIDGET		option,
 					EditDocument *		ed );
@@ -753,7 +713,7 @@ extern int appFinishDocumentSetup(	EditApplication *	ea,
 
 extern int appMakeDocumentWindow(	EditDocument **		pEd,
 					EditApplication *	ea,
-					int			readonly,
+					int			read_only,
 					const char *		title,
 					const char *		filename );
 
@@ -817,8 +777,7 @@ extern int appQuestionRunSubjectYesNoDialog( EditApplication *	ea,
 
 extern void * appMakePageTool(	EditApplication *	ea,
 				APP_WIDGET		printOption,
-				const char *		pixmapName,
-				PageToolSetProperties	setProperties );
+				const char *		pixmapName );
 
 extern void appShowPageTool(		EditApplication *	ea );
 
@@ -843,28 +802,22 @@ extern int appRunMailDialog(		EditApplication *	ea,
 					const unsigned char *	subject,
 					void *			voiddoc );
 
-extern int appPrintDocument(	EditDocument *			ed,
-				int				printer,
-				const DocumentGeometry *	dgPaper,
-				int				nup,
-				int				horizontal,
+extern int appPrintDocument(	int				printer,
+				const PrintJob *		pj,
+				const PrintGeometry *		pg,
 				int				firstPage,
 				int				lastPage );
 
 extern int appFaxDocument(	EditDocument *			ed,
 				const char *			faxNumber,
-				const DocumentGeometry *	dgPaper,
-				int				nup,
-				int				horizontal,
+				const PrintGeometry *		pg,
 				int				firstPage,
 				int				lastPage );
 
 extern void appDocPrintToFile(	APP_WIDGET			option,
 				APP_WIDGET			panel,
 				EditDocument *			ed,
-				const DocumentGeometry *	dgPaper,
-				int				nup,
-				int				horizontal,
+				const PrintGeometry *		pg,
 				int				firstPage,
 				int				lastPage );
 
@@ -881,8 +834,8 @@ extern void appRunOpenChooser(	APP_WIDGET		option,
 				APP_OPEN_DOCUMENT	openDocument,
 				EditApplication *	ea );
 
-extern APP_EVENT_HANDLER( appDocCopyPasteHandler, w, voided, event );
-extern APP_EVENT_HANDLER( appAppGotPasteCall, w, voided, event );
+extern APP_EVENT_HANDLER_H( appDocCopyPasteHandler, w, voided, event );
+extern APP_EVENT_HANDLER_H( appAppGotPasteCall, w, voided, event );
 
 extern int appDocOwnSelection(	EditDocument *			ed,
 				const char *			selection,
@@ -902,18 +855,10 @@ extern void appDocHorizontalScrollbarCallback(	APP_WIDGET	w,
 extern void appDocVerticalScrollbarCallback(	APP_WIDGET	w,
 						void *		voided,
 						void *		voidscbs );
-extern void * appMakeFontsTool(	APP_WIDGET		fontsOption,
-				const char *		widgetName,
-				APP_BITMAP_IMAGE	iconPixmap,
-				APP_BITMAP_MASK		iconMask,
-				AppToolDestroy		destroy,
-				FontChooserSetFont	setFont,
-				EditApplication * 	ea );
-
 extern void appMakeButtonInRow(	APP_WIDGET *		pButton,
 				APP_WIDGET		buttonRow,
 				const char *		text,
-				APP_BUTTON_CALLBACK	callback,
+				APP_BUTTON_CALLBACK_T	callback,
 				void *			through,
 				int			position,
 				int			showAsDefault );
@@ -941,7 +886,7 @@ extern void appMakeLabelAndTextRow(	APP_WIDGET *	pRow,
 
 extern APP_WIDGET appMakeToggleInRow(	APP_WIDGET		buttonRow,
 					const char *		text,
-					APP_TOGGLE_CALLBACK	callback,
+					APP_TOGGLE_CALLBACK_T	callback,
 					void *			through,
 					int			position );
 
@@ -950,7 +895,7 @@ extern void appMakeToggleAndTextRow(	APP_WIDGET *		pRow,
 					APP_WIDGET *		pText,
 					APP_WIDGET		parent,
 					char *			labelText,
-					APP_TOGGLE_CALLBACK	callback,
+					APP_TOGGLE_CALLBACK_T	callback,
 					void *			through,
 					int			textColumns,
 					int			textEnabled );
@@ -976,7 +921,7 @@ extern void appMakeTextInColumn(	APP_WIDGET *	pText,
 
 extern void appMakeColumnToggle(	APP_WIDGET *		pToggle,
 					APP_WIDGET		column,
-					APP_TOGGLE_CALLBACK	callback,
+					APP_TOGGLE_CALLBACK_T	callback,
 					void *			through,
 					const char *		labelText,
 					int			set );
@@ -984,15 +929,15 @@ extern void appMakeColumnToggle(	APP_WIDGET *		pToggle,
 extern void appMakeColumnDrawing(	APP_WIDGET *		pButton,
 					APP_WIDGET		column,
 					APP_DRAW_BUTTON_CALLBACK exposeCallback,
-					APP_BUTTON_CALLBACK	pushedCallback,
+					APP_BUTTON_CALLBACK_T	pushedCallback,
 					void *			through,
 					int			width );
 
 extern void appMakeVerticalDialog(	AppDialog *		ad,
 					APP_WIDGET *		pPaned,
 					EditApplication *	ea,
-					APP_CLOSE_CALLBACK	closeCallback,
-					APP_DESTROY_CALLBACK	destroyCallback,
+					APP_CLOSE_CALLBACK_T	closeCallback,
+					APP_DESTROY_CALLBACK_T	destroyCallback,
 					void *			through,
 					char *			widgetName );
 
@@ -1036,20 +981,12 @@ extern void appDrawNupDiagram(	APP_WIDGET			w,
 				AppDrawingData *		add,
 				APP_FONT *			xfs,
 				double				widgetHighCm,
-				const DocumentGeometry *	dgSheet,
 				const DocumentGeometry *	dgPage,
-				int				nup,
-				int				horizontal );
+				const PrintGeometry *		pg );
 
-extern void appFontToolTextReadonly(	void *	voidafc,
-					int	readonly );
-
-
-extern void appCallPrintFunction( FILE *			f,
-				EditDocument *			ed,
-				const DocumentGeometry *	dgPaper,
-				int				nup,
-				int				horizontal,
+extern int appCallPrintFunction( SimpleOutputStream *		sos,
+				const PrintJob *		pj,
+				const PrintGeometry *		pg,
 				int				firstPage,
 				int				lastPage );
 
@@ -1065,27 +1002,37 @@ extern void appDoubleToTextWidget(	APP_WIDGET		w,
 extern int appImgMakeFileExtensions(	AppFileExtension **	pAfeList,
 					int *			pAfeCount );
 
-extern void appInspectorSelectSubject(	AppInspector *		ai,
-					int			subject );
-
 extern void appInspectorMakeButtonRow(	APP_WIDGET *		pRow,
 					APP_WIDGET		parent,
 					APP_WIDGET *		pLeftButton,
 					APP_WIDGET *		pRightButton,
-					char *			leftLabel,
-					char *			rightLabel,
-					APP_BUTTON_CALLBACK	leftCallback,
-					APP_BUTTON_CALLBACK	rightCallback,
+					const char *		leftLabel,
+					const char *		rightLabel,
+					APP_BUTTON_CALLBACK_T	leftCallback,
+					APP_BUTTON_CALLBACK_T	rightCallback,
+					void *			through );
+
+extern void appInspectorMake3ButtonRow( APP_WIDGET *		pRow,
+					APP_WIDGET		parent,
+					APP_WIDGET *		pLeftButton,
+					APP_WIDGET *		pMiddleButton,
+					APP_WIDGET *		pRightButton,
+					const char *		leftLabel,
+					const char *		middleLabel,
+					const char *		rightLabel,
+					APP_BUTTON_CALLBACK_T	leftCallback,
+					APP_BUTTON_CALLBACK_T	middleCallback,
+					APP_BUTTON_CALLBACK_T	rightCallback,
 					void *			through );
 
 extern void appInspectorMakeToggleRow(	APP_WIDGET *		pRow,
 					APP_WIDGET		parent,
 					APP_WIDGET *		pLeftToggle,
 					APP_WIDGET *		pRightToggle,
-					char *			leftText,
-					char *			rightText,
-					APP_TOGGLE_CALLBACK	leftCallback,
-					APP_TOGGLE_CALLBACK	rightCallback,
+					const char *		leftText,
+					const char *		rightText,
+					APP_TOGGLE_CALLBACK_T	leftCallback,
+					APP_TOGGLE_CALLBACK_T	rightCallback,
 					void *			through );
 
 extern AppInspector * 	appMakeInspector(
@@ -1098,15 +1045,6 @@ extern AppInspector * 	appMakeInspector(
 				    AppToolDestroy		closeInspector,
 				    void *			through );
 
-extern void appFinishInspector(		AppInspector *		ai );
-
-extern void appEnableInspector(		AppInspector *		ai,
-					int			enabled );
-
-extern void appEnableInspectorSubject(	AppInspector *		ai,
-					int			subject ,
-					int			enabled );
-
 extern void appInspectorMakeMenuRow(	APP_WIDGET *		pRow,
 					AppOptionmenu *		aom,
 					APP_WIDGET *		pLabel,
@@ -1118,13 +1056,13 @@ extern void appFillInspectorMenu(	int			count,
 					APP_WIDGET *		items,
 					char * const *		texts,
 					AppOptionmenu *		aom,
-					APP_BUTTON_CALLBACK	callBack,
+					APP_OITEM_CALLBACK_T	callBack,
 					void *			target );
 
 extern int appMakeDocumentWidget(	EditApplication *	ea,
 					EditDocument *		ed );
 
-extern APP_EVENT_HANDLER( appDocExposeHandler, w, d, exposeEvent );
+extern APP_EVENT_HANDLER_H( appDocExposeHandler, w, d, exposeEvent );
 
 extern int appFileConvert(	EditApplication *	ea,
 				const char *		fromName,
@@ -1135,6 +1073,11 @@ extern int appPrintToFile(	EditApplication *	ea,
 				const char *		toName,
 				const char *		paperString );
 
+extern int appPrintToPrinter(	EditApplication *	ea,
+				const char *		fromName,
+				const char *		toName,
+				const char *		paperString );
+
 extern void appDestroyEditDocument(	APP_WIDGET		w,
 					void *			voided,
 					void *			callData );
@@ -1142,7 +1085,7 @@ extern void appDestroyEditDocument(	APP_WIDGET		w,
 extern APP_WIDGET appMakePageDrawing(	APP_WIDGET		parent,
 					EditApplication *	ea,
 					int			mmHigh,
-					APP_EVENT_HANDLER	redraw,
+					APP_EVENT_HANDLER_T	redraw,
 					void *			through );
 
 extern void appScrollToRectangle(	EditDocument *		ed,
@@ -1161,7 +1104,7 @@ extern void appMakeVerticalTool( APP_WIDGET *		pShell,
 				const char *		widgetName,
 				int			userResizable,
 				APP_WIDGET		option,
-				APP_CLOSE_CALLBACK	closeCallback,
+				APP_CLOSE_CALLBACK_T	closeCallback,
 				void *			through );
 
 extern APP_WIDGET appMakeRowInColumn(	APP_WIDGET	parent,
@@ -1189,7 +1132,8 @@ extern void appFindToolDisableReplace(	void *	voidaft );
 
 extern void appMakeColumnInRow(		APP_WIDGET *	pColumn,
 					APP_WIDGET	row,
-					int		position );
+					int		position,
+					int		colspan );
 
 extern void appMakeRowFrameInColumn(	APP_WIDGET *	pFrame,
 					APP_WIDGET *	pRow,
@@ -1265,11 +1209,7 @@ extern void appGuiEnableOptionmenu(	AppOptionmenu *		aom,
 
 extern APP_WIDGET appAddItemToOptionmenu( AppOptionmenu *	aom,
 					const char *		label,
-					APP_BUTTON_CALLBACK	callBack,
-					void *			target );
-
-extern APP_WIDGET appAddPixmapItemToOptionmenu( AppOptionmenu *	aom,
-					APP_BUTTON_CALLBACK	callBack,
+					APP_OITEM_CALLBACK_T	callBack,
 					void *			target );
 
 extern void appSetShellTitle(		APP_WIDGET		shell,
@@ -1280,7 +1220,7 @@ extern void appSetShellTitle(		APP_WIDGET		shell,
 /*  Event handler to set minimum size.					*/
 /************************************************************************/
 
-extern APP_EVENT_HANDLER( appSetSizeAsMinimum, w, through, event );
+extern APP_EVENT_HANDLER_H( appSetSizeAsMinimum, w, through, event );
 
 /************************************************************************/
 /*  Names of the X11 events.						*/
@@ -1312,25 +1252,25 @@ extern void appMakeOptionmenuInRow(	AppOptionmenu *	aom,
 					int		column,
 					int		colspan );
 
-extern int appGuiGetToggleStateFromCallback( APP_WIDGET		toggle,
-					void *			voidcbs );
-
-extern int appGuiGetMenuToggleStateFromCallback( APP_WIDGET		toggle,
+extern int appGuiGetMenuToggleStateFromCallback(
+					APP_WIDGET		toggle,
 					void *			voidcbs );
 
 extern void appGuiMakeListInColumn(	APP_WIDGET *		pList,
 					APP_WIDGET		column,
 					int			visibleItems,
-					APP_CALLBACK_FUNC	callback,
+					APP_LIST_CALLBACK_T	callback,
 					void *			through );
 
-extern void appGuiSetTypingCallbackForText( APP_WIDGET		text,
-					APP_CALLBACK_FUNC	callBack,
-					void *		through );
+extern void appGuiSetTypingCallbackForText(
+					APP_WIDGET		text,
+					APP_TXTYPING_CALLBACK_T	callBack,
+					void *			through );
 
-extern void appGuiSetGotValueCallbackForText( APP_WIDGET	text,
-					APP_CALLBACK_FUNC	callBack,
-					void *		through );
+extern void appGuiSetGotValueCallbackForText(
+				    APP_WIDGET			text,
+				    APP_TXACTIVATE_CALLBACK_T	callBack,
+				    void *			through );
 
 extern void appGuiGetScrollbarValues(	int *		pValue,
 					int *		pSliderSize,
@@ -1371,9 +1311,6 @@ extern void appGuiSelectPositionInListWidget(	APP_WIDGET	list,
 
 extern void appGuiRemoveSelectionFromListWidget(	APP_WIDGET	list );
 
-extern int appGuiGetPositionFromListCallback(	APP_WIDGET	list,
-						void *		voidlcs );
-
 extern APP_WIDGET appSetMenuItem(	APP_WIDGET		menu,
 					AppToplevel *		at,
 					AppMenuItem *		ami,
@@ -1395,8 +1332,9 @@ extern APP_WIDGET appMakeMenuInParent(	APP_WIDGET *		pButton,
 					const char *		itemText,
 					int			isHelp );
 
-extern char * appGuiGetStringFromListCallback(	APP_WIDGET	list,
-						void *		voidlcs );
+extern char * appGuiGetStringFromListCallback(
+					    APP_LIST		list,
+					    APP_LIST_CHOICE *	voidlcs );
 
 extern void appFreeStringFromListCallback(	char *		s );
 
@@ -1425,7 +1363,7 @@ extern void appGuiMakeDrawingAreaInColumn( APP_WIDGET *		pDrawing,
 				int				wide,
 				int				high,
 				int				heightResizable,
-				APP_EVENT_HANDLER		redraw,
+				APP_EVENT_HANDLER_T		redraw,
 				void *				through );
 
 void appGuiGetStringFromKeyboardEvent(	APP_INPUT_CONTEXT	ic,
@@ -1449,26 +1387,26 @@ extern void appInitOptionmenu(		AppOptionmenu *		aom );
 extern void appRunDragLoop(	APP_WIDGET		w,
 				EditApplication *	ea,
 				APP_EVENT *		downEvent,
-				APP_EVENT_HANDLER	upHandler,
-				APP_EVENT_HANDLER	moveHandler,
+				APP_EVENT_HANDLER_T	upHandler,
+				APP_EVENT_HANDLER_T	moveHandler,
 				int			timerInterval,
 				APP_TIMER_CALLBACK	timerHandler,
 				void *			through );
 
 extern void appGuiSetFocusChangeHandler( APP_WIDGET		shell,
-					APP_EVENT_HANDLER	handler,
+					APP_EVENT_HANDLER_T	handler,
 					void *			through );
 
 extern void appGuiSetToggleItemState(	APP_WIDGET		toggle,
 					int			set );
 
 extern void appSetDestroyCallback(	APP_WIDGET		shell,
-					APP_DESTROY_CALLBACK	destroyCallback,
+					APP_DESTROY_CALLBACK_T	destroyCallback,
 					void *			through );
 
 extern void appSetCloseCallback(	APP_WIDGET		shell,
 					EditApplication *	ea,
-					APP_CLOSE_CALLBACK	closeCallback,
+					APP_CLOSE_CALLBACK_T	closeCallback,
 					void *			through );
 
 extern void appGuiSetToggleItemLabel(	APP_WIDGET		toggle,
@@ -1480,12 +1418,18 @@ extern int appGuiGtkGetChildLabel(	char **			pLabel,
 extern int appGuiGtkSetChildLabel(	APP_WIDGET		w,
 					const char *		s );
 
+extern void appGuiGetResourceValuesGtkX(
+				EditApplication *		ea,
+				void *				pValues,
+				AppConfigurableResource *	acr,
+				int				acrCount );
+
 extern int appFormatDocumentTitle(	const char **		pWindowTitle,
 					const char **		pIconTitle,
 					EditApplication *	ea,
 					const char *		title );
 
-extern APP_EVENT_HANDLER( appDocConfigure, w, voided, event );
+extern APP_EVENT_HANDLER_H( appDocConfigure, w, voided, event );
 
 extern void appDocumentCalculateExtraSize(	EditDocument *	ed );
 
@@ -1507,6 +1451,85 @@ extern APP_GIVE_COPY( appDocReplyToCopyRequest, w, gsd, voided );
 extern void appCopyPixmapValue(	APP_SELECTION_EVENT *	event,
 				APP_BITMAP_IMAGE	pixmapCopied );
 
-extern void appPixmapOptionmenuSetButtonMargins(	AppOptionmenu *	aom );
+extern void appGetApplicationResourceValues(	EditApplication *	ea );
+
+extern void appDocExposeRectangle(
+				const EditDocument *		ed,
+				const DocumentRectangle *	drChanged,
+				int				scrolledX,
+				int				scrolledY );
+
+extern void appPrintJobForEditDocument(	PrintJob *		pj,
+					EditDocument *		ed );
+
+extern int appRunPrintToFileChooser(	APP_WIDGET		option,
+					APP_WIDGET		panel,
+					EditApplication *	ea,
+					const EditDocument *	ed,
+					char **			pFilename );
+
+extern int appDocumentTestCanSave(
+				EditApplication *		ea,
+				const AppFileExtension *	afe,
+				void *				privateData,
+				unsigned int			useFlags,
+				int				format );
+
+extern int appDocumentGetSaveFormat(
+				EditApplication *		ea,
+				const char *			filename,
+				void *				privateData,
+				unsigned int			useFlags,
+				int				format );
+
+extern void appMakeDrawnPulldownInColumn(
+				AppDrawnPulldown *		adp,
+				APP_EVENT_HANDLER_T		redrawInplace,
+				APP_EVENT_HANDLER_T		redrawPulldown,
+				APP_EVENT_HANDLER_T		clickHandler,
+				APP_WIDGET			column,
+				void *				through );
+
+extern void appMakeDrawnPulldownInRow(
+				AppDrawnPulldown *		adp,
+				APP_EVENT_HANDLER_T		redrawInplace,
+				APP_EVENT_HANDLER_T		redrawPulldown,
+				APP_EVENT_HANDLER_T		clickHandler,
+				APP_WIDGET			row,
+				int				column,
+				int				colspan,
+				void *				through );
+
+extern void appGuiEnableDrawnPulldown(	AppDrawnPulldown *	adp,
+					int			sensitive );
+
+extern void appInitDrawnPulldown(	AppDrawnPulldown *	adp );
+
+extern void appGuiSetDrawnPulldownHeight(
+				    AppDrawnPulldown *	adp,
+				    int			height );
+
+extern void appGuiSetDrawnPulldownStrips(
+				    AppDrawnPulldown *	adp,
+				    int			strips );
+
+extern int appGuiDrawnPulldownGetStrip(
+				int *				pStrip,
+				const AppDrawnPulldown *	adp,
+				const APP_WIDGET		w,
+				const APP_EVENT *		mouseEvent );
+
+extern void appDrawnPulldownDrawArrow(	int *			pRestWide,
+					int *			pRestHigh,
+					APP_WIDGET		w,
+					AppDrawingData *	add );
+
+extern APP_FONT * appGuiGetLabelFont(	APP_WIDGET	w );
+
+extern int appGuiGetPositionFromListCallback(	APP_LIST	list,
+						void *		voidlcs );
+
+extern void appOptionmenuItemSetVisibility(	APP_WIDGET	w,
+						int		visible );
 
 #   endif
