@@ -21,17 +21,22 @@
 
 void appGuiGetResourceValues(	EditApplication *		ea,
 				void *				pValues,
-				AppConfigurableResource *	acr,
+				AppConfigurableResource *	acrList,
 				int				acrCount )
     {
-    char *	values= (char *)pValues;
-    int		i;
+    AppConfigurableResource *	acr;
+    char *			values= (char *)pValues;
+    int				i;
 
+    acr= acrList;
     for ( i= 0; i < acrCount; acr++, i++ )
 	{
 	*((const char **)(values+acr->acrStructOffset))= acr->acrDefaultValue;
 	}
 
+    if  ( ea->eaToplevel.atTopWidget )
+	{ appGuiGetResourceValuesGtkX( ea, pValues, acrList, acrCount ); }
+
     return;
     }
 
@@ -80,14 +85,19 @@ int appGuiInitApplication(	EditApplication *	ea,
      * Facilities for making menu option accelerators.
      */
     gAccelGroup = gtk_accel_group_new();
-    gtk_accel_group_attach (gAccelGroup, GTK_OBJECT (gWindow));
     ea->eaToplevel.atAccelGroup= gAccelGroup;
 
+#   if GTK_MAJOR_VERSION < 2
+    gtk_accel_group_attach (gAccelGroup, GTK_OBJECT (gWindow));
+#   else
+    gtk_window_add_accel_group( GTK_WINDOW( gWindow ), gAccelGroup );
+#   endif
+
     /* 
      * Do this here so that we don't get an error when
      * we create the pixmap using the window.
      */
-    gtk_widget_realize (gWindow);
+    gtk_widget_realize( gWindow );
 
     ea->eaScreenPixelsWide= gdk_screen_width();
     ea->eaScreenPixelsHigh= gdk_screen_height();
@@ -253,15 +263,14 @@ int appGuiGetToggleState(	APP_WIDGET		toggle )
     return gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( toggle ) ) != 0;
     }
 
-int appGuiGetToggleStateFromCallback(	APP_WIDGET		toggle,
-					void *			voidcbs )
+int appGuiGetToggleStateFromCallbackGtk( GtkToggleButton *	toggle )
     {
-    return gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( toggle ) ) != 0;
+    return gtk_toggle_button_get_active( toggle ) != 0;
     }
 
 APP_WIDGET appMakeToggleInRow(	APP_WIDGET		row,
 				const char *		text,
-				APP_TOGGLE_CALLBACK	callback,
+				APP_TOGGLE_CALLBACK_T	callback,
 				void *			through,
 				int			position )
     {
@@ -277,7 +286,8 @@ APP_WIDGET appMakeToggleInRow(	APP_WIDGET		row,
 
     gtk_widget_show( toggle );
 
-    gtk_signal_connect( GTK_OBJECT( toggle ), "toggled", callback, through );
+    gtk_signal_connect( GTK_OBJECT( toggle ), "toggled",
+					    (GtkSignalFunc)callback, through );
 
     return toggle;
     }
@@ -307,37 +317,10 @@ APP_WIDGET appMakeRowInColumn(	APP_WIDGET	column,
     return row;
     }
 
-void appMakeLabelInRow(		APP_WIDGET *		pLabel,
-				APP_WIDGET		row,
-				int			column,
-				int			colspan,
-				const char *		labelText )
-    {
-    GtkWidget *		label= gtk_label_new( labelText );
-
-    gtk_label_set_line_wrap( GTK_LABEL( label ), FALSE );
-    gtk_label_set_justify( GTK_LABEL( label ), GTK_JUSTIFY_LEFT );
-    gtk_misc_set_alignment( GTK_MISC( label ), 0.0, 0.5 );
-
-    gtk_table_attach( GTK_TABLE( row ),
-			label,
-			column, column+ colspan,
-			0, 1,
-			GTK_FILL | GTK_EXPAND | GTK_SHRINK,
-			GTK_FILL | GTK_EXPAND | GTK_SHRINK,
-			ROW_XPADDING_GTK, ROW_YPADDING_GTK );
-
-    gtk_widget_show( label );
-
-    *pLabel= label;
-
-    return;
-    }
-
 void appMakeButtonInRow(	APP_WIDGET *		pButton,
 				APP_WIDGET		row,
 				const char *		text,
-				APP_BUTTON_CALLBACK	callback,
+				APP_BUTTON_CALLBACK_T	callback,
 				void *			through,
 				int			position,
 				int			showAsDefault )
@@ -354,7 +337,8 @@ void appMakeButtonInRow(	APP_WIDGET *		pButton,
 
     gtk_widget_show( button );
 
-    gtk_signal_connect( GTK_OBJECT( button ), "clicked", callback, through );
+    gtk_signal_connect( GTK_OBJECT( button ), "clicked",
+					(GtkSignalFunc)callback, through );
 
     *pButton= button;
 
@@ -391,14 +375,6 @@ void appGuiGetScrollbarValues(		int *		pValue,
     return;
     }
 
-void appGuiChangeLabelText(	APP_WIDGET	labelWidget,
-				const char *	label )
-    {
-    gtk_label_set_text( GTK_LABEL( labelWidget ), label );
-    
-    return;
-    }
-
 void appGuiChangeButtonText(	APP_WIDGET	button,
 				const char *	label )
     {
@@ -406,23 +382,6 @@ void appGuiChangeButtonText(	APP_WIDGET	button,
 	{ SDEB(label);	}
     }
 
-void appMakeLabelInColumn(	APP_WIDGET *		pLabel,
-				APP_WIDGET		column,
-				const char *		labelText )
-    {
-    GtkWidget *		label= gtk_label_new( labelText );
-
-    gtk_box_pack_start( GTK_BOX( column ), label, FALSE, TRUE, 0 );
-
-    gtk_label_set_line_wrap( GTK_LABEL( label ), FALSE );
-    gtk_label_set_justify( GTK_LABEL( label ), GTK_JUSTIFY_LEFT );
-    gtk_misc_set_alignment( GTK_MISC( label ), 0.0, 0.5 );
-
-    gtk_widget_show( label );
-
-    *pLabel= label;
-    }
-
 void appShowShellWidget(		APP_WIDGET		shell )
     {
     gtk_widget_show( shell );
@@ -461,13 +420,14 @@ void appGuiInsertSeparatorInColumn(	APP_WIDGET *	pSeparator,
 
 void appMakeColumnInRow(	APP_WIDGET *	pColumn,
 				APP_WIDGET	row,
-				int		position )
+				int		position,
+				int		colspan )
     {
     GtkWidget *		column= gtk_vbox_new( FALSE, COLUMN_SPACING_GTK );
 
     gtk_table_attach( GTK_TABLE( row ),
 			column,
-			position, position+ 1,
+			position, position+ colspan,
 			0, 1,
 			GTK_FILL | GTK_EXPAND | GTK_SHRINK,
 			GTK_FILL | GTK_EXPAND | GTK_SHRINK,
@@ -483,7 +443,7 @@ void appGuiMakeDrawingAreaInColumn( APP_WIDGET *		pDrawing,
 				    int				wide,
 				    int				high,
 				    int				heightResizable,
-				    APP_EVENT_HANDLER		redraw,
+				    APP_EVENT_HANDLER_T		redraw,
 				    void *			through )
     {
     GtkWidget *	drawing= gtk_drawing_area_new();
@@ -526,7 +486,7 @@ void appMakeVerticalTool(	APP_WIDGET *		pShell,
 				const char *		widgetName,
 				int			userResizable,
 				APP_WIDGET		option,
-				APP_CLOSE_CALLBACK	closeCallback,
+				APP_CLOSE_CALLBACK_T	closeCallback,
 				void *			through )
     {
     GtkWidget *		shell;
@@ -659,8 +619,8 @@ void appMakeImageInColumn(	APP_WIDGET *		pImage,
 
 typedef struct DragLoop
     {
-    GtkSignalFunc		dlUpHandler;
-    GtkSignalFunc		dlMoveHandler;
+    APP_EVENT_HANDLER_T		dlUpHandler;
+    APP_EVENT_HANDLER_T		dlMoveHandler;
     int				dlTimerInterval;
     GtkFunction			dlTimerHandler;
     void *			dlThrough;
@@ -718,8 +678,8 @@ static void appDragMouseMove(		APP_WIDGET		w,
 void appRunDragLoop(	APP_WIDGET		w,
 			EditApplication *	ea,
 			APP_EVENT *		downEvent,
-			APP_EVENT_HANDLER	upHandler,
-			APP_EVENT_HANDLER	moveHandler,
+			APP_EVENT_HANDLER_T	upHandler,
+			APP_EVENT_HANDLER_T	moveHandler,
 			int			timerInterval,
 			APP_TIMER_CALLBACK	timerHandler,
 			void *			through )
@@ -737,12 +697,18 @@ void appRunDragLoop(	APP_WIDGET		w,
     gtk_widget_add_events( w,
 			GDK_BUTTON_RELEASE_MASK | GDK_BUTTON1_MOTION_MASK );
 
-    dl.dlUpId= gtk_signal_connect( GTK_OBJECT( w ), "button_release_event",
-					    appDragMouseUp, (void *)&dl );
-    dl.dlDownId= gtk_signal_connect( GTK_OBJECT( w ), "button_press_event",
-					    appDragMouseUp, (void *)&dl );
-    dl.dlMoveId= gtk_signal_connect( GTK_OBJECT( w ), "motion_notify_event",
-					    appDragMouseMove, (void *)&dl );
+    dl.dlUpId= gtk_signal_connect( GTK_OBJECT( w ),
+					    "button_release_event",
+					    (GtkSignalFunc)appDragMouseUp,
+					    (void *)&dl );
+    dl.dlDownId= gtk_signal_connect( GTK_OBJECT( w ),
+					    "button_press_event",
+					    (GtkSignalFunc)appDragMouseUp,
+					    (void *)&dl );
+    dl.dlMoveId= gtk_signal_connect( GTK_OBJECT( w ),
+					    "motion_notify_event",
+					    (GtkSignalFunc)appDragMouseMove,
+					    (void *)&dl );
 
     if  ( dl.dlTimerHandler && dl.dlTimerInterval > 0 )
 	{
@@ -801,36 +767,39 @@ void appGuiGetStringFromKeyboardEvent(	APP_INPUT_CONTEXT	ic,
     }
 
 void appGuiSetFocusChangeHandler(	APP_WIDGET		w,
-					APP_EVENT_HANDLER	handler,
+					APP_EVENT_HANDLER_T	handler,
 					void *			through )
     {
     gtk_widget_add_events( w, GDK_FOCUS_CHANGE_MASK );
 
-    gtk_signal_connect( GTK_OBJECT( w ), "focus_in_event", handler, through );
+    gtk_signal_connect( GTK_OBJECT( w ),
+				    "focus_in_event",
+				    (GtkSignalFunc)handler, through );
 
-    gtk_signal_connect( GTK_OBJECT( w ), "focus_out_event", handler, through );
+    gtk_signal_connect( GTK_OBJECT( w ), "focus_out_event",
+				    (GtkSignalFunc)handler, through );
     }
 
 void appSetDestroyCallback(	APP_WIDGET		shell,
-				APP_DESTROY_CALLBACK	destroyCallback,
+				APP_DESTROY_CALLBACK_T	destroyCallback,
 				void *			through )
     {
     if  ( destroyCallback )
 	{
 	gtk_signal_connect( GTK_OBJECT( shell ), "destroy_event",
-						    destroyCallback, through );
+				    (GtkSignalFunc)destroyCallback, through );
 	}
     }
 
 void appSetCloseCallback(	APP_WIDGET		shell,
 				EditApplication *	ea,
-				APP_CLOSE_CALLBACK	closeCallback,
+				APP_CLOSE_CALLBACK_T	closeCallback,
 				void *			through )
     {
     if  ( closeCallback )
 	{
 	gtk_signal_connect( GTK_OBJECT( shell ), "delete_event",
-						    closeCallback, through );
+				    (GtkSignalFunc)closeCallback, through );
 	}
     }
 
