@@ -1,8 +1,7 @@
-#   include	"config.h"
+#   include	"appFrameConfig.h"
 
 #   include	<stdlib.h>
 #   include	<stdio.h>
-#   include	<appDebugon.h>
 
 #   include	"appFrame.h"
 #   include	"appSystem.h"
@@ -11,7 +10,6 @@
 #   ifdef USE_MOTIF
 
 #   include	<X11/Xatom.h>
-#   include	<X11/IntrinsicP.h>
 #   include	<Xm/RowColumn.h>
 #   include	<Xm/PushB.h>
 #   include	<Xm/PushBG.h>
@@ -30,6 +28,8 @@
 #   include	<Xm/SashP.h>
 #   include	<Xm/DrawingA.h>
 
+#   include	<appDebugon.h>
+
 /************************************************************************/
 /*									*/
 /*  Tame the sashes in an XmPanedWindow.				*/
@@ -143,10 +143,19 @@ void appSetShellTitle(	Widget			shell,
 
 /************************************************************************/
 /*									*/
-/*  Remember te initial size of a shell as its minimum size.		*/
+/*  1)  Remember te initial size of a shell as its minimum size.	*/
+/*									*/
+/*  2)  Remember te initial size of a shell as its minimum and its	*/
+/*	maximum size. This event handler can be used to prevent the	*/
+/*	user from resizing a dialog. E.G. because KDE does not listen	*/
+/*	to the window manager hints about this. On the other hand some	*/
+/*	window managers (mwm on vms!) cause this event handler to be	*/
+/*	called too early. That results in dialogs that are MUCH too	*/
+/*	small.								*/
 /*									*/
 /************************************************************************/
 
+/*  1  */
 void appSetSizeAsMinimum(	Widget			w,
 				void *			through,
 				XEvent *		event,
@@ -169,6 +178,7 @@ void appSetSizeAsMinimum(	Widget			w,
     return;
     }
 
+/*  2  */
 static void appFixSize(	Widget			w,
 			void *			through,
 			XEvent *		event,
@@ -221,7 +231,7 @@ char * APP_X11EventNames[]=
 void appMakeButtonInRow(	APP_WIDGET *		pButton,
 				APP_WIDGET		row,
 				const char *		text,
-				APP_BUTTON_CALLBACK	callback,
+				APP_BUTTON_CALLBACK_T	callback,
 				void *			through,
 				int			position,
 				int			showAsDefault )
@@ -278,7 +288,7 @@ void appMakeButtonInRow(	APP_WIDGET *		pButton,
 
 APP_WIDGET appMakeToggleInRow(	APP_WIDGET		row,
 				const char *		text,
-				APP_TOGGLE_CALLBACK	callback,
+				APP_TOGGLE_CALLBACK_T	callback,
 				void *			through,
 				int			position )
     {
@@ -305,11 +315,8 @@ APP_WIDGET appMakeToggleInRow(	APP_WIDGET		row,
     XtSetArg( al[ac], XmNrightOffset,		0 ); ac++;
 
     XtSetArg( al[ac], XmNrecomputeSize,		True ); ac++;
-#   if 1
-    XtSetArg( al[ac], XmNmarginTop,		6 ); ac++;
-#   else
-    XtSetArg( al[ac], XmNmarginHeight,		8 ); ac++;
-#   endif
+
+    XtSetArg( al[ac], XmNmarginTop,		TBrmargT ); ac++;
 
     XtSetArg( al[ac], XmNalignment,		XmALIGNMENT_BEGINNING ); ac++;
 
@@ -336,9 +343,10 @@ APP_WIDGET appMakeRowInColumn(	APP_WIDGET	column,
 
     ac= 0;
     XtSetArg( al[ac],	XmNfractionBase,	columnCount ); ac++;
-    XtSetArg( al[ac],	XmNallowResize,		True ); ac++;
     XtSetArg( al[ac],	XmNskipAdjust,		heightResizable == 0 ); ac++;
 
+    XtSetArg( al[ac],	XmNallowResize,		True ); ac++;
+
     row= XmCreateForm( column, WIDGET_NAME, al, ac );
 
     XtManageChild( row );
@@ -348,56 +356,6 @@ APP_WIDGET appMakeRowInColumn(	APP_WIDGET	column,
     return row;
     }
 
-/************************************************************************/
-/*									*/
-/*  Make a row with a label and a text widget.				*/
-/*									*/
-/************************************************************************/
-
-void appMakeLabelInRow(		APP_WIDGET *		pLabel,
-				APP_WIDGET		row,
-				int			column,
-				int			colspan,
-				const char *		labelText )
-    {
-    Widget		label;
-
-    XmString		labelString;
-
-    Arg			al[20];
-    int			ac= 0;
-
-    labelString= XmStringCreateLocalized( (char *)labelText );
-
-    ac= 0;
-    XtSetArg( al[ac], XmNtopAttachment,		XmATTACH_FORM ); ac++;
-    XtSetArg( al[ac], XmNtopOffset,		0 ); ac++;
-
-    XtSetArg( al[ac], XmNleftAttachment,	XmATTACH_POSITION ); ac++;
-    XtSetArg( al[ac], XmNleftPosition,		column ); ac++;
-
-    XtSetArg( al[ac], XmNrightAttachment,	XmATTACH_POSITION ); ac++;
-    XtSetArg( al[ac], XmNrightPosition,		column+ colspan ); ac++;
-
-    XtSetArg( al[ac], XmNrecomputeSize,		True ); ac++;
-    XtSetArg( al[ac], XmNalignment,		XmALIGNMENT_BEGINNING ); ac++;
-    XtSetArg( al[ac], XmNlabelString,		labelString ); ac++;
-
-#   if 1
-    XtSetArg( al[ac], XmNmarginTop,		6 ); ac++;
-#   else
-    XtSetArg( al[ac], XmNmarginHeight,		8 ); ac++;
-#   endif
-
-    label= XmCreateLabel( row, WIDGET_NAME, al, ac );
-
-    XmStringFree( labelString );
-
-    XtManageChild( label );
-
-    *pLabel= label;
-    }
-
 /************************************************************************/
 /*									*/
 /*  Make a frame, possibly with a title.				*/
@@ -464,22 +422,22 @@ void appMakeColumnFrameInColumn(	APP_WIDGET *		pFrame,
     appMakeFrame( &frame, parent, title );
 
     ac= 0;
-    XtSetArg( al[ac],	XmNmarginWidth,		0 ); ac++;
-    XtSetArg( al[ac],	XmNmarginHeight,	0 ); ac++;
+    XtSetArg( al[ac], XmNmarginWidth,		0 ); ac++;
+    XtSetArg( al[ac], XmNmarginHeight,	0 ); ac++;
     bboard= XmCreateForm( frame, WIDGET_NAME, al, ac );
 
     ac= 0;
-    XtSetArg( al[ac],	XmNleftAttachment,	XmATTACH_FORM ); ac++;
-    XtSetArg( al[ac],	XmNrightAttachment,	XmATTACH_FORM ); ac++;
-    XtSetArg( al[ac],	XmNtopAttachment,	XmATTACH_FORM ); ac++;
-    XtSetArg( al[ac],	XmNbottomAttachment,	XmATTACH_FORM ); ac++;
+    XtSetArg( al[ac], XmNleftAttachment,	XmATTACH_FORM ); ac++;
+    XtSetArg( al[ac], XmNrightAttachment,	XmATTACH_FORM ); ac++;
+    XtSetArg( al[ac], XmNtopAttachment,		XmATTACH_FORM ); ac++;
+    XtSetArg( al[ac], XmNbottomAttachment,	XmATTACH_FORM ); ac++;
 
-    XtSetArg( al[ac],	XmNsashWidth,		1 ); ac++;
-    XtSetArg( al[ac],	XmNsashHeight,		1 ); ac++;
-    XtSetArg( al[ac],	XmNseparatorOn,		False ); ac++;
-    XtSetArg( al[ac],	XmNmarginWidth,		5 ); ac++;
-    XtSetArg( al[ac],	XmNmarginHeight,	5 ); ac++;
-    XtSetArg( al[ac],	XmNspacing,		5 ); ac++;
+    XtSetArg( al[ac], XmNsashWidth,		1 ); ac++;
+    XtSetArg( al[ac], XmNsashHeight,		1 ); ac++;
+    XtSetArg( al[ac], XmNseparatorOn,		False ); ac++;
+    XtSetArg( al[ac], XmNmarginWidth,		PWmargW ); ac++;
+    XtSetArg( al[ac], XmNmarginHeight,		PWmargH ); ac++;
+    XtSetArg( al[ac], XmNspacing,		PWspacing ); ac++;
     paned= XmCreatePanedWindow( bboard, WIDGET_NAME, al, ac );
 
     XtManageChild( paned );
@@ -530,7 +488,8 @@ void appMakeRowFrameInColumn(	APP_WIDGET *		pFrame,
 
 void appMakeColumnInRow(	APP_WIDGET *	pColumn,
 				APP_WIDGET	row,
-				int		position )
+				int		position,
+				int		colspan )
     {
     Arg			al[20];
     int			ac= 0;
@@ -538,20 +497,20 @@ void appMakeColumnInRow(	APP_WIDGET *	pColumn,
     Widget		column;
 
     ac= 0;
-    XtSetArg( al[ac],	XmNtopAttachment,	XmATTACH_FORM ); ac++;
-    XtSetArg( al[ac],	XmNbottomAttachment,	XmATTACH_FORM ); ac++;
-    XtSetArg( al[ac],	XmNleftAttachment,	XmATTACH_POSITION ); ac++;
-    XtSetArg( al[ac],	XmNrightAttachment,	XmATTACH_POSITION ); ac++;
-    XtSetArg( al[ac],	XmNleftPosition,	position ); ac++;
-    XtSetArg( al[ac],	XmNrightPosition,	position+ 1 ); ac++;
+    XtSetArg( al[ac], XmNtopAttachment,		XmATTACH_FORM ); ac++;
+    XtSetArg( al[ac], XmNbottomAttachment,	XmATTACH_FORM ); ac++;
+    XtSetArg( al[ac], XmNleftAttachment,	XmATTACH_POSITION ); ac++;
+    XtSetArg( al[ac], XmNrightAttachment,	XmATTACH_POSITION ); ac++;
+    XtSetArg( al[ac], XmNleftPosition,		position ); ac++;
+    XtSetArg( al[ac], XmNrightPosition,		position+ colspan ); ac++;
 
-    XtSetArg( al[ac],	XmNsashWidth,		1 ); ac++;
-    XtSetArg( al[ac],	XmNsashHeight,		1 ); ac++;
-    XtSetArg( al[ac],	XmNseparatorOn,		False ); ac++;
-    XtSetArg( al[ac],	XmNmarginWidth,		5 ); ac++;
-    XtSetArg( al[ac],	XmNmarginHeight,	5 ); ac++;
-    XtSetArg( al[ac],	XmNspacing,		5 ); ac++;
-    XtSetArg( al[ac],	XmNallowResize,		True ); ac++;
+    XtSetArg( al[ac], XmNsashWidth,		1 ); ac++;
+    XtSetArg( al[ac], XmNsashHeight,		1 ); ac++;
+    XtSetArg( al[ac], XmNseparatorOn,		False ); ac++;
+    XtSetArg( al[ac], XmNmarginWidth,		PWmargW ); ac++;
+    XtSetArg( al[ac], XmNmarginHeight,		PWmargH ); ac++;
+    XtSetArg( al[ac], XmNspacing,		PWspacing ); ac++;
+    XtSetArg( al[ac], XmNallowResize,		True ); ac++;
 
     column= XmCreatePanedWindow( row, WIDGET_NAME, al, ac );
 
@@ -560,36 +519,6 @@ void appMakeColumnInRow(	APP_WIDGET *	pColumn,
     *pColumn= column; return;
     }
 
-void appMakeLabelInColumn(	APP_WIDGET *		pLabel,
-				APP_WIDGET		column,
-				const char *		labelText )
-    {
-    Widget		label;
-
-    XmString		labelString;
-
-    Arg			al[20];
-    int			ac= 0;
-
-    labelString= XmStringCreateLocalized( (char *)labelText );
-
-    ac= 0;
-    XtSetArg( al[ac], XmNskipAdjust,		True ); ac++;
-    XtSetArg( al[ac], XmNrecomputeSize,		True ); ac++;
-    XtSetArg( al[ac], XmNalignment,		XmALIGNMENT_BEGINNING ); ac++;
-    XtSetArg( al[ac], XmNlabelString,		labelString ); ac++;
-
-    label= XmCreateLabel( column, WIDGET_NAME, al, ac );
-
-    XmStringFree( labelString );
-
-    XtManageChild( label );
-
-    appMotifTurnOfSashTraversal( column );
-
-    *pLabel= label; return;
-    }
-
 void appMakeImageInColumn(	APP_WIDGET *		pLabel,
 				APP_WIDGET		column,
 				APP_BITMAP_IMAGE	pixmap,
@@ -641,8 +570,8 @@ void appMakeColumnToggle(	Widget *		pToggle,
     XtSetArg( al[ac], XmNalignment,		XmALIGNMENT_BEGINNING ); ac++;
     XtSetArg( al[ac], XmNset,			set ); ac++;
 
-    XtSetArg( al[ac], XmNmarginTop,		1 ); ac++;
-    XtSetArg( al[ac], XmNmarginHeight,		5 ); ac++;
+    XtSetArg( al[ac], XmNmarginTop,		TBcmargT ); ac++;
+    XtSetArg( al[ac], XmNmarginHeight,		TBcmargH ); ac++;
 
     if  ( labelString )
 	{ XtSetArg( al[ac], XmNlabelString,	labelString ); ac++; }
@@ -717,7 +646,7 @@ void appMakeVerticalTool(	APP_WIDGET *		pShell,
 				const char *		widgetName,
 				int			userResizable,
 				APP_WIDGET		option,
-				APP_CLOSE_CALLBACK	closeCallback,
+				APP_CLOSE_CALLBACK_T	closeCallback,
 				void *			through )
     {
     Widget		shell;
@@ -749,6 +678,8 @@ void appMakeVerticalTool(	APP_WIDGET *		pShell,
     XtSetArg( al[ac], XmNinput,			True ); ac++;
     XtSetArg( al[ac], XmNallowShellResize,	True ); ac++;
     XtSetArg( al[ac], XmNuseAsyncGeometry,	True ); ac++;
+    XtSetArg( al[ac], XmNwaitForWm,		False ); ac++; /* LessTif BUG */
+    XtSetArg( al[ac], XmNwmTimeout,		0 ); ac++; /* LessTif BUG */
     XtSetArg( al[ac], XmNmwmDecorations,	hints.decorations ); ac++;
     XtSetArg( al[ac], XmNmwmFunctions,		hints.functions ); ac++;
 
@@ -770,21 +701,24 @@ void appMakeVerticalTool(	APP_WIDGET *		pShell,
 
     appSetShellTitle( shell, option, ea->eaApplicationName );
 
+#   if 0
+    See above.
     if  ( ! userResizable )
 	{
 	XtAddEventHandler( shell, StructureNotifyMask, False,
 						    appFixSize, through );
 	}
+#   endif
 
     appSetCloseCallback( shell, ea, closeCallback, through );
 
     ac= 0;
-    XtSetArg( al[ac],	XmNsashWidth,		1 ); ac++;
-    XtSetArg( al[ac],	XmNsashHeight,		1 ); ac++;
-    XtSetArg( al[ac],	XmNseparatorOn,		False ); ac++;
-    XtSetArg( al[ac],	XmNmarginWidth,		5 ); ac++;
-    XtSetArg( al[ac],	XmNmarginHeight,	5 ); ac++;
-    XtSetArg( al[ac],	XmNspacing,		5 ); ac++;
+    XtSetArg( al[ac], XmNsashWidth,		1 ); ac++;
+    XtSetArg( al[ac], XmNsashHeight,		1 ); ac++;
+    XtSetArg( al[ac], XmNseparatorOn,		False ); ac++;
+    XtSetArg( al[ac], XmNmarginWidth,		PWmargW ); ac++;
+    XtSetArg( al[ac], XmNmarginHeight,		PWmargH ); ac++;
+    XtSetArg( al[ac], XmNspacing,		PWspacing ); ac++;
     paned= XmCreatePanedWindow( shell, WIDGET_NAME, al, ac );
 
     XtManageChild( paned );
@@ -843,8 +777,24 @@ void appGuiGetResourceValues(	EditApplication *		ea,
 				AppConfigurableResource *	acr,
 				int				acrCount )
     {
-    XtGetApplicationResources( ea->eaToplevel.atTopWidget, pValues,
+    if  ( ea->eaToplevel.atTopWidget )
+	{
+	XtGetApplicationResources( ea->eaToplevel.atTopWidget, pValues,
 						    acr, acrCount, NULL, 0 );
+	}
+    else{
+	char *		values= (char *)pValues;
+	int		i;
+
+	for ( i= 0; i < acrCount; acr++, i++ )
+	    {
+#	    define	acrStructOffset	resource_offset
+#	    define	acrDefaultValue	default_addr
+
+	    *((const char **)(values+acr->acrStructOffset))=
+						    acr->acrDefaultValue;
+	    }
+	}
 
     return;
     }
@@ -1349,8 +1299,14 @@ int appGuiInitApplication(	EditApplication *	ea,
 	{ SLDEB(getenv("LANG"),XSupportsLocale()); }
 
 #   if 0
-    This is actually counter productive.
+
+    This is actually counter productive. It is already done by Motif/Xt.
+    The document that states XtSetLanguageProc() is indespensable
+    refers to a lower level of software. At the application level,
+    we can trust the libraries to call XtSetLanguageProc().
+
     XtSetLanguageProc( NULL, NULL, NULL );
+
 #   endif
 
     /*  1  */
@@ -1366,6 +1322,8 @@ int appGuiInitApplication(	EditApplication *	ea,
     XtSetArg( al[ac], XmNinput,			True ); ac++;
     XtSetArg( al[ac], XmNallowShellResize,	True ); ac++;
     XtSetArg( al[ac], XmNuseAsyncGeometry,	True ); ac++;
+    XtSetArg( al[ac], XmNwaitForWm,		False ); ac++; /* LessTif BUG */
+    XtSetArg( al[ac], XmNwmTimeout,		0 ); ac++; /* LessTif BUG */
     XtSetArg( al[ac], XmNmwmDecorations,	hints.decorations ); ac++;
     XtSetArg( al[ac], XmNmwmFunctions,		hints.functions ); ac++;
 
@@ -1530,8 +1488,7 @@ int appGuiGetToggleState(	APP_WIDGET		toggle )
     return XmToggleButtonGetState( toggle ) != 0;
     }
 
-int appGuiGetToggleStateFromCallback(	APP_WIDGET		toggle,
-					void *			voidcbs )
+int appGuiGetToggleStateFromCallbackMotif( void *	voidcbs )
     {
     XmToggleButtonCallbackStruct *	cbs;
 
@@ -1586,23 +1543,6 @@ void appIconifyShellWidget(		APP_WIDGET		shell )
     XIconifyWindow( display, win, DefaultScreen( display ) );
     }
 
-void appGuiChangeLabelText(	APP_WIDGET	labelWidget,
-				const char *	label )
-    {
-    XmString			labelString;
-
-    labelString= XmStringCreateLocalized( (char *)label );
-
-    XtVaSetValues( labelWidget,
-			XmNlabelString,		labelString,
-			NULL );
-
-    XmStringFree( labelString );
-    
-    return;
-    }
-
-
 void appGuiChangeButtonText(	APP_WIDGET	button,
 				const char *	label )
     {
@@ -1659,12 +1599,29 @@ void appGuiSetIconTitle(		APP_WIDGET		shell,
     return;
     }
 
+/************************************************************************/
+/*									*/
+/*  Make drawing areas. Typically these are small drawings so it does	*/
+/*  no harm to redraw the whole drawing area after a resize.		*/
+/*									*/
+/************************************************************************/
+
+static void appGuiSendExposeForResize(	APP_WIDGET		w,
+					void *			through,
+					void *			voidcbs )
+    {
+    if  ( XtIsRealized( w ) )
+	{
+	XClearArea( XtDisplay( w ), XtWindow( w ), 0, 0, 0, 0, True );
+	}
+    }
+
 void appGuiMakeDrawingAreaInColumn( APP_WIDGET *		pDrawing,
 				    APP_WIDGET			column,
 				    int				wide,
 				    int				high,
 				    int				heightResizable,
-				    APP_EVENT_HANDLER		redraw,
+				    APP_EVENT_HANDLER_T		redraw,
 				    void *			through )
     {
     APP_WIDGET	drawing;
@@ -1683,8 +1640,8 @@ void appGuiMakeDrawingAreaInColumn( APP_WIDGET *		pDrawing,
 	}
 
     if  ( high <= 0 || heightResizable )
-	{ XtSetArg( al[ac], XmNskipAdjust,	True ); ac++;	}
-    else{ XtSetArg( al[ac], XmNskipAdjust,	False ); ac++;	}
+	{ XtSetArg( al[ac], XmNskipAdjust,	False ); ac++;	}
+    else{ XtSetArg( al[ac], XmNskipAdjust,	True ); ac++;	}
 
     if  ( wide > 0 )
 	{
@@ -1700,6 +1657,9 @@ void appGuiMakeDrawingAreaInColumn( APP_WIDGET *		pDrawing,
 	XtAddEventHandler( drawing, ExposureMask, False, redraw, through );
 	}
 
+    XtAddCallback( drawing, XmNresizeCallback, appGuiSendExposeForResize,
+								(void *)0 );
+
     XtManageChild( drawing );
 
     *pDrawing= drawing;
@@ -1874,8 +1834,8 @@ static void appDragMouseMove(		APP_WIDGET		w,
 void appRunDragLoop(	APP_WIDGET		w,
 			EditApplication *	ea,
 			APP_EVENT *		downEvent,
-			APP_EVENT_HANDLER	upHandler,
-			APP_EVENT_HANDLER	moveHandler,
+			APP_EVENT_HANDLER_T	upHandler,
+			APP_EVENT_HANDLER_T	moveHandler,
 			int			timerInterval,
 			APP_TIMER_CALLBACK	timerHandler,
 			void *			through )
@@ -1918,14 +1878,14 @@ void appRunDragLoop(	APP_WIDGET		w,
     }
 
 void appGuiSetFocusChangeHandler(	APP_WIDGET		w,
-					APP_EVENT_HANDLER	handler,
+					APP_EVENT_HANDLER_T	handler,
 					void *			through )
     {
     XtAddEventHandler( w, FocusChangeMask, False, handler, through );
     }
 
 void appSetDestroyCallback(	APP_WIDGET		shell,
-				APP_DESTROY_CALLBACK	destroyCallback,
+				APP_DESTROY_CALLBACK_T	destroyCallback,
 				void *			through )
     {
     if  ( destroyCallback )
@@ -1937,7 +1897,7 @@ void appSetDestroyCallback(	APP_WIDGET		shell,
 
 void appSetCloseCallback(	APP_WIDGET		shell,
 				EditApplication *	ea,
-				APP_CLOSE_CALLBACK	closeCallback,
+				APP_CLOSE_CALLBACK_T	closeCallback,
 				void *			through )
     {
     if  ( closeCallback && ea->eaCloseAtom > 0 )
