@@ -1,10 +1,10 @@
-#   include	"config.h"
+#   include	"appFrameConfig.h"
 
 #   include	<stddef.h>
 #   include	<stdlib.h>
 
 #   include	<appMacPict.h>
-#   include	<appPs.h>
+#   include	<utilPs.h>
 #   include	<sioEndian.h>
 
 #   define	y0	math_y0
@@ -31,11 +31,14 @@
 #    define	PICTLOG(x)	/*nothing*/
 # endif
 
-int appMacPictPlayFilePs( FILE *			f,
+int appMacPictPlayFilePs( SimpleOutputStream *		sos,
 			SimpleInputStream *		sis,
 			const char *			afmDirectory,
-			int				xExt,
-			int				yExt,
+			int				useFilters,
+			int				indexedImages,
+			int				mapMode,
+			int				xWinExt,
+			int				yWinExt,
 			int				twipsWide,
 			int				twipsHigh )
     {
@@ -51,13 +54,13 @@ LDEB(0); return 0;
 /*									*/
 /************************************************************************/
 
-int appMacPictListFontsPs( SimpleInputStream *	sis,
+int appMacPictListFontsPs( PostScriptFaceList *	psfl,
+			SimpleInputStream *	sis,
 			const char *		afmDirectory,
 			const char *		prefix,
-			PostScriptFont **	pFontList,
-			int *			pCount,
-			int			xExt,
-			int			yExt,
+			int			mapMode,
+			int			xWinExt,
+			int			yWinExt,
 			int			twipsWide,
 			int			twipsHigh )
     {
