@@ -4,12 +4,14 @@
 /*									*/
 /************************************************************************/
 
-#   include	"config.h"
+#   include	"appFrameConfig.h"
 
 #   include	<stddef.h>
 #   include	<stdlib.h>
 #   include	<stdio.h>
+#   include	<string.h>
 #   include	<locale.h>
+#   include	<signal.h>
 
 #   include	<appSystem.h>
 
@@ -63,7 +65,7 @@ void appAppFileMini(	APP_WIDGET	option,
 /*									*/
 /************************************************************************/
 
-APP_CLOSE_CALLBACK( appAppWmClose, w, voidea )
+APP_CLOSE_CALLBACK_H( appAppWmClose, w, voidea )
     {
     EditApplication *		ea= (EditApplication *)voidea;
 
@@ -366,7 +368,7 @@ static void appAppSetWindowsOption(	APP_WIDGET		menu,
     ami.amiItemText= label;
     ami.amiKey= (char *)0;
     ami.amiKeyText= (char *)0;
-    ami.amiCallback= appDocToFront;
+    ami.amiCallback= (APP_MENU_CALLBACK)appDocToFront;
 
     windowsOption= appSetToggleMenuItem( menu, &(ed->edToplevel),
 							&ami, (void *)ed );
@@ -415,64 +417,82 @@ void appSetDocument(	EditApplication *	ea,
 /************************************************************************/
 
 static AppConfigurableResource	APP_ApplicationResourceTable[]=
-    {
-	APP_RESOURCE( "unit",
-		    offsetof(EditApplication,eaUnitString),
-		    "inch" ),
-	APP_RESOURCE( "paper",
-		    offsetof(EditApplication,eaPaperString),
-		    "letter" ),
-	APP_RESOURCE( "leftMargin",
-		    offsetof(EditApplication,eaLeftMarginString),
-		    "90pt" ),
-	APP_RESOURCE( "rightMargin",
-		    offsetof(EditApplication,eaRightMarginString),
-		    "90pt" ),
-	APP_RESOURCE( "topMargin",
-		    offsetof(EditApplication,eaTopMarginString),
-		    "72pt" ),
-	APP_RESOURCE( "bottomMargin",
-		    offsetof(EditApplication,eaBottomMarginString),
-		    "72pt" ),
-	APP_RESOURCE( "magnification",
-		    offsetof(EditApplication,eaMagnificationString),
-		    "1.2" ),
+{
+    APP_RESOURCE( "unit",
+		offsetof(EditApplication,eaUnitString),
+		"inch" ),
+    APP_RESOURCE( "paper",
+		offsetof(EditApplication,eaPaperString),
+		"letter" ),
+    APP_RESOURCE( "leftMargin",
+		offsetof(EditApplication,eaLeftMarginString),
+		"90pt" ),
+    APP_RESOURCE( "rightMargin",
+		offsetof(EditApplication,eaRightMarginString),
+		"90pt" ),
+    APP_RESOURCE( "topMargin",
+		offsetof(EditApplication,eaTopMarginString),
+		"72pt" ),
+    APP_RESOURCE( "bottomMargin",
+		offsetof(EditApplication,eaBottomMarginString),
+		"72pt" ),
+    APP_RESOURCE( "magnification",
+		offsetof(EditApplication,eaMagnificationString),
+		"1.2" ),
 
-	APP_RESOURCE( "author",
-		    offsetof(EditApplication,eaAuthor),
-		    (char *)0 ),
+    APP_RESOURCE( "supportXvCopyPaste",
+		offsetof(EditApplication,eaSupportXvCopyPasteString),
+		(char *)0 ),
 
-	APP_RESOURCE( "pageNumberFormat",
-		    offsetof(EditApplication,eaPageNumberFormat),
-		    "Page %d" ),
+    APP_RESOURCE( "hideSaveToOption",
+		offsetof(EditApplication,eaHideSaveToOptionString),
+		(char *)0 ),
 
-	APP_RESOURCE( "afmDirectory",
-		    offsetof(EditApplication,eaAfmDirectory),
-		    AFMDIR ),
-	APP_RESOURCE( "defaultFont",
-		    offsetof(EditApplication,eaDefaultFont),
-		    "Helvetica,,,10" ),
-	APP_RESOURCE( "faxCommand",
-		    offsetof(EditApplication,eaFaxCommand),
-		    (char *)0 ),
-	APP_RESOURCE( "customPrintCommand",
-		    offsetof(EditApplication,eaCustomPrintCommand),
-		    (char *)0 ),
-	APP_RESOURCE( "customPrinterName",
-		    offsetof(EditApplication,eaCustomPrinterName),
-		    (char *)0 ),
+    APP_RESOURCE( "usePostScriptFilters",
+		offsetof(EditApplication,eaUsePostScriptFiltersString),
+		(char *)0 ),
 
-	APP_RESOURCE( "focusColor",
-		    offsetof(EditApplication,eaFocusColor),
-		    "CadetBlue" ),
+    APP_RESOURCE( "usePostScriptIndexedImages",
+		offsetof(EditApplication,eaUsePostScriptIndexedImagesString),
+		(char *)0 ),
 
-	APP_RESOURCE( "rulerFont",
-		    offsetof(EditApplication,eaRulerFont),
-		    "-*-helvetica-medium-r-*-*-%d-*-*-*-*-*-iso8859-1" ),
-	APP_RESOURCE( "printDialogFont",
-		    offsetof(EditApplication,eaPrintDialogFont),
-		    "-*-lucidatypewriter-bold-r-*-*-%d-*-*-*-m-*-iso8859-1" ),
-    };
+    APP_RESOURCE( "author",
+		offsetof(EditApplication,eaAuthor),
+		(char *)0 ),
+
+    APP_RESOURCE( "pageNumberFormat",
+		offsetof(EditApplication,eaPageNumberFormat),
+		"Page %d" ),
+
+    APP_RESOURCE( "afmDirectory",
+		offsetof(EditApplication,eaAfmDirectory),
+		AFMDIR ),
+    APP_RESOURCE( "ghostscriptFontmap",
+		offsetof(EditApplication,eaGhostscriptFontmap),
+		(char *)0 ),
+    APP_RESOURCE( "ghostscriptFontToXmapping",
+		offsetof(EditApplication,eaGhostscriptFontToXmapping),
+		(char *)0 ),
+    APP_RESOURCE( "defaultFont",
+		offsetof(EditApplication,eaDefaultFont),
+		"Helvetica,,,10" ),
+    APP_RESOURCE( "faxCommand",
+		offsetof(EditApplication,eaFaxCommand),
+		(char *)0 ),
+    APP_RESOURCE( "customPrintCommand",
+		offsetof(EditApplication,eaCustomPrintCommand),
+		(char *)0 ),
+    APP_RESOURCE( "customPrinterName",
+		offsetof(EditApplication,eaCustomPrinterName),
+		(char *)0 ),
+
+    APP_RESOURCE( "rulerFont",
+		offsetof(EditApplication,eaRulerFont),
+		"-*-helvetica-medium-r-*-*-%d-*-*-*-*-*-iso8859-1" ),
+    APP_RESOURCE( "printDialogFont",
+		offsetof(EditApplication,eaPrintDialogFont),
+		"-*-lucidatypewriter-bold-r-*-*-%d-*-*-*-m-*-iso8859-1" ),
+};
 
 /************************************************************************/
 /*									*/
@@ -533,14 +553,35 @@ static void appInitializeGeometrySettings(	EditApplication *	ea )
 	    { SDEB(ea->eaBottomMarginString);	}
 	}
 
-    {
-    double			horPixPerMM;
-    double			verPixPerMM;
-    double			xfac;
-    double			yfac;
+    ea->eaSupportXvCopyPaste= 0;
+    if  ( ea->eaSupportXvCopyPasteString			&&
+	  ! strcmp( ea->eaSupportXvCopyPasteString, "1" )	)
+	{ ea->eaSupportXvCopyPaste= 1;	}
 
-    appGetFactors( ea, &horPixPerMM, &verPixPerMM, &xfac, &yfac );
-    }
+    ea->eaHideSaveToOption= 0;
+    if  ( ea->eaHideSaveToOptionString			&&
+	  ! strcmp( ea->eaHideSaveToOptionString, "1" )	)
+	{ ea->eaHideSaveToOption= 1;	}
+
+    ea->eaUsePostScriptFilters= 0;
+    if  ( ea->eaUsePostScriptFiltersString			&&
+	  ! strcmp( ea->eaUsePostScriptFiltersString, "1" )	)
+	{ ea->eaUsePostScriptFilters= 1;	}
+
+    ea->eaUsePostScriptIndexedImages= 0;
+    if  ( ea->eaUsePostScriptIndexedImagesString			&&
+	  ! strcmp( ea->eaUsePostScriptIndexedImagesString, "1" )	)
+	{ ea->eaUsePostScriptIndexedImages= 1;	}
+
+    if  ( ea->eaToplevel.atTopWidget )
+	{
+	double			horPixPerMM;
+	double			verPixPerMM;
+	double			xfac;
+	double			yfac;
+
+	appGetFactors( ea, &horPixPerMM, &verPixPerMM, &xfac, &yfac );
+	}
 
     return;
     }
@@ -658,28 +699,12 @@ static int appFinishApplicationWindow(	EditApplication *	ea )
 
 /************************************************************************/
 /*									*/
-/*  Generic main() procedure.						*/
-/*									*/
-/*  1)  Connect to the GUI system.					*/
-/*  2)  Retrieve resource values and derive some application wide	*/
-/*	settings from them.						*/
+/*  Resolve application resources.					*/
 /*									*/
 /************************************************************************/
 
-int appMain(	EditApplication *	ea,
-		int			argc,
-		char *			argv[] )
+void appGetApplicationResourceValues(	EditApplication *	ea )
     {
-    int				arg;
-
-    setlocale( LC_ALL, "" );
-
-    /*  1  */
-    if  ( appGuiInitApplication( ea, &argc, &argv ) )
-	{ LDEB(1); return 1;	}
-
-    appInitDocumentGeometry( &(ea->eaDefaultDocumentGeometry) );
-
     /*  b  */
     appGuiGetResourceValues( ea, ea->eaResourceData,
 				ea->eaResourceTable, ea->eaResourceCount );
@@ -695,6 +720,169 @@ int appMain(	EditApplication *	ea,
 
     appInitializeGeometrySettings( ea );
 
+    return;
+    }
+
+/************************************************************************/
+/*									*/
+/*  Handle Special Calls.						*/
+/*									*/
+/*  1)  It is not necessary to initialize TedApplication to convert	*/
+/*	files. Intercept conversion calls to prevent the program from	*/
+/*	becoming interactive. NOTE that the calls differfrom the calls	*/
+/*	that initialize the application: They begin with -- instead of	*/
+/*	++ . To override configurable options, the ++Something calls	*/
+/*	should be used.							*/
+/*									*/
+/************************************************************************/
+
+static int appMainHandleSpecialCall(	EditApplication *	ea,
+					const char *		prefix,
+					int			getResources,
+					int			argc,
+					char *			argv[] )
+    {
+    int			prefixLength= strlen( prefix );
+    const char *	command;
+
+    if  ( argc < 2 )
+	{ return 0;	}
+
+    if  ( strncmp( argv[1], prefix, prefixLength ) )
+	{ return 0;	}
+
+    command= argv[1]+ prefixLength;
+
+    /*  1  */
+    if  ( argc == 4				&&
+	  ! strcmp( command, "saveTo" )	)
+	{
+	if  ( getResources )
+	    { appGetApplicationResourceValues( ea );	}
+
+	if  ( appFileConvert( ea, argv[2], argv[3] ) )
+	    { SSSDEB(argv[1],argv[2],argv[3]); return -1; }
+
+	return 4;
+	}
+
+    /**  print to file **/
+
+    if  ( argc == 4				&&
+	  ! strcmp( command, "printToFile" )	)
+	{
+	if  ( getResources )
+	    { appGetApplicationResourceValues( ea );	}
+
+	if  ( appPrintToFile( ea, argv[2], argv[3], (const char *)0 ) )
+	    { SSSDEB(argv[1],argv[2],argv[3]); return -1; }
+
+	return 4;
+	}
+
+    if  ( argc == 5					&&
+	  ! strcmp( command, "printToFilePaper" )	)
+	{
+	if  ( getResources )
+	    { appGetApplicationResourceValues( ea );	}
+
+	if  ( appPrintToFile( ea, argv[2], argv[3], argv[4] ) )
+	    { SSSDEB(argv[1],argv[2],argv[3]); return -1; }
+
+	return 5;
+	}
+
+    /**  print to printer **/
+
+    if  ( argc == 3			&&
+	  ! strcmp( command, "print" )	)
+	{
+	if  ( getResources )
+	    { appGetApplicationResourceValues( ea );	}
+
+	if  ( appPrintToPrinter( ea, argv[2],
+					(const char *)0, (const char *)0 ) )
+	    { SSDEB(argv[1],argv[2]); return -1; }
+
+	return 3;
+	}
+
+    if  ( argc == 4				&&
+	  ! strcmp( command, "printPaper" )	)
+	{
+	if  ( getResources )
+	    { appGetApplicationResourceValues( ea );	}
+
+	if  ( appPrintToPrinter( ea, argv[2], (const char *)0, argv[3] ) )
+	    { SSDEB(argv[1],argv[2]); return -1; }
+
+	return 4;
+	}
+
+    if  ( argc == 4				&&
+	  ! strcmp( command, "printToPrinter" )	)
+	{
+	if  ( getResources )
+	    { appGetApplicationResourceValues( ea );	}
+
+	if  ( appPrintToPrinter( ea, argv[2], argv[3], (const char *)0 ) )
+	    { SSDEB(argv[1],argv[2]); return -1; }
+
+	return 4;
+	}
+
+    if  ( argc == 5					&&
+	  ! strcmp( command, "printToPrinterPaper" )	)
+	{
+	if  ( getResources )
+	    { appGetApplicationResourceValues( ea );	}
+
+	if  ( appPrintToPrinter( ea, argv[2], argv[3], argv[4] ) )
+	    { SSDEB(argv[1],argv[2]); return -1; }
+
+	return 5;
+	}
+
+    /**  ----- **/
+
+    return 0;
+    }
+
+/************************************************************************/
+/*									*/
+/*  Generic main() procedure.						*/
+/*									*/
+/*  1)  Connect to the GUI system.					*/
+/*  2)  Retrieve resource values and derive some application wide	*/
+/*	settings from them.						*/
+/*  3)  Do not die too easily.						*/
+/*									*/
+/************************************************************************/
+
+int appMain(	EditApplication *	ea,
+		int			argc,
+		char *			argv[] )
+    {
+    int			arg;
+    int			res;
+
+    setlocale( LC_ALL, "" );
+
+    res= appMainHandleSpecialCall( ea, "--", 1, argc, argv );
+    if  ( res < 0 )
+	{ LDEB(res); return 1;	}
+    if  ( res > 0 )
+	{ return 0;		}
+
+    /*  1  */
+    if  ( appGuiInitApplication( ea, &argc, &argv ) )
+	{ LDEB(1); return 1;	}
+
+    appInitDocumentGeometry( &(ea->eaDefaultDocumentGeometry) );
+
+    /*  b  */
+    appGetApplicationResourceValues( ea );
+
     if  ( ea->eaPrintDestinationCount == 0				&&
 	  utilPrinterGetPrinters( &(ea->eaPrintDestinationCount),
 				    &(ea->eaDefaultPrintDestination),
@@ -716,36 +904,19 @@ int appMain(	EditApplication *	ea,
 	}
 #   endif
 
-    if  ( argc == 4				&&
-	  ! strcmp( argv[1], "++saveTo" )	)
-	{
-	if  ( appFileConvert( ea, argv[2], argv[3] ) )
-	    { SSSDEB(argv[1],argv[2],argv[3]); return 1; }
+    res= appMainHandleSpecialCall( ea, "++", 0, argc, argv );
+    if  ( res < 0 )
+	{ LDEB(res); return 1;	}
+    if  ( res > 0 )
+	{ return 0;		}
 
-	return 0;
-	}
-
-    if  ( argc == 4				&&
-	  ! strcmp( argv[1], "++printToFile" )	)
-	{
-	if  ( appPrintToFile( ea, argv[2], argv[3], (const char *)0 ) )
-	    { SSSDEB(argv[1],argv[2],argv[3]); return 1; }
-
-	return 0;
-	}
-
-    if  ( argc == 5					&&
-	  ! strcmp( argv[1], "++printToFilePaper" )	)
-	{
-	if  ( appPrintToFile( ea, argv[2], argv[3], argv[4] ) )
-	    { SSSDEB(argv[1],argv[2],argv[3]); return 1; }
-
-	return 0;
-	}
+    /*  3  */
+    (void) signal( SIGHUP, SIG_IGN );
+    (void) signal( SIGINT, SIG_IGN );
 
     for ( arg= 1; arg < argc; arg++ )
 	{
-	const int			readonly= 0;
+	const int			read_only= 0;
 	AppFileMessageResources *	afmr= &(ea->eaFileMessageResources);
 	static char			absolute[1000+1];
 
@@ -757,7 +928,7 @@ int appMain(	EditApplication *	ea,
 		{ LLDEB(arg,argc); arg= argc- 1; continue;	}
 
 	    ed= appOpenDocument( ea, ea->eaToplevel.atTopWidget, (APP_WIDGET)0,
-						readonly, argv[arg+ 1] );
+						read_only, argv[arg+ 1] );
 
 	    if  ( ed )
 		{
@@ -779,7 +950,7 @@ int appMain(	EditApplication *	ea,
 		{ LLDEB(arg,argc); arg= argc- 1; continue;	}
 
 	    ed= appOpenDocument( ea, ea->eaToplevel.atTopWidget, (APP_WIDGET)0,
-						readonly, argv[arg+ 1] );
+						read_only, argv[arg+ 1] );
 
 	    if  ( ed )
 		{
@@ -833,15 +1004,18 @@ int appMain(	EditApplication *	ea,
 		}
 	    }
 	else{
-	    if  ( appAbsoluteName( absolute, 1000, argv[arg] ) < 0 )
+	    const char * const	fileRelativeTo= (const char *)0;
+
+	    if  ( appAbsoluteName( absolute, 1000,
+					argv[arg], fileRelativeTo ) < 0 )
 		{
 		SDEB(argv[arg]);
 		ed= appOpenDocument( ea, ea->eaToplevel.atTopWidget,
-					(APP_WIDGET)0, readonly, argv[arg] );
+					(APP_WIDGET)0, read_only, argv[arg] );
 		}
 	    else{
 		ed= appOpenDocument( ea, ea->eaToplevel.atTopWidget,
-					(APP_WIDGET)0, readonly, absolute );
+					(APP_WIDGET)0, read_only, absolute );
 		}
 	    }
 	}
