@@ -1,8 +1,7 @@
-#   include	"config.h"
+#   include	"appFrameConfig.h"
 
 #   include	<stdlib.h>
 #   include	<stdio.h>
-#   include	<appDebugon.h>
 
 #   include	"appFrame.h"
 #   include	"appSystem.h"
@@ -10,12 +9,13 @@
 
 #   ifdef USE_MOTIF
 
-#   include	<X11/IntrinsicP.h>
 #   include	<Xm/RowColumn.h>
 #   include	<Xm/PushB.h>
 #   include	<Xm/PushBG.h>
 #   include	<Xm/PanedW.h>
 
+#   include	<appDebugon.h>
+
 /************************************************************************/
 /*									*/
 /*  Get around Motif's ridiculous resize behavior.			*/
@@ -74,33 +74,6 @@ void appOptionmenuRefreshWidth(	AppOptionmenu *		aom )
     return;
     }
 
-void appPixmapOptionmenuSetButtonMargins(	AppOptionmenu *	aom )
-    {
-    Display *		display= XtDisplay( aom->aomInplace );
-    int			screen= DefaultScreen( display );
-    int			depth= DefaultDepth( display, screen );
-
-    Widget		buttonGadget;
-
-    APP_BITMAP_IMAGE	pixmap;
-
-    pixmap= XCreatePixmap( display, XtWindow( aom->aomInplace ), 1, 1, depth );
-
-    buttonGadget= XmOptionButtonGadget( aom->aomInplace );
-    XtVaSetValues( buttonGadget,
-			XmNcascadePixmap,	pixmap,
-			XmNmarginRight,		0,
-			XmNmarginTop,		0,
-			XmNmarginBottom,	2,
-			XmNmarginHeight,	2,
-			XmNmarginRight,		0,
-			/*
-			XmNbackground,		WhitePixel( display, screen ),
-			*/
-			NULL );
-    return;
-    }
-
 /************************************************************************/
 /*									*/
 /*  Make a pulldown that is usable as a child of a form.		*/
@@ -272,36 +245,6 @@ APP_WIDGET appAddItemToOptionmenu(	AppOptionmenu *		aom,
     return fresh;
     }
 
-APP_WIDGET appAddPixmapItemToOptionmenu( AppOptionmenu *	aom,
-					XtCallbackProc		callBack,
-					void *			target )
-    {
-    Widget	fresh;
-
-    Arg		al[20];
-    int		ac= 0;
-
-    /*
-    Display *		display= XtDisplay( aom->aomInplace );
-    int			screen= DefaultScreen( display );
-    */
-
-    XtSetArg( al[ac], XmNlabelType,	XmPIXMAP ); ac++;
-    XtSetArg( al[ac], XmNlabelPixmap,	XmUNSPECIFIED_PIXMAP ); ac++;
-    /*
-    XtSetArg( al[ac], XmNbackground,	WhitePixel( display, screen ) ); ac++;
-    */
-
-    fresh= XmCreatePushButton( aom->aomPulldown, WIDGET_NAME, al, ac );
-
-    if  ( callBack )
-	{ XtAddCallback( fresh, XmNactivateCallback, callBack, target ); }
-
-    XtManageChild( fresh );
-
-    return fresh;
-    }
-
 /************************************************************************/
 /*									*/
 /*  Select a particular option in an option menu.			*/
@@ -350,4 +293,31 @@ void appInitOptionmenu(		AppOptionmenu *		aom )
     aom->aomPulldown= (APP_WIDGET)0;
     aom->aomInplace= (APP_WIDGET)0;
     }
+
+int appGuiGetOptionmenuItemIndexMotif(	AppOptionmenu *	aom,
+					Widget		w )
+    {
+    short		pos= -1;
+
+    XtVaGetValues( w,	XmNpositionIndex,	&pos,
+			NULL );
+
+    return pos;
+    }
+
+void appOptionmenuItemSetVisibility(	APP_WIDGET	w,
+					int		visible )
+    {
+    visible= ( visible != 0 );
+
+    if  ( XtIsManaged( w ) == visible )
+	{ return;	}
+
+    if  ( visible )
+	{ XtManageChild( w );	}
+    else{ XtUnmanageChild( w );	}
+
+    return;
+    }
+
 #   endif
