@@ -4,11 +4,13 @@
 /*									*/
 /************************************************************************/
 
-#   include	"config.h"
+#   include	"appFrameConfig.h"
 
 #   include	<stddef.h>
 #   include	<stdlib.h>
 #   include	<stdio.h>
+#   include	<string.h>
+
 #   include	<appDebugon.h>
 
 #   include	<appFrame.h>
@@ -17,7 +19,7 @@
 #   include	"appPaperChooser.h"
 #   include	"appPaper.h"
 
-#   define	DRH_MM	45
+#   define	DRH_MM	40
 
 /************************************************************************/
 /*									*/
@@ -35,25 +37,34 @@
 
 typedef struct PlacementSpecification
     {
-    int		psNup;
-    int		psHorizontal;
+    int		psGridRows;
+    int		psGridCols;
+    int		psGridHorizontal;
+    int		psScalePagesToFit;
     char *	psOptionText;
     APP_WIDGET	psOptionWidget;
     } PlacementSpecification;
 
-#   define	PRINTps_1_UP	0
-#   define	PRINTps_2_UP	1
-#   define	PRINTps_4H_UP	2
-#   define	PRINTps_4V_UP	3
-
-#   define	PRINTpsCOUNT	4
-
-static PlacementSpecification	APP_Placements[]=
+typedef enum PlacementSpecificationNumber
     {
-	{ 1, 0, "1 Up"		},
-	{ 2, 1, "2 Up"		},
-	{ 4, 1, "4 Up Hor"	},
-	{ 4, 0, "4 Up Ver"	},
+    PSnum_1x1_ASIS= 0,
+    PSnum_1x1_SCALED,
+    PSnum_2x1_HOR,
+    PSnum_1x2_HOR,
+    PSnum_2x2_HOR,
+    PSnum_2x2_VER,
+
+    PSnum__COUNT
+    } PlacementSpecificationNumber;
+
+static PlacementSpecification	APP_Placements[PSnum__COUNT]=
+    {
+	{ 1, 1, 0, 0, "1x1 As Is"	},
+	{ 1, 1, 1, 1, "1x1 Scale"	},
+	{ 2, 1, 1, 1, "2x1 Hor"		},
+	{ 1, 2, 1, 1, "1x2 Hor"		},
+	{ 2, 2, 1, 1, "2x2 Hor"		},
+	{ 2, 2, 0, 1, "2x2 Ver"		},
     };
 
 typedef struct AppPrintDialog
@@ -67,15 +78,17 @@ typedef struct AppPrintDialog
     AppOptionmenu		apdPlacementOptionmenu;
     AppOptionmenu		apdSelectionOptionmenu;
 
+    APP_WIDGET			apdCustomTransformRow;
+    APP_WIDGET			apdRotate90Toggle;
+    APP_WIDGET			apdCenterHToggle;
+
     int				apdUnitType;
     PaperChooser		apdPaperChooser;
 
     APP_WIDGET			apdFaxNumberText;
-    APP_WIDGET			apdPrintToFileOption;
 
     APP_WIDGET			apdPageDrawing;
     AppDrawingData		apdDrawingData;
-    int				apdDrawingDataAllocated;
     APP_FONT *			apdFontStruct;
 
     APP_WIDGET			apdSelectionOptions[PRINTselCOUNT];
@@ -84,16 +97,23 @@ typedef struct AppPrintDialog
 
     int				apdPrinterCount;
     int				apdPrintOptionCount;
-    int				apdDestinationChosen;
+    int				apdOptionNrChosen;
+    int				apdVisiblePrinterCount;
 
     int				apdActivateFaxOptions;
-    int				apdFileDestination;
-    int				apdFaxDestination;
+    int				apdToFileOptionNr;
+    int				apdFaxOptionNr;
+    int				apdUpOptionNr;
+    int				apdFirstPrinterOptionNr;
+    int				apdDownOptionNr;
+    int				apdFirstVisiblePrinter;
 
     DocumentGeometry		apdDocumentGeometry;
     DocumentGeometry		apdPrinterGeometry;
-    PlacementSpecification	apdPlacements[PRINTpsCOUNT];
+    PlacementSpecification	apdPlacements[PSnum__COUNT];
     int				apdPlacementChosen;
+    int				apdRotate90Chosen;
+    int				apdCenterHChosen;
 
     int				apdPageCount;
     int				apdFirstSelectedPage;
@@ -111,10 +131,17 @@ typedef struct AppPrintDialog
 
     char *			apdFaxText;
     char *			apdPrintToFileText;
+    char *			apdVisiblePrinterCountText;
+    char *			apdPrintersUpText;
+    char *			apdPrintersDownText;
 
     char *			apdPaperSizeText;
     char *			apdCustomPaperSizeText;
     char *			apdPlacementText;
+    char *			apdShowPlacementDrawingString;
+    int				apdShowPlacementDrawingInt;
+    char *			apdRotate90Text;
+    char *			apdCenterHText;
 
     char *			apdSelectionText;
     char *			apdAllPagesText;
@@ -125,12 +152,9 @@ typedef struct AppPrintDialog
 
     char *			apdDefaultPrinter;
 
-    APP_WIDGET			apdPrinterOptions[1]; /* LAST! */
-
+    APP_WIDGET			apdPrinterOptions[1]; 	/* LAST! */
     } AppPrintDialog;
 
-static AppPrintDialog *	APP_PrintDialog;
-
 static AppConfigurableResource APP_PrintDialogresourceTable[]=
     {
 	APP_RESOURCE( "printDialogPrinter",
@@ -159,27 +183,48 @@ static AppConfigurableResource APP_PrintDialogresourceTable[]=
 		    offsetof(AppPrintDialog,apdCustomPaperSizeText),
 		    "Custom" ),
 
+	/***/
 	APP_RESOURCE( "printDialogPlacement",
 		    offsetof(AppPrintDialog,apdPlacementText),
 		    "Placement" ),
 
+	APP_RESOURCE( "printDialogShowPlacementDrawing",
+		    offsetof(AppPrintDialog,apdShowPlacementDrawingString),
+		    "1" ),
+
 	APP_RESOURCE( "printDialog_1_UpText",
 		    offsetof(AppPrintDialog,
-				apdPlacements[PRINTps_1_UP].psOptionText),
+				apdPlacements[PSnum_1x1_ASIS].psOptionText),
 		    "1 Page/Sheet" ),
-	APP_RESOURCE( "printDialog_2_UpText",
+	APP_RESOURCE( "printDialog_1_ScaledText",
 		    offsetof(AppPrintDialog,
-				apdPlacements[PRINTps_2_UP].psOptionText),
-		    "2 Pages/Sheet" ),
+				apdPlacements[PSnum_1x1_SCALED].psOptionText),
+		    "1 Page/Sheet Scale to Fit" ),
+	APP_RESOURCE( "printDialog_2Row_UpText",
+		    offsetof(AppPrintDialog,
+				apdPlacements[PSnum_2x1_HOR].psOptionText),
+		    "2 Pages/Sheet Vertical" ),
+	APP_RESOURCE( "printDialog_2Col_UpText",
+		    offsetof(AppPrintDialog,
+				apdPlacements[PSnum_1x2_HOR].psOptionText),
+		    "2 Pages/Sheet Horizontal" ),
 	APP_RESOURCE( "printDialog_4H_UpText",
 		    offsetof(AppPrintDialog,
-				apdPlacements[PRINTps_4H_UP].psOptionText),
+				apdPlacements[PSnum_2x2_HOR].psOptionText),
 		    "4 Pages/Sheet Horizontal" ),
 	APP_RESOURCE( "printDialog_4V_UpText",
 		    offsetof(AppPrintDialog,
-				apdPlacements[PRINTps_4V_UP].psOptionText),
+				apdPlacements[PSnum_2x2_VER].psOptionText),
 		    "4 Pages/Sheet Vertical" ),
 
+	APP_RESOURCE( "printDialogRotate90",
+		    offsetof(AppPrintDialog,apdRotate90Text),
+		    "Rotate 90\260" ),
+	APP_RESOURCE( "printDialogCenterH",
+		    offsetof(AppPrintDialog,apdCenterHText),
+		    "Center Horizontally" ),
+
+	/***/
 	APP_RESOURCE( "printDialogSelection",
 		    offsetof(AppPrintDialog,apdSelectionText),
 		    "Selection" ),
@@ -204,8 +249,105 @@ static AppConfigurableResource APP_PrintDialogresourceTable[]=
 	APP_RESOURCE( "defaultPrinter",
 		    offsetof(AppPrintDialog,apdDefaultPrinter),
 		    "" ),
+
+	APP_RESOURCE( "printDialogPrinterCount",
+		    offsetof(AppPrintDialog,apdVisiblePrinterCountText),
+		    "15" ),
+	APP_RESOURCE( "printDialogPrintersUp",
+		    offsetof(AppPrintDialog,apdPrintersUpText),
+		    "%d up" ),
+	APP_RESOURCE( "printDialogPrintersDown",
+		    offsetof(AppPrintDialog,apdPrintersDownText),
+		    "%d down" ),
     };
 
+/************************************************************************/
+/*									*/
+/*  Adapt the menu with printers to the currently selected range of	*/
+/*  printers.								*/
+/*									*/
+/*  1)  All printers can be shown.. No need to offer browsing options	*/
+/*  2)  Make the browsing options visible.				*/
+/*  3)  Hide all printers before the fist visible one.			*/
+/*  4)  Show the range of visible printers.				*/
+/*  5)  Hide all printers after the last visible one.			*/
+/*  6)  Enable/Disable the browsing options depending on whether there	*/
+/*	are any printers before/after the current range.		*/
+/*									*/
+/************************************************************************/
+
+static void appPrintDialogMenuRange(	AppPrintDialog *	apd )
+    {
+    int		i;
+    int		gotBefore= 0;
+    int		gotAfter= 0;
+
+    /*  1  */
+    if  ( apd->apdPrinterCount <= apd->apdVisiblePrinterCount )
+	{
+	if  ( apd->apdUpOptionNr >= 0 )
+	    {
+	    appOptionmenuItemSetVisibility(
+			    apd->apdPrinterOptions[apd->apdUpOptionNr], 0 );
+	    }
+
+	if  ( apd->apdDownOptionNr >= 0 )
+	    {
+	    appOptionmenuItemSetVisibility(
+			    apd->apdPrinterOptions[apd->apdDownOptionNr], 0 );
+	    }
+
+	return;
+	}
+
+    /*  2  */
+    appOptionmenuItemSetVisibility(
+			    apd->apdPrinterOptions[apd->apdUpOptionNr], 1 );
+    appOptionmenuItemSetVisibility(
+			    apd->apdPrinterOptions[apd->apdDownOptionNr], 1 );
+
+    /*  3  */
+    for ( i= 0; i < apd->apdFirstVisiblePrinter; i++ )
+	{
+	int	opt= apd->apdFirstPrinterOptionNr+ i;
+
+	appOptionmenuItemSetVisibility( apd->apdPrinterOptions[opt], 0 );
+	gotBefore= 1;
+	}
+
+    /*  4  */
+    for ( i= apd->apdFirstVisiblePrinter;
+	  i < apd->apdFirstVisiblePrinter+ apd->apdVisiblePrinterCount &&
+	  i < apd->apdPrinterCount;
+	  i++ )
+	{
+	int	opt= apd->apdFirstPrinterOptionNr+ i;
+
+	appOptionmenuItemSetVisibility( apd->apdPrinterOptions[opt], 1 );
+	}
+
+    /*  5  */
+    for ( i= apd->apdFirstVisiblePrinter+ apd->apdVisiblePrinterCount;
+	  i < apd->apdPrinterCount;
+	  i++ )
+	{
+	int	opt= apd->apdFirstPrinterOptionNr+ i;
+
+	appOptionmenuItemSetVisibility( apd->apdPrinterOptions[opt], 0 );
+	gotAfter= 1;
+	}
+
+    /*  6  */
+    appGuiEnableWidget(
+		    apd->apdPrinterOptions[apd->apdUpOptionNr], gotBefore );
+    appGuiEnableWidget(
+		    apd->apdPrinterOptions[apd->apdDownOptionNr], gotAfter );
+
+    appOptionmenuRefreshWidth( &(apd->apdPrinterOptionmenu) );
+
+    return;
+    }
+
 /************************************************************************/
 /*									*/
 /*  Show a range of pages in the Print Dialog.				*/
@@ -249,9 +391,7 @@ static void appPrintCheckFaxNumber(	AppPrintDialog *	apd )
     return;
     }
 
-static void appPrintFaxChanged(	APP_WIDGET	w,
-				void *		voidapd,
-				void *		call_data )
+static APP_TXTYPING_CALLBACK_H( appPrintFaxChanged, w, voidapd )
     {
     AppPrintDialog *	apd= (AppPrintDialog *)voidapd;
 
@@ -268,7 +408,7 @@ static void appPrintFaxChanged(	APP_WIDGET	w,
 
 static void appPrinterAdaptToDestination(	AppPrintDialog *	apd )
     {
-    if  ( apd->apdDestinationChosen == apd->apdFaxDestination )
+    if  ( apd->apdOptionNrChosen == apd->apdFaxOptionNr )
 	{
 	if  ( apd->apdFaxNumberText )
 	    {
@@ -290,55 +430,116 @@ static void appPrinterAdaptToDestination(	AppPrintDialog *	apd )
 	}
     }
 
-static void appDestinationChosen(	APP_WIDGET	w,
-					void *		voidapd,
-					void *		voidpbcs )
+static APP_OITEM_CALLBACK_H( appDestinationChosen, w, voidapd )
     {
     AppPrintDialog *	apd= (AppPrintDialog *)voidapd;
     short		n= -1;
 
-    for ( n= 0; n < apd->apdPrintOptionCount; n++ )
-	{
-	if  ( apd->apdPrinterOptions[n] == w )
-	    { break;	}
-	}
-
+    n= appGuiGetOptionmenuItemIndex( &(apd->apdPrinterOptionmenu), w );
     if  ( n < 0 || n >= apd->apdPrintOptionCount )
 	{ LLDEB(n,apd->apdPrintOptionCount); return;	}
 
-    apd->apdDestinationChosen= n;
+    apd->apdOptionNrChosen= n;
 
     appPrinterAdaptToDestination( apd );
 
     return;
     }
 
+static APP_OITEM_CALLBACK_H( appPrintersUpChosen, w, voidapd )
+    {
+    AppPrintDialog *	apd= (AppPrintDialog *)voidapd;
+    int			printerChosen;
+
+    apd->apdFirstVisiblePrinter -= apd->apdVisiblePrinterCount;
+    if  ( apd->apdFirstVisiblePrinter < 0 )
+	{ apd->apdFirstVisiblePrinter= 0;	}
+
+    printerChosen= apd->apdFirstVisiblePrinter;
+    apd->apdOptionNrChosen= apd->apdFirstPrinterOptionNr+ printerChosen;
+
+    appPrintDialogMenuRange( apd );
+    appSetOptionmenu( &(apd->apdPrinterOptionmenu), apd->apdOptionNrChosen );
+
+    return;
+    }
+
+static APP_OITEM_CALLBACK_H( appPrintersDownChosen, w, voidapd )
+    {
+    AppPrintDialog *	apd= (AppPrintDialog *)voidapd;
+    int			printerChosen;
+
+    apd->apdFirstVisiblePrinter += apd->apdVisiblePrinterCount;
+    if  ( apd->apdFirstVisiblePrinter+ apd->apdVisiblePrinterCount >
+						    apd->apdPrinterCount )
+	{
+	apd->apdFirstVisiblePrinter=
+			apd->apdPrinterCount- apd->apdVisiblePrinterCount;
+	if  ( apd->apdFirstVisiblePrinter < 0 )
+	    { apd->apdFirstVisiblePrinter= 0;	}
+	}
+
+    printerChosen= apd->apdFirstVisiblePrinter;
+    apd->apdOptionNrChosen= apd->apdFirstPrinterOptionNr+ printerChosen;
+
+    appPrintDialogMenuRange( apd );
+    appSetOptionmenu( &(apd->apdPrinterOptionmenu), apd->apdOptionNrChosen );
+
+    return;
+    }
+
+/************************************************************************/
+/*									*/
+/*  Refresh placement drawing.						*/
+/*									*/
+/************************************************************************/
+
+static void appPrintDialogRefreshPlacementDrawing(
+					    AppPrintDialog *	apd )
+    {
+    AppDrawingData *	add= &(apd->apdDrawingData);
+
+    if  ( ! apd->apdPageDrawing )
+	{ return;	}
+
+    appExposeRectangle( add, 0, 0, 0, 0 );
+    }
+
 /************************************************************************/
 /*									*/
 /*  A placement was chosen.						*/
 /*									*/
 /************************************************************************/
 
-static void appPlacementChosen(		APP_WIDGET	w,
-					void *		voidapd,
-					void *		voidpbcs )
+static APP_OITEM_CALLBACK_H( appPrintDialogPlacementChosen, w, voidapd )
     {
     AppPrintDialog *	apd= (AppPrintDialog *)voidapd;
-    AppDrawingData *	add= &(apd->apdDrawingData);
     int			i;
 
-    for ( i= 0; i < PRINTpsCOUNT; i++ )
-	{
-	if  ( apd->apdPlacements[i].psOptionWidget == w )
-	    { break;	}
-	}
-
-    if  ( i < 0 || i >= PRINTpsCOUNT )
-	{ LLDEB(i,PRINTpsCOUNT); return;	}
+    i= appGuiGetOptionmenuItemIndex( &(apd->apdPlacementOptionmenu), w );
+    if  ( i < 0 || i >= PSnum__COUNT )
+	{ LLDEB(i,PSnum__COUNT); return;	}
 
     apd->apdPlacementChosen= i;
 
-    appExposeRectangle( add, 0, 0, 0, 0 );
+    if  ( apd->apdCustomTransformRow )
+	{
+	int	rotate90= apd->apdRotate90Chosen;
+	int	centerHorizontally= apd->apdCenterHChosen;
+
+	if  ( apd->apdPlacementChosen != 0 )
+	    { rotate90= centerHorizontally= 0;	}
+
+	appGuiEnableWidget( apd->apdRotate90Toggle,
+				    apd->apdPlacementChosen == 0 );
+	appGuiEnableWidget( apd->apdCenterHToggle,
+				    apd->apdPlacementChosen == 0 );
+
+	appGuiSetToggleState( apd->apdRotate90Toggle, rotate90 );
+	appGuiSetToggleState( apd->apdCenterHToggle, centerHorizontally );
+	}
+
+    appPrintDialogRefreshPlacementDrawing( apd );
 
     return;
     }
@@ -349,9 +550,7 @@ static void appPlacementChosen(		APP_WIDGET	w,
 /*									*/
 /************************************************************************/
 
-static void appSelectionChosen(		APP_WIDGET	w,
-					void *		voidapd,
-					void *		voidpbcs )
+static APP_OITEM_CALLBACK_H( appSelectionChosen, w, voidapd )
     {
     AppPrintDialog *	apd= (AppPrintDialog *)voidapd;
     int			n;
@@ -362,12 +561,7 @@ static void appSelectionChosen(		APP_WIDGET	w,
 				    RevertToNone, CurrentTime );
 #   endif
 
-    for ( n= 0; n < PRINTselCOUNT; n++ )
-	{
-	if  ( apd->apdSelectionOptions[n] == w )
-	    { break;	}
-	}
-
+    n= appGuiGetOptionmenuItemIndex( &(apd->apdSelectionOptionmenu), w );
     if  ( n < 0 || n >= PRINTselCOUNT )
 	{ LLDEB(n,PRINTselCOUNT); return;	}
 
@@ -412,9 +606,7 @@ static void appSelectionChosen(		APP_WIDGET	w,
 /*									*/
 /************************************************************************/
 
-static void appPrintDialogCancelPushed(	APP_WIDGET	w,
-					void *		voidapd,
-					void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( appPrintDialogCancelPushed, w, voidapd )
     {
     AppPrintDialog *	apd= (AppPrintDialog *)voidapd;
 
@@ -423,14 +615,12 @@ static void appPrintDialogCancelPushed(	APP_WIDGET	w,
     return;
     }
 
-static void appPrintDialogPrintPushed(	APP_WIDGET	w,
-					void *		voidapd,
-					void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( appPrintDialogPrintPushed, w, voidapd )
     {
     AppPrintDialog *	apd= (AppPrintDialog *)voidapd;
 
-    if  ( apd->apdDestinationChosen < 0	)
-	{ LDEB(apd->apdDestinationChosen); return;	}
+    if  ( apd->apdOptionNrChosen < 0	)
+	{ LDEB(apd->apdOptionNrChosen); return;	}
 
     if  ( apd->apdSelectionChosen == PRINTselPAGE_RANGE )
 	{
@@ -460,38 +650,51 @@ static void appPrintDialogPrintPushed(	APP_WIDGET	w,
 /*									*/
 /************************************************************************/
 
-static void appPrintDialogSizeChosen(	APP_WIDGET	w,
-					void *		voidapd,
-					void *		voidpbcs )
+static void appPrintDialogPaperRectChanged(
+				    PaperChooser *		pc,
+				    void *			voidapd,
+				    const DocumentGeometry *	dg )
     {
     AppPrintDialog *		apd= (AppPrintDialog *)voidapd;
-    AppDrawingData *		add= &(apd->apdDrawingData);
 
-    appPaperSizeChosen( w, &(apd->apdPaperChooser),
-						&(apd->apdPrinterGeometry),
-						apd->apdUnitType );
+    apd->apdPrinterGeometry.dgPageWideTwips= dg->dgPageWideTwips;
+    apd->apdPrinterGeometry.dgPageHighTwips= dg->dgPageHighTwips;
 
-    appExposeRectangle( add, 0, 0, 0, 0 );
+    appPrintDialogRefreshPlacementDrawing( apd );
 
     return;
     }
 
-static void appPrintDialogPaperSizeChanged(	APP_WIDGET	w,
-						void *		voidapd,
-						void *		voidcbs	 )
+/************************************************************************/
+/*									*/
+/*  The custom placement toggles were activated.			*/
+/*									*/
+/************************************************************************/
+
+static APP_TOGGLE_CALLBACK_H( appPrintDialogRotate90Toggled, w, voidapd, e )
     {
     AppPrintDialog *		apd= (AppPrintDialog *)voidapd;
-    AppDrawingData *		add= &(apd->apdDrawingData);
-    const int			set= 0;
+    int				set;
 
-    appPaperSizeChanged( &(apd->apdPaperChooser),
-						&(apd->apdPrinterGeometry),
-						apd->apdUnitType );
+    set= appGuiGetToggleStateFromCallback( w, e );
 
-    appPaperChooserAdaptToGeometry( &(apd->apdPaperChooser),
-			apd->apdUnitType, &(apd->apdPrinterGeometry), set );
+    apd->apdRotate90Chosen= ( set != 0 );
 
-    appExposeRectangle( add, 0, 0, 0, 0 );
+    appPrintDialogRefreshPlacementDrawing( apd );
+
+    return;
+    }
+
+static APP_TOGGLE_CALLBACK_H( appPrintDialogCenterHToggled, w, voidapd, e )
+    {
+    AppPrintDialog *		apd= (AppPrintDialog *)voidapd;
+    int				set;
+
+    set= appGuiGetToggleStateFromCallback( w, e );
+
+    apd->apdCenterHChosen= ( set != 0 );
+
+    appPrintDialogRefreshPlacementDrawing( apd );
 
     return;
     }
@@ -502,79 +705,230 @@ static void appPrintDialogPaperSizeChanged(	APP_WIDGET	w,
 /*									*/
 /************************************************************************/
 
-static APP_EVENT_HANDLER( appPrintDialogDrawPage, w, voidapd, exposeEvent )
+static int appPrintDialogPrintGeometry(
+				    PrintGeometry *		pg,
+				    AppPrintDialog *		apd )
     {
-    AppPrintDialog *		apd= (AppPrintDialog *)voidapd;
-    AppDrawingData *		add= &(apd->apdDrawingData);
-    PlacementSpecification *	ps;
+    const PlacementSpecification *	ps;
 
-    if  ( apd->apdPlacementChosen < 0			||
-	  apd->apdPlacementChosen >= PRINTpsCOUNT	)
-	{ LLDEB(apd->apdPlacementChosen,PRINTpsCOUNT); return;	}
+    PropertyMask			updMask;
+
+    if  ( appPaperChooserGetSize( &updMask,
+				    &(apd->apdPaperChooser),
+				    &(apd->apdPrinterGeometry) ) )
+	{ return -1;	}
 
     ps= apd->apdPlacements+ apd->apdPlacementChosen;
 
+    pg->pgSheetGeometry= apd->apdPrinterGeometry;
+
+    pg->pgRotatePage90= apd->apdRotate90Chosen;
+    pg->pgCenterPageHorizontally= apd->apdCenterHChosen;
+
+    if  ( ps->psGridRows * ps->psGridCols != 1 )
+	{
+	pg->pgRotatePage90= 0;
+	pg->pgCenterPageHorizontally= 0;
+	}
+
+    pg->pgGridRows= ps->psGridRows;
+    pg->pgGridCols= ps->psGridCols;
+    pg->pgGridHorizontal= ps->psGridHorizontal;
+    pg->pgScalePagesToFit= ps->psScalePagesToFit;
+
+    return 0;
+    }
+
+static APP_EVENT_HANDLER_H( appPrintDialogDrawPage, w, voidapd, exposeEvent )
+    {
+    AppPrintDialog *		apd= (AppPrintDialog *)voidapd;
+    AppDrawingData *		add= &(apd->apdDrawingData);
+
+    PrintGeometry		pg;
+
+    if  ( ! apd->apdPageDrawing )
+	{ XDEB(apd->apdPageDrawing); return;	}
+
+    if  ( apd->apdPlacementChosen < 0			||
+	  apd->apdPlacementChosen >= PSnum__COUNT	)
+	{ LLDEB(apd->apdPlacementChosen,PSnum__COUNT); return;	}
+
+    if  ( appPrintDialogPrintGeometry( &pg, apd ) )
+	{ LDEB(1);	}
+
     appDrawNupDiagram( apd->apdPageDrawing, add, apd->apdFontStruct, DRH_MM,
-					    &(apd->apdPrinterGeometry),
-					    &(apd->apdDocumentGeometry),
-					    ps->psNup, ps->psHorizontal );
+					    &(apd->apdDocumentGeometry), &pg );
     }
 
 /************************************************************************/
 /*									*/
 /*  Fill the list of printers.						*/
 /*									*/
+/*  1)  Empty option menu. It should already be empty.			*/
+/*  2)  Insert print to file option.					*/
+/*  3)  Insert fax option.						*/
+/*  4)  Insert printers up option.					*/
+/*  5)  Insert options for the actual printers.				*/
+/*  6)  Insert printers down option.					*/
+/*									*/
 /************************************************************************/
 
-static void appPrintDialogFillPrinterMenu( AppPrintDialog *	apd,
+static void appPrintDialogFillPrinterMenu(
+					AppPrintDialog *	apd,
 					int			printerCount,
 					int			defaultPrinter,
 					PrintDestination *	pd )
     {
-    int			i;
-    int			destinationChosen= -1;
+    int			opt;
+    int			prt;
 
+    int			optionChosen= -1;
+    int			firstPrinter= -1;
+
+    /*  1  */
     appEmptyOptionmenu( &(apd->apdPrinterOptionmenu) );
 
-    for ( i= 0; i < printerCount; pd++, i++ )
+    opt= 0;
+
+    /*  2  */
+    {
+    apd->apdPrinterOptions[opt]= appAddItemToOptionmenu(
+					&(apd->apdPrinterOptionmenu),
+					apd->apdPrintToFileText,
+					appDestinationChosen, (void *)apd );
+
+    if  ( optionChosen < 0 )
+	{ optionChosen= opt;	}
+
+    apd->apdToFileOptionNr= opt++;
+    }
+
+    /*  3  */
+    {
+    apd->apdPrinterOptions[opt]= appAddItemToOptionmenu(
+					&(apd->apdPrinterOptionmenu),
+					apd->apdFaxText,
+					appDestinationChosen, (void *)apd );
+
+    if  ( ! apd->apdActivateFaxOptions )
+	{ appGuiEnableWidget( apd->apdPrinterOptions[opt], 0 ); }
+
+    apd->apdFaxOptionNr= opt++;
+    }
+
+    /*  4  */
+    {
+    int		ltxt= strlen( apd->apdPrintersUpText );
+    int		lval= strlen( apd->apdVisiblePrinterCountText );
+    char *	scratch;
+
+    scratch= malloc( ltxt+ lval+ 1 );
+    if  ( ! scratch )
 	{
-	apd->apdPrinterOptions[i]=
+	LLXDEB(ltxt,lval,scratch);
+	apd->apdPrinterOptions[opt]= appAddItemToOptionmenu(
+					&(apd->apdPrinterOptionmenu),
+					apd->apdPrintersUpText,
+					appPrintersUpChosen, (void *)apd );
+	}
+    else{
+	int	pos;
+
+	for ( pos= 0; pos < ltxt; pos++ )
+	    {
+	    if  ( apd->apdPrintersUpText[pos+ 0] == '%'	&&
+		  apd->apdPrintersUpText[pos+ 1] == 'd'	)
+		{ break;	}
+	    }
+
+	strncpy( scratch, apd->apdPrintersUpText, pos );
+	strncpy( scratch+ pos, apd->apdVisiblePrinterCountText, lval );
+
+	if  ( pos < ltxt )
+	    { strcpy( scratch+ pos+ lval ,apd->apdPrintersUpText+ pos+ 2 ); }
+	else{ strcpy( scratch+ pos+ lval ,apd->apdPrintersUpText+ pos );    }
+	apd->apdPrinterOptions[opt]= appAddItemToOptionmenu(
+					&(apd->apdPrinterOptionmenu),
+					scratch,
+					appPrintersUpChosen, (void *)apd );
+	free( scratch );
+	}
+
+    apd->apdUpOptionNr= opt++;
+    }
+
+    /*  5  */
+    apd->apdFirstPrinterOptionNr= opt;
+
+    for ( prt= 0; prt < printerCount; pd++, opt++, prt++ )
+	{
+	apd->apdPrinterOptions[opt]=
 			appAddItemToOptionmenu( &(apd->apdPrinterOptionmenu),
 			pd->pdPrinterName, appDestinationChosen,
 			(void *)apd );
 
-	if  ( i == defaultPrinter )
-	    { destinationChosen= i;	}
-	if  ( i == 0 )
-	    { destinationChosen= i;	}
+	if  ( prt == defaultPrinter )
+	    {
+	    optionChosen= opt;
+	    firstPrinter= prt;
+	    }
+	if  ( prt == 0 )
+	    {
+	    optionChosen= opt;
+	    firstPrinter= prt;
+	    }
 	}
 
-    apd->apdPrinterOptions[i]= appAddItemToOptionmenu(
-		    &(apd->apdPrinterOptionmenu),
-		    apd->apdFaxText, appDestinationChosen, (void *)apd );
+    /*  6  */
+    {
+    int		ltxt= strlen( apd->apdPrintersDownText );
+    int		lval= strlen( apd->apdVisiblePrinterCountText );
+    char *	scratch;
 
-    if  ( ! apd->apdActivateFaxOptions )
-	{ appGuiEnableWidget( apd->apdPrinterOptions[i], 0 ); }
+    scratch= malloc( ltxt+ lval+ 1 );
+    if  ( ! scratch )
+	{
+	LLXDEB(ltxt,lval,scratch);
+	apd->apdPrinterOptions[opt]= appAddItemToOptionmenu(
+					&(apd->apdPrinterOptionmenu),
+					apd->apdPrintersDownText,
+					appPrintersDownChosen, (void *)apd );
+	}
+    else{
+	int	pos;
 
-    apd->apdFaxDestination= i++;
+	for ( pos= 0; pos < ltxt; pos++ )
+	    {
+	    if  ( apd->apdPrintersDownText[pos+ 0] == '%'	&&
+		  apd->apdPrintersDownText[pos+ 1] == 'd'	)
+		{ break;	}
+	    }
+
+	strncpy( scratch, apd->apdPrintersDownText, pos );
+	strncpy( scratch+ pos, apd->apdVisiblePrinterCountText, lval );
+
+	if  ( pos < ltxt )
+	    { strcpy( scratch+ pos+ lval ,apd->apdPrintersDownText+ pos+ 2 ); }
+	else{ strcpy( scratch+ pos+ lval ,apd->apdPrintersDownText+ pos );    }
+
+	apd->apdPrinterOptions[opt]= appAddItemToOptionmenu(
+					&(apd->apdPrinterOptionmenu),
+					scratch,
+					appPrintersDownChosen, (void *)apd );
+	free( scratch );
+	}
+
+    apd->apdDownOptionNr= opt++;
+    }
+
+    apd->apdPrintOptionCount= opt;
     
-    apd->apdPrinterOptions[i]= apd->apdPrintToFileOption=
-	appAddItemToOptionmenu( &(apd->apdPrinterOptionmenu),
-	    apd->apdPrintToFileText, appDestinationChosen, (void *)apd );
-
-    if  ( destinationChosen < 0 )
-	{ destinationChosen= printerCount+ 1;	}
-
-    apd->apdFileDestination= i++;
-    apd->apdPrintOptionCount= i;
-    
-    appSetOptionmenu( &(apd->apdPrinterOptionmenu), destinationChosen );
-
     apd->apdPrinterCount= printerCount;
+    apd->apdOptionNrChosen= optionChosen;
+    apd->apdFirstVisiblePrinter= firstPrinter;
 
-    if  ( defaultPrinter >= 0 )
-	{ apd->apdDestinationChosen= defaultPrinter;	}
-    else{ apd->apdDestinationChosen= destinationChosen; }
+    appSetOptionmenu( &(apd->apdPrinterOptionmenu), apd->apdOptionNrChosen );
+    appPrintDialogMenuRange( apd );
 
     appOptionmenuRefreshWidth( &(apd->apdPrinterOptionmenu) );
     }
@@ -594,15 +948,18 @@ static void appPrintDialogFillPlacementMenu( AppPrintDialog *	apd )
     appEmptyOptionmenu( &(apd->apdPlacementOptionmenu) );
 
     ps= apd->apdPlacements;
-    for ( i= 0; i < PRINTpsCOUNT; ps++, i++ )
+    for ( i= 0; i < PSnum__COUNT; ps++, i++ )
 	{
 	ps->psOptionWidget= appAddItemToOptionmenu(
 				&(apd->apdPlacementOptionmenu),
 				ps->psOptionText,
-				appPlacementChosen, (void *)apd );
+				appPrintDialogPlacementChosen, (void *)apd );
 	}
 
     apd->apdPlacementChosen= 0;
+    apd->apdRotate90Chosen= 0;
+    apd->apdCenterHChosen= 0;
+
     appSetOptionmenu( &(apd->apdPlacementOptionmenu), 0 );
 
     appOptionmenuRefreshWidth( &(apd->apdPlacementOptionmenu) );
@@ -650,7 +1007,7 @@ static void appPrintDialogFillSelectionMenu( AppPrintDialog *	apd )
 /*									*/
 /************************************************************************/
 
-static APP_CLOSE_CALLBACK( appClosePrintDialog, w, voidapd )
+static APP_CLOSE_CALLBACK_H( appClosePrintDialog, w, voidapd )
     {
     AppPrintDialog *	apd= (AppPrintDialog *)voidapd;
 
@@ -659,7 +1016,7 @@ static APP_CLOSE_CALLBACK( appClosePrintDialog, w, voidapd )
     return;
     }
 
-static APP_DESTROY_CALLBACK( appDestroyPrintDialog, w, voidapd )
+static APP_DESTROY_CALLBACK_H( appDestroyPrintDialog, w, voidapd )
     {
     AppPrintDialog *	apd= (AppPrintDialog *)voidapd;
 
@@ -716,13 +1073,28 @@ static APP_WIDGET appPrintDialogMakePlacementFrame( APP_WIDGET	parent,
     APP_WIDGET		placementFrame;
     APP_WIDGET		frameColumn;
 
+    const int		heightResizable= 0;
+
     appMakeColumnFrameInColumn( &placementFrame, &frameColumn, parent,
 						    apd->apdPlacementText );
 
     appMakeOptionmenuInColumn( &(apd->apdPlacementOptionmenu), frameColumn );
 
-    apd->apdPageDrawing= appMakePageDrawing( parent, ea, DRH_MM,
+    apd->apdCustomTransformRow=
+			appMakeRowInColumn( frameColumn, 2, heightResizable );
+    apd->apdRotate90Toggle= appMakeToggleInRow( apd->apdCustomTransformRow,
+				apd->apdRotate90Text,
+				appPrintDialogRotate90Toggled, (void *)apd, 0 );
+    apd->apdCenterHToggle= appMakeToggleInRow( apd->apdCustomTransformRow,
+				apd->apdCenterHText,
+				appPrintDialogCenterHToggled, (void *)apd, 1 );
+
+    apd->apdPageDrawing= (APP_WIDGET)0;
+    if  ( apd->apdShowPlacementDrawingInt >= 0 )
+	{
+	apd->apdPageDrawing= appMakePageDrawing( parent, ea, DRH_MM,
 					appPrintDialogDrawPage, (void *)apd );
+	}
 
     return placementFrame;
     }
@@ -748,6 +1120,7 @@ static APP_WIDGET appPrintDialogMakeSelectionFrame( APP_WIDGET	parent,
 
     const int		heightResizable= 0;
     const int		textEnabled= 0;
+    const int		colspan= 1;
 
     appMakeColumnFrameInColumn( &selectionFrame, &frameColumn, parent,
 						    apd->apdSelectionText );
@@ -756,8 +1129,8 @@ static APP_WIDGET appPrintDialogMakeSelectionFrame( APP_WIDGET	parent,
 
     pageSelectionRow= appMakeRowInColumn( frameColumn, 2, heightResizable );
 
-    appMakeColumnInRow( &leftColumn, pageSelectionRow, 0 );
-    appMakeColumnInRow( &rightColumn, pageSelectionRow, 1 );
+    appMakeColumnInRow( &leftColumn, pageSelectionRow, 0, colspan );
+    appMakeColumnInRow( &rightColumn, pageSelectionRow, 1, colspan );
 
     appMakeLabelAndTextRow( &row, &fromLabel, &(apd->apdPageFromTextWidget),
 			    leftColumn, apd->apdFromPageText, 8, textEnabled );
@@ -805,7 +1178,6 @@ static AppPrintDialog * appMakePrintDialog( EditApplication *	ea,
 					const char *		pixmapName )
     {
     AppPrintDialog *	apd;
-    AppDrawingData *	add;
     
     APP_WIDGET		printerFrame;
     APP_WIDGET		selectionFrame;
@@ -816,6 +1188,7 @@ static AppPrintDialog * appMakePrintDialog( EditApplication *	ea,
 
     int			high;
     int			wide;
+    int			d;
 
     APP_BITMAP_IMAGE	iconPixmap= (APP_BITMAP_IMAGE)0;
     APP_BITMAP_MASK	iconMask= (APP_BITMAP_MASK)0;
@@ -837,17 +1210,20 @@ static AppPrintDialog * appMakePrintDialog( EditApplication *	ea,
 
     apd= (AppPrintDialog *)malloc(
 		    sizeof(AppPrintDialog)+
-		    ( ea->eaPrintDestinationCount+ 2 )* sizeof(APP_WIDGET) );
+		    ( ea->eaPrintDestinationCount+ 4 )* sizeof(APP_WIDGET) );
     if  ( ! apd )
 	{
 	LXDEB(ea->eaPrintDestinationCount,apd);
 	return (AppPrintDialog *)0;
 	}
-    add= &(apd->apdDrawingData);
 
-    appInitDrawingData( add );
+    apd->apdCustomTransformRow= (APP_WIDGET)0;
+    apd->apdRotate90Toggle= (APP_WIDGET)0;
+    apd->apdCenterHToggle= (APP_WIDGET)0;
 
-    for ( i= 0; i < PRINTpsCOUNT; i++ )
+    appInitDrawingData( &(apd->apdDrawingData) );
+
+    for ( i= 0; i < PSnum__COUNT; i++ )
 	{ apd->apdPlacements[i]= APP_Placements[i];	}
 
     appGuiGetResourceValues( ea, apd,
@@ -855,7 +1231,38 @@ static AppPrintDialog * appMakePrintDialog( EditApplication *	ea,
 				    sizeof(APP_PrintDialogresourceTable)/
 				    sizeof(AppConfigurableResource) );
 
-    if  ( ea->eaFaxCommand )
+    apd->apdShowPlacementDrawingInt= 0;
+    if  ( apd->apdShowPlacementDrawingString )
+	{
+	if  ( ! strcmp( apd->apdShowPlacementDrawingString, "1" ) )
+	    { apd->apdShowPlacementDrawingInt=  1;	}
+	if  ( ! strcmp( apd->apdShowPlacementDrawingString, "0" ) )
+	    { apd->apdShowPlacementDrawingInt= -1;	}
+
+	if  ( apd->apdShowPlacementDrawingInt == 0 )
+	    { SDEB(apd->apdShowPlacementDrawingString);	}
+	}
+
+    apd->apdVisiblePrinterCount= 0;
+    if  ( apd->apdVisiblePrinterCountText )
+	{
+	char *	past= apd->apdVisiblePrinterCountText;
+
+	apd->apdVisiblePrinterCount=
+		strtol( apd->apdVisiblePrinterCountText, &past, 0 );
+
+	while( *past == ' ' )
+	    { past++;	}
+	if  ( *past )
+	    {
+	    SDEB(apd->apdVisiblePrinterCountText);
+	    apd->apdVisiblePrinterCount= 0;
+	    }
+	}
+    if  ( apd->apdVisiblePrinterCount < 1 )
+	{ apd->apdVisiblePrinterCount= 15;	}
+
+    if  ( ea->eaFaxCommand && ea->eaFaxCommand[0] )
 	{ apd->apdActivateFaxOptions= 1;	}
     else{ apd->apdActivateFaxOptions= 0;	}
 
@@ -873,8 +1280,8 @@ static AppPrintDialog * appMakePrintDialog( EditApplication *	ea,
     apd->apdTarget= (void *)ea;
     apd->apdPrinterCount= 0;
     apd->apdPrintOptionCount= 0;
-    apd->apdFileDestination= -1;
-    apd->apdFaxDestination= -1;
+    apd->apdToFileOptionNr= -1;
+    apd->apdFaxOptionNr= -1;
     apd->apdUnitType= ea->eaUnitInt;
 
     appInitPaperChooser( &(apd->apdPaperChooser) );
@@ -891,8 +1298,9 @@ static AppPrintDialog * appMakePrintDialog( EditApplication *	ea,
     printerFrame= appPrintDialogMakePrinterFrame( paned, apd );
 
     appMakePaperChooserWidgets( paned, apd->apdPaperSizeText,
+					    apd->apdUnitType,
 					    &(apd->apdPaperChooser),
-					    appPrintDialogPaperSizeChanged,
+					    appPrintDialogPaperRectChanged,
 					    (void *)apd );
 
     appPrintDialogMakePlacementFrame( paned, ea, apd );
@@ -908,8 +1316,7 @@ static AppPrintDialog * appMakePrintDialog( EditApplication *	ea,
     appPrintDialogFillSelectionMenu( apd );
 
     appPaperChooserFillMenu( &(apd->apdPaperChooser),
-				apd->apdCustomPaperSizeText,
-				appPrintDialogSizeChosen, (void *)apd );
+					    apd->apdCustomPaperSizeText );
 
     appGuiShowDialog( &(apd->apdDialog), ed->edToplevel.atTopWidget );
 
@@ -919,19 +1326,37 @@ static AppPrintDialog * appMakePrintDialog( EditApplication *	ea,
     appOptionmenuRefreshWidth( &(apd->apdPlacementOptionmenu) );
     appOptionmenuRefreshWidth( &(apd->apdSelectionOptionmenu) );
 
-    if  ( appSetDrawingDataForWidget( apd->apdPageDrawing,
-						    magnification, add ) )
-	{ LDEB(1);	}
+    if  ( apd->apdPageDrawing )
+	{
+	AppDrawingData *	add= &(apd->apdDrawingData);
 
-    appDrawGetSizeOfWidget( &wide, &high, apd->apdPageDrawing );
+	if  ( appSetDrawingDataForWidget( apd->apdPageDrawing,
+							magnification, add ) )
+	    { LDEB(1);	}
 
-    sprintf( name, ea->eaPrintDialogFont, high/ 12 );
-    apd->apdFontStruct= appDrawOpenFont( add, name );
+	appDrawGetSizeOfWidget( &wide, &high, apd->apdPageDrawing );
 
-    if  ( ! apd->apdFontStruct )
-	{ SXDEB(ea->eaPrintDialogFont,apd->apdFontStruct);	}
-    else{
-	appDrawSetFont( add, apd->apdFontStruct );
+	d= high/ 24;
+
+	for ( i= 0; i <= d; i++ )
+	    {
+	    /**/
+	    sprintf( name, ea->eaPrintDialogFont, high/ 12- i );
+	    apd->apdFontStruct= appDrawOpenFont( add, name );
+
+	    if  ( apd->apdFontStruct )
+		{ appDrawSetFont( add, apd->apdFontStruct ); break; }
+
+	    /**/
+	    sprintf( name, ea->eaPrintDialogFont, high/ 12+ i );
+	    apd->apdFontStruct= appDrawOpenFont( add, name );
+
+	    if  ( apd->apdFontStruct )
+		{ appDrawSetFont( add, apd->apdFontStruct ); break; }
+	    }
+
+	if  ( ! apd->apdFontStruct )
+	    { SLXDEB(ea->eaPrintDialogFont,high/12,apd->apdFontStruct);	}
 	}
 
     return apd;
@@ -942,7 +1367,7 @@ static AppPrintDialog * appMakePrintDialog( EditApplication *	ea,
 /*  Show the 'Print...' dialog.						*/
 /*									*/
 /*  1)	Make or just show it.						*/
-/*  2)	Set the default destination.					*/
+/*  2)	Default destination is set by appMakePrintDialog().		*/
 /*  3)	Make Fax View by Martin Vermeer happy.				*/
 /*									*/
 /************************************************************************/
@@ -956,29 +1381,26 @@ void appRunPrintDialog(			EditDocument *		ed,
 					const char *		pixmapName )
     {
     EditApplication *		ea= ed->edApplication;
-    AppPrintDialog *		apd= APP_PrintDialog;
-    PlacementSpecification *	ps;
+    AppPrintDialog *		apd= (AppPrintDialog *)ea->eaPrintDialog;
+
+    PrintJob			pj;
+    PrintGeometry		pg;
+
+    appPrintJobForEditDocument( &pj, ed );
 
     /*  1  */
     if  ( ! apd )
 	{
-	apd= APP_PrintDialog=
-			appMakePrintDialog( ea, ed, printOption, pixmapName );
+	apd= appMakePrintDialog( ea, ed, printOption, pixmapName );
 
 	if  ( ! apd )
 	    { XDEB(apd); return;	}
 
-	/*  2  */
-	if  ( ea->eaDefaultPrintDestination >= 0			  &&
-	      ea->eaDefaultPrintDestination < ea->eaPrintDestinationCount )
-	    { apd->apdDestinationChosen= ea->eaDefaultPrintDestination; }
-	else{
-	    if  ( apd->apdPrinterCount > 0 )
-		{ apd->apdDestinationChosen= 0;	}
-	    else{ apd->apdDestinationChosen= 1; }
-	    }
+	ea->eaPrintDialog= (void *)apd;
 
-	/*  1  */
+	/*  2  */
+
+	/*  3  */
 	if  ( apd->apdActivateFaxOptions )
 	    {
 	    const char *	faxNumber= getenv( "APP_FAX_TO" );
@@ -986,7 +1408,7 @@ void appRunPrintDialog(			EditDocument *		ed,
 	    if  ( faxNumber )
 		{
 		appStringToTextWidget( apd->apdFaxNumberText, faxNumber );
-		apd->apdDestinationChosen= apd->apdFaxDestination;
+		apd->apdOptionNrChosen= apd->apdFaxOptionNr;
 		}
 	    }
 	}
@@ -1016,37 +1438,37 @@ void appRunPrintDialog(			EditDocument *		ed,
 			      apd->apdLastSelectedPage >= 0	)	);
 
     appSetOptionmenu( &(apd->apdPrinterOptionmenu),
-						apd->apdDestinationChosen );
+						apd->apdOptionNrChosen );
     appPrinterAdaptToDestination( apd );
 
     appPaperChooserAdaptToGeometry( &(apd->apdPaperChooser),
-			apd->apdUnitType, &(apd->apdPrinterGeometry), 1 );
+						&(apd->apdPrinterGeometry) );
 
     appGuiRunDialog( &(apd->apdDialog), AQDrespNONE, ea );
 
     if  ( apd->apdPlacementChosen < 0			||
-	  apd->apdPlacementChosen >= PRINTpsCOUNT	)
+	  apd->apdPlacementChosen >= PSnum__COUNT	)
 	{
-	LLDEB(apd->apdPlacementChosen,PRINTpsCOUNT);
+	LLDEB(apd->apdPlacementChosen,PSnum__COUNT);
 	appGuiHideDialog( &(apd->apdDialog) );
 	return;
 	}
 
-    ps= apd->apdPlacements+ apd->apdPlacementChosen;
+    if  ( appPrintDialogPrintGeometry( &pg, apd ) )
+	{ appGuiHideDialog( &(apd->apdDialog) ); return; }
 
     switch( apd->apdDialog.adResponse )
 	{
 	case AQDrespOK:
-	    if  ( apd->apdDestinationChosen == apd->apdFileDestination )
+	    if  ( apd->apdOptionNrChosen == apd->apdToFileOptionNr )
 		{
-		appDocPrintToFile( printOption, apd->apdDialog.adTopWidget, ed,
-						&(apd->apdPrinterGeometry),
-						ps->psNup, ps->psHorizontal,
+		appDocPrintToFile( printOption, apd->apdDialog.adTopWidget,
+						ed, &pg,
 						apd->apdFirstPageChosen,
 						apd->apdLastPageChosen );
 		}
 	    else{
-		if  ( apd->apdDestinationChosen == apd->apdFaxDestination )
+		if  ( apd->apdOptionNrChosen == apd->apdFaxOptionNr )
 		    {
 		    char *	faxNumber;
 		    char *	s;
@@ -1060,9 +1482,7 @@ void appRunPrintDialog(			EditDocument *		ed,
 		    if  ( ! *s )
 			{ SDEB(faxNumber);	}
 		    else{
-			if  ( appFaxDocument( ed, s,
-						&(apd->apdPrinterGeometry),
-						ps->psNup, ps->psHorizontal,
+			if  ( appFaxDocument( ed, s, &pg,
 						apd->apdFirstPageChosen,
 						apd->apdLastPageChosen ) )
 			    { SDEB(faxNumber);	}
@@ -1071,11 +1491,15 @@ void appRunPrintDialog(			EditDocument *		ed,
 		    appFreeStringFromTextWidget( faxNumber );
 		    }
 		else{
-		    if  ( appPrintDocument( ed, apd->apdDestinationChosen,
-			    &(apd->apdPrinterGeometry),
-			    ps->psNup, ps->psHorizontal,
-			    apd->apdFirstPageChosen, apd->apdLastPageChosen ) )
-			{ LDEB(apd->apdDestinationChosen); }
+		    int		printer;
+
+		    printer=
+			apd->apdOptionNrChosen- apd->apdFirstPrinterOptionNr;
+
+		    if  ( appPrintDocument( printer, &pj, &pg,
+						    apd->apdFirstPageChosen,
+						    apd->apdLastPageChosen ) )
+			{ LDEB(apd->apdOptionNrChosen); }
 		    }
 		}
 
@@ -1084,8 +1508,10 @@ void appRunPrintDialog(			EditDocument *		ed,
 
 	default:
 	    LDEB(apd->apdDialog.adResponse);
-	    /*FALLTHROUGH*/
+	    goto error;
+
 	case AQDrespCANCEL:
+	error:
 	    appGuiHideDialog( &(apd->apdDialog) );
 	    return;
 	}
