@@ -4,7 +4,7 @@
 /*									*/
 /************************************************************************/
 
-#   include	"config.h"
+#   include	"appFrameConfig.h"
 
 #   include	<stdlib.h>
 #   include	<stdio.h>
@@ -25,6 +25,12 @@
 #	include	<X11/keysym.h>
 #   endif
 
+#   ifdef __VMS
+    static const char * const		APPSPELL_IndexExtension= "IND";
+#   else
+    static const char * const		APPSPELL_IndexExtension= "ind";
+#   endif
+
 /************************************************************************/
 /*									*/
 /*  Resources for the spell tool.					*/
@@ -86,9 +92,15 @@ static AppConfigurableResource APP_SpellToolResourceTable[]=
 		offsetof(AppSpellToolResources,astrNoDicts),
 		"None" ),
 
+#   ifdef __VMS
+    APP_RESOURCE( "spellToolPrivateDicts",
+		offsetof(AppSpellToolResources,astrPrivateDictionaries),
+		"_Dictionaries" ),
+#   else
     APP_RESOURCE( "spellToolPrivateDicts",
 		offsetof(AppSpellToolResources,astrPrivateDictionaries),
 		".Dictionaries" ),
+#   endif
     APP_RESOURCE( "spellToolSystemDicts",
 		offsetof(AppSpellToolResources,astrSystemDictionaries),
 		INDDIR ),
@@ -164,10 +176,12 @@ typedef struct AppSpellTool
 
 static FILE *	appSpellOpenDictionary(	const char *	prefix,
 					AppSpellTool *	ast,
-					int		readonly );
+					int		read_only );
 
 /************************************************************************/
+/*									*/
 /*  Open index files for a certain dictionary.				*/
+/*									*/
 /************************************************************************/
 
 static int appSpellToolOpenIndices(	SpellCheckContext *	scc,
@@ -176,11 +190,21 @@ static int appSpellToolOpenIndices(	SpellCheckContext *	scc,
     char *		systemdicts= ast->astSystemDictionaries;
 
     char		scratch[FILEL];
-    char *		format= "%s/%s.ind";
-
+#   ifdef __VMS
+    char *		format= "%s.%s";
+#   else
+    char *		format= "%s/%s.%s";
+#   endif
+       
     FILE *		privateDict;
 
-    sprintf( scratch, format, systemdicts, scc->sccDictionaryPrefix );
+#   ifdef __VMS
+    sprintf( scratch, format,
+			scc->sccDictionaryPrefix, APPSPELL_IndexExtension );
+#   else
+    sprintf( scratch, format, systemdicts,
+			scc->sccDictionaryPrefix, APPSPELL_IndexExtension );
+#   endif
 
     scc->sccStaticInd= indRead( scratch, 1 );
 
@@ -221,9 +245,7 @@ static int appSpellToolOpenIndices(	SpellCheckContext *	scc,
 /*									*/
 /************************************************************************/
 
-static void appSpellToolLearnPushed(	APP_WIDGET	w,
-					void *		voidast,
-					void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( appSpellToolLearnPushed, w, voidast )
     {
     AppSpellTool *		ast= (AppSpellTool *)voidast;
     unsigned char *		word;
@@ -269,11 +291,12 @@ static void appSpellToolLearnPushed(	APP_WIDGET	w,
     }
 
 /************************************************************************/
+/*									*/
 /*  'Forget' button has been pushed.					*/
+/*									*/
 /************************************************************************/
-static void appSpellToolForgetPushed(	APP_WIDGET	w,
-					void *		voidast,
-					void *		voidpbcs	 )
+
+static APP_BUTTON_CALLBACK_H( appSpellToolForgetPushed, w, voidast )
     {
     AppSpellTool *		ast= (AppSpellTool *)voidast;
     unsigned char *		word;
@@ -342,14 +365,13 @@ static void appSpellToolGotAlternative(	AppSpellTool *	ast,
     }
 
 /************************************************************************/
+/*									*/
 /*  'Find Next' button has been pushed.					*/
+/*									*/
 /************************************************************************/
 
-static void appSpellToolFindNext(	APP_WIDGET	w,
-					void *		voidast,
-					void *		voidpbcs )
+static void appSpellToolFindNext(	AppSpellTool *	ast )
     {
-    AppSpellTool *		ast= (AppSpellTool *)voidast;
     AppSpellCheckContext *	ascc;
     SpellCheckContext *		scc;
 
@@ -376,15 +398,22 @@ static void appSpellToolFindNext(	APP_WIDGET	w,
     appSpellToolGotAlternative( ast, 0 );
     }
 
+static APP_BUTTON_CALLBACK_H( appSpellToolFindNextPushed, w, voidast )
+    {
+    AppSpellTool *		ast= (AppSpellTool *)voidast;
+
+    appSpellToolFindNext( ast );
+
+    return;
+    }
+
 /************************************************************************/
 /*									*/
 /*  'Ignore' button has been pushed.					*/
 /*									*/
 /************************************************************************/
 
-static void editSpellToolIgnorePushed(	APP_WIDGET	w,
-					void *		voidast,
-					void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( appSpellToolIgnorePushed, w, voidast )
     {
     AppSpellTool *		ast= (AppSpellTool *)voidast;
     unsigned char *		word;
@@ -409,7 +438,7 @@ static void editSpellToolIgnorePushed(	APP_WIDGET	w,
     if  ( indMoveWord( scc->sccForgotInd, scc->sccLearntInd, word ) )
 	{ SDEB((char *)word); return;	}
 
-    appSpellToolFindNext( w, voidast, voidpbcs );
+    appSpellToolFindNext( ast );
 
     appFreeStringFromTextWidget( (char *)word );
 
@@ -417,12 +446,13 @@ static void editSpellToolIgnorePushed(	APP_WIDGET	w,
     }
 
 /************************************************************************/
+/*									*/
 /*  The user typed something in the Coreection, Turn on the the		*/
 /*  'Correct' button.							*/
+/*									*/
 /************************************************************************/
-static void editSpellCorrectionTyped(	APP_WIDGET	w,
-					void *		voidast,
-					void *		ignored	 )
+
+static APP_TXTYPING_CALLBACK_H( appSpellCorrectionTyped, w, voidast )
     {
     AppSpellTool *	ast= (AppSpellTool *)voidast;
 
@@ -432,13 +462,13 @@ static void editSpellCorrectionTyped(	APP_WIDGET	w,
     }
 
 /************************************************************************/
+/*									*/
 /*  'Correct' button has been pushed.					*/
 /*  Or a double click on the listbox with guesses.			*/
+/*									*/
 /************************************************************************/
 
-static void editSpellToolCorrect(	APP_WIDGET	w,
-					void *		voidast,
-					void *		ignored	 )
+static APP_BUTTON_CALLBACK_H( appSpellToolCorrect, w, voidast )
     {
     AppSpellTool *	ast= (AppSpellTool *)voidast;
     char *		guess;
@@ -452,7 +482,7 @@ static void editSpellToolCorrect(	APP_WIDGET	w,
 
     appFreeStringFromTextWidget( guess );
 
-    appSpellToolFindNext( w, voidast, ignored );
+    appSpellToolFindNext( ast );
 
     return;
     }
@@ -461,9 +491,7 @@ static void editSpellToolCorrect(	APP_WIDGET	w,
 /*  The 'Guess' button has been pushed.					*/
 /************************************************************************/
 
-static void editSpellGuessButtonPushed(	APP_WIDGET	w,
-					void *		voidast,
-					void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( appSpellGuessButtonPushed, w, voidast )
     {
     AppSpellTool *	ast= (AppSpellTool *)voidast;
     char *		word;
@@ -481,7 +509,7 @@ static void editSpellGuessButtonPushed(	APP_WIDGET	w,
 /*  A guess in the list has been selected.				*/
 /************************************************************************/
 
-static APP_LIST_CALLBACK( appSpellGuessChosen, w, voidast, voidlcs )
+static APP_LIST_CALLBACK_H( appSpellGuessChosen, w, voidast, voidlcs )
     {
     AppSpellTool *		ast= (AppSpellTool *)voidast;
 
@@ -510,8 +538,8 @@ static void appSpellToolMakeButtonRow(	APP_WIDGET *		pRow,
 					APP_WIDGET *		pRightButton,
 					char *			leftLabel,
 					char *			rightLabel,
-					APP_CALLBACK_FUNC	leftCallback,
-					APP_CALLBACK_FUNC	rightCallback,
+					APP_BUTTON_CALLBACK_T	leftCallback,
+					APP_BUTTON_CALLBACK_T	rightCallback,
 					AppSpellTool *		ast,
 					APP_WIDGET		parent )
 
@@ -582,13 +610,13 @@ static void appSpellMakeButtonRows(	APP_WIDGET		spellForm,
     appSpellToolMakeButtonRow( &buttonRow,
 		&(ast->astIgnoreButton), &(findNextButton),
 		astr->astrIgnore, astr->astrFindNext,
-		editSpellToolIgnorePushed, appSpellToolFindNext,
+		appSpellToolIgnorePushed, appSpellToolFindNextPushed,
 		ast, spellForm );
 
     appSpellToolMakeButtonRow( &buttonRow,
 		&(ast->astGuessButton), &(ast->astCorrectButton),
 		astr->astrGuess, astr->astrCorrect,
-		editSpellGuessButtonPushed, editSpellToolCorrect,
+		appSpellGuessButtonPushed, appSpellToolCorrect,
 		ast, spellForm );
 
     return;
@@ -623,7 +651,7 @@ static void appDestroySpellTool(	AppSpellTool *	ast )
     return;
     }
 
-static APP_CLOSE_CALLBACK( appCloseSpellTool, w, voidast )
+static APP_CLOSE_CALLBACK_H( appCloseSpellTool, w, voidast )
     {
     AppSpellTool *	ast= (AppSpellTool *)voidast;
 
@@ -670,6 +698,10 @@ static int appSpellAddDictionary(	const char *		filename,
     AppSpellCheckContext *	fresh;
     char *			name;
 
+#   ifdef __VMS
+    char *			semicolon;
+#   endif
+
     const char *	s;
     int			l;
 
@@ -678,11 +710,17 @@ static int appSpellAddDictionary(	const char *		filename,
 	{ s++;		}
     else{ s= filename;	}
 
+#   ifdef __VMS
+    semicolon= strrchr( s , ';' );
+    if  ( semicolon )
+	{ *semicolon= '\0';	}
+#   endif
+
     l= strlen( s );
     if  ( l == 0 )
 	{ LDEB(l); return 0;	}
 
-    if  ( strcmp( s+ l- 4, ".ind" ) )
+    if  ( s[l- 4] != '.' || strcmp( s+ l- 3, APPSPELL_IndexExtension ) )
 	{ SSDEB(filename,s+ l- 4); return 0;	}
 
     fresh= (AppSpellCheckContext *)realloc( ast->astDictionaryContexts,
@@ -719,6 +757,12 @@ static int appSpellAddDictionary(	const char *		filename,
 	return 0;
 	}
 
+    if  ( ! strcmp( name, "Greek" )	)
+	{
+	indSpellIso7CharacterKinds( &(fresh->asccSpellCheckContext) );
+	return 0;
+	}
+
     indSpellIso1CharacterKinds( &(fresh->asccSpellCheckContext) );
 
     return 0;
@@ -729,7 +773,7 @@ static int appSpellGetDictionaries(	AppSpellTool *		ast,
     {
     char *		systemdicts= ast->astSystemDictionaries;
 
-    if  ( appForAllFiles( systemdicts, "ind",
+    if  ( appForAllFiles( systemdicts, APPSPELL_IndexExtension,
 					(void *)ast, appSpellAddDictionary ) )
 	{ SDEB(systemdicts); return -1;	}
 
@@ -765,19 +809,12 @@ static int appSpellGetDictionaries(	AppSpellTool *		ast,
 /*									*/
 /************************************************************************/
 
-static void editSpellDictionaryChosen(	APP_WIDGET	w,
-					void *		voidast,
-					void *		voidpbcs )
+static APP_OITEM_CALLBACK_H( appSpellDictionaryChosen, w, voidast )
     {
     AppSpellTool *	ast= (AppSpellTool *)voidast;
     int			i;
 
-    for ( i= 0; i < ast->astDictionaryCount; i++ )
-	{
-	if  ( ast->astDictionaryContexts[i].asccMenuWidget == w )
-	    { break;	}
-	}
-
+    i= appGuiGetOptionmenuItemIndex( &(ast->astDictionaryOptionmenu), w );
     if  ( i < 0 || i >= ast->astDictionaryCount )
 	{ LLDEB(i,ast->astDictionaryCount); return;	}
 
@@ -806,14 +843,14 @@ static void appSpellFillDictionaryMenu(		AppSpellTool *		ast,
 	ascc->asccMenuWidget= appAddItemToOptionmenu(
 			    &(ast->astDictionaryOptionmenu),
 			    ascc->asccSpellCheckContext.sccDictionaryPrefix,
-			    editSpellDictionaryChosen, (void *)ast );
+			    appSpellDictionaryChosen, (void *)ast );
 	}
 
     if  ( ast->astDictionaryCount == 0 )
 	{
 	(void) appAddItemToOptionmenu( &(ast->astDictionaryOptionmenu),
 			    astr->astrNoDicts,
-			    editSpellDictionaryChosen, (void *)ast );
+			    appSpellDictionaryChosen, (void *)ast );
 
 	appGuiEnableWidget( ast->astTopWidget, 0 );
 	}
@@ -912,13 +949,13 @@ void * appMakeSpellTool(	APP_WIDGET		spellOption,
 
 #   ifdef USE_MOTIF
     XtAddCallback( ast->astGuessList, XmNdefaultActionCallback,
-					editSpellToolCorrect, (void *)ast );
+					appSpellToolCorrect, (void *)ast );
 #   endif
 
     appMakeTextInColumn( &(ast->astWordText), ast->astMainWidget, 0, 1 );
 
     appGuiSetTypingCallbackForText( ast->astWordText,
-				    editSpellCorrectionTyped, (void *)ast );
+				    appSpellCorrectionTyped, (void *)ast );
 
     appSpellMakeButtonRows( ast->astMainWidget, &astr, ast );
 
@@ -1043,7 +1080,7 @@ void appSpellMakeGuesses(	void *			voidast,
 
 static FILE *	appSpellOpenDictionary(	const char *	prefix,
 					AppSpellTool *	ast,
-					int		readonly )
+					int		read_only )
     {
     char *		dir= ast->astPrivateDictionaries;
     char		scratch[FILEL+ 1];
@@ -1060,11 +1097,17 @@ static FILE *	appSpellOpenDictionary(	const char *	prefix,
 
     if  ( dir[0] == '/' )
 	{ strcpy( scratch, dir );			}
-    else{ sprintf( scratch, "%s/%s", home, dir );	}
+    else{
+#	ifdef __VMS
+	sprintf( scratch, "%.*s.%s]", len- 1, home, dir );
+#	else
+	sprintf( scratch, "%s/%s", home, dir );
+#	endif
+	}
 
     if  ( appTestDirectory( scratch ) )
 	{
-	if  ( readonly )
+	if  ( read_only )
 	    { return (FILE *)0;	}
 
 	if  ( appMakeDirectory( scratch ) )
@@ -1081,14 +1124,22 @@ static FILE *	appSpellOpenDictionary(	const char *	prefix,
 
     if  ( dir[0] == '/' )
 	{ sprintf( scratch, "%s/%s.changes", dir, prefix );		}
-    else{ sprintf( scratch, "%s/%s/%s.changes", home, dir, prefix );	}
+    else{
+#	ifdef __VMS
+	char *	vms_file= strchr( prefix, ']' );
 
-    if  ( readonly )
+	sprintf( scratch, "%.*s.%s%s.changes", len- 1, home, dir, vms_file );
+#	else
+	sprintf( scratch, "%s/%s/%s.changes", home, dir, prefix );
+#	endif
+	}
+
+    if  ( read_only )
 	{ mode= "r";	}
     else{ mode= "a";	}
 
     f= fopen( scratch, mode );
-    if  ( ! f && ! readonly )
+    if  ( ! f && ! read_only )
 	{ SLDEB(scratch,errno); return (FILE *)0; }
 
     return f;
