@@ -1,18 +1,21 @@
 /************************************************************************/
+/*									*/
 /*  A Symbol picker.							*/
+/*									*/
 /************************************************************************/
 
-#   include	"config.h"
+#   include	"appFrameConfig.h"
 
 #   include	<stdlib.h>
 #   include	<stdio.h>
 #   include	<stddef.h>
 
-#   include	<appDebugon.h>
-
 #   include	<appSymbolPicker.h>
+#   include	<appEncodingMenu.h>
 #   include	<appDraw.h>
 
+#   include	<appDebugon.h>
+
 /************************************************************************/
 /*									*/
 /*  Resources for the Symbol Picker.					*/
@@ -24,13 +27,14 @@ typedef struct AppSymbolPickerResources
     char *	asprInsert;
     char *	asprFont;
     char *	asprNone;
+    char *	asprEncodings[ENCODINGps_COUNT];
     } AppSymbolPickerResources;
 
 static AppConfigurableResource APP_SymbolPickerResourceTable[]=
     {
     APP_RESOURCE( "symbolPickerFont",
 		offsetof(AppSymbolPickerResources,asprFont),
-		"Font" ),
+		"Font, Encoding" ),
     APP_RESOURCE( "symbolPickerNone",
 		offsetof(AppSymbolPickerResources,asprNone),
 		"None" ),
@@ -56,6 +60,7 @@ typedef struct AppSymbolPicker
 
     APP_WIDGET			aspFontFrame;
     AppOptionmenu		aspFontOptionmenu;
+    AppEncodingMenu		aspEncodingMenu;
 
     APP_WIDGET			aspSymbolDrawing;
     APP_WIDGET			aspInsertButton;
@@ -69,7 +74,7 @@ typedef struct AppSymbolPicker
 
     int				aspSymbolSelected;
 
-    int				aspFontFamilySet;
+    ExpandedTextAttribute	aspTextAttribute;
     int				aspFontFamilyChosen;
 
     APP_FONT *			aspFont;
@@ -78,33 +83,35 @@ typedef struct AppSymbolPicker
     int				aspCellsWide;
     int				aspCellsHigh;
 
-    APP_WIDGET			aspFontFamilyOptions[1];
+    APP_WIDGET			aspFontFamilyOptions[1]; /* LAST! */
     } AppSymbolPicker;
 
 /************************************************************************/
 /*									*/
-/*  Adapt the symbol Picker to the current font family.			*/
+/*  Adapt the symbol Picker to the current font.			*/
+/*									*/
+/*  9)  Also remember that no symbol has been selected yet, and that	*/
+/*	for that reason, no symbol can be inserted.			*/
 /*									*/
 /************************************************************************/
 
-static int appSymbolAdaptToFamily(  AppSymbolPicker *	asp,
-				    int			fontFamilyNumber )
+static int appSymbolAdaptDrawingToFont(		AppSymbolPicker *	asp )
     {
-    AppDrawingData *	add= &(asp->aspDrawingData);
+    AppDrawingData *		add= &(asp->aspDrawingData);
+    char			scratch[120];
+    APP_FONT *			xfs;
+    const AppFontFamily *	aff;
+    ExpandedTextAttribute *	eta= &(asp->aspTextAttribute);
 
-    char		scratch[120];
+    const int			twipsSize= 20* 10;
 
-    APP_FONT *		xfs;
+    if  ( asp->aspFontFamilyChosen < 0 )
+	{ LDEB(asp->aspFontFamilyChosen); return -1;	}
 
-    AppFontFamily *	aff;
-
-    const int		twipsSize= 20* 10;
-
-    aff= asp->aspFontFamilies+ fontFamilyNumber;
-
-    if  ( appFontXFont( scratch, add,
-			aff, aff->affFaces, twipsSize, DOCfontREGULAR ) )
-	{ SDEB(aff->affFontFamilyName); return- 1; }
+    aff= asp->aspFontFamilies+ asp->aspFontFamilyChosen;
+    if  ( appFontXFont( scratch, add, aff, eta->etaFontEncoding,
+				aff->affFaces, twipsSize, DOCfontREGULAR ) )
+	{ SLDEB(aff->affFontFamilyName,eta->etaFontEncoding); return- 1; }
 
     xfs= appDrawOpenFont( add, scratch );
     if  ( ! xfs )
@@ -118,32 +125,57 @@ static int appSymbolAdaptToFamily(  AppSymbolPicker *	asp,
 
     appExposeRectangle( add, 0, 0, 0, 0 );
 
+    /*  9  */
     asp->aspSymbolSelected= -1;
     appGuiEnableWidget( asp->aspInsertButton, 0 );
 
     return 0;
     }
 
+static int appSymbolAdaptToFamily(  AppSymbolPicker *	asp )
+    {
+    if  ( asp->aspFontFamilyChosen < 0 )
+	{ LDEB(asp->aspFontFamilyChosen); return -1;	}
+
+    appEncodingMenuAdaptToFamilyEncodings( &(asp->aspEncodingMenu),
+			    asp->aspFontFamilies+ asp->aspFontFamilyChosen );
+
+    if  ( appSymbolAdaptDrawingToFont( asp ) )
+	{ LDEB(asp->aspFontFamilyChosen); return -1;	}
+
+    return 0;
+    }
+
 /************************************************************************/
 /*									*/
 /*  'Insert' button has been pushed.					*/
 /*									*/
 /************************************************************************/
 
-static void appSymbolInsertPushed(	APP_WIDGET	w,
-					void *		voidasp,
-					void *		voidpbcs )
+static void appSymbolPickerInsertSymbol(	AppSymbolPicker *	asp )
     {
-    AppSymbolPicker *	asp= (AppSymbolPicker *)voidasp;
-    AppFontFamily *	aff;
+    PropertyMask	setMask;
+
+    PROPmaskCLEAR( &setMask );
 
     if  ( asp->aspSymbolSelected < 0 || ! asp->aspInsert )
 	{ LXDEB(asp->aspSymbolSelected,asp->aspInsert); return;	}
 
-    aff= asp->aspFontFamilies+ asp->aspFontFamilyChosen;
-    (*asp->aspInsert)( asp->aspTarget, asp->aspSymbolSelected,
-			    asp->aspFontFamilyChosen, aff->affFontFamilyName );
+    if  ( asp->aspFontFamilyChosen >= 0 )
+	{ PROPmaskADD( &setMask, TApropFONTFAMILY );	}
 
+    (*asp->aspInsert)( asp->aspTarget, asp->aspSymbolSelected,
+				    &(asp->aspTextAttribute), &setMask );
+
+    return;
+    }
+
+static APP_BUTTON_CALLBACK_H( appSymbolInsertPushed, w, voidasp )
+    {
+    AppSymbolPicker *	asp= (AppSymbolPicker *)voidasp;
+
+    appSymbolPickerInsertSymbol( asp );
+    
     return;
     }
 
@@ -197,7 +229,7 @@ static void appSymbolRectangle(		DocumentRectangle *	drSym,
     return;
     }
 
-static APP_EVENT_HANDLER( appSymbolRedraw, w, voidasp, exposeEvent )
+static APP_EVENT_HANDLER_H( appSymbolRedraw, w, voidasp, exposeEvent )
     {
     AppSymbolPicker *			asp= (AppSymbolPicker *)voidasp;
     AppDrawingData *			add= &(asp->aspDrawingData);
@@ -218,6 +250,9 @@ static APP_EVENT_HANDLER( appSymbolRedraw, w, voidasp, exposeEvent )
     const int				ox= 0;
     const int				oy= 0;
 
+    if  ( ! asp->aspFont )
+	{ XDEB(asp->aspFont); return;	}
+
     appCollectExposures( &drClip, add, ox, oy, exposeEvent );
 
     appDrawSetForegroundColor( add, &(add->addBackColor) );
@@ -325,7 +360,7 @@ static APP_EVENT_HANDLER( appSymbolRedraw, w, voidasp, exposeEvent )
 /*									*/
 /************************************************************************/
 
-static APP_EVENT_HANDLER( appSymbolMousePress, w, voidasp, downEvent )
+static APP_EVENT_HANDLER_H( appSymbolMousePress, w, voidasp, downEvent )
     {
     AppSymbolPicker *	asp= (AppSymbolPicker *)voidasp;
 
@@ -342,6 +377,7 @@ static APP_EVENT_HANDLER( appSymbolMousePress, w, voidasp, downEvent )
     int			button;
     int			upDown;
     int			seq;
+    unsigned int	keyState;
 
     DocumentRectangle	drExpose;
 
@@ -353,7 +389,7 @@ static APP_EVENT_HANDLER( appSymbolMousePress, w, voidasp, downEvent )
 	}
 
     if  ( appGetCoordinatesFromMouseButtonEvent(
-			    &x, &y, &button, &upDown, &seq, w, downEvent ) )
+		    &x, &y, &button, &upDown, &seq, &keyState, w, downEvent ) )
 	{ return;	}
 
     if  ( upDown < 1 || button != 1 )
@@ -391,49 +427,85 @@ static APP_EVENT_HANDLER( appSymbolMousePress, w, voidasp, downEvent )
 					    drExpose.drY1- drExpose.drY0+ 1 );
 
     if  ( sameAsPrevious && seq == 2 )
-	{
-	AppFontFamily *	aff;
-
-	if  ( ! asp->aspInsert )
-	    { XDEB(asp->aspInsert); return;	}
-
-	aff= asp->aspFontFamilies+ asp->aspFontFamilyChosen;
-	(*asp->aspInsert)( asp->aspTarget, asp->aspSymbolSelected,
-			    asp->aspFontFamilyChosen, aff->affFontFamilyName );
-
-	}
+	{ appSymbolPickerInsertSymbol( asp );	}
 
     return;
     }
 
 /************************************************************************/
 /*									*/
-/*  A font family was selected by the user.				*/
+/*  1)  A font family was selected by the user.				*/
+/*  2)  A font encoding was selected by the user.			*/
 /*									*/
 /************************************************************************/
 
-static void appSymbolFontFamilyChosen(		APP_WIDGET	w,
-						void *		voidasp,
-						void *		voidpbcs )
+/*  1  */
+static APP_OITEM_CALLBACK_H( appSymbolFontFamilyChosen, w, voidasp )
     {
-    int			i;
-    AppSymbolPicker *	asp= (AppSymbolPicker *)voidasp;
-
-    for ( i= 0; i < asp->aspFontFamilyCount; i++ )
-	{
-	if  ( w == asp->aspFontFamilyOptions[i] )
-	    { break;	}
-	}
+    int				i;
+    AppSymbolPicker *		asp= (AppSymbolPicker *)voidasp;
+    int				changed= 0;
+    const AppFontFamily *	aff;
 
+    i= appGuiGetOptionmenuItemIndex( &(asp->aspFontOptionmenu), w );
     if  ( i < 0 || i >= asp->aspFontFamilyCount )
 	{ LLDEB(i,asp->aspFontFamilyCount); return;	}
 
-    if  ( i != asp->aspFontFamilyChosen )
+    aff= asp->aspFontFamilies+ i;
+    if  ( docExpandedTextAttributeSetFamilyAndEncoding(
+				    &(asp->aspTextAttribute),
+				    &changed, aff,
+				    asp->aspTextAttribute.etaFontEncoding ) )
+	{ LDEB(i); return;	}
+
+    if  ( changed )
 	{
-	if  ( appSymbolAdaptToFamily( asp, i ) )
-	    { LDEB(i);	}
+	AppEncodingMenu *		aem= &(asp->aspEncodingMenu);
 
 	asp->aspFontFamilyChosen= i;
+
+	appEncodingMenuSetEncoding( aem, aff,
+				    asp->aspTextAttribute.etaFontEncoding );
+
+	if  ( appSymbolAdaptToFamily( asp ) )
+	    { LDEB(i);	}
+	}
+
+    return;
+    }
+
+/*  2  */
+static APP_OITEM_CALLBACK_H( appSymbolFontEncodingChosen, w, voidasp )
+    {
+    int				i;
+    AppSymbolPicker *		asp= (AppSymbolPicker *)voidasp;
+    int				changed= 0;
+    const AppFontFamily *	aff;
+
+    i= appGuiGetOptionmenuItemIndex(
+			&(asp->aspEncodingMenu.aemEncodingOptionmenu), w );
+    if  ( i < 0 || i >= ENCODINGps_COUNT )
+	{ LLDEB(i,ENCODINGps_COUNT); return;	}
+
+    if  ( asp->aspFontFamilyChosen >= 0 )
+	{
+	aff= asp->aspFontFamilies+ asp->aspFontFamilyChosen;
+
+	if  ( docExpandedTextAttributeSetFamilyAndEncoding(
+						&(asp->aspTextAttribute),
+						&changed, aff, i ) )
+	    { LDEB(i); return;	}
+
+	if  ( changed )
+	    {
+	    AppEncodingMenu *		aem= &(asp->aspEncodingMenu);
+
+	    if  ( appSymbolAdaptDrawingToFont( asp ) )
+		{ LDEB(asp->aspFontFamilyChosen);	}
+
+	    appEncodingMenuSetEncoding( aem, aff,
+				    asp->aspTextAttribute.etaFontEncoding );
+	    }
 	}
 
     return;
@@ -441,7 +513,7 @@ static void appSymbolFontFamilyChosen(		APP_WIDGET	w,
 
 /************************************************************************/
 /*									*/
-/*  Make the font part of the symbol picker.				*/
+/*  Make the font/encoding part of the symbol picker.			*/
 /*									*/
 /************************************************************************/
 
@@ -449,12 +521,23 @@ static void appSymbolMakeFontPart(	APP_WIDGET			parent,
 					AppSymbolPickerResources *	aspr,
 					AppSymbolPicker *		asp )
     {
-    APP_WIDGET	paned;
+    APP_WIDGET	row;
 
-    appMakeColumnFrameInColumn( &(asp->aspFontFrame),
-					    &paned, parent, aspr->asprFont );
+    const int	fontColumn= 0;
+    const int	fontColspan= 1;
+    const int	encodingColumn= fontColumn+ fontColspan;
+    const int	encodingColspan= 1;
+    const int	columnCount= encodingColumn+ encodingColspan;
 
-    appMakeOptionmenuInColumn( &(asp->aspFontOptionmenu), paned );
+    appMakeRowFrameInColumn( &(asp->aspFontFrame),
+					    &row, parent,
+					    columnCount, aspr->asprFont );
+
+    appMakeOptionmenuInRow( &(asp->aspFontOptionmenu), row,
+					    fontColumn, fontColspan );
+
+    appMakeOptionmenuInRow( &(asp->aspEncodingMenu.aemEncodingOptionmenu),
+					row, encodingColumn, encodingColspan );
 
     return;
     }
@@ -521,6 +604,7 @@ static APP_WIDGET appSymbolMakeButtonRow( APP_WIDGET		parent,
 /*									*/
 /************************************************************************/
 
+/*  1  */
 static void appSymbolFillFontMenu(	AppSymbolPickerResources *	aspr,
 					AppSymbolPicker *		asp )
     {
@@ -556,15 +640,18 @@ static void appSymbolFillFontMenu(	AppSymbolPickerResources *	aspr,
     asp->aspFontFamilyChosen= 0;
 
     appOptionmenuRefreshWidth( &(asp->aspFontOptionmenu) );
+
+    return;
     }
 
+
 /************************************************************************/
 /*									*/
 /*  A symbol picker must be destroyed.					*/
 /*									*/
 /************************************************************************/
 
-static APP_CLOSE_CALLBACK( appCloseSymbolPicker, w, voidasp )
+static APP_CLOSE_CALLBACK_H( appCloseSymbolPicker, w, voidasp )
     {
     AppSymbolPicker *	asp= (AppSymbolPicker *)voidasp;
 
@@ -574,6 +661,8 @@ static APP_CLOSE_CALLBACK( appCloseSymbolPicker, w, voidasp )
     if  ( asp->aspFont )
 	{ appDrawFreeFont( &(asp->aspDrawingData), asp->aspFont );	}
 
+    docCleanExpandedTextAttribute( &(asp->aspTextAttribute) );
+
     appCleanDrawingData( &(asp->aspDrawingData) );
 
     free( voidasp );
@@ -598,25 +687,27 @@ void * appMakeSymbolPicker(	APP_WIDGET		symbolOption,
 				AppToolDestroy		destroy,
 				void *			target )
     {
-    AppSymbolPicker *		asp;
+    AppSymbolPicker *			asp;
     
-    APP_WIDGET			buttonForm;
+    APP_WIDGET				buttonForm;
 
-    const int			userResizable= 0;
+    const int				userResizable= 0;
 
-    double			horPixPerMM;
-    double			verPixPerMM;
-    double			xfac;
-    double			yfac;
+    double				horPixPerMM;
+    double				verPixPerMM;
+    double				xfac;
+    double				yfac;
 
-    AppFontFamily *		fontFamilies;
-    int				fontFamilyCount;
+    AppFontFamily *			fontFamilies;
+    int					fontFamilyCount;
 
     static AppSymbolPickerResources	aspr;
     static int				gotResources;
 
     if  ( ! gotResources )
 	{
+	appEncodingMenuGetOptionTexts( aspr.asprEncodings, ea );
+
 	appGuiGetResourceValues( ea, (void *)&aspr,
 					APP_SymbolPickerResourceTable,
 					sizeof(APP_SymbolPickerResourceTable)/
@@ -655,12 +746,13 @@ void * appMakeSymbolPicker(	APP_WIDGET		symbolOption,
     asp->aspTarget= target;
 
     asp->aspSymbolSelected= -1;
-    asp->aspFontFamilySet= -1;
     asp->aspFontFamilyChosen= -1;
 
     asp->aspFont= (APP_FONT *)0;
     appInitDrawingData( &(asp->aspDrawingData) );
 
+    docInitExpandedTextAttribute( &(asp->aspTextAttribute) );
+
     appMakeVerticalTool( &(asp->aspTopWidget), &(asp->aspMainWidget), ea,
 			    iconPixmap, iconMask, widgetName, userResizable,
 			    symbolOption, appCloseSymbolPicker, (void *)asp );
@@ -671,6 +763,10 @@ void * appMakeSymbolPicker(	APP_WIDGET		symbolOption,
     buttonForm= appSymbolMakeButtonRow( asp->aspMainWidget, &aspr, asp );
 
     appSymbolFillFontMenu( &aspr, asp );
+    appEncodingMenuFillOptionmenu( aspr.asprEncodings,
+				    appSymbolFontEncodingChosen, (void *)asp,
+				    &(asp->aspEncodingMenu) );
+
 
 #   ifdef USE_MOTIF
     XtRealizeWidget( asp->aspTopWidget );
@@ -681,6 +777,7 @@ void * appMakeSymbolPicker(	APP_WIDGET		symbolOption,
 #   endif
 
     appOptionmenuRefreshWidth( &(asp->aspFontOptionmenu) );
+    appOptionmenuRefreshWidth( &(asp->aspEncodingMenu.aemEncodingOptionmenu) );
 
     appSetDrawingDataForWidget( asp->aspSymbolDrawing,
 				ea->eaMagnification, &(asp->aspDrawingData) );
@@ -709,23 +806,59 @@ void appShowSymbolPicker(	void *	voidasp	)
 /*									*/
 /************************************************************************/
 
-int appAdaptSymbolPickerToFontFamily(	void *		voidasp,
-					int		fontFamilyNumber )
+int appAdaptSymbolPickerToFontFamily(
+				void *				voidasp,
+				const ExpandedTextAttribute *	etaNew,
+				const PropertyMask *		newMask )
     {
-    AppSymbolPicker *	asp= (AppSymbolPicker *)voidasp;
+    AppSymbolPicker *		asp= (AppSymbolPicker *)voidasp;
+    AppEncodingMenu *		aem= &(asp->aspEncodingMenu);
+    int				rval= 0;
 
-    appSetOptionmenu( &(asp->aspFontOptionmenu), fontFamilyNumber );
+    PropertyMask		doneMask;
 
-    if  ( fontFamilyNumber != asp->aspFontFamilyChosen )
+    PROPmaskCLEAR( &doneMask );
+    docUpdateExpandedTextAttribute( &doneMask, &(asp->aspTextAttribute),
+							    etaNew, newMask );
+
+    if  ( PROPmaskISSET( &doneMask, TApropFONTFAMILY )	&&
+	  etaNew->etaFontFamilyName			)
 	{
-	if  ( appSymbolAdaptToFamily( asp, fontFamilyNumber ) )
-	    { LDEB(fontFamilyNumber);	}
+	const AppFontFamily *	aff;
+	int			i;
+
+	aff= asp->aspFontFamilies;
+	for ( i= 0; i < asp->aspFontFamilyCount; aff++, i++ )
+	    {
+	    if  ( ! strcmp( etaNew->etaFontFamilyName,
+						aff->affFontFamilyName ) )
+		{ asp->aspFontFamilyChosen= i; break;	}
+	    }
 	}
 
-    asp->aspFontFamilySet= fontFamilyNumber;
-    asp->aspFontFamilyChosen= fontFamilyNumber;
+    if  ( asp->aspFontFamilyChosen < 0 )
+	{
+	int		changed= 0;
 
-    return 0;
+	asp->aspFontFamilyChosen=  0; 
+
+	docExpandedTextAttributeSetFamilyAndEncoding(
+				&(asp->aspTextAttribute), &changed,
+				asp->aspFontFamilies,
+				asp->aspFontFamilies->affDefaultEncoding );
+	}
+
+    if  ( appSymbolAdaptToFamily( asp ) )
+	{ LDEB(asp->aspFontFamilyChosen); rval= -1;	}
+
+    appSetOptionmenu( &(asp->aspFontOptionmenu),
+					    asp->aspFontFamilyChosen );
+
+    appEncodingMenuSetEncoding( aem,
+			    asp->aspFontFamilies+ asp->aspFontFamilyChosen,
+			    asp->aspTextAttribute.etaFontEncoding );
+
+    return rval;
     }
 
 void appEnableSymbolPicker(	void *	voidasp,
