@@ -1,13 +1,17 @@
 /************************************************************************/
+/*									*/
 /*  A Symbol picker.							*/
+/*									*/
 /************************************************************************/
 
 #   include	<appFrame.h>
+#   include	<docExpandedTextAttribute.h>
 
-typedef void (*SymbolPickerInsert)(	void *		target,
-					int		symbol,
-					int		fontFamilyNumber,
-					char *		fontFamilyName );
+typedef void (*SymbolPickerInsert)(
+				void *				target,
+				int				symbol,
+				const ExpandedTextAttribute *	etaSet,
+				const PropertyMask *		taSetMask );
 
 extern void * appMakeSymbolPicker(	APP_WIDGET		symbolOption,
 					EditApplication *	ea,
@@ -20,8 +24,10 @@ extern void * appMakeSymbolPicker(	APP_WIDGET		symbolOption,
 
 extern void appShowSymbolPicker(	void *	voidasp	);
 
-extern int appAdaptSymbolPickerToFontFamily(	void *	voidasp,
-						int	fontFamilyNumber );
+extern int appAdaptSymbolPickerToFontFamily(
+				void *				voidasp,
+				const ExpandedTextAttribute *	etaSet,
+				const PropertyMask *		setMask );
 
 extern void appEnableSymbolPicker(	void *	voidasp,
 					int	enabled );
