@@ -1,10 +1,11 @@
-#   include	"config.h"
+#   include	"appFrameConfig.h"
 
 #   include	<stddef.h>
 #   include	<stdlib.h>
+#   include	<string.h>
 
 #   include	<appWinMeta.h>
-#   include	<appPs.h>
+#   include	<utilPs.h>
 #   include	<sioEndian.h>
 
 #   define	y0	math_y0
@@ -25,10 +26,115 @@
 #    define	WMFDEB(x)	/*nothing*/
 # endif
 
+static void appWinMetaSetTransform(	DeviceContext *		dc,
+					int			mapMode,
+					int			xWinOrg,
+					int			yWinOrg,
+					int			xWinExt,
+					int			yWinExt,
+					int			xViewOrg,
+					int			yViewOrg,
+					int			xViewExt,
+					int			yViewExt,
+					int			twipsWide,
+					int			twipsHigh )
+    {
+    int		xwe;
+    int		ywe;
+    int		xve;
+    int		yve;
+
+    switch( mapMode )
+	{
+	case MM_TEXT:
+	case MM_LOMETRIC:
+	case MM_HIMETRIC:
+	case MM_LOENGLISH:
+	case MM_HIENGLISH:
+	    LDEB(mapMode);
+	    break;
+
+	case MM_TWIPS:
+	    dc->dcTransform.dct_xWinOrg= 0;
+	    dc->dcTransform.dct_yWinOrg= 0;
+	    dc->dcTransform.dct_xWinExt=  twipsWide;
+	    dc->dcTransform.dct_yWinExt= -twipsHigh;
+
+	    dc->dcTransform.dct_xViewOrg= 0;
+	    dc->dcTransform.dct_yViewOrg= 0;
+	    dc->dcTransform.dct_xViewExt= twipsWide;
+	    dc->dcTransform.dct_yViewExt= twipsHigh;
+
+	    break;
+
+	case MM_ISOTROPIC:
+	    xwe= xWinExt;
+	    ywe= yWinExt;
+	    xve= xViewExt;
+	    yve= yViewExt;
+
+	    if  ( xwe < 0 )	{ xwe= -xwe;	}
+	    if  ( ywe < 0 )	{ ywe= -ywe;	}
+	    if  ( xve < 0 )	{ xve= -xve;	}
+	    if  ( yve < 0 )	{ yve= -yve;	}
+
+	    if  ( xwe < ( ywe* xve )/ yve )
+		{ xwe=  ( ywe* xve )/ yve;	}
+	    else{
+		if  ( ywe < ( xwe* yve )/ xve )
+		    { ywe=  ( xwe* yve )/ xve;	}
+		}
+
+	    if  ( xWinExt < 0 )
+		{ xWinExt= -xwe;	}
+	    else{ xWinExt=  xwe;	}
+
+	    if  ( yWinExt < 0 )
+		{ yWinExt= -ywe;	}
+	    else{ yWinExt=  ywe;	}
+
+	    dc->dcTransform.dct_xWinOrg= xWinOrg;
+	    dc->dcTransform.dct_yWinOrg= yWinOrg;
+	    dc->dcTransform.dct_xWinExt= xWinExt;
+	    dc->dcTransform.dct_yWinExt= yWinExt;
+
+	    dc->dcTransform.dct_xViewOrg= xViewOrg;
+	    dc->dcTransform.dct_yViewOrg= yViewOrg;
+	    dc->dcTransform.dct_xViewExt= xViewExt;
+	    dc->dcTransform.dct_yViewExt= yViewExt;
+
+	    break;
+
+	case MM_ANISOTROPIC:
+	    dc->dcTransform.dct_xWinOrg= xWinOrg;
+	    dc->dcTransform.dct_yWinOrg= yWinOrg;
+	    dc->dcTransform.dct_xWinExt= xWinExt;
+	    dc->dcTransform.dct_yWinExt= yWinExt;
+
+	    dc->dcTransform.dct_xViewOrg= xViewOrg;
+	    dc->dcTransform.dct_yViewOrg= yViewOrg;
+	    dc->dcTransform.dct_xViewExt= xViewExt;
+	    dc->dcTransform.dct_yViewExt= yViewExt;
+
+	    break;
+
+	default:
+	    LDEB(mapMode);
+	    break;
+	}
+
+    dc->dcMapMode= mapMode;
+
+    return;
+    }
+
 int appMetaInitDeviceContext(	DeviceContext *		dc,
 				int			objectCount,
-				int			xExt,
-				int			yExt,
+				int			mapMode,
+				int			xWinExt,
+				int			yWinExt,
+				int			xViewExt,
+				int			yViewExt,
 				int			twipsWide,
 				int			twipsHigh )
     {
@@ -68,10 +174,10 @@ int appMetaInitDeviceContext(	DeviceContext *		dc,
     dc->dcJustificationSpaces= 0;
     dc->dcTextAlignment= TA_LEFT|TA_TOP;
 
-    dc->dcTransform.dctOrgX= 0;
-    dc->dcTransform.dctOrgY= 0;
-    dc->dcTransform.dctExtX= xExt;
-    dc->dcTransform.dctExtY= yExt;
+    appWinMetaSetTransform( dc, mapMode,
+				    0, 0, xWinExt, yWinExt,
+				    0, 0, xViewExt, yViewExt,
+				    twipsWide, twipsHigh );
     dc->dcTransform.dctPushed= (DeviceCoordinateTransform *)0;
 
     dc->dcTwipsWide= twipsWide;
@@ -102,13 +208,14 @@ int appMetaInitDeviceContext(	DeviceContext *		dc,
 
     dc->dcPoints= (APP_POINT *)0;
     dc->dcAfi= (AfmFontInfo *)0;
+    dc->dcFontEncoding= -1;
 
     dc->dcRedSet= -1;
     dc->dcGreenSet= -1;
     dc->dcBlueSet= -1;
     dc->dcPenSet.lpStyle= -1;
 
-    docInitTextAttribute( &(dc->dcTextAttributeSet) );
+    utilInitTextAttribute( &(dc->dcTextAttributeSet) );
 
     return 0;
     }
@@ -390,15 +497,20 @@ int appMetaCreateFontIndirect(	DeviceContext *		dc,
 				int			recordSize,
 				SimpleInputStream *	sis )
     {
-    LogicalFont *	lf;
-    DocumentFont *	df;
+    int			rval= 0;
 
-    char *		familyStyle= "fnil";
+    LogicalFont *	lf;
+
+    int			familyStyle= DFstyleFNIL;
 
     int			count;
     int			done;
     int			ob;
-    int			i;
+
+    int			fontNum;
+    DocumentFont	dfNew;
+
+    docInitDocumentFont( &dfNew );
 
     for ( ob= 0; ob < dc->dcObjectCount; ob++ )
 	{
@@ -437,8 +549,8 @@ int appMetaCreateFontIndirect(	DeviceContext *		dc,
     DeviceCoordinateTransform *	dct= &(dc->dcTransform);
 
     if  ( lf->lfHeight < 0 )
-	{ done= ( dc->dcTwipsHigh* -lf->lfHeight )/ dct->dctExtY; }
-    else{ done= ( dc->dcTwipsHigh*  lf->lfHeight )/ dct->dctExtY; }
+	{ done= ( dc->dcTwipsHigh* -lf->lfHeight )/ dct->dct_yWinExt; }
+    else{ done= ( dc->dcTwipsHigh*  lf->lfHeight )/ dct->dct_yWinExt; }
 
     if  ( done < 0 )
 	{ done= -done;	}
@@ -448,49 +560,33 @@ int appMetaCreateFontIndirect(	DeviceContext *		dc,
     lf->lfTextAttribute.taFontSizeHalfPoints= ( done+ 5 )/ 10;
     lf->lfTextAttribute.taFontIsBold= lf->lfWeight > 500;
     lf->lfTextAttribute.taFontIsSlanted= lf->lfItalic != 0;
-    lf->lfTextAttribute.taIsUnderlined= lf->lfUnderline != 0;
+    lf->lfTextAttribute.taTextIsUnderlined= lf->lfUnderline != 0;
+    lf->lfTextAttribute.taHasStrikethrough= lf->lfStrikeOut != 0;
 
     switch( lf->lfPitchAndFamily & 0xf0 )
 	{
-	case FF_ROMAN:		familyStyle= "froman";	break;
-	case FF_SWISS:		familyStyle= "fswiss";	break;
-	case FF_MODERN:		familyStyle= "fmodern";	break;
-	case FF_SCRIPT:		familyStyle= "fscript";	break;
-	case FF_DECORATIVE:	familyStyle= "fdecor";	break;
-	case 0x60:		familyStyle= "ftech";	break;
+	case FF_ROMAN:		familyStyle= DFstyleFROMAN;	break;
+	case FF_SWISS:		familyStyle= DFstyleFSWISS;	break;
+	case FF_MODERN:		familyStyle= DFstyleFMODERN;	break;
+	case FF_SCRIPT:		familyStyle= DFstyleFSCRIPT;	break;
+	case FF_DECORATIVE:	familyStyle= DFstyleFDECOR;	break;
+	case 0x60:		familyStyle= DFstyleFTECH;	break;
 
 	case FF_DONTCARE:
 	default:
 	    break;
 	}
 
-    df= dc->x_dcFontList.dflFonts;
-    for ( i= 0; i < dc->x_dcFontList.dflCount; df++, i++ )
-	{
-	if  ( df->dfDocFamilyNumber < 0	 )
-	    { continue;	}
+    if  ( docFontSetFamilyName( &dfNew, lf->lfFaceName ) )
+	{ LDEB(1); rval= -1; goto ready;	}
+    if  ( docFontSetFamilyStyle( &dfNew, familyStyle ) )
+	{ LDEB(1); rval= -1; goto ready;	}
 
-	if  ( lf->lfFaceName[0]			&&
-	      ! df->dfName			)
-	    { continue;	}
+    fontNum= docMergeFontIntoFontlist( &(dc->x_dcFontList), &dfNew );
+    if  ( fontNum < 0 )
+	{ SLDEB(lf->lfFaceName,fontNum); goto ready;	}
 
-	if  ( ! lf->lfFaceName[0]		&&
-	      ! df->dfName			)
-	    { break;	}
-
-	if  ( ! strcmp( df->dfName, lf->lfFaceName ) )
-	    { break;	}
-	}
-
-    if  ( i >= dc->x_dcFontList.dflCount )
-	{
-	df= docInsertFont( &(dc->x_dcFontList), -1,
-						familyStyle, lf->lfFaceName );
-	if  ( ! df )
-	    { SXDEB(lf->lfFaceName,df); return -1;	}
-	}
-
-    lf->lfTextAttribute.taFontNumber= df->dfDocFamilyNumber;
+    lf->lfTextAttribute.taFontNumber= fontNum;
 
     lf->lfPrivateFont= -1;
 
@@ -499,7 +595,11 @@ int appMetaCreateFontIndirect(	DeviceContext *		dc,
 			    lf->lfWidth,
 			    lf->lfFaceName, ob ));
 
-    return 0;
+  ready:
+
+    docCleanDocumentFont( &dfNew );
+
+    return rval;
     }
 
 int appMeta_ExtTextOut(		SimpleInputStream *	sis,
@@ -628,6 +728,26 @@ int appMeta_TextOut(	SimpleInputStream *	sis,
 /*									*/
 /************************************************************************/
 
+int appMetaSetMapMode(		DeviceContext *		dc,
+				int			recordSize,
+				SimpleInputStream *	sis )
+    {
+    int		mapMode;
+
+    mapMode= sioEndianGetLeInt16( sis );
+
+    WMFDEB(appDebug("SetMapMode(mapMode=%d)\n", mapMode ));
+
+    appWinMetaSetTransform( dc, mapMode,
+				    DC_xWinOrg(dc), DC_yWinOrg(dc),
+				    DC_xWinExt(dc), DC_yWinExt(dc),
+				    DC_xViewOrg(dc), DC_yViewOrg(dc),
+				    DC_xViewExt(dc), DC_yViewExt(dc),
+				    dc->dcTwipsWide, dc->dcTwipsHigh );
+
+    return 0;
+    }
+
 int appMetaOffsetWindowOrg(	DeviceContext *		dc,
 				int			recordSize,
 				SimpleInputStream *	sis )
@@ -640,8 +760,8 @@ int appMetaOffsetWindowOrg(	DeviceContext *		dc,
 
     WMFDEB(appDebug("OffsetWindowOrg(yWinOrg+=%d,xWinOrg+=%d)\n", y0,x0 ));
 
-    dc->dcTransform.dctOrgY += y0;
-    dc->dcTransform.dctOrgX += x0;
+    dc->dcTransform.dct_yWinOrg += y0;
+    dc->dcTransform.dct_xWinOrg += x0;
 
     return 0;
     }
@@ -658,8 +778,8 @@ int appMetaSetWindowOrg(	DeviceContext *		dc,
 
     WMFDEB(appDebug("SetWindowOrg(yWinOrg=%d,xWinOrg=%d)\n", y0,x0 ));
 
-    dc->dcTransform.dctOrgY= y0;
-    dc->dcTransform.dctOrgX= x0;
+    dc->dcTransform.dct_yWinOrg= y0;
+    dc->dcTransform.dct_xWinOrg= x0;
 
     return 0;
     }
@@ -674,10 +794,102 @@ int appMetaSetWindowExt(	DeviceContext *		dc,
     h= sioEndianGetLeInt16( sis );
     w= sioEndianGetLeInt16( sis );
 
-    WMFDEB(appDebug("SetWindowExt(yExt=%d,xExt=%d)\n", h, w ));
+    WMFDEB(appDebug("SetWindowExt(yWinExt=%d,xWinExt=%d)\n", h, w ));
 
-    dc->dcTransform.dctExtY= h;
-    dc->dcTransform.dctExtX= w;
+    dc->dcTransform.dct_yWinExt= h;
+    dc->dcTransform.dct_xWinExt= w;
+
+    /*
+    Necessary?
+    appWinMetaSetTransform( dc, dc->dcMapMode,
+				    DC_xWinOrg(dc), DC_yWinOrg(dc),
+				    DC_xWinExt(dc), DC_yWinExt(dc),
+				    DC_xViewOrg(dc), DC_yViewOrg(dc),
+				    DC_xViewExt(dc), DC_yViewExt(dc),
+				    dc->dcTwipsWide, dc->dcTwipsHigh );
+    */
+
+    return 0;
+    }
+
+int appMetaSetViewportOrg(	DeviceContext *		dc,
+				int			recordSize,
+				SimpleInputStream *	sis )
+    {
+    int		x0;
+    int		y0;
+
+    y0= sioEndianGetLeInt16( sis );
+    x0= sioEndianGetLeInt16( sis );
+
+    WMFDEB(appDebug("SetViewportOrg(yViewOrg=%d,xViewOrg=%d)\n", y0,x0 ));
+
+    dc->dcTransform.dct_yViewOrg= y0;
+    dc->dcTransform.dct_xViewOrg= x0;
+
+    return 0;
+    }
+
+int appMetaSetViewportExt(	DeviceContext *		dc,
+				int			recordSize,
+				SimpleInputStream *	sis )
+    {
+    int		w;
+    int		h;
+
+    h= sioEndianGetLeInt16( sis );
+    w= sioEndianGetLeInt16( sis );
+
+    WMFDEB(appDebug("SetViewportExt(yViewExt=%d,xViewExt=%d)\n", h, w ));
+
+    dc->dcTransform.dct_yViewExt= h;
+    dc->dcTransform.dct_xViewExt= w;
+
+    return 0;
+    }
+
+int appMetaScaleViewportExt(	DeviceContext *		dc,
+				int			recordSize,
+				SimpleInputStream *	sis )
+    {
+    int		yd;
+    int		yn;
+    int		xd;
+    int		xn;
+
+    yd= sioEndianGetLeInt16( sis );
+    yn= sioEndianGetLeInt16( sis );
+    xd= sioEndianGetLeInt16( sis );
+    xn= sioEndianGetLeInt16( sis );
+
+    WMFDEB(appDebug("SetScaleViewportExt(xe*=%d/%d,ye*=%d/%d)\n",
+							xn, xd, yn, yd ));
+
+    dc->dcTransform.dct_yViewExt= ( xn* dc->dcTransform.dct_yViewExt )/ yd;
+    dc->dcTransform.dct_yViewExt= ( yn* dc->dcTransform.dct_yViewExt )/ yd;
+
+    return 0;
+    }
+
+int appMetaScaleWindowExt(	DeviceContext *		dc,
+				int			recordSize,
+				SimpleInputStream *	sis )
+    {
+    int		yd;
+    int		yn;
+    int		xd;
+    int		xn;
+
+    yd= sioEndianGetLeInt16( sis );
+    yn= sioEndianGetLeInt16( sis );
+    xd= sioEndianGetLeInt16( sis );
+    xn= sioEndianGetLeInt16( sis );
+
+    WMFDEB(appDebug("SetScaleWindowExt(xe*=%d/%d,ye*=%d/%d)\n",
+							xn, xd, yn, yd ));
+
+    dc->dcTransform.dct_yWinExt= ( xn* dc->dcTransform.dct_yWinExt )/ yd;
+    dc->dcTransform.dct_yWinExt= ( yn* dc->dcTransform.dct_yWinExt )/ yd;
 
     return 0;
     }
@@ -846,6 +1058,40 @@ int appMetaSetTextAlign(	DeviceContext *		dc,
     return 0;
     }
 
+/************************************************************************/
+/*									*/
+/*  Color Setting.							*/
+/*									*/
+/************************************************************************/
+
+int appMetaSetBkColor(		DeviceContext *		dc,
+				int			recordSize,
+				SimpleInputStream *	sis )
+    {
+    appMetaGetColor( sis, &(dc->dcBkColor) );
+
+    WMFDEB(appDebug("SetBkColor( r=%d, g=%d, b= %d )\n",
+					dc->dcBkColor.rgb8Red,
+					dc->dcBkColor.rgb8Green,
+					dc->dcBkColor.rgb8Blue ));
+
+    return 0;
+    }
+
+int appMetaSetTextColor(	DeviceContext *		dc,
+				int			recordSize,
+				SimpleInputStream *	sis )
+    {
+    appMetaGetColor( sis, &(dc->dcTextColor) );
+
+    WMFDEB(appDebug("SetTextColor( r=%d, g=%d, b= %d )\n",
+					dc->dcTextColor.rgb8Red,
+					dc->dcTextColor.rgb8Green,
+					dc->dcTextColor.rgb8Blue ));
+
+    return 0;
+    }
+
 /************************************************************************/
 /*									*/
 /*  Extract a bitmap from the metafile stream.				*/
