@@ -25,7 +25,7 @@
 #    define	WMFDEB(x)	/*nothing*/
 # endif
 
-static void appMetaCleanObjextX11(	DeviceContextX11 *	dcx,
+static void appMetaCleanObjectX11(	DeviceContextX11 *	dcx,
 					MetaFileObject *	mfo )
     {
     DeviceContext *	dc= &(dcx->dcxDeviceContext);
@@ -66,7 +66,7 @@ static void appMetaCleanDeviceContextX11(	DeviceContextX11 *	dcx )
     int			ob;
 
     for ( ob= 0; ob < dc->dcObjectCount; ob++ )
-	{ appMetaCleanObjextX11( dcx, &(dc->dcObjects[ob]) );	}
+	{ appMetaCleanObjectX11( dcx, &(dc->dcObjects[ob]) );	}
 
     appMetaCleanDeviceContext( dc );
 
@@ -537,6 +537,8 @@ static int appMetaGetPointsX11(	DeviceContextX11 *	dcx,
 	xp->y= DCX_yViewport( y, dcx );
 	}
 
+    *xp= dc->dcPoints[0];
+
     return 0;
     }
 
@@ -623,15 +625,11 @@ static int appMetaBitmapImageX11(	AppBitmapImage **	pAbi,
     AppBitmapImage *	abi;
     BitmapDescription *	bd;
 
-    int			twipsWide;
-    int			twipsHigh;
     int			pixelsWide;
     int			pixelsHigh;
 
     APP_IMAGE *		xim= (APP_IMAGE *)0;
 
-    int			dstYExtAbs;
-
     if  ( expectBytes < 40 )
 	{
 	int	done= 0;
@@ -661,63 +659,8 @@ static int appMetaBitmapImageX11(	AppBitmapImage **	pAbi,
 	return 0;
 	}
 
-    dstYExtAbs= dstYExt;
-    if  ( dstYExtAbs < 0 )
-	{ dstYExtAbs= -dstYExtAbs;	}
-
-    switch( bd->bdUnit )
-	{
-	case BMunM:
-	    twipsWide= ( dstXExt* TWIPS_PER_M )/ bd->bdXResolution;
-	    twipsHigh= ( dstYExtAbs* TWIPS_PER_M )/ bd->bdYResolution;
-
-	    pixelsWide= ( dcx->dcxPixelsWide* twipsWide )/ dc->dcTwipsWide;
-	    pixelsHigh= ( dcx->dcxPixelsHigh* twipsHigh )/ dc->dcTwipsHigh;
-
-	    dstXExt= DC_wViewport( dstXExt, dc );
-	    dstYExt= DC_hViewport( dstYExtAbs, dc );
-
-	    break;
-
-	case BMunINCH:
-	    twipsWide= ( dstXExt* 20* 72 )/ bd->bdXResolution;
-	    twipsHigh= ( dstYExtAbs* 20* 72 )/ bd->bdYResolution;
-
-	    pixelsWide= ( dcx->dcxPixelsWide* twipsWide )/ dc->dcTwipsWide;
-	    pixelsHigh= ( dcx->dcxPixelsHigh* twipsHigh )/ dc->dcTwipsHigh;
-
-	    dstXExt= DC_wViewport( dstXExt, dc );
-	    dstYExt= DC_hViewport( dstYExtAbs, dc );
-	    break;
-
-	case BMunPOINT:
-	    twipsWide= ( dstXExt* 20 )/ bd->bdXResolution;
-	    twipsHigh= ( dstYExtAbs* 20 )/ bd->bdYResolution;
-
-	    pixelsWide= ( dcx->dcxPixelsWide* twipsWide )/ dc->dcTwipsWide;
-	    pixelsHigh= ( dcx->dcxPixelsHigh* twipsHigh )/ dc->dcTwipsHigh;
-
-	    dstXExt= DC_wViewport( dstXExt, dc );
-	    dstYExt= DC_hViewport( dstYExtAbs, dc );
-	    break;
-
-	case BMunPIXEL:
-	    dstXExt= DC_wViewport( dstXExt, dc );
-	    dstYExt= DC_hViewport( dstYExtAbs, dc );
-
-	    pixelsWide= dstXExt;
-	    pixelsHigh= dstYExt;
-	    break;
-
-	default:
-	    SDEB(bmunIntToString(bd->bdUnit));
-	    dstXExt= DC_wViewport( dstXExt, dc );
-	    dstYExt= DC_hViewport( dstYExtAbs, dc );
-
-	    pixelsWide= dstXExt;
-	    pixelsHigh= dstYExt;
-	    break;
-	}
+    dstXExt= DC_wViewport( dstXExt, dc );
+    dstYExt= DC_hViewport( dstYExt, dc );
 
     dstX= DCX_xViewport( dstX, dcx );
     dstY= DCX_yViewport( dstY, dcx );
@@ -727,6 +670,9 @@ static int appMetaBitmapImageX11(	AppBitmapImage **	pAbi,
     if  ( dstYExt < 0 )
 	{ dstYExt= -dstYExt;	}
 
+    pixelsWide= dstXExt;
+    pixelsHigh= dstYExt;
+
     if  ( appImgMakeImage( add, &xim, pixelsWide, pixelsHigh, ac, abi ) )
 	{ LDEB(1); appCleanBitmapImage( abi ); free( abi ); return -1; }
 
@@ -734,6 +680,11 @@ static int appMetaBitmapImageX11(	AppBitmapImage **	pAbi,
     XPutImage( add->addDisplay, add->addDrawable, add->addGc,
 					    xim, srcX, srcY, dstX, dstY,
 					    pixelsWide, pixelsHigh );
+#   ifdef USE_MECHECK
+    if  ( xim->data )
+	{ free( xim->data ); xim->data= (char *)0; }
+#   endif
+
     XDestroyImage( xim );
 #   endif
 
@@ -1339,6 +1290,18 @@ static int appMeta_PolygonX11(	SimpleInputStream *	sis,
     WMFDEB(appDebug("Polygon( count=%d, ... ) Fill= %d Borders= %d\n",
 			    count, dc->dcFillInsides, dc->dcDrawBorders ));
 
+#   if 1
+    {
+    int	i;
+
+    for ( i= 0; i < count; i++ )
+	{
+	WMFDEB(appDebug(" %3d: [%3d,%3d]\n",
+			   i,  dc->dcPoints[i].x, dc->dcPoints[i].y ));
+	}
+    }
+#   endif
+
     if  ( dc->dcFillInsides )
 	{
 	appMeta_SetFillX11( dcx, 1 );
@@ -1352,7 +1315,7 @@ static int appMeta_PolygonX11(	SimpleInputStream *	sis,
 	{
 	appDrawSetForegroundColor( add, &(dcx->dcxPenColor) );
 
-	appDrawDrawLines( add, dc->dcPoints, count );
+	appDrawDrawLines( add, dc->dcPoints, count+ 1 );
 	}
 
     return 0;
@@ -1432,7 +1395,7 @@ static void appMetaDrawStringX11(	int			x0,
 
     appDrawDrawString( add, x0, y0, (char *)dc->dcString, count );
 
-    if  ( dc->dcFont.lfTextAttribute.taIsUnderlined )
+    if  ( dc->dcFont.lfTextAttribute.taTextIsUnderlined )
 	{
 	appDrawFillRectangle( add,
 				x0, y0+ apf->apfUnderLinePositionPixels,
@@ -1661,7 +1624,7 @@ static int appMeta_PolyPolygonX11(	SimpleInputStream *	sis,
 	if  ( appMetaGetPointsX11( dcx, counts[i], sis ) )
 	    { LDEB(counts[i]); return -1;	}
 
-#	if 0
+#	if 1
 	if  ( dc->dcFillInsides )
 	    {
 	    appDrawSetForegroundColor( add, &(dcx->dcxBrushColor) );
@@ -1674,14 +1637,20 @@ static int appMeta_PolyPolygonX11(	SimpleInputStream *	sis,
 	    {
 	    appDrawSetForegroundColor( add, &(dcx->dcxPenColor) );
 
-	    appDrawDrawLines( add, dc->dcPoints, counts[i] );
+	    appDrawDrawLines( add, dc->dcPoints, counts[i]+ 1 );
 	    }
 	}
 
     return 0;
     }
 
-static void appMetaFillBackground(	DeviceContextX11 *	dcx )
+/************************************************************************/
+/*									*/
+/*  Fill the background of the pixmap.					*/
+/*									*/
+/************************************************************************/
+
+static void appMetaFillBackgroundX11(	DeviceContextX11 *	dcx )
     {
     DeviceContext *	dc= &(dcx->dcxDeviceContext);
     AppDrawingData *	add= &(dc->dcDrawingData);
@@ -1896,7 +1865,7 @@ static int appWinMetaPlayRecords(	SimpleInputStream *	sis,
 		if  ( ob < 0 || ob >= dc->dcObjectCount )
 		    { LLDEB(ob,dc->dcObjectCount); return -1;	}
 
-		appMetaCleanObjextX11( dcx, &(dc->dcObjects[ob]) );
+		appMetaCleanObjectX11( dcx, &(dc->dcObjects[ob]) );
 		appMetaCleanObject( &(dc->dcObjects[ob]) );
 
 		dc->dcObjects[ob].mfoType= MFtypeFREE;
@@ -2192,7 +2161,7 @@ int appMetaPlayWmfX11(	SimpleInputStream *	sis,
     dcx.dcxFillTiled= 0;
     dcx.dcxFillHatched= 0;
 
-    appMetaFillBackground( &dcx );
+    appMetaFillBackgroundX11( &dcx );
 
     WMFDEB(appDebug(
 		"X11 WMF winExt: [%dx%d], pix: [%dx%d], twips: [%dx%d]\n",
