@@ -4,6 +4,9 @@
 /*									*/
 /************************************************************************/
 
+#   ifndef	APP_SYSTEM_H
+#   define	APP_SYSTEM_H
+
 typedef int (*FILE_CALLBACK)	(		const char *	name,
 						void *		through );
 
@@ -60,7 +63,8 @@ extern int appMakeUniqueString(	char *		target,
 
 extern int appAbsoluteName(	char *		absolute,
 				int		len,
-				const char *	filename );
+				const char *	filename,
+				const char *	fileRelativeTo );
 
 extern int appRemoveFile(	const char *	filename );
 
@@ -80,3 +84,7 @@ extern int appListenSocket(	const char *		portName,
 extern int appAcceptSocket(	int			lfd,
 				void *			through,
 				APP_COMPLAIN		complain );
+
+extern const char * appFileExtensionOfName(	const char *	filename );
+
+#   endif	/*  APP_SYSTEM_H	*/
