@@ -1,15 +1,17 @@
 /************************************************************************/
 /*									*/
-/*  Simple io streams, recursive use for hexadecimal data.		*/
+/*  Simple io streams translating from/to the MIME (RFC 2045)		*/
+/*  'Base 64' content transfer encoding.				*/
 /*									*/
 /************************************************************************/
 
-#   include	"config.h"
+#   include	"appUtilConfig.h"
 
 #   include	<stdlib.h>
 #   include	<string.h>
 
 #   include	"sioBase64.h"
+#   include	"utilBase64.h"
 #   include	<appDebugon.h>
 
 
@@ -19,14 +21,6 @@
 /*									*/
 /************************************************************************/
 
-static const unsigned char	SioBase64Digits[]=
-				       "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
-				       "abcdefghijklmnopqrstuvwxyz"
-				       "0123456789+/";
-
-static unsigned char		SioBase64Indices[256];
-
-
 static int sioBase64Seek(	void *			voidsos,
 				long			pos )
     { LDEB(1); return -1; return 0; }
@@ -49,6 +43,7 @@ typedef struct Base64InputStream
     unsigned char		bisBytes[3];
     int				bisCount;
     int				bisColumn;
+    const int *			bisBase64Indices;
     } Base64InputStream;
 
 static int sioInBase64ReadBytes(	void *		voidbis,
@@ -64,6 +59,8 @@ static int sioInBase64ReadBytes(	void *		voidbis,
 	int			d[4];
 	int			n;
 
+	d[3]= d[2]= d[1]= d[0]= 0;
+
 	if  ( bis->bisCount > 0 )
 	    {
 	    memcpy( buffer, bis->bisBytes, bis->bisCount );
@@ -93,23 +90,24 @@ static int sioInBase64ReadBytes(	void *		voidbis,
 	    { LDEB(n); break; }
 
 	for ( i= 0; i < n; i++ )
-	    { d[i]= SioBase64Indices[d[i]]; }
+	    {
+	    d[i]= bis->bisBase64Indices[d[i]];
 
-	bis->bisBytes[bis->bisCount++]=	( ( d[0] & 0xcf ) << 2 )	|
-					( ( d[1] & 0x30 ) >> 4 )	;
+	    if  ( d[i] >= 64 )
+		{ LDEB(d[i]); return -1;	}
+	    }
+
+	bis->bisBytes[bis->bisCount++]=	utilBase64Byte0( d );
 
 	if  ( n == 2 )
 	    { break; }
 
-	bis->bisBytes[bis->bisCount++]=	( ( d[1] & 0x0f ) << 4 )	|
-					( ( d[2] & 0x3c ) >> 2 )	;
+	bis->bisBytes[bis->bisCount++]=	utilBase64Byte1( d );
 
 	if  ( n == 3 )
 	    { break; }
 
-	bis->bisBytes[bis->bisCount++]=	( ( d[2] & 0x03 ) << 6 )	|
-					( ( d[3] & 0x3f ) >> 0 )	;
-
+	bis->bisBytes[bis->bisCount++]=	utilBase64Byte2( d );
 	}
 
     if  ( done < count && bis->bisCount > 0 )
@@ -161,18 +159,6 @@ SimpleInputStream * sioInBase64Open(	SimpleInputStream *	sisBase64 )
     SimpleInputStream *	sis;
     Base64InputStream *	bis;
 
-    if  ( SioBase64Indices[0] == 0 )
-	{
-	int	i;
-
-	for ( i= 0; i < sizeof(SioBase64Indices); i++ )
-	    { SioBase64Indices[i]= 0xff;	}
-
-	i= 0;
-	while( SioBase64Digits[i] )
-	    { SioBase64Indices[SioBase64Digits[i]]= i++;	}
-	}
-
     bis= (Base64InputStream *)malloc( sizeof(Base64InputStream) );
     if  ( ! bis )
 	{ XDEB(bis); return (SimpleInputStream *)0;	}
@@ -180,6 +166,8 @@ SimpleInputStream * sioInBase64Open(	SimpleInputStream *	sisBase64 )
     bis->bisSisBase64= sisBase64;
     bis->bisCount= 0;
     bis->bisColumn= 0;
+    bis->bisBase64Indices= utilBase64GetIndexArray();
+
 
     sis= sioInOpen( (void *)bis, sioInBase64ReadBytes,
 					    sioBase64Seek, sioInBase64Close );
@@ -204,15 +192,6 @@ typedef struct Base64OutputStream
     int				bosColumn;
     } Base64OutputStream;
 
-#   define	sosBase64Make(d,b)				\
-			d[0]=	( ( (b)[0] & 0xfc ) >> 2 ) ;	\
-			d[1]=	( ( (b)[0] & 0x03 ) << 4 ) |	\
-				( ( (b)[1] & 0xf0 ) >> 4 ) ;	\
-			d[2]=	( ( (b)[1] & 0x0f ) << 2 ) |	\
-				( ( (b)[2] & 0xc0 ) >> 6 ) ;	\
-			d[3]=	( ( (b)[2] & 0x3f ) >> 0 ) ;
-
-
 /************************************************************************/
 /*									*/
 /*  Close a base64 stream.						*/
@@ -250,11 +229,13 @@ static int sioOutBase64Close(	void *	voidbos )
 	    bos->bosColumn= 0;
 	    }
 
-	sosBase64Make( d, bos->bosBytes );
+	util_Base64Make( d, bos->bosBytes );
 
 	/*  4  */
 	for ( i= 0; i < bos->bosCount+ 1; i++ )
-	    { sioOutPutCharacter( SioBase64Digits[d[i]], bos->bosSosBase64 ); }
+	    {
+	    sioOutPutCharacter( UTIL_Base64Digits[d[i]], bos->bosSosBase64 );
+	    }
 
 	for ( i= bos->bosCount; i < 3; i++ )
 	    { sioOutPutCharacter( '=', bos->bosSosBase64 ); }
@@ -305,9 +286,9 @@ static int sioOutBase64WriteBytes(	void *			voidbos,
 	bos->bosColumn= 0;
 	}
 
-    sosBase64Make( d, bos->bosBytes );
+    util_Base64Make( d, bos->bosBytes );
     for ( i= 0; i < 4; i++ )
-	{ sioOutPutCharacter( SioBase64Digits[d[i]], bos->bosSosBase64 ); }
+	{ sioOutPutCharacter( UTIL_Base64Digits[d[i]], bos->bosSosBase64 ); }
     bos->bosColumn += 4;
 
     bos->bosCount= 0;
@@ -322,9 +303,11 @@ static int sioOutBase64WriteBytes(	void *			voidbos,
 	    bos->bosColumn= 0;
 	    }
 
-	sosBase64Make( d, buffer );
+	util_Base64Make( d, buffer );
 	for ( i= 0; i < 4; i++ )
-	    { sioOutPutCharacter( SioBase64Digits[d[i]], bos->bosSosBase64 ); }
+	    {
+	    sioOutPutCharacter( UTIL_Base64Digits[d[i]], bos->bosSosBase64 );
+	    }
 	bos->bosColumn += 4;
 
 	buffer += 3; done += 3;
