@@ -9,4 +9,8 @@
 
 extern SimpleInputStream * sioInHexOpen(	SimpleInputStream * sisHex );
 
+extern SimpleOutputStream * sioOutHexOpenFolded(
+				    SimpleOutputStream *	sosHex,
+				    int				wide );
+
 extern SimpleOutputStream * sioOutHexOpen(	SimpleOutputStream * sosHex );
