@@ -16,6 +16,8 @@
 /*		Unique names).						*/
 /*  26 Jun 96 - Version 3.0 by Eric S. Raymond (Full GIF89 support)	*/
 /*									*/
+/*  Also includes fragments borrowed from the ImageMagick code.		*/
+/*									*/
 /************************************************************************/
 
 #   include	"appUtilConfig.h"
@@ -55,11 +57,11 @@ typedef struct LzwState
 			/*  indicate that we are at the beginning of	*/
 			/*  the output.					*/
 			/************************************************/
-    int			lsRunningCode;
+    int			lsNextCode;
 			/************************************************/
 			/*  The next code the algorithm can generate.	*/
 			/************************************************/
-    int			lsRunningBits;
+    int			lsCurrentCodeSize;
 			/************************************************/
 			/*  The number of bits required to represent	*/
 			/*  RunningCode.				*/
@@ -79,8 +81,8 @@ typedef struct LzwState
 			/************************************************/
 			/*  Special Codes.				*/
 			/************************************************/
-    int			lsCrntShiftState;
-    unsigned long	lsCrntShiftDWord;
+    int			lsBitsAccumulated;
+    unsigned long	lsAccumulator;
 			/************************************************/
 			/*  For collecting codes in bytes.		*/
 			/*  -   Number of bits in CrntShiftDWord	*/
@@ -100,11 +102,11 @@ static void sioLzwInitState(	LzwState *	ls,
     ls->lsInitialCodeSize= codeSize;
     ls->lsClearCode= ( 1 << codeSize );
     ls->lsEOFCode= ls->lsClearCode+ 1;
-    ls->lsRunningCode= ls->lsEOFCode+ 1;
-    ls->lsRunningBits= codeSize + 1;
-    ls->lsMaxCode1= 1 << ls->lsRunningBits;
-    ls->lsCrntShiftState= 0;
-    ls->lsCrntShiftDWord= 0;
+    ls->lsNextCode= ls->lsEOFCode+ 1;
+    ls->lsCurrentCodeSize= codeSize + 1;
+    ls->lsMaxCode1= 1 << ls->lsCurrentCodeSize;
+    ls->lsBitsAccumulated= 0;
+    ls->lsAccumulator= 0;
 
     ls->lsCurrentCode= FIRST_CODE;
 
@@ -155,7 +157,7 @@ typedef struct LzwInputStream
 /*									*/
 /*  This routine is responsable for the decompression of the bit	*/
 /*  stream from 8 bits (bytes) packets, into the real codes.		*/
-/*  Returns GIF_OK if read succesfully.					*/
+/*  Returns 0 if read succesfully, -1 otherwise.			*/
 /*									*/
 /*  1)  As long as we need to get more bytes from input stream for	*/
 /*	next code							*/
@@ -193,7 +195,7 @@ static int sioInLzwReadCode(	LzwInputStream *	lis,
 	};
 
     /*  1  */
-    while( ls->lsCrntShiftState < ls->lsRunningBits )
+    while( ls->lsBitsAccumulated < ls->lsCurrentCodeSize )
 	{
 	int			byte;
 
@@ -201,23 +203,23 @@ static int sioInLzwReadCode(	LzwInputStream *	lis,
 	if  ( byte == EOF )
 	    { XDEB(byte); return -1; }
 
-	ls->lsCrntShiftDWord |= ((unsigned long) byte) << ls->lsCrntShiftState;
-	ls->lsCrntShiftState += 8;
+	ls->lsAccumulator |= ((unsigned long) byte) << ls->lsBitsAccumulated;
+	ls->lsBitsAccumulated += 8;
 	}
 
     /*  2  */
-    *pCode = ls->lsCrntShiftDWord & CodeMasks[ls->lsRunningBits];
+    *pCode = ls->lsAccumulator & CodeMasks[ls->lsCurrentCodeSize];
 
     /*  3  */
-    ls->lsCrntShiftDWord >>= ls->lsRunningBits;
-    ls->lsCrntShiftState -=  ls->lsRunningBits;
+    ls->lsAccumulator >>= ls->lsCurrentCodeSize;
+    ls->lsBitsAccumulated -=  ls->lsCurrentCodeSize;
 
     /*  4  */
-    if  ( ++ls->lsRunningCode > ls->lsMaxCode1	&&
-	  ls->lsRunningBits < LZ_BITS		)
+    if  ( ++ls->lsNextCode > ls->lsMaxCode1	&&
+	  ls->lsCurrentCodeSize < LZ_BITS		)
 	{
 	ls->lsMaxCode1 <<= 1;
-	ls->lsRunningBits++;
+	ls->lsCurrentCodeSize++;
 	}
 
     return 0;
@@ -229,8 +231,8 @@ static int sioInLzwReadCode(	LzwInputStream *	lis,
 /*  which is not a code, but a pixel value (less than ClearCode).	*/
 /*  Returns that pixel value.						*/
 /*  If the image is defective, we might loop here forever, so we limit	*/
-/*  the loops to the maximum possible if is image O.k.: LZ_MAX_CODE	*/
-/*  times.								*/
+/*  the loops to the maximum possible when the image is image Ok:	*/
+/*  LZ_MAX_CODE times.							*/
 /*									*/
 /************************************************************************/
 
@@ -310,9 +312,9 @@ static int sioInLzwReadBytes(	void *		voidlis,
 	else if (CrntCode == ClearCode) {
 	    /* We need to start over again: */
 	    for (j = 0; j <= LZ_MAX_CODE; j++) Prefix[j] = NO_SUCH_CODE;
-	    ls->lsRunningCode = ls->lsEOFCode + 1;
-	    ls->lsRunningBits = ls->lsInitialCodeSize + 1;
-	    ls->lsMaxCode1 = 1 << ls->lsRunningBits;
+	    ls->lsNextCode = ls->lsEOFCode + 1;
+	    ls->lsCurrentCodeSize = ls->lsInitialCodeSize + 1;
+	    ls->lsMaxCode1 = 1 << ls->lsCurrentCodeSize;
 	    LastCode= lis->lisLastCode= NO_SUCH_CODE;
 	}
 	else {
@@ -333,9 +335,9 @@ static int sioInLzwReadBytes(	void *		voidlis,
 		    /* In that case CrntCode = XXXCode, CrntCode or the	     */
 		    /* prefix code is last code and the suffix char is	     */
 		    /* exactly the prefix of last code!			     */
-		    if (CrntCode == ls->lsRunningCode - 2) {
+		    if (CrntCode == ls->lsNextCode - 2) {
 			CrntPrefix = LastCode;
-			Suffix[ls->lsRunningCode - 2] =
+			Suffix[ls->lsNextCode - 2] =
 			Stack[StackPtr++] = sioInLzwGetPrefixChar(Prefix,
 							LastCode, ClearCode);
 		    }
@@ -370,18 +372,18 @@ static int sioInLzwReadBytes(	void *		voidlis,
 		    buffer[i++] = Stack[--StackPtr];
 	    }
 	    if (LastCode != NO_SUCH_CODE) {
-		Prefix[ls->lsRunningCode - 2] = LastCode;
+		Prefix[ls->lsNextCode - 2] = LastCode;
 
-		if (CrntCode == ls->lsRunningCode - 2) {
+		if (CrntCode == ls->lsNextCode - 2) {
 		    /* Only allowed if CrntCode is exactly the running code: */
 		    /* In that case CrntCode = XXXCode, CrntCode or the	     */
 		    /* prefix code is last code and the suffix char is	     */
 		    /* exactly the prefix of last code!			     */
-		    Suffix[ls->lsRunningCode - 2] =
+		    Suffix[ls->lsNextCode - 2] =
 			sioInLzwGetPrefixChar(Prefix, LastCode, ClearCode);
 		}
 		else {
-		    Suffix[ls->lsRunningCode - 2] =
+		    Suffix[ls->lsNextCode - 2] =
 			sioInLzwGetPrefixChar(Prefix, CrntCode, ClearCode);
 		}
 	    }
@@ -404,26 +406,26 @@ static int sioInLzwReadBytes(	void *		voidlis,
 /*									*/
 /************************************************************************/
 
-static int sioInLzwClose(	void *	voidfis )
+static int sioInLzwClose(	void *	voidlis )
     {
     int				rval= 0;
-    LzwInputStream *		lis= (LzwInputStream *)voidfis;
+    LzwInputStream *		lis= (LzwInputStream *)voidlis;
 
     /*  1 */
-    while ( ! lis->lisExhausted )
+    while( ! lis->lisExhausted )
 	{
 	unsigned char	buffer[SIOsizBUF];
 
-	if  ( sioInLzwReadBytes( voidfis, buffer, SIOsizBUF ) < 0 )
+	if  ( sioInLzwReadBytes( voidlis, buffer, SIOsizBUF ) < 0 )
 	    { LDEB(1); rval= -1; break;	}
 	}
 
-    free( voidfis );
+    free( voidlis );
 
     return rval;
     }
 
-SimpleInputStream * sioInLzwOpen(	SimpleInputStream *	sisLzw,
+SimpleInputStream * sioInLzwGifOpen(	SimpleInputStream *	sisLzw,
 					int			codeSize )
     {
     SimpleInputStream *		sis;
@@ -461,8 +463,23 @@ SimpleInputStream * sioInLzwOpen(	SimpleInputStream *	sisLzw,
 /*									*/
 /*  SimpleOutputStreams doing LZW compression.				*/
 /*									*/
+/*  1)  Mask nonsense bits from the input of the compressor.		*/
+/*									*/
 /************************************************************************/
 
+static const unsigned char SIO_LZW_OutMasks[]=
+    {
+    0x00,
+    0x01,
+    0x03,
+    0x07,
+    0x0f,
+    0x1f,
+    0x3f,
+    0x7f,
+    0xff
+    };
+
 #define HT_SIZE			8192	   /* 12bits = 4096 or twice as big! */
 #define HT_KEY_MASK		0x1FFF			      /* 13bits keys */
 #define HT_KEY_NUM_BITS		13			      /* 13bits keys */
@@ -543,52 +560,45 @@ static int _ExistsHashTable(	GifHashTableType *	HashTable,
 /*									*/
 /************************************************************************/
 
-typedef struct LzwOutputStream
+typedef struct LzwGifOutputStream
     {
     GifHashTableType		losHashTable;
     LzwState			losState;
     SimpleOutputStream *	losSosLzw;
-    } LzwOutputStream;
+    } LzwGifOutputStream;
 
 /************************************************************************/
 /*									*/
 /*  The actual output routine to the (blocked) stream: 			*/
-/*  This routine is responsible for the compression of the bit stream	*/
+/*  This routine is responsible for the translation of the bit stream	*/
 /*  into 8 bits (byte) packets.						*/
 /*									*/
-/*  1)  If code cannot fit into RunningBits bits, must raise its size.	*/
-/*	Note however that codes above 4095 are used for special		*/
-/*	signaling.							*/
+/*  1)  Emit full bytes.						*/
 /*									*/
 /************************************************************************/
 
-static int sioOutLzwWriteCode(	LzwOutputStream *	los,
-				int			Code )
-{
+static int sioOutLzwGifWriteCode(	LzwGifOutputStream *	los,
+					int			code )
+    {
     LzwState *			ls= &(los->losState);
     int				rval= 0;
 
-    ls->lsCrntShiftDWord |= ( (long)Code ) << ls->lsCrntShiftState;
-    ls->lsCrntShiftState += ls->lsRunningBits;
+    ls->lsAccumulator |= ( (long)code ) << ls->lsBitsAccumulated;
+    ls->lsBitsAccumulated += ls->lsCurrentCodeSize;
 
-    while( ls->lsCrntShiftState >= 8 )
+    while( ls->lsBitsAccumulated >= 8 )
 	{
-	/* Dump out full bytes: */
-	if  ( sioOutPutCharacter( ls->lsCrntShiftDWord & 0xff,
+	/*  1  */
+	if  ( sioOutPutCharacter( ls->lsAccumulator & 0xff,
 						    los->losSosLzw ) == EOF )
 	    { rval= -1;	}
 
-	ls->lsCrntShiftDWord >>= 8;
-	ls->lsCrntShiftState -= 8;
+	ls->lsAccumulator >>= 8;
+	ls->lsBitsAccumulated -= 8;
 	}
 
-    /*  1  */
-    if  ( ls->lsRunningCode >= ls->lsMaxCode1	&&
-	  Code <= LZ_MAX_CODE			)
-	{ ls->lsMaxCode1= 1 << ++ls->lsRunningBits;	}
-
     return rval;
-}
+    }
 
 /************************************************************************/
 /*									*/
@@ -602,29 +612,29 @@ static int sioOutLzwWriteCode(	LzwOutputStream *	los,
 /*									*/
 /************************************************************************/
 
-static int sioOutLzwClose(	void *		voidlos )
+static int sioOutLzwGifClose(	void *		voidlos )
     {
     int				rval= 0;
-    LzwOutputStream *		los= (LzwOutputStream *)voidlos;
+    LzwGifOutputStream *	los= (LzwGifOutputStream *)voidlos;
     LzwState *			ls= &(los->losState);
 
     /*  1  */
-    if  ( sioOutLzwWriteCode( los, ls->lsCurrentCode ) )
+    if  ( sioOutLzwGifWriteCode( los, ls->lsCurrentCode ) )
 	{ XDEB(ls->lsCurrentCode); rval= -1;	}
 
     /*  2  */
-    if  ( sioOutLzwWriteCode( los, ls->lsEOFCode ) )
-	{ XDEB(ls->lsCurrentCode); rval= -1;	}
+    if  ( sioOutLzwGifWriteCode( los, ls->lsEOFCode ) )
+	{ XDEB(ls->lsEOFCode); rval= -1;	}
 
     /*  3  */
-    while( ls->lsCrntShiftState > 0 )
+    while( ls->lsBitsAccumulated > 0 )
 	{
-	if  ( sioOutPutCharacter( ls->lsCrntShiftDWord & 0xff,
+	if  ( sioOutPutCharacter( ls->lsAccumulator & 0xff,
 						    los->losSosLzw ) == EOF )
 	    { LDEB(1); rval= -1;	}
 
-	ls->lsCrntShiftDWord >>= 8;
-	ls->lsCrntShiftState -=  8;
+	ls->lsAccumulator >>= 8;
+	ls->lsBitsAccumulated -=  8;
 	}
 
     /*  4  */
@@ -636,7 +646,7 @@ static int sioOutLzwClose(	void *		voidlos )
 /************************************************************************/
 /*									*/
 /*  The LZ compression routine:						*/
-/*  This version compress the given buffer Line of length LineLen.	*/
+/*  This version compress the given buffer of length count.		*/
 /*  This routine can be called several times (one per scan line, for	*/
 /*  example), in order to compress the whole image			*/
 /*									*/
@@ -648,85 +658,77 @@ static int sioOutLzwClose(	void *		voidlos )
 /*  4)  If it is there, simply take the new code as the current code.	*/
 /*  5)  If not, emit the current code and set the current code to the	*/
 /*	pixel value.							*/
+/*      If next code cannot fit into the current code size, must raise	*/
+/*	the code size.							*/
 /*  6)  BUT.. If the hash table is full, send a Clear code and clear	*/
 /*	the hash table as well.						*/
-/*  7)  Writing is some codes behind as long as there is a code for the	*/
-/*	current string in the hash table. Keep the current code on the	*/
-/*	compressor to be used in the next round.			*/
 /*									*/
 /************************************************************************/
 
-static int sioOutLzwWriteBytes(	void *			voidlos,
+static int sioOutLzwGifWriteBytes(
+				void *			voidlos,
 				const unsigned char *	buffer,
 				int			count )
 {
-    LzwOutputStream *		los= (LzwOutputStream *)voidlos;
+    LzwGifOutputStream *	los= (LzwGifOutputStream *)voidlos;
     LzwState *			ls= &(los->losState);
 
-    int				currentCode;
     int				i= 0;
 
-    static unsigned char CodeMask[]=
-	{
-	0x00,
-	0x01,
-	0x03,
-	0x07,
-	0x0f,
-	0x1f,
-	0x3f,
-	0x7f,
-	0xff
-	};
-
     /*  1  */
     if  ( ls->lsCurrentCode == FIRST_CODE )
-	{ currentCode= buffer[i++] & CodeMask[ls->lsInitialCodeSize];	}
-    else{ currentCode= ls->lsCurrentCode;				}
+	{
+	ls->lsCurrentCode=
+		    buffer[i++] & SIO_LZW_OutMasks[ls->lsInitialCodeSize];
+	}
 
     while( i < count )
 	{
-	int	Pixel= buffer[i++] & CodeMask[ls->lsInitialCodeSize];
-	int	NewKey;
-	int	NewCode;
+	int	val= buffer[i++] & SIO_LZW_OutMasks[ls->lsInitialCodeSize];
+	int	nextKey;
+	int	nextCode;
 
 	/*  2  */
-	NewKey= ( ((unsigned long) currentCode ) << 8) + Pixel;
+	nextKey= ( ((unsigned long) ls->lsCurrentCode ) << 8) + val;
 
 	/*  3  */
-	NewCode= _ExistsHashTable( &(los->losHashTable), NewKey );
+	nextCode= _ExistsHashTable( &(los->losHashTable), nextKey );
 
 	/*  4  */
-	if  ( NewCode >= 0 )
-	    { currentCode= NewCode;	}
+	if  ( nextCode >= 0 )
+	    { ls->lsCurrentCode= nextCode;	}
 	else{
 	    /*  5  */
-	    if  ( sioOutLzwWriteCode( los, currentCode ) )
-		{ LDEB(currentCode); return -1;	}
+	    if  ( sioOutLzwGifWriteCode( los, ls->lsCurrentCode ) )
+		{ LDEB(ls->lsCurrentCode); return -1;	}
 
-	    currentCode= Pixel;
+	    ls->lsCurrentCode= val;
+
+	    if  ( ls->lsNextCode >= ls->lsMaxCode1 )
+		{ ls->lsMaxCode1= 1 << ++(ls->lsCurrentCodeSize);	}
 
 	    /*  6  */
-	    if  ( ls->lsRunningCode >= LZ_MAX_CODE )
+	    if  ( ls->lsCurrentCodeSize > 12 )
 		{
-		if  ( sioOutLzwWriteCode( los, ls->lsClearCode ) )
-		    { LDEB(currentCode); return -1;	}
+		ls->lsCurrentCodeSize= 12;
+
+		if  ( sioOutLzwGifWriteCode( los, ls->lsClearCode ) )
+		    { LDEB(ls->lsCurrentCode); return -1;	}
+
+		ls->lsNextCode= ls->lsEOFCode+ 1;
+		ls->lsCurrentCodeSize= ls->lsInitialCodeSize+ 1;
+		ls->lsMaxCode1= 1 << ls->lsCurrentCodeSize;
 
-		ls->lsRunningCode = ls->lsEOFCode + 1;
-		ls->lsRunningBits = ls->lsInitialCodeSize + 1;
-		ls->lsMaxCode1= 1 << ls->lsRunningBits;
 		_ClearHashTable( &(los->losHashTable) );
 		}
 	    else{
 		_InsertHashTable( &(los->losHashTable),
-					    NewKey, ls->lsRunningCode++ );
+					    nextKey, ls->lsNextCode );
+		ls->lsNextCode++;
 		}
 	   }
 	}
 
-    /*  7  */
-    ls->lsCurrentCode= currentCode;
-
     return count;
 }
 
@@ -748,14 +750,14 @@ static int sioOutLzwWriteBytes(	void *			voidlos,
 /*									*/
 /************************************************************************/
 
-SimpleOutputStream * sioOutLzwOpen(	SimpleOutputStream *	sosLzw,
+SimpleOutputStream * sioOutLzwGifOpen(	SimpleOutputStream *	sosLzw,
 					int			codeSize )
     {
     SimpleOutputStream *	sos;
-    LzwOutputStream *		los;
+    LzwGifOutputStream *	los;
 
     /*  1  */
-    los= (LzwOutputStream *)malloc( sizeof(LzwOutputStream) );
+    los= (LzwGifOutputStream *)malloc( sizeof(LzwGifOutputStream) );
     if  ( ! los )
 	{ XDEB(los); return (SimpleOutputStream *)0;	}
 
@@ -768,15 +770,369 @@ SimpleOutputStream * sioOutLzwOpen(	SimpleOutputStream *	sosLzw,
     sioLzwInitState( &(los->losState), codeSize );
 
     /*  4  */
-    sos= sioOutOpen( (void *)los, sioOutLzwWriteBytes,
-					    sioLzwSeek, sioOutLzwClose );
+    sos= sioOutOpen( (void *)los, sioOutLzwGifWriteBytes,
+					    sioLzwSeek, sioOutLzwGifClose );
 
     if  ( ! sos )
 	{ XDEB(sos); free( los ); return (SimpleOutputStream *)0; }
 
     /*  Z  */
-    if  ( sioOutLzwWriteCode( los, los->losState.lsClearCode ) )
+    if  ( sioOutLzwGifWriteCode( los, los->losState.lsClearCode ) )
 	{ LDEB(1); sioOutClose( sos ); return (SimpleOutputStream *)0; }
 
     return sos;
     }
+
+/************************************************************************/
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   L Z W E n c o d e I m a g e                                               %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  Copied from the ImageMagick source and adapted to fit into the conventions 
+%  of this file.
+%
+%  Method LZWEncodeImage compresses an image via LZW-coding specific to
+%  Postscript Level II or Portable Document Format. To ensure portability, the
+%  binary LZW bytes are usually encoded as ASCII base-85.
+%
+%  The format of the LZWEncodeImage method is:
+%
+%      unsigned int LZWEncodeImage(Image *image,const size_t length,
+%        unsigned char *pixels)
+%
+%  A description of each parameter follows:
+%
+%    o length:  A value that specifies the number of pixels to compress.
+%
+%    o pixels: The address of an unsigned array of characters containing the
+%      pixels to compress.
+%
+%
+*/
+
+typedef struct StringTable
+{
+short
+  prefix,
+  suffix,
+  next;
+} StringTable;
+
+/************************************************************************/
+/*									*/
+/*  Actual Output stream object.					*/
+/*									*/
+/************************************************************************/
+
+typedef struct LzwPsOutputStream
+    {
+    StringTable			losStringTable[4096];
+    LzwState			losState;
+    SimpleOutputStream *	losSosLzw;
+    } LzwPsOutputStream;
+
+/************************************************************************/
+/*									*/
+/*  String table to look for previously known strings.			*/
+/*									*/
+/*  Initially it contains all strings of length 1: The values in the	*/
+/*  original alphabet.							*/
+/*									*/
+/*  When a string is extended, we append a new item to the end of the	*/
+/*  table.								*/
+/*									*/
+/*  Every item has a 'next' pointer to somewhere else in the array.	*/
+/*  ( or -1 if there is no next. ) Every time a possible continuation	*/
+/*  after a string (=code) is stored in the table, it is inserted at	*/
+/*  the head of the linked list of 'next' pointers from this string.	*/
+/*									*/
+/*  In this way, the 'next' pointers are a linked list of continuations	*/
+/*  of the string that belongs to the current code. The head of the	*/
+/*  list is the 'next' of the current code.				*/
+/*									*/
+/*  TODO: Make the following true:					*/
+/*  Though the linked lists are longer than that, we can stop as soon	*/
+/*  as soon as the prefix no longer equals the current code: after	*/
+/*  that, we will no longer look at direct successors of the current	*/
+/*  code, but at indirect ones.						*/
+/*									*/
+/************************************************************************/
+
+static void sioLzwResetStringTable(	StringTable *	table )
+    {
+    int		i;
+
+    for ( i= 0; i < 256; i++ )
+	{
+	table[i].prefix= -1;
+	table[i].suffix= i;
+	table[i].next= -1;
+	}
+    }
+
+static int sioLzwFindNextInStringTable(	StringTable *	table,
+					int		currentCode,
+					int		val )
+    {
+    int		idx= table[currentCode].next;
+
+    while( idx >= 0 )
+	{
+	/*  Try later! Makes it a tree!
+	if  ( table[idx].prefix != currentCode )
+	    { break;	}
+	*/
+
+	if  ( table[idx].prefix == currentCode	&&
+	      table[idx].suffix == val		)
+	    { return idx;	}
+
+	idx= table[idx].next;
+	}
+
+    return -1;
+    }
+
+static void sioLzwAddNextToStringTable(	StringTable *	table,
+					int		currentCode,
+					int		nextCode,
+					int		val )
+    {
+    table[nextCode].prefix= currentCode;
+    table[nextCode].suffix= val;
+    table[nextCode].next= table[currentCode].next;
+    table[currentCode].next= nextCode;
+
+    return;
+    }
+
+/************************************************************************/
+/*									*/
+/*  The actual output routine to the (blocked) stream: 			*/
+/*  This routine is responsible for the translation of the bit stream	*/
+/*  into 8 bits (byte) packets.						*/
+/*									*/
+/*  1)  Emit full bytes.						*/
+/*									*/
+/************************************************************************/
+
+static int sioOutLzwPsWriteCode(	LzwPsOutputStream *	los,
+					int			code )
+    {
+    LzwState *			ls= &(los->losState);
+    int				rval= 0;
+    unsigned			shift;
+
+#   if 0
+    if  ( code >= ls->lsMaxCode1 )
+	{ LLDEB(code,ls->lsMaxCode1); }
+    if  ( code>=ls->lsNextCode )
+	{ LLDEB(code,ls->lsNextCode); }
+#   endif
+
+    shift= 32- ls->lsCurrentCodeSize- ls->lsBitsAccumulated;
+
+    ls->lsAccumulator	  |= ( (unsigned long) code ) << shift;
+    ls->lsBitsAccumulated += ls->lsCurrentCodeSize;
+
+    while( ls->lsBitsAccumulated >= 8 )
+	{
+        if  ( sioOutPutCharacter( ( ls->lsAccumulator  >> 24 ) & 0xff,
+							    los->losSosLzw ) )
+	    { rval= -1;	}
+
+        ls->lsAccumulator <<= 8;
+        ls->lsBitsAccumulated -= 8;
+	}
+
+    return rval;
+    }
+
+/************************************************************************/
+/*									*/
+/*  Close an LZW output stream.						*/
+/*									*/
+/*  1)  As the encoding process is one code behind, issue the last	*/
+/*	code.								*/
+/*  2)  Issue the EOF code.						*/
+/*  3)  Emit all bits that are heaped up in the encoder.		*/
+/*  4)  Free memory.							*/
+/*									*/
+/************************************************************************/
+
+static int sioOutLzwPsClose(	void *		voidlos )
+    {
+    int				rval= 0;
+    LzwPsOutputStream *		los= (LzwPsOutputStream *)voidlos;
+    LzwState *			ls= &(los->losState);
+
+    /*  1  */
+    if  ( sioOutLzwPsWriteCode( los, ls->lsCurrentCode ) )
+	{ XDEB(ls->lsCurrentCode); rval= -1;	}
+
+    /*  2  */
+    if  ( sioOutLzwPsWriteCode( los, ls->lsEOFCode ) )
+	{ XDEB(ls->lsEOFCode); rval= -1;	}
+
+    /*  3  */
+    /* while? */
+    if  ( ls->lsBitsAccumulated > 0 )
+	{
+	if  ( sioOutPutCharacter( ls->lsAccumulator  >> 24,
+						    los->losSosLzw ) == EOF )
+	    { rval= -1;	}
+
+        ls->lsAccumulator <<= 8;
+        ls->lsBitsAccumulated -= 8;
+	}
+
+    /*  4  */
+    free( los );
+
+    return rval;
+    }
+
+/************************************************************************/
+/*									*/
+/*  The LZ compression routine:						*/
+/*  This version compress the given buffer of length count.		*/
+/*  This routine can be called several times (one per scan line, for	*/
+/*  example), in order to compress the whole image			*/
+/*									*/
+/*  1)  If the Current code is the initial value that serves to		*/
+/*	flag this, set the current code to the first character to	*/
+/*	compress. (and consume it)					*/
+/*  2)  Make a new key to search the hash table.			*/
+/*  3)  Lookup in hash table.						*/
+/*  4)  If it is there, simply take the new code as the current code.	*/
+/*  5)  If not, emit the current code and set the current code to the	*/
+/*	pixel value.							*/
+/*      If next code cannot fit into the current code size, must raise	*/
+/*	the code size.							*/
+/*  6)  BUT.. If the hash table is full, send a Clear code and clear	*/
+/*	the hash table as well.						*/
+/*									*/
+/************************************************************************/
+
+static int sioOutLzwPsWriteBytes(	void *			voidlos,
+					const unsigned char *	buffer,
+					int			count )
+    {
+    LzwPsOutputStream *		los= (LzwPsOutputStream *)voidlos;
+    LzwState *			ls= &(los->losState);
+
+    int				i= 0;
+
+    /*  1  */
+    if  ( ls->lsCurrentCode == FIRST_CODE )
+	{
+	ls->lsCurrentCode=
+		    buffer[i++] & SIO_LZW_OutMasks[ls->lsInitialCodeSize];
+	}
+
+    while( i < count )
+	{
+	int	val= buffer[i++] & SIO_LZW_OutMasks[ls->lsInitialCodeSize];
+	int	nextCode;
+
+	nextCode= sioLzwFindNextInStringTable( los->losStringTable,
+						    ls->lsCurrentCode, val );
+
+	if  ( nextCode >= 0 )
+	    { ls->lsCurrentCode= nextCode;	}
+	else{
+	    int		currentCode;
+
+	    sioOutLzwPsWriteCode( los, ls->lsCurrentCode );
+
+	    currentCode= ls->lsCurrentCode;
+	    ls->lsCurrentCode= val;
+
+	    if  ( ls->lsNextCode >= ls->lsMaxCode1 )
+		{ ls->lsMaxCode1= 1 << ++(ls->lsCurrentCodeSize);	}
+
+	    if  ( ls->lsCurrentCodeSize > 12 )
+		{
+		ls->lsCurrentCodeSize= 12;
+
+		if  ( sioOutLzwPsWriteCode( los, ls->lsClearCode ) )
+		    { LDEB(ls->lsCurrentCode); return -1;	}
+
+		ls->lsNextCode= ls->lsEOFCode+ 1;
+		ls->lsCurrentCodeSize= ls->lsInitialCodeSize+ 1;
+		ls->lsMaxCode1= 1 << ls->lsCurrentCodeSize;
+
+		sioLzwResetStringTable( los->losStringTable );
+		}
+	    else{
+		sioLzwAddNextToStringTable( los->losStringTable,
+					currentCode, ls->lsNextCode, val );
+
+		ls->lsNextCode++;
+		}
+	    }
+	}
+
+    return count;
+    }
+
+/************************************************************************/
+/*									*/
+/*  Open a 12 bit LZW output stream.					*/
+/*									*/
+/*  NOTE that the initial code size has been communicated outside the	*/
+/*	compressed stream. In the GIF case even before the packeted	*/
+/*	stream packet header of the first packet.			*/
+/*									*/
+/*  1)  Allocate memory and remember the output stream.			*/
+/*  2)  Initialize the hash table by clearing it.			*/
+/*  3)  Initialize the general LZW administration for the current code	*/
+/*	size.								*/
+/*  4)  General SioOut administration.					*/
+/*  Z)  Start with a clear code to be sure that the decoder has also	*/
+/*	cleared its hash table.						*/
+/*									*/
+/************************************************************************/
+
+SimpleOutputStream * sioOutLzwPsOpen(	SimpleOutputStream *	sosLzw )
+    {
+    SimpleOutputStream *	sos;
+    LzwPsOutputStream *		los;
+
+    const int			codeSize= 8;
+
+    /*  1  */
+    los= (LzwPsOutputStream *)malloc( sizeof(LzwPsOutputStream) );
+    if  ( ! los )
+	{ XDEB(los); return (SimpleOutputStream *)0;	}
+
+    los->losSosLzw= sosLzw;
+
+    /*  2  */
+    sioLzwResetStringTable( los->losStringTable );
+
+    /*  3  */
+    sioLzwInitState( &(los->losState), codeSize );
+
+    /*  4  */
+    sos= sioOutOpen( (void *)los, sioOutLzwPsWriteBytes,
+					    sioLzwSeek, sioOutLzwPsClose );
+
+    if  ( ! sos )
+	{ XDEB(sos); free( los ); return (SimpleOutputStream *)0; }
+
+    /*  Z  */
+    if  ( sioOutLzwPsWriteCode( los, los->losState.lsClearCode ) )
+	{ LDEB(1); sioOutClose( sos ); return (SimpleOutputStream *)0; }
+
+    return sos;
+    }
+
