@@ -41,7 +41,6 @@ extern const char *	bmcoStrings[];
 
 #   define	x_POINTS_PER_CM		28.3465
 #   define	POINTS_PER_M		2834.65
-#   define	x_TWIPS_PER_CM		(20*POINTS_PER_CM)
 #   define	TWIPS_PER_M		(20*POINTS_PER_M)
 
 extern const char *	bmunIntToString( int unitInt );
@@ -169,6 +168,31 @@ typedef struct	BitmapSelection
     int		bsPixelsHigh;
     } BitmapSelection;
 
+/************************************************************************/
+/*									*/
+/*  Context for writing images to PostScript.				*/
+/*									*/
+/************************************************************************/
+
+typedef struct BitmapPrinter
+    {
+    int				bpUseFilters;
+    int				bpIndexedImages;
+    SimpleOutputStream *	bpOutput;
+    SimpleOutputStream *	bpHexed;
+    SimpleOutputStream *	bpBase85;
+    SimpleOutputStream *	bpFlate;
+    } BitmapPrinter;
+
+extern int bmPsOpenBitmapPrinter(
+				BitmapPrinter *			bp,
+				SimpleOutputStream *		sos,
+				const BitmapDescription *	bd,
+				int				useFilters,
+				int				indexedImages );
+
+extern void bmCloseBitmapPrinter(	BitmapPrinter *		bp );
+
 /************************************************************************/
 /*									*/
 /*  Catalog of available of a bitmap file formats.			*/
@@ -213,7 +237,7 @@ extern int bmCanWriteJpegFile(	const BitmapDescription *	bd,
 /*									*/
 /************************************************************************/
 
-extern int bmPsPrintBitmap(	FILE *				f,
+extern int bmPsPrintBitmap(	SimpleOutputStream *		sos,
 				int				level,
 				double				xscale,
 				double				yscale,
@@ -223,10 +247,12 @@ extern int bmPsPrintBitmap(	FILE *				f,
 				int				y0,
 				int				wide,
 				int				high,
+				int				useFilters,
+				int				indexedImages,
 				const BitmapDescription *	bd,
-				const unsigned char *		buffer	);
+				const unsigned char *		buffer );
 
-extern int bmPsPrintBitmapImage( FILE *				f,
+extern int bmPsPrintBitmapImage( SimpleOutputStream *		sos,
 				int				level,
 				double				xscale,
 				double				yscale,
@@ -236,25 +262,30 @@ extern int bmPsPrintBitmapImage( FILE *				f,
 				int				y0,
 				int				wide,
 				int				high,
+				int				useFilters,
+				int				indexedImages,
 				const BitmapDescription *	bd,
-				const unsigned char *		buffer	);
+				const unsigned char *		buffer );
 
-extern void bmPsWriteImageInstructions(	FILE *				f,
-					const BitmapDescription *	bd,
-					int				wide,
-					int				high,
-					const char *			proc );
+extern void bmPsWriteImageInstructions(
+				SimpleOutputStream *		sos,
+				const BitmapDescription *	bd,
+				int				wide,
+				int				high,
+				const char *			source,
+				int				indexedImages );
 
-extern int bmPsWriteBitmapData(	    FILE *			f,
-				    int				x0,
-				    int				y0,
-				    int				wide,
-				    int				high,
-				    const BitmapDescription *	bd,
-				    const unsigned char *	inputBuffer );
+extern int bmPsWriteBitmapData(	BitmapPrinter *			bp,
+				int				x0,
+				int				y0,
+				int				wide,
+				int				high,
+				const BitmapDescription *	bd,
+				const unsigned char *		inputBuffer );
 
 extern int bmPsRowStringSize(	const BitmapDescription *	bd,
-				int				pixelsWide );
+				int				pixelsWide,
+				int				indexedImages );
 
 extern int bmPngWritePng(	const BitmapDescription *	bd,
 				const unsigned char *		buffer,
@@ -316,35 +347,35 @@ extern void bmCleanDescription(	BitmapDescription *	bd	);
 extern int bmCopyDescription(	BitmapDescription *		bdOut,
 				const BitmapDescription *	bdIn	);
 
-extern int bmWrite(	const char *		filename,
-			const unsigned char *	buffer,
-			BitmapDescription *	bd,
-			int			fileFormat,
-			double			compressionFactor	);
+extern int bmWrite(	const char *			filename,
+			const unsigned char *		buffer,
+			const BitmapDescription *	bd,
+			int				fileFormat,
+			double				compressionFactor );
 
-extern int bmCanWrite(	BitmapDescription *	bd,
-			int			fileFormat,
-			double			compressionFactor	);
+extern int bmCanWrite(	const BitmapDescription *	bd,
+			int				fileFormat,
+			double				compressionFactor );
 
 extern int bmRead(	const char *		filename,
 			unsigned char **	pBuffer,
 			BitmapDescription *	bd,
 			int *			pFileFormat,
-			double *		pCompressionFactor	);
+			double *		pCompressionFactor );
 
-extern int bmSelect(	unsigned char **		pBuffer,
-			BitmapDescription *		bdOut,
-			const unsigned char *		inputBuffer,
+extern int bmSelect(	BitmapDescription *		bdOut,
+			unsigned char **		pBufOut,
 			const BitmapDescription *	bdIn,
+			const unsigned char *		bufIn,
 			int				x0,
 			int				y0,
-			int				xs,
-			int				ys		);
+			int				wide,
+			int				high );
 
 extern int bmComponents( void ***		pComponents,
 			int *			pCount,
 			const unsigned char *	buffer,
-			const BitmapDescription * bd			);
+			const BitmapDescription * bd );
 
 extern int bmComponentBitmap( unsigned char **		buffer,
 			BitmapDescription *		bdout,
@@ -373,7 +404,9 @@ extern void bmFreeGroups( void **			groups,
 			int				numberOfGroups	);
 
 /************************************************************************/
+/*									*/
 /*  Palette expansion for drawing.					*/
+/*									*/
 /************************************************************************/
 
 int bmExpandRGB8Palette(	unsigned char *		to,
@@ -384,9 +417,21 @@ int bmExpandRGB8Palette(	unsigned char *		to,
 				int			hasAlpha );
 
 /************************************************************************/
+/*									*/
 /*  Bitmap transformations.						*/
+/*									*/
 /************************************************************************/
 
+typedef void (*BM_PIX_FUN)(	void *				through,
+				int				row,
+				int				col );
+
+typedef void (*BM_SEG_FUN)(	void *				through,
+				int				x0,
+				int				y0,
+				int				x1,
+				int				y1 );
+
 typedef int (*BitmapTransformation)(BitmapDescription *		bdOut,
 				const BitmapDescription *	bdIn,
 				unsigned char **		pBufOut,
@@ -429,12 +474,6 @@ extern int bmColorReduce(	BitmapDescription *		bdOut,
 				const unsigned char *		bufIn,
 				int				colorCount );
 
-extern int bmBlur(	BitmapDescription *		bdOut,
-			const BitmapDescription *	bdIn,
-			unsigned char **		pBufOut,
-			const unsigned char *		bufIn,
-			int				colorCount );
-
 extern int bmRotate(	BitmapDescription *		bdOut,
 			const BitmapDescription *	bdIn,
 			unsigned char **		pBufOut,
@@ -459,6 +498,18 @@ extern int bmFilterSmoothe(	BitmapDescription *		bdOut,
 				const unsigned char *		bufIn,
 				int				ignoredInt );
 
+extern int bmFilterBlur(	BitmapDescription *		bdOut,
+				const BitmapDescription *	bdIn,
+				unsigned char **		pBufOut,
+				const unsigned char *		bufIn,
+				int				ignoredInt );
+
+extern int bmFilterRelative(	BitmapDescription *		bdOut,
+				const BitmapDescription *	bdIn,
+				unsigned char **		pBufOut,
+				const unsigned char *		bufIn,
+				int				ignoredInt );
+
 extern int bmFlipBytes(		unsigned char *	buffer,
 				int		byteCount,
 				int		bitsPerPixel );
@@ -476,15 +527,17 @@ extern int bmAverage(		BitmapDescription *		bdOut,
 extern int bmExpandPaletteImage( BitmapDescription *		bdOut,
 				const BitmapDescription *	bdIn,
 				unsigned char **		pBufOut,
-				const unsigned char *		bufIn );
+				const unsigned char *		bufIn,
+				int				ignoredInt );
 
 extern int bmFillImage(		ColorAllocator *		ca,
 				int				bitmapUnit,
 				int				swapBitmapBytes,
 				int				swapBitmapBits,
+				int				dither,
 				unsigned char *			bufferOut,
 				const unsigned char *		bufferIn,
-				BitmapDescription *		bdOut,
+				const BitmapDescription *	bdOut,
 				const BitmapDescription *	bdIn );
 
 extern int bmTransparentImage(	BitmapDescription *		bdOut,
@@ -493,6 +546,15 @@ extern int bmTransparentImage(	BitmapDescription *		bdOut,
 				int				wide,
 				int				high );
 
+extern int bmRGBImage(		BitmapDescription *		bdOut,
+				unsigned char **		pBufOut,
+				int				colorEncoding,
+				int				r,
+				int				g,
+				int				b,
+				int				wide,
+				int				high );
+
 extern int bmGammaCorrect(	BitmapDescription *		bdOut,
 				const BitmapDescription *	bdIn,
 				unsigned char **		pBufOut,
@@ -502,11 +564,138 @@ extern int bmGammaCorrect(	BitmapDescription *		bdOut,
 extern int bmInvertImage(	BitmapDescription *		bdOut,
 				const BitmapDescription *	bdIn,
 				unsigned char **		pBufOut,
-				const unsigned char *		bufIn );
+				const unsigned char *		bufIn,
+				int				ignoredInt );
 
 extern void bmImageSizeTwips(
 			int *				pImageWideTwips,
 			int *				pImageHighTwips,
 			const BitmapDescription *	bd );
 
+extern int bmCalculateSizes(	BitmapDescription *	bd );
+
+extern int bmToGrayscale(
+			BitmapDescription *		bdOut,
+			const BitmapDescription *	bdIn,
+			unsigned char **		pBufOut,
+			const unsigned char *		bufferIn,
+			int				ignoredInt );
+
+extern int bmThreshold(	BitmapDescription *		bdOut,
+			const BitmapDescription *	bdIn,
+			unsigned char **		pBufOut,
+			const unsigned char *		bufferIn,
+			int				ignoredInt );
+
+extern int bmMorphoDilateSimple(
+			BitmapDescription *		bdOut,
+			const BitmapDescription *	bdIn,
+			unsigned char **		pBufOut,
+			const unsigned char *		bufferIn,
+			int				ignoredInt );
+
+extern int bmMorphoErodeSimple(
+			BitmapDescription *		bdOut,
+			const BitmapDescription *	bdIn,
+			unsigned char **		pBufOut,
+			const unsigned char *		bufferIn,
+			int				ignoredInt );
+
+extern int bmMorphoDilate(	BitmapDescription *		bdOut,
+				const BitmapDescription *	bdIn,
+				unsigned char **		pBufOut,
+				const unsigned char *		bufIn,
+				const BitmapDescription *	bdSe,
+				const unsigned char *		bufSe,
+				int				rowOrig,
+				int				colOrig );
+
+extern int bmMorphoErode(	BitmapDescription *		bdOut,
+				const BitmapDescription *	bdIn,
+				unsigned char **		pBufOut,
+				const unsigned char *		bufIn,
+				const BitmapDescription *	bdSe,
+				const unsigned char *		bufSe,
+				int				rowOrig,
+				int				colOrig );
+
+extern int bmMorphoSetSimpleSe(	BitmapDescription *		bdOut,
+				unsigned char **		pBufOut );
+
+extern int bmMorphoLineElement(	BitmapDescription *		bdOut,
+				unsigned char **		pBufOut,
+				int				wide,
+				int				high,
+				int				x0,
+				int				y0,
+				int				x1,
+				int				y1 );
+
+extern void bmDraw1BitImage(	const BitmapDescription *	bdTo,
+				unsigned char *			bufTo,
+				const BitmapDescription *	bdIn,
+				const unsigned char *		bufIn,
+				unsigned char			invertMask,
+				int				rowTo0,
+				int				colTo0 );
+
+extern void bmForAll1Pixels(	const BitmapDescription *	bdIn,
+				const unsigned char *		bufIn,
+				unsigned char			invertMaskIn,
+				void *				through,
+				BM_PIX_FUN			pixFun );
+
+extern int bmFindLineSegments(	const BitmapDescription *	bdIn,
+				const unsigned char *		bufIn,
+				int				size,
+				void *				through,
+				BM_SEG_FUN			segFun );
+
+extern int bmTextureMap(	ColorAllocator *		ca,
+				int				swapBitmapUnit,
+				int				swapBitmapBytes,
+				int				swapBitmapBits,
+				int				dither,
+				unsigned char *			bufferOut,
+				const unsigned char *		bufferIn,
+				const BitmapDescription *	bdOut,
+				const BitmapDescription *	bdIn,
+				double				E_x,
+				double				E_y,
+				double				E_z,
+				double				P_x,
+				double				P_y,
+				double				P_z,
+				double				N_x,
+				double				N_y,
+				double				N_z,
+				double				M_x,
+				double				M_y,
+				double				M_z );
+
+extern int bmTextureMapInverse(	ColorAllocator *		ca,
+				int				swapBitmapUnit,
+				int				swapBitmapBytes,
+				int				swapBitmapBits,
+				int				dither,
+				unsigned char *			bufferOut,
+				const unsigned char *		bufferIn,
+				const BitmapDescription *	bdOut,
+				const BitmapDescription *	bdIn,
+				double				E_x,
+				double				E_y,
+				double				E_z,
+				double				P_x,
+				double				P_y,
+				double				P_z,
+				double				N_x,
+				double				N_y,
+				double				N_z,
+				double				M_x,
+				double				M_y,
+				double				M_z );
+
+extern int bmSetColorAllocatorForImage(	ColorAllocator *		ca,
+					const BitmapDescription *	bd );
+
 #   endif
