@@ -1,4 +1,4 @@
-#   include	"config.h"
+#   include	"bitmapConfig.h"
 
 #   include	<string.h>
 #   include	"bmSegments.h"
@@ -1031,11 +1031,11 @@ int bmcDrawComponent(	const BitmapSegment *	bs,
     bmDrawLine( buffer,
 	bs->bsX0- col0, row0,
 	bs->bsX1- col0, row0+ bs->bsY1- bs->bsY0,
-	2, bytesPerRow, colorEncoding );
+	2, bytesPerRow );
     bmDrawLine( buffer,
 	bs->bsX0- col0, row0+ bs->bsY1- bs->bsY0,
 	bs->bsX1- col0, row0,
-	2, bytesPerRow, colorEncoding );
+	2, bytesPerRow );
 
     return 0;
 #   endif
@@ -1051,7 +1051,7 @@ int bmcDrawComponent(	const BitmapSegment *	bs,
 	    bmFillBlock( buffer,
 			    dr->drX0- col0, row,
 			    dr->drX1- col0+ 1, row+ dr->drRepeatCount,
-			    bytesPerRow, colorEncoding );
+			    bytesPerRow );
 
 	    row += dr->drRepeatCount;
 	    }
