@@ -1,4 +1,4 @@
-#   include	"config.h"
+#   include	"bitmapConfig.h"
 
 #   include	"bmintern.h"
 #   include	<string.h>
@@ -45,9 +45,10 @@ int		bmcoStringToInt( const char * colorEncodingString )
 
 const char *	bmunStrings[]=
     {
-    "cm",
+    "m",
     "inch",
     "point",
+    "pixel",
     0
     };
 
@@ -74,8 +75,11 @@ int		bmunStringToInt( const char * unitString )
     }
 
 /************************************************************************/
+/*									*/
 /*  Initialise a BitmapDescription.					*/
+/*									*/
 /************************************************************************/
+
 void bmInitDescription	( BitmapDescription *	bd )
     {
     bd->bdBufferLength= 0;
@@ -100,8 +104,11 @@ void bmInitDescription	( BitmapDescription *	bd )
     }
 
 /************************************************************************/
+/*									*/
 /*  Clean a BitmapDescription.						*/
+/*									*/
 /************************************************************************/
+
 void bmCleanDescription	( BitmapDescription *	bd )
     {
     switch( bd->bdColorEncoding )
@@ -117,8 +124,11 @@ void bmCleanDescription	( BitmapDescription *	bd )
 
 
 /************************************************************************/
+/*									*/
 /*  Copy a BitmapDescription.						*/
+/*									*/
 /************************************************************************/
+
 int bmCopyDescription	(	BitmapDescription *		to,
 				const BitmapDescription *	from )
     {
@@ -142,11 +152,113 @@ int bmCopyDescription	(	BitmapDescription *		to,
     }
 
 /************************************************************************/
+/*									*/
 /*  Byte masks								*/
+/*									*/
 /************************************************************************/
+
 unsigned char	Bmc1Masks[8]=
     { 0x80, 0x40, 0x20, 0x10, 0x8, 0x4, 0x2, 0x1 };
 
 unsigned char	Bmc7Masks[8]=
     { 0x7f, 0xbf, 0xdf, 0xef, 0xf7, 0xfb, 0xfd, 0xfe };
 
+/************************************************************************/
+/*									*/
+/*  Geometry.								*/
+/*									*/
+/************************************************************************/
+
+void bmImageSizeTwips(	int *				pImageWideTwips,
+			int *				pImageHighTwips,
+			const BitmapDescription *	bd )
+    {
+    int		imageWideTwips;
+    int		imageHighTwips;
+
+    switch( bd->bdUnit )
+	{
+	case BMunM:
+	    imageWideTwips= (int) ( ( TWIPS_PER_M* bd->bdPixelsWide )/
+							bd->bdXResolution );
+	    imageHighTwips= (int) ( ( TWIPS_PER_M* bd->bdPixelsHigh )/
+							bd->bdYResolution );
+	    break;
+
+	case BMunINCH:
+	    imageWideTwips= (int)( ( 72* 20* bd->bdPixelsWide )/
+							bd->bdXResolution );
+	    imageHighTwips= (int)( ( 72* 20* bd->bdPixelsHigh )/
+							bd->bdYResolution );
+	    break;
+
+	case BMunPOINT:
+	    imageWideTwips= (int)( ( 20* bd->bdPixelsWide )/
+							bd->bdXResolution );
+	    imageHighTwips= (int)( ( 20* bd->bdPixelsHigh )/
+							bd->bdYResolution );
+	    break;
+
+	case BMunPIXEL:
+	    imageWideTwips= 20* bd->bdPixelsWide;
+	    imageHighTwips= 20* bd->bdPixelsHigh;
+	    break;
+
+	default:
+	    LDEB(bd->bdUnit);
+	    imageWideTwips= 20* bd->bdPixelsWide;
+	    imageHighTwips= 20* bd->bdPixelsHigh;
+	    break;
+	}
+
+    *pImageWideTwips= imageWideTwips;
+    *pImageHighTwips= imageHighTwips;
+
+    return;
+    }
+
+/************************************************************************/
+/*									*/
+/*  Derive the sizes of a description from the other fileds.		*/
+/*									*/
+/************************************************************************/
+
+int bmCalculateSizes(	BitmapDescription *	bd )
+    {
+    switch( bd->bdColorEncoding )
+	{
+	case BMcoBLACKWHITE:
+	case BMcoWHITEBLACK:
+	    if  ( bd->bdHasAlpha )
+		{ bd->bdSamplesPerPixel= 2;	}
+	    else{ bd->bdSamplesPerPixel= 1;	}
+
+	    bd->bdBitsPerPixel= bd->bdSamplesPerPixel* bd->bdBitsPerSample;
+	    break;
+
+	case BMcoRGB:
+	    if  ( bd->bdHasAlpha )
+		{ bd->bdSamplesPerPixel= 4;	}
+	    else{ bd->bdSamplesPerPixel= 3;	}
+
+	    bd->bdBitsPerPixel= bd->bdSamplesPerPixel* bd->bdBitsPerSample;
+	    break;
+
+	case BMcoRGB8PALETTE:
+	    if  ( bd->bdHasAlpha )
+		{ bd->bdSamplesPerPixel= 4;	}
+	    else{ bd->bdSamplesPerPixel= 3;	}
+
+	    break;
+
+	default:
+	    LDEB(bd->bdColorEncoding);
+	    return -1;
+	}
+
+    bd->bdBytesPerRow= ( bd->bdBitsPerPixel* bd->bdPixelsWide+ 7 )/ 8;
+    bd->bdBufferLength= bd->bdPixelsHigh* bd->bdBytesPerRow;
+
+    return 0;
+    }
+
