@@ -1,4 +1,4 @@
-#   include	"config.h"
+#   include	"bitmapConfig.h"
 
 #   include	"bmintern.h"
 
@@ -17,8 +17,11 @@
 #   endif
 
 /************************************************************************/
+/*									*/
 /*  Flip a scan line consisting of 1 bit pixels				*/
+/*									*/
 /************************************************************************/
+
 static unsigned char	bmFlipLeft[256];
 static unsigned char	bmFlipRight[256];
 
@@ -91,20 +94,21 @@ int bmFlipBytes(		unsigned char *	buffer,
     switch( bitsPerPixel )
 	{
 	case  1:
-	    bmFlip1Init( 8/bitsPerPixel ); break;
 	case  2:
-	    bmFlip2Init( 8/bitsPerPixel ); break;
 	case  4:
-	    bmFlip4Init( 8/bitsPerPixel ); break;
+	    bmFlip1Init( 8/bitsPerPixel );
+	    break;
+
 	case  8:
 	    return 0;
+
 	case 24:
 	default:
 	    LDEB(bitsPerPixel); return -1;
 	}
 
     while( byteCount-- > 0 )
-	{ *(buffer++)= bmFlipRight[*buffer];	}
+	{ *buffer= bmFlipRight[*buffer]; buffer++;	}
 
     return 0;
     }
@@ -119,7 +123,7 @@ int bmFlipBytes(		unsigned char *	buffer,
 static void bmFlipBits(	unsigned char *		to,
 			const unsigned char *	from,
 			int			pixelsWide,
-			int			pixelsPerByte	)
+			int			pixelsPerByte )
     {
     int			i;
     int			count;
@@ -145,8 +149,11 @@ static void bmFlipBits(	unsigned char *		to,
     }
 
 /************************************************************************/
+/*									*/
 /*  Flip a scan line consisting of 8 bit pixels				*/
+/*									*/
 /************************************************************************/
+
 static void bmFlip8Bits(	unsigned char *		to,
 				const unsigned char *	from,
 				int			pixelsWide,
@@ -161,8 +168,11 @@ static void bmFlip8Bits(	unsigned char *		to,
     }
 
 /************************************************************************/
+/*									*/
 /*  Flip a scan line consisting of 24 bit pixels			*/
+/*									*/
 /************************************************************************/
+
 static void bmFlip24Bits(	unsigned char *		to,
 				const unsigned char *	from,
 				int			pixelsWide,
@@ -180,13 +190,16 @@ static void bmFlip24Bits(	unsigned char *		to,
     }
 
 /************************************************************************/
+/*									*/
 /*  Mirror bitmap images: Flip over horizontal axis.			*/
+/*									*/
 /************************************************************************/
+
 int bmVerticalFlip(	BitmapDescription *		bdOut,
 			const BitmapDescription *	bdIn,
 			unsigned char **		pBufOut,
 			const unsigned char *		bufIn,
-			int				ignoredInt	)
+			int				ignoredInt )
     {
     const unsigned char *	from;
     unsigned char *		to;
@@ -211,13 +224,16 @@ int bmVerticalFlip(	BitmapDescription *		bdOut,
     }
 
 /************************************************************************/
+/*									*/
 /*  Mirror bitmap images: Rotate 180 degrees.				*/
+/*									*/
 /************************************************************************/
+
 int bmUpsideDown(	BitmapDescription *		bdOut,
 			const BitmapDescription *	bdIn,
 			unsigned char **		pBufOut,
 			const unsigned char *		bufIn,
-			int				ignoredInt	)
+			int				ignoredInt )
     {
     const unsigned char *	from;
     unsigned char *		to;
@@ -261,13 +277,16 @@ int bmUpsideDown(	BitmapDescription *		bdOut,
     }
 
 /************************************************************************/
+/*									*/
 /*  Mirror bitmap images: Flip over vertical axis.			*/
+/*									*/
 /************************************************************************/
+
 int bmHorizontalFlip(	BitmapDescription *		bdOut,
 			const BitmapDescription *	bdIn,
 			unsigned char **		pBufOut,
 			const unsigned char *		bufIn,
-			int				ignoredInt	)
+			int				ignoredInt )
     {
     const unsigned char *	from;
     unsigned char *		to;
@@ -310,13 +329,16 @@ int bmHorizontalFlip(	BitmapDescription *		bdOut,
     }
 
 /************************************************************************/
+/*									*/
 /*  Rotate a bitmap by 0,90,180,270 degrees.				*/
+/*									*/
 /************************************************************************/
+
 int bmRotate90(	BitmapDescription *		bdOut,
 		const BitmapDescription *	bdIn,
 		unsigned char **		pBufOut,
 		const unsigned char *		bufIn,
-		int				angle	)
+		int				angle )
     {
     const unsigned char *	from;
     unsigned char *		to;
@@ -425,10 +447,13 @@ int bmRotate90(	BitmapDescription *		bdOut,
 
 		    for ( s= 0; s < row % step; s++ )
 			{
+			int ri= bdIn->bdPixelsHigh- 1;
+
 			to= bufOut+ row* bd.bdBytesPerRow;
 			from= bufIn+ ( bdIn->bdPixelsHigh- 1 )*
 							bdIn->bdBytesPerRow;
 			from -= s* bdIn->bdBytesPerRow;
+			ri -= s;
 			from += row/ step;
 
 			shift= row % step- s;
@@ -436,17 +461,24 @@ int bmRotate90(	BitmapDescription *		bdOut,
 
 			for ( col= 0; col < bd.bdPixelsWide; col += step )
 			    {
+			    if  ( ri < 0 )
+				{ break;	}
+
 			    *(to++) |= ( *from & mr ) << shift;
 			    from -= step* bdIn->bdBytesPerRow;
+			    ri -= step;
 			    }
 			}
 
 		    for ( s= row % step; s < step; s++ )
 			{
+			int ri= bdIn->bdPixelsHigh- 1;
+
 			to= bufOut+ row* bd.bdBytesPerRow;
 			from= bufIn+ ( bdIn->bdPixelsHigh- 1 )*
 							bdIn->bdBytesPerRow;
 			from -= s* bdIn->bdBytesPerRow;
+			ri -= s;
 			from += row/ step;
 
 			shift= s- row % step;
@@ -454,8 +486,12 @@ int bmRotate90(	BitmapDescription *		bdOut,
 
 			for ( col= 0; col < bd.bdPixelsWide; col += step )
 			    {
+			    if  ( ri < 0 )
+				{ break;	}
+
 			    *(to++) |= ( *from & mr ) >> shift;
 			    from -= step* bdIn->bdBytesPerRow;
+			    ri -= step;
 			    }
 			}
 		    }
@@ -525,8 +561,11 @@ int bmRotate90(	BitmapDescription *		bdOut,
     }
 
 /************************************************************************/
+/*									*/
 /*  Rotate a 1, 2, 4 bit scan line over a sharp angle.			*/
+/*									*/
 /************************************************************************/
+
 static void bmRotateScanline124( const unsigned char *	from,
 				int			pixelsWide,
 				unsigned char *		bufOut,
@@ -584,8 +623,11 @@ static void bmRotateScanline124( const unsigned char *	from,
     }
 
 /************************************************************************/
+/*									*/
 /*  Rotate an 8 bit scan line over a sharp angle.			*/
+/*									*/
 /************************************************************************/
+
 static void bmRotateScanline8(	const unsigned char *	from,
 				int			pixelsWide,
 				unsigned char *		bufOut,
@@ -621,8 +663,11 @@ static void bmRotateScanline8(	const unsigned char *	from,
     }
 
 /************************************************************************/
+/*									*/
 /*  Rotate a 24 bit scan line over a sharp angle.			*/
+/*									*/
 /************************************************************************/
+
 static void bmRotateScanline24(	const unsigned char *	from,
 				int			pixelsWide,
 				unsigned char *		bufOut,
@@ -634,7 +679,7 @@ static void bmRotateScanline24(	const unsigned char *	from,
 				int			xRow,
 				int			xCol,
 				int			sRow,
-				int			d		)
+				int			d )
     {
     int		fcol;
     int		tbyte= tcol* 3;
@@ -665,14 +710,18 @@ static void bmRotateScanline24(	const unsigned char *	from,
     }
 
 /************************************************************************/
+/*									*/
 /*  Rotate a bitmap over an arbitrary angle.				*/
+/*									*/
 /*  1)  Sanity of the input.						*/
+/*									*/
 /************************************************************************/
+
 int bmRotate(	BitmapDescription *		bdOut,
 		const BitmapDescription *	bdIn,
 		unsigned char **		pBufOut,
 		const unsigned char *		bufIn,
-		double				angle	)
+		double				angle )
     {
     int				rotations;
 
