@@ -1,4 +1,4 @@
-#   include	"config.h"
+#   include	"bitmapConfig.h"
 
 #   include	<stdlib.h>
 #   include	<stdio.h>
@@ -17,6 +17,8 @@
 
 #   include	"bm_gif_lib.h"
 
+int _GifError = 0;
+
 /************************************************************************/
 /*									*/
 /*  Read a GIF file.							*/
@@ -28,6 +30,7 @@ int bmGifReadGif(		BitmapDescription *	bd,
 				SimpleInputStream *	sis )
     {
     GifFileType *	gft;
+    GifImageDesc *	gid;
     int			transparent= -1;
     int			gotImage= 0;
 
@@ -37,13 +40,20 @@ int bmGifReadGif(		BitmapDescription *	bd,
     if  ( ! gft )
 	{ XDEB(gft); return -1;	}
 
+    gid= &(gft->gftCurrentImageDescriptor);
+
     for (;;)
 	{
 	GifRecordType		grt;
 	unsigned int		row;
 	int			bpr;
 	int			wide;
-	ColorMapObject *	cmap;
+
+	int			cc;
+
+	GifColorMap *		gcm;
+	const RGB8Color *	rgb8From;
+	RGB8Color *		rgb8To;
 
 	if  ( DGifGetRecordType( gft, &grt ) != GIF_OK )
 	    { LDEB(1); DGifCloseFile( gft ); return -1;	}
@@ -52,8 +62,10 @@ int bmGifReadGif(		BitmapDescription *	bd,
 	    {
 	    case UNDEFINED_RECORD_TYPE:
 		LDEB(grt); continue;
+
 	    case SCREEN_DESC_RECORD_TYPE:
 		LDEB(grt); continue;
+
 	    case IMAGE_DESC_RECORD_TYPE:
 		if  ( gotImage )
 		    { LDEB(gotImage); break;	}
@@ -61,8 +73,8 @@ int bmGifReadGif(		BitmapDescription *	bd,
 
 		if  ( DGifGetImageDesc( gft ) != GIF_OK )
 		    { LDEB(1); DGifCloseFile( gft ); return -1; }
-		bd->bdPixelsWide= gft->Image.Width;
-		bd->bdPixelsHigh= gft->Image.Height;
+		bd->bdPixelsWide= gid->Width;
+		bd->bdPixelsHigh= gid->Height;
 		if  ( transparent >= 0 )
 		    { bd->bdHasAlpha= 1;	}
 		else{ bd->bdHasAlpha= 0;	}
@@ -71,21 +83,15 @@ int bmGifReadGif(		BitmapDescription *	bd,
 		bd->bdYResolution= 72;
 
 		bd->bdBitsPerSample= 8;
-		bd->bdSamplesPerPixel= 3;
-		bd->bdBitsPerPixel= 8;
+		if  ( bd->bdHasAlpha )
+		    { bd->bdBitsPerPixel= 16;	}
+		else{ bd->bdBitsPerPixel= 8;	}
 		bd->bdColorEncoding= BMcoRGB8PALETTE;
 		bd->bdColorCount= 256;
 		bd->bdRGB8Palette= (RGB8Color *)0;
-		bd->bdBytesPerRow= bd->bdPixelsWide;
-		bd->bdBufferLength= bd->bdPixelsHigh* bd->bdBytesPerRow;
 
-		if  ( bd->bdHasAlpha )
-		    {
-		    bd->bdSamplesPerPixel= 4;
-		    bd->bdBitsPerPixel *= 2;
-		    bd->bdBytesPerRow *= 2;
-		    bd->bdBufferLength *= 2;
-		    }
+		if  ( bmCalculateSizes( bd ) )
+		    { LDEB(1); return -1;	}
 
 		buffer= (unsigned char *)malloc( bd->bdBufferLength );
 		if  ( ! buffer )
@@ -104,29 +110,29 @@ int bmGifReadGif(		BitmapDescription *	bd,
 
 		wide= bd->bdPixelsWide;
 		bpr= bd->bdBytesPerRow;
-		if  ( gft->Image.Interlace )
+		if  ( gid->Interlace )
 		    {
 		    for ( row= 0; row < bd->bdPixelsHigh; row += 8 )
 			{
-			if  ( DGifGetLine( gft, buffer+ row* bpr, wide ) !=
+			if  ( bmGifGetPixels( gft, buffer+ row* bpr, wide ) !=
 								    GIF_OK )
 			    { LDEB(row); DGifCloseFile( gft ); return -1; }
 			}
 		    for ( row= 4; row < bd->bdPixelsHigh; row += 8 )
 			{
-			if  ( DGifGetLine( gft, buffer+ row* bpr, wide ) !=
+			if  ( bmGifGetPixels( gft, buffer+ row* bpr, wide ) !=
 								    GIF_OK )
 			    { LDEB(row); DGifCloseFile( gft ); return -1; }
 			}
 		    for ( row= 2; row < bd->bdPixelsHigh; row += 4 )
 			{
-			if  ( DGifGetLine( gft, buffer+ row* bpr, wide ) !=
+			if  ( bmGifGetPixels( gft, buffer+ row* bpr, wide ) !=
 								    GIF_OK )
 			    { LDEB(row); DGifCloseFile( gft ); return -1; }
 			}
 		    for ( row= 1; row < bd->bdPixelsHigh; row += 2 )
 			{
-			if  ( DGifGetLine( gft, buffer+ row* bpr, wide ) !=
+			if  ( bmGifGetPixels( gft, buffer+ row* bpr, wide ) !=
 								    GIF_OK )
 			    { LDEB(row); DGifCloseFile( gft ); return -1; }
 			}
@@ -134,26 +140,30 @@ int bmGifReadGif(		BitmapDescription *	bd,
 		else{
 		    for ( row= 0; row < bd->bdPixelsHigh; row++ )
 			{
-			if  ( DGifGetLine( gft, buffer+ row* bpr, wide ) !=
+			if  ( bmGifGetPixels( gft, buffer+ row* bpr, wide ) !=
 								    GIF_OK )
 			    { LDEB(row); DGifCloseFile( gft ); return -1; }
 			}
 		    }
 
-		if  ( gft->Image.ColorMap )
-		    { cmap= gft->Image.ColorMap;	}
-		else{ cmap= gft->SColorMap;		}
+		if  ( gid->gidImageColorMap.gcmColorCount > 0 )
+		    { gcm= &(gid->gidImageColorMap);		  }
+		else{ gcm= &(gft->gftScreenDescriptor.gsdScreenColorMap); }
 
-		bd->bdColorCount= cmap->ColorCount;
-		for ( row= 0; row < bd->bdColorCount; row++ )
+		bd->bdColorCount= gcm->gcmColorCount;
+
+		rgb8From= gcm->gcmColors;
+		rgb8To= bd->bdRGB8Palette;
+		for ( row= 0; row < bd->bdColorCount;
+					    rgb8From++, rgb8To++, row++ )
 		    {
-		    bd->bdRGB8Palette[row].rgb8Red=   cmap->Colors[row].Red;
-		    bd->bdRGB8Palette[row].rgb8Green= cmap->Colors[row].Green;
-		    bd->bdRGB8Palette[row].rgb8Blue=  cmap->Colors[row].Blue;
-		    bd->bdRGB8Palette[row].rgb8Alpha=
-						255* ( row != transparent );
+		    rgb8To->rgb8Red= rgb8From->rgb8Red;
+		    rgb8To->rgb8Green= rgb8From->rgb8Green;
+		    rgb8To->rgb8Blue= rgb8From->rgb8Blue;
+		    rgb8To->rgb8Alpha= 255* ( row != transparent );
 		    }
 
+		cc= 0;
 		if  ( bd->bdHasAlpha )
 		    {
 		    for ( row= 0; row < bd->bdPixelsHigh; row++ )
@@ -168,10 +178,31 @@ int bmGifReadGif(		BitmapDescription *	bd,
 				{ *(to--)= 0x00; }
 			    else{ *(to--)= 0xff; }
 
+			    if  ( *from >= cc )
+				{ cc= *from+ 1;	}
+
 			    *(to--)= *(from--);
 			    }
 			}
 		    }
+		else{
+		    for ( row= 0; row < bd->bdPixelsHigh; row++ )
+			{
+			unsigned char *	from= buffer+ row* bpr+ bpr-  1;
+			int		col;
+
+			for ( col= 0; col < bd->bdPixelsWide; col++ )
+			    {
+			    if  ( *from >= cc )
+				{ cc= *from+ 1;	}
+
+			    from--;
+			    }
+			}
+		    }
+
+		bd->bdColorCount= cc;
+
 		continue;
 
 	    case EXTENSION_RECORD_TYPE:
@@ -190,17 +221,21 @@ int bmGifReadGif(		BitmapDescription *	bd,
 			if  ( extension[1] & 0x1 )
 			    { transparent= extension[4];	}
 			break;
+
 		    case COMMENT_EXT_FUNC_CODE:
 			XDEB(extensionType);
 			appDebug( "Comment=\"%.*s\"\n",
 						extension[0], extension+ 1 );
 			break;
+
 		    case PLAINTEXT_EXT_FUNC_CODE:
 			XDEB(extensionType); break;
+
 		    case APPLICATION_EXT_FUNC_CODE:
 			XDEB(extensionType);
 			appDebug( "Application=\"%.8s\"\n", extension+ 1 );
 			break;
+
 		    default:
 			XDEB(extensionType); break;
 		    }
@@ -267,9 +302,7 @@ int bmGifWriteGif(		const BitmapDescription *	bd,
     {
     GifFileType *		gft;
 
-    RGB8Color			palette[256];
-    GifColorType		gct[256];
-    ColorMapObject		cmap;
+    GifColorMap			gcm;
 
     const int			left= 0;
     const int			top= 0;
@@ -278,64 +311,73 @@ int bmGifWriteGif(		const BitmapDescription *	bd,
     unsigned int		row;
     const unsigned char *	from;
 
+    int				bpcolor;
+
     int				transparentColor= -1;
+    int				backgroundColor= 0;
 
     gft= EGifOpenFileHandle( sos );
     if  ( ! gft )
 	{ XDEB(gft); return -1;	}
 
+    bmGifInitGifColorMap( &gcm );
+
     /*  1  */
     switch( bd->bdColorEncoding )
 	{
 	case BMcoWHITEBLACK:
 	case BMcoBLACKWHITE:
-	    if  ( bmMakeGrayPalette( bd, &(cmap.ColorCount), palette ) )
+	    if  ( bmMakeGrayPalette( bd, &(gcm.gcmColorCount), gcm.gcmColors ) )
 		{ LDEB(bd->bdBitsPerPixel); return -1;	}
 
-	    cmap.BitsPerPixel= bd->bdBitsPerPixel;
-	    cmap.Colors= gct;
-
-	    for ( row= 0; row < (unsigned)cmap.ColorCount; row++ )
-		{
-		gct[row].Red= palette[row].rgb8Red;
-		gct[row].Green= palette[row].rgb8Green;
-		gct[row].Blue= palette[row].rgb8Blue;
-		}
+	    gcm.gcmBitsPerPixel= bd->bdBitsPerPixel;
 
 	    break;
 
 	case BMcoRGB8PALETTE:
-	    switch( bd->bdBitsPerPixel )
+	    bpcolor= 1;
+	    while( ( 1 << bpcolor ) < bd->bdColorCount )
+		{ bpcolor++;	}
+
+	    if  ( bpcolor > 8 )
 		{
-		case 1: case 2: case 4: case 8:
-		    cmap.BitsPerPixel= bd->bdBitsPerPixel;
-		    cmap.ColorCount= 1 << cmap.BitsPerPixel;
-		    cmap.Colors= gct;
-
-		    for ( row= 0; row < bd->bdColorCount; row++ )
-			{
-			gct[row].Red=   bd->bdRGB8Palette[row].rgb8Red;
-			gct[row].Green= bd->bdRGB8Palette[row].rgb8Green;
-			gct[row].Blue=  bd->bdRGB8Palette[row].rgb8Blue;
-
-			if  ( transparentColor < 0			&&
-			      bd->bdRGB8Palette[row].rgb8Alpha == 0	)
-			    { transparentColor= row;	}
-			}
-
-		    if  ( bd->bdHasAlpha && transparentColor < 0 )
-			{
-			transparentColor= bd->bdColorCount;
-			if  ( transparentColor >= 256 )
-			    { LDEB(transparentColor); transparentColor= 255; }
-			}
-
-		    break;
-		default:
-		    LLDEB(bd->bdColorEncoding,bd->bdBitsPerPixel);
-		    return -1;
+		LDEB(bpcolor);
+		LLDEB(bd->bdColorEncoding,bd->bdBitsPerPixel);
+		return -1;
 		}
+
+	    gcm.gcmBitsPerPixel= bpcolor;
+	    gcm.gcmColorCount= 1 << gcm.gcmBitsPerPixel;
+
+	    for ( row= 0; row < bd->bdColorCount; row++ )
+		{
+		gcm.gcmColors[row]= bd->bdRGB8Palette[row];
+
+		if  ( transparentColor < 0			&&
+		      bd->bdRGB8Palette[row].rgb8Alpha == 0	)
+		    { transparentColor= backgroundColor= row;	}
+		}
+
+	    if  ( bd->bdHasAlpha && transparentColor < 0 )
+		{
+		transparentColor= bd->bdColorCount;
+
+		if  ( gcm.gcmBitsPerPixel < 8			&&
+		      transparentColor >= gcm.gcmColorCount	)
+		    {
+		    gcm.gcmBitsPerPixel++;
+		    gcm.gcmColorCount *= 2;
+		    }
+
+		if  ( transparentColor >= gcm.gcmColorCount )
+		    {
+		    LDEB(transparentColor);
+		    transparentColor= backgroundColor= gcm.gcmColorCount- 1;
+		    }
+		}
+
 	    break;
+
 	case BMcoRGB:
 	    switch( bd->bdBitsPerPixel )
 		{
@@ -345,47 +387,45 @@ int bmGifWriteGif(		const BitmapDescription *	bd,
 		    LLDEB(bd->bdColorEncoding,bd->bdBitsPerPixel);
 		    return -1;
 		}
-	    /* colorBits= bd->bdBitsPerPixel; */
-	    /* break; */
+
 	default:
 	    LLDEB(bd->bdColorEncoding,bd->bdBitsPerPixel);
 	    return -1;
 	}
 
     if  ( bd->bdHasAlpha )
-	{ EGifSetGifVersion( "89a" );	}
-    else{ EGifSetGifVersion( "87a" );	}
+	{ bmGifSetVersion( gft, "89a" );	}
+    else{ bmGifSetVersion( gft, "87a" );	}
 
     if  ( EGifPutScreenDesc( gft, bd->bdPixelsWide, bd->bdPixelsHigh,
-				cmap.ColorCount, 0, &cmap ) != GIF_OK )
-	{ LDEB(GifLastError()); EGifCloseFile( gft ); return -1;	}
+				    gcm.gcmBitsPerPixel, 0, &gcm ) != GIF_OK )
+	{ LDEB(1); EGifCloseFile( gft ); return -1;	}
 
     if  ( bd->bdHasAlpha )
 	{
-	unsigned char	extension[5];
+	unsigned char	extension[4];
 
-	extension[0]= 5;
-	extension[1]= 0x1;
+	extension[0]= 0x01;
+	extension[1]= 0;
 	extension[2]= 0;
-	extension[3]= 0;
-	extension[4]= transparentColor;
+	extension[3]= transparentColor;
 
 	if  ( EGifPutExtension( gft, GRAPHICS_EXT_FUNC_CODE,
-				    sizeof(extension), extension ) != GIF_OK )
-	    { LDEB(GifLastError()); EGifCloseFile( gft ); return -1;	}
+				sizeof(extension), extension ) != GIF_OK )
+	    { LDEB(1); EGifCloseFile( gft ); return -1;	}
 	}
 
     if  ( EGifPutImageDesc( gft, left, top,
 				bd->bdPixelsWide, bd->bdPixelsHigh,
-				interlace, &cmap ) != GIF_OK )
-	{ LDEB(GifLastError()); EGifCloseFile( gft ); return -1;	}
+						interlace, &gcm ) != GIF_OK )
+	{ LDEB(1); EGifCloseFile( gft ); return -1;	}
 
     if  ( bd->bdBitsPerPixel == 8 && ! bd->bdHasAlpha )
 	{
 	from= buffer;
 	for ( row= 0; row < bd->bdPixelsHigh; row++ )
 	    {
-	    if  ( EGifPutLine( gft, (unsigned char *)from,
+	    if  ( bmGifPutPixels( gft, (unsigned char *)from,
 						bd->bdPixelsWide ) != GIF_OK )
 		{ LDEB(row); EGifCloseFile( gft ); return -1; }
 
@@ -395,13 +435,13 @@ int bmGifWriteGif(		const BitmapDescription *	bd,
     else{
 	unsigned char *		line;
 
-	line= (unsigned char *)malloc( bd->bdPixelsWide );
+	line= (unsigned char *)malloc( bd->bdPixelsWide+ 7 );
 	if  ( ! line )
 	    { XDEB(line); EGifCloseFile( gft ); return -1; }
 
 	switch( bd->bdBitsPerPixel )
 	    {
-	    case 1: case 2: case 4: case 8:
+	    case 1: case 2: case 4: case 8: case 16:
 		for ( row= 0; row < bd->bdPixelsHigh; row++ )
 		    {
 		    from= buffer+ row* bd->bdBytesPerRow;
@@ -409,7 +449,8 @@ int bmGifWriteGif(		const BitmapDescription *	bd,
 		    bmInflateTo8bit( line, from, bd,
 					    transparentColor, bd->bdHasAlpha );
 
-		    if  ( EGifPutLine( gft, line, bd->bdPixelsWide ) != GIF_OK )
+		    if  ( bmGifPutPixels( gft, line, bd->bdPixelsWide )
+								    != GIF_OK )
 			{
 			LDEB(row);
 			EGifCloseFile( gft ); free( line );
@@ -440,6 +481,9 @@ int bmWriteGifFile(	const char *			filename,
     {
     SimpleOutputStream *	sos;
 
+    if  ( bmCanWriteGifFile( bd, privateFormat, compressionFactor ) )
+	{ LDEB(1); return -1;	}
+
     sos= sioOutStdioOpen( filename );
     if  ( ! sos )
 	{ SXDEB(filename,sos); return -1;	}
@@ -462,5 +506,10 @@ int bmCanWriteGifFile(	const BitmapDescription *	bd,
     if  ( bd->bdBitsPerPixel <= 8 )
 	{ return 0;	}
 
+    if  ( bd->bdColorEncoding == BMcoRGB8PALETTE	&&
+	  bd->bdHasAlpha				&&
+	  bd->bdBitsPerPixel <= 16			)
+	{ return 0;	}
+
     return -1;
     }
