@@ -10,6 +10,8 @@
 #   undef	APP_DEB
 #   endif
 
+/*  # include "../mecheck/mecheck.h"  */
+
 #ifdef	__GNUC__
 extern int appDebug(	const char *	format, ... )
 			__attribute__ (( format ( printf, 1, 2 ) ));
@@ -27,7 +29,7 @@ extern int appDebugSetFile(	const char *	filename,
 #	define	APP_DEB(x)
 # endif
 
-# define DEBS(sa)	(sa)?"\"":"", (sa)?(sa):"0x0", (sa)?"\"":""
+# define DEBS(sa)	((sa)?"\"":""), ((sa)?(sa):"0x0"), ((sa)?"\"":"")
 # define DEBX(lo)	((unsigned long)(lo))
 # define DEBO(lo)	((unsigned long)(lo))
 # define DEBL(lo)	((long)(lo))
@@ -85,6 +87,15 @@ extern int appDebugSetFile(	const char *	filename,
 			    #c1, DEBC(c1),			\
 			    #c2, DEBC(c2)			))
 
+# define LCCDEB(l1,c2,c3) APP_DEB(appDebug( "%s(%3d)"		\
+			    DFML				\
+			    DFMC				\
+			    DFMC				\
+			    "\n", __FILE__, __LINE__,		\
+			    #l1, DEBL(l1),			\
+			    #c2, DEBC(c2),			\
+			    #c3, DEBC(c3)			))
+
 # define SSDEB(s1,s2) APP_DEB(appDebug( "%s(%3d)"		\
 			    DFMS				\
 			    DFMS				\
@@ -92,6 +103,13 @@ extern int appDebugSetFile(	const char *	filename,
 			    #s1, DEBS(s1),			\
 			    #s2, DEBS(s2)			))
 
+# define SCDEB(s1,c2) APP_DEB(appDebug( "%s(%3d)"		\
+			    DFMS				\
+			    DFMC				\
+			    "\n", __FILE__, __LINE__,		\
+			    #s1, DEBS(s1),			\
+			    #c2, DEBC(c2)			))
+
 # define LLDEB(l1,l2) APP_DEB(appDebug( "%s(%3d)"		\
 			    DFML				\
 			    DFML				\
@@ -148,6 +166,13 @@ extern int appDebugSetFile(	const char *	filename,
 			    #l1, DEBL(l1),			\
 			    #s2, DEBS(s2)			))
 
+# define FSDEB(d1,s2) APP_DEB(appDebug( "%s(%3d)"		\
+			    DFMD				\
+			    DFMS				\
+			    "\n", __FILE__, __LINE__,		\
+			    #d1, DEBD(d1),			\
+			    #s2, DEBS(s2)			))
+
 # define SXDEB(s1,l2) APP_DEB(appDebug( "%s(%3d)"		\
 			    DFMS				\
 			    DFMX				\
@@ -161,6 +186,15 @@ extern int appDebugSetFile(	const char *	filename,
 			    DFMS				\
 			    "\n", __FILE__, __LINE__,		\
 			    #s1, DEBS(s1),			\
+			    #l2, DEBX(l2),			\
+			    #s3, DEBS(s3)			))
+
+# define SLSDEB(s1,l2,s3) APP_DEB(appDebug( "%s(%3d)"		\
+			    DFMS				\
+			    DFML				\
+			    DFMS				\
+			    "\n", __FILE__, __LINE__,		\
+			    #s1, DEBS(s1),			\
 			    #l2, DEBL(l2),			\
 			    #s3, DEBS(s3)			))
 
@@ -210,6 +244,15 @@ extern int appDebugSetFile(	const char *	filename,
 			    #l2, DEBL(l2),			\
 			    #l3, DEBL(l3)			))
 
+# define SLXDEB(s1,l2,l3) APP_DEB(appDebug( "%s(%3d)"		\
+			    DFMS				\
+			    DFML				\
+			    DFMX				\
+			    "\n", __FILE__, __LINE__,		\
+			    #s1, DEBS(s1),			\
+			    #l2, DEBL(l2),			\
+			    #l3, DEBX(l3)			))
+
 # define SXXDEB(s1,l2,l3) APP_DEB(appDebug( "%s(%3d)"		\
 			    DFMS				\
 			    DFMX				\
@@ -228,6 +271,17 @@ extern int appDebugSetFile(	const char *	filename,
 			    #d2, DEBD(d2),			\
 			    #l3, DEBL(l3)			))
 
+# define SLFFDEB(s1,l2,d3,d4) APP_DEB(appDebug( "%s(%3d)"	\
+			    DFMS				\
+			    DFML				\
+			    DFMD				\
+			    DFMD				\
+			    "\n", __FILE__, __LINE__,		\
+			    #s1, DEBS(s1),			\
+			    #l2, DEBL(l2),			\
+			    #d3, DEBD(d3),			\
+			    #d4, DEBD(d4)			))
+
 # define SLFDEB(s1,l2,d3) APP_DEB(appDebug( "%s(%3d)"		\
 			    DFMS				\
 			    DFML				\
@@ -266,6 +320,15 @@ extern int appDebugSetFile(	const char *	filename,
 			    #s2, DEBS(s2),			\
 			    #l3, DEBL(l3)			))
 
+# define LSXDEB(l1,s2,l3) APP_DEB(appDebug( "%s(%3d)"		\
+			    DFML				\
+			    DFMS				\
+			    DFMX				\
+			    "\n", __FILE__, __LINE__,		\
+			    #l1, DEBL(l1),			\
+			    #s2, DEBS(s2),			\
+			    #l3, DEBX(l3)			))
+
 # define LSFDEB(l1,s2,d3) APP_DEB(appDebug( "%s(%3d)"		\
 			    DFML				\
 			    DFMS				\
@@ -308,6 +371,17 @@ extern int appDebugSetFile(	const char *	filename,
 			    #l3, DEBL(l3),			\
 			    #l4, DEBL(l4)			))
 
+# define LLLSDEB(l1,l2,l3,s4) APP_DEB(appDebug( "%s(%3d)"	\
+			    DFML				\
+			    DFML				\
+			    DFML				\
+			    DFMS				\
+			    "\n", __FILE__, __LINE__,		\
+			    #l1, DEBS(l1),			\
+			    #l2, DEBL(l2),			\
+			    #l3, DEBL(l3),			\
+			    #s4, DEBS(s4)			))
+
 # define LLLDEB(l1,l2,l3) APP_DEB(appDebug( "%s(%3d)"		\
 			    DFML				\
 			    DFML				\
@@ -350,6 +424,17 @@ extern int appDebugSetFile(	const char *	filename,
 			    #l4, DEBL(l4),			\
 			    #l5, DEBL(l5)			))
 
+# define LLLFDEB(l1,l2,l3,d4) APP_DEB(appDebug( "%s(%3d)"	\
+			    DFML				\
+			    DFML				\
+			    DFML				\
+			    DFMD				\
+			    "\n", __FILE__, __LINE__,		\
+			    #l1, DEBL(l1),			\
+			    #l2, DEBL(l2),			\
+			    #l3, DEBL(l3),			\
+			    #d4, DEBD(d4)			))
+
 # define LXXDEB(l1,l2,l3) APP_DEB(appDebug( "%s(%3d)"		\
 			    DFML				\
 			    DFMX				\
@@ -359,6 +444,15 @@ extern int appDebugSetFile(	const char *	filename,
 			    #l2, DEBX(l2),			\
 			    #l3, DEBX(l3)			))
 
+# define XXLDEB(l1,l2,l3) APP_DEB(appDebug( "%s(%3d)"		\
+			    DFMX				\
+			    DFMX				\
+			    DFML				\
+			    "\n", __FILE__, __LINE__,		\
+			    #l1, DEBX(l1),			\
+			    #l2, DEBX(l2),			\
+			    #l3, DEBL(l3)			))
+
 # define LLXDEB(l1,l2,l3) APP_DEB(appDebug( "%s(%3d)"		\
 			    DFML				\
 			    DFML				\
@@ -377,6 +471,43 @@ extern int appDebugSetFile(	const char *	filename,
 			    #d2, DEBD(d2),			\
 			    #d3, DEBD(d3)			))
 
+# define LLFFFDEB(l1,l2,d3,d4,d5) APP_DEB(appDebug( "%s(%3d)"	\
+			    DFML				\
+			    DFML				\
+			    DFMD				\
+			    DFMD				\
+			    DFMD				\
+			    "\n", __FILE__, __LINE__,		\
+			    #l1, DEBL(l1),			\
+			    #l2, DEBL(l2),			\
+			    #d3, DEBD(d3),			\
+			    #d4, DEBD(d4),			\
+			    #d5, DEBD(d5)			))
+
+# define LFFFDEB(l1,d2,d3,d4) APP_DEB(appDebug( "%s(%3d)"	\
+			    DFML				\
+			    DFMD				\
+			    DFMD				\
+			    DFMD				\
+			    "\n", __FILE__, __LINE__,		\
+			    #l1, DEBL(l1),			\
+			    #d2, DEBD(d2),			\
+			    #d3, DEBD(d3),			\
+			    #d4, DEBD(d4)			))
+
+# define LFFFFDEB(l1,d2,d3,d4,d5) APP_DEB(appDebug( "%s(%3d)"	\
+			    DFML				\
+			    DFMD				\
+			    DFMD				\
+			    DFMD				\
+			    DFMD				\
+			    "\n", __FILE__, __LINE__,		\
+			    #l1, DEBL(l1),			\
+			    #d2, DEBD(d2),			\
+			    #d3, DEBD(d3),			\
+			    #d4, DEBD(d4),			\
+			    #d5, DEBD(d5)			))
+
 # define LLFDEB(l1,l2,d3) APP_DEB(appDebug( "%s(%3d)"		\
 			    DFML				\
 			    DFML				\
@@ -433,6 +564,17 @@ extern int appDebugSetFile(	const char *	filename,
 			    #s3, DEBS(s3),			\
 			    #l4, DEBL(l4)			))
 
+# define SSSFDEB(s1,s2,s3,d4) APP_DEB(appDebug( "%s(%3d)"	\
+			    DFMS				\
+			    DFMS				\
+			    DFMS				\
+			    DFMD				\
+			    "\n", __FILE__, __LINE__,		\
+			    #s1, DEBS(s1),			\
+			    #s2, DEBS(s2),			\
+			    #s3, DEBS(s3),			\
+			    #d4, DEBD(d4)			))
+
 # define SSSSDEB(s1,s2,s3,s4) APP_DEB(appDebug( "%s(%3d)"	\
 			    DFMS				\
 			    DFMS				\
@@ -455,6 +597,17 @@ extern int appDebugSetFile(	const char *	filename,
 			    #l3, DEBL(l3),			\
 			    #l4, DEBL(l4)			))
 
+# define SSLFDEB(s1,s2,l3,d4) APP_DEB(appDebug( "%s(%3d)"	\
+			    DFMS				\
+			    DFMS				\
+			    DFML				\
+			    DFMD				\
+			    "\n", __FILE__, __LINE__,		\
+			    #s1, DEBS(s1),			\
+			    #s2, DEBS(s2),			\
+			    #l3, DEBL(l3),			\
+			    #d4, DEBD(d4)			))
+
 # define LLSLDEB(l1,l2,s3,l4) APP_DEB(appDebug( "%s(%3d)"	\
 			    DFML				\
 			    DFML				\
@@ -486,6 +639,24 @@ extern int appDebugSetFile(	const char *	filename,
 			    #s3, DEBS(s3),			\
 			    #s4, DEBS(s4)			))
 
+# define LSSDEB(l1,s2,s3) APP_DEB(appDebug( "%s(%3d)"	\
+			    DFML				\
+			    DFMS				\
+			    DFMS				\
+			    "\n", __FILE__, __LINE__,		\
+			    #l1, DEBL(l1),			\
+			    #s2, DEBS(s2),			\
+			    #s3, DEBS(s3)			))
+
+# define XSSDEB(l1,s2,s3) APP_DEB(appDebug( "%s(%3d)"	\
+			    DFMX				\
+			    DFMS				\
+			    DFMS				\
+			    "\n", __FILE__, __LINE__,		\
+			    #l1, DEBX(l1),			\
+			    #s2, DEBS(s2),			\
+			    #s3, DEBS(s3)			))
+
 # define FFFDEB(d1,d2,d3) APP_DEB(appDebug( "%s(%3d)"		\
 			    DFMD				\
 			    DFMD				\
