@@ -8,6 +8,7 @@
 #   include	<charnames.h>
 
 #   include	"indlocal.h"
+#   include	<utilFontEncoding.h>
 #   include	<appDebugon.h>
 
 /****************************************************************/
@@ -370,25 +371,7 @@ void indSpellIso1CharacterKinds(	SpellCheckContext *	scc )
     for ( i= 0; i < 256; i++ )
 	{ scc->sccCharShifts[i]= i; }
 
-    for ( i= 0; i < 256; i++ )
-	{
-	if  ( ISO1_islower( i ) )
-	    {
-	    scc->sccCharKinds[i] |= CHARisLOWER;
-	    if  ( ! ISO1_isupper( i ) )
-		{ scc->sccCharShifts[i]= ISO1_toupper( i );	}
-	    }
-
-	if  ( ISO1_isupper( i ) )
-	    {
-	    scc->sccCharKinds[i] |= CHARisUPPER;
-	    if  ( ! ISO1_islower( i ) )
-		{ scc->sccCharShifts[i]= ISO1_tolower( i );	}
-	    }
-
-	if  ( ISO1_isdigit( i ) )
-	    { scc->sccCharKinds[i] |= CHARisDIGIT;	}
-	}
+    utilSetLatin1CharacterKinds( scc->sccCharKinds, scc->sccCharShifts );
 
     return;
     }
@@ -401,25 +384,20 @@ void indSpellIso2CharacterKinds(	SpellCheckContext *	scc )
     for ( i= 0; i < 256; i++ )
 	{ scc->sccCharShifts[i]= i; }
 
+    utilSetLatin2CharacterKinds( scc->sccCharKinds, scc->sccCharShifts );
+
+    return;
+    }
+
+void indSpellIso7CharacterKinds(	SpellCheckContext *	scc )
+    {
+    int		i;
+
+    memset( scc->sccCharKinds, 0, 256 );
     for ( i= 0; i < 256; i++ )
-	{
-	if  ( ISO2_islower( i ) )
-	    {
-	    scc->sccCharKinds[i] |= CHARisLOWER;
-	    if  ( ! ISO2_isupper( i ) )
-		{ scc->sccCharShifts[i]= ISO2_toupper( i );	}
-	    }
+	{ scc->sccCharShifts[i]= i; }
 
-	if  ( ISO2_isupper( i ) )
-	    {
-	    scc->sccCharKinds[i] |= CHARisUPPER;
-	    if  ( ! ISO2_islower( i ) )
-		{ scc->sccCharShifts[i]= ISO2_tolower( i );	}
-	    }
-
-	if  ( ISO2_isdigit( i ) )
-	    { scc->sccCharKinds[i] |= CHARisDIGIT;	}
-	}
+    utilSetLatin2CharacterKinds( scc->sccCharKinds, scc->sccCharShifts );
 
     return;
     }
@@ -432,25 +410,7 @@ void indSpellKoi8rCharacterKinds(	SpellCheckContext *	scc )
     for ( i= 0; i < 256; i++ )
 	{ scc->sccCharShifts[i]= i; }
 
-    for ( i= 0; i < 256; i++ )
-	{
-	if  ( KOI8R_islower( i ) )
-	    {
-	    scc->sccCharKinds[i] |= CHARisLOWER;
-	    if  ( ! KOI8R_isupper( i ) )
-		{ scc->sccCharShifts[i]= KOI8R_toupper( i );	}
-	    }
-
-	if  ( KOI8R_isupper( i ) )
-	    {
-	    scc->sccCharKinds[i] |= CHARisUPPER;
-	    if  ( ! KOI8R_islower( i ) )
-		{ scc->sccCharShifts[i]= KOI8R_tolower( i );	}
-	    }
-
-	if  ( KOI8R_isdigit( i ) )
-	    { scc->sccCharKinds[i] |= CHARisDIGIT;	}
-	}
+    utilSetKoi8rCharacterKinds( scc->sccCharKinds, scc->sccCharShifts );
 
     return;
     }
