@@ -29,7 +29,7 @@
  *
  */
 
-#   include	"config.h"
+#   include	"libregConfig.h"
 
 #   include	<stdio.h>
 #   include	<string.h>
@@ -143,9 +143,8 @@
 #define	UCHARAT(p)	((int)*(p)&CHARBITS)
 #endif
 
-#define	FAIL(m)	{ regErrorMessage(m); return(NULL); }
 #define	ISMULT(c)	((c) == '*' || (c) == '+' || (c) == '?')
-#define	META	"^$.[()|?+*\\"
+#define	META		"^$.[()|?+*\\"
 
 /*
  * Flags to be passed up and down.
@@ -296,8 +295,8 @@ regProg * regCompile( const unsigned char *	exp )
 
     unsigned char		dummy;
 
-    if (exp == NULL)
-	    FAIL("NULL argument");
+    if  ( exp == NULL )
+	{ XDEB(exp); return (regProg *)0;	}
 
     /* First pass: determine size, legality. */
     regInitParsingJob( &epj );
@@ -315,8 +314,8 @@ regProg * regCompile( const unsigned char *	exp )
 
     /* Allocate space. */
     r = (regProg *)malloc(sizeof(regProg) + (unsigned)epj.epjSize);
-    if (r == NULL)
-	    FAIL("out of space");
+    if  ( ! r )
+	{ LXDEB(epj.epjSize,r); return (regProg *)0;	}
 
     /* Second pass: emit code. */
     regInitParsingJob( &epj );
@@ -1331,7 +1330,7 @@ static int regmatch(	unsigned char *		prog,
 		*pPast= string; return 1;	/* Success! */
 		break;
 	default:
-		regErrorMessage("memory corruption");
+		SDEB("memory corruption");
 		return 0;
 		break;
 	}
@@ -1343,7 +1342,7 @@ static int regmatch(	unsigned char *		prog,
      * We get here only if there's trouble -- normally "case END" is
      * the terminating point.
      */
-    regErrorMessage("corrupted pointers");
+    SDEB("corrupted pointers");
     return 0;
 }
 
@@ -1385,7 +1384,7 @@ static int regrepeat(	unsigned char *		p,
 	    break;
 
 	default:		/* Oh dear.  Called inappropriately. */
-	    regErrorMessage("internal foulup");
+	    SDEB("internal foulup");
 	    count = 0;	/* Best compromise. */
 	    break;
 	}
@@ -1526,7 +1525,7 @@ static char * regprop( unsigned char *op )
 		p = "PLUS";
 		break;
 	default:
-		regErrorMessage("corrupted opcode");
+		SDEB("corrupted opcode");
 		break;
 	}
 	if (p != NULL)
